package com.despark.tuning3d.model.cars {
    import com.utils.ObjectUtils;

    public class CarBackgroundAnimatedObject {
        public static const NONE:String = "none";
        public static const FRAME_BY_FRAME:String = "frameByFrame";
        public static const BLINKING:String = "blinking";
        public static const MOVING:String = "moving";
        //
        private var _animationType:String;
        private var _name:String;
        private var _layerPosition:int;
        private var _index:int;
        private var _frameRate:int;
        private var _delay:Number;
        private var _speed:Number;
        private var _blinkCount:Object;
        private var _frequency:Object;
        private var _cam1:ObjectCamera;
        private var _cam2:ObjectCamera;
        public function CarBackgroundAnimatedObject(source:Object) {
            _animationType = ObjectUtils.getValue(source, "type", null);
            _name = ObjectUtils.getValue(source, "name", null);
            _layerPosition = ObjectUtils.getValue(source, "layerPosition", 0);
            _frameRate = ObjectUtils.getValue(source, "frameRate", 12);
            _delay = ObjectUtils.getValue(source, "delay", 0);
            _speed = ObjectUtils.getValue(source, "speed", 0);
            _blinkCount = ObjectUtils.getValue(source, "blinkCount", null);
            _frequency = ObjectUtils.getValue(source, "frequency", null);
            if(source.hasOwnProperty("cam1") && source.cam1){
                _cam1 = new ObjectCamera(source.cam1);
            }
            if(source.hasOwnProperty("cam2") && source.cam2){
                _cam2 = new ObjectCamera(source.cam2);
            }
        }

        public function get cam1():ObjectCamera {
            return _cam1;
        }

        public function get cam2():ObjectCamera {
            return _cam2;
        }

        public function get layerPosition():int {
            return _layerPosition;
        }

        public function get name():String {
            return _name;
        }

        public function get animationType():String {
            return _animationType;
        }

        public function get frameRate():int {
            return _frameRate;
        }

        public function get delay():Number {
            return _delay;
        }

        public function get blinkCount():Object {
            return _blinkCount;
        }

        public function get frequency():Object {
            return _frequency;
        }

        public function get index():int {
            return _index;
        }

        public function set index(value:int):void {
            _index = value;
        }

        public function get speed():Number {
            return _speed;
        }
    }
}

import com.utils.ObjectUtils;

class ObjectCamera{
    private var _imagePath:String;
    private var _x:Number;
    private var _y:Number;
    private var _startX:Number;
    private var _startY:Number;
    private var _endX:Number;
    private var _endY:Number;
    private var _width:Number;
    private var _height:Number;
    public function ObjectCamera(source:Object){
        _imagePath = ObjectUtils.getValue(source, "imagePath", null);
        _x = ObjectUtils.getValue(source, "x", 0);
        _y = ObjectUtils.getValue(source, "y", 0);
        _startX = ObjectUtils.getValue(source, "startX", 0);
        _startY = ObjectUtils.getValue(source, "startY", 0);
        _endX = ObjectUtils.getValue(source, "endX", 0);
        _endY = ObjectUtils.getValue(source, "endY", 0);
        _width = ObjectUtils.getValue(source, "w", 0);
        _height = ObjectUtils.getValue(source, "h", 0);
    }

    public function get imagePath():String {
        return _imagePath;
    }

    public function get x():Number {
        return _x;
    }

    public function get y():Number {
        return _y;
    }

    public function get width():Number {
        return _width;
    }

    public function get height():Number {
        return _height;
    }

    public function get startX():Number {
        return _startX;
    }

    public function get startY():Number {
        return _startY;
    }

    public function get endX():Number {
        return _endX;
    }

    public function get endY():Number {
        return _endY;
    }

    public function set x(value:Number):void {
        _x = value;
    }

    public function set y(value:Number):void {
        _y = value;
    }
}
