package com.despark.tuning3d.model.cars {
import com.despark.tuning3d.model.brands.BrandsParser;
    import com.despark.tuning3d.model.cars.CarsParser;
    import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.model.core.Parser;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.packages.Packet;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartColor;
import com.despark.tuning3d.model.parts.PartDisk;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterialImage;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.model.parts.PartSlotDisk;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.ZipManager;
import com.utils.DevHelper;
import com.utils.Logger;
import com.utils.MemoryTracker;

import flame.utils.ArrayUtil;

import flash.utils.Dictionary;
import flash.utils.getTimer;

import org.as3commons.lang.ArrayUtils;

public class CarsParserOld extends Parser {
    private static const SHADOW_OFFSET_X:int = 52;


    public function parseProductInfo(car:Car, packet:Packet = null):void {

        if (!car)
            return;


        if (CarsParser.CHOPPER_ID_LIST.indexOf(car.id) != -1) {
            car.isChopper = true;
        }else if (CarsParser.BEETLE_ID_LIST.indexOf(car.id) != -1) {
            car.isBeetle = true;
        }

        if (packet) {

            car.addPacket(packet);//packet=packet;
        }
    }

    private function initSuspensionData(car:Car, carJSON:Object):void{
        if(carJSON.hasOwnProperty("suspensionForComponentsStructure")) {
            if(carJSON.suspensionForComponentsStructure is Array) {
                var suspForComp : Array = (carJSON.suspensionForComponentsStructure as Array).concat();
                var sortFunction : Function = function(a : Object, b : Object):Number {
                    var diff:int = b.components.length - a.components.length;
                    if(diff == 0){
                        return diff;
                    }
                    return diff / Math.abs(diff);
                };

                suspForComp.sort(sortFunction);
                //
                // parsing suspension settings for component groups
                var i:int;
                var j:int;
                var prop:Object;
                var prop2:Object;
                for(i = 0; i < suspForComp.length; i++){
                    prop = suspForComp[i];
                    if(prop.hasOwnProperty("components") && prop.components && prop.components.length > 1){
                        for(j = i + 1; j < suspForComp.length; j++){
                            prop2 = suspForComp[j];
                            if(prop2.hasOwnProperty("components") && prop2.components && prop2.components.length == 1) {
                                var index:int = prop.components.indexOf(prop2.components[0]);
                                if (index != -1) {
                                    prop2.settings ||= prop.settings;
                                }
                            }
                        }
                    }
                }
                //
                car.suspensionComponentsArray = suspForComp;
            }else {
                car.suspensionForComponents =  carJSON.suspensionForComponentsStructure;
            }
        }
    }

    private function initSuspensionData_old(car:Car, carJSON:Object):void{
        if(carJSON.hasOwnProperty("suspensionForComponentsStructure"))
        {
            if(carJSON.suspensionForComponentsStructure is Array)
            {
                var suspForComp : Array = carJSON.suspensionForComponentsStructure;
                var sortFunction : Function = function(a : Object, b : Object):Number
                {
                    return b.components.length - a.components.length
                };

                suspForComp.sort(sortFunction);
                //
                // parsing suspension settings for component groups
                var i:int;
                var j:int;
                var prop:Object;
                var prop2:Object;
                for(i = 0; i < suspForComp.length; i++){
                    prop = suspForComp[i];
                    if(prop.hasOwnProperty("components") && prop.components && prop.components.length > 1){
                        for(j = i + 1; j < suspForComp.length; j++){
                            prop2 = suspForComp[j];
                            if(prop2.hasOwnProperty("components") && prop2.components && prop2.components.length == 1) {
                                var index:int = prop.components.indexOf(prop2.components[0]);
                                if (index != -1) {
                                    prop2.settings = prop.settings;
                                }
                            }
                        }
                    }
                }

                var ln:int = suspForComp.length;
                for(i = 0; i < ln; i++){
                    prop = suspForComp[i];
                    if(prop.hasOwnProperty("components") && prop.components && prop.components.length > 1){
                        for(j = prop.components.length - 1; j > 0; j--){
                            prop2 = {components:[prop.components[j]], settings: prop.settings};
                            suspForComp.push(prop2);
                        }
                    }
                }
                //
                car.suspensionComponentsArray = suspForComp;
            }
            else {
                car.suspensionForComponents =  carJSON.suspensionForComponentsStructure;
            }
        }
    }


    private function parseGeneralInfoAndGetImagesPath(car:Car, carJSON:Object):String {
        /*if(carJSON.hasOwnProperty("newRenderCarId")){
            car.isEditable = false;
        }*/
        car.id = carJSON._id;
        car.model = carJSON.model;
        car.year = carJSON.year;
        car.startYear = carJSON.start_year;
        car.endYear = carJSON.end_year;
        car.name = carJSON.make;
        car.suspension = carJSON.suspension;
        car.defaultSuspension = carJSON.suspension;
        car.maxSuspension = carJSON.maxSuspension;
        car.limiterMaxSuspension = carJSON.maxSuspension;
        car.minSuspension = carJSON.minSuspension;
        car.limiterMinSuspension= carJSON.minSuspension;
        car.custom_offset = carJSON.custom_offset;
        car.defaultZoom = (carJSON.hasOwnProperty("startZoom") && carJSON.startZoom)?carJSON.startZoom: 0;
        car.bodyTypeList = Car.formatBodyType(carJSON.classicTypeApp);
        car.tintDisabled = carJSON.hasOwnProperty("tintDisable")?carJSON.tintDisable: false;
        if (car.hasOwnProperty('suspension')) {
            var susp : Number = carJSON.suspension;

            if (susp != 0) {
                susp = -car.maxSuspension + susp;
            }
            car.suspension = susp;
        }


        car.isFeatured = carJSON.featured != null;

        if(carJSON.hasOwnProperty("body"))
        {
            car.body = carJSON.body;
        }

        if(carJSON.hasOwnProperty("suspensionForState"))
        {
            car.suspensionForState =  carJSON.suspensionForState;
        }

        initSuspensionData(car, carJSON);



		if (carJSON.hasOwnProperty("carChooserBgName")) {
			car.carChooserBgName = carJSON.carChooserBgName;
		}
		else {
			car.carChooserBgName = null;
		}
		
		if(carJSON.hasOwnProperty("backgrounds")){
			car.backgrounds = carJSON.backgrounds;
		}
        if (carJSON.hasOwnProperty('useZipSplitCam')) {
            car.useZipSplitCam = carJSON.useZipSplitCam;
        } else {
            car.useZipSplitCam = false;
        }


		if (carJSON.hasOwnProperty("startFrame")) {
			car.startFrame = carJSON.startFrame;
		}
		else {
			car.startFrame = 0;
		}
		
		if(carJSON.hasOwnProperty("partMaterialZip")){
			car.partMaterialZip=carJSON.partMaterialZip;
		}else{
			car.partMaterialZip = false;
		}

        if (carJSON.hasOwnProperty('offsetDifference')) {
            car.offsetDifference = true;
        }

        if(carJSON.hasOwnProperty("partMaterialEncrypted")){
            car.partMaterialEncrypted=carJSON.partMaterialEncrypted;
        }else{
            car.partMaterialEncrypted = false;
        }
		
		ZipManager.getInctance().useForThisCar = car.partMaterialZip;
		ZipManager.getInctance().isEncrypted = car.partMaterialEncrypted;
        ZipManager.getInctance().useZipSplitCam = car.useZipSplitCam;

        if (carJSON.hasOwnProperty('haveSound')) {
            car.haveSound = carJSON.haveSound;
        } else{
            car.haveSound = false;
        }

        if (carJSON.hasOwnProperty('haveSoundRevs')) {
            car.hasSoundRevs = carJSON.haveSoundRevs;
        }

        if (!carJSON.displayName) {
            car.displayName = car.model;
        } else {
            car.displayName = carJSON.displayName;
        }

        if (CarsParser.CHOPPER_ID_LIST.indexOf(car.id) != -1) {
            car.isChopper = true;
        }else if (CarsParser.BEETLE_ID_LIST.indexOf(car.id) != -1) {
            car.isBeetle = true;
        }
        car.countAngels = 1;

        if (carJSON.hasOwnProperty("frameCount")) {
            car.countFrames = carJSON.frameCount;
        } else {
            car.countFrames = 50;
        }

        if (carJSON.hasOwnProperty("haveIconFolder")) {
            car.haveIconFolder = carJSON.haveIconFolder;
        } else {
            car.haveIconFolder = false;
        }
        if (car.isBeetle) {
            car.haveIconFolder = true;
        }
        if (carJSON.hasOwnProperty("camera2Frames")) {
            car.camera2Frames = carJSON.camera2Frames;
        }
		
		
		/////////////////////////////////////////////////

		var screenWidth:Number = Constants.STAGE.fullScreenWidth;
        if (car.countFrames == 50) {
            if (!car.camera2Frames) {
                car.camera2Frames = {"0": 0, "10": 12, "20": 24, "30": 32, "40": 48};
                car.stepOfFrameIndexUp = 10;
            }
			
			if(Device.isIos()){
				if ((Constants.TOTAL_MEMORY>=0 && Constants.TOTAL_MEMORY <= 1024))
				{
	                car.camera1Frames = {
	                    "0": 0,
	                    "5": 4,
	                    "10": 10,
	                    "15": 14,
	                    "20": 20,
	                    "25": 24,
	                    "30": 30,
	                    "35": 34,
	                    "40": 40,
	                    "45": 44
	                };
	
	                car.stepOfFrameIndex = 5;
	                car.stepOfFrameIndexUp = 10;
	            } else {
	                car.stepOfFrameIndex = 2;
	                car.stepOfFrameIndexUp = 10;
	            }
			} else {
				if ((screenWidth==1920 && Constants.TOTAL_MEMORY>=0 && Constants.TOTAL_MEMORY <= 2048) ||
					(Constants.TOTAL_MEMORY>=0 && Constants.TOTAL_MEMORY < 1538))
				{
					car.camera1Frames = {
						"0": 0,
						"5": 4,
						"10": 10,
						"15": 14,
						"20": 20,
						"25": 24,
						"30": 30,
						"35": 34,
						"40": 40,
						"45": 44
					};
					car.stepOfFrameIndex = 5;
					car.stepOfFrameIndexUp = 10;
				}
				else {
					car.stepOfFrameIndex = 2;
					car.stepOfFrameIndexUp = 12;
				}
			}
			
			car.camera1LayerPositionStep = 2;
			car.camera2LayerPositionStep = 12;
        } else if (car.countFrames == 48) {
            if (Device.isIos()) {
                if (Constants.TOTAL_MEMORY <=1024) {
                    car.stepOfFrameIndex = 4;
                    car.stepOfFrameIndexUp = 12;
                } else {
                    car.stepOfFrameIndex = 2;
                    car.stepOfFrameIndexUp = 12;
                }
            } else {
				if (screenWidth > 2100) {
					car.stepOfFrameIndex = 2;
					car.stepOfFrameIndexUp = 12;
				}
				else {
					if ((Constants.TOTAL_MEMORY>1200 && Constants.TOTAL_MEMORY <= 2048) ||
						(Constants.TOTAL_MEMORY>1200 && Constants.TOTAL_MEMORY < 1538))
					{
						car.stepOfFrameIndex = 4;
						car.stepOfFrameIndexUp = 12;
						
					} else if(Constants.TOTAL_MEMORY<=1200){
						car.stepOfFrameIndex = 4;
						car.stepOfFrameIndexUp = 12;
					}
					else {
						car.stepOfFrameIndex = 2;
						car.stepOfFrameIndexUp = 12;
					}
				}
            }

            if (!car.camera2Frames) {
                car.camera2Frames = {"0": 4, "12": 20, "24": 30, "36": 46};
                car.stepOfFrameIndexUp = 12;
            }
			
			car.camera1LayerPositionStep = 2;
			car.camera2LayerPositionStep = 12;
        }
     
		if (carJSON.hasOwnProperty("layersPositions")) {
			car.layersPositions = carJSON.layersPositions;
		}

		
		if(carJSON.hasOwnProperty("camera24Frames") && carJSON.camera24Frames || (car.layersPositions && car.layersPositions.length == 48)){
			car.camera24Frames = true;
		}

		if(car.camera24Frames){
		    car.camera2Frames = {
	            "0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
	            "20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
	            "40": 40, "42": 42, "44": 44, "46": 46
	        };
	
	        car.camera1Frames = {
	            "0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
	            "20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
	            "40": 40, "42": 42, "44": 44, "46": 46
	        };
			
			car.stepOfFrameIndex = 2;
			car.stepOfFrameIndexUp = 2;
			car.camera1LayerPositionStep = 2;
			car.camera2LayerPositionStep = 2;
			
			if(Device.isIos()){
				if ((Constants.TOTAL_MEMORY>0 && Constants.TOTAL_MEMORY <= 1024)) {
					car.stepOfFrameIndex = 4;
					car.stepOfFrameIndexUp = 4;
				}
			}else
			{
				if ((screenWidth==1920 && Constants.TOTAL_MEMORY>1200 && Constants.TOTAL_MEMORY <= 2048) ||
					(Constants.TOTAL_MEMORY>1200 && Constants.TOTAL_MEMORY < 1538) ) {
					car.stepOfFrameIndex = 4;
					car.stepOfFrameIndexUp = 4;
				}else if(Constants.TOTAL_MEMORY<=1200){
					car.stepOfFrameIndex = 6;
					car.stepOfFrameIndexUp = 6;
					car.camera2Frames = {
						"0": 0,
						"6": 6,
						"12": 12,
						"18": 18,
						"24": 24,
						"30": 30,
						"36": 36,
						"42": 42
					};

					car.camera1Frames = {
						"0": 0,
						"6": 4,
						"12": 12,
						"18": 16,
						"24": 24,
						"30": 28,
						"36": 36,
						"42": 42
					};
				}
			}
		}

        if (carJSON.hasOwnProperty("camera12Frames") && carJSON.camera12Frames == true) {
			car.camera12Frames = true;
			car.camera2Frames = {
			"0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
			"20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
			"40": 40, "42": 42, "44": 44, "46": 46
			};
			
			car.camera1Frames = {
			"0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
			"20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
			"40": 40, "42": 42, "44": 44, "46": 46
			};

			if(Device.isIos()){
				car.stepOfFrameIndex = 4;
				car.stepOfFrameIndexUp = 4;
			}else{
				if(Constants.TOTAL_MEMORY<=1200){
					car.stepOfFrameIndex = 6;
					car.stepOfFrameIndexUp = 6;
					car.camera2Frames = {
						"0": 0, 
						"6": 4, 
						"12": 12, 
						"18": 16,
						"24": 24, 
						"30": 28, 
						"36": 36, 
						"42": 44
					};
					
					car.camera1Frames = {
						"0": 0, 
						"6": 4, 
						"12": 12, 
						"18": 16,
						"24": 24, 
						"30": 28, 
						"36": 36, 
						"42": 44
					};
				}else{
					car.stepOfFrameIndex = 4;
					car.stepOfFrameIndexUp = 4;
				}
			}
			car.camera1LayerPositionStep = 4;
			car.camera2LayerPositionStep = 4;
        }

		///////////////////////////////////////////////////////
		if(!car.camera1Frames){
			if(car.countFrames == 50){
				car.camera1Frames = {
					"0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
					"20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
					"40": 40, "42": 42, "44": 44, "46": 46, "48": 48
				};
					
			}else if(car.countFrames == 48){
				car.camera1Frames = {
					"0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
					"20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
					"40": 40, "42": 42, "44": 44, "46": 46
				};
			}
		}

        if (carJSON.carchooserOrder && carJSON.carchooserOrder['US'])
            car.carchooserOrder = carJSON.carchooserOrder['US'];
        else
            car.carchooserOrder = carJSON.carchooserOrder;
        if (carJSON.hasOwnProperty('hdsize')){
            car.hdsize = carJSON.hdsize;
        }
        else{
            car.hdsize = false;
        }
        if(carJSON.hasOwnProperty("custom_offset"))
        {
            car.carType = carJSON.custom_offset;
        }

        if (carJSON.hasOwnProperty("features")) {
            car.carFeaturesNew = carJSON.features['new'];
            car.carFeaturesFull = carJSON.features['full'];

            car.topCameraIcon = carJSON.features['topviewcamera'];
            car.ultraIcon = carJSON.features['ultrarealistic'];
            car.offsetIcon = carJSON.features['offset'];
            car.stanceIcon = carJSON.features['stance'];

            if (car.topCameraIcon) {
                car.ultraIcon = false;
            }
        } else {
            car.carFeaturesNew = false;
            car.carFeaturesFull = false;
        }

        if (carJSON.hasOwnProperty("allMaterialChangeFallback")) {
            car.pizdec = carJSON.allMaterialChangeFallback;
        }

        if (carJSON.hasOwnProperty("isBigSize")) {
            car.isBigSize = carJSON.isBigSize;
        }
		
		if (carJSON.hasOwnProperty("fullsize")) {
			car.fullsize = carJSON.fullsize;
		}

        if (carJSON.hasOwnProperty("defaultBkgMobile")) {
            car.defaultBkgMobile = carJSON.defaultBkgMobile;
        }

        if (carJSON.hasOwnProperty("defaultBkg")) {
            car.defaultBkg = carJSON.defaultBkg;
        }

        if (carJSON.hasOwnProperty("hashTags")) {
            car.hashTags = carJSON.hashTags;
        }

        if (carJSON.hasOwnProperty("size")) {
            car.size = carJSON.size;
        }

        if (BrandsParser.allCarsList && !car.brandImageUrl) {
            if (BrandsParser.allCarsList[car.id])
                car.brandImageUrl = BrandsParser.allCarsList[car.id].brandImageUrl;
        }

        if (carJSON.hasOwnProperty("disks")) {
            car.disks = carJSON.disks;
        }

        if (carJSON.hasOwnProperty("disksPosition")) {
            car.disksPosition = carJSON.disksPosition;

            //Тут нужно еще добавить позиции для таир копируем их пока с wheel
            var newList:Object = {};
            var slot_name:String;

            for (slot_name in car.disksPosition) {
                var new_slot_name:String = slot_name.replace("wheel", "tair");
                newList[new_slot_name] = car.disksPosition[slot_name];
            }

            for (slot_name in newList) {

                car.disksPosition[slot_name] = newList[slot_name];
            }
        }

        car.componentsDependenciesWheel = [
            {"rear_wheel_left": ["rear_tair_left", "brake_rear_left", "tire_vinyl_rear_left"]},
            {"rear_wheel_right": ["rear_tair_right", "brake_rear_right", "tire_vinyl_rear_right"]},
            {"front_wheel_right": ["front_tair_right", "brake_front_right", "tire_vinyl_front_right"]},
            {"front_wheel_left": ["front_tair_left", "brake_front_left", "tire_vinyl_front_left"]}
        ];

        if (carJSON.hasOwnProperty("componentsDependencies")) {
            car.componentsDependencies = carJSON.componentsDependencies;
        }

        if (carJSON.images) {
            car.imageURL = carJSON.images.side; //main;
            car.featuredImageUrl = carJSON.images.slides2[0];
        }

        car.path = carJSON.folderName + "/";
        var imagesPath:String = Constants.IMAGES_URL + encodeURI(car.path);
		car.imagePath= imagesPath;
        car.path = "Cars/" + encodeURI(car.path);

        return imagesPath;
    }

    private function parseColors(car:Car, carJSON:Object):void {
        var defaultColors:Array = carJSON.factoryColors;

        if (defaultColors) {
            car.addPossiblePartColor(PartColor.customColor());

            var defaultColorsLength:int = defaultColors.length;
            // var isWhiteAppeared:Boolean = false;
            for (var i:int = 0; i < defaultColorsLength; i++) {
                var colorJSON:Object = defaultColors[i];

                var partColor:PartColor = new PartColor();
                var colorValue:uint = parseInt(colorJSON.value, 16) as uint;

                partColor.name = colorJSON.name;
                partColor.value = colorValue == 0xFFFFFF ? 0xFEFFFF : colorValue;
                /*if (partColor.value == 0xfeffff) {
                    isWhiteAppeared = true;
                    continue;
                }*/

                car.addPossiblePartColor(partColor);
            }

            /*
            if (!isWhiteAppeared) {
                car.addWhiteColor();
            }
            */
        }
    }

    public override function parseObject(carJSON:Object, key:String, ...args):Object {
        var componentsJSON:Object = args[0];
        var tuningJSON:Object = args[2];

        var car:Car = new Car();
        var imagesPath:String = parseGeneralInfoAndGetImagesPath(car, carJSON);

        if (carJSON.hasOwnProperty("type")) {
            car.type = carJSON.type;
        } else {
            car.type = "";
        }

        car.isChallenge = carJSON.hasOwnProperty("challengeNewSystemCar")?carJSON["challengeNewSystemCar"]:false;

        if(carJSON.hasOwnProperty("carType") && carJSON.carType == "wheel_only"){
            car.wheelConfigurator = true;
        }else if (carJSON.hasOwnProperty("wheelConfigurator")) {
            car.wheelConfigurator = carJSON.wheelConfigurator;
        }else{
            car.wheelConfigurator = CarsParser.wheelOnlyList[carJSON.id];

        }

        if (tuningJSON && tuningJSON.suspension) {
            car.suspension = tuningJSON.suspension * -1;
        }

        if(carJSON.hasOwnProperty("suvTires"))
        {
            car.suvTires = carJSON.suvTires;
        }

        if(carJSON.hasOwnProperty("tireMasks"))
        {
            car.tireMasks = carJSON.tireMasks;
        }

        if(carJSON.hasOwnProperty("carType"))
        {
            car.carType = carJSON.carType;
        }

        if (carJSON.hasOwnProperty("updateText")) {
            car.updateText = carJSON.updateText;
        }

        if (carJSON.hasOwnProperty("defaultBgId")) {
            car.defaultBgId = carJSON.defaultBgId;
        }

        // "updated": "2015-08-21T16:30:55.300Z",
        if (carJSON.hasOwnProperty("updated")) {
            var updated:String = carJSON.updated;
            updated = updated.replace("T", " ");
            updated = updated.replace("Z", " ");
            //2015-08-20 21:06:07.456
            updated = updated.substr(0, 19);

            updated = updated.replace("-", "/");
            updated = updated.replace("-", "/");

            var update_date:Date = new Date();
            update_date.setTime(Date.parse(updated));

            car.updated = update_date.time;
        }

        parseColors(car, carJSON);

        if (componentsJSON) {
            parseComponents(car, imagesPath, carJSON, componentsJSON, tuningJSON);
        }

        return car;
    }

    private function parseFiles(part:Part, addedPartSlot:PartSlot, car:Car, componentPath:String, componentJSON:Object):void {
        var partMaterialImage:PartMaterialImage;
		var shadowOffset:Number;
		var filesKey:String;
		var partMaterial:PartMaterial;
		var i:int;
		var filesList:Array;

        for (filesKey in componentJSON.files) {
            partMaterial = new PartMaterial(part);
            partMaterial.name = filesKey;
            //partMaterial.isPaintable = filesKey != "metal";

           
            partMaterial.componentPath = componentPath + partMaterial.name + "/";
            partMaterial.parentPart = part;

            shadowOffset = 0;
            if (addedPartSlot.name === 'shadow' || addedPartSlot.name.indexOf('shadow_') > -1) {
                shadowOffset = -SHADOW_OFFSET_X;
            }
			part.addPossibleMaterial(partMaterial);
			
			partMaterial.isPartMaterialImagesLoaded = false;
			
			if(Constants.COMPONENT_OPTIMIZE && !defaultComponentMap[part.id]) continue;

			filesList = componentJSON.files[filesKey];
            for (i=0; i<filesList.length; i++) {
             	partMaterial.addImage(filesList[i], car.stepOfFrameIndex,shadowOffset);
            }

			partMaterial.isPartMaterialImagesLoaded = true;
        }

		if(partMaterial){
			if (addedPartSlot.name == "front_bumper") {
				existsFrontBamper = true;
				partMaterialImage = partMaterial.getImage(0);
				
				if(partMaterialImage)
					car.frontBumperX = partMaterialImage.x;
			}
			
			if (addedPartSlot.name == "black") {
				if(!partMaterialImage)
					partMaterialImage = partMaterial.getImage(0);
				
				if(partMaterialImage)
					shadowBlackX = partMaterialImage.x; 
			}
		}
		
		partMaterialImage = null;
    }

    private var existsFrontBamper:Boolean = false;
    private var shadowBlackX:Number = 0;
    public static var carArray:Array;
	private var	defaultComponentMap:Object;

    private function parseComponents(car:Car, imagesPath:String, carJSON:Object, componentsJSON:Object, tuningJSON:Object):void {
        var i:int;
        PartSlot.TWO_TONE_VINYLS_LIST = [];

        if(carJSON.components){
            var str:String;
            for(i = 0; i < carJSON.settings.length; i++){
                var componentInfoJSON:Object = carJSON.settings[i];
                if(componentInfoJSON && componentInfoJSON.componentType==PartSlot.VINYL_GROUP){
                    if(componentInfoJSON.hasOwnProperty("grouping")) {
                        str = componentInfoJSON.grouping;
                        str = str.replace(/\s+/gms, '');
                        PartSlot.VINYLS = str.split(',');
                        PartSlot.VINYLS.push(componentInfoJSON);
                    }
                }else if(componentInfoJSON && componentInfoJSON.componentType==PartSlot.TWO_TONE_VINYL_GROUP){
                    if(componentInfoJSON.hasOwnProperty("grouping")) {
                        str = componentInfoJSON.grouping;
                        str = str.replace(/\s+/gms, '');
                        PartSlot.TWO_TONE_VINYLS_LIST = str.split(',');
                        PartSlot.TWO_TONE_VINYLS_LIST.push(componentInfoJSON);
                    }
                }
            }
        }

		defaultComponentMap = {};
        var usedPartSlotsMap:Object = {};
        var partsMap:Dictionary = new Dictionary(true);
        var components:Array = componentsJSON as Array;
        var componentsLength:int = components.length;
        if (carJSON.hasOwnProperty("newMask")) {
            car.isNewMask = carJSON.newMask;
        } else {
            car.isNewMask = false;
        }

        var bodyId:String = null;
        var addedPartSlot:PartSlot;
        var partSlotDisk:PartSlotDisk = null;
        var part:Part;
        var partDisk:PartDisk;
        var partSlotName:String;
        var partSlot:PartSlot;

        existsFrontBamper = false;
		
		if(Constants.COMPONENT_OPTIMIZE){
			var selectedComponents:Array = tuningJSON ? tuningJSON.components : carJSON.components;
			var defaultComponentJSON:Object;
			
			for each(defaultComponentJSON in selectedComponents){
				if(!defaultComponentJSON) continue;
				defaultComponentMap[defaultComponentJSON.component] = true;
			}
		}
		PartMaterial.ResetPartMaterialImageMem();

        for (var k:int = 0; k < componentsLength; k++) {
            var componentJSON:Object = components[k];
            if (componentJSON.type == "podium") {
                continue;
            }

            addedPartSlot = usedPartSlotsMap[componentJSON.type];

            if (!addedPartSlot) {
                if (Car.DISKS_SLOTS.indexOf(componentJSON.type) >= 0) { //Для слота "Диски" используем расширеную модель
                    partSlotDisk = new PartSlotDisk(car);
                    partSlotDisk.parentCar = car;
                    addedPartSlot = partSlotDisk;
                } else {
                    addedPartSlot = new PartSlot(car);
                    addedPartSlot.parentCar = car;
                }
                addedPartSlot.name = componentJSON.type;
                usedPartSlotsMap[componentJSON.type] = addedPartSlot;
            }

            if(componentJSON.type == "bg")
            {
                carArray = [];
                carArray.push(componentJSON);
                continue;
            }
            if (componentJSON.type == PartSlot.BODY_PACK || componentJSON.type == PartSlot.FACTORY_TRIM) {
                var isDiskSubpart:Boolean = false;
                for each(var nested:Object in componentJSON.nesteds) {
                    if (nested.folderName == "disks") {
                        isDiskSubpart = true;
                        break;
                    }
                }

                if (componentJSON["additionalTypes"] && componentJSON.additionalTypes.indexOf('disks') < 0 && isDiskSubpart) {
                    componentJSON.additionalTypes.push('disks');
                }
            } //Тут пока не понятно как сделать

            if (Car.DISKS_SLOTS.indexOf(componentJSON.type) >= 0) {
                part = new PartDisk(addedPartSlot);
            } else {
                part = new Part(addedPartSlot);
            }
            part.name = componentJSON.name;
            part.removeTypes = componentJSON["removeTypes"]||[];
            part.removeTypesOnRemove = componentJSON["removeTypesOnRemove"]||[];
            part.addTypesOnRemove = componentJSON["addTypesOnRemove"]||[];
            part.priceString = componentJSON["priceString"]||null;

			if(componentJSON.hasOwnProperty("universalIcons") ){
				part.universalIcons = componentJSON.universalIcons;
			}else{
				part.universalIcons = false;
			}

            part.excludedPartSlotsNames = componentJSON.additionalTypes;


            if (componentJSON.hasOwnProperty("sortKey")) {

                part.sortKey = parseInt(componentJSON.sortKey);
            }

            if (part.name.charAt(part.name.length - 1) == " ") {
                part.name = part.name.substr(0, part.name.length - 1);
            }
			
            part.id = componentJSON._id;
            part.parentSlot = addedPartSlot;
            part.folder = encodeURI(componentJSON.folderName);

            if (part.parentSlot.name == "body" && bodyId == null) {
                bodyId = part.id;
            }
            if (componentJSON.hasOwnProperty("manufacturer")) {
                if (componentJSON.manufacturer == '5508389948c99bfc7e23fe0c') {
                    part.manufacturer = null;
                } else {
                    part.manufacturer = componentJSON.manufacturer;
                }
            }
           
            part.isDefault = (componentJSON.folderName == "default");

            var partImageFileName:String = part.isDefault ? "default" : part.folder;

            part.imageUrl = "";
            part.imageUrl += part.parentSlot.name;
            part.imageUrl += "/";
            part.imageUrl += componentJSON.folderName;
            part.imageUrl += "/";

            part.parentSlotName = part.parentSlot.name;

            var componentPath:String = part.imageUrl;

            part.imageUrl += partImageFileName;
            part.imageUrl += "_big.png";

            if (part.parentSlot.name == "body") {
                part.imageUrl = Constants.IMAGES_URL + "body_big.png";
            }
            else if (part.parentSlot.name == "windows") {
				part.imageUrl = Constants.IMAGES_URL + "windows_big.png";
				trace("windows:",part.remoteUrl)
            }
            part.imageUrl += "?v=1";
            part.isEnabled = componentJSON.isEnabled || (!UsersProxy.loggedInUser() || !UsersProxy.loggedInUser().isSuperAdmin);
            if (componentJSON.hasOwnProperty("properties") && componentJSON.properties != null) {
                part.properties = componentJSON.properties;
            }

            if (Car.DISKS_SLOTS.indexOf(componentJSON.type) >= 0) {  //Диски обрабатываются расширеной моделью
                partDisk = part as PartDisk;
                partDisk.selectedDiskSize = 0;

                partDisk.radius = 0;// Если радиус не задан ставим нуль
                if (part.properties && part.properties.hasOwnProperty('size') && part.properties.size) {
                    partDisk.radius = part.properties.size;
                }
            }

            if (componentJSON.hasOwnProperty("affectedComponents")) {
                part.affectedSlots = componentJSON.affectedComponents;
            } else {
                part.affectedSlots = [];
            }

            partsMap[part.id] = part;
            addedPartSlot.addPossiblePart(part);

			if(componentJSON.hasOwnProperty("multiselect")){
				part.multiselect = componentJSON.multiselect;
				if(part.multiselect) trace("multiselect true");
			}

            if(componentJSON.hasOwnProperty("activeForInformation")){
                part.activeForInformation = componentJSON.activeForInformation;
                if(part.activeForInformation) {
                    // DevHelper._log("activeForInformation true");
                }
            }

            CarsParser.componentJSONRestoreOptimizedMaterials(componentJSON);

            parseFiles(part, addedPartSlot, car, componentPath, componentJSON);
            parseSubparts(part, componentJSON, componentPath, car);
            parseLinked(part, addedPartSlot, car, imagesPath, componentJSON, components);
            parseStates(part, componentJSON, componentPath, car);

            if (part.getPossibleMaterial("mask") != null) {
                addedPartSlot = usedPartSlotsMap['masks'];

                if (!addedPartSlot) {
                    addedPartSlot = new PartSlot(car);
                    addedPartSlot.parentCar = car;
                    addedPartSlot.name = 'masks';
                    usedPartSlotsMap['masks'] = addedPartSlot;
                    addedPartSlot.isEditable = false;
                    addedPartSlot.layerIndex = 200;

                    addedPartSlot.selectedColor = PartColor.noColor();
                }
                var maskPartMaterial:PartMaterial = part.getPossibleMaterial("mask");

                if (Car.DISKS_SLOTS.indexOf(part.parentSlot.name) >= 0) {
                    part = new PartDisk(addedPartSlot);
                    part.name = maskPartMaterial.parentPart.name;

                    (part as PartDisk).radius = (maskPartMaterial.parentPart as PartDisk).radius;
                    (part as PartDisk).selectedDiskSize = (maskPartMaterial.parentPart as PartDisk).selectedDiskSize;

                    part.id = maskPartMaterial.parentPart.id + "_";
                    part.folder = maskPartMaterial.parentPart.folder;

                    part.nameOfState = "";
                    part.maskForSlots = maskPartMaterial.parentPart.affectedSlots;
                    part.imageUrl = "";
                    partsMap[part.id] = part;
					
                    addedPartSlot.addPossiblePart(part);
                    part.addPossibleMaterial(maskPartMaterial);
                    part.selectedMaterial = maskPartMaterial;
                } else {
                    part = new Part(addedPartSlot);

                    part.id = maskPartMaterial.parentPart.id + "_";
                    part.folder = maskPartMaterial.parentPart.folder;
                    part.name = maskPartMaterial.parentPart.name;
                    part.nameOfState = "";
                    part.maskForSlots = maskPartMaterial.parentPart.affectedSlots;
                    part.imageUrl = "";
                    partsMap[part.id] = part;

                    addedPartSlot.addPossiblePart(part);
                    part.addPossibleMaterial(maskPartMaterial);
                    part.selectedMaterial = maskPartMaterial;
                }
            }
			car.frontBumperX = shadowBlackX;
        }

        //Тут парсим маску как дополнительный слот, если она есть
        {

            if (carJSON.hasOwnProperty("masks")) {

                for (var maskKey:int in carJSON.masks) {

                    var masksJSON:Object = carJSON.masks[maskKey];

                    addedPartSlot = usedPartSlotsMap['masks'];

                    if (!addedPartSlot) {
                        addedPartSlot = new PartSlot(car);
                        addedPartSlot.parentCar = car;

                        addedPartSlot.layerIndex = 200;
                        addedPartSlot.name = 'masks';
                        usedPartSlotsMap['masks'] = addedPartSlot;
                        addedPartSlot.isEditable = false;

                        addedPartSlot.selectedColor = PartColor.noColor();
                    }

                    part = new Part(addedPartSlot);

                    part.id = masksJSON._id;
                    part.folder = encodeURI(masksJSON.folderName);
                    part.name = masksJSON.state;
                    part.nameOfState = masksJSON.state;
                    part.maskForSlots = masksJSON.componentTypes;

                    part.imageUrl = "";

                    partsMap[part.id] = part;
                    addedPartSlot.addPossiblePart(part);

                    var baseUrl:String;
                    baseUrl = '';
                    baseUrl += "masks/disks/";
                    baseUrl += part.nameOfState;
                    baseUrl += "/";

                    for (var filesKey:String in masksJSON.files) {
                        var partMaterial:PartMaterial = new PartMaterial(part);
                        partMaterial.name = filesKey;
                       // partMaterial.isPaintable = filesKey != "metal";

                        var imageURLPrefix:String = baseUrl;
                        imageURLPrefix += partMaterial.name;
                        imageURLPrefix += "/";

                        partMaterial.componentPath = imageURLPrefix;
						
                        for each(var imageURL:String in masksJSON.files[filesKey]) {
                            partMaterial.addImage(imageURL, car.stepOfFrameIndex );
                        }
						
                        part.addPossibleMaterial(partMaterial);

                        if (!part.selectedMaterial)
                            part.selectedMaterial = partMaterial;
                    }
                }
            }
        }

        //Тут маска и заканчивается

        for (partSlotName in usedPartSlotsMap) {
            partSlot = usedPartSlotsMap[partSlotName] as PartSlot;
            partSlot.afterParseAllSlot();
        }

        parseSelectedComponents(car, bodyId, usedPartSlotsMap, partsMap, carJSON, tuningJSON);

        var bodyPart:Part = partsMap[bodyId];
		var partId:String;
		var linked_id:String;
		var z:int = 0;
		var linkedPart:Part;
		var linkedPartMaterial:PartMaterial;
		var emptyPartMaterial:PartMaterial;
		var j:int = 0;
		var find:Boolean = false;
        for (partId in partsMap) {
            part = partsMap[partId] as Part;
            if (part && part.parentSlot && part.parentSlot.selectedColor && part.parentSlot.selectedColor.value == 0xFEFFFF) {
                if (part.selectedMaterial && part.selectedMaterial.name != "metal" && part.parentSlot.name != "windows")
                    part.parentSlot.selectedColor.value = bodyPart.parentSlot.selectedColor.value;
            }
            //Ищем и добавляем в linkedParts

            if (part && part.linked && part.linked.length > 0) {
                for (i = 0; i < part.linked.length; i++) {
                    linked_id = part.linked[i];
                    if (part.linkedParts.indexOf(partsMap[linked_id]) < 0) {
                        if (part.possibleMaterials.length == 0) {
                            linkedPart = partsMap[linked_id];
                            for (z = 0; z < linkedPart.possibleMaterials.length; z++) {
                                linkedPartMaterial = linkedPart.possibleMaterials[z];
                                if (!part.getPossibleMaterial(linkedPartMaterial.name)) {
                                    emptyPartMaterial = new PartMaterial(part);
                                    emptyPartMaterial.name = linkedPartMaterial.name;
                                    //emptyPartMaterial.isPaintable = linkedPartMaterial.isPaintable;
                                    emptyPartMaterial.componentPath = "";
                                    part.addPossibleMaterial(emptyPartMaterial);
                                }
                            }
                        }
						var casddascasc : * = partsMap[linked_id];
						if(casddascasc != null)
						{
	                        part.linkedParts.push(casddascasc);
							casddascasc.parentLinkedParts.push(part);
						}
                    }
                }
            }

            var newExcludePartSlot:Array = [];

            if (part.excludedPartSlotsNames) {
                for (j = 0; j < part.excludedPartSlotsNames.length; j++) {
                    find = false;
                    for (z = 0; z < part.linkedParts.length; z++) {
                        linkedPart = part.linkedParts[z];
                        if (linkedPart.parentSlot.name == part.excludedPartSlotsNames[j]) {
                            find = true;
                            break;
                        }
                    }

                    if (!find) {
                        newExcludePartSlot.push(part.excludedPartSlotsNames[j]);
                    }
                }
                part.excludedPartSlotsNames = newExcludePartSlot;
            }
        }

        var frameIndex:int;
        var nextFrameIndex:int = 2000;
		var slotNames:Array;
		
        if (car.layersPositions && car.layersPositions.length > 0) {

            for (i = 0; i < car.layersPositions.length; i++) {
                slotNames = car.layersPositions[i];
				
                for (partSlotName in usedPartSlotsMap) {
                    partSlot = usedPartSlotsMap[partSlotName] as PartSlot;

                    frameIndex = slotNames.indexOf(partSlot.name);

                    if (partSlot.name == "mask") frameIndex = 2500;

                    if (frameIndex < 0) {
                        frameIndex = nextFrameIndex;
                    } else {
                        frameIndex = frameIndex * 10;
                    }

                    partSlot.layerIndexForFrames.push(frameIndex);

                }
            }

            for (partSlotName in usedPartSlotsMap) {

                partSlot = usedPartSlotsMap[partSlotName] as PartSlot;
            }
        }


		_partsMap = partsMap;
		
		var startTime:int = getTimer();
        this.prepareAll(car);
		updateExcludeForTuning(car, bodyId, usedPartSlotsMap,partsMap,carJSON, tuningJSON, components);
		trace("prepareAll time:", getTimer()-startTime );
		
		partsMap = null;
		_partsMap = null;

        MemoryTracker.gcAndCheck("CarsParser.parseComponents()");
    }
	
	private var _partsMap:Dictionary;

	private function getPartById(part_id:String, car:Car):Part {
		var partSlot:PartSlot;
		var part:Part;
		
		if(_partsMap) return _partsMap[part_id];
		
		for each(partSlot in car.possiblePartSlots) {
			
			if (partSlot) {
				part = partSlot.getPartById(part_id);
				
				if (part) return part;
			}
		}
		return null;
	}

    private function prepareAll(car:Car):void {

        var i:int = 0;
        var j:int = 0;
        var partSlot:PartSlot;

        var part:Part;
        var newPart:Part;
        var partSlotIsDependence:Boolean;
        var partSlotIsWide:Boolean;
        var depId:String;

        for each(partSlot in car.possiblePartSlots) {
			
			if(!partSlot) continue;
			if(partSlot.name == "masks") continue;
			if(partSlot.name == "antimask") continue;
			
            partSlotIsDependence = false;
            partSlotIsWide = false;

            for (j = 0; j < partSlot.possibleParts.length; j++) {
                part = partSlot.possibleParts[j];

                if (part.wide) partSlotIsWide = true;
                if (part.dependence as String) {
                    depId = part.dependence;
                    newPart = getPartById(depId, car);
	                if (newPart && newPart.depsMap.indexOf(partSlot.name) < 0) {
						newPart.depsMap.push(partSlot.name);
                    }
                } else if (part.dependence.length > 0) {
                    for (i = 0; i < part.dependence.length; i++) {
                        depId = part.dependence[i];
                        newPart = getPartById(depId, car)//partSlot.getPartById(depId);
                        if (newPart && newPart.depsMap.indexOf(partSlot.name) < 0) newPart.depsMap.push(partSlot.name);
                    }

                    part.isDependence = true;
                    partSlotIsDependence = true;
                } 
            }

            partSlot.isDependence = partSlotIsDependence;
            partSlot.isWide = partSlotIsWide;
        }

        var rearWheelPartSlot:PartSlot = car.getPossiblePartSlot("rear_wheel");
        if (rearWheelPartSlot) {
            for each(partSlot in car.possiblePartSlots) {
                if (partSlot == rearWheelPartSlot) continue;
                if (!partSlot.isWide) continue;

                for (j = 0; j < rearWheelPartSlot.possibleParts.length; j++) {
                    part = rearWheelPartSlot.possibleParts[j];
                    if (part.depsMap.indexOf(partSlot.name) < 0) {
                        part.depsMap.push(partSlot.name);
                    }
                }
            }
        }
    }

    private function parseLinked(part:Part, addedPartSlot:PartSlot, car:Car, imagesPath:String, componentJSON:Object, components:Array):void {
        if (!componentJSON.linked)
            return;

        part.linked = componentJSON.linked;
    }


    private function findComponentById(id:String, components:Array):Object {
		var i:int;
		var comp:Object;
        for (i = 0; i < components.length; i++) {
            comp = components[i];
            if (comp._id == id) {
                return comp;
            }
        }
        return null;
    }


    private function orderLastName(a:Object, b:Object):int {
        var a_num:int = a.layerNumber;
        var b_num:int = b.layerNumber;
        if (a_num < b_num) {
            return -1;
        }
        else if (a_num > b_num) {
            return 1;
        }
        else {
            return 0;
        }
    }
	
	private function findInArrayByType(partsMap:Dictionary, components:Array, type:String):Part{
		
		var component:Object;
		var part:Part;
		
		for each (component in components){
			part = partsMap[component.component];
			if(!part) continue;
			if(part.parentSlot.name==type) return part;
		}
		
		return part;
	}
	
	private function updateExcludeForTuning(car:Car, bodyId:String, usedPartSlotsMap:Object, partsMap:Dictionary, carJSON:Object, tunningJSON:Object, components:Array):void{
		if(!tunningJSON) return;
		
		var partSlots:Array = car.possiblePartSlots;
		var partSlot:PartSlot;
		var selPart:Part;
		var bodyPart:Part = partsMap[bodyId];
		var componentJSON:Object;
		for each(partSlot in partSlots){
			if (partSlot.selectedPart && partSlot.selectedPart.excludedPartSlotsNames) {
				for each(var excludedPartSlotName:String in partSlot.selectedPart.excludedPartSlotsNames) {
					var excludedPartSlot:PartSlot = car.getPossiblePartSlot(excludedPartSlotName);
					if (partSlot.selectedPart.excludedPartSlotsNames.indexOf(excludedPartSlot.name) >= 0) {
						if (!excludedPartSlot.selectedPart) {
							selPart = findInArrayByType(partsMap, carJSON.components, excludedPartSlot.name);
							if(selPart){
								excludedPartSlot.isExcluded = true;
								excludedPartSlot.oldBeforeExludeSelectedPart = selPart //partsMap[component.component];
								componentJSON = this.findComponentById(selPart.id, components);
								
								if(componentJSON){
									selPart.parseFiles(componentJSON);
									selPart.parseSubparts(componentJSON);
									selPart.parseStates(componentJSON);
								}

								if(excludedPartSlot.selectedPart && excludedPartSlot.selectedPart.selectedMaterial){	
									var partMaterial:PartMaterial = selPart.getPossibleMaterial(excludedPartSlot.selectedPart.selectedMaterial.name);
									if(partMaterial){
										selPart.selectedMaterial =  partMaterial;
										if(excludedPartSlot.selectedPart.color){
											selPart.color = excludedPartSlot.selectedPart.color;	
										}
										
										if(!selPart.color && bodyPart){
											selPart.color = bodyPart.color.copy();
										}
									}
								}
							}
						}
					}
				}
			}
			
			
			if (partSlot.selectedPart && partSlot.selectedPart.linkedParts.length > 0) {
				for (var i:int = 0; i < partSlot.selectedPart.linkedParts.length; i++) {
					var linkedPart:Part = partSlot.selectedPart.linkedParts[i];

					selPart = findInArrayByType(partsMap, carJSON.components, linkedPart.parentSlot.name);
					if(selPart){
						if(selPart.parentSlot.name==linkedPart.parentSlot.name){
							trace("ВОССТАНОВИЛИ ДЕФАУЛТ ДЛЯ ",  linkedPart.parentSlot.name)
							trace("selPart:", selPart.name, selPart.id, selPart.parentSlot.name);
							
							linkedPart.parentSlot.savePartLinked = selPart;
							
							componentJSON = this.findComponentById(selPart.id, components);
							
							if(componentJSON){
								selPart.parseFiles(componentJSON);
								selPart.parseSubparts(componentJSON);
								selPart.parseStates(componentJSON);
							}

							if(linkedPart.selectedMaterial){	
								selPart.selectedMaterial =  selPart.getPossibleMaterial(linkedPart.selectedMaterial.name);
								if(selPart.selectedMaterial){
									if(linkedPart.color){
										selPart.color = linkedPart.color;	
									}
									
									if(!selPart.color && bodyPart && bodyPart.color){
										
										selPart.color = bodyPart.color.copy();
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
    private function parseSelectedComponents(car:Car, bodyId:String, usedPartSlotsMap:Object, partsMap:Dictionary, carJSON:Object, tunningJSON:Object):void {
        var selectedComponents:Array = tunningJSON ? tunningJSON.components : carJSON.components;
        var bodyColor:int = -1;
        var bodyComponent:Object = null;
        var colorString:String;
        var component:Object;
        var partSlot:PartSlot;
        var disksLayerIndex:int;
        var componentInfoJSON:Object;
        var defaultComponentJSON:Object;
        var parentSlot:PartSlot;
		var bodyPartColor:PartColor;
		var arrOfSettings:Array;
		var part:Part;

        if (bodyId == null && selectedComponents && selectedComponents.length > 0) bodyId = selectedComponents[0].component;

		for each(component  in selectedComponents) {
			
			part =partsMap[component.component]
			if (part && part.parentSlot.name=="body" /*component.component == bodyId*/) {
				bodyId = part.id;
				bodyComponent = component;
				bodyColor = uint(parseInt(component.color, 16));
				bodyColor = bodyColor == 0xFFFFFF ? 0xFEFFFF : bodyColor;
			}
		}

        bodyPartColor = car.getPossiblePartColor(bodyColor);

        if (!bodyPartColor) {
            bodyPartColor = PartColor.customColor();
            bodyPartColor.name = "color";
            bodyPartColor.value = bodyColor;
        }

        car.bodyPartColor = bodyPartColor.copy();

        if (carJSON.settings) {
            car.maxPartSlotLayerIndex = -1;
            arrOfSettings = carJSON.settings;
            arrOfSettings.sort(orderLastName);
            var layerNumber:int = 1;
            for each(componentInfoJSON in arrOfSettings) {
              
                partSlot = usedPartSlotsMap[componentInfoJSON.componentType];
				if(!partSlot) continue;
				if(componentInfoJSON.componentType == "podium") continue;
                disksLayerIndex = 0;
                if (Car.DISKS_SLOTS.indexOf(partSlot.name) >= 0) {
                    layerNumber++;
                }

                partSlot.name = componentInfoJSON.componentType;
                partSlot.displayName = displayNameForComponentType(componentInfoJSON.componentType);
                partSlot.layerIndex = componentInfoJSON.layerNumber * 10;


                if (componentInfoJSON.sortKey) {
                    partSlot.sortKey = componentInfoJSON.sortKey;
                }

                partSlot.isEditable = componentInfoJSON.isEditable || partSlot.name == "body";
                partSlot.isMandatory = componentInfoJSON.isMandatory;

                if (componentInfoJSON.hasOwnProperty('lockSuspension')) {
                    partSlot.lockSuspension = componentInfoJSON.lockSuspension;
                    if(partSlot.isAffectedBySuspension && partSlot.lockSuspension){
                        partSlot.isAffectedBySuspension = !componentInfoJSON.lockSuspension;
                    }
                }

                if (componentInfoJSON.hasOwnProperty('unselectableComponent')) {
                    partSlot.unselectableComponent = componentInfoJSON.unselectableComponent;
                }

                if (componentInfoJSON.hasOwnProperty('hideFromMenu')) {
                    partSlot.hideFromMenu = componentInfoJSON.hideFromMenu;
                }

                if (componentInfoJSON.hasOwnProperty('strictShowInMenu')) {
                    partSlot.strictShowInMenu = componentInfoJSON.strictShowInMenu;
                }

                partSlot.turnFiveDesign = componentInfoJSON["turnFiveDesign"]||false;

                partSlot.bestViewFrameIndex = int(componentInfoJSON.position/2);
                partSlot.selectedColor = PartColor.noColor();
                car.maxPartSlotLayerIndex = Math.max(car.maxPartSlotLayerIndex, partSlot.layerIndex);
                if (partSlot.name.indexOf(PartSlot.AEROGRAPHY)>=0 || partSlot.name.indexOf(PartSlot.TWO_TONE_VINYL_TYPE)>=0) {
                    partSlot.isMandatory = false;
                    partSlot.selectedColor = PartColor.customColor();
                }

                if (bodyColor >= 0 && partSlot.name != PartSlot.AEROGRAPHY && partSlot.name != PartSlot.TWO_TONE_VINYLS && PartSlot.VINYLS.indexOf(partSlot.name)==-1  && PartSlot.TWO_TONE_VINYLS_LIST.indexOf(partSlot.name)==-1 && partSlot.name.indexOf(PartSlot.TWO_TONE_VINYL_TYPE) == -1) {
                    partSlot.selectedColor = bodyPartColor.copy();
                }
                partSlot.sizeInMegabytes = componentInfoJSON.size ? componentInfoJSON.size : 0.0;
                car.addPossiblePartSlot(partSlot);
            }

            partSlot = usedPartSlotsMap['masks'];

	        if (partSlot) {
	            partSlot.name = "masks";
	            partSlot.layerIndex = 2000;
	            car.maxPartSlotLayerIndex = Math.max(car.maxPartSlotLayerIndex, partSlot.layerIndex);
	            car.maxPartSlotLayerIndex++;
	            partSlot.isEditable = false;
	            partSlot.isMandatory = false;
	            partSlot.isMask = true;
	            partSlot.isAffectedBySuspension = true;
	            car.addPossiblePartSlot(partSlot);
	        }
        }
		
		var addedPart:Part;
		var partDiskSlot:PartSlotDisk;
		var partDisk:PartDisk;
		var selectedRadiusPartDisk:PartDisk;
		var color:PartColor;
		var selectedTheState:Boolean;
		var nameOfState:String;
		var objectOfState:Object;
		var active:Boolean = false;
		var partState:Part;
		var selectedPartColor:PartColor;
		var colorValue2:uint;
		var material:PartMaterial;

        for each(defaultComponentJSON in selectedComponents) {
            addedPart = partsMap[defaultComponentJSON.component];
            partDisk = null;
            if (!addedPart)
                continue;

            if (Car.DISKS_SLOTS.indexOf(addedPart.parentSlot.name) >= 0) {
                selectedRadiusPartDisk = addedPart as PartDisk;
                partDiskSlot = addedPart.parentSlot as PartSlotDisk;
                if(selectedRadiusPartDisk.main) {
                    partDisk = selectedRadiusPartDisk;
                }
                partDisk ||= partDiskSlot.findMainPartDiskByName(addedPart.name);
                if (!partDisk) {
                    var prefix:String = PartSlotDisk.getPrefixDep(addedPart);
                    partDisk = partDiskSlot.findMainPartDiskByName(addedPart.name + prefix);
                }
                partDisk ||= partDiskSlot.factoryDiskPart;
                partDisk ||= selectedRadiusPartDisk;
                if (partDisk) {
                    partDisk.selectedDiskSize = selectedRadiusPartDisk.radius;
                    addedPart = partDisk;
                }
            }
            parentSlot = addedPart.parentSlot;

            try {

                if (Car.DISKS_SLOTS.indexOf(parentSlot.name) >= 0) {

                    addedPart.selectedMaterial = addedPart.getPossibleMaterial(defaultComponentJSON.material);
                } else {

                    if (!addedPart.selectedMaterial) {
                        addedPart.selectedMaterial = addedPart.getPossibleMaterial(defaultComponentJSON.material);
                    } else if (addedPart.selectedMaterial.name != 'glossy') {

                        addedPart.selectedMaterial = addedPart.getPossibleMaterial(defaultComponentJSON.material);
                    }
                }


            } catch (e:Error) {
				Tracer.print("error:", e.message/*, e.getStackTrace()*/);
                if (!addedPart)  continue;
            }

            if (!addedPart.selectedMaterial) {
				addedPart.selectedMaterial = selectPossibleMaterial(addedPart.possibleMaterials);
            }

            if (Car.DISKS_SLOTS.indexOf(parentSlot.name) >=0) {
                addedPart.selectedMaterial = selectPossibleMaterial(addedPart.possibleMaterials, parentSlot.name);
            }

            addedPart.brightness = defaultComponentJSON.hasOwnProperty("brightness") ? defaultComponentJSON.brightness : 1.0;

            if (parentSlot.name == PartSlot.AEROGRAPHY/* || parentSlot.name == PartSlot.TWO_TONE_VINYLS*/) {
				
                if (!parentSlot.selectedParts)
                    parentSlot.selectedParts = [];

                if (addedPart.saveIndex == "")
                    addedPart.saveIndex = (Math.random() * 10000000000).toFixed(0);

                colorString = defaultComponentJSON.color;

                if (colorString == null) colorString = "ffffff";
                colorValue2 = parseInt(colorString, 16) as uint;
                color = new PartColor();
                color.name = "color";
                color.value = colorValue2 == 0xFFFFFF ? 0xFEFFFF : colorValue2;
                addedPart.color = color;
                parentSlot.selectedParts.push(addedPart);
               
            } else {
                parentSlot.selectedPart = addedPart;
            }

            car.addSelectedPartSlot(parentSlot);
            /*
				тут надо обработать states
				пример:
				defaultComponentJSON.component = 5319acfd69584a8803371238
				defaultComponentJSON.id = null
				defaultComponentJSON.material = metal;
				defaultComponentJSON.states = statesObject;
				statesObject.lights.active = true;


			*/
            addedPart.selectedStates = defaultComponentJSON.states;
            selectedTheState = false;

            if (addedPart.selectedStates) {
                for (nameOfState in addedPart.selectedStates) {
                    //nameOfState lights
                    objectOfState = addedPart.selectedStates[nameOfState];
                    active = false;
                    if (objectOfState && objectOfState.hasOwnProperty("active")) {
                        active = objectOfState.active;
                    }

                    if (active) {
                        partState = addedPart.states[nameOfState];
                        addedPart.selectedState = partState;
                        selectedTheState = true;
                        //обрабатываем только одно (возможно других вариантов и нет)
                        break;
                    }
                }
            }

            colorString = defaultComponentJSON.color;
            if (defaultComponentJSON.material == "metal") {
                colorString = null;
            }

            if (parentSlot.name == PartSlot.AEROGRAPHY || parentSlot.name == PartSlot.TWO_TONE_VINYLS || PartSlot.VINYLS.indexOf(parentSlot.name)==-1 || PartSlot.TWO_TONE_VINYLS_LIST.indexOf(parentSlot.name)==-1 || parentSlot.name.indexOf(PartSlot.TWO_TONE_VINYL_TYPE) == -1) {
                if (colorString == null) colorString = "ffffff";
            }


           

            if (colorString) {
                colorValue2 = parseInt(colorString, 16) as uint;
                selectedPartColor = car.getPossiblePartColor(colorValue2);
                if (!selectedPartColor) {
                    selectedPartColor = PartColor.customColor();
                    selectedPartColor.value = (colorValue2 == 0xFFFFFF ? 0xFEFFFF : colorValue2);
                }
                addedPart.color = selectedPartColor.copy();
                parentSlot.selectedColor = selectedPartColor;
            }
            else {
                parentSlot.selectedColor = PartColor.noColor();

                if (parentSlot.selectedPart.selectedMaterial && parentSlot.selectedPart.selectedMaterial.name == "metal") {
                    parentSlot.selectedColor.value = 0xFFFFFF;
                }

                if (bodyColor >= 0 && parentSlot.name != PartSlot.AEROGRAPHY && PartSlot.VINYLS.indexOf(parentSlot.name)==-1 && PartSlot.TWO_TONE_VINYLS_LIST.indexOf(parentSlot.name)==-1 && parentSlot.name != PartSlot.TWO_TONE_VINYLS && parentSlot.name.indexOf(PartSlot.TWO_TONE_VINYL_TYPE)==-1) {
                    material = parentSlot.selectedPart.selectedMaterial;
                    if (!material || material.name != "metal") {
                        parentSlot.selectedColor = bodyPartColor.copy();
                    }
                }

                if (selectedTheState)//костыль (если без костыля то нужно эту логику согласовывать как на сайте делалось)
                {
                    parentSlot.selectedColor.value = 0xFFFFFF;
                }
            }
        }

        var bodyPart:Part = partsMap[bodyId];
        if (bodyComponent) {
            bodyPart.selectedMaterial = bodyPart.getPossibleMaterial(bodyComponent.material);
        }

		var tempPart:Part;
		var tempParentSlot:PartSlot;

        //зададим материал для деталей которым не задан материал
        for each(tempPart in partsMap) {
            if (tempPart.selectedMaterial) continue;
			
            tempParentSlot = tempPart.parentSlot;
            if (tempParentSlot.selectedPart && tempParentSlot.selectedPart.selectedMaterial) {
                tempPart.selectedMaterial = tempPart.getPossibleMaterial(tempParentSlot.selectedPart.selectedMaterial.name);
                if (tempPart.selectedMaterial) {
                    continue;
                }
                continue;
            }

            if (bodyPart.selectedMaterial) {

                tempPart.selectedMaterial = tempPart.getPossibleMaterial(bodyPart.selectedMaterial.name);
            }

            if (tempPart.selectedMaterial) {
                continue;
            }

            tempPart.selectedMaterial = selectPossibleMaterial(tempPart.possibleMaterials);
        }
		
        car.arrangePartSlots();
		
    }

    private function selectPossibleMaterial(materials:Array, partSlotName:String = null):PartMaterial {
        var order:Array;
        //front_lights rear_lights
        if (!partSlotName)
            order = ["glossy", "metal", "mate", "white", "redblack"];
        else if (Car.DISKS_SLOTS.indexOf(partSlotName) >= 0)
            order = ["metal", "glossy", "mate", "white", "redblack"];
        else if (partSlotName == "front_lights")
            order = ["metal", "glossy", "mate", "white", "redblack"];
        else if (partSlotName == "rear_lights")
            order = ["metal", "glossy", "mate", "white", "redblack"];
        else
            order = ["glossy", "metal", "mate", "white", "redblack"];

        for (var i:int; i < order.length; i++) {
            for each(var partMaterial:PartMaterial in materials) {
                if (partMaterial.name == order[i]) {
                    return partMaterial;
                }
            }
        }

        return materials[0];
    }


    private function parseSubparts(part:Part, componentJSON:Object, componentPath:String, car:Car):void {
        var nesteds:Array = componentJSON.nesteds as Array;
		
		var nested:Object;
		var filesKey:String;
		var partMaterial:PartMaterial;
		var imageURLPrefix:String;
		var imageURL:String;

        for each(nested in nesteds) {
            var subpart:Part = new Part(part.parentSlot);
            subpart.subposition = nested.position;
            subpart.parentSlot = part.parentSlot;
            part.addSubpart(subpart);

			
			
            for (filesKey in nested.files) {
                partMaterial = new PartMaterial(subpart); 
                partMaterial.name = filesKey;

                imageURLPrefix = componentPath;
                imageURLPrefix += "nesteds/";
                imageURLPrefix += nested.folderName;
                imageURLPrefix += "/";
                imageURLPrefix += partMaterial.name;
                imageURLPrefix += "/";

                partMaterial.componentPath = imageURLPrefix;
				
				subpart.addPossibleMaterial(partMaterial);
				
				partMaterial.isPartMaterialImagesLoaded = false;
				
				if(Constants.COMPONENT_OPTIMIZE && !defaultComponentMap[part.id]) continue;

                for each(imageURL in nested.files[filesKey]) {
                  
                    partMaterial.addImage(imageURL, car.stepOfFrameIndex);
                }
				
				partMaterial.isPartMaterialImagesLoaded = true;
               
            }
        }
    }

    private function getCountState(states:Object):int {

        if (!states)
            return 0;

        var i:int = 0;
        var state:String;
        for (state in states) {

            i++;
        }


        return i;

    }

    private function parseStates(part:Part, componentJSON:Object, componentPath:String, car:Car):void {

		var states:Object = componentJSON.states as Object;
		if(!states) return;

        var partMaterialState:PartMaterial;
        var prefixUrl:String;
        var imageURLofState:String;
        var imageURLPrefixOfState:String;
        var filesOfState:String;
        var subpartOfState:Part;
        var type:String;
        var state:Object;
        var nameOfState:String;
        var isTestDisk:Boolean = false;
		
		var isReplaceState:Boolean = Car.SlotNamesForStatesMap.indexOf(part.parentSlot.name)>=0;
		//var statesMap:Object;
		
		//if(componentJSON.hasOwnProperty("statesMap")){
			
		//	statesMap = componentJSON.statesMap;
		//}
		
		

        for (nameOfState in states) {
            if (nameOfState && !part.existCamber) {
				
				/*if(nameOfState.indexOf("neon")>=0){
					trace("neon part:", part.name, part.parentSlot.name);
				}*/

                if (nameOfState.indexOf('trr') >= 0 || nameOfState.indexOf('trl') >= 0 || nameOfState.indexOf('cm1') >= 0 ||
                        nameOfState.indexOf('cm2') >= 0 || nameOfState.indexOf('cm3') >= 0 || nameOfState.indexOf('cm4') >= 0 ||
                        nameOfState.indexOf('of1') >= 0 || nameOfState.indexOf('of2') >= 0 || nameOfState.indexOf('of3') >= 0 || nameOfState.indexOf('of4') >= 0) {

                    part.existCamber = true;

                    if (!Device.isIos() && Constants.DISABLED_CAMBER_FOR_ANDROID_TRUCK_SUV) {
                        if (car.type == "Truck" || car.type == "Suv") {
                            part.existCamber = false;
                        }
                    }
                }
            }

            state = states[nameOfState];
            type = state.type;//например addition(фар зажженные) или replace(диски с неоном)

            if (!state.files) {
                continue;
            }

            subpartOfState = new Part(part.parentSlot);

            if (type == "addition") {
                subpartOfState.isAdditionState = true;
            }

            
            subpartOfState.parentSlot = part.parentSlot;
            part.addState(subpartOfState, nameOfState);

            for (filesOfState in state.files) {
				
                partMaterialState = new PartMaterial(subpartOfState);
                if (!subpartOfState.selectedMaterial) {
                    subpartOfState.selectedMaterial = partMaterialState;
                }

				
				if(isReplaceState && Car.statesMap[nameOfState]){
					
					nameOfState = Car.statesMap[nameOfState];
				}

                partMaterialState.name = filesOfState;
                //partMaterialState.isPaintable = filesOfState != "metal";

                imageURLPrefixOfState = componentPath;

                imageURLPrefixOfState += "states/";
                imageURLPrefixOfState += nameOfState;
                imageURLPrefixOfState += "/";
                imageURLPrefixOfState += partMaterialState.name;
                imageURLPrefixOfState += "/";


                partMaterialState.componentPath = imageURLPrefixOfState
					
				
				
				//trace("filesOfState:", filesOfState,  part.parentSlot.name);
					
				subpartOfState.addPossibleMaterial(partMaterialState);	
				partMaterialState.parentPart = part;
				
				partMaterialState.isPartMaterialImagesLoaded = false;
				
				if(Constants.COMPONENT_OPTIMIZE  && !defaultComponentMap[part.id]) continue;

                for each(imageURLofState in state.files[filesOfState]) {
                    partMaterialState.addImage(imageURLofState, car.stepOfFrameIndex);
                }
				
				partMaterialState.isPartMaterialImagesLoaded = true;

                
            }
        }
    }

    private function displayNameForComponentType(type:String):String {
        var map:Object = {
            "neon": "Neon",
            "shadow": "Shadow",
            "aerography": "Vinilis",
            "spoiler_top": "Top spoiler",
            "mirors": "Mirrors",
            "windows": "Tinted glass",
            "sides": "Sides",
            "exhaus": "Exhaust",
            "air_intake_motor": "Front air intake",
            "fox_eyes": "Fox eyes",
            "front_lights": "Front lights",
            "disks": "Wheel",
            "podium": "Podium",
            "body": "Body",
            "radiator": "Radiator",
            "rear_lights": "Rear lights",
            "motor_cover": "Hood",
            "wings": "Wings",
            "pipe": "Pipe",
            "air_intake_wings": "Wings air intake",
            "protectors": "Protector",
            "air_intake_top": "Top air intake",
            "spoiler_back": "Spoiler",
            "packs": "Packs",
            "black": "Black",
            "rear_bumper": "Rear bumper",
            "front_bumper": "Front bumper"
        };

        return map[type];
    }
}
}

//import com.despark.tuning3d.model.cars;


