package com.despark.tuning3d.model.challenges {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarsParser;
    import com.utils.DateParser;
    import com.utils.ObjectUtils;

    public class ChallengeData {
        public static const TYPE_STANCE:String = "stance";
        public static const TYPE_BACKGROUND:String = "background";
        public static const TYPE_WIDE:String = "wide";
        public static const TYPE_GARAGE:String = "garage";

        public static const TYPE_CAMBER:String = "camber";
        public static const TYPE_OFFSET:String = "offset";
        public static const TYPE_STRIPES:String = "stripes";
        public static const TYPE_VINYLS:String = "vinyls";
        public static const TYPE_TWO_TONE:String = "two_tone";


        //
        public static const IN_CHALLENGE:String = 'in_challenge';
        public static const IN_VOTING:String = 'in_voting';
        public static const FINISHED:String = 'finished';
        public static const TIME_TRIAL:String = 'timeTrial';


        private var _id:String;
        private var _activeForChallenge:Boolean;
        private var _car:Car;
        private var _cars:Vector.<Car>;
        private var _carName:String;
        private var _country:String;
        private var _description:String;
        private var _requirements:String;
        private var _location:String;
        private var _name:String;
        private var _cost:String;
        private var _reward:String;
        private var _carType:String;
        private var _challengeType:String;
        private var _startDate:Date;
        private var _finishDate:Date;
        private var _votingFinishDate:Date;
        private var _votingStartDate:Date;
        private var _status:String;
        private var _featuresRules:FeaturedRules;
        private var _rewardRules:Array;
        private var _isTimeTrial:Boolean;

        private var _timeTrialsPercent:int;
        private var _timeTrialsSeconds:Number;
        private var _timeTrialsScore:Number;

        private var _timeTrialsDefaults:Vector.<ChallengeRequirement>;

        private var _isFromGarage:Boolean;
        function ChallengeData(data: Object) {
            var i:int;
            _id = ObjectUtils.getValue(data, "_id");
            _activeForChallenge = ObjectUtils.getValue(data, "activeForChallenge", false);
            if(data.car){
                if(data.car is Car){
                    _car = data.car as Car;
                }else if(!(data.car is String)){
                    _car = new CarsParser().parseObject(data.car, null) as Car;
                }
            }
            if(data.cars && data.cars.length){
                _cars = new Vector.<Car>(data.cars.length);
                for(i = 0; i < data.cars.length; i++){
                    var listCar:Car;
                    if(data.cars[i] is Car){
                        listCar = data.cars[i] as Car;
                    }else if(!(data.cars[i] is String)){
                        listCar = new CarsParser().parseObject(data.cars[i], null) as Car;
                    }
                    _cars[i] = listCar;
                }
            }
            _carName = ObjectUtils.getValue(data, "carName");
            _cost = ObjectUtils.getValue(data, "cost", "0");
            _country = ObjectUtils.getValue(data, "country", "");
            _description = ObjectUtils.getValue(data, "description", "");
            _requirements = ObjectUtils.getValue(data, "requirements", null);
            _location = ObjectUtils.getValue(data, "location", "");
            _carType = ObjectUtils.getValue(data, "carType", "");
            _challengeType = ObjectUtils.getValue(data, "type", "");
            _name = ObjectUtils.getValue(data, "name");
            _reward = ObjectUtils.getValue(data, "reward", "0");
            _startDate = DateParser.parseDate(ObjectUtils.getValue(data, "startDate", "00-00-00T00:00:00.000Z"));
            _finishDate = DateParser.parseDate(ObjectUtils.getValue(data, "finishDate", "00-00-00T00:00:00.000Z"));
            _votingFinishDate = DateParser.parseDate(ObjectUtils.getValue(data, "votingFinishDate", "00-00-00T00:00:00.000Z"));
            _votingStartDate = DateParser.parseDate(ObjectUtils.getValue(data, "votingStartDate", "00-00-00T00:00:00.000Z"));
            _featuresRules = new FeaturedRules(ObjectUtils.getValue(data, "featuresRules", {}));
            _rewardRules = ObjectUtils.getValue(data, "rewardRules", []);
            //
            _isFromGarage = ObjectUtils.getValue(data, "garageType", false) || _challengeType == TYPE_GARAGE;
            _isTimeTrial = ObjectUtils.getValue(data, "timeTrials", false);
            //
            _timeTrialsPercent = ObjectUtils.getValue(data, "timeTrialsPercent", 0);
            _timeTrialsSeconds = ObjectUtils.getValue(data, "timeTrialsSeconds", 0);
            _timeTrialsScore = ObjectUtils.getValue(data, "timeTrialsScore", 0);
            //
            var timeTrailReqs:Array = ObjectUtils.getValue(data, "timeTrialsDefaults", null);
            if(timeTrailReqs && timeTrailReqs.length){
                _timeTrialsDefaults = new Vector.<ChallengeRequirement>(timeTrailReqs.length);
                for(i = 0; i < _timeTrialsDefaults.length; i++){
                    _timeTrialsDefaults[i] = new ChallengeRequirement(timeTrailReqs[i]);
                }
            }

            if(_isTimeTrial && _rewardRules){
                for(i = 0; i < _rewardRules.length; i++){
                    if(_rewardRules[i].hasOwnProperty("min")){
                        _rewardRules[i].min = parseFloat(_rewardRules[i].min);
                    }
                    if(_rewardRules[i].hasOwnProperty("max")){
                        _rewardRules[i].max = parseFloat(_rewardRules[i].max);
                    }
                    if(_rewardRules[i].hasOwnProperty("reward")){
                        _rewardRules[i].reward = parseFloat(_rewardRules[i].reward);
                    }
                }
            }

            //
            if(_carName){
                _carName = initCarName(_carName);
            }
            if(_car){
                _car.model = initCarName(_car.model);
            }

            //
            var currentDate:Date = new Date();
            if(finishDate.time > currentDate.time){
                if(_status != ChallengeData.IN_CHALLENGE){
                    _status = ChallengeData.IN_CHALLENGE;
                }
            }else if(votingStartDate.time < currentDate.time && votingFinishDate.time > currentDate.time){
                if(_status != ChallengeData.IN_VOTING){
                    _status = ChallengeData.IN_VOTING;
                }
            }else{
                if(_status != ChallengeData.FINISHED){
                    _status = ChallengeData.FINISHED;
                }
            }
        }

        public static function initCarName(str:String):String{
            str ||= "";
            return str.replace(/\s+?\bchallenge\b/im, "");
        }

        public function get activeForChallenge():Boolean {
            return _activeForChallenge;
        }

        public function get car():Car {
            return _car;
        }

        public function get carName():String {
            return _carName;
        }

        public function get country():String {
            return _country;
        }

        public function get description():String {
            return _description;
        }

        public function get location():String {
            return _location;
        }

        public function get name():String {
            return _name;
        }

        public function get cost():String {
            return _cost;
        }

        public function get reward():String {
            return _reward;
        }

        public function get startDate():Date {
            return _startDate;
        }

        public function get finishDate():Date {
            return _finishDate;
        }

        public function get votingFinishDate():Date {
            return _votingFinishDate;
        }

        public function get votingStartDate():Date {
            return _votingStartDate;
        }

        public function get id():String {
            return _id;
        }

        public function set car(value:Car):void {
            _car = value;
        }

        public function get status():String {
            return _status;
        }

        public function set status(value:String):void {
            _status = value;
        }

        public function get requirements():String {
            return _requirements;
        }

        public function get featuresRules():FeaturedRules {
            return _featuresRules;
        }

        public function get rewardRules():Array {
            return _rewardRules;
        }

        public function get carType():String {
            return _carType;
        }

        public function set carType(value:String):void {
            _carType = value;
        }

        public function get challengeType():String {
            return _challengeType;
        }

        public function set challengeType(value:String):void {
            _challengeType = value;
        }

        public function get cars():Vector.<Car> {
            return _cars;
        }

        public function get isTimeTrial():Boolean {
            return _isTimeTrial;
        }

        public function get timeTrialsDefaults():Vector.<ChallengeRequirement> {
            return _timeTrialsDefaults;
        }

        public function get isFromGarage():Boolean {
            return _isFromGarage;
        }

        public function get timeTrialsPercent():int {
            return _timeTrialsPercent;
        }

        public function get timeTrialsSeconds():Number {
            return _timeTrialsSeconds;
        }

        public function get timeTrialsScore():Number {
            return _timeTrialsScore;
        }
    }
}

import com.utils.ObjectUtils;

class FeaturedRules{
    public var arMode:Boolean = false;
    public var tint:Boolean = false;
    public var lights:Boolean = false;
    public var background:Boolean = false;
    public var customAngle:Boolean = false;
    public var neon:Boolean = false;
    public var color:Boolean = false;
    public var material:Boolean = false;
    public var suspension:Boolean = false;
    public var transform:Boolean = false;
    public var factoryMaterial:Boolean = false;
    public function FeaturedRules(value:Object){
        suspension = ObjectUtils.getValue(value, "suspension", true);
        background = ObjectUtils.getValue(value, "background", true);
        color = ObjectUtils.getValue(value, "color", true);
        lights = ObjectUtils.getValue(value, "lights", true);
        material = ObjectUtils.getValue(value, "material", true);
        neon = ObjectUtils.getValue(value, "neon", true);
        tint = ObjectUtils.getValue(value, "tint", true);
        transform = ObjectUtils.getValue(value, "transform", true);
        arMode = ObjectUtils.getValue(value, "arMode", false);
        customAngle = ObjectUtils.getValue(value, "customFrame", false);
        factoryMaterial = ObjectUtils.getValue(value, "factoryMaterial", false);
    }
}