package com.despark.tuning3d.model.challenges {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.parts.PartDisk;
    import com.despark.tuning3d.model.parts.PartSlot;

    public class ChallengeValidator {
        public function ChallengeValidator() {
        }

        public static function getRewards(challengeData:ChallengeData, rating:Number):int{
            if(rating == 0){
                return 0;
            }
            for(var i:int = 0; i < challengeData.rewardRules.length; i++){
                if(rating > challengeData.rewardRules[i].min && rating <= challengeData.rewardRules[i].max){
                    return challengeData.rewardRules[i].reward;
                }
            }
            return challengeData.rewardRules[challengeData.rewardRules.length - 1].reward;
        }

        public static function getRequirementStats(car:Car, requirements:Vector.<ChallengeRequirement>):Object{
            var result:Object = {};
            var totalNum:int = 0;
            var correctNum:int = 0;
            for(var i:int = 0; i < requirements.length; i++){
                result[requirements[i].componentType] = requirementStats(car, requirements[i]);
            }
            for(var componentType:String in result){
                if(result[componentType]){
                    for(var prop:String in result[componentType]){
                        if(prop == "sortKey"){
                            continue;
                        }
                        totalNum++;
                        if(result[componentType][prop]){
                            correctNum++;
                        }
                    }
                }
            }
            result.rating = Math.round(100 * correctNum / totalNum);
            return result;
        }

        public static function requirementStats(car:Car, requirement:ChallengeRequirement):Object{
            var result:Object = {};
            var partSlot:PartSlot = car.getPossiblePartSlot(requirement.componentType);
            result.sortKey = partSlot.sortKey;
            if(requirement.ids.indexOf("none") != -1){
                result["component"] = !(partSlot && partSlot.selectedPart);
                return result;
            }
            var i:int;
            if(partSlot && partSlot.selectedPart){
                result["component"] = (requirement.ids.indexOf(partSlot.selectedPart.id) != -1);

                if(requirement.materials){
                    result["material"] = !(!partSlot.selectedPart.selectedMaterial || requirement.materials.indexOf(partSlot.selectedPart.selectedMaterial.name) == -1);
                }

                if(requirement.colors){
                    result["color"] = !(!partSlot.selectedColor || requirement.colors.indexOf(partSlot.selectedColor.value) == -1);
                }

                if(requirement.stance){
                    if(!partSlot.selectedPart.selectedState){
                        result["stance"] = false;
                    }else{
                        var camber:Array = partSlot.selectedPart.selectedState.nameOfState.match(/cm[1-9]/i);
                        var offset:Array = partSlot.selectedPart.selectedState.nameOfState.match(/of[1-9]/i);
                        if(camber && camber.length == 0){
                            camber = null;
                        }
                        if(offset && offset.length == 0){
                            offset = null;
                        }
                        result["stance"] = true;
                        for(i = 0; i < requirement.stance.length; i++){ // any, all, cm, of, cm1, cm2, of1, of2
                            if(requirement.stance[i] == "any" && !camber && !offset){
                                result["stance"] = false;
                                break;
                            }
                            if(requirement.stance[i] == "all" && (!camber || !offset)){
                                result["stance"] = false;
                                break;
                            }
                            // Camber
                            if(requirement.stance[i] == "cm" && !camber){
                                result["stance"] = false;
                                break;
                            }
                            if(requirement.stance[i] == "cm1" && (!camber || camber.indexOf("cm1") == -1)){
                                result["stance"] = false;
                                break;
                            }
                            if(requirement.stance[i] == "cm2" && (!camber || camber.indexOf("cm2") == -1)){
                                result["stance"] = false;
                                break;
                            }

                            // Offset
                            if(requirement.stance[i] == "of" && !offset){
                                result["stance"] = false;
                                break;
                            }
                            if(requirement.stance[i] == "of1" && (!offset || offset.indexOf("of1") == -1)){
                                result["stance"] = false;
                                break;
                            }
                            if(requirement.stance[i] == "of2" && (!offset || offset.indexOf("of2") == -1)){
                                result["stance"] = false;
                                break;
                            }
                        }
                    }
                }

                if(requirement.radius){
                    var partDisk:PartDisk = partSlot.selectedPart as PartDisk;
                    if(!partDisk){
                        result["radius"] = false;
                    }else{
                        result["radius"] = requirement.radius.indexOf(partDisk.selectedDiskSize) != -1;
                    }
                }
            }else if(partSlot){
                result["component"] = false;
            }
            return result;
        }
    }
}
