package com.despark.tuning3d.model.challenges.feed {
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.view.components.challenges.controls.TuningLabel;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.utils.DateParser;
    import com.utils.FontSelector;
    import com.utils.NumberUtils;

    import feathers.controls.Label;

    import flash.text.TextFormatAlign;

    public class ChallengeWonNoteData extends ChallengeNotificationData {
    private var _tuningLabel:Label;
    private var _resultsLabel:Label;
    private var _earnedLabel:Label;
    private var _diamondLabel:ExtendedLabel;
    private var _ratingLabel:ExtendedLabel;


    private var _tuning:Tuning;
    private var _touchMoved:Boolean;
    public function ChallengeWonNoteData(data:Object) {
        _iconTextureId = "challenge_won_icon";
        if(data.tuning){
            _tuning = TuningsProxy.parseTuning(data.tuning, false, true);
        }
        super(data);
        _time = new Date().time - DateParser.parseDate(data.updated).time;
        _touchMoved = false;
    }

    override public function createView(width:Number, height:Number):void {
        if(!_disposed){
            return;
        }
        super.createView(width, height);

        _tuningLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratSemiBold(), 12, 0x58595b, TextFormatAlign.LEFT, false);
        _tuningLabel.text = _data.challengeName;
        _tuningLabel.width = Math.max(rendererWidth - 300, 180);
        _tuningLabel.includeInLayout = false;

        _resultsLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratRegular(), 11, 0x58595b, TextFormatAlign.LEFT, false);
        _resultsLabel.text = "Voting results:";
        _resultsLabel.includeInLayout = false;

        _ratingLabel = new ExtendedLabel();
        _ratingLabel.textRendererFactory = LabelFactory.challengeNotificationsRewardsLabel;
        _ratingLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));
        _ratingLabel.defaultIcon.scale = 3 / 4 * ButtonFactory.assetManager.scaleFactor / 2;
        _ratingLabel.labelOffsetX = 3;
        _ratingLabel.paddingTop = -1;
        _ratingLabel.text = NumberUtils.toFixed(parseFloat(_data.rating), 2);
        _ratingLabel.includeInLayout = false;

        _earnedLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratRegular(), 11, 0x58595b, TextFormatAlign.LEFT, false);
        _earnedLabel.text = "Earned:";
        _earnedLabel.paddingLeft = 5;
        _earnedLabel.includeInLayout = false;

        _diamondLabel = new ExtendedLabel();
        _diamondLabel.textRendererFactory = LabelFactory.challengeNotificationsRewardsLabel;
        _diamondLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));
        _diamondLabel.defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 3;
        _diamondLabel.labelOffsetX = 3;
        _diamondLabel.labelOffsetY = -2;
        _diamondLabel.paddingTop = 1;
        _diamondLabel.text = _data.diamond;
        _diamondLabel.includeInLayout = false;


        _messageLabel.addChild(_tuningLabel);
        _messageLabel.addChild(_resultsLabel);
        _messageLabel.addChild(_ratingLabel);
        _messageLabel.addChild(_earnedLabel);
        _messageLabel.addChild(_diamondLabel);
        _messageLabel.visible = false;

        if(_tuning){
            _tuningView = new TuningLabel(_tuning.thumbnailURL);
            TuningLabel(_tuningView).loadComplete = function():void{
                _holder.updateView();
            };
        }

    }

    override public function layout(rendererWidth:Number, rendererHeight:Number):void{
        super .layout(rendererWidth, rendererHeight);
        if(_disposed){
            return;
        }

        _tuningLabel.validate();
        _tuningLabel.x = leftMargin + 12;
        _tuningLabel.y = 22;

        _resultsLabel.validate();
        _resultsLabel.x = _tuningLabel.x;
        _resultsLabel.y = 41;

        _ratingLabel.validate();
        _ratingLabel.x = _resultsLabel.x + _resultsLabel.width + 3;
        _ratingLabel.y = _resultsLabel.y;

        _earnedLabel.validate();
        _earnedLabel.x = _ratingLabel.x + _ratingLabel.width + 8;
        _earnedLabel.y = _resultsLabel.y;

        _diamondLabel.validate();
        _diamondLabel.x = _earnedLabel.x + _earnedLabel.width + 3;
        _diamondLabel.y = _resultsLabel.y;

        _messageLabel.visible = true;
    }

    override public function dispose():void {
        super.dispose();
        if(_tuningLabel){
            _tuningLabel.removeFromParent(true);
            _tuningLabel = null;
        }
        if(_resultsLabel){
            _resultsLabel.removeFromParent(true);
            _resultsLabel = null;
        }
        if(_earnedLabel){
            _earnedLabel.removeFromParent(true);
            _earnedLabel = null;
        }
        if(_diamondLabel){
            _diamondLabel.removeFromParent(true);
            _diamondLabel = null;
        }
        if(_ratingLabel){
            _ratingLabel.removeFromParent(true);
            _ratingLabel = null;
        }
    }

    }
}
