package com.despark.tuning3d.model.core {

    import by.blooddy.crypto.image.JPEGEncoder;

    import com.despark.tuning3d.view.components.core.AlertView;

    import com.distriqt.extension.camerarollextended.Asset;
    import com.distriqt.extension.camerarollextended.AuthorisationStatus;
    import com.distriqt.extension.camerarollextended.CameraRollExtended;
    import com.distriqt.extension.camerarollextended.CameraRollExtendedBrowseOptions;
    import com.distriqt.extension.camerarollextended.events.AssetEvent;
    import com.distriqt.extension.camerarollextended.events.AuthorisationEvent;
    import com.distriqt.extension.camerarollextended.events.CameraRollExtendedEvent;

    import flash.display.Bitmap;
    import flash.display.Loader;
    import flash.display.LoaderInfo;
    import flash.events.Event;

    import flash.filesystem.File;
    import flash.net.URLRequest;
    import flash.utils.ByteArray;

    import starling.events.EventDispatcher;

public class ImagePicker extends EventDispatcher{

    public static const NOT_SUPPORTED : String = 'NotSupported';
    public static const DENIED : String = 'denied';
    public static const ERROR_OCCURRED : String = 'Error';
    public static const CANCELED : String = 'Canceled';
    public static const SUCCEED : String = 'Succeed';

    private var _tracer:Tracer;
    private var _initialized : Boolean = false;

    private var options:CameraRollExtendedBrowseOptions;
    private var _isPending:Boolean;
    private var _pendingData:Array;
    private var _requested:Boolean;
    private static const ALLOWED_EXTENSIONS:Array = ['jpg', 'jpeg', 'png'];
    public function ImagePicker(maximumCount:int = 2) {
        _tracer = new Tracer('ImagePicker');
        _requested = false;
        CameraRollExtended.init("com.3dtuning.Tuning3D");
        if (CameraRollExtended.isSupported) {
            CameraRollExtended.service.addEventListener( AuthorisationEvent.CHANGED, authorisationStatusChangedHandler );
            CameraRollExtended.service.addEventListener(CameraRollExtendedEvent.CANCEL, cancelHandler);
            CameraRollExtended.service.addEventListener(CameraRollExtendedEvent.SELECT, selectHandler);
            CameraRollExtended.service.assets.addEventListener( AssetEvent.ASSET_LOADED, assetLoadedHandler);
            options = new CameraRollExtendedBrowseOptions();
            options.maximumCount = maximumCount;
            options.type = Asset.IMAGE;
            options.useNativePicker = false;
            _initialized = true;
        }
    }

    public function get isPending(): Boolean {
        return _isPending;
    }

    private function authorisationStatusChangedHandler(event:AuthorisationEvent):void {
        if(_requested) {
            _requested = false;
            var status:String = event.status;
            handleStatus(status);
        }
    }

    public function pickImage():void{
        _requested = true;
        var status:String = CameraRollExtended.service.authorisationStatus();
        handleStatus(status);
    }

    private function handleStatus(status:String):void{
        switch (status){
            case AuthorisationStatus.SHOULD_EXPLAIN:
            case AuthorisationStatus.NOT_DETERMINED:
                CameraRollExtended.service.requestAuthorisation();
                return;
            case AuthorisationStatus.DENIED:
                dispatchEventWith(DENIED, false, {canOpenDeviceSettings: CameraRollExtended.service.canOpenDeviceSettings});
                return;
            case AuthorisationStatus.UNKNOWN:
            case AuthorisationStatus.RESTRICTED:
                dispatchEventWith(NOT_SUPPORTED);
                return;
            case AuthorisationStatus.AUTHORISED:
                _isPending = false;
                if(_initialized){
                    CameraRollExtended.service.browseForAsset(options);
                }else {
                    dispatchEventWith(NOT_SUPPORTED);
                }
                break;
        }
    }

    public function openSettings():void{
        if (CameraRollExtended.service.canOpenDeviceSettings) {
            CameraRollExtended.service.openDeviceSettings();
        }
    }


    private function selectHandler( event:CameraRollExtendedEvent ):void {
        _pendingData = event.assets.concat();
        commitImageLoading();
    }

    public function commitImageLoading():void{

        if(_pendingData.length) {
            var asset:Asset = _pendingData.shift() as Asset;
            var file:File;
            try {
                file = new File(asset.url);
                if(unsupportedFileFormat(file)){
                    return;
                }
                loadImage(file);
                _isPending = (_pendingData.length > 0);
            }catch (e:Error) {
                file = CameraRollExtended.service.getFileForAsset(asset);
                if(file){
                    if(unsupportedFileFormat(file)){
                        return;
                    }
                    loadImage(file);
                    _isPending = (_pendingData.length > 0);
                }
            }
        }
    }

    private function unsupportedFileFormat(file:File):Boolean{
        var ext:String = file.extension.toLowerCase();
        if(ALLOWED_EXTENSIONS.indexOf(ext) == -1){
            AlertView.show("Failed to upload image: the format is not supported\nJPEG and PNG images are allowed only", 5, false);
            dispatchEventWith(CANCELED);
            return true;
        }
        return false;
    }

    private function assetLoadedHandler( event:AssetEvent ):void {
        var bitmap:Bitmap = new Bitmap(event.image);
        var byteArray:ByteArray = bitmap.bitmapData.getPixels(bitmap.bitmapData.rect);
        dispatchEventWith(SUCCEED, false, {
            bitmapData: bitmap.bitmapData,
            byteArray: byteArray
        });
    }

    private function cancelHandler( event:CameraRollExtendedEvent ):void {
        dispatchEventWith(CANCELED);
    }

    private function loadImage(file:File):void{
        if(file && file.exists){
            var loader:Loader = new Loader();
            var urlRequest:URLRequest = new URLRequest(file.url);
            loader.contentLoaderInfo.addEventListener(Event.COMPLETE, imgLoadCompleteHandler);
            loader.load(urlRequest);
        }
    }

    private function imgLoadCompleteHandler(event:Event):void {
        var loader:Loader = LoaderInfo(event.currentTarget).loader;
        loader.contentLoaderInfo.removeEventListener(event.type, imgLoadCompleteHandler);
        var bitmap:Bitmap = loader.content as Bitmap;
        var byteArray:ByteArray = JPEGEncoder.encode(bitmap.bitmapData);
        dispatchEventWith(SUCCEED, false, {
            bitmapData: bitmap.bitmapData,
            byteArray: byteArray
        });
    }

}
}
