package com.despark.tuning3d.model.languages
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.settings.Settings;
	import com.despark.tuning3d.model.settings.SettingsProxy;

	public class Language
	{
		public var name : String;
		private var _code : String;
		public var shortname:String;
		public var dictionaries : Object;
		private static const _languageData:Object = {};
		private static var _selectedLanguage:Language;
		private static const _pool:Object = {};
		public static const CODES_SUPPORTING_EMBEDDED_FONT : Array = [ 
			"en-US", 
			"ru-RU"
		];
		
		public function Language(aDictionaries : Object) {
			super();
			dictionaries = aDictionaries;
			shortname = "";
		}

		public static function createInstance(langCode:String, landDict:Object = null):Language{
			if(_pool.hasOwnProperty(langCode)){
				if(!hasDictionary(langCode) && landDict){
					_languageData[langCode] = landDict;
				}
				Language(_pool[langCode]).code = langCode;
				return _pool[langCode];
			}
			var result:Language = new Language(landDict);
			result.code = langCode;
			_pool[langCode] = result;
			return result;
		}
		
		public function translate(category : String, key : String) : String {
			if(!key){
				return "";  
			}
			
			if(!dictionaries){
				return key.replace("_"," ").replace("_"," ").replace("_"," ");
			}
			
			if(!dictionaries[category]){
				
				return key.replace("_"," ").replace("_"," ").replace("_"," ");
			}
			
			
			if(!dictionaries[category][key]){
				
				return key.replace("_"," ").replace("_"," ").replace("_"," ");
			}
			
			
		
			
			return dictionaries[category][key];
		}

		public function forceTranslate(category : String, key : String) : String {
			if(!key || !dictionaries || !dictionaries[category] || !dictionaries[category][key]){
				return null;
			}
			return dictionaries[category][key];
		}
		
		public function get supportsEmbeddedFont() : Boolean 
		{
			return true;//CODES_SUPPORTING_EMBEDDED_FONT.indexOf(code) != -1;
		}
		
		public static function get supportsEmbeddedFont():Boolean{
			
			var facade :ApplicationFacade = ApplicationFacade.getInstance();
			
			var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			var settings : com.despark.tuning3d.model.settings.Settings = settingsProxy.currentSettings;
			
			
			return true;//CODES_SUPPORTING_EMBEDDED_FONT.indexOf(settings.selectedLanguageCode) != -1;
		}

		public function get code():String {
			return _code;
		}

		public function set code(value:String):void {
			_code = value;
			if(_code && _languageData.hasOwnProperty(_code)){
				dictionaries = _languageData[_code];
			}else if(dictionaries){
				_languageData[_code] = dictionaries;
			}
		}

		public static function getDictionary(langCode:String):Object{
			if(_languageData.hasOwnProperty(langCode)){
				return _languageData[langCode];
			}
			return null;
		}

		public static function hasDictionary(langCode:String):Object{
			return _languageData.hasOwnProperty(langCode);
		}

		public static function get selectedLanguage():Language {
			return _selectedLanguage;
		}

		public static function set selectedLanguage(value:Language):void {
			_selectedLanguage = value;
		}
	}
}