package com.despark.tuning3d.model.languages
{
    import by.blooddy.crypto.serialization.JSONer;

    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.core.http.HttpClient;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.Language;

    import flash.events.Event;
    import flash.filesystem.File;
    import flash.globalization.LocaleID;
    import flash.globalization.StringTools;
    import flash.net.SharedObject;
    import flash.net.URLLoader;
    import flash.net.URLRequest;

    import org.puremvc.as3.patterns.proxy.Proxy;

    import starling.utils.AssetManager;

    public class LanguagesProxy extends Proxy {
    public static const NAME : String = "LanguagesProxy";

    public var httpClient :HttpClient;
    public var assetManager : AssetManager;

    private var map : Object;
    private var languagesParser : LanguagesParser;
    private var _languages : Array;
    private var loadingStringForCurrentLanguageSharedObject : SharedObject;

    public static const LANG_NAMES : Object = {
        "en-US":"English",
        "ru-RU":"Russian"
        /*"de-DE":"German",
        "es-ES":"Spanish",
        "fr-FR":"French",
        "pt-PT":"Portuguese",
        "az-AZ":"Azeri (Latin)",
        "kk-KZ":"Kazakh",
        "it-IT":"Italian"*/
    };

    //tempstab

    public function LanguagesProxy() {
        super(NAME);

        map = {};
        languagesParser = new LanguagesParser();
        loadingStringForCurrentLanguageSharedObject = SharedObject.getLocal("loadingStringForCurrentLanguageSharedObject");
    }

    public function get languages() : Array {
        return _languages;
    }

    public function get defaultLanguageCode() : String {
        var stringTool:StringTools=new StringTools(LocaleID.DEFAULT);
        var locale:String=stringTool.actualLocaleIDName;/*(locale == 'ru-GB' || locale == 'ru-RU' || *//*)*/
        if(locale && (locale.indexOf("ru-") == 0 || locale.indexOf("ru_") == 0)){
            locale = 'ru-RU';
        } else {
            locale = "en-US";
        }
        return locale;
    }

    public function get defaultLanguage() : Language {
        var language : Language = Language.createInstance(defaultLanguageCode);
        language.name = LANG_NAMES[defaultLanguageCode];
        return language;
    }

    public function getLanguage(code : String) : Language {
        if(!map[code] && _languages){
            for(var i:int=0; i<_languages.length; i++){
                if(_languages[i].code==code){
                    return _languages[i];
                }
            }
        }

        return map[code];
    }

    public function loadLanguages() : void
    {
        var result:String='{"langs":[{"code":"ru-RU","name":"Русский"},{"code":"en-US","name":"English"}]}';
        var languagesJSON : Object = JSONer.parse(result as String);
        _languages = languagesParser.parseObjects(languagesJSON.langs);
        sendNotification(Notifications.LANGUAGES_LOADED, {languages: languages});
    }


    public function loadLanguage(language : Language) : void {
        if(Language.CODES_SUPPORTING_EMBEDDED_FONT.indexOf(language.code) < 0) {
            language.code = 'en-US';
        }
        if(Language.hasDictionary(language.code)){
            loadCompleteHandler(language, Language.getDictionary(language.code));
            return;
        }

        var path : String = File.applicationDirectory.resolvePath('assets/manualCache/api/v1/translations').resolvePath(language.code).url;
        var urlLoader : URLLoader = new URLLoader();
        urlLoader.addEventListener(Event.COMPLETE, function (e : Event) : void {
            var languageJSON : Object = JSONer.parse(urlLoader.data as String);
            loadCompleteHandler(language, languageJSON);
        });

        urlLoader.load(new URLRequest(path));
    }

    private function loadCompleteHandler(language:Language, languageJSON:Object):void{
        var loadedLanguage : Language = Language.createInstance(language.code, languageJSON);
        loadedLanguage.name = language.name;
        map[loadedLanguage.code] = loadedLanguage;
        try{
            loadingStringForCurrentLanguageSharedObject.data.string = loadedLanguage.translate("mob_alertsAndOverlays", "loading");
            loadingStringForCurrentLanguageSharedObject.flush();
        }catch (e:Error) {

        }

        sendNotification(Notifications.LANGUAGE_LOADED, {language: loadedLanguage});
    }
}
}