package com.despark.tuning3d.model.levels {
    import com.utils.ObjectUtils;

    public class LevelData {
        private var _id:String;
        private var _level:int = 0;
        private var _materials:Vector.<String>;
        private var _components:Object;

        private var _levelUpGift:BonusData;
        private var _maxKeys:int;
        private var _minChallengeCount:int;
        private var _unlockedData:LevelUpData;
        private var _source:Object;
        private static var _levelDataList:Vector.<LevelData>;
        private static var _levelDataArr:Array;
        private static var _currentUserLevelData:LevelData;
        public function LevelData(data:Object = null) {
            if(data){
                updateValues(data);
                _source = data;
            }
        }

        public function updateValues(data:Object):Boolean{
            var updated:Boolean = false;
            if(_level != 0 && data.hasOwnProperty("level") && data.level > _level){
                updated = true;
            }
            _id = ObjectUtils.getValue(data, "_id", _id);
            _level = ObjectUtils.getValue(data, "level", _level);
            _maxKeys = ObjectUtils.getValue(data, "maxKeys", _maxKeys);
            _minChallengeCount = ObjectUtils.getValue(data, "minCount", _minChallengeCount);

            _levelUpGift ||= new BonusData();
            _levelUpGift.updateValues(ObjectUtils.getValue(data, "oneTimeBonus", _levelUpGift));

            var materialsData:Array = ObjectUtils.getValue(data, "activate.materials", null);
            if(materialsData){
                _materials = new Vector.<String>();
                _materials.push.apply(null, materialsData);
            }
            _components = ObjectUtils.getValue(data, "activate.components", _components);
            return updated;
        }

        public function get source():Object{
            return _source;
        }

        public function getUnlockedMaterials():Array{
            var levelIndex:int = level;
            var prevLevelData:LevelData;
            var unlockedMaterials:Array = [];
            var i:int;
            if(levelIndex == 0){
                if(materials.length){
                    for(i = 0; i < materials.length; i++){
                        unlockedMaterials.push(materials[i]);
                    }
                    return unlockedMaterials;
                }
            }else if(_levelDataList){
                prevLevelData = _levelDataList[levelIndex - 1];
            }
            if(prevLevelData){
                if(materials.length > prevLevelData.materials.length){
                    for(i = 0; i < materials.length; i++){
                        if(prevLevelData.materials.indexOf(materials[i]) == -1){
                            unlockedMaterials.push(materials[i]);
                        }
                    }
                    return unlockedMaterials;
                }
            }
            return null;
        }

        public function getUnlockedComponents():Array{
            var levelIndex:int = level;
            var prevLevelData:LevelData;
            if(levelIndex != 0 && _levelDataList){
                prevLevelData = _levelDataList[levelIndex - 1];
            }
            if(prevLevelData){
                var unlockedComponents:Array = [];
                for (var componentName:String in prevLevelData.components){
                    var componentData:Object = {name:componentName, value: 0};
                    if(components.hasOwnProperty(componentName)){
                        componentData.value = components[componentName] - prevLevelData.components[componentName];
                    }else{
                        componentData.value = "All";
                    }
                    if(componentData.value != 0){
                        unlockedComponents.push(componentData);
                    }
                }
                return unlockedComponents;
            }
            return null;
        }

        public function get unlockedData():LevelUpData{
            if(_unlockedData == null){
                _unlockedData = new LevelUpData();
                _unlockedData.materials = getUnlockedMaterials();
                _unlockedData.components = getUnlockedComponents();
                if(level == 0){
                    _unlockedData.limit = maxKeys;
                }else{
                    _unlockedData.limit = maxKeys - levelDataList[level - 1].maxKeys;
                }
            }
            return _unlockedData;
        }

        public function get level():int {
            return _level;
        }

        public function get materials():Vector.<String> {
            return _materials;
        }

        public function get components():Object {
            return _components;
        }

        public function get levelUpGift():BonusData {
            return _levelUpGift;
        }

        public function get maxKeys():int {
            return _maxKeys;
        }

        public function get minChallengeCount():int {
            return _minChallengeCount;
        }

        public function get nextMinChallengeCount():int {
            var index:int = level + 1;
            if(_levelDataList && index >= 0 && index < _levelDataList.length){
                return _levelDataList[level + 1].minChallengeCount;
            }
            return 0;
        }

        public static function get levelDataList():Vector.<LevelData> {
            return _levelDataList;
        }

        public static function get levelDataArray():Array {
            if(_levelDataList && !_levelDataArr){
                var result:Array = [];
                for(var i:int = 0; i < _levelDataList.length; i++){
                    result.push(_levelDataList[i]);
                }
                _levelDataArr = result;
            }
            return _levelDataArr;
        }

        public static function setLevelData(value:Array):void {
            if(value && !_levelDataList){
                _levelDataList = new <LevelData>[];
                var i:int;
                for(i = 0; i < value.length; i++){
                    _levelDataList.push(new LevelData(value[i]));
                }
                _levelDataList.sort(function(a:LevelData, b:LevelData):Number{
                    return a.level - b.level;
                });
            }/*else{
                _levelDataList = null;
            }*/
        }

        public static function getLevelDataById(id:String):LevelData {
            if(_levelDataList){
                var i:int;
                for(i = 0; i < _levelDataList.length; i++){
                    if(_levelDataList[i].id == id){
                        return _levelDataList[i];
                    }
                }
            }
            return null
        }

        public static function getMaterialOpenLevel(materialName:String):int{
            if(_currentUserLevelData && _currentUserLevelData.materials && _currentUserLevelData.materials.indexOf(materialName) != -1){
                return -1;
            }
            if(_levelDataList){
                var materialsBlocked:Boolean = false;
                for(var  i:int = 0; i < _levelDataList.length; i++){
                    if(_levelDataList[i].materials){
                        materialsBlocked = true;
                        if(_levelDataList[i].materials.indexOf(materialName) != -1){
                            return _levelDataList[i].level;
                        }
                    }
                }
                if(materialsBlocked){
                    return _levelDataList[_levelDataList.length - 1].level;
                }
            }
            return -1;
        }


        public static function getComponentOpenLevel(componentName:String, index:int):int{
            if(!_currentUserLevelData || !_currentUserLevelData.components || !_currentUserLevelData.components.hasOwnProperty(componentName) || _currentUserLevelData.components[componentName] > index){
                return -1;
            }
            if(_levelDataList){
                var lastFoundIndex:int = -1;
                for(var  i:int = 0; i < _levelDataList.length; i++){
                    if(_levelDataList[i].components && _levelDataList[i].components.hasOwnProperty(componentName)){
                        lastFoundIndex = i;
                        if(index < _levelDataList[i].components[componentName]){
                            return _levelDataList[i].level;
                        }
                    }
                }
                return _levelDataList[lastFoundIndex].level;
            }
            return -1;
        }

        public static function setUserLevelData(value:Array):void{
            LevelData.setLevelData(value);
        }

        public static function get currentUserLevelData():LevelData {
            return _currentUserLevelData;
        }

        public static function set currentUserLevelData(value:LevelData):void {
            _currentUserLevelData = value;
        }

        public function get id():String {
            return _id;
        }
    }
}
class LevelUpData{
    public var materials:Array;
    public var components:Array;
    public var limit:int = 0;
    public function LevelUpData(){

    }
}
