package com.despark.tuning3d.model.parts
{

	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.core.images.PersistentImageProvider;
	import com.despark.tuning3d.model.disks.Disk;
	import com.despark.tuning3d.model.disks.DiskBrand;
	import com.despark.tuning3d.model.packages.Packet;
	import com.utils.ObjectUtils;

	import flash.filesystem.File;
	import flash.utils.Dictionary;

	public class Part implements PersistentImageProvider
	{
		public static const LOADED_PARTS:Object = {};
		public var id : String;
		private var _name : String;
		
		public var isAdditionState : Boolean = false;
		public var isEnabled : Boolean = true;

		//public var oldSelectedColor : PartColor;
		public var _oldSelectedMaterial : PartMaterial;
		public var _selectedMaterial : PartMaterial;
		
		public var _selectedMaterialOfAddState : PartMaterial;
		public var _selectedStates : Object;
		public var _selectedState : Part;
		public var mem_selectedState : Part;//временное хранилище selectedState
		
		public var lockPacket:Packet=null;
		
		public var linked:Array=[];
		
		public var linkedParts:Array=[];
		public var parentLinkedParts:Array=[];
		
		protected var _parentSlot : PartSlot;
		public var imageUrl : String;
		protected var _folder : String;
		public var subparts : Array;
		public var _states : Object;
		public var isSubpart : Boolean;
		private var _saveIndex:String="";
		public var _color:PartColor;
		public var _changeableColor:PartColor;

		public var enabled:Boolean=true;
		
		public var properties:Object={};
		
		public var manufacturer : String;
		public var isBack : Boolean;
		
		public var isState : Boolean;
		public var nameOfState : String;
		
		public var subposition : int;
		public var excludedPartSlotsNames : Array;
		
		public var _possibleMaterials : Array;
		public var _possibleMaterialsMap : Dictionary;
		private var _brightness : Number;
		
		public var subpartType:String;
		
		private var _isDefault : Boolean;
		
		protected var _affectedSlots:Array;
		
		private var _maskForSlots:Array;
		
		private var _lock:Boolean=false;
		
		public var active:Boolean=false;
		
		public var maskSet:String="";
		
		public var camberLock:Boolean=false;
		
		public var _existCamber:Boolean=false;
		
		public var sortKey:int=-1;
		
		public var isDependence:Boolean=false;
		
		public var depsMap:Array = [];
        public var  isDependentlyDisabled: Boolean = false;
		
		public var _partNum:int=1;
		public static var PartNum:int=1;
		
		private var _saveFastModeLoadedCurrantFrame:int = -1;
		
		protected var _isFastModeLoaded:Boolean = false;
		
		public var multiselect:Boolean = false;
		public var activeForInformation:Boolean = false;
		
		private var _universalIcons:Boolean = false;
		public var unlockLevel:int = -1;
		
		public function get universalIcons():Boolean{

			return _universalIcons;
		}
		
		public function set universalIcons(value:Boolean):void{
			
			_universalIcons = value;
		}
		
		//protected var _imagesUrlBytes:ByteArray;
		
		public function get saveFastModeLoadedCurrantFrame():int{
			
			return _saveFastModeLoadedCurrantFrame;
		}
		
		public function set saveFastModeLoadedCurrantFrame(value:int):void{
			_saveFastModeLoadedCurrantFrame = value;
		}
		
		public function get isFastModeLoaded():Boolean
		{
			return _isFastModeLoaded;
		}
		
		public function set isFastModeLoaded(value:Boolean):void{
			_isFastModeLoaded = value;
		}
		
		public function get folder():String{
			return _folder;
		}
		
		public function set folder(value:String):void{
			_folder = value;
		}

		public function get name():String{
			return _name;
		}

		public function set name(value:String):void{
            _name = value;
		}
		
		public function set affectedSlots(value:Array):void{
			_affectedSlots=value;
		}
		
		public function get affectedSlots():Array{
			
			return _affectedSlots;
		}
		
		public function get realId():String{
			
			return id;
		}
		
		public function get dependence():*{
			
			return this.properties && this.properties.dependence ?  this.properties.dependence : [];
		}
		
		
		public function get wide():String{
			
			return this.properties && this.properties.wide ?  this.properties.wide+"" : null;
		}
		
		
		public function unloadPart():void{
			if(!LOADED_PARTS.hasOwnProperty(id)){
				return;
			}
			delete LOADED_PARTS[id];
			var i:int;
			var key:String;
			var partMaterial:PartMaterial;
			if(_possibleMaterials){
				for(i = 0; i < _possibleMaterials.length; i++){
					partMaterial = _possibleMaterials[i];
					if(!partMaterial){
						continue;
					}
					if(!ObjectUtils.isEmpty(partMaterial.imagesMap)) {
						for(var angle:String in partMaterial.imagesMap){
							for(var frame:String in partMaterial.imagesMap[angle]){
								var materialImage:PartMaterialImage = partMaterial.imagesMap[angle][frame];
								if(materialImage){
									PartMaterialImagePool.holdInstance(materialImage);
								}
							}
						}
						partMaterial.imagesMap = {};
					}
					partMaterial.isPartMaterialImagesLoaded = false;
				}
			}

			if(states){
				for(key in states){
					var statePart:Part = states[key];
					if(statePart){
						statePart.unloadPart();
					}
				}
			}
			if(subparts){
				for(i = 0; i < subparts.length; i++){
					var subPart:Part = subparts[i];
					if(subPart){
						subPart.unloadPart();
					}
				}
			}
		}

		public static function clearLoadedParts():void{
			for (var key:String in LOADED_PARTS){
				delete LOADED_PARTS[key];
			}
		}
		
		public function parseFiles(componentJSON:Object):void {
			if(!parentSlot){
				return;
			}
			var shadowOffset:Number = 0;
			var filesKey:String;
			var partMaterial:PartMaterial;
			var imageURL:String;
			var car:Car = parentSlot.parentCar;
			
			for (filesKey in componentJSON.files) {
				
				partMaterial = getPossibleMaterial(filesKey);
				
				if(!partMaterial) continue;
				
				if(partMaterial.isPartMaterialImagesLoaded) continue;
				
				partMaterial.imagesMap = {};
                if (componentJSON.type === 'shadow') {
                    shadowOffset = -52;
                }

				for each(imageURL in componentJSON.files[filesKey]) {
					//trace("imageURL:", imageURL)
					partMaterial.addImage(imageURL, car.stepOfFrameIndex,shadowOffset);
				}
				
				partMaterial.isPartMaterialImagesLoaded = true;
			}
		}

		public function parseStates(componentJSON:Object):void {
			
			
			var partMaterialState:PartMaterial;
			var imageURLofState:String;
			var filesOfState:String;
			var subpartOfState:Part;
			var type:String;
			var state:Object;
			var nameOfState:String;
			var statesObj:Object = componentJSON.states as Object;
			var car:Car = (parentSlot?parentSlot.parentCar:null);
			
			if (statesObj) {//добавим структуру states
				for (nameOfState in statesObj) {
					state = statesObj[nameOfState];
					type = state?state.type:null;//например addition(фар зажженные) или replace(диски с неоном)
					
					if (!state || !state.files) {
						continue;
					}

					if(nameOfState){
						subpartOfState = this.states[nameOfState];
					}
					if(!subpartOfState) {
						continue;
					}
					for (filesOfState in state.files) {
						partMaterialState = subpartOfState.getPossibleMaterial(filesOfState);  //new PartMaterial(subpartOfState);
						if(!partMaterialState){
							continue;
						}
						partMaterialState.imagesMap = {};
						for each(imageURLofState in state.files[filesOfState]) {
							partMaterialState.addImage(imageURLofState, car.stepOfFrameIndex);
						}
					}
				}
			}
		}
		
		
		public function parseSubparts(componentJSON:Object):void {
			if(!componentJSON.nesteds || !parentSlot || !parentSlot.parentCar){
				return;
			}
			var nestedsList:Array = componentJSON.nesteds as Array;
			var nested:Object;
			var subpart:Part;
			var filesKey:String;
			var partMaterial:PartMaterial;
			var imageURL:String;
			var nestedIndex:int = 0;
			var car:Car = parentSlot.parentCar;
			
			for each(nested in nestedsList) {
				subpart = this.subparts[nestedIndex]; //new Part(part.parentSlot);
				nestedIndex++;
				if(!subpart){
					continue;
				}
				if (nested && nested.files) {
					for (filesKey in nested.files) {
						partMaterial = subpart.getPossibleMaterial(filesKey);
						for each(imageURL in nested.files[filesKey]) {
							partMaterial.addImage(imageURL, car.stepOfFrameIndex);
						}
						partMaterial.isPartMaterialImagesLoaded = true;
					}
				}
			}
		}

		public function dispose():void{
			lockPacket = null;
			manufacturer = null;
			nameOfState = null;
			
			subpartType = null;
			_affectedSlots= null;
			
			_oldSelectedMaterial=null;
			_selectedMaterial=null;
			_selectedMaterialOfAddState=null;
			_selectedStates=null;
			
			_selectedState=null;
			mem_selectedState=null;
			
			linked = null;
			linkedParts = null;
			parentLinkedParts = null;
			_parentSlot=null;
			imageUrl=null;
			folder=null;
			
			var i:int=0;
			var key:String;
			var part:Part;
			
			if(subparts){
				for(i=0; i<subparts.length; i++){
					part=subparts[i];
					if(part){
						part.dispose();
						part=null;
					}
				}
				subparts=null;
			}
			if(_states){
				for(key in _states){
					part = _states[key];
					if(part){
						part.dispose();
					}
					delete _states[key];
				}
				_states = null;
			}
			
			_color = null;
			properties = null;
			excludedPartSlotsNames = null;
			_addTypesOnRemove = null;
			_removeTypes = null;
			_removeTypesOnRemove = null;

			if(_possibleMaterials){
				var partMaterial:PartMaterial;
				for(i=0; i<_possibleMaterials.length; i++){
					partMaterial=_possibleMaterials[i];
					if(partMaterial){
						partMaterial.dispose();
						partMaterial=null;
						_possibleMaterials[i]=null;
					}
				}
				
				_possibleMaterials=null;
			}
			
			if(_possibleMaterialsMap){
				for(key in _possibleMaterialsMap){
					_possibleMaterialsMap[key] = null;
					delete _possibleMaterialsMap[key];
				}
				_possibleMaterialsMap=null;
			}
			
			affectedSlots=null;
			_maskForSlots=null;
			depsMap=null;
			id = null;
		}
		
		public function get existCamber():Boolean{
			return _existCamber;
		}
		
		public function set existCamber(value:Boolean):void{
			
			_existCamber=value;
		}
		
		public function get frameIndeX() : int 
		{
			return -1;
		}
		
		public function get isDefault() : Boolean 
		{
			return _isDefault;
		}
		
		public function set isDefault(value : Boolean) : void
		{
			_isDefault = value;
		}
		
		public function Part(partSlot:PartSlot) {
			super();
			
			this.parentSlot=partSlot;
			
			_possibleMaterials = [];
			_possibleMaterialsMap = new Dictionary(true);
			_removeTypes = [];
			
			subparts = [];
			states = {};
			_brightness = 1.0;
			_isDefault = false;
			
			//_imagesUrlBytes = new ByteArray();
			//_imagesUrlBytes.position=0;
			
			PartNum++;
			this._partNum=PartNum;
		}
		
		public function get color():PartColor{
			return _color;
		}
		
		public function set color(value:PartColor):void{
			_color=value;
		}
		
		public function set maskForSlots(value:Array):void{
			_maskForSlots=value;
		}
		
		public function get maskForSlots():Array{
			return _maskForSlots;
		}
		
		public function get states():Object{
			return _states;
		}
		
		public function set states(value:Object):void{
			_states=value;
		}
		
		public function get selectedState():Part{
			return _selectedState;
		}
		
		public function set selectedState(value:Part):void{
			_selectedState=value;
		}
		
		public function get selectedStates():Object{
			return _selectedStates;
		}
		
		public function set selectedStates(value:Object):void{
			_selectedStates=value; 
		}
		
		public function get oldSelectedMaterial():PartMaterial{
			return _oldSelectedMaterial;
		}
		
		public function set oldSelectedMaterial(value:PartMaterial):void{
			_oldSelectedMaterial=value;
		}
		
		public function get selectedMaterialOfAddState():PartMaterial{
			return _selectedMaterialOfAddState;
		}
		
		public function set selectedMaterialOfAddState(value:PartMaterial):void{
			_selectedMaterialOfAddState=value;
		}
		
		public function get selectedMaterial():PartMaterial{
			return _selectedMaterial;
		}
		
		public function set selectedMaterial(value:PartMaterial):void{
			if(parentSlot.name == "front_fender"){
				trace();
			}
			var setChangeableColor:Boolean = _selectedMaterial && value && _selectedMaterial.isPaintable && !value.isPaintable;
			if(setChangeableColor){
				_changeableColor = parentSlot.previousColor = parentSlot.selectedColor||color;
			}
			_selectedMaterial = value;
			if(value){
				if(!value.isPaintable && color){
					color = PartColor.defaultColor();
					parentSlot.selectedColor = color;
				}else if(value.isPaintable && _changeableColor){
					if(setChangeableColor && _changeableColor){
						color = _changeableColor;
					}
					if(color){
						parentSlot.selectedColor = color;
					}
				}
			}
		}
		
		
		public function get parentSlot():PartSlot{
			
			return _parentSlot;
		}
		
		public function set parentSlot(value:PartSlot):void{
			
			_parentSlot=value;
		}
		
		
		public function addPossibleMaterial(material : PartMaterial) : void
		{
			material.parentPart = this;
			
			_possibleMaterials.push(material);
			
			_possibleMaterials.sort(orderSortKey);
			_possibleMaterialsMap[material.name] = material;
		}
		
		private static const partMaterialOrder:Array = [  "glossy", "metal", "mate","white","redblack", "mask"];
		private static const partDiskMaterialOrder:Array = [  "metal",  "glossy", "mate","white","redblack", "mask"];
        private var _parentSlotName:String;
		
		
		//  может пригодится
		private var _removeTypes:Array;
		private var _removeTypesOnRemove:Array;
		private var _addTypesOnRemove:Array;
		private var _priceString:String;
		private function orderSortKey(a:*,b:*):int
		{
			var order:Array;
			
			order=partMaterialOrder;
			
			if(this.parentSlot && (Car.CAR_DISKS_SLOTS.indexOf(parentSlot.name)>=0))
				order=partDiskMaterialOrder;
			
			
			var a_i : int = order.indexOf(a.name);
			var b_i : int = order.indexOf(b.name);
			
			if(a_i<b_i)return -1;
			if(a_i>b_i)return 1;
			
			return 0;
		}
		
		private function sortMaterials(materials:Array):Array{
			
			
			
			
			return null
		}
		
		
		public function get possibleMaterialsMap():Dictionary{
			
			return _possibleMaterialsMap;
		}
		
		
		
		
		
		public function getPossibleMaterial(name : String) : PartMaterial {
			return possibleMaterialsMap[name];
		}
		
		public function get possibleMaterials() : Array {
			return _possibleMaterials;
		}

		public function get vinylIconUrl():String{
			if(PartSlot.VINYLS.indexOf(parentSlot.name) != -1) {
				if (!universalIcons && parentSlot && parentSlot.parentCar) {
					return parentSlot.parentCar.imagePath.replace(Constants.IMAGES_URL, "") + parentSlotName + "/" + folder + "/" + folder + "_big.png";
					//return parentSlot.parentCar.imagePath.replace(Constants.IMAGES_URL, "") + parentSlotName + "/" + name + "/" + name + "_big.png";
				}

				if (parentSlot && manufacturer) {
					var brand:DiskBrand = Disk.getDiskBrandById(manufacturer);
					if (brand) {
						if (brand.name == "factory") {
							return "_icons/" + parentSlotName + "/" + name + "_big.png";
						}
						return "_icons/" + parentSlotName + "/" + brand.name + " " + name + "_big.png";
					}
				}
			}
			return null;
		}
		
		public function get localFile() : String {
			var vinylUrl:String = vinylIconUrl;
            if(vinylUrl){
                return File.cacheDirectory.resolvePath("cars/" + vinylUrl).nativePath;
            }

			if(this.universalIcons){
				return File.cacheDirectory.resolvePath("Cars/_icons/"+parentSlotName+"/"+name+"_big.png").nativePath;
			}

			var localImagePath : String;
			if(imageUrl.indexOf(Constants.IMAGES_URL)>=0){
				localImagePath= (imageUrl).replace(Constants.IMAGES_URL, "Cars/");
			}else{
				localImagePath  = (parentSlot.parentCar.imagePath+imageUrl).replace(Constants.IMAGES_URL, "Cars/");
			}
			/*var file1: File = File.cacheDirectory.resolvePath(localImagePath);
			file1.preventBackup = true;*/
			return localImagePath;
		}
		
		public function get remoteUrl() : String {
            var vinylUrl:String = vinylIconUrl;
            if(vinylUrl){
                return Constants.IMAGES_URL + vinylUrl;
            }

			if(this.universalIcons){
				return Constants.IMAGES_URL+"_icons/"+parentSlot.name+"/"+encodeURI(name)+"_big.png?v=1";
			}

			if(parentSlot && parentSlot.parentCar && parentSlot.parentCar.haveIconFolder) {
				return parentSlot.parentCar.imagePath+"icons/"+parentSlotName+"/"+name+"_big.png";
			}

			if(imageUrl.indexOf(Constants.IMAGES_URL)>=0){
				return imageUrl;
			}else{
				return parentSlot.parentCar.imagePath+imageUrl;
			}
		}

		public function get parentSlotName():String{
            return _parentSlotName;
		}

		public function set parentSlotName(value:String):void {
			_parentSlotName = value;
		}

		public function get isEncrypted() : Boolean {
			return false;
		}
		
		public function get ignoresTransparentPixelsOnTouch() : Boolean {
			return false;
		}
		
		public function get touchable() : Boolean {
			return true;
		}
		
		public function get important() : Boolean
		{
			return false;
		}
		
		public function get brightness() : Number
		{
			return _brightness;
		}
		
		public function set brightness(value : Number) : void
		{
			_brightness = value;
		}
		
		public function addSubpart(part : Part) : void
		{
			part.isSubpart = true;
			subparts.push(part);
		}
		
		public function addState(part : Part, nameOfState: String) : void
		{
			part.isState = true;
			part.nameOfState = nameOfState;
			states[nameOfState] = part;
		}
		
		public function get material3dtProvider() : Material3dtProvider
		{
			return null;
		}
		
		public function getMainPartFromLinked():Part{
			
			var optativeLinkedPartTypes:Array=["sides", "wings"]; //Желательные типы главного материала
			var mainPart:Part=null;
			
			for(var j:int=0; j<optativeLinkedPartTypes.length; j++){
				for(var k:int=0; k<linkedParts.length; k++){
					
					var linkedPart:Part=linkedParts[k];
					if(linkedPart.parentSlot.name==optativeLinkedPartTypes[j]){
						mainPart=linkedPart;
						break;
					}
				}
				
				if(mainPart){
					break;
				}
			}
			
			if(!mainPart){
				mainPart=linkedParts[0];
			}
			
			return mainPart;
		}


		public function get lock():Boolean {
			return _lock;
		}

		public function set lock(value:Boolean):void {
			_lock = value;
		}

		public function set removeTypes(removeTypes:Array):void {
			_removeTypes = removeTypes;
		}

		public function get removeTypes():Array {
			return _removeTypes;
		}

		public function get removeTypesOnRemove():Array {
			return _removeTypesOnRemove;
		}

		public function set removeTypesOnRemove(value:Array):void {
			_removeTypesOnRemove = value;
		}

		public function get addTypesOnRemove():Array {
			return _addTypesOnRemove;
		}

		public function set addTypesOnRemove(value:Array):void {
			_addTypesOnRemove = value;
		}

		public function get hideFromMenu():Boolean{
			return (properties && properties.hideFromCompList)?properties.hideFromCompList: false;
		}

		public function get priceString():String {
			return _priceString;
		}

		public function set priceString(value:String):void {
			_priceString = value;
		}

		public function get saveIndex():String {
			return _saveIndex;
		}

		public function set saveIndex(value:String):void {
			_saveIndex = value;
		}

		public function get isTouchable():Boolean{
			return enabled && !parentSlot.isExcluded;
		}
	}
}


