package com.despark.tuning3d.model.parts
{
	import com.utils.ObjectUtils;

	public class PartDisk extends Part
	{
		
		private var _selectedDiskSize:int=0;
		private var _radius:int=0;
		public var minRadius:int=0;
		public var maxRadius:int=0;
		
		public var main:Boolean=false;
		private var _profile:String='default';
        public static const PROFILE_REG_EXP:RegExp = /\((?P<profile>DEEP|MID|FLAT|S\.DEEP)\)/ms;
		
		
		public function PartDisk(partSlot:PartSlot) {
			super(partSlot);
		}
		
		override public function get realId():String{
			var prefix:String=PartSlotDisk.getPrefixDep(this);
			var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
			if(partDiskObj){
				var partDisk:PartDisk=partDiskObj.part;
				return partDisk.id;
			}else{
				return id;
			}
		}
		
		override public function get dependence():*{
			var prefix:String=PartSlotDisk.getPrefixDep(this);
			var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
			if(partDiskObj){
				var partDisk:PartDisk=partDiskObj.part;
				return partDisk.properties && partDisk.properties.dependence ?  partDisk.properties.dependence : [];
			}else{
				return super.dependence;
			}
		}

        override public function set name(value:String):void {
            super.name = value;
			if(value){
				modifyName();
            }
        }

        private function modifyName():String{
            var result:Object = PROFILE_REG_EXP.exec(name);
			if(result && result.profile){
				_profile = result.profile;
                super.name = name.replace(PROFILE_REG_EXP, '');
            }
            return name;
		}

		public function set radius(rad: int):void {
			_radius = rad;
		}

		public function get radius(): int {
			return _radius;
		}

		public function set selectedDiskSize(size: int):void {
			_selectedDiskSize = size;
		}

		public function get selectedDiskSize(): int {
			return _selectedDiskSize;
		}

		override public function get wide():String{
			
			var prefix:String=PartSlotDisk.getPrefixDep(this);
			
			var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
			
			if(partDiskObj){
				
				var partDisk:PartDisk=partDiskObj.part;
				return partDisk.properties && partDisk.properties.wide ?  partDisk.properties.wide : null;
				
			}else{
				return super.wide;
			}
			
		}
		
		
		public function getSelectedRadiusId():String{
			var prefix:String = PartSlotDisk.getPrefixDep(this);
			var partDiskObj:Object;
			if(parentSlotDisk){
				partDiskObj = parentSlotDisk.findPartDiskForRadius(name + prefix, selectedDiskSize);
			}
			if(partDiskObj && partDiskObj.part){
				var partDisk:PartDisk=partDiskObj.part;
				return partDisk.id;
			}else{
				return id;
			}
		}
		
		override public function get color():PartColor{

			return _color;
		}
		
		override public function set color(value:PartColor):void{
			_color=value;
		}
		
		override public function get existCamber():Boolean{
			
			if(radius==selectedDiskSize || selectedDiskSize==0){
				
				return _existCamber;
				
			}else{
				
				if(!parentSlotDisk)
					return false;
				
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					
					var partDisk:PartDisk=partDiskObj.part;
					return partDisk._existCamber;
					
				}else{
					return _existCamber;
				}
			}
		}

		override public function set affectedSlots(value:Array):void{
			if(radius==selectedDiskSize || selectedDiskSize==0){
				_affectedSlots=value;
			}else{
				if(!parentSlotDisk)
					return;
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name, selectedDiskSize);
				if(partDiskObj){
					var partDisk:PartDisk=partDiskObj.part;
					partDisk._affectedSlots=value;
				}else{
					_affectedSlots=value;
				}
				
			}
		}
		
		override public function get affectedSlots():Array{
			if(radius==selectedDiskSize || selectedDiskSize==0){
				return _affectedSlots;
			}else{
				if(!parentSlotDisk){
					return [];
				}
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				if(partDiskObj){
					var partDisk:PartDisk=partDiskObj.part;
					return partDisk.affectedSlots;
				}else{
					return _affectedSlots;
				}
			}
		}
		
		
		override public function get isFastModeLoaded():Boolean{
			if(radius==selectedDiskSize || selectedDiskSize==0){
				return _isFastModeLoaded;
			}else{
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				if(partDiskObj){
					var partDisk:PartDisk=partDiskObj.part;
					return partDisk.isFastModeLoaded;
				}else{
					return _isFastModeLoaded;
				}
			}
		}
		
		
		
		override public function set isFastModeLoaded(value:Boolean):void{
			
			if(radius==selectedDiskSize || selectedDiskSize==0){
				_isFastModeLoaded=value;
				
			}else{
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				if(!parentSlotDisk) return;
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					
					var partDisk:PartDisk=partDiskObj.part  ;
					partDisk.isFastModeLoaded=value;
					
				}else{
					_isFastModeLoaded=value;
				}
				
			}
			
		}
		
		
		override public function get folder():String{
			
			if(radius==selectedDiskSize || selectedDiskSize==0){
				
				return _folder;
				
			}else{
				
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					
					var partDisk:PartDisk=partDiskObj.part;
					return partDisk.folder;
					
				}else{
					return _folder;
				}
			}
		}
		
		
		
		override public function set folder(value:String):void{
			
			if(radius==selectedDiskSize || selectedDiskSize==0){
				_folder=value;
				
			}else{
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				if(!parentSlotDisk) return;
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					
					var partDisk:PartDisk=partDiskObj.part;
					partDisk.folder=value;
					
				}else{
					_folder=value;
				}
				
			}
			
		}
		
		
		override public function get states():Object{
			
			if(radius==selectedDiskSize || selectedDiskSize==0){
				
				return _states;
				
			}else{
				
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					
					var partDisk:PartDisk=partDiskObj.part  
					return partDisk.states;
					
				}else{
					return _states;
				}
			}
		}

		public function getDefaultProfile():String{
			var profileData:Object;
			if(parentSlotDisk.possiblePartList[name] && parentSlotDisk.possiblePartList[name][selectedDiskSize]){
				profileData = parentSlotDisk.possiblePartList[name][selectedDiskSize];
				for(var i:int = PartSlotDisk.PROFILE_LIST.length - 1; i >= 0; i--){
					if(profileData.hasOwnProperty(PartSlotDisk.PROFILE_LIST[i].id)){
						return PartSlotDisk.PROFILE_LIST[i].id;
					}
				}
				if(!ObjectUtils.isEmpty(profileData)){
					return ObjectUtils.getFirstKey(profileData);
				}
			}
			return "default";
		}

		public function getProfilePart(str:String):PartDisk{
			var profileData:Object;
			if(parentSlotDisk.possiblePartList[name] && parentSlotDisk.possiblePartList[name][selectedDiskSize]){
				profileData = parentSlotDisk.possiblePartList[name][selectedDiskSize];
				if(profileData && profileData[str]){
					return profileData[str];
				}
			}
			return this;
		}

		override public function set states(value:Object):void{

			if (value == null) {
				trace('setNull');
			}
 			if(radius==selectedDiskSize || selectedDiskSize==0){
				_states=value;
				
			}else{
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					
					var partDisk:PartDisk=partDiskObj.part  
					partDisk.states=value;
					
				}else{
					_states=value;
				}
				
			}
			
		}
		
		public function get parentSlotDisk():PartSlotDisk{
			
			return _parentSlot as PartSlotDisk;
		}
		
		
		
		override public function get selectedStates():Object{
			
			return _selectedStates;
		}
		
		override public function set selectedStates(value:Object):void{
			if (value == null) {
				trace('setNull')
			}
			_selectedStates=value;
			
		}
		
		
		override public function get oldSelectedMaterial():PartMaterial{
			if(radius==selectedDiskSize || selectedDiskSize==0){
				
				return _oldSelectedMaterial;
				
			}else{
				
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					//var nameMaterial:String=_oldSelectedMaterial.name;
					var partDisk:PartDisk=partDiskObj.part;
					return partDisk.oldSelectedMaterial;
				}else{
					return _oldSelectedMaterial;
				}
				
				
			}
		}
		
		override public function set oldSelectedMaterial(value:PartMaterial):void{
			
			
			
			if(radius==selectedDiskSize || selectedDiskSize==0){
				_oldSelectedMaterial=value;
				
			}else{
				
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					
					var partDisk:PartDisk=partDiskObj.part  
					partDisk.oldSelectedMaterial=value;
					
				}else{
					_oldSelectedMaterial=value;
				}
			}
		}
		
		override public function get selectedMaterialOfAddState():PartMaterial{
			return _selectedMaterialOfAddState;
		}
		
		override public function set selectedMaterialOfAddState(value:PartMaterial):void{
			_selectedMaterialOfAddState=value;
		}
		
		
		
		override public function get selectedState():Part{

			if(radius==selectedDiskSize || selectedDiskSize==0){
				
				return _selectedState;//possibleMaterialsMap[nameMaterial];
				
			}else{
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					
					
					var partDisk:PartDisk=partDiskObj.part;
					
					return partDisk.selectedState;
					//return (partDiskObj.part as PartDisk).getPossibleMaterial(nameMaterial);
					
				}else{
					
					
					return _selectedState;
					
				}
				
			}
			
			
		}
		
		override public function set selectedState(value:Part):void{
			var diskPart:PartDisk;
			var diskPartObj:Object;
			var nameOfState:String;
			var flag:Boolean=false;
			var diskSize:String;
			if (value == null) {
				trace('Null');
			}else{
				trace('Not Null');
			}
			if(radius==selectedDiskSize || selectedDiskSize==0){
				_selectedState=value;
				for(nameOfState in _states){
					if(_states[nameOfState]==_selectedState){
						parentSlotDisk.selectedStateName=nameOfState;
						flag=true;
					}
				}
				
				if(!flag){
					parentSlotDisk.selectedStateName=null;
				}

				for(diskSize in parentSlotDisk.diskRadiusMap[name]) {
                    for  each(diskPartObj in parentSlotDisk.diskRadiusMap[name][diskSize]) {
                        diskPart = diskPartObj.part as PartDisk;
                        if (parentSlotDisk.selectedStateName){
							diskPart._selectedState = diskPart._states[parentSlotDisk.selectedStateName];
						} else {
                            diskPart._selectedState = null;
                        }
                    }
                }
			}else{
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				if(partDiskObj){
					var partDisk:PartDisk=partDiskObj.part
					partDisk._selectedState=value;
					for(nameOfState in partDisk._states){
						if(partDisk._states[nameOfState]==partDisk._selectedState){
							parentSlotDisk.selectedStateName=nameOfState;
							flag=true;
						}
					}

					if(!flag){
						parentSlotDisk.selectedStateName=null;
					}

					for(diskSize in parentSlotDisk.diskRadiusMap[name]) {
						for  each(diskPartObj in parentSlotDisk.diskRadiusMap[name][diskSize]) {
							diskPart = diskPartObj.part as PartDisk;
							if (parentSlotDisk.selectedStateName){
								diskPart._selectedState = diskPart._states[parentSlotDisk.selectedStateName];
							} else {
								diskPart._selectedState = null;
							}
						}
					}
				}else{
					throw new Error("partDiskObj not found");
				}
			}
		}

		
		override public function getPossibleMaterial(nameMaterial : String) : PartMaterial {
			if(radius==selectedDiskSize || selectedDiskSize==0){
				
				return possibleMaterialsMap[nameMaterial];
				
			}else{
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					//var partDisk:PartDisk=parentSlotDisk.findPartDiskForRadius(name, selectedDiskSize);
					
					return (partDiskObj.part as PartDisk).getPossibleMaterial(nameMaterial);
					
				}else{
					
					
					return possibleMaterialsMap[nameMaterial];
					
				}
				
			}
		}
		
		override public function get possibleMaterials() : Array 
		{
			if(radius==selectedDiskSize || selectedDiskSize==0){
				
				return _possibleMaterials;
				
			}else{
				var prefix:String=PartSlotDisk.getPrefixDep(this);
				var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, selectedDiskSize);
				
				if(partDiskObj){
					//var partDisk:PartDisk=parentSlotDisk.findPartDiskForRadius(name, selectedDiskSize);
					return (partDiskObj.part as PartDisk).possibleMaterials;
					
				}else{
					
					
					return _possibleMaterials;
					
				}
				
			}
			
		}
		
		
		
		
		
		override public function get selectedMaterial():PartMaterial{
			
			
			if(main){
				return _selectedMaterial;
				
			}else{
				
				
				if(_selectedMaterial==null){
					
					var mainPartDisk:PartDisk=parentSlotDisk.findMainPartDisk();
					
					for(var r:int=mainPartDisk.minRadius; r<=mainPartDisk.maxRadius; r++){ // parentSlotDisk.diskRadiusMap[name]
						
						var prefix:String=PartSlotDisk.getPrefixDep(this);
						var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, r);
						
						if(partDiskObj){
							
							var partDisk:PartDisk=partDiskObj.part;
							
							return partDisk.possibleMaterialsMap[parentSlotDisk.selectedMaterialName];
						}
						
					}
					
					return _selectedMaterial;
					
				}else{
					
					return _selectedMaterial;
				}
			}
			
			
			
		}
		
		override public function set selectedMaterial(value:PartMaterial):void{
			
			super.selectedMaterial=value;
			
			if(main){
				
				if(value){
					
					parentSlotDisk.selectedMaterialName=value.name;
					
					for(var r:int=minRadius; r<=maxRadius; r++){ // parentSlotDisk.diskRadiusMap[name]
						var prefix:String=PartSlotDisk.getPrefixDep(this);
						var partDiskObj:Object=parentSlotDisk.findPartDiskForRadius(name+prefix, r);
						
						if(partDiskObj){
							
							var partDisk:PartDisk=partDiskObj.part;
							
							partDisk.selectedMaterial=partDisk.possibleMaterialsMap[parentSlotDisk.selectedMaterialName];
						}
						
					}
					
				}
				
			}else{
				
				
				
				
				
				
			}
		}


        public function get profile():String {
            return _profile;
        }
    }
}

