package com.despark.tuning3d.model.parts
{
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarsProxy;
	import com.despark.tuning3d.model.core.images.ImageCacher;
	import com.despark.tuning3d.model.core.images.PersistentImageProvider;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.utils.Logger;
	
	import flash.filesystem.File;
	import flash.utils.getTimer;
	
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	public class PartSlotsProxy extends Proxy
	{
		public static const NAME : String = "PartSlotsProxy";
		public var progressHandler : Function;
		
		private var addedCachersCount : int;
		private var completedCachersCount : int;
		private var partSlots : Array;
		private var _isDownloading : Boolean;
		private var currentPartSlot : PartSlot;
		private var currentPart : Part;
		private var currentPartMaterial : PartMaterial;
		private var startTime : Number = 0;
		private var count : int = 0;
		private var didSendFailNotfication : Boolean;
		private var cachers : Array;
		private var runningCachers : Array;
		private var maxGroupSize : int = 30;
		private var currentGroupSize : int = 0;
		private var q : Array;
		private var addedImageProviders : Array;
		private var downloadFinished : Boolean;
		
		public function PartSlotsProxy()
		{
			super(NAME);
		}
		
		public function get isDownloading() : Boolean 
		{
			return _isDownloading;
		}
		
		public function prepareForDownload(car : Car) : void
		{
			_isDownloading = false;
			didSendFailNotfication = false;
			
			q = [];
			partSlots = [];
			cachers = [];
			runningCachers = [];
			addedImageProviders = [];
			
			downloadFinished = false;
			currentGroupSize = 0;
			completedCachersCount = 0;
			addedCachersCount = 0;
			
			for(var i : int = 0; i < maxGroupSize; i++) 
			{
				var cacher : ImageCacher = new ImageCacher();
				
				//cacher = new ImageCacher();
				cacher.completeHandler = singleCacherCompleteHandler;
				cacher.errorHandler = singleCacherErrorHandler;
				cacher.cachingErrorHandler = singleCacherCachingErrorHandler;
				
				cachers.push(cacher);
			}
			
			addPrimalPartSlotsIfNeccessary(car);
		}
		
		private function addPrimalPartSlotsIfNeccessary(car : Car) : void
		{
			var shadowPartSlot : PartSlot = car.getPossiblePartSlot("shadow");
			if(shadowPartSlot.percentLoaded < 1.0)
			{
				addPartSlotForDownload(shadowPartSlot);
			}
			
			var bodyPartSlot : PartSlot = car.getPossiblePartSlot("body");
			if(bodyPartSlot.percentLoaded < 1.0)
			{
				addPartSlotForDownload(bodyPartSlot);
			}
			
			var blackPartSlot : PartSlot = car.getPossiblePartSlot("black");
			if(blackPartSlot.percentLoaded < 1.0)
			{
				addPartSlotForDownload(blackPartSlot);
			}
		}
		
		public function addPartSlotForDownload(partSlot : PartSlot) : void
		{
			partSlots.push(partSlot);
		}
		
		public function startDownload(car : Car) : void
		{
			
			if(!_isDownloading)
			{	
				progressHandler(0);
				
				startTime = getTimer();
				_isDownloading = true;
				
				var ArrOfCustDownloads:Array = new Array();
				
				var carLocalFile : File = File.cacheDirectory.resolvePath(car.path);
				carLocalFile.preventBackup = true;
				
				for(var i : int = 0; i < partSlots.length; i++) 
				{
					var partSlot : PartSlot = partSlots[i];
					
					var partSlotDirectory : File = carLocalFile.resolvePath(partSlot.name);
					partSlotDirectory.preventBackup = true;
					
					for(var j : int = 0; j < partSlot.possibleParts.length; j++)
					{	
						var part: Part = partSlot.possibleParts[j];
						if(part.folder != "default"){
							var partDirectory : File = partSlotDirectory.resolvePath(part.folder);
							partDirectory.preventBackup = true;
							ArrOfCustDownloads.push(partDirectory.nativePath);
						}
						go(i, j);
					}
				}
				
				var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
				usersProxy.setArrOfCustDownloads(ArrOfCustDownloads,car.id);
				run();
			}
		}
		
		public function go(i : int, j : int) : void
		{
			currentPartSlot = partSlots[i];
			currentPart = currentPartSlot.possibleParts[j];
			
			var localPartFile: File = File.cacheDirectory.resolvePath(currentPart.localFile); 
			localPartFile.preventBackup = true;
			
			
			
			if(currentPart.parentSlot.isEditable && !localPartFile.exists) 
			{
				addedImageProviders.push(currentPart);
				addedCachersCount++;
			}	
			
			cachePart(currentPart);
			
			for each(var subpart : Part in currentPart.subparts)
			{
				cachePart(subpart);
			}
		}
		
		public function run() : void
		{
			try{
				if(downloadFinished)
				{
					return;
				}
				
				if(addedImageProviders.length == 0 && currentGroupSize == 0)
				{
					downloadFinished = true;
					d();
					return;
				}
				
				for(; currentGroupSize < maxGroupSize && addedImageProviders.length > 0; currentGroupSize++)
				{
					var persistentImageProvider : PersistentImageProvider = addedImageProviders.pop();
					var cacher : ImageCacher = cachers.pop();
					runningCachers.push(cacher);
					
					cacher.source = persistentImageProvider.remoteUrl;
					cacher.cache = true;
					cacher.decryptAndLoad = false;
					cacher.important = persistentImageProvider.important;
					cacher.persistentImageProvider = persistentImageProvider;
					cacher.load();
					
					q.push(cacher);
				} 
				
			}catch(e:Error){
				
				
				Logger.e("PartSlotsProxy run error:"+e.message, this);
				
				//trace("error:", e.message, e.getStackTrace());
			}
		}
		
		private function cachePart(part : Part) : void
		{
			/*for each(var partMaterial : PartMaterial in part.possibleMaterials)
			{
				for each(var partMaterialImage : PartMaterialImage in partMaterial.imagesMap)
				{
					
					var localFile: File = File.cacheDirectory.resolvePath(partMaterialImage.localFile); 
					localFile.preventBackup = true;
					
					
					
					
					var url1 : String = null;
					
					url1 = partMaterialImage.url;
					//	}
					
					if(!localFile.exists && url1.indexOf("Thumbs.db") < 0)
					{
						addedImageProviders.push(partMaterialImage);
						addedCachersCount++;
						
					}
				}
			}*/
		}
		
		private function singleCacherCompleteHandler(cacher : ImageCacher) : void
		{
			completedCachersCount++;
			currentGroupSize--;
			
			runningCachers.splice(runningCachers.indexOf(cacher), 1);
			cachers.push(cacher);
			
			run();
			
			progressHandler(completedCachersCount / addedCachersCount);
			
		}
		
		private function d() : void
		{
			//AlertView.show("Done " + (getTimer() - startTime), -1, false);
			
			clearState();
			
			sendNotification(Notifications.CAR_PARTS_DOWNLOADED);
		}
		
		private function singleCacherErrorHandler(cacher : ImageCacher) : void
		{
			if(!didSendFailNotfication)
			{	
				didSendFailNotfication = true;
				cancelDownload();
				
				
				sendNotification(Notifications.CAR_PARTS_DOWNLOAD_FAILED);
			}
		}
		
		private function singleCacherCachingErrorHandler(cacher : ImageCacher)  : void
		{
			if(!didSendFailNotfication)
			{	
				didSendFailNotfication = true;
				cancelDownload();
				
				sendNotification(Notifications.CAR_PARTS_DOWNLOAD_CACHING_FAILED);
			}
		}
		
		public function cancelDownload() : void
		{
			for each(var imageCacher : ImageCacher in runningCachers)
			{
				imageCacher.cancel();
			}
			
			clearState();
		}
		
		private function clearState() : void
		{
			cachers = null;
			runningCachers = null;
			addedImageProviders = null;
			q = null;
			partSlots = null;
			_isDownloading = false;
		}
		
		
		public function deleteDownloadedPartSlots(car : Car, partSlots : Array) : void
		{
			var carLocalFile : File = File.cacheDirectory.resolvePath(car.path);
			carLocalFile.preventBackup = true;
			
			for each(var partSlot : PartSlot in partSlots)
			{
				var partSlotDirectory : File = carLocalFile.resolvePath(partSlot.name);
				partSlotDirectory.preventBackup = true;
				if(partSlotDirectory.exists)
				{
					var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
					usersProxy.deleteArrOfCustDownload(partSlotDirectory,car.id);
					
					partSlotDirectory.deleteDirectory(true);
					partSlot.percentLoaded = 0;
				}
			}
		}
	}
}

