package com.despark.tuning3d.model.users {
	import com.despark.tuning3d.model.levels.LevelData;
	import com.utils.ArrayUtil;

	public class User {

		public var username : String;
		public var email : String;
		private var _id : String;

		public var name : String;
		public var avatarPath : String;
		public var roles:Array=[];
		public var emailVerified:Boolean = false;
		public var dayIndex:int = 0;

		//
		public var rating:Number = 0;
		public var keys:int = 0;
		public var diamonds:int = 0;
		public var submittedChallenges:Array = [];
		private var _onUpdate:Function;
		//
		private var _levelData:LevelData;
		private var _isLevelUpdated:Boolean = false;
		private var _socialNetwork:String;
		public function User(data : Object = null) {
			super();
			updateData(data);
		}

		public function updateData(data:Object = null):void{
			var userIdUpdated:Boolean = false;
			if(data){
				if(data is String){
					return;
				}
				if(data.hasOwnProperty("username")){
					username = data.username;
				}

				if(data.hasOwnProperty("avatarPath") && data.avatarPath && data.avatarPath.length){
					avatarPath = data.avatarPath;
				}else if(data.hasOwnProperty("avatar") && data.avatar is Array){
					for(var i:int = 0; i < data.avatar.length; i++){
						if(data.avatar[i].current){
							avatarPath = data.avatar[i].path;
							break;
						}
					}
					avatarPath ||= "";
				}else if(data.hasOwnProperty("avatar") && data.avatar is String){
					avatarPath = data.avatar;
				}else {
					avatarPath = "";
				}

				if(data.hasOwnProperty("name")){
					name = data.name;
				}
				if(data.hasOwnProperty("_id") && data._id){
					if(_id && _id != data._id){
						userIdUpdated = true;
					}
					_id = data._id;
				}
				if(data.hasOwnProperty("emailVerified")){
					emailVerified = data.emailVerified;
				}
				if(data.hasOwnProperty("roles")){
					roles=data.roles;
				}
			}else{
				_id = null;
				email = null;
				name = null;
				avatarPath = null;
				username = null;
				roles = null;
				emailVerified = false;
			}

			// Challenge New System Data
			if(!data){
				if(rating != 0){
					rating = 0;
					updated = true;
				}
				if(keys != 0){
					keys = 0;
					updated = true;
				}
				if(diamonds != 0){
					diamonds = 0;
					updated = true;
				}
				if(updated && _onUpdate != null){
					_onUpdate(false);
				}
				return;
			}
			var updated:Boolean = false;
			var nv:Number = 0;
			if(data.hasOwnProperty("challengeNewSystemRatings")){
				if(data["challengeNewSystemRatings"] is Array){
					nv = countAvg(data["challengeNewSystemRatings"]);
				}else if(data["challengeNewSystemRatings"] is Number){
					nv = data["challengeNewSystemRatings"];
				}
				if(rating != nv){
					rating = nv;
					updated = true;
				}
			}else if(data.hasOwnProperty("averageRating")){
				nv = data["averageRating"];
				if(rating != nv){
					rating = nv;
					updated = true;
				}
			}
			if(data.hasOwnProperty("challengeNewSystemKeys")){
				nv = data["challengeNewSystemKeys"];
				if(keys != nv){
					keys = nv;
					updated = true;
				}
			}
			if(data.hasOwnProperty("challengeNewSystemDiamonds")){
				nv = data["challengeNewSystemDiamonds"];
				if(diamonds != nv){
					diamonds = nv;
					updated = true;
				}
			}
			if(data.hasOwnProperty("challengeNewList")){
				var arr:Array = data["challengeNewList"];
				if(arr && !ArrayUtil.equals(submittedChallenges, arr)){
					submittedChallenges = arr;
					updated = true;
				}
			}
			//
			var levelUpdated:Boolean = false;
			if(data.hasOwnProperty("level") && !userIdUpdated){
				var currentLevelData:LevelData = new LevelData(data["level"]);
				if(_levelData && _levelData.level != currentLevelData.level){
					levelUpdated = true;
				}
				if(LevelData.levelDataList){
					_levelData = LevelData.levelDataList[currentLevelData.level];
				}else{
					_levelData = currentLevelData;
				}
			}else if(!_levelData && LevelData.levelDataList){
				_levelData = LevelData.levelDataList[0];
			}

			if(updated && _onUpdate != null){
				_onUpdate(levelUpdated);
			}
		}

		public function copy() : User {
			var user : User = new User();
			user._id = _id;
			user.email = email;
			user.name = name;
			user.avatarPath = avatarPath;
			user.username = username;
			user.roles = roles;
			user.emailVerified = emailVerified;
			return user;
		}

		public function get nameToShow() : String {
			if(name == null || name.length <= 2 || name == "" ) {
				return username;
			}
			return name;
		}

		private static function countAvg(arr:Array):Number{
			var result:Number = 0;
			for (var i:int = 0; i < arr.length; i++){
				result += arr[i];
			}
			return result / (arr.length + 1);
		}

		public function set onUpdate(value:Function):void {
			_onUpdate = value;
		}

		public function get onUpdate():Function {
			return _onUpdate;
		}

		public function get isSuperAdmin():Boolean{
			return (roles && roles.indexOf("superadmin")>=0);
		}

		public function get levelData():LevelData {
			return _levelData;
		}

		public function set levelData(value:LevelData):void {
			_levelData = value;
		}

		public function get isLevelUpdated():Boolean {
			return _isLevelUpdated;
		}

		public function set isLevelUpdated(value:Boolean):void {
			_isLevelUpdated = value;
		}

		public function get socialNetwork():String {
			return _socialNetwork;
		}

		public function set socialNetwork(value:String):void {
			_socialNetwork = value;
		}

		public function get id():String {
			return _id;
		}

		public function set id(value:String):void {
			_id = value;
		}
	}
}