package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.view.components.challenges.ChallengesLeaderboardScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.utils.ObjectUtils;

    import org.puremvc.as3.interfaces.INotification;

    import starling.events.Event;

    public class ChallengesLeaderboardScreenMediator extends ChallengesScreenMediator {
        public function ChallengesLeaderboardScreenMediator(stackedScreen:StackedScreen) {
            super(stackedScreen);
        }

        override public function onRegister():void {
            super.onRegister();
            challengeScreen.showLoadBar();
            challengeScreen.addEventListener(ChallengesLeaderboardScreen.CHALLENGE_SELECTED, challengeSelectedHandler);
            challengeScreen.addEventListener(ChallengesLeaderboardScreen.USER_SELECTED, userSelectedHandler);
            challengeScreen.addEventListener(ChallengesLeaderboardScreen.LOAD_LEADERBOARD, loadLeaderboardHandler);
            Constants.selectedUser = null;
            loadChallengeList(ChallengesLeaderboardScreen.LAST_SELECTED_TYPE, ChallengesLeaderboardScreen.LAST_SELECTED_PERIOD, false);
        }

        override public function onRemove():void {
            super.onRemove();
            challengeScreen.removeEventListener(ChallengesLeaderboardScreen.CHALLENGE_SELECTED, challengeSelectedHandler);
            challengeScreen.dispose();
        }

        private function challengeSelectedHandler(event:Event):void {
            sendNotification(Notifications.DISPLAY_LEADERBOARD_CHALLENGE_TUNINGS, {challengeData:event.data, featureRules:event.data.featuresRules});
        }

        private function userSelectedHandler(event:Event):void {
            sendNotification(Notifications.DISPLAY_USER_CHALLENGE_TUNINGS, {userData: event.data, userGarage: false});
        }

        private function loadLeaderboardHandler(event:Event):void {
            var challengeType:String = ObjectUtils.getValue(event.data, "challengeType", null);
            var challengePeriod:String = ObjectUtils.getValue(event.data, "challengePeriod", null);
            loadChallengeList(challengeType, challengePeriod, true);
        }


        override protected function challengeScreenHomeSelectedSelectedHandler(event:Event):void{
            sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
        }

        public function get challengeScreen():ChallengesLeaderboardScreen {
            return viewComponent as ChallengesLeaderboardScreen;
        }

        override public function handleNotification(note:INotification):void {
            switch (note.getName()) {

            }
            super.handleNotification(note);
        }

        private function loadChallengeList(challengeType:String, period:String, excludeTop:Boolean = false):void{
            challengeScreen.showLoadBar();
            var challengeProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.getLeaderboardData(function(result:Object, success:Boolean = true):void{
                if(success){
                    challengeScreen.loadLeaderboard(result, challengeType, period);
                    challengeScreen.hideLoadBar();
                }
            }, challengeType, period, excludeTop);
        }
    }
}
