package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.view.components.challenges.ChallengesListScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.utils.Console;
    import com.utils.ObjectUtils;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;

    import starling.events.Event;

    public class ChallengesListScreenMediator extends ChallengesScreenMediator {
        public function ChallengesListScreenMediator(stackedScreen:StackedScreen) {
            super(stackedScreen);
        }

        override public function onRegister():void {
            super.onRegister();
            challengesListScreen.showLoadBar();
            challengesListScreen.addEventListener(ChallengesListScreen.ENTER_CHALLENGE_SELECTED, enterChallengeSelectedHandler);
            challengesListScreen.addEventListener(ChallengesListScreen.REQUEST_CHALLENGE_LIST, requestChallengeList);
            Constants.selectedUser = null;
        }

        override public function onRemove():void {
            super.onRemove();
            challengesListScreen.removeEventListener(ChallengesListScreen.ENTER_CHALLENGE_SELECTED, enterChallengeSelectedHandler);
            challengesListScreen.removeEventListener(ChallengesListScreen.REQUEST_CHALLENGE_LIST, requestChallengeList);
            challengesListScreen.dispose();
        }


        private function loadChallengeList():void{
            if(challengesListScreen.skip % challengesListScreen.limit != 0){
                return;
            }
            challengesListScreen.isLoading = true;
            if(challengesListScreen.skip == 0){
                challengesListScreen.showLoadBar();
            }
            var challengeProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.getChallengeList(function(result:Object, success:Boolean = true):void{
                if(success){
                    challengesListScreen.loadChallenges(result);
                    challengesListScreen.hideLoadBar();
                }
                challengesListScreen.isLoading = false;
            }, challengesListScreen.skip, challengesListScreen.limit, challengesListScreen.finishSkip);
        }

        public function get challengesListScreen():ChallengesListScreen {
            return viewComponent as ChallengesListScreen;
        }

        private function enterChallengeSelectedHandler(event: Event):void {
            var startChallengeData:Object = ObjectUtils.merge(event.data, {user: challengesListScreen.user});
            sendNotification(Notifications.DISPLAY_ENTER_CHALLENGE_POPUP, startChallengeData);
        }

        private function requestChallengeList(event: Event):void {
            loadChallengeList();
        }

        override public function handleNotification(note:INotification):void {
            switch (note.getName()) {
                case Notifications.USER_AUTHENTICATED:
                    if(!note.getBody() || !note.getBody().hasOwnProperty("wasAlreadyAuthenticated") || !note.getBody()["wasAlreadyAuthenticated"]){
                        challengesListScreen.clearChallengeData();
                    }
                    break;
            }
            super.handleNotification(note);
        }
    }
}
