package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengeAlertEvent;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.rewards.DailyRewardsData;
    import com.despark.tuning3d.view.components.challenges.ChallengesNotificationsScreen;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.rewards.RewardType;
    import com.despark.tuning3d.view.components.rewards.RewardsDiamondLabel;

    import org.puremvc.as3.interfaces.INotification;

    import starling.events.Event;

    public class ChallengesNotificationsScreenMediator extends ChallengesScreenMediator {
        private var challengeProxy : ChallengesProxy;
        public function ChallengesNotificationsScreenMediator(stackedScreen:StackedScreen) {
            super(stackedScreen);
        }

        override public function onRegister():void {
            super.onRegister();
            challengeProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            Constants.selectedUser = null;
            challengesNotificationsScreen.showLoadBar();
            challengesNotificationsScreen.addEventListener(ChallengeEvent.GET_NOTIFICATIONS, getNotificationsHandler);
            challengesNotificationsScreen.addEventListener(ChallengeEvent.TUNING_SELECTED, tuningSelectedHandler);
            challengesNotificationsScreen.addEventListener(ChallengeEvent.CHALLENGE_CLAIM_REWARD, claimRewardHandler);
            markUnreadNotifications();
        }

        private function tuningSelectedHandler(event:Event):void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_TUNING_POPUP, event.data);
        }

        private function claimRewardHandler(event:Event):void {
            var data:Object = event.data;
            challengeProxy.claimRewards(data._id, data.diamond);

            var alertView:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.COLLECT_REWARDS, new DailyRewardsData({bonus:{diamond: data.diamond * Constants.UNIT_MULTIPLIER}}), challengesNotificationsScreen.stage, false);
            alertView.addEventListener(ChallengeAlertEvent.COLLECT_TRIGGERED, collectEventHandler);
            alertView.closeOnForegroundTouch = false;
            alertView.open();
        }

        private function collectEventHandler(event:Event):void{
            challengesNotificationsScreen.addResource(RewardType.DIAMONDS, event.data[0].value, RewardsDiamondLabel.COLOR, challengesNotificationsScreen.stage.stageWidth / 2, challengesNotificationsScreen.stage.stageHeight / 2);
        }

        override public function onRemove():void {
            super.onRemove();
            challengesNotificationsScreen.removeEventListener(ChallengeEvent.GET_NOTIFICATIONS, getNotificationsHandler);
            challengesNotificationsScreen.removeEventListener(ChallengeEvent.CHALLENGE_CLAIM_REWARD, claimRewardHandler);
            challengesNotificationsScreen.removeEventListener(ChallengeEvent.TUNING_SELECTED, tuningSelectedHandler);
            challengesNotificationsScreen.dispose();
            challengeProxy = null;
        }

        override protected function challengeScreenHomeSelectedSelectedHandler(event:Event):void{
            sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
        }


        private function getNotificationsHandler(event:Event):void {
            challengeProxy.getNewNotifications(event.data.skip, event.data.limit, challengesNotificationsScreen.loadNotifications);
        }

        private function markUnreadNotifications():void {
            challengeProxy.markUnreadNotifications();
            Constants.UNREAD_CHALLENGE_NOTIFICATIONS = 0;
        }

        public function get challengesNotificationsScreen():ChallengesNotificationsScreen {
            return viewComponent as ChallengesNotificationsScreen;
        }

        override public function handleNotification(note:INotification):void {
            switch (note.getName()) {

            }
            super.handleNotification(note);
        }
    }
}
