package com.despark.tuning3d.view
{
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.auth.ForgottenPasswordScreen;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.StackedScreen;
	import com.despark.tuning3d.view.core.StackedScreenMediator;
import com.utils.ArrayUtil;

import org.puremvc.as3.interfaces.INotification;
	
	import starling.events.Event;
	
	public class ForgottenPasswordScreenMediator extends StackedScreenMediator
	{		
		public function ForgottenPasswordScreenMediator(stackedScreen : StackedScreen)
		{
			super(stackedScreen);
		}
		
		public function get forgottenPasswordScreen() : ForgottenPasswordScreen
		{
			return stackedScreen as ForgottenPasswordScreen;
		}
		
		override public function onRegister() : void
		{
			super.onRegister();
			
			forgottenPasswordScreen.addEventListener(Event.ADDED_TO_STAGE, forgottenPasswordScreenAddedToStageHandler);
			forgottenPasswordScreen.addEventListener(ForgottenPasswordScreen.EMAIL_CHANGED, forgottenPasswordScreenEmailChangedHandler);
			forgottenPasswordScreen.addEventListener(ForgottenPasswordScreen.SEND_SELECTED, forgottenPasswordScreenSendSelectedHandler);
		}
		
		override public function onRemove() : void
		{
			super.onRemove();
			
			forgottenPasswordScreen.removeEventListener(Event.ADDED_TO_STAGE, forgottenPasswordScreenAddedToStageHandler);
			forgottenPasswordScreen.removeEventListener(ForgottenPasswordScreen.EMAIL_CHANGED, forgottenPasswordScreenEmailChangedHandler);
			forgottenPasswordScreen.removeEventListener(ForgottenPasswordScreen.SEND_SELECTED, forgottenPasswordScreenSendSelectedHandler);
		}
		
		override public function listNotificationInterests() : Array 
		{
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
				Notifications.PASSWORD_RESET,
				Notifications.PASSWORD_RESETTING_FAILED,
				Notifications.APPLICATION_BECAME_ACTIVE
			]);
		}
		
		override public function handleNotification(note : INotification) : void
		{
			switch(note.getName())
			{
				case Notifications.PASSWORD_RESET:
					handlePasswordReset();
					break;
				
				case Notifications.PASSWORD_RESETTING_FAILED:
					handlePasswordResettingFailed(note.getBody() as String);
					break;
				
				case Notifications.APPLICATION_BECAME_ACTIVE:{
					
					handleApplicationBecameActive();
				}
					break;
			}
			
			super.handleNotification(note);
		}
		
		private function handleApplicationBecameActive() : void
		{
			if(Device.isIos())
				return;
			
			
			if(this.forgottenPasswordScreen){
				
				forgottenPasswordScreen.invalidate();
			}
		}
		
		private function forgottenPasswordScreenAddedToStageHandler(event : Event) : void
		{
			forgottenPasswordScreen.setSendButtonEnabled(false);
		}
		
		private function forgottenPasswordScreenEmailChangedHandler(event : Event) : void
		{
			forgottenPasswordScreen.setSendButtonEnabled(forgottenPasswordScreen.email.length > 0);
		}
		
		private function forgottenPasswordScreenSendSelectedHandler(event : Event) : void
		{
			forgottenPasswordScreen.setSendButtonEnabled(false);
			alertView = AlertView.show("Loading", -1);
			
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.sendForgottenPassword(forgottenPasswordScreen.email);
		}
		
		private function handlePasswordReset()  : void
		{
			alertView.hide();
			
			AlertView.show(AlertView.language.translate("mob_signup", "checkEmail"), -1, false);
			
			sendNotification(Notifications.DISPLAY_PREVIOUS_SCREEN);
		}
		
		private function handlePasswordResettingFailed(errorText : String) : void
		{
			alertView.hide();
			forgottenPasswordScreen.setSendButtonEnabled(true);
			
			if(errorText)
			{
				AlertView.show(errorText, -1, false);
			}
		}
	}
}
		