package com.despark.tuning3d.view {
    import by.blooddy.crypto.serialization.JSONer;

    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.assets.AssetsProxy;
    import com.despark.tuning3d.model.brands.BrandsProxy;
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarsProxy;
    import com.despark.tuning3d.model.challenges.ChallengeAlertEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.friend.FriendProxy;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.LanguagesProxy;
    import com.despark.tuning3d.model.levels.LevelData;
    import com.despark.tuning3d.model.packages.PackagesParser;
    import com.despark.tuning3d.model.packages.PackagesProxy;
    import com.despark.tuning3d.model.packages.Packet;
    import com.despark.tuning3d.model.packages.Purchase;
    import com.despark.tuning3d.model.parts.Part;
    import com.despark.tuning3d.model.parts.PartMaterial;
    import com.despark.tuning3d.model.parts.PartMaterialImageTest;
    import com.despark.tuning3d.model.parts.PartSlot;
    import com.despark.tuning3d.model.rewards.DailyRewardsData;
    import com.despark.tuning3d.model.settings.Settings;
    import com.despark.tuning3d.model.settings.SettingsProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.AlertViewHL;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.BlockerView;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.StackedBannerScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.friends.FriendsListPopUp;
    import com.despark.tuning3d.view.components.rewards.RewardType;
    import com.despark.tuning3d.view.components.rewards.RewardsDiamondLabel;
    import com.despark.tuning3d.view.components.rewards.RewardsKeyLabel;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.despark.tuning3d.view.core.*;
    import com.distriqt.AdsManager;
    import com.distriqt.GameService;
    import com.distriqt.GameServiceManager;
    import com.distriqt.RaterManager;
    import com.utils.ArrayUtil;
    import com.utils.Console;
    import com.utils.DateParser;
    import com.utils.DateParser;
    import com.utils.ObjectUtils;
    import com.utils.RandomSplashManager;
    import com.utils.Tracker;

    import feathers.core.PopUpManager;

    import flash.system.System;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;
    import starling.events.Event;
    import starling.textures.Texture;
    import starling.utils.ScaleMode;

    public class HomeScreenPopUpMediator extends StackedBannerScreenMediator {

    public static var isBrandScreenFirstOpen : Boolean = true;
    public static var isInitialOpen : Boolean = true;
    public function HomeScreenPopUpMediator(screen:StackedScreen) {
        super(screen);
    }

    public function get homeScreenPopUp() : HomeScreenPopUp {
        return HomeScreenPopUp(stackedScreen);
    }


    override public function onRegister():void {
        super.onRegister();

        homeScreenPopUp.addEventListener(HomeScreenPopUp.MODE_SELECTED, onModeSelectedHandler);
        AppMode.self.addEventListener("featuredCompleteLoad", onFeaturedCompletedLoad);
        loadData();
        //
        homeScreenPopUp.addEventListener(StackedScreen.TUNING_SELECTED, carSelectedHandler);
        homeScreenPopUp.addEventListener(BrandsScreen.LANGUAGE_CHANGED, languageChangedHandler);
        homeScreenPopUp.addEventListener(StackedScreen.NEWS_SELECTED, newsSelectedHandler);

        if(isInitialOpen){
            var challengeProxy:ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.getLevelList(levelDataReceivedHandler);

            var brandsProxy : BrandsProxy = facade.retrieveProxy(BrandsProxy.NAME) as BrandsProxy;
            brandsProxy.loadBrands();

            var userProxy:UsersProxy =  facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(userProxy.currentUser){
                userProxy.extendSession();
                handleUserAuthenticated();
                if(userProxy.currentUser.socialNetwork == UsersProxy.typeOfSN_GS){
                    GameServiceManager.instance.signIn();
                }
            }else{

                challengeProxy.getGuestDeviceData(Settings.getUniqueId(), function (result:Object):void {
                    if(result){
                        var alertView:ChallengeAlertView;
                        if(result.hasOwnProperty("firstVisit")){
                            alertView = ChallengeAlertView.show(ChallengeAlertView.WELCOME_GUEST_POPUP, null, homeScreenPopUp.stage);
                            challengeProxy.updateGuestDeviceData(Settings.getUniqueId());
                            if(GameServiceManager.instance){
                                GameServiceManager.instance.signIn();
                            }
                        }else{
                            var lastVisitDate:Date = DateParser.parseDate(result.date as String);
                            if(DateParser.getDateTime(lastVisitDate) != DateParser.getDateTime(new Date())){
                                alertView = ChallengeAlertView.show(ChallengeAlertView.DAILY_GUEST_WELCOME, null, homeScreenPopUp.stage);
                                challengeProxy.updateGuestDeviceData(Settings.getUniqueId());
                            }
                        }

                        if(alertView){
                            alertView.open();
                            alertView.addEventListener(ChallengeAlertEvent.LOGIN_TRIGGERED, loginSelectHandler);
                        }
                    }
                });
            }
            //
            if(!HomeScreenPopUp.UPDATE_OBJECT){
                var assetsProxy:AssetsProxy = facade.retrieveProxy(AssetsProxy.NAME) as AssetsProxy;
                assetsProxy.loadUpdateText(newsLoadedCallback);
            }

            isInitialOpen = false;
            SoundManager.playSound(SoundManager.THEME, 1000, true, 0.5);
        }else{
            RaterManager.showRateDialogue();
        }
        Starling.juggler.delayCall(updateNotifications, .5);

        AdsManager.$.initializedCallback = function():void{
            Starling.juggler.delayCall(BannerStackedScreen.showAdBanner, 0.1);
            AdsManager.$.requestReward();
        }
        Tracker.trackEvent("Home Screen", "open");
        System.pauseForGCIfCollectionImminent(1);
    }

    private function levelDataReceivedHandler(value:Array):void{
        LevelData.setUserLevelData(value);
        var userProxy:UsersProxy =  facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        homeScreenPopUp.updateLevelData(userProxy.currentUser);
        homeScreenPopUp.invalidate();
    }

    override public function onRemove():void {
        super.onRemove();
        homeScreenPopUp.removeEventListener(HomeScreenPopUp.MODE_SELECTED, onModeSelectedHandler);
        AdsManager.$.initializedCallback = null;
        AdsManager.$.bannerHide(true);

        homeScreenPopUp.removeEventListener(StackedScreen.TUNING_SELECTED, carSelectedHandler);
        homeScreenPopUp.removeEventListener(BrandsScreen.LANGUAGE_CHANGED, languageChangedHandler);
        homeScreenPopUp.removeEventListener(StackedScreen.NEWS_SELECTED, newsSelectedHandler);
        homeScreenPopUp.dispose();

        Starling.juggler.removeDelayedCalls(dailyRewardsHandler);
    }

    private function newsLoadedCallback():void{
        sendNotification(Notifications.NEWS_LOADED);
    }

    private function newsSelectedHandler(event:Event):void {
        sendNotification(Notifications.DISPLAY_NEWS_POPUP, {
            data: HomeScreenPopUp.UPDATE_OBJECT,
            showBannerAfterExit: true,
            homeScreenPopUp: homeScreenPopUp
        });
        Starling.juggler.delayCall(homeScreenPopUp.updateNotificationCount, 2);
    }

    private function loginSelectHandler(event:Event):void {
        userAuthenticate(null);
    }

    private function languageChangedHandler(event:Event):void {
        var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        languagesProxy.loadLanguage(homeScreenPopUp.selectedLanguage);
    }

    private function carSelectedHandler(event : Event) : void {
        /*var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(usersProxy.httpClient.isUserSigned){
            continueCarSelect(event);
            return;
        }*/

        var textOfMessage:String = AlertView.language.translate("carchooserTop", "agreement_text");
        var title:String = AlertView.language.translate("carchooserTop", "agreement_title");
        var agree:String = AlertView.language.translate("carchooserTop", "agree");
        var disagree:String = AlertView.language.translate("carchooserTop", "disagree");
        if(!textOfMessage){
            textOfMessage = "Вы согласны с <u><a href=\"http://google.com\">лицензионным соглашением</a></u>?";
        }
        SoundManager.playSound(SoundManager.SELECT_PART);
        AlertViewHL.show(title,textOfMessage, agree,disagree, -1, false, function(yesSelected : Boolean) : void {
            if(yesSelected){
                continueCarSelect(event);
            }
        },  0x49cc46);
    }


    private function continueCarSelect(event : Event) : void {
        AppMode.self.setModeTuning();
        var car:Car = new Car();
        car.id = event.data.carId;
        car.carType = AppMode.getCarTypeForMode();
        var info : Object = {
            car : car
        };
        var carSplashImage:ImageLoaderWithStates = RandomSplashManager.createSplashImage(car);
        BlockerView.show();
        if(Starling.current.stage) {
            Starling.current.stage.addChild(carSplashImage);
            carSplashImage.completeHandler = function (image:ImageLoaderWithStates):void {
                info.carSplashImage = carSplashImage;
                showInterstitial(function():void{
                    BlockerView.hide();
                    PopUpManager.removeAllPopUps(true);
                    sendNotification(Notifications.DISPLAY_TUNING_SCREEN, info);
                });
            };
        }
    }

    private function showInterstitial(callback:Function):void{
        if(AdsManager.ENABLED){
            AdsManager.$.requestInterstitial(function(success:Boolean):void{
                callback();
            });
        }else{
            callback();
        }
    }

    private function onFeaturedCompletedLoad(event:Event):void {
        if(homeScreenPopUp.brandsLoaded()) {
            AppMode.self.removeEventListener("featuredCompleteLoad", onFeaturedCompletedLoad);
        }
    }

    private function onModeSelectedHandler(event:Event):void {
        if(isBrandScreenFirstOpen) {
            sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
        } else {
            sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
        }
        isBrandScreenFirstOpen = false;
    }

    private function loadData() : void {
        var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        languagesProxy.loadLanguages();

        var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
        var languageToLoad : Language = Language.createInstance(settingsProxy.currentSettings.selectedLanguageCode);

        if(languagesProxy.defaultLanguageCode.indexOf("ru")<0 && settingsProxy.currentSettings.selectedLanguageCode && settingsProxy.currentSettings.selectedLanguageCode.indexOf("en")<0){
            settingsProxy.currentSettings.selectedLanguageCode = "en-US";
        }

        languageToLoad.code = settingsProxy.currentSettings.selectedLanguageCode;
        languagesProxy.loadLanguage(languageToLoad);
    }

    private function handleUserLoggedIn():void{
        var brandsProxy : BrandsProxy = facade.retrieveProxy(BrandsProxy.NAME) as BrandsProxy;
        brandsProxy.loadBrands();
        homeScreenPopUp.updateUserData(UsersProxy.loggedInUserJSON());
        Starling.juggler.removeDelayedCalls(dailyRewardsHandler);

        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(usersProxy.loginThroughSN == UsersProxy.typeOfSN_GS){
            GameServiceManager.instance.loadUserIcon();
            GameServiceManager.instance.iconCompleteCallback = function(texture:Texture):void{
                UserCircularAvatar.setIconTexture(usersProxy.currentUser.id, texture);
            }
        }
    }

    private function handleUserAuthenticated() : void {
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(usersProxy.currentUser && usersProxy.loginThroughSN == UsersProxy.typeOfSN_GS){
            GameServiceManager.instance.loadUserIcon();
            GameServiceManager.instance.iconCompleteCallback = function(texture:Texture):void{
                UserCircularAvatar.setIconTexture(usersProxy.currentUser.id, texture);
            }
        }
    }


    private function handleLanguageLoaded(loadedLanguage : Language) : void {
        if(homeScreenPopUp) {
            homeScreenPopUp.updateWithLanguage(loadedLanguage);
            if(alertView) {
                alertView.hide();
            }
            sendNotification(Notifications.LANGUAGE_CHANGED);
        }
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [

            Notifications.LANGUAGES_LOADED,
            Notifications.LANGUAGES_LOADING_FAILED,
            Notifications.LANGUAGE_LOADED,
            Notifications.LANGUAGE_LOADING_FAILED,

            Notifications.SHARING_READY,
            Notifications.TUNING_SHARING_MESSAGE_ENTERED,
            Notifications.SHARING_PREPARING_ERROR,
            Notifications.SHARING_PHOTO_UPLOADED,
            Notifications.SHARING_PHOTO_UPLOADING_ERROR,

            Notifications.CHALLENGE_NOTIFICATIONS_VIEWED,
            Notifications.SHARING_PHOTO_UPLOADING_ERROR,
            Notifications.AUTHENTICATE_USER,
            Notifications.CAR_BACKGROUNDS_LOADED,
            Notifications.USER_AUTHENTICATED,
            Notifications.USER_LOGGED_IN,
            Notifications.USER_LOGGED_OUT,
            Notifications.DISPLAY_PREVIOUS_SCREEN,
            Notifications.REMOVE_POP_UP,

            Notifications.BRANDS_LOADED,
            Notifications.PACKAGES_LOADED,
            Notifications.PURCHASES_LOADED,
            Notifications.PENDING_FRIENDS_LOADED,
            Notifications.TIMELINE_SCREEN_FEED_PREVIEW,
            Notifications.UPDATE_NOTIFICATION_COUNTS,
            Notifications.USER_DATA_UPDATED,

            Notifications.NEWS_LOADED
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.BRANDS_LOADED:
                handleBrandsLoaded(note.getBody().brands);
                break;
            case Notifications.LANGUAGE_LOADED:
                handleLanguageLoaded(note.getBody().language);
                break;
            case Notifications.INTERNET_CONNECTED:
                handleInternetConnected();
                break;

            case Notifications.INTERNET_DISCONNECTED:
                handleInternetDisconnected();
                break;

            case Notifications.USER_LOGGED_OUT:
                handleUserLoggedOut();
                break;

            case Notifications.USER_AUTHENTICATED:
                handleUserAuthenticated();
                break;

            case Notifications.USER_LOGGED_IN:
                handleUserLoggedIn();
                break;

            case Notifications.LANGUAGES_LOADED:
                handleLanguagesLoaded();
                break;

            case Notifications.LANGUAGES_LOADING_FAILED:
                handleLanguagesLoadingFailed();
                break;

            case Notifications.LANGUAGE_LOADING_FAILED:
                handleLanguageLoadingFailed();
                break;

            case Notifications.PENDING_FRIENDS_LOADED:
                handlePendingFriendsLoaded(note.getBody());
                homeScreenPopUp.updateNotificationCount();
                break;

            case Notifications.UPDATE_NOTIFICATION_COUNTS:
                homeScreenPopUp.updateNotificationCount();
                break;

            case Notifications.NEWS_LOADED:
                homeScreenPopUp.updateNotificationCount();
                break;

            case Notifications.TIMELINE_SCREEN_FEED_PREVIEW:
                handleDisplayTimelineScreenFeed(note.getBody());
                break;

            case Notifications.PACKAGES_LOADED:
                handleCarPackagesLoaded(note.getBody().packages);
                break;

            case Notifications.PURCHASES_LOADED:
                handlePurchasesLoaded(note.getBody().purchases);
                break;

            case Notifications.AUTHENTICATE_USER:
                handleAuthenticateUser();
                break;

            case Notifications.DISPLAY_PREVIOUS_SCREEN:
                if(homeScreenPopUp.stage){
                    handlePopupRemoved();
                }
                break;
            case Notifications.REMOVE_POP_UP:
                 Console.log(PopUpManager.popUpCount);
                break;

        }

        super.handleNotification(note);
    }

    override protected function userDataUpdatedHandler(event:Event):void {
        updateClaimedRewards(showDailyRewardsList);
    }

    private function handleAuthenticateUser():void {

    }

    private function handlePopupRemoved():void {
        Starling.juggler.delayCall(BannerStackedScreen.showAdBanner, 0.2);
    }

    private function handlePurchasesLoaded(products:Array):void{
        var i:int;
        var packet:Packet;
        var packagePacketMap:Object={};
        var existAllPacket:Boolean=false;
        var existCurrentAllPacket:Boolean=false;
        var packetCurrentAllPacket:Packet=null;
        var purchases:Array=[];
        var packet_id:String;
        var j:int;
        if(!products)
            return;
        var packagesProxy : PackagesProxy = facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;
        if(!packagesProxy.packages)
            return;
        for(i=0; i<packagesProxy.packages.length; i++){
            packet=packagesProxy.packages[i];
            for(j=0; j<products.length; j++ ){
                packet_id=products[j].product;
                if(packet.id==packet_id || packet.productId === products[j].productId){
                    products[j].product = packet.id;
                    packagePacketMap[packet_id]=packet;
                }
            }
        }

        for each( packet in packagePacketMap){
            purchases.push(new Purchase(packet.id, packet.title, true, packet.type, packet.productId));
            packet.lock=false;
            if(packet.type==Packet.TYPE_ALLPACKET){
                existAllPacket=true;
            }

            if(packet.type==Packet.TYPE_CURRENTALLPACKET){
                existCurrentAllPacket=true;
                packetCurrentAllPacket=packet;
            }
        }

        if(existAllPacket){
            for(i=0; i<packagesProxy.packages.length; i++){
                packet=packagesProxy.packages[i];
                packet.lock=false;
            }
        }else if(existCurrentAllPacket){
            for(i=0; i<packagesProxy.packages.length; i++){
                packet=packagesProxy.packages[i];
                if(packetCurrentAllPacket.packets.indexOf(packet.id)>=0){
                    packet.lock=false;
                }
            }
        }

        if(purchases.length>0){
            AdsManager.turnOffAds();
        }
    }


    private function handleBrandsLoaded(brands:Array):void{
        var packagesProxy:PackagesProxy =  facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(usersProxy.currentUser && usersProxy.currentUser.isSuperAdmin){
            packagesProxy.packages = [];
            CarsProxy.IS_FULL = true;
            Constants.IN_APP_PURCHASE_ENABLED = false;
            Constants.update_API_URLS(true);
        }else{
            Constants.update_API_URLS(false);
            CarsProxy.IS_FULL = false;
            Constants.IN_APP_PURCHASE_ENABLED = true;
            packagesProxy.loadPackages(BrandsProxy.makeAllCarsList(brands));
        }
    }

    private function handleCarPackagesLoaded(packages : Array) : void {
        var usersProxy:UsersProxy= facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(Constants.IN_APP_PURCHASE_ENABLED && !usersProxy.purchasesLoaded) {
            usersProxy.getPurchases();
        }
        // packagesLoaded = true;
    }

    private function handleDisplayTimelineScreenFeed(data:Object):void {
        sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, data);
    }

    private function handlePendingFriendsLoaded(friends: Object): void {
        homeScreenPopUp.dispatchEventWith(FriendsListPopUp.PENDING_FRIENDS_READ, false, friends);
    }

    private function updateNotifications():void{
        var friendProxy:FriendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
        friendProxy.loadPendings();
        (facade.retrieveProxy(TimelineProxy.NAME) as TimelineProxy).loadFullUnreadNotificationCount(function (obj:Object) : void {
            homeScreenPopUp.updateNotificationCount();
        });
    }

    private function handleInternetConnected():void{
        trace("handleInternetConnected");
    }


    private function handleInternetDisconnected():void{
        AlertView.show("Lost internet connection",-1, false);
    }


    private function handleUserLoggedOut() : void {
        sendNotification(Notifications.PENDING_FRIENDS_LOADED, {friends: null, unread:false});
        homeScreenPopUp.user = null;
        homeScreenPopUp.updateNotificationCount();
        handleUserLoggedIn();
    }


    private function handleLanguagesLoaded():void{
        Starling.juggler.delayCall(refreshSettings, 1.5);
    }

    private function handleLanguagesLoadingFailed() : void {
        homeScreenPopUp.selectedLanguage = null;
    }

    private function handleLanguageLoadingFailed():void {

    }

    private function refreshSettings() : void {
        var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
        var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        var settings : Settings = settingsProxy.currentSettings;

        if(!settings.selectedLanguageCode) {
            settings.selectedLanguageCode = languagesProxy.defaultLanguageCode;
        }

        homeScreenPopUp.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
        if(!homeScreenPopUp.selectedLanguage){
            settings.selectedLanguageCode = "en-US";
            homeScreenPopUp.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
        }

        homeScreenPopUp.languages = languagesProxy.languages;
        if(homeScreenPopUp.languages != null && homeScreenPopUp.languages.length<=0){
            languagesProxy.loadLanguages();
        }else if (homeScreenPopUp.languages == null){
            languagesProxy.loadLanguages();
        }
    }

        private function updateClaimedRewards(updateCompleteHandler:Function = null):void{
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser){
                return;
            }
            var challengeProxy:ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.getUserRewardData(usersProxy.currentUser.id, function (result:Object):void {
                if(result){
                    var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
                    var updated:Boolean = false;
                    //
                    if(result.claimedDate){
                        result.claimedDate = JSON.parse(result.claimedDate);
                        for(var dayIndex:String in result.claimedDate){
                            var date:Date = DateParser.parseDate(result.claimedDate[dayIndex] as String);
                            if(new Date().getTime() - date.getTime() >= DateParser.DAY){
                                result.claimedDate[dayIndex] = null;
                                delete result.claimedDate[dayIndex];
                                updated = true;
                            }
                        }
                        if(updated){
                            challengeProxy.updateRewardData(usersProxy.currentUser.id, result);
                        }
                    }
                    if(result.unlockedPacks){
                        for(var packId:String in result.unlockedPacks){
                            var expireDate:Date = new Date();
                            expireDate.setTime(result.unlockedPacks[packId]);
                            if(new Date().getTime() > expireDate.getTime() + GameService.REWARDED_PACKAGE_ACTIVE_DURATION){
                                delete result.unlockedPacks[packId];
                                updated = true;
                            }
                        }
                        if(updated){
                            challengeProxy.updateRewardData(usersProxy.currentUser.id, result);
                        }
                        var userId:String = usersProxy.currentUser.id;
                        GameService.$.getSaveData(function (data: String): void {
                            var rewardData: Object = JSONer.parse(data);
                            if (!rewardData.hasOwnProperty(userId) || !rewardData[userId].hasOwnProperty('rewardsPackets')) {
                                rewardData[userId] = {rewardsPackets:{}};
                            }
                            rewardData[userId].rewardsPackets = ObjectUtils.merge(rewardData[userId].rewardsPackets, result.unlockedPacks);
                            GameService.$.saveGameData(JSONer.stringify(rewardData), false);
                        });
                    }
                    applyDailyRewards(result);
                    if(updateCompleteHandler !== null){
                        updateCompleteHandler(result);
                    }
                }else{
                    Console.log();
                }
            });
        }

        private function showDailyRewardsList(result:Object):void{
            if(!Starling.juggler.containsDelayedCalls(dailyRewardsHandler)){
                dailyRewardsHandler(result);
                var nowDate:Date = new Date();
                var tomorrowDate:Date = new Date();
                tomorrowDate.date += 1;
                var delay:Number = 1 + (DateParser.getDateTime(tomorrowDate) - nowDate.time)/DateParser.SECOND;
                Starling.juggler.delayCall(dailyRewardsHandler, delay, result);
            }
        }

        private function applyDailyRewards(result:Object):void{
            var carProxy:CarsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
            carProxy.clearRewardedCarIds();
            carProxy.clearRewardedCarTypes();
            Constants.UNIT_MULTIPLIER = 1;
            var dailyRewards:DailyRewardsData;
            for(var keyIndex:String in result.claimedDate){
                dailyRewards = DailyRewardsData.getRewardsData(parseInt(keyIndex));
                if(dailyRewards.carId){
                    carProxy.addRewardedCarId([dailyRewards.carId]);
                }
                if(dailyRewards.carType){
                    carProxy.addRewardedCarType(dailyRewards.carType);
                }
                if(dailyRewards.rewardsMultiplier){
                    Constants.UNIT_MULTIPLIER = dailyRewards.rewardsMultiplier;
                }
            }
        }

        protected function dailyRewardsHandler(result:Object):void{
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            var challengeProxy:ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            if(!usersProxy.currentUser){
                return;
            }
            if(result){
                var alertView:ChallengeAlertView;
                if(result.hasOwnProperty("firstVisit")){
                    var challengeUser:User = new User(usersProxy.currentUserJSON);
                    challengeProxy.addUserDailyRewardData(UsersProxy.loggedInUser().id, {keys: 0, diamonds: 0});
                    var popupDate:Object = {bonus:{diamond: challengeUser.diamonds, key: challengeUser.keys}};
                    alertView = ChallengeAlertView.show(ChallengeAlertView.WELCOME_USER_POPUP, new DailyRewardsData(popupDate), stackedScreen.stage);
                }else{
                    var lastClaimedDate:Date = DateParser.parseDate(result.date as String);
                    var lastClaimedDay:Number = DateParser.getDateTime(lastClaimedDate);
                    var currentDay:Number = DateParser.getDateTime(new Date());
                    if(lastClaimedDay != currentDay){
                        usersProxy.currentUser.dayIndex = result.dayIndex;
                        var dailyRewardsData:Object = {userData: result, cars: PackagesParser.allCarsList, claimHandler: claimDailyRewards};
                        alertView = ChallengeAlertView.show(ChallengeAlertView.DAILY_REWARDS_LIST, dailyRewardsData, stackedScreen.stage);
                    }
                }

                if(alertView){
                    alertView.addEventListener(ChallengeAlertEvent.VIEW_OPEN, function(event:Event):void{
                        BannerStackedScreen.hideAdBanner(false);
                        AdsManager.$.showBannerAfterComplete = false;
                    });
                    alertView.addEventListener(ChallengeAlertEvent.VIEW_CLOSED, function(event:Event):void{
                        BannerStackedScreen.showAdBanner();
                    });
                    alertView.open();
                }
            }else{
                Console.log();
            }
        }

        private function claimDailyRewards(result:Object):void{
            var challengeProxy:ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            var challengeUser:User = new User(usersProxy.currentUserJSON);
            if(!usersProxy.currentUser){
                return;
            }
            var dailyRewards:DailyRewardsData = DailyRewardsData.getRewardsData(usersProxy.currentUser.dayIndex);
            result.claimedDate ||= {};
            result.dayIndex ||= 0;
            result.claimedDate[result.dayIndex] = DateParser.toW3CDTF(new Date());
            usersProxy.currentUser.dayIndex++;
            if(usersProxy.currentUser.dayIndex >= DailyRewardsData.dailyRewardsList.length){
                usersProxy.currentUser.dayIndex = 0;
            }
            result.dayIndex = usersProxy.currentUser.dayIndex;
            result.date = DateParser.toW3CDTF(new Date());


            var rewardData:Object = {keys: 0, diamonds: 0};
            if(dailyRewards.keys || dailyRewards.diamonds){
                rewardData.keys = Math.min(dailyRewards.keys, challengeUser.levelData.maxKeys - challengeUser.keys);
                rewardData.diamonds = dailyRewards.diamonds;
                challengeProxy.addUserDailyRewardData(UsersProxy.loggedInUser().id, rewardData);
                var bannerScreen:StackedBannerScreen = stackedScreen as StackedBannerScreen;
                if(bannerScreen && bannerScreen.stage){
                    if(dailyRewards.diamonds && dailyRewards.keys){
                        bannerScreen.addResource(RewardType.DIAMONDS, dailyRewards.diamonds, RewardsDiamondLabel.COLOR, bannerScreen.stage.stageWidth / 2, bannerScreen.stage.stageHeight / 2);
                        Starling.juggler.delayCall(function():void{
                            bannerScreen.addResource(RewardType.KEYS, dailyRewards.keys, RewardsKeyLabel.COLOR, bannerScreen.stage.stageWidth / 2, bannerScreen.stage.stageHeight / 2);
                        }, 0.25);
                    }else if(dailyRewards.diamonds){
                        bannerScreen.addResource(RewardType.DIAMONDS, dailyRewards.diamonds, RewardsDiamondLabel.COLOR, bannerScreen.stage.stageWidth / 2, bannerScreen.stage.stageHeight / 2);
                    }else if(dailyRewards.keys){
                        bannerScreen.addResource(RewardType.KEYS, dailyRewards.keys, RewardsKeyLabel.COLOR, bannerScreen.stage.stageWidth / 2, bannerScreen.stage.stageHeight / 2);
                    }
                }
            }
            challengeProxy.updateRewardData(usersProxy.currentUser.id, result);
            applyDailyRewards(result);
        }
}
}
