package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.assets.AssetsProxy;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.news.NewsPopUp;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.despark.tuning3d.view.core.HomeScreenPopUp;
    import com.despark.tuning3d.view.core.PopUpMediator;
    import com.utils.ArrayUtil;
    import com.utils.Tracker;

    public class NewsPopUpMediator extends PopUpMediator{

    private var _loadingAlert:AlertView;
    private var _newsPopup:NewsPopUp;

    public function NewsPopUpMediator(popUp:PopUp) {
        super(popUp);
        _newsPopup = popUp as NewsPopUp;
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        FeedData.CURRENT_USER = usersProxy.currentUser;
    }

    override public function onRegister():void {
        super.onRegister();
        Tracker.trackEvent("Notifications Popup", "open");
        BannerStackedScreen.hideAdBanner(false);
        if(!HomeScreenPopUp.UPDATE_OBJECT){
            var assetsProxy:AssetsProxy = facade.retrieveProxy(AssetsProxy.NAME) as AssetsProxy;
            assetsProxy.loadUpdateText(newsLoadedCallback);
        }
    }

    private function newsLoadedCallback():void{
        _newsPopup.data = HomeScreenPopUp.UPDATE_OBJECT;
        _newsPopup.updateView();
        sendNotification(Notifications.NEWS_LOADED);
    }

    override public function onRemove():void {
        super.onRemove();

        if(_loadingAlert && _loadingAlert.parent)
        {
            _loadingAlert.hide();
        }
        _loadingAlert = null;
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
        ]);
    }
}
}
