package com.despark.tuning3d.view
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.friend.FriendProxy;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.LanguagesProxy;
	import com.despark.tuning3d.model.settings.Settings;
	import com.despark.tuning3d.model.settings.SettingsProxy;
	import com.despark.tuning3d.model.votes.VotesProxy;
	import com.despark.tuning3d.view.components.brands.BrandsScreen;
	import com.despark.tuning3d.view.components.core.StackedScreen;
	import com.despark.tuning3d.view.components.instructions.TutorialView;
	import com.despark.tuning3d.view.components.votes.VotesListScreen;
	import com.despark.tuning3d.view.core.StackedScreenExtendedMediator;
	import com.utils.ArrayUtil;

	import org.puremvc.as3.interfaces.INotification;

	import starling.core.Starling;
	import starling.events.Event;

	public class VotesListScreenMediator extends StackedScreenExtendedMediator {
		private var screenShown:Boolean=false;
		public function VotesListScreenMediator(stackedScreen : StackedScreen) {
			super(stackedScreen);
		}

		override public function onRegister() : void {
			super.onRegister();
			votesListScreen.addEventListener(Event.ADDED_TO_STAGE, votesScreenAddedToStageHandler);
			votesListScreen.addEventListener(VotesListScreen.VOTE_ITEM_LIKED, votesScreenVoteSelectedHandler);
			votesListScreen.addEventListener(VotesListScreen.CHANGE_SEARCH_TEXT, votesScreenChangeSearchTextHandler);
			votesListScreen.addEventListener(VotesListScreen.VOTE_TYPE_CHANGE, votesScreenTypeChangeHandler);
			votesListScreen.addEventListener(BrandsScreen.LANGUAGE_CHANGED,votesScreenLanguageChangedHandler);
			votesListScreen.addEventListener(BrandsScreen.INSTRUCTION_SELECTED, votesScreenInstructionSelectedHandler);
		}

		override public function onRemove() : void {
			super.onRemove();
			votesListScreen.removeEventListener(Event.ADDED_TO_STAGE, votesScreenAddedToStageHandler);
			votesListScreen.removeEventListener(VotesListScreen.VOTE_ITEM_LIKED, votesScreenVoteSelectedHandler);
			votesListScreen.removeEventListener(VotesListScreen.CHANGE_SEARCH_TEXT, votesScreenChangeSearchTextHandler);
			votesListScreen.removeEventListener(VotesListScreen.VOTE_TYPE_CHANGE, votesScreenTypeChangeHandler);
			votesListScreen.removeEventListener(BrandsScreen.LANGUAGE_CHANGED,votesScreenLanguageChangedHandler);
			votesListScreen.removeEventListener(BrandsScreen.INSTRUCTION_SELECTED, votesScreenInstructionSelectedHandler);
			votesListScreen.dispose();
		}

		private function handleUserAuthenticated() : void {
			if (votesListScreen){
				votesListScreen.userSignedIn = true;
			}

			if (stackedScreen && !stackedScreen.stage) {
				return;
			}

            var friendProxy:FriendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
            friendProxy.loadPendings();
		}

		private function votesScreenInstructionSelectedHandler(event:Event):void{
			TutorialView.showCustomUrl();
		}

		public function get votesListScreen() : VotesListScreen {
			return viewComponent as  VotesListScreen;
		}
		
		override public function listNotificationInterests() : Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
				Notifications.USER_AUTHENTICATED,
				Notifications.VOTES_LOADED,
				Notifications.VOTES_LOADING_FAILED,
				Notifications.APPLICATION_BECAME_ACTIVE,
				Notifications.LANGUAGES_LOADED,
				Notifications.LANGUAGES_LOADING_FAILED,
				Notifications.LANGUAGE_LOADED,
				Notifications.LANGUAGE_LOADING_FAILED,
				Notifications.USER_LOGGED_OUT,
				Notifications.USER_AUTHENTICATED,
				Notifications.USER_SESSION_EXTENDED,
				Notifications.USER_SESSION_EXTENDING_FAILED,
				Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE,
                Notifications.TIMELINE_SCREEN_FEED_PREVIEW
			]);
		}
		
		override public function handleNotification(note : INotification) : void {
			switch(note.getName()) {
				case Notifications.VOTES_LOADED:
					handleVotesLoaded(note.getBody());
					break;
				
				case Notifications.VOTES_LOADING_FAILED:
					handleVotesLoadingFailed();
					break;

				case Notifications.APPLICATION_BECAME_ACTIVE:
					handleApplicationBecameActive();
					break;
				
				case Notifications.LANGUAGES_LOADED:
					handleLanguagesLoaded();
					break;
				
				case Notifications.LANGUAGES_LOADING_FAILED:
					handleLanguagesLoadingFailed();
					break;
				
				case Notifications.LANGUAGE_LOADED:
					handleLanguageLoaded(note.getBody().language);
					break;
				
				case Notifications.LANGUAGE_LOADING_FAILED:
					handleLanguageLoadingFailed();
					break;
				
				case Notifications.USER_AUTHENTICATED:
					handleUserAuthenticated();
					break;
				
				case Notifications.USER_SESSION_EXTENDED:
					handleUserSessionExtented();
					break;
				
				case Notifications.USER_SESSION_EXTENDING_FAILED:
					handleUserSessionExtendingFailed();
					break;
				
				case Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE:
					handleUserSessionExtendingNotPossible();
					break;

                case Notifications.TIMELINE_SCREEN_FEED_PREVIEW:
                    handleDisplayTimelineScreenFeed(note.getBody());
				
			}
			
			super.handleNotification(note);
		}
		
		private function handleUserSessionExtendingNotPossible() : void {
			/*	var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.logoutCurrentUser();
			sendNotification(Notifications.DISPLAY_LOGIN_SCREEN);*/
		}
		
		private function handleUserSessionExtented() : void {
			
		}
		
		private function handleUserSessionExtendingFailed() : void {
			
		}

        private function handleDisplayTimelineScreenFeed(data:Object):void {
            sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, data);
        }
		
		private function handleLanguageLoadingFailed() : void {
			
		}
		
		private function handleLanguagesLoadingFailed() : void {
			votesListScreen.selectedLanguage = null;
		}
		
		private function handleLanguageLoaded(loadedLanguage : Language) : void {
			if(votesListScreen) {
				votesListScreen.updateWithLanguage(loadedLanguage);
				
				updateSettings();
				
				sendNotification(Notifications.LANGUAGE_CHANGED);
			}
		}
		
		private function handleLanguagesLoaded():void{
			Starling.juggler.delayCall(refreshSettings, 1.5);
		}
		
		
		private function votesScreenLanguageChangedHandler(event:Event):void{
			var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
			languagesProxy.loadLanguage(votesListScreen.selectedLanguage);
		}
		
		
		private function updateSettings() : void {
			var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			var settings : Settings = settingsProxy.currentSettings;
			if(votesListScreen.selectedLanguage) {
				settings.selectedLanguageCode = votesListScreen.selectedLanguage.code;
			}
			settingsProxy.update(settings);
		}
		
		
		private function refreshSettings() : void {
			var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
			var settings : Settings = settingsProxy.currentSettings;
			
			if(!settings.selectedLanguageCode) {
				settings.selectedLanguageCode = languagesProxy.defaultLanguageCode;
			}
			
			//this.brandsScreen
			votesListScreen.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
			if(!votesListScreen.selectedLanguage){
				settings.selectedLanguageCode="en-US";
				votesListScreen.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
			}
			
			votesListScreen.languages = languagesProxy.languages;
			if(votesListScreen.languages!=null && votesListScreen.languages.length<=0){
				languagesProxy.loadLanguages();
			}else if (votesListScreen.languages==null){
				languagesProxy.loadLanguages();
			}
			
		}
		
		
			
		private function handleApplicationBecameActive() : void {
			if(Device.isIos())
				return;
			
			
			if(this.votesListScreen){
				
				votesListScreen.invalidate();
			}
		}
		
		
		protected override function stackedScreenBackButtonSelectedHandler(event : Event) : void {
			super.stackedScreenBackButtonSelectedHandler(event);
		}
	
		
		private function handleVotesLoaded(votesInfo : Object) : void {
			votesListScreen.votes=votesInfo.votes;
			if(alertView) {
				alertView.hide();
			}
		}
		
		private function handleVotesLoadingFailed() : void {
			if(alertView)
			alertView.hide();
		}
	
		private function votesScreenAddedToStageHandler(event : Event=null) : void {			
			if(!screenShown) {			
				screenShown = true;
				
				var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
				languagesProxy.loadLanguages();
				
			
				var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
				var languageToLoad : Language = Language.createInstance(settingsProxy.currentSettings.selectedLanguageCode);
				languagesProxy.loadLanguage(languageToLoad);
				
				
				
				var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
				votesProxy.load(votesListScreen.queryText, votesListScreen.votingType, votesListScreen.sortBy, votesListScreen.sortByOrder);

                Starling.juggler.delayCall(checkIfVotesLoadedDownloaded, 0.3);
			}
			
		}

		private function checkIfVotesLoadedDownloaded() : void {
			if(votesListScreen.loadedVotesCount == 0) {
                var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
				votesProxy.load(votesListScreen.queryText, votesListScreen.votingType, votesListScreen.sortBy, votesListScreen.sortByOrder);
                Starling.juggler.delayCall(checkIfVotesLoadedDownloaded, 0.2);
			}
		}
		
		private function votesScreenChangeSearchTextHandler(event:Event):void{
			var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
			votesProxy.load(votesListScreen.queryText, votesListScreen.votingType, votesListScreen.sortBy, votesListScreen.sortByOrder);
		}

		private function votesScreenTypeChangeHandler(event:Event):void{
			var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
			votesProxy.load(votesListScreen.queryText, votesListScreen.votingType, votesListScreen.sortBy, votesListScreen.sortByOrder);
		}


		private function votesScreenVoteSelectedHandler(event : Event) : void {
			var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
			if(votesListScreen.selectedVote){
				if(votesListScreen.selectedVote.youHaveLiked){
					votesProxy.vote(votesListScreen.selectedVote.id);
				}else{
					votesProxy.delete_vote(votesListScreen.selectedVote.id);
				}
			}
		}
		

	}
}