package com.despark.tuning3d.view.components.auth
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.despark.tuning3d.view.components.core.StackedScreen;
	import com.distriqt.AdsManager;
	import com.distriqt.extension.applesignin.AppleSignIn;
	import com.utils.FontSelector;

	import feathers.controls.Button;
	import feathers.controls.LayoutGroup;
	import feathers.controls.ScrollContainer;
	import feathers.layout.FlowLayout;
	import feathers.layout.HorizontalAlign;
	import feathers.layout.VerticalLayout;

	import flash.display.Bitmap;
	import flash.display.StageOrientation;
	import flash.events.StageOrientationEvent;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.core.Starling;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;
	import starling.textures.Texture;

	public class AuthScreen extends StackedScreen implements Multilingual {
		public static const TYPE_NORMAL:int=0;
		public static const TYPE_RESTORE:int=1;
		public static const TYPE_FOR_PURCHASE:int=2;
		public static var LEFT_MARGIN:int = 220;
		protected static const inputWidth:int = 266;
		protected var variantLeftMargin:int = 0;
		protected var variantRightMargin:int = 0;

		public static const BACK_BUTTON_SELECTED : String = "BackButtonSelected";
		public static const VK_LOGIN_SELECTED : String = "VK_LOGIN_SELECTED";
		public static const GAME_SERVICE_LOGIN_SELECTED : String = "GAME_SERVICE_LOGIN_SELECTED";
		public static const FB_LOGIN_SELECTED : String = "FB_LOGIN_SELECTED";
		public static const TW_LOGIN_SELECTED : String = "TW_LOGIN_SELECTED";
		public static const APPLE_LOGIN_SELECTED : String = "APPLE_LOGIN_SELECTED";
		public static const CLOSE_WEBVIEW:String = "CLOSE_WEBVIEW";
		
		protected var backgroundQuad : Quad;
		protected var logoImage : Image;
		public var scrollContainer : ScrollContainer;

		
		protected var socialTitleLabel:MultilingualLabel;
		protected var inputTitle:MultilingualLabel;
		protected var gameServiceButton : Button;
		protected var faceBookButton : Button;
		protected var _socialContainer:LayoutGroup;
		protected var _inputContainer:LayoutGroup;

		protected var appleButton : Button;
		protected var _language:Language;
		
		protected var _type:int = TYPE_NORMAL;
		protected var _verticalGap:int = 20;

        [Embed(source="/assets/textures/logo4.png")]
        public static var LOGO_CLASS_4 : Class;

		[Embed(source="/assets/textures/logo3.png")]
        public static var LOGO_CLASS_3 : Class;

		[Embed(source="/assets/textures/logo2.png")]
        public static var LOGO_CLASS_2 : Class;

		[Embed(source="/assets/textures/watermark_for_snapshots.png")]
		public static var WATERMARK_FOR_SNAPSHOTS : Class;
		
		public function AuthScreen(screenName:String = "AuthScreen") {
			super(screenName);
		}

		private function stageOrientationChangeHandler(event:StageOrientationEvent):void {
			updateButtonPosition();
		}

		private function updateButtonPosition():void{
			var marginSize:Number = 24;
			if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_RIGHT){
				variantLeftMargin = marginSize;
				variantRightMargin = 0;
			}else if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_LEFT){
				variantLeftMargin = 0;
				variantRightMargin = marginSize;
			}else{
				variantLeftMargin = 0;
				variantRightMargin = 0;
			}
			invalidate(INVALIDATION_FLAG_SIZE);
		}

		public function set type(value:int):void{
			_type = value;
		}
		
		public function get type():int{
			return _type;
		}
		
		public function updateWithLanguage(language : Language) : void {
			_language = language;

			if(socialTitleLabel){
				socialTitleLabel.updateWithLanguage(language);
				socialTitleLabel.text = socialTitleLabel.text;
			}

			if(inputTitle){
				inputTitle.updateWithLanguage(language);
				inputTitle.text = inputTitle.text;
			}
		}
		
		override protected function initialize() : void {
			super.initialize();
			if(Device.isIphoneXVariant()){
				Starling.current.nativeStage.addEventListener(StageOrientationEvent.ORIENTATION_CHANGE, stageOrientationChangeHandler);
				updateButtonPosition();
			}
			LEFT_MARGIN = stage.stageWidth * 1 / 2.62;

			backgroundQuad = new Quad(1, 1, 0x131313);
			addChild(backgroundQuad);

			scrollContainer = new ScrollContainer();
			addChild(scrollContainer);

			_socialContainer = new LayoutGroup();
			var socLayout:VerticalLayout = new VerticalLayout();
			_socialContainer.layout = socLayout;


			_inputContainer = new LayoutGroup();
			var inputLayout:FlowLayout = new FlowLayout();
			inputLayout.verticalGap = _verticalGap;
			inputLayout.horizontalGap = 0;
			_inputContainer.width = inputWidth + 1;
			_inputContainer.layout = inputLayout;

			socialTitleLabel = new MultilingualLabel();
			socialTitleLabel.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0xFFFFFF);
			socialTitleLabel.textFormat.align = TextFormatAlign.CENTER;
			socialTitleLabel.textFormat.bold = false;
			socialTitleLabel.width = LEFT_MARGIN;
			socialTitleLabel.isHTML = true;
			socialTitleLabel.text = "Log in using your social\naccount to get quick access";
			socialTitleLabel.paddingTop = 1;
			socialTitleLabel.paddingBottom = 25;
			_socialContainer.addChild(socialTitleLabel);


			inputTitle = new MultilingualLabel();
			inputTitle.textFormat = new TextFormat(FontSelector.getMontserratSemiBold(), 14, 0xFFFFFF);
			inputTitle.textFormat.align = TextFormatAlign.CENTER;
			inputTitle.textFormat.bold = false;
			inputTitle.width = inputWidth;
			inputTitle.isHTML = true;
			_inputContainer.addChild(inputTitle);

			var logoBitmap: Bitmap = new LOGO_CLASS_2();
			logoImage = new Image(Texture.fromBitmap(logoBitmap, false, false, assetManager.scaleFactor));
			logoImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
			_socialContainer.addChild(logoImage);


			faceBookButton  = ButtonFactory.defaultButton("Sign in with Facebook", 0x4267b2); //ButtonFactory.circleButton("a_fb_icon2");
			faceBookButton.labelFactory = LabelFactory.profileButtonLabel2;
			var fbIcon:Image = new Image(ButtonFactory.assetManager.getTexture("a_fb_icon"));
			fbIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
			faceBookButton.defaultIcon = fbIcon;
			faceBookButton.iconOffsetX = -8;
			faceBookButton.labelOffsetX = -5;
			faceBookButton.addEventListener(Event.TRIGGERED, faceBookLoginHandler);
			_socialContainer.addChild(faceBookButton);
			faceBookButton.validate();

			var gameServiceLabel:String = Device.isIos()?"Game Center Login":"Game Services Login";
			gameServiceButton  = ButtonFactory.defaultButton(gameServiceLabel, 0xFFFFFE);
			gameServiceButton.labelFactory = LabelFactory.profileButtonLabelBlack;
			var gamesIcon:Image;
			if(Device.isIos()){
				gamesIcon = new Image(ButtonFactory.assetManager.getTexture("a_game_center_icon"));
			}else{
				gamesIcon = new Image(ButtonFactory.assetManager.getTexture("a_game_service_icon"));
			}
			gamesIcon.height = 25;
			gamesIcon.scale = gamesIcon.scaleY;
			gameServiceButton.defaultIcon = gamesIcon;
			var iconOffset:Number = (Device.isIos()?-15:-8);
			var labelOffset:Number = (Device.isIos()?-12:-8);
			gameServiceButton.iconOffsetX = iconOffset;
			gameServiceButton.labelOffsetX = labelOffset;

			gameServiceButton.addEventListener(Event.TRIGGERED, gameServiceLoginHandler);
			_socialContainer.addChild(gameServiceButton);
			gameServiceButton.validate();


			appleButton  = ButtonFactory.defaultButton("Sign in with Apple", 0xFFFFFE); //ButtonFactory.circleButton("a_fb_icon2");
			appleButton.labelFactory = LabelFactory.profileButtonLabelBlack;
			var appleIcon:Image = new Image(ButtonFactory.assetManager.getTexture("a_apple_icon2"));
			appleIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
			appleButton.defaultIcon = appleIcon;
			appleButton.iconOffsetX = -20;
			appleButton.labelOffsetX = -20;
			appleButton.addEventListener(Event.TRIGGERED, appleLoginHandler);
			appleButton.visible = (Device.isIos() && AppleSignIn.isSupported);
			_socialContainer.addChild(appleButton);

			faceBookButton.width = 175;
			faceBookButton.height = 30;

			gameServiceButton.width = 175;
			gameServiceButton.height = 30;

			appleButton.width = 175;
			appleButton.height = 30;

			scrollContainer.addChild(_socialContainer);
			appleButton.validate();
			initializeBackButton();
		}

		override public function initializeBackButton():void{
			backButton = ButtonFactory.circleButton("model_close", 0.6);
			backButton.setSize(30, 30);
			backButton.x = stage.stageWidth - (backButton.width + 5);
			backButton.y = 5;

			if(Device.isIphoneXVariant()){
				backButton.x -= 7;
				backButton.y += 7;
			}
			backButton.addEventListener(Event.TRIGGERED, backButtonTriggeredHandler);
			addChild(backButton);
		}
		
		private function logout():void{
			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.logoutCurrentUser();
		}
		
		private function vkLoginHandler(event : Event):void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			logout();
			dispatchEventWith(VK_LOGIN_SELECTED);
		}

		private function gameServiceLoginHandler(event : Event):void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			logout();
			dispatchEventWith(GAME_SERVICE_LOGIN_SELECTED);
		}

		private function faceBookLoginHandler(event : Event):void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			logout();
			dispatchEventWith(FB_LOGIN_SELECTED);
		}

		private function appleLoginHandler(event : Event): void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			logout();
			dispatchEventWith(APPLE_LOGIN_SELECTED);
		}

		override protected function draw() : void {
			super.draw();
			if(isInvalid(INVALIDATION_FLAG_SIZE)) {
				backgroundQuad.width = variantLeftMargin + LEFT_MARGIN;
				backgroundQuad.height = actualHeight;
				
				scrollContainer.width = actualWidth;
				scrollContainer.height = actualHeight;

				logoImage.x = variantLeftMargin + (LEFT_MARGIN - logoImage.width) / 2;
				logoImage.y = 0;

				socialTitleLabel.x = variantLeftMargin + (LEFT_MARGIN - socialTitleLabel.width) / 2;
				socialTitleLabel.y = logoImage.y + logoImage.height + 10;
				socialTitleLabel.height = NaN;
				socialTitleLabel.validate();

				gameServiceButton.x = variantLeftMargin + (LEFT_MARGIN - gameServiceButton.width) / 2;
				gameServiceButton.y = socialTitleLabel.y + socialTitleLabel.height + _verticalGap;

				faceBookButton.x = gameServiceButton.x;
				faceBookButton.y = gameServiceButton.y + gameServiceButton.height + _verticalGap;

				appleButton.x = faceBookButton.x;
				appleButton.y = faceBookButton.y + faceBookButton.height + _verticalGap;

				_socialContainer.height = NaN;
				_socialContainer.validate();
				_socialContainer.y = Math.max((actualHeight - _socialContainer.height) / 2 - 75, 25);
			}
		}

		override public function dispose():void{
			if(Device.isIphoneXVariant()){
				Starling.current.nativeStage.removeEventListener(StageOrientationEvent.ORIENTATION_CHANGE, stageOrientationChangeHandler);
			}
		}

		protected override function backButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
			dispatchEventWith(BACK_BUTTON_SELECTED);
			super.backButtonTriggeredHandler(event);
		}
	}
}