package com.despark.tuning3d.view.components.auth
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.ExtentedButton;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.core.MultilingualButton;
	import com.utils.FontSelector;
	import com.utils.Tracker;

	import feathers.controls.Button;
	import feathers.controls.LayoutGroup;
	import feathers.layout.HorizontalAlign;
	import feathers.layout.VerticalAlign;

	import flash.events.FocusEvent;

	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.display.Image;
	import starling.display.Quad;

	import starling.events.Event;
	import starling.events.KeyboardEvent;
	import starling.utils.StringUtil;
	import starling.utils.StringUtil;

	public class LoginScreen extends AuthScreen {
		public static const TYPE_NORMAL:int = 0;
		public static const USERNAME_CHANGED : String = "UsernameChanged";
		public static const PASSWORD_CHANGED : String = "PasswordChanged";
		public static const REGISTER_SELECTED : String = "RegisterSelected";
		public static const LOGIN_SELECTED : String = "LoginSelected";
		public static const PASSWORD_RESET_SELECTED : String = "PasswordResetSelected";
		public static const BACK_BUTTON_SELECTED : String = "BackButtonSelected";
		public static const CLOSE_WEBVIEW:String = "CLOSE_WEBVIEW";

		private var userInputView : ProfilePageLabel;
		private var passwordInputView : ProfilePageLabel;
		private var loginButton : MultilingualButton;
		private var registerButton : MultilingualButton;
		private var resetPasswordButton : MultilingualButton;
		public var closeWebviewButton:Button;

		public function LoginScreen() {
			super("LoginScreen");
		}

		public override function updateWithLanguage(language : Language) : void {
			super .updateWithLanguage(language);
			userInputView.placeholderText = language.translate("mob_signup", "username").toUpperCase();
			
			passwordInputView.placeholderText = language.translate("mob_signup", "password").toUpperCase();
			
			resetPasswordButton.updateWithLanguage(language);
			resetPasswordButton.label = language.translate("mob_signup", "forgotPassword");

			loginButton.label = language.translate("mob_signup", "login").toLocaleUpperCase();
			
			registerButton.updateWithLanguage(language);
			registerButton.label = '<font color="#FFFFFF">Don\'t have an account?</font><br><b>' + language.translate("mob_signup", "createAccount") + '</b>';
		}

		override public function ShowDone():void{
			Tracker.trackEvent("Login Screen", "open");
		}
		
		override protected function initialize() : void {
			super.initialize();

			inputTitle.paddingTop = 5;
			inputTitle.paddingBottom = -4;

			closeWebviewButton  = ButtonFactory.circleButton("closeButton");
			(closeWebviewButton as ExtentedButton).color = 0xFFFFFF;
			closeWebviewButton.addEventListener(Event.TRIGGERED, closeWebviewHandler);
			closeWebviewButton.visible = false;
			closeWebviewButton.width = 30;
			closeWebviewButton.height = 30;
			addChild(closeWebviewButton);

			registerButton = new MultilingualButton();
			registerButton.addEventListener(Event.TRIGGERED, registerButtonTriggeredHandler);
			registerButton.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0x3a91fa);
			registerButton.textFormat.align = TextFormatAlign.CENTER;
			registerButton.isHTML = true;
			registerButton.height = NaN;
			registerButton.validate();
			registerButton.verticalAlign = VerticalAlign.TOP;
			registerButton.paddingTop = -5;
			registerButton.paddingBottom = 19;
			_inputContainer.addChild(registerButton);


			userInputView = new ProfilePageLabel();
			userInputView.addEventListener(Event.CHANGE, userInputViewChangeHandler);
			userInputView.focusInCallback = function():void{
				registerButton.isEnabled = false;
			}
			userInputView.focusOutCallback = function():void{
				registerButton.isEnabled = true;
			}

			userInputView.width = inputWidth;
			userInputView.height = 30;
			_inputContainer.addChild(userInputView);
			
			passwordInputView = new ProfilePageLabel();
			passwordInputView.addEventListener(Event.CHANGE, passwordInputViewChangeHandler);
			passwordInputView.focusInCallback = function():void{
				registerButton.isEnabled = false;
			}
			passwordInputView.focusOutCallback = function():void{
				registerButton.isEnabled = true;
			}
			passwordInputView.displayAsPassword = true;
			passwordInputView.width = inputWidth;
			passwordInputView.height = 30;
			passwordInputView.addEventListener(KeyboardEvent.KEY_DOWN,textInputHandler);
			_inputContainer.addChild(passwordInputView);
			
			loginButton = ButtonFactory.defaultButton("LOGIN", 0xC90809);
			var logoutIcon:Image = new Image(ButtonFactory.assetManager.getTexture("a_logout_icon"));
			logoutIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
			loginButton.defaultIcon = logoutIcon;
			loginButton.iconOffsetX = -12;
			loginButton.labelOffsetX = -3;
			loginButton.labelFactory = LabelFactory.profileButtonLabel;
			loginButton.width = 105;
			loginButton.height = 30;
			loginButton.addEventListener(Event.TRIGGERED, loginButtonTriggeredHandler);
			_inputContainer.addChild(loginButton);

			
			resetPasswordButton = new MultilingualButton();
			resetPasswordButton.addEventListener(Event.TRIGGERED, resetPasswordButtonTriggeredHandler);
			resetPasswordButton.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 12, 0xFFFFFF);
			resetPasswordButton.textFormat.align = TextFormatAlign.RIGHT;
			resetPasswordButton.width = _inputContainer.width - loginButton.width;
			resetPasswordButton.height = 30;
			resetPasswordButton.horizontalAlign = HorizontalAlign.RIGHT;
			_inputContainer.addChild(resetPasswordButton);

			inputTitle.text = "<b>LOGIN IN TO 3DT ACCOUNT</b>";
			inputTitle.height = NaN;
			scrollContainer.addChild(_inputContainer);

			userInputView.onEnterPress = function() : void {
				passwordInputView.setFocus();
			}

			passwordInputView.onEnterPress = function() : void {
				loginButtonTriggeredHandler(null);
			}
		}
		
		private function logout():void{
			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.logoutCurrentUser();
		}

		private function closeWebviewHandler(event:Event):void{

			dispatchEventWith(CLOSE_WEBVIEW);
		}

		override protected function draw() : void {
			super.draw();
			if(isInvalid(INVALIDATION_FLAG_SIZE)) {
				inputTitle.height = NaN;
				inputTitle.validate();

				closeWebviewButton.x = actualWidth - 6 - closeWebviewButton.width;

				closeWebviewButton.y = 2.5;
				if(registerButton){
					registerButton.x = (_inputContainer.width - registerButton.width) / 2;
				}

				_inputContainer.validate();
				_inputContainer.x = (actualWidth + LEFT_MARGIN - _inputContainer.width) / 2 - variantRightMargin;
				_inputContainer.y = _socialContainer.y;//Math.max((actualHeight - _inputContainer.height) / 2 - 50, 25);
			}
		}
		
		public function textInputHandler(event:KeyboardEvent):void {

		}

		private function registerButtonTriggeredHandler(event : Event) : void {
			if(userInputView.hasFocus){
				userInputView.clearFocus();
				return;
			}
			if(passwordInputView.hasFocus){
				passwordInputView.clearFocus();
				return;
			}
			SoundManager.playSound(SoundManager.PART_MENU);
			dispatchEventWith(REGISTER_SELECTED);
		}
		
		public function setLoginButtonEnabled(value : Boolean) : void {
			if(loginButton) {
				loginButton.alpha = value ? 1.0 : 0.5;
				loginButton.touchable = value;
			}
		}
		
		public function get username() : String {
			return StringUtil.trim(userInputView.text);
		}
		
		public function get password() : String {
			return StringUtil.trim(passwordInputView.text);
		}
		
		private function userInputViewChangeHandler(event : Event) : void {
			dispatchEventWith(USERNAME_CHANGED);
		}
		
		private function passwordInputViewChangeHandler(event : Event) : void {
			dispatchEventWith(PASSWORD_CHANGED);
		}

		private function loginButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			//logout();
			dispatchEventWith(LOGIN_SELECTED);
		}
		
		public function clearTextInputViews() : void {
			if(userInputView) {
				userInputView.text = null;
			}
			
			if(passwordInputView) {
				passwordInputView.text = null;
			}
		}
		
		private function resetPasswordButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.PART_MENU);
			logout();
			dispatchEventWith(PASSWORD_RESET_SELECTED);
		}
	}
}