package com.despark.tuning3d.view.components.auth {
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.greensock.TweenLite;
import com.utils.FontSelector;

import feathers.controls.Button;

import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.FeathersControl;
import feathers.core.ITextRenderer;
import feathers.layout.FlowLayout;
import feathers.layout.HorizontalAlign;
import feathers.layout.VerticalAlign;

import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.core.Starling;
import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class ProfileAlertView extends FeathersControl {
    private var backgroundImage:Image;
    private var confirmButton:MultilingualButton;
    private var closeButton:Button;
    private var titleField:Label;
    private var messageField:Label;
    private var onConfirm:Function;
    private var onOpen:Function;
    private var controlGroup:LayoutGroup;
    private var _exclamationIcon:Image;
    //
    private var titleText:String;
    private var messageText:String;
    private var confirmLabel:String;
    public function ProfileAlertView() {
        super();
        width = 240;
        height = 130;
    }

    public static function show(title:String = '', message:String = '', time:Number = 0, confirmHandler:Function = null, confirmLabel:String = null):ProfileAlertView{
        SoundManager.playSound(SoundManager.DISABLED_PART);
        var alertView:ProfileAlertView = new ProfileAlertView();
        alertView.confirmLabel = (confirmLabel||"CONFIRM NOW");
        alertView.titleText = title;
        alertView.messageText = message;
        alertView.onConfirm = confirmHandler;
        alertView.onOpen = function():void{
            if(time > 0){
                TweenLite.delayedCall(time, alertView.close);
            }
        };
        alertView.open();
        return alertView;
    }

    private function open():void{
        alpha = 0;
        x = (Starling.current.stage.stageWidth - width)/2;
        y = (Starling.current.stage.stageHeight - height)/2;
        Starling.current.stage.addChild(this);
        var alertView:ProfileAlertView = this;
        TweenLite.to(alertView, .2, {alpha: 1, onComplete: onOpen});
    }

    private function close():void{
        TweenLite.killDelayedCallsTo(close);
        var alertView:ProfileAlertView = this;
        TweenLite.to(alertView, .1, {alpha: 0, onComplete: function():void{
                if(alertView){
                    alertView.removeFromParent(true);
                    alertView = null;
                }
            }});
    }

    override protected function initialize():void {
        super.initialize();
        //
        if(backgroundImage){
            backgroundImage.removeFromParent(true);
        }
        var backgroundTexture : Texture = ButtonFactory.assetManager.getTexture("controlBackground");
        backgroundImage = new Image(backgroundTexture);
        backgroundImage.scale9Grid = new Rectangle(backgroundTexture.frameWidth/3, backgroundTexture.frameHeight/3,backgroundTexture.frameWidth/3, backgroundTexture.frameHeight/3);
        backgroundImage.width = width;
        backgroundImage.height = height;
        backgroundImage.alpha = .9;
        addChildAt(backgroundImage, 0);

        controlGroup = new LayoutGroup();
        var verticalLayout:FlowLayout = new FlowLayout();
        verticalLayout.verticalGap = 12.5;
        verticalLayout.horizontalGap = 4;
        verticalLayout.horizontalAlign = HorizontalAlign.CENTER;
        verticalLayout.rowVerticalAlign = VerticalAlign.MIDDLE;
        controlGroup.layout = verticalLayout;
        controlGroup.width = width - 30;
        controlGroup.height = height - 30;
        controlGroup.x = controlGroup.y = 10;


        _exclamationIcon = new Image(ButtonFactory.assetManager.getTexture("profile_not"));
        _exclamationIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        _exclamationIcon.color = 0xb00000;


        titleField = new Label();
        titleField.text = titleText;
        titleField.height = 15;
        titleField.paddingTop = -1;
        titleField.textRendererFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratSemiBold(), 12, 0xb00000);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.isHTML = true;
            return textRenderer;
        };

        messageField = new Label();
        messageField.text = messageText;
        messageField.wordWrap = true;
        messageField.width = controlGroup.width;
        messageField.height = 30;
        messageField.textRendererFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratSemiBold(), 9, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.isHTML = true;
            textRenderer.wordWrap = true;
            return textRenderer;
        };

        confirmButton = ButtonFactory.defaultButton(confirmLabel, 0xC90809);
        confirmButton.width = 125;
        confirmButton.height = 30;
        confirmButton.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratSemiBold(), 12, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        confirmButton.addEventListener(Event.TRIGGERED, confirmHandler);

        closeButton = ButtonFactory.popupCloseButton(10, 10);
        closeButton.addEventListener(Event.TRIGGERED, closeHandler);
        closeButton.x = actualWidth - (closeButton.width + 9);
        closeButton.y = 9;

        controlGroup.addChild(_exclamationIcon);
        controlGroup.addChild(titleField);
        controlGroup.addChild(messageField);
        controlGroup.addChild(confirmButton);
        addChild(controlGroup);
        addChild(closeButton);
    }

    override public function dispose():void {
        super.dispose();
        if(backgroundImage){
            backgroundImage.removeFromParent(true);
            backgroundImage = null;
        }
        if(confirmButton){
            confirmButton.removeFromParent(true);
            confirmButton = null;
        }
        if(closeButton){
            closeButton.removeFromParent(true);
            closeButton = null;
        }
        if(titleField){
            titleField.removeFromParent(true);
            titleField = null;
        }
        if(messageField){
            messageField.removeFromParent(true);
            messageField = null;
        }
        if(controlGroup){
            controlGroup.removeFromParent(true);
            controlGroup = null;
        }
        if(_exclamationIcon){
            _exclamationIcon.removeFromParent(true);
            _exclamationIcon = null;
        }
    }

    private function closeHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        close();
    }

    private function confirmHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART);
        close();
        if(onConfirm){
            onConfirm.call();
        }
    }
}
}
