package com.despark.tuning3d.view.components.auth
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.core.MultilingualButton;
	import com.despark.tuning3d.view.components.core.MultilingualCheck;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.utils.FontSelector;
	import com.utils.Tracker;

	import feathers.controls.Button;
	import feathers.core.FeathersControl;
	import feathers.skins.ImageStateValueSelector;

	import flash.text.TextFormat;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;
	import starling.utils.StringUtil;

	public class RegisterScreen extends AuthScreen {
		public static const REGISTER_SELECTED : String = "RegisterSelected";
		public static const AGREEMENT_SELECTED : String = "AgreemnetSelected";
		public static const AGREE_CHECK_CHANGED : String = "AgreeCheckChanged";
		public static const EMAIL_CHANGED : String = "EmailChanged";
		public static const USERNAME_CHANGED : String = "UsernameChanged";
		public static const PASSWORD_CHANGED : String = "PasswordChanged";
		
		private var registerButton : MultilingualButton;

		private var emailInputView : ProfilePageLabel;
		private var userInputView : ProfilePageLabel;
		private var passwordInputView : ProfilePageLabel;

		private var check : MultilingualCheck;
		private var dummyQuad : Quad;
		private var emailErrorLabel : MultilingualLabel;
		private var userErrorLabel : MultilingualLabel;
		private var passwordErrorLabel : MultilingualLabel;
		private var errorLabelsMap : Object;
		private var agreementButton : Button;
		private var subscribeCheck : MultilingualCheck;
		
		public function RegisterScreen() {
			super();
			
			//title = "REGISTER";
			errorLabelsMap = {};   
			
			name="RegisterScreen";
		}
		
		public override function updateWithLanguage(language : Language) : void {
			super. updateWithLanguage(language);
			inputTitle.text = language.translate("signUp", "register").toUpperCase();
			
			emailInputView.placeholderText = language.translate("signUp", "email").toUpperCase()+' <font color="#ED3F25" ><i><b>(optional)</b></i></font>';
			
			userInputView.placeholderText = language.translate("signUp", "username").toUpperCase();
			
			passwordInputView.placeholderText = language.translate("signUp", "password").toUpperCase();
			
			registerButton.label = language.translate("signUp", "register").toUpperCase();
			
			check.updateWithLanguage(language);
			check.label = language.translate("mob_signup", "readAndAgree") + " <font color='#ED3F25'>" + language.translate("mob_signup", "userAgreement") + "</font>      ";
			check.validate();
			
			subscribeCheck.updateWithLanguage(language);
			subscribeCheck.label = language.translate("signUp", "newsletter");
			
			emailErrorLabel.updateWithLanguage(language);
			userErrorLabel.updateWithLanguage(language);
			passwordErrorLabel.updateWithLanguage(language);
		}

		override public function ShowDone():void{
			Tracker.trackEvent("Register Screen", "open");
		}
		
		override protected function initialize() : void {
			super.initialize();

			inputTitle.paddingTop = 5;
			inputTitle.paddingBottom = -2;

			userErrorLabel = errorLabelWithKey("username");
			_inputContainer.addChild(userErrorLabel);

			userInputView = new ProfilePageLabel();
			userInputView.addEventListener(Event.CHANGE, userInputViewChangeHandler);
			userInputView.setSize(240, 30);
			_inputContainer.addChild(userInputView);

			passwordInputView = new ProfilePageLabel();
			passwordInputView.addEventListener(Event.CHANGE, passwordInputViewChangeHandler);
			passwordInputView.displayAsPassword = true;
			passwordInputView.setSize(240, 30);
			_inputContainer.addChild(passwordInputView);

			emailErrorLabel = errorLabelWithKey("email");
			_inputContainer.addChild(emailErrorLabel);
			
			emailInputView = new ProfilePageLabel();
			emailInputView.addEventListener(Event.CHANGE, emailInputViewChangeHandler);
			emailInputView.setSize(240, 30);
			_inputContainer.addChild(emailInputView);

			
			passwordErrorLabel = errorLabelWithKey("password");
			_inputContainer.addChild(passwordErrorLabel);

			
			check = checkFactory();
			check.isSelected = true;
			check.addEventListener(Event.CHANGE, checkChangeHandler);
			check.includeInLayout = false;
			_inputContainer.addChild(check);
			
			subscribeCheck = checkFactory();
			subscribeCheck.isSelected = true;
			subscribeCheck.addEventListener(Event.CHANGE, subscribeCheckTriggeredHandler);
			subscribeCheck.includeInLayout = false;
			_inputContainer.addChild(subscribeCheck);

			registerButton = ButtonFactory.defaultButton("REGISTER", 0xC90809);
			registerButton.addEventListener(Event.TRIGGERED, registerButtonTriggeredHandler);
			registerButton.labelFactory = LabelFactory.profileButtonLabel;
			registerButton.setSize(120, 30);
			registerButton.includeInLayout = false;
			_inputContainer.addChild(registerButton);
			
			dummyQuad = new Quad(1, 1, 0xFFFFFF);
			dummyQuad.alpha = 0;
			_inputContainer.addChild(dummyQuad);
			
			agreementButton = new Button();
			agreementButton.defaultSkin = new Quad(1,1,0x000000);
			agreementButton.defaultSkin.alpha = 0;
			agreementButton.addEventListener(Event.TRIGGERED, agreementButtonTriggeredHandler);
			agreementButton.width = 85;
			agreementButton.height = 20;
			agreementButton.includeInLayout = false;
			_inputContainer.addChild(agreementButton);

			emailInputView.onEnterPress = function() : void {
				userInputView.setFocus();
			}

			userInputView.onEnterPress = function() : void {
				passwordInputView.setFocus();
			}

			passwordInputView.onEnterPress = function() : void {
				registerButtonTriggeredHandler(null);
			}

			scrollContainer.addChild(_inputContainer);
			super.initializeBackButton();
		}
		
		private function checkFactory() : MultilingualCheck {
			var iconSelector : ImageStateValueSelector = new ImageStateValueSelector();
			iconSelector.defaultValue = assetManager.getTexture("check_new");
			iconSelector.defaultSelectedValue = assetManager.getTexture("check_sel_new");
			
			var check : MultilingualCheck = new MultilingualCheck();
			check.labelOffsetX = 8;
			check.stateToIconFunction = iconSelector.updateValue;
			check.textFormat = new TextFormat(FontSelector.getFont(), 10, 0xFFFFFF);
			check.isHTML = true;
			check.defaultIcon = new Image(assetManager.getTexture("check_new"));
			check.selectedUpIcon = new Image(assetManager.getTexture("check_sel_new"));
			check.selectedDownIcon = check.selectedUpIcon;
			check.x = 0;
			
			return check;
		}
		
		private function errorLabelWithKey(key : String) : MultilingualLabel {
			var errorLabel : MultilingualLabel = new MultilingualLabel();
			errorLabel.textFormat = new TextFormat(FontSelector.getFont(), 9, 0xFF0000);
			errorLabel.isHTML=true;
				
			errorLabelsMap[key] = errorLabel;
			errorLabel.includeInLayout = false;
			return errorLabel;
		}
		
		override protected function draw() : void {
			super.draw();
			inputTitle.textFormat.font = FontSelector.getMontserratLight();
			inputTitle.text = "<b>CREATE NEW ACCOUNT</b><br><font size='9'>Fill in the forms below to get instant access:</font>";
			inputTitle.height = NaN;
			inputTitle.validate();

			check.x = passwordInputView.x;
			check.y = emailInputView.y + emailInputView.height + _verticalGap;
			
			subscribeCheck.x = check.x;
			subscribeCheck.y = check.y + _verticalGap;
			
			agreementButton.x = check.x + check.width - agreementButton.width;
			agreementButton.y = check.y - 2;

			registerButton.x = (_inputContainer.width - registerButton.width) / 2;
			registerButton.y = subscribeCheck.y + subscribeCheck.height + _verticalGap;
			
			dummyQuad.y = registerButton.y + registerButton.height + 15;


			_inputContainer.x = (actualWidth + LEFT_MARGIN - _inputContainer.width) / 2 - variantRightMargin;
			_inputContainer.y = _socialContainer.y;
		}
		
		private function checkChangeHandler(event : Event) : void
		{
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(AGREE_CHECK_CHANGED);
		}
		
		public function get agree() : Boolean 
		{
			return check.isSelected;
		}
		
		public function setRegisterButtonEnabled(value : Boolean) : void {
			registerButton.alpha = value ? 1.0 : 0.5;
			registerButton.touchable = value;
		}
		
		public function get email() : String 
		{
			return StringUtil.trim(emailInputView.text);
		}
		
		public function get username() : String 
		{
			return StringUtil.trim(userInputView.text);
		}
		
		public function get password() : String 
		{
			return StringUtil.trim(passwordInputView.text);
		}
		
		private function emailInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(EMAIL_CHANGED);
		}
		
		private function userInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(USERNAME_CHANGED);
		}
		
		private function passwordInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(PASSWORD_CHANGED);
		}
		
		private function registerButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(REGISTER_SELECTED);
		}

		private function subscribeCheckTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
		}

		protected override function backButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
			super.backButtonTriggeredHandler(event);
		}
		
		private function agreementButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(AGREEMENT_SELECTED);
		}
		
		public function clearErrorLabels() : void {
			for(var key : String in errorLabelsMap) {
				errorLabelsMap[key].text = null;
			}
		}
		
		public function setErrorText(key : String, text : String) : void {
			if(errorLabelsMap[key]) {
				errorLabelsMap[key].text = text;
				errorLabelsMap[key].validate();
			} else {
				AlertView.show("Unrecognized key: " + key, -1, false);
			}
		}

		public function setErrorTexts(errorInfo:Object) : void {
			var error : Object;
			if(errorInfo.hasOwnProperty("statusCode") && errorInfo.statusCode == 400
					&& errorInfo.hasOwnProperty("success") && !errorInfo.success
					&& errorInfo.hasOwnProperty("errorText") && errorInfo.errorText is String){
				error = JSON.parse(errorInfo.errorText);
				if(error.hasOwnProperty("msg") && error.msg == "duplicate_email"){
					if(errorInfo.hasOwnProperty("errorMessage")){
						setErrorText("email", errorInfo.errorMessage);
					}else{
						setErrorText("email", AlertView.language.translate("errorCodes", error.error));
					}
				}
			}else{
				for each(error in errorInfo) {
					if(error && error.hasOwnProperty("param") && error.hasOwnProperty("msg")){
						var key : String = error.param;
						var code : String = error.msg;
						setErrorText(key, AlertView.language.translate("errorCodes", code));
					}
				}
			}


			var offsetX:Number = 8;
			var offsetY:Number = -2;

			userErrorLabel.x = userInputView.x + offsetX;
			userErrorLabel.y = userInputView.y - userErrorLabel.height + offsetY;

			passwordErrorLabel.x = passwordInputView.x + offsetX;
			passwordErrorLabel.y = passwordInputView.y - passwordErrorLabel.height + offsetY;

			emailErrorLabel.x = emailInputView.x + offsetX;
			emailErrorLabel.y = emailInputView.y - emailErrorLabel.height + offsetY;
		}
		
		public function get shouldSubscribe() : Boolean {
			return subscribeCheck.isSelected;
		}
	}
}