package com.despark.tuning3d.view.components.brands
{
	import com.despark.tuning3d.model.brands.Brand;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.BrandListItemRenderer;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.utils.FontSelector;

	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.ITextRenderer;

	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.display.Image;
	import starling.display.Quad;

	public class BrandItemRenderer extends BrandListItemRenderer
	{
		private var background:Quad;
		private var underline:Quad;
		private var leftBorder:Quad;
		private var rightBorder:Quad;
		protected var labelSelected:MultilingualLabel;
		protected var imageLoader : ImageLoaderWithStates;
		private var customImage : Image;
		private var isLastAdded:Boolean = false;
		public function BrandItemRenderer()
		{
			super();
			overflowMultiLang = true;
		}
		
		public function get brand() : Brand 
		{
			return data as Brand;
		}

		protected override function commitData() : void {
			if (brand) {
				if (brand.name){
					var str : String = brand.name;
					if (str.indexOf("last added") != -1){
						label.text = "LAST ADDED";
					} else if (str.indexOf("all") == 0){
						label.text = "ALL";
						label.scale = 3;
					} else {
						label.text = brand.name.toUpperCase();
                    }
				} else {
					label.text = "";
				}
					
				if (brand.name && !(brand.lastAdded) && (brand.name=="Coming Soon" || brand.name=="Under Construction" || brand.name=="With \"stance\"" || brand.name=="With most parts")) {
					label.textFormat.italic = true;
					label.text = brand.name.toUpperCase();
				}
				label.height = (actualHeight - label.height) / 2;
				if (brand.lastAdded) {
					label.text = AlertView.language.translate("carchooserTop", 'last_added').toLocaleUpperCase();
					label.textFormat.italic = true;
				}
				if(imageLoader){
					imageLoader.removeFromParent(true);
					imageLoader = null;
				}
				if(customImage){
					customImage.removeFromParent(true);
					customImage = null;
				}
				labelSelected.text = label.text;
				if(brand.lastAdded){
					customImage = new Image(ButtonFactory.assetManager.getTexture("last_added_icon"));
					addChild(customImage);
					customImage.height = actualHeight / 2;
					customImage.scale = customImage.scaleY;
					customImage.x = (actualWidth - customImage.width) / 2;
					customImage.y = (actualHeight - 15 - customImage.height) / 2;
					customImage.color = isSelected? 0x252525: 0xFEFFFF;
				}else{
					imageLoader = new ImageLoaderWithStates();
					imageLoader.persistentImageProvider = brand;
					imageLoader.completeHandler = imageLoadCompleteHandler;
					imageLoader.errorHandler = imageLoadErrorHandler;
					addChild(imageLoader);
				}
			}else{
				labelSelected.text = "";
			}
		}

		private function imageLoadCompleteHandler(image:ImageLoaderWithStates):void{
			imageLoader.width = actualWidth / 1.5;
			imageLoader.height = actualHeight / 2;
			imageLoader.x = (actualWidth - imageLoader.width) / 2;
			imageLoader.y = (actualHeight - label.height - imageLoader.height) / 2;
		}

		private function imageLoadErrorHandler(image:ImageLoaderWithStates = null):void{
			if(imageLoader){
				imageLoader.removeFromParent(true);
				imageLoader = null;
			}
		}

		protected override function commitSelection() : void {
			super.commitSelection();

			background.alpha = isSelected? 1: 0;
			textColor = isSelected? 0x252525: 0xFFFFFF;
			label.textFormat.color = isSelected? 0x252525: 0xFFFFFF;

			label.visible = !isSelected;
			labelSelected.visible = isSelected;

			if(brand.lastAdded && customImage){
				customImage.color = isSelected? 0x252525: 0xFEFFFF;
			}
		}
		
		override protected function initialize() : void {

			background = new Quad(1, 1, 0xFFFFFF);
			addChild(background);
			
			underline = new Quad(1, 1, 0x4b4b4b);
			addChild(underline);

			leftBorder = new Quad(1, 1, 0x4b4b4b);
			addChild(leftBorder);

			rightBorder = new Quad(1, 1, 0x4b4b4b);
			addChild(rightBorder);
			
			super.initialize();

			label.textRendererFactory = getTextRenderer;
			label.paddingLeft = 0;
			addChild(label);

			labelSelected = new MultilingualLabel();
			labelSelected.textRendererFactory = getSelectedTextRenderer;
			labelSelected.paddingLeft = 0;
			addChild(labelSelected);
		}
		
		private function getTextRenderer():ITextRenderer {
			var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
			textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), tFontSize, 0xFFFFFF);
			textRenderer.textFormat.align = TextFormatAlign.CENTER;
			textRenderer.embedFonts = true;
			return textRenderer;
		}

		private function getSelectedTextRenderer():ITextRenderer {
			var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
			textRenderer.textFormat = new TextFormat(FontSelector.getMontserratBold(), tFontSize, 0x252525);
			textRenderer.textFormat.align = TextFormatAlign.CENTER;
			textRenderer.embedFonts = true;
			return textRenderer;
		}

		private static function get tFontSize():Number{
			return 10;
			var ft_size : Number;
			if (Constants.SCALE_FACTOR == 4) {
				ft_size = Constants.isIOS ? 13 : 13.5;
			}
			else if (Constants.SCALE_FACTOR == 2){
				ft_size = Constants.isIOS ? 14 : 15;
			}
			else {
				ft_size = 15;
			}
			ft_size *= 0.65;
			return ft_size;
		}

		protected override function layout():void {
			super.layout();

			var borderMargin:Number = 5;
			label.width = actualWidth - 2 * borderMargin;
			label.y = actualHeight - label.height - borderMargin;
			label.x = borderMargin;

			labelSelected.width = actualWidth - 2 * borderMargin;
			labelSelected.y = actualHeight - labelSelected.height - borderMargin;
			labelSelected.x = borderMargin;
			
			background.width  = actualWidth;
			background.height = actualHeight;
			
			underline.width = actualWidth;
			underline.height = 1;
			underline.x = 0;
			underline.y = actualHeight - underline.height;

			leftBorder.width = 1;
			leftBorder.height = actualHeight;
			leftBorder.x = 0;

			rightBorder.width = 1;
			rightBorder.height = actualHeight;
			rightBorder.x = actualWidth - rightBorder.width;
		}
		
		override protected function draw():void {
			overflowMultiLang = true;
			super.draw();

			textColor = isSelected? 0x252525: 0xFFFFFF;
			var newHeight:Number = 35;
			if(label.text == "ALL"){
				label.height = 16;
				labelSelected.height = 16;
				label.scale = labelSelected.scale = 1.4;
				label.y = (actualHeight - label.height) / 2 - 3;
				labelSelected.y = (actualHeight - labelSelected.height) / 2 - 3;
			}else{
				label.scale = labelSelected.scale = 1;
				label.height = 12;
				labelSelected.height = 12;
				label.y = actualHeight - label.height - 6;
				labelSelected.y = actualHeight - labelSelected.height - 6;
			}


		}

		override public function set textColor(value : uint) : void {
			super.textColor = value;
			if (labelSelected && labelSelected.textFormat) {
				labelSelected.textFormat.color = value;
				labelSelected.invalidate();
			}
		}
		
		protected override function autoSizeIfNeeded() : Boolean {
			var needsWidth  : Boolean = isNaN(explicitWidth);
			var needsHeight : Boolean = isNaN(explicitHeight);
			
			if (!needsWidth && !needsHeight){
				return false;
			}

			label.width  = NaN;
			label.height = NaN;
			label.validate();

			labelSelected.width  = NaN;
			labelSelected.height = NaN;
			labelSelected.validate();


			
			var newWidth : Number = explicitWidth;
			
			if (needsWidth){
				newWidth = Math.ceil(label.width) + 20;
				label.width = newWidth;
				labelSelected.width = newWidth;
			}
			
			var newHeight : Number = explicitHeight;
			if  (needsHeight) {
				newHeight = 35;
				if(label.text == "ALL"){
					label.height = 16;
					labelSelected.height = 16;
				}else{
					label.height = newHeight - 13;
					labelSelected.height = newHeight - 13;
				}
			}

			label.y = (actualHeight - label.height) / 2 - 3;
			labelSelected.y = (actualHeight - labelSelected.height) / 2 - 3;

			return setSizeInternal(newWidth, newHeight, false);
		}
	}
}