package com.despark.tuning3d.view.components.brands {
    import com.despark.tuning3d.model.brands.Brand;
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.packages.Packet;
    import com.despark.tuning3d.model.settings.Settings;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.cars.CarItemRenderer;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.gallery.SearchInput;
    import com.utils.ArrayUtil;
    import com.utils.DisplayObjectUtil;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.ScrollBar;
    import feathers.controls.ScrollBarDisplayMode;
    import feathers.controls.ScrollContainer;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.ToggleButton;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.ToggleGroup;
    import feathers.data.ListCollection;
    import feathers.layout.Direction;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.TiledRowsLayout;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.text.TextFormat;

    import mx.utils.StringUtil;

    import starling.core.Starling;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.events.Event;
    import starling.utils.ScaleMode;

    public class BrandScreenCarChooser extends LayoutGroup {
        private var _brandsList:ExtendedList;
        private var _carsList:CarsList;
        //
        private var _closeButton:Button;
        private var _searchButton:SearchInput;
        private var _topButtonsMenu:LayoutGroup;
        private var _typeMenu:ScrollContainer;
        private var brandListTopBorder:Quad;
        private var brandListBottomBorder:Quad;
        private var _hdOnlyToggle:ToggleButton;

        private var carsListTopBorder:Quad;

        private var bodyTypesList:Array = [];
        private var typeToggleGroup:ToggleGroup;
        public var brandsScreen:BrandsScreen;
        private var _car_filter:String = "";
        private var _brands:Array;
        private var _filteredCars:Array;
        private var _sideMargin:Number = 0;
        private var _messageLabel : Label;
        public static const BODY_TYPE_LABELS:Array = function():Array{
            var result:Array = [
                {name:'TRUCK', value:'TRUCK'},
                {name:'MUSCLE', value:'MUSCLE'},
                {name:"'JDM'", value:'JDM'},
                {name:"'EURO'", value:'EURO'},
                {name:"'USDM'", value:'USDM'},
                {name:"'S. SPORT'", value:'S. SPORT'},
                {name:'SUV', value:'SUV'},
                {name:"SEMI's", value:"SEMI"},
                {name:'BIKE', value:'BIKE'}
            ];
            for(var i:int = 0; i < result.length; i++){
                result[i].name = result[i].name.toUpperCase();
                result[i].value = result[i].value.toUpperCase();
            }
            return result;
        }();

        public static function getLabel(str:String):String{
            for(var i:int = 0; i < BODY_TYPE_LABELS.length; i++){
                if(BODY_TYPE_LABELS[i].value == str){
                    return BODY_TYPE_LABELS[i].name;
                }
            }
            return "";
        }

        public function BrandScreenCarChooser() {
            super();
        }

        override protected function initialize():void{
            super .initialize();
            backgroundSkin = new Quad(1, 1, 0x252525);
            _sideMargin = Device.isIphoneXVariant()?24:0;

            width = stage.stageWidth;
            height = stage.stageHeight;
            typeToggleGroup = new ToggleGroup();

            _topButtonsMenu = new LayoutGroup();
            var topMenuLayoutData:HorizontalLayout = new HorizontalLayout();
            topMenuLayoutData.verticalAlign = VerticalAlign.MIDDLE;
            topMenuLayoutData.gap = 12.5;
            _topButtonsMenu.layout = topMenuLayoutData;
            _topButtonsMenu.height = BannerStackedScreen.BANNER_SIZE;


            _typeMenu = new ScrollContainer();
            var typeMenuLayoutData:HorizontalLayout = new HorizontalLayout();
            typeMenuLayoutData.verticalAlign = VerticalAlign.MIDDLE;
            typeMenuLayoutData.horizontalAlign = HorizontalAlign.LEFT;
            typeMenuLayoutData.gap = 0;
            _typeMenu.layout = typeMenuLayoutData;
            _typeMenu.height = BannerStackedScreen.BANNER_SIZE;
            _typeMenu.horizontalScrollPolicy = ScrollPolicy.AUTO;
            _typeMenu.scrollBarDisplayMode = ScrollBarDisplayMode.FIXED;
            addChild(_typeMenu);

            _hdOnlyToggle = ButtonFactory.simpleToggleButton("", 0xFEFFFF, "timeline_filter_default", "timeline_filter_selected");
            _topButtonsMenu.addChild(_hdOnlyToggle);
            var textFormat:TextFormat = new TextFormat(FontSelector.getMontserratBold(), 10, 0xFFFFFF);
            textFormat.leading = -1;
            _hdOnlyToggle.labelFactory = LabelFactory.createTextRenderer(textFormat);
            _hdOnlyToggle.label = "<font size='12'><b>HD</b></font><br><font size='6'>ONLY</font>";
            _hdOnlyToggle.labelOffsetX = 4;
            _hdOnlyToggle.labelOffsetY = -2;
            _hdOnlyToggle.verticalAlign = VerticalAlign.MIDDLE;
            _hdOnlyToggle.paddingRight = 10;
            _hdOnlyToggle.isSelected = Settings.hdOnly;
            _hdOnlyToggle.addEventListener(Event.CHANGE, hdOnlyChangeHandler);

            _searchButton = new SearchInput(false, 26, "a_search_icon", ScaleMode.NONE, 0x3d3d3d, true);
            _topButtonsMenu.addChild(_searchButton);
            _searchButton.addEventListener(Event.CHANGE, searchChangeHandler);

            _closeButton = ButtonFactory.circleDefaultButton2("a_close_icon", 0.6, 0xFFFFFE, 0, 0x3d3d3d);
            _closeButton.addEventListener(Event.TRIGGERED, closeButtonTriggeredHandler);
            _topButtonsMenu.addChild(_closeButton);


            brandListTopBorder = new Quad(1, 1, 0x4b4b4b);
            addChild(brandListTopBorder);
            brandListBottomBorder = new Quad(1, 1, 0x4b4b4b);
            addChild(brandListBottomBorder);

            var brandsLayout : VerticalLayout = new VerticalLayout();
            brandsLayout.scrollPositionVerticalAlign = VerticalAlign.TOP;
            brandsLayout.useVirtualLayout = true;
            brandsLayout.gap = 0;
            _brandsList = new ExtendedList();
            _brandsList.scrollBarDisplayMode = ScrollBarDisplayMode.FLOAT;
            _brandsList.clipContent = true;
            _brandsList.addEventListener(Event.CHANGE, brandsListChangeHandler);
            _brandsList.layout = brandsLayout;
            _brandsList.itemRendererFactory = brandsListItemRendererFactory;
            addChild(_brandsList);

            var carsLayout : TiledRowsLayout = new TiledRowsLayout();
            carsLayout.requestedColumnCount = 3;
            carsLayout.useSquareTiles = false;
            carsLayout.verticalAlign = VerticalAlign.TOP;
            carsLayout.horizontalAlign = HorizontalAlign.LEFT;
            carsLayout.useVirtualLayout = true;
            carsLayout.gap = 0;
            _carsList = new CarsList();
            _carsList.name = "carsList";
            _carsList.clipContent = true;
            _carsList.layout = carsLayout;
            _carsList.verticalScrollPolicy = ScrollPolicy.AUTO;

            _carsList.addEventListener(Event.CHANGE, carsListChangeHandler);
            _carsList.scrollBarDisplayMode = ScrollBarDisplayMode.FIXED;
            _carsList.verticalScrollBarFactory = verticalScrollBarFactory;
            _carsList.itemRendererFactory = carsListItemRendererFactory;
            addChild(_carsList);

            carsListTopBorder = new Quad(1, 1, 0x4b4b4b);
            addChild(carsListTopBorder);
            addChild(_topButtonsMenu);

            _messageLabel = new Label();
            _messageLabel.includeInLayout = false;
            _messageLabel.visible = false;
            _messageLabel.height = 14;
            _messageLabel.textRendererFactory = LabelFactory.profileButtonLabel;
            _messageLabel.text = "No search results found";
            _messageLabel.width = actualWidth;
            addChild(_messageLabel);

        }

        private function hdOnlyChangeHandler(event:Event):void {
            typeGroupChangeHandler(null);
            brandsListChangeHandler(null);
            Settings.hdOnly = _hdOnlyToggle.isSelected;
        }

        private function searchChangeHandler(event:Event):void {
            if(_brandsList){
                _brandsList.removeEventListener(Event.CHANGE, brandsListChangeHandler);
                _brandsList.selectedIndex = 0;
            }

            if(typeToggleGroup && typeToggleGroup.numItems && typeToggleGroup.selectedIndex != 0){
                typeToggleGroup.removeEventListener(Event.CHANGE, typeGroupChangeHandler);
                typeToggleGroup.selectedIndex = 0;
                typeToggleGroup.addEventListener(Event.CHANGE, typeGroupChangeHandler);
            }
            if(_searchButton && _searchButton.text){
                _car_filter = StringUtil.trim(_searchButton.text).toLowerCase();
            }else{
                _car_filter = "";
            }
            brandsListChangeHandler(null);
            var brandsArr:Array;
            brandsArr = _brands?_brands.concat():[];
            var ln:int = brandsArr.length - 1;
            if(_filteredCars){
                for(var i:int = ln; i >= 0; i--){
                    for(var j:int = 0; j < _filteredCars.length; j++){
                        if(Brand(brandsArr[i]).cars.indexOf(_filteredCars[j]) != -1){
                            break;
                        }
                    }
                    if(j == _filteredCars.length){
                        brandsArr.removeAt(i);
                    }
                }
            }

            if(_brandsList){
                if(brandsArr){
                    _brandsList.dataProvider = new ListCollection(brandsArr);
                }
                _brandsList.addEventListener(Event.CHANGE, brandsListChangeHandler);
                _brandsList.selectedIndex = 0;
            }
        }

        private function brandsListChangeHandler(event:Event) : void {
            if(!_brandsList || !_brands || !_carsList){
                return;
            }
            var selectedCarBrand : Brand = _brandsList.selectedItem as Brand;
            if (selectedCarBrand == null) {
                selectedCarBrand = _brands[0] as Brand;
            }
            var showBrand : Boolean = selectedCarBrand && (selectedCarBrand.lastAdded || selectedCarBrand.limitedOption || selectedCarBrand.name == "ALL");
            var updateFilteredBrands:Boolean = (_filteredCars && _filteredCars.length);
            _filteredCars = filteredCarsList;
            var selectedCars:Array = [];
            for(var i:int = 0; i < _filteredCars.length; i++){
                if(Car(_filteredCars[i]).limitedOptions == selectedCarBrand.limitedOption){
                    selectedCars.push(_filteredCars[i]);
                }
                if(!Car(_filteredCars[i]).packet){
                    trace(Car(_filteredCars[i]).id, Car(_filteredCars[i]).displayName, Car(_filteredCars[i]).year, Car(_filteredCars[i]).carType);
                }
            }
            var collection : ListCollection = new ListCollection(selectedCars);
            _carsList.showBrandName = showBrand;
            _carsList.dataProvider = collection;
            _carsList.data = {
                showBrand : showBrand
            }

            _messageLabel.visible = (collection.length == 0);
            brandListBottomBorder.visible = !_messageLabel.visible;

            if(_brandsList.visible && selectedCars && selectedCars.length && !updateFilteredBrands){
                SoundManager.playSound(SoundManager.SELECT_PART);
            }
        }

        public function selectCarType(value:String, disableOthers:Boolean = false):void{
            if(typeToggleGroup){
                for(var i:int = 0; i < typeToggleGroup.numItems; i++){
                    var item:ToggleButton = typeToggleGroup.getItemAt(i) as ToggleButton;
                    if(disableOthers){
                        if(item.name == value){
                            typeToggleGroup.selectedItem = item;
                        }else{
                            item.alpha = 0.5;
                            item.isEnabled = item.touchable = false;
                        }
                    }else{
                        if(item.name == value){
                            typeToggleGroup.selectedItem = item;
                            break;
                        }
                    }
                }
            }
        }


        private function carsListChangeHandler(event : Event) : void {
            if (AppMode.self.mode == AppMode.MODE_TUNING) {
                close();
                if (_carsList.selectedItem) {
                    SoundManager.playSound(SoundManager.SELECT_PART);
                    brandsScreen.dispatchEventWith(BrandsScreen.CAR_SELECTED, false, _carsList.selectedItem);
                    if (_carsList) {
                        _carsList.selectedIndex = -1;
                    }
                }
            }
        }


        private function  carsListItemRendererFactory() : IListItemRenderer {
            var renderer : CarItemRenderer = new CarItemRenderer();
            renderer.setSize((_carsList.width - _carsList.paddingRight - _carsList.paddingLeft - 17) / 3, (_carsList.height - 1) / 3);
            renderer._carOffset = 7;
            return renderer;
        }


        private function verticalScrollBarFactory() : ScrollBar {
            var scrollbar:ScrollBar = new ScrollBar();
            scrollbar.direction = Direction.VERTICAL;
            scrollbar.thumbFactory = function() : Button {
                var button : Button = new Button();
                var sprite : Sprite = new Sprite();
                var back : Quad = new Quad(8, 25, 0xffffff);
                back.alpha = 0;
                var quad : Quad = new Quad(4, 25, 0xC1C3C2);
                quad.x = -2;
                sprite.addChild(back);
                sprite.addChild(quad);
                button.defaultSkin = sprite;
                return button;
            };
            return scrollbar;
        }

        public function set brands(value:Array):void{
            if(!value){
                return;
            }
            _brands = value;
            var car:Car;
            var i:int;
            typeToggleGroup.removeEventListener(Event.CHANGE, typeGroupChangeHandler);
            typeToggleGroup.removeAllItems();
            bodyTypesList = [];

            if(_brands[0].name != "ALL"){
                var allBrand:Brand = new Brand();
                allBrand.name = "ALL";
                allBrand.imageURL = "";
                allBrand.cars = [];
                _brands.unshift(allBrand);
            }

            _brandsList.dataProvider = new ListCollection(_brands);
            for(i = 0; i < _brands.length; i++){
                var brand:Brand = _brands[i] as Brand;
                if(brand.cars){
                    for(var j:int = 0; j < brand.cars.length; j++){
                        if(brand.limitedOption){
                            continue;
                        }
                        car = brand.cars[j];
                        if(!car.bodyTypeList || !car.bodyTypeList.length){
                            continue;
                        }
                        if(allBrand){
                            allBrand.addCar(car);
                        }
                    }
                }
            }
            DisplayObjectUtil.removeChildren(_typeMenu);
            bodyTypesList = BODY_TYPE_LABELS;
            typeToggleGroup.addItem(ButtonFactory.carChooserTopButton("ALL", "all"));
            _typeMenu.addChild(typeToggleGroup.getItemAt(typeToggleGroup.numItems - 1) as ToggleButton);
            if(_brandsList.dataProvider && bodyTypesList){
                for(i = 0; i < bodyTypesList.length; i++){
                    typeToggleGroup.addItem(ButtonFactory.carChooserTopButton(bodyTypesList[i].name, bodyTypesList[i].value));
                    _typeMenu.addChild(typeToggleGroup.getItemAt(typeToggleGroup.numItems - 1) as ToggleButton);
                }
                for(i = 0; i < _brandsList.dataProvider.length; i++){
                    if(Brand(_brandsList.dataProvider.getItemAt(i)).lastAdded){
                        _brandsList.selectedIndex = i;
                        break;
                    }
                }
                typeToggleGroup.addEventListener(Event.CHANGE, typeGroupChangeHandler);
            }
        }

        private function get filteredCarsList():Array{
            var bodyTypeBtn:ToggleButton;
            var bodyType:String;
            var hdOnly:Boolean = _hdOnlyToggle.isSelected;
            if(typeToggleGroup && typeToggleGroup.numItems){
                if(typeToggleGroup.selectedItem){
                    bodyTypeBtn = typeToggleGroup.selectedItem as ToggleButton;
                }else{
                    bodyTypeBtn = typeToggleGroup.getItemAt(0) as ToggleButton;
                }
                bodyType = bodyTypeBtn.name;
            }else{
                bodyType = "all";
            }

            var selectedBrand:Brand = (_brandsList.selectedItem || _brands[0]) as Brand;
            var result:Array = [];
            var resultOrder:Array;
            var allCars:Array = Brand(_brands[0]).cars.concat();
            var i:int;
            var k:int;
            if(Brand(_brands[_brands.length - 1]).limitedOption){
                allCars = allCars.concat(Brand(_brands[_brands.length - 1]).cars);
            }

            var searchRegexps:Array = [];
            if(_car_filter && _car_filter.length) {
                var filter1:String = _car_filter.replace("-", " ");
                var filter2:String = _car_filter.replace("-", "");
                var searchTerms:Array = ArrayUtil.uniqueConcat(filter1.split(" "), filter2.split(" "));
                for(k = searchTerms.length - 1; k >= 0; k--){
                    if(!searchTerms[k] || !searchTerms[k].length){
                        searchTerms.removeAt(k);
                        continue;
                    }
                    searchRegexps.unshift(new RegExp('(^|\\s)' + searchTerms[k].toLowerCase() + '.*', 'gmi'));
                }
            }

            for (i = 0; i < allCars.length; i++) {
                var car:Car = allCars[i] as Car;
                if (bodyType != "all" && !(car.bodyTypeList && car.bodyTypeList.indexOf(bodyType) != -1)) {
                    continue;
                }

                if (hdOnly && Car.HD_TYPES.indexOf(car.carType) == -1) {
                    continue;
                }

                if (!car.id) {
                    continue;
                }

                if(searchTerms && searchTerms.length) {
                    var matchCount:int = 0;
                    for (k = 0; k < searchTerms.length; k++) {
                        var srcFound:Boolean = false;
                        var searchIndex:int = -1;
                        var priority:int = -1;
                        var searchText:String = searchTerms[k];
                        var searchReg:RegExp = searchRegexps[k];
                        var carName:String = car.name;
                        var carModel:String = car.model;


                        var carNameStr:String;
                        if (carName) {
                            carNameStr = carName.replace('-', ' ').toLowerCase();
                            searchIndex = carNameStr.search(searchReg);

                            srcFound = (searchIndex != -1);
                            if (srcFound) {
                                priority = 0;
                            } else {
                                carNameStr = carName.replace('-', '').toLowerCase();
                                searchIndex = carNameStr.search(searchReg);
                                srcFound = (searchIndex != -1);
                            }
                        }


                        var carModelStr:String;
                        if (!srcFound && carModel) {
                            carModelStr = carModel.replace('-', ' ').toLowerCase();

                            searchIndex = carModelStr.search(searchReg);

                            srcFound = (searchIndex != -1);
                            if (srcFound) {
                                priority = 0;
                            } else {
                                carModelStr = carModel.replace('-', '').toLowerCase();
                                searchIndex = carModelStr.search(searchReg);
                                srcFound = (searchIndex != -1);
                            }
                        }


                        if (!srcFound) {
                            var carYear:int = parseInt(searchText);
                            if (carYear == car.year) {
                                searchIndex = 0;
                                priority = 2;
                            } else if (car.startYear && car.endYear && carYear >= car.startYear && carYear <= car.endYear) {
                                searchIndex = carYear - car.startYear;
                                priority = 3;
                            } else if (searchText.length) {
                                searchIndex = car.year.toString().indexOf(searchText);
                                if (searchIndex == -1 && car.startYear) {
                                    searchIndex = car.startYear.toString().indexOf(searchText);
                                }
                                if (searchIndex == -1 && car.endYear) {
                                    searchIndex = car.endYear.toString().indexOf(searchText);
                                }
                                priority = 4;
                            } else {
                                searchIndex = -1;
                            }
                            srcFound = (searchIndex != -1);
                        }
                        if (!srcFound) {
                            break;
                        }
                        matchCount++;
                    }
                }
                if(resultOrder == null){
                    resultOrder = [];
                }
                if (searchTerms && searchTerms.length) {
                    if(matchCount == searchTerms.length){
                        resultOrder.push({priority: priority, index: searchIndex, year: car.year, car: car});
                    }
                }else if(result.indexOf(car) == -1 && selectedBrand.cars.indexOf(car) != -1){
                    result.push(car);
                    if (!car.year && car.startYear) {
                        car.year = car.startYear;
                    }else if(car.year && !car.startYear){
                        car.startYear = car.year;
                    }
                }
            }

            if(resultOrder && resultOrder.length){
                resultOrder.sortOn(["priority", "index", "endYear", "startYear", "year"], [Array.NUMERIC, Array.NUMERIC, Array.NUMERIC|Array.DESCENDING, Array.NUMERIC|Array.DESCENDING, Array.NUMERIC|Array.DESCENDING]);
                result = [];
                for(var j:int = 0; j < resultOrder.length; j++){
                    if(result.indexOf(resultOrder[j].car) == -1){
                        if(selectedBrand.limitedOption){
                            result.push(resultOrder[j].car);
                        }else if(selectedBrand.cars.indexOf(resultOrder[j].car) != -1 || Car(resultOrder[j].car).limitedOptions){
                            result.push(resultOrder[j].car);
                        }
                    }
                }
            }else if(!selectedBrand.lastAdded){
                result.sortOn(["endYear", "startYear", "year"], Array.NUMERIC | Array.DESCENDING);
            }else if(selectedBrand.lastAdded){
                result.sortOn(["updated"], Array.NUMERIC | Array.DESCENDING);
            }
            return result;
        }

        private function typeGroupChangeHandler(event:Event):void {
            if(!_brandsList || !_carsList || !typeToggleGroup || !_brands){
                return;
            }
            var bodyTypeBtn:ToggleButton;
            var bodyType:String;
            var i:int;
            var j:int;
            if(typeToggleGroup && typeToggleGroup.numItems){
                if(typeToggleGroup.selectedItem){
                    bodyTypeBtn = typeToggleGroup.selectedItem as ToggleButton;
                }else{
                    bodyTypeBtn = typeToggleGroup.getItemAt(0) as ToggleButton;
                }
                bodyType = bodyTypeBtn.name;
            }else{
                bodyType = "all";
            }
            var selectedBrand:Brand = _brandsList.selectedItem as Brand;
            var brandsArr:Array;

            brandsArr = getFilteredBrands(bodyType);
            _brandsList.removeEventListener(Event.CHANGE, brandsListChangeHandler);
            _brandsList.dataProvider = new ListCollection(brandsArr);
            if(_brandsList.dataProvider.contains(selectedBrand)){
                _brandsList.selectedItem = selectedBrand;
            }else{
                _brandsList.selectedIndex = 0;
            }
            _brandsList.addEventListener(Event.CHANGE, brandsListChangeHandler);
            brandsListChangeHandler(null);
            if(_car_filter && _car_filter.length && _filteredCars){
                _brandsList.removeEventListener(Event.CHANGE, brandsListChangeHandler);
                brandsArr = [];
                for(j = 0; j < _brands.length; j++){
                    for(i = 0; i < _filteredCars.length; i++){
                        if(Brand(_brands[j]).cars.indexOf(_filteredCars[i]) != -1){
                            brandsArr.push(_brands[j]);
                            break;
                        }
                    }
                }
                _brandsList.dataProvider = new ListCollection(brandsArr);
                if(_brandsList.dataProvider.contains(selectedBrand)){
                    _brandsList.selectedItem = selectedBrand;
                }else{
                    _brandsList.selectedIndex = 0;
                }
                selectedBrand = _brandsList.selectedItem as Brand;
                var fCars:Array = [];
                if(_filteredCars){
                    for(i = 0; i < _filteredCars.length; i++){
                        if(selectedBrand.cars.indexOf(_filteredCars[i]) != -1){
                            fCars.push(_filteredCars[i]);
                        }
                    }
                }
                _brandsList.addEventListener(Event.CHANGE, brandsListChangeHandler);
                var showBrand : Boolean = selectedBrand && (selectedBrand.lastAdded || selectedBrand.limitedOption || selectedBrand.name == "ALL");
                var collection : ListCollection = new ListCollection(fCars);
                _carsList.showBrandName = showBrand;
                _carsList.dataProvider = collection;
                _carsList.data = {
                    showBrand : showBrand
                }
            }
        }

        private function getFilteredBrands(bodyType:String):Array{
            var brandsArr:Array;
            brandsArr = _brands.concat();
            var hdOnly:Boolean = _hdOnlyToggle.isSelected;
            for(var i:int = brandsArr.length - 1; i > 0; i--){
                var brand:Brand = brandsArr[i] as Brand;
                for(var j:int = 0; j < brand.cars.length; j++){
                    var car:Car = brand.cars[j];
                    var hdCheck:Boolean = (!hdOnly || Car.HD_TYPES.indexOf(car.carType) != -1);
                    if(hdCheck && car.bodyTypeList && (car.bodyTypeList.indexOf(bodyType) != -1 || bodyType == "all")){
                        break;
                    }
                }
                if(j == brand.cars.length){
                    brandsArr.removeAt(i);
                }
            }
            return brandsArr;
        }

        private function closeButtonTriggeredHandler(event:Event):void {
            close();
        }

        public function open():void{
            if(_searchButton){
                _searchButton.removeEventListener(Event.CHANGE, searchChangeHandler);
                _searchButton.text = "";
                _car_filter = "";
                _searchButton.addEventListener(Event.CHANGE, searchChangeHandler);
            }

            if(typeToggleGroup && typeToggleGroup.numItems){
                typeToggleGroup.removeEventListener(Event.CHANGE, typeGroupChangeHandler);
                typeToggleGroup.selectedIndex = 0;
                typeToggleGroup.addEventListener(Event.CHANGE, typeGroupChangeHandler);
            }

            if(_brandsList && _brandsList.dataProvider && _brandsList.dataProvider.length){
                _brandsList.removeEventListener(Event.CHANGE, brandsListChangeHandler);
                _brandsList.addEventListener(Event.CHANGE, brandsListChangeHandler);
                brandsListChangeHandler(null);
            }

            if(_carsList && _carsList.dataProvider){
                _carsList.validate();
            }

            dispatchEventWith(Event.OPEN);
            if(Starling.juggler.containsTweens(this)){
                Starling.juggler.removeTweens(this);
            }
            alpha = 0;
            visible = true;
            Starling.juggler.tween(this, 0.15, {alpha: 1});
        }

        public function close():void{
            if(Starling.juggler.containsTweens(this)){
                Starling.juggler.removeTweens(this);
            }
            Starling.juggler.tween(this, 0.15, {alpha: 0, onComplete: function():void{
                    if(_searchButton){
                        _searchButton.text = "";
                        _searchButton.close();
                    }
                    //
                    for(var i:int = 0; i < typeToggleGroup.numItems; i++){
                        var item:ToggleButton = typeToggleGroup.getItemAt(i) as ToggleButton;
                        item.alpha = 1;
                        item.isEnabled = item.touchable = true;
                    }
                    //
                    visible = false;
                    dispatchEventWith(Event.CLOSE);
                }});
        }

        private function brandsListItemRendererFactory() : IListItemRenderer {
            var renderer : BrandItemRenderer = new BrandItemRenderer();
            renderer.setSize(_brandsList.width, 66);
            return renderer;
        }

        override public function validate():void {
            super .validate();
            if(_carsList && _carsList.dataProvider){
                _carsList.dataProvider.updateAll();
                _carsList.validate();
            }
            if(_brandsList && _brandsList.dataProvider){
                _brandsList.dataProvider.updateAll();
                _brandsList.validate();
            }
        }

        override protected function draw():void{
            super .draw();
            var margin:Number = 5;
            _brandsList.width = 105;
            _brandsList.height = actualHeight - BannerStackedScreen.BANNER_SIZE - 2 * margin;
            _brandsList.x = margin + _sideMargin;
            _brandsList.y = BannerStackedScreen.BANNER_SIZE;

            brandListTopBorder.width = _brandsList.width;
            brandListTopBorder.x = _brandsList.x;
            brandListTopBorder.y = _brandsList.y;

            brandListBottomBorder.width = _brandsList.width;
            brandListBottomBorder.x = _brandsList.x;
            brandListBottomBorder.y = _brandsList.y + _brandsList.height - brandListBottomBorder.height;

            _carsList.width = actualWidth - _brandsList.width - margin - 2 * _sideMargin;
            _carsList.height = _brandsList.height;
            _carsList.x = _brandsList.x + _brandsList.width + margin;
            _carsList.y = BannerStackedScreen.BANNER_SIZE;

            carsListTopBorder.width = _carsList.width - 17;
            carsListTopBorder.x = _carsList.x;
            carsListTopBorder.y = _carsList.y;

            _topButtonsMenu.validate();
            _typeMenu.width = actualWidth - _topButtonsMenu.width - 6 * margin - 2 * _sideMargin;
            _typeMenu.x = 2 * margin + _sideMargin;
            _topButtonsMenu.x = _typeMenu.x + _typeMenu.width + 2 * margin;

            _messageLabel.y = (actualHeight + _topButtonsMenu.height - _messageLabel.height) / 2;
        }

        public function get brands():Array {
            return _brands;
        }

        public function updateTempOpenPacket(packet:Packet):void {
            if(brands){
                var allCars:Array = Brand(brands[0]).cars.concat();
                if(Brand(_brands[_brands.length - 1]).limitedOption){
                    allCars = allCars.concat(Brand(_brands[_brands.length - 1]).cars);
                }
                allCars = ArrayUtil.onlyUnique(allCars);
                for(var i:int = 0; i < allCars.length; i++){
                    var car:Car = allCars[i] as Car;
                    if(car){
                        if(car.packet && car.packet.productId == packet.productId){
                            car.packet.tempOpened = packet.tempOpened;
                            car.packet.oneTimeUnlocked = packet.oneTimeUnlocked;
                            car.packet.lock = packet.lock;
                        }
                        if(car.packets){
                            for(var j:int = 0; j < car.packets.length; j++){
                                var pk:Packet = car.packets[j] as Packet;
                                if(pk.productId == packet.productId){
                                    pk.tempOpened = packet.tempOpened;
                                    pk.oneTimeUnlocked = packet.oneTimeUnlocked;
                                    pk.lock = packet.lock;
                                }
                            }
                        }
                    }
                }

                //
            }
        }




    }
}
