package com.despark.tuning3d.view.components.brands
{
import com.despark.tuning3d.model.cars.Car;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.despark.tuning3d.view.components.core.PacketView;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.text.TextFieldTextRenderer;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
import starling.events.TouchEvent;
import starling.textures.Texture;

public class BuyPacketItemRenderer extends ListItemRenderer
	{
		public static const TOUCH_PACKET_INFO:String="touchPacketInfo";		
		
		public static const ITEM_TOUCH_PACKET:String="itemTouchPacket";		
		
		
		public static const BUY_NOW_PACKET:String="BUY_NOW_PACKET";
		
		public static const VIEW_CAR_360:String="VIEW_CAR_360";
		
		
		private var _line:Quad=null;
		
		
		private var lockImage:Image;
		private var unlockImage:Image;		
		private var freeImage:Image;	
		
		private var carImage:ImageLoaderWithStates;
		
		private var titleDisplay:TextFieldTextRenderer;
		
		private var textLabel:TextFieldTextRenderer;  
		
		private var packNameLabel:TextFieldTextRenderer
		
		
		private var _buyNowButton:Button=null;
		
		private var featured360:Button;
		
		private var _textFontSize:int=10;
		
		private var _titleFontSize:int=12;
		
		
		public function BuyPacketItemRenderer()
		{
			super();
			
			if(Constants.SCALE_FACTOR==4){
				_textFontSize=8;
				_titleFontSize=10;
			}
			
			fontSize = 12;
			textColor = 0x454545;
		}
		
//		private var _factoryID:String;
//		private function get factoryID():String{
//			return _factoryID;
//		}
//
//
//		private function set factoryID(value:String):void{
//			_factoryID = value;
//		}
		
		protected override function  initialize():void{
			
			carImage=new ImageLoaderWithStates();
			carImage.overrideDispose = true;  
			carImage.snapToPixels=false;   
			addChild(carImage);
			
			titleDisplay=new TextFieldTextRenderer();
			titleDisplay.textFormat = new TextFormat(FontSelector.getFont(true), _titleFontSize, 0x454545);   
			titleDisplay.textFormat.align = TextFormatAlign.LEFT;
			titleDisplay.textFormat.bold=true;
			titleDisplay.touchable=false;     
			titleDisplay.embedFonts=FontSelector.embedFonts();
			titleDisplay.isHTML=true;
			titleDisplay.wordWrap=true;
			addChild(titleDisplay);    
			
			var featured360Texture:Texture=ButtonFactory.assetManager.getTexture("featured_360");
			featured360=new Button();
			featured360.addEventListener(Event.TRIGGERED, buttonFeatured360Handler);
			featured360.defaultSkin=new Image(featured360Texture);
			featured360.visible=false;
			featured360.width=featured360.defaultSkin.width;
			featured360.height=featured360.defaultSkin.height;  
			addChild(featured360);  
			
			textLabel=new TextFieldTextRenderer();
			textLabel.textFormat = new TextFormat(FontSelector.getFont(), _textFontSize, 0x454545);   
			textLabel.textFormat.align = TextFormatAlign.LEFT;
			textLabel.wordWrap=true;
			textLabel.isHTML=true;
			textLabel.embedFonts=FontSelector.embedFonts();
			textLabel.touchable=false;     
			addChild(textLabel);    
		}
		
		override public function set textColor(value : uint) : void
		{
			_textColor = value;
		}
		
		override public function set bold(value : Boolean) : void
		{
			_bold = value;
		}
		
		override public function set isSelected(value : Boolean) : void
		{
			  
		}
		
		override protected function touchHandler(event :TouchEvent):void
		{
			 
		}
		
		private function buttonFeatured360Handler(event:Event):void{
			owner.dispatchEventWith(VIEW_CAR_360, false, {car:car});    
		}
		
		private function buyNowButtonnTriggeredHandler(event : Event) : void{	
			owner.dispatchEventWith(BUY_NOW_PACKET, false, {packet:car.packet});
		}
		
		public function get car() :Car
		{
			return data as Car;
		}
		
		protected override function commitData() : void    
		{
			
			super.commitData();  
			
			if(!car)
				return;
			
			if(AlertView.language)   
			{	
				var url:String=AlertView.language.translate(car.id, "purchase_car_image").replace("data", "cdn");
				
				var newCar:Car;
				
				if(carImage.persistentImageProvider && carImage.persistentImageProvider.remoteUrl!=url){
					newCar=new Car();
					newCar.imageURL=url;  
					carImage.persistentImageProvider = newCar;  
				}else if(!carImage.persistentImageProvider){
					
					newCar=new Car();
					newCar.imageURL=url;   
					carImage.persistentImageProvider = newCar;  
				}
				
				//car.remoteUrl=imageURL;     
			}	
			else{
				
				carImage.persistentImageProvider=car;    
				
			}    
			
			titleDisplay.width=PacketView.PACKET_VIEW_WIDTH-(featured360.width+5);
			titleDisplay.text=car.name+" "+car.model+" "+car.year;
		
			textLabel.removeFromParent(true);
			textLabel=null;
			
			textLabel=new TextFieldTextRenderer();
			textLabel.width=PacketView.PACKET_VIEW_WIDTH;
			textLabel.textFormat = new TextFormat(FontSelector.getFont(), _textFontSize, 0x454545);   
			textLabel.textFormat.align = TextFormatAlign.LEFT;
			textLabel.wordWrap=true;  
			textLabel.isHTML=true;
			//textLabel.snapToPixels=false;  
			textLabel.embedFonts=FontSelector.embedFonts();	
			addChild(textLabel);      
			
//			if(car.body && car.body!=""){
//				textLabel.text = '<textformat rightmargin="3" >'+car.body+'</textformat>';
//			}else{
				
				if(AlertView.language ){  
				//if(PackagesProxy.productsTrans && PackagesProxy.productsTrans[car.id] && PackagesProxy.productsTrans[car.id].hasOwnProperty("purchase_car_body")){
					textLabel.text = '<textformat rightmargin="3" >'+AlertView.language.translate(car.id, "purchase_car_body")+'</textformat>';
					
				}else if(car.packet){
				
				
					textLabel.text = '<textformat rightmargin="3" >'+car.packet.body+'</textformat>';    
				}  
//			}
			this.invalidate("change_text");
			
			if(car && car.packet){
				
			}

			if(car && car.packet &&  car.packet.show){
				
				featured360.visible=true;
			}else{
				
				featured360.visible=false;
			}
		}
		
		protected override function commitSelection() :void
		{
		}
		
		protected override function autoSizeIfNeeded():Boolean{
		
			var needsWidth : Boolean = false;//isNaN(explicitWidth);
			var needsHeight : Boolean = isNaN(explicitHeight);
			
			//explicitWidth=PacketView.PACKET_VIEW_WIDTH;
			if(!needsWidth && !needsHeight)   
			{
				return false;
			}
			
			titleDisplay.width = PacketView.PACKET_VIEW_WIDTH-(featured360.width+5);;
			titleDisplay.height = NaN;
			titleDisplay.validate();
			
			var newHeight : Number = explicitHeight;
			
			if(needsHeight)  
			{	
				newHeight=titleDisplay.height+carImage.height+40;  
			}
			
			return setSizeInternal(PacketView.PACKET_VIEW_WIDTH, newHeight, false);
		}
		
		protected override function layout() : void
		{
			
			titleDisplay.x =0;
			titleDisplay.y=0;
			titleDisplay.width = PacketView.PACKET_VIEW_WIDTH-(featured360.width+5);;
				
				
			featured360.x=PacketView.PACKET_VIEW_WIDTH-(featured360.width);     
			
			featured360.y=0;//label.y-featured360.height;  
			//titleDisplay.height = 15;
		
		
			
		//	label.textRendererProperties.textFormat.horizontalAlign = TextFormatAlign.LEFT;
			
		    
			
			carImage.y=20;
			carImage.x=-22;
			
			if(Constants.SCALE_FACTOR==4){
				carImage.width=actualWidth+162;
				carImage.x=((PacketView.PACKET_VIEW_WIDTH-carImage.width)/2)+10;    
				carImage.height=170;
			}else{
				carImage.width=actualWidth+132;
				carImage.x=(PacketView.PACKET_VIEW_WIDTH-carImage.width)/2;  
				carImage.height=180;
			}
			
		
			
				
			
			   
			
			
			
	
			  
			
			textLabel.x=0;
			textLabel.y=carImage.y+carImage.height+10;  
			textLabel.width=PacketView.PACKET_VIEW_WIDTH;
			
			textLabel.height=NaN;     
			  
			textLabel.validate();
			
			setSizeInternal(PacketView.PACKET_VIEW_WIDTH, textLabel.height+carImage.height+40, false);
			
			
		}
		
		override public function dispose():void{
			
			
			super.dispose();
			
		}
		
		protected override function  draw():void{
			
			super.draw();  
			
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
			
				titleDisplay.x=0;
				titleDisplay.y=0;
				titleDisplay.width = PacketView.PACKET_VIEW_WIDTH-(featured360.width+5);;   
				
				featured360.x=PacketView.PACKET_VIEW_WIDTH-(featured360.width);     
				
				featured360.y=0;//label.y-featured360.height;    
				
				
				if(AlertView.language)            
				{	
					
				
					var url:String=AlertView.language.translate(car.id, "purchase_car_image").replace("data", "cdn");
					
					var newCar:Car;
					
					if(carImage.persistentImageProvider && carImage.persistentImageProvider.remoteUrl!=url){
						newCar=new Car();
						newCar.imageURL=url;  
						carImage.persistentImageProvider = newCar;  
					}else if(!carImage.persistentImageProvider){
						
						newCar=new Car();
						newCar.imageURL=url;  
						carImage.persistentImageProvider = newCar;  
					}
				}	
				else{
					carImage.persistentImageProvider=car;      
				}
				
				carImage.y=20;
				
				if(Constants.SCALE_FACTOR==4){
					carImage.width=actualWidth+162;
					carImage.x=((PacketView.PACKET_VIEW_WIDTH-carImage.width)/2)+10;  
					carImage.height=170;       
				}else{
					carImage.width=actualWidth+132;
					carImage.x=(PacketView.PACKET_VIEW_WIDTH-carImage.width)/2;        
					carImage.height=180;     
				}
				  
				
			//	carImage.width=PacketView.PACKET_VIEW_WIDTH+132;//(actualWidth-10)/2.5;
				
				
				
				textLabel.x=0;
				textLabel.y=carImage.y+carImage.height+10;  
				textLabel.width=PacketView.PACKET_VIEW_WIDTH;   
				
				textLabel.height=NaN;     
				
				textLabel.validate();
				
				
				setSizeInternal(PacketView.PACKET_VIEW_WIDTH, textLabel.height+carImage.height+40, false);

			}
			
			if(isInvalid("change_text"))
			{
				textLabel.x=0;
				textLabel.y=carImage.y+carImage.height+10;
				textLabel.width=PacketView.PACKET_VIEW_WIDTH;    
				textLabel.height=NaN;  
				textLabel.validate();
				

			}
			  
			
			
		}
		
		
	
		/*public override function get useEmbeddedFont() : Boolean 
		{
			return language.supportsEmbeddedFont;
		}*/
	}
}