package com.despark.tuning3d.view.components.brands
{
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.packages.Packet;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.core.ListItemRenderer;
	import com.despark.tuning3d.view.components.core.PackagesView;
	import com.distriqt.GameService;
	import com.utils.DateParser;
	import com.utils.FontSelector;
	import com.utils.RegExpUtil;

	import feathers.controls.Label;
	import feathers.controls.text.TextFieldTextRenderer;

	import flash.geom.Rectangle;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import mx.utils.StringUtil;

	import starling.display.DisplayObject;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;
	import starling.textures.Texture;

	public class PacketItemRenderer extends ListItemRenderer {
		public static const TOUCH_PACKET_INFO:String="touchPacketInfo";
		public static const ITEM_TOUCH_PACKET:String="itemTouchPacket";
		public static const ITEM_RECIEVE_WIDTH:String="ITEM_RECIEVE_WIDTH";


		private static const PRICE_LABEL_DEFAULT:String="default";
		private static const PRICE_LABEL_PURCHASED:String="purchased";
		private static const PRICE_LABEL_UNLOCKED:String="unlocked";

		private static const RIGHT_MARGIN:Number = 82;



		private var _line:Quad=null;
		private var _background:Quad=null;
		
		private var _carCountLabel:Label=null;
		private var _rewardLabelImage: Image;
		
		private var _titleLabel:Label;
		private var _priceDisplay:TextFieldTextRenderer;
		private var _leftTimeLabel:Label;

		private var _carCheckIcon:Image=null;
		private var _priceTextColor:uint;
		private static const HIDDEN_ITEM_LIST:Array = ["55f893244d7db53459a59083", "56b37c4171819144463d2114", "56b37c5271819144463d2115", "56422502be65a71aa4475da6"];
		
		public function PacketItemRenderer() {
			super();
			fontSize = 11;
			textColor = 0xffffff;
			_priceDisplay = new TextFieldTextRenderer();
		}
		
		public function get priceTextColor():uint{
			return _priceTextColor;
		}

		private function setPriceLabelStyle(priceType:String):void{
			var size:Number;
			var fontName:String;
			var color:uint;
			switch(priceType){
				case PRICE_LABEL_DEFAULT:
					fontName = FontSelector.getMontserratSemiBold();
					size = 8;
					color = 0xFFFFFF;
					break;
				case PRICE_LABEL_PURCHASED:
					fontName = FontSelector.getAlumniSansMedium();
					size = 11;
					color = 0xff8c11;
					break;
				case PRICE_LABEL_UNLOCKED:
					fontName = FontSelector.getAlumniSansMedium();
					size = 11;
					color = 0x01c3fe;
					break;

			}
			_priceDisplay.textFormat = getPriceTextFieldTextFormat(fontName, size, color);
			_priceDisplay.embedFonts = true;
			_priceDisplay.isHTML = true;
			_priceDisplay.validate();
		}

		override public function set textColor(value : uint) : void {
			super.textColor = value;

		}
		
		protected override function  initialize():void{
			_background = new Quad(actualWidth, actualHeight, 0x111111);
			_background.addEventListener(Event.TRIGGERED, itemTouchTriggeredHandler);
			_background.touchable=true;
			addChild(_background);
			super.initialize();


			var textFormat:TextFormat;
			if(Constants.isNarrowScreen){
				textFormat = new TextFormat();
				textFormat.leading = -8;
			}
			_titleLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 21, 0xFFFFFF, TextFormatAlign.LEFT, Constants.isNarrowScreen, false, textFormat);
			_titleLabel.wordWrap = Constants.isNarrowScreen;
			_titleLabel.width = actualWidth - 75;
			addChild(_titleLabel);

			label.visible = false;
			
			
			_line=new Quad(actualWidth-32, 0.75, 0x7e8181);
			addChild(_line);

			_carCountLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 12, 0x0ba403, TextFormatAlign.CENTER, true, true);
			addChild(_carCountLabel);  

			setPriceLabelStyle(PRICE_LABEL_DEFAULT);
			addChild(_priceDisplay);

			var buyTxT:Texture = ButtonFactory.assetManager.getTexture("button_gradient");
			Image.bindScale9GridToTexture(buyTxT, new Rectangle(buyTxT.width / 3, 0, buyTxT.width / 3, buyTxT.height));

			_leftTimeLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratRegular(), 7, 0xFFFFFF, TextFormatAlign.LEFT, false);
			addChild(_leftTimeLabel);
			_leftTimeLabel.visible = false;
			
			var carCheckIconTexture:Texture=ButtonFactory.assetManager.getTexture("featured_price_check");  
			_carCheckIcon = new Image(carCheckIconTexture);
			_carCheckIcon.visible = false;
			_carCheckIcon.color = 0xFFFFFE;
			_carCheckIcon.width = 12;
			_carCheckIcon.scale = _carCheckIcon.scaleX;
			addChild(_carCheckIcon);

			var rewardedLabelTexture: Texture = ButtonFactory.assetManager.getTexture("challenge_rewards_btn_icon");
			_rewardLabelImage = new Image(rewardedLabelTexture);
            _rewardLabelImage.width = 12;
            _rewardLabelImage.scale = _rewardLabelImage.scaleX;
			_rewardLabelImage.color = 0xFFFFFE;
            _rewardLabelImage.visible = false;
            addChild(_rewardLabelImage);
		}

		public function getPriceTextFieldTextFormat(fontName:String, size:Number, color:uint) : TextFormat {
			var textFormat:TextFormat = new TextFormat(fontName, size, color);
			textFormat.align = TextFormatAlign.LEFT;
			textFormat.bold = true;
			textFormat.italic = false;
			return textFormat;
		}

		private function itemTouchTriggeredHandler(event : Event) : void {
			owner.dispatchEventWith(ITEM_TOUCH_PACKET,false, {packet:packet});
		}
		
		public function get packet() :Packet {
			return data as Packet;
		}

		protected override function commitData() : void {
			super.commitData();
			var title:String;
			if(AlertView.language) {
				title=AlertView.language.forceTranslate(packet.id, "title");
			}
			title ||= packet.title

			if(HIDDEN_ITEM_LIST.indexOf(packet.id) != -1){
				visible = !packet.lock;
				scaleY = visible? 1: 0;
			}

			_carCountLabel.visible = (packet.cars && packet.cars.length && packet.projectType && packet.lock);

			if (packet.type == "banner") {
				_carCountLabel.visible = false;
			}

			if (packet.type == Packet.TYPE_ALLPACKET) {
				_carCountLabel.visible = false;
			}

			if (packet.type == Packet.TYPE_CURRENTALLPACKET) {
				_carCountLabel.visible = false;
			}

			if (packet.type == Packet.TYPE_SUBCOMPONENT && (!packet.cars || packet.cars.length == 0)) {
				_carCountLabel.visible = false;
			}

			title = RegExpUtil.removeHtmlTags(title);
			_titleLabel.text = title;
			_titleLabel.validate();
			
			
			if(PackagesView.calacItemCount){
				if(_titleLabel.width + 150 > PackagesView.PACKAGE_ITEM_WIDTH){
					PackagesView.PACKAGE_ITEM_WIDTH = _titleLabel.width + 150;
					if(Device.isIphoneXVariant()){
						PackagesView.PACKAGE_ITEM_WIDTH += 25;
					}
				}
			}
			PackagesView.ITEM_COUNT++;
			if(!packet.cars){
				packet.cars=[];
			}

			_carCountLabel.text = packet.countAmountLabel + "<br>MODEL"+(packet.hdCarAmount>1?"S":"");
			_carCountLabel.width = 40;
			_carCountLabel.validate();
			
			var currency:String=""; 
				
			if(packet.feeCurrency){
				currency = packet.feeCurrency.replace("₽", "RUB");
			}

			currency=currency.replace("$", "USD");
			
			if(packet.lock){
				_leftTimeLabel.visible = false;
				setPriceLabelStyle(PRICE_LABEL_DEFAULT);
				_priceDisplay.text = packet.fee.toFixed(2) +" "+StringUtil.trim(currency);
				_priceDisplay.validate();
				_rewardLabelImage.visible = false;
				_carCheckIcon.visible = false;
			}else{
				if(AlertView.language){
                    if (packet.tempOpened) {
						setPriceLabelStyle(PRICE_LABEL_UNLOCKED);
                        _priceDisplay.text = AlertView.language.translate("mob_settings", "unlocked").toUpperCase();
                    } else {
						setPriceLabelStyle(PRICE_LABEL_PURCHASED);
                        _priceDisplay.text = AlertView.language.translate("mob_settings", "purchased").toUpperCase();
					}
				}else{
					if (packet.tempOpened) {
						setPriceLabelStyle(PRICE_LABEL_UNLOCKED);
                        _priceDisplay.text = "UNLOCKED";
                    } else {
						setPriceLabelStyle(PRICE_LABEL_PURCHASED);
                        _priceDisplay.text = "PURCHASED";
                    }
				}
				_priceDisplay.validate();
				_leftTimeLabel.visible = true;
				if (packet.tempOpened) {
					updateRewardPacketTimeLeft();
					_rewardLabelImage.visible = true;
					_carCheckIcon.visible = false;
					updateRewardsImage();
				} else {
					_leftTimeLabel.text = "LIFE TIME";
                    _rewardLabelImage.visible = false;
					_carCheckIcon.visible = true;
				}
			}
			
			
			
			this.invalidate("change_text");     

			if(PackagesView.calacItemCount){
				if(owner){
					owner.dispatchEventWith(ITEM_RECIEVE_WIDTH,false);
				}
			}

			updatePrice();
		}

		private function updateRewardsImage():void{
			setItemXPosition(_rewardLabelImage);
			_rewardLabelImage.y = 12;

			setItemXPosition(_carCheckIcon);
			_carCheckIcon.y = 12;
		}

		private function updatePrice():void{
			_priceDisplay.validate();
			setItemXPosition(_priceDisplay);
			setItemXPosition(_leftTimeLabel);
			setItemXPosition(_carCountLabel);
			_priceDisplay.y = actualHeight - (_priceDisplay.height + 7);
			if(packet){
				_priceDisplay.visible = !packet.lock;
				if(packet.lock && packet.coin3dFee){

				}else{
					_leftTimeLabel.y = (actualHeight - _leftTimeLabel.height) / 2 + 2.5;
				}
			}else{
				_leftTimeLabel.y = (actualHeight - _leftTimeLabel.height) / 2 + 2.5;
			}
			updateRewardsImage();
		}

		private function setItemXPosition(item:DisplayObject):void{
			item.x = actualWidth - (RIGHT_MARGIN + item.width) / 2;
		}
		
		protected override function commitSelection() :void
		{
			super.commitSelection();
			

			if(isSelected){
				_background.color=0x3d3e3f;
				_background.height=this.actualHeight;
				_background.y=0;
				_line.visible=false;
			}else{
				_background.color=0x111111;
				_background.height=this.actualHeight-1;
				_background.y=1; 
				_line.visible=true;
			}
			
		}
		
		protected override function layout() : void {
			_background.width = PackagesView.PACKAGE_ITEM_WIDTH - 7.5;
			_background.height = actualHeight;
			
			_line.width = PackagesView.PACKAGE_ITEM_WIDTH - 32;
			_line.x = 12;
			_line.y = actualHeight;
			
			if(Device.isIphoneXVariant()){
				_line.x += 25;
			}

			_carCheckIcon.x = PackagesView.PACKAGE_ITEM_WIDTH - 89;
			_carCheckIcon.y = (actualHeight - _carCheckIcon.height) / 2;

			_priceDisplay.validate();
			setItemXPosition(_priceDisplay);
			updatePrice();
		}

		private function updateRewardPacketTimeLeft():void{
			GameService.$.getRewardedPacketUnlockTime(packet.id, function (time:Number):void {
				if(time != 0){
					if(packet.oneTimeUnlocked){
						_leftTimeLabel.text = "ONE CAR";
					}else{
						_leftTimeLabel.text = DateParser.getInChallengeLabel(GameService.REWARDED_PACKAGE_ACTIVE_DURATION - (new Date().time - time));
					}
					_leftTimeLabel.validate();
					setItemXPosition(_leftTimeLabel);
					_leftTimeLabel.y = (actualHeight - _leftTimeLabel.height) / 2 + 2.5;
				}
			});
		}
		
		override protected function  draw():void{
			super.draw();
			if(isInvalid(INVALIDATION_FLAG_SIZE)) {
				if(packet){
					var title:String=packet.title;
					if(AlertView.language && AlertView.language.translate(packet.id, "title") && AlertView.language.translate(packet.id, "title")!="title") {
						title=AlertView.language.translate(packet.id, "title");
					}
					_titleLabel.validate();
					_titleLabel.x = 12;
					_titleLabel.y = (actualHeight - _titleLabel.height) / 2;
					if(Device.isIphoneXVariant()){
						_titleLabel.x += 24;
					}

					if(title){
						title = RegExpUtil.removeHtmlTags(title);
						if(_titleLabel.text != title){
							_titleLabel.text = title;
						}
					}

					_carCountLabel.validate();
					_carCountLabel.y = (actualHeight - _carCountLabel.height) / 2;
				}
				
				if(packet && packet.lock){
					_leftTimeLabel.visible = false;
					_priceDisplay.visible = false;
                    _rewardLabelImage.visible = false;
					_carCheckIcon.visible = false;
					_carCountLabel.visible = true;
					updatePrice();
					
				}else if (packet) {
					_carCountLabel.visible = false;
					_priceDisplay.visible = true;
					if(AlertView.language){
						if (packet.tempOpened) {
							setPriceLabelStyle(PRICE_LABEL_UNLOCKED);
							_priceDisplay.text = AlertView.language.translate("mob_settings", "unlocked").toUpperCase();
						} else {
							setPriceLabelStyle(PRICE_LABEL_PURCHASED);
							_priceDisplay.text = AlertView.language.translate("mob_settings", "purchased").toUpperCase();
						}
					}else{
						if (packet.tempOpened) {
							setPriceLabelStyle(PRICE_LABEL_UNLOCKED);
							_priceDisplay.text = "UNLOCKED";
						} else {
							setPriceLabelStyle(PRICE_LABEL_PURCHASED);
							_priceDisplay.text = "PURCHASED";
						}
					}

					_priceDisplay.validate();
                    if (packet.tempOpened) {
						updateRewardPacketTimeLeft();
						_rewardLabelImage.visible = true;
						_carCheckIcon.visible = false;
						updateRewardsImage();
                    } else {
						_leftTimeLabel.text = "LIFE TIME";
                        _rewardLabelImage.visible = false;
						_carCheckIcon.visible = true;
                    }

					_leftTimeLabel.visible = true;
					updatePrice();
				}

			}
			
			
			if(isInvalid("change_text")) {
				_titleLabel.validate();
				_titleLabel.y = (actualHeight - _titleLabel.height) / 2;
			}
			
		}
		
	}
}