package com.despark.tuning3d.view.components.cars
{
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarBackground;
    import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.packages.Packet;
    import com.despark.tuning3d.view.BrandsScreenMediator;
    import com.despark.tuning3d.view.components.core.BackgroundAnimationClip;
    import com.despark.tuning3d.view.components.core.BackgroundAnimationLoader;
    import com.despark.tuning3d.view.components.core.BackgroundContainer;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.MultilingualLabel;
    import com.utils.Console;

    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import flash.display.Loader;
    import flash.display.LoaderInfo;
    import flash.events.Event;
    import flash.events.HTTPStatusEvent;
    import flash.events.IEventDispatcher;
    import flash.events.IOErrorEvent;
    import flash.events.MouseEvent;
    import flash.events.ProgressEvent;
    import flash.events.SecurityErrorEvent;
    import flash.filesystem.File;
    import flash.geom.Rectangle;
    import flash.net.URLRequest;
    import flash.system.ApplicationDomain;
    import flash.system.LoaderContext;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.textures.Texture;
    import starling.utils.AssetManager;
    import starling.utils.ScaleMode;

    public class FeaturedCarItemRenderer extends ListItemRenderer {
    private var quad:Quad;
    private var carLockIcon:Image;
    private var carUnlockIcon:Image;
    private static var embbededBgTexture : Texture;
    private var loader:Loader;
    private var _clipRectQuad:Quad;

    public static var assetManager : AssetManager;
    private var _carLastId : String = "";
    private static var _cachedTextures:Object;
    public var isLoaded: Boolean = false;
   // private var _backgroundContainer:BackgroundContainer;
    private var _carBackground:CarBackground;

    public function FeaturedCarItemRenderer()
    {
        super();

        if(!embbededBgTexture)
        {
            embbededBgTexture = Texture.fromColor(600, 600/1.7);
        }
        _cachedTextures ||= {};
        _hitArea=new Rectangle(0,0,0,0);
        this._isQuickHitAreaEnabled=true;
    }

    public function get car() : Car{
        return data as Car;
    }

    override protected  function autoSizeIfNeeded():Boolean{
        return true;
    }

    public function get isCurrentImage():Boolean{
        var currentIndex:int = owner.dataProvider.getItemIndex(car);
        return (owner.horizontalPageIndex == currentIndex);
    }

    public function get isPrevImage():Boolean{
        var currentIndex:int = owner.dataProvider.getItemIndex(car);
        return (owner.horizontalPageIndex == currentIndex + 1);
    }

    public function get isNextImage():Boolean{
        var currentIndex:int = owner.dataProvider.getItemIndex(car);
        return (owner.horizontalPageIndex == currentIndex - 1);
    }

    public function get isAllowedImage():Boolean{
        return isCurrentImage || isPrevImage || isNextImage;
    }

    public function loading(): void {
        if (!imageLoader) {
            initialize();
        }
        if (!imageLoader.completeHandler) {
            imageLoader.completeHandler = completeLoadImage;
        }
        if(car.featuredImageUrl){
            car.featuredImageUrl = car.featuredImageUrl.replace("default.png", "car_chooser_app.jpg");
        }
        var url:String = car.featuredImageUrl;
        var newCar:Car;
        if (!imageLoader.persistentImageProvider || imageLoader.persistentImageProvider.remoteUrl != url) {
            newCar = new Car();
            newCar.id = car.id;
            newCar.imageURL = url;

            imageLoader.completeHandler = completeLoadImage;
            imageLoader.errorHandler2 = errorLoadImage;
            imageLoader.persistentImageProvider = newCar;
        }

        if (car && car.packet && car.packet.lock) {
            if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                carLockIcon.visible = true;
                carUnlockIcon.visible = false;
            } else {
                carLockIcon.visible = false;
                carUnlockIcon.visible = false;
            }
        } else if (car && car.packet) {
            if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                carLockIcon.visible = false;
                carUnlockIcon.visible = true;
            } else {
                carLockIcon.visible = false;
                carUnlockIcon.visible = false;
            }
        } else {
            carLockIcon.visible = false;
            carUnlockIcon.visible = false;
        }
    }

    protected override function initialize():void {
        _clipRectQuad = new Quad(1,1);

        quad = new Quad(this.actualWidth, this.actualHeight, 0x008800);
        quad.alpha = 0;
        addChild(quad);

        /*_backgroundContainer = new BackgroundContainer();
        _backgroundContainer.addEventListener(BackgroundContainer.ANIMATIONS_LOAD_COMPLETE, animationLoadHandler);
        _backgroundContainer.carBackground = carBackground;
        addChild(_backgroundContainer);*/

        label = new MultilingualLabel();
        addChild(label);

        imageLoader = new ImageLoaderWithStates();
        imageLoader.overrideDispose = true;
        imageLoader.scaleMode = ScaleMode.NONE;
        addChild(imageLoader);

        var carLockTexture:Texture=ButtonFactory.assetManager.getTexture("challenge_rewards_popup_icon");
        carLockIcon=new Image(carLockTexture);
        carLockIcon.visible=false;
        //addChild(carLockIcon);

        var carUnlockTexture:Texture=ButtonFactory.assetManager.getTexture("car_unlock_big_icon");
        carUnlockIcon=new Image(carUnlockTexture);
        carUnlockIcon.color = 0x1fb207;
        carUnlockIcon.visible=false;
        //addChild(carUnlockIcon);

        carBackground.mainUrl = carBackground.mainUrl.replace(/^\/\//, 'http://');
    }

        /*private function animationLoadHandler(event:starling.events.Event):void {
            _backgroundContainer.setSize(actualWidth, actualHeight);
            setBackgroundSize();
        }*/

        private function loadBackground(url:String):void{
            if(loader){
                return;
            }
            loader = new Loader();
            loader.contentLoaderInfo.addEventListener(flash.events.Event.COMPLETE, function (e: flash.events.Event):void {
                var currentTarget:LoaderInfo = e.currentTarget as LoaderInfo;
                var smallBMD:BitmapData = (currentTarget.content as Bitmap).bitmapData;

                /*if (_backgroundContainer) {
                    var bgUrl:String = url;
                    var texture:Texture = Texture.fromBitmapData(smallBMD);
                    texture.root.onRestore = function():void{
                        loadBackground(bgUrl);
                    };
                    _backgroundContainer.updateBackground(texture);
                    if(carBackground.animatedObjects){
                        _backgroundContainer.createAnimatedObjects(carBackground.animatedObjects, 0);
                    }
                }*/

                smallBMD.dispose();
            });
            loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, function (e:IOErrorEvent):void {
                trace('error', url);
            });
            loader.contentLoaderInfo.addEventListener(SecurityErrorEvent.SECURITY_ERROR, function (e:SecurityErrorEvent):void {
                trace('error', url);
            });
            var cacheUrl:String = getManualCacheURL(url);
            if(cacheUrl){
                url = cacheUrl;
            }

            var request:URLRequest = new URLRequest(url);
            loader.load(request);
        }

        private static function getManualCacheURL(url: String) : String{
            if(url == null) {
                return null;
            }
            url = url.split('com')[1];
            var file : File = File.applicationDirectory.resolvePath(Constants.MANUAL_CACHE_PATH + url);
            if(file.exists && !file.isDirectory){
                return file.url;
            } else{
                return null;
            }
        }
/*

        private function setBackgroundSize():void{
            if(_backgroundContainer && _backgroundContainer.texture){
                updatePositions();
            }
            _backgroundContainer.x = (actualWidth - _backgroundContainer.width) / 2;
            _backgroundContainer.y = (actualHeight - _backgroundContainer.height) / 2;
        }
*/


        /*public function updatePositions():void{
            if(_backgroundContainer.texture){
                setSize(owner.width, owner.height);
                if(imageLoader.image){
                    imageLoader.validate();
                    _backgroundContainer.background.scale = imageLoader.image.scale;
                    _backgroundContainer.width = _backgroundContainer.background.width;
                    _backgroundContainer.height = _backgroundContainer.background.height;
                    _backgroundContainer.validate();
                }
                _backgroundContainer.x = (actualWidth - _backgroundContainer.width) / 2;
                _backgroundContainer.y = (actualHeight - _backgroundContainer.height) / 2;
                updateAnimationList();
            }
        }

        private function updateAnimationList():void{
            if(_backgroundContainer.animationList){
                for(var i:int = 0; i < _backgroundContainer.animationList.length; i++){
                    if(_backgroundContainer.animationList[i]){
                        for(var j:int = 0; j < _backgroundContainer.animationList[i].length; j++){
                            updateAnimationPosition(_backgroundContainer.animationList[i][j]);
                        }
                    }
                }
            }
        }

        private function updateAnimationPosition(animation:BackgroundAnimationClip):void{
            if(animation){
                var atlasLoader:BackgroundAnimationLoader = animation.loader;
                animation.loader = atlasLoader;
                animation.width = _backgroundContainer.background.width;
                animation.scale = animation.scaleX;
                animation.x = _backgroundContainer.background.x + atlasLoader.animatedObjectData.cam1.x * animation.scaleX;
                animation.y = _backgroundContainer.background.y + atlasLoader.animatedObjectData.cam1.y * animation.scaleY;
            }
        }
*/
    public function specialDispose():void{
        if(car){
            _carLastId = car.id;
        }else{
            _carLastId = "";
        }
        disposeLoader();
        if(imageLoader ) {
            imageLoader.sepcialDispose();
        }
        isLoaded = false;
    }

    override public function dispose():void{
        super.dispose();
    }

    protected override function layout():void {
        if(_clipRectQuad){
            _clipRectQuad.width = actualWidth;
            _clipRectQuad.height = actualHeight;
            this.mask = _clipRectQuad;
        }

        label.visible=false;
        resampleCar(imageLoader);

        quad.width=this.actualWidth;
        quad.height=this.actualHeight;
        layoutLockIcons();

        carLockIcon.visible = false;
        carUnlockIcon.visible = false;


        _hitArea.width=actualWidth;
        _hitArea.height=actualHeight;
    }

    private function layoutLockIcons():void{
        carLockIcon.x = actualWidth / 2 + 70;
        carUnlockIcon.x = actualWidth / 2 + 70;
        if(imageLoader && imageLoader.currentTexture){
            carLockIcon.y = imageLoader.y + (imageLoader.height - carLockIcon.height) / 2 - 20;
            carUnlockIcon.y = imageLoader.y + (imageLoader.height - carUnlockIcon.height) / 2 - 20;
        }else{
            carLockIcon.y = (actualHeight - carLockIcon.height) / 2 - 20;
            carUnlockIcon.y = (actualHeight - carUnlockIcon.height) / 2 - 20;
        }
    }

    private function resampleCar(target:ImageLoaderWithStates): void {
        if(!target){
            return;
        }
        var tmpWidth:Number;
        var tmpHeight:Number;
        var y_offset:Number = actualHeight * 47 / 320;
        if(target.currentTexture){
            if(AppMode.self.mode == AppMode.MODE_TUNING){
                tmpWidth = target.currentTexture.frameWidth * 1.5 / 3.75;
                tmpHeight = target.currentTexture.frameHeight * 1.5 / 3.75;
            }else{
                tmpWidth = target.currentTexture.frameWidth * 1.5 / 4;
                tmpHeight = target.currentTexture.frameHeight * 1.5 / 4;
            }
        }else{
            isLoaded = false;
        }

        target.width = tmpWidth;
        target.height = tmpHeight;

        target.x = (actualWidth - target.width) / 2;
        //target.y = actualHeight - target.height - y_offset;
        target.y = (actualHeight - target.height) / 2;
        target.validate();
        layoutLockIcons();
    }

    protected override function commitData() : void {
        super.commitData();
        Starling.juggler.delayCall(completeLoadImage, 5, null); // call complete to hide menu, in case internet is to slow, and
        if(car.id == _carLastId) {
            completeLoadImage(null);
            return;
        }
        _carLastId = car.id;
        specialDispose();
        if (car) {
            if(car.featuredImageUrl){
                car.featuredImageUrl = car.featuredImageUrl.replace("default.png", "car_chooser_app.jpg");
            }
            var url:String = car.featuredImageUrl;
            var newCar:Car;

            if (!imageLoader.completeHandler) {
                imageLoader.completeHandler = completeLoadImage;
            }

            if (!imageLoader.persistentImageProvider || imageLoader.persistentImageProvider.remoteUrl != url) {
                newCar = new Car();
                newCar.id = car.id;
                newCar.imageURL = url;

                imageLoader.completeHandler = completeLoadImage;
                imageLoader.errorHandler2 = errorLoadImage;
                imageLoader.persistentImageProvider = newCar;
            }

            if (car && car.packet && car.packet.lock) {
                if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                    carLockIcon.visible = true;
                    carUnlockIcon.visible = false;
                } else {
                    carLockIcon.visible = false;
                    carUnlockIcon.visible = false;
                }
            } else if (car && car.packet) {
                if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                    carLockIcon.visible = false;
                    carUnlockIcon.visible = true;
                } else {
                    carLockIcon.visible = false;
                    carUnlockIcon.visible = false;
                }
            } else {
                carLockIcon.visible = false;
                carUnlockIcon.visible = false;
            }
        }
    }

    private function disposeLoader() : void
    {
        if(loader)
        {
            if(loader.contentLoaderInfo && loader.contentLoaderInfo.hasEventListener(flash.events.Event.COMPLETE))
            {
                deleteListeners(loader.contentLoaderInfo);
            }
            try
            {
                loader.unloadAndStop(true);

            }
            catch (e:Error) {trace(e.getStackTrace())}

            loader=null;
        }
    }


    private function configureListeners(dispatcher:IEventDispatcher):void {
        dispatcher.addEventListener(flash.events.Event.COMPLETE, completeHandler);
        dispatcher.addEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusHandler);
        dispatcher.addEventListener(flash.events.Event.INIT, initHandler);
        dispatcher.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        dispatcher.addEventListener(flash.events.Event.OPEN, openHandler);
        dispatcher.addEventListener(ProgressEvent.PROGRESS, progressHandler);
        dispatcher.addEventListener(flash.events.Event.UNLOAD, unLoadHandler);
    }

    private function deleteListeners(dispatcher:IEventDispatcher):void {
        dispatcher.removeEventListener(flash.events.Event.COMPLETE, completeHandler);
        dispatcher.removeEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusHandler);
        dispatcher.removeEventListener(flash.events.Event.INIT, initHandler);
        dispatcher.removeEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        dispatcher.removeEventListener(flash.events.Event.OPEN, openHandler);
        dispatcher.removeEventListener(ProgressEvent.PROGRESS, progressHandler);
        dispatcher.removeEventListener(flash.events.Event.UNLOAD, unLoadHandler);
    }

    private function completeHandler(event:flash.events.Event):void {
        isLoaded = true;
        var bitmap:Bitmap;
        try{
            bitmap = loader.content as Bitmap;
        }catch (e:Error) {

        }
        if(!bitmap) {
            return;
        }

        var bitmapData:BitmapData = bitmap.bitmapData;
        try{
            bitmapData.width;
            if(!bitmapData) {
                return;
            }
        }catch (e:Error) {
            return;
        }
        _cachedTextures ||= {};
        if(_cachedTextures[loader.contentLoaderInfo.url]){
            Texture(_cachedTextures[loader.contentLoaderInfo.url]).dispose();
            delete _cachedTextures[loader.contentLoaderInfo.url];
        }
        var texture:Texture = Texture.fromBitmapData(bitmapData,false,false,1);
        _cachedTextures[loader.contentLoaderInfo.url] = texture;
        resetBackground(texture);

        bitmapData.dispose();
    }

    private function destroyCachedTexture():void{
        if(!_cachedTextures)
            return;
        for(var key:String in _cachedTextures){
            Texture(_cachedTextures[key]).dispose();
        }
        _cachedTextures = {};
    }

    private function httpStatusHandler(event:HTTPStatusEvent):void {
        Tracer.print(event.toString());
    }

    private function initHandler(event:flash.events.Event):void {
    }

    private function ioErrorHandler(event:IOErrorEvent):void {
        Tracer.print(event.toString());
        resetBackground(embbededBgTexture);
    }

    private function resetBackground(texture:Texture):void{
        if(loader) {
            deleteListeners(loader.contentLoaderInfo);
        }
    }

    private function openHandler(event:flash.events.Event):void {
    }

    private function progressHandler(event:ProgressEvent):void {
    }

    private function unLoadHandler(event:flash.events.Event):void {
    }

    private function errorLoadImage(image:ImageLoaderWithStates):void{
        completeLoadImage(null)
    }

    private function completeLoadImage(image:ImageLoaderWithStates):void{
        BrandsScreenMediator.FIRST_LOAD_COMPLETE = true;
        BrandsScreenMediator.SECOND_LOAD_COMPLETE = true;
        isLoaded = true;
        resampleCar(imageLoader);
        if(this.owner){
            this.owner.dispatchEventWith("featuredCompleteLoad");
        }
        loadBackground(carBackground.mainUrl);
    }

    protected override function draw():void {
        super.draw();
        if (car && car.featuredImageUrl) {
            var url:String=car.featuredImageUrl;
            var newCar:Car;
            var deleteFile:Boolean=false;

            if (imageLoader.persistentImageProvider) {
                var localFileURL:String = car.featuredImageUrl.replace("http://cdn1.3dtuning.com/", "");

                var localFile: File = File.cacheDirectory.resolvePath(localFileURL);

                if (localFile.exists) {
                    var creationDate:Date=new Date();
                    creationDate.setTime(Date.parse(localFile.creationDate));
                    var nowDate:Date=new Date();
                    var interval:int=(nowDate.getTime()/1000)-(creationDate.getTime()/1000);

                    if(interval>60*60*3){
                        localFile.deleteFile();
                        deleteFile=true;
                    }
                }
            }


            var b:Boolean = true;
            url = url.replace("default.png", "car_chooser_app.jpg");
            if ((imageLoader.persistentImageProvider && imageLoader.persistentImageProvider.remoteUrl != url && b) || deleteFile) {
                newCar=new Car();
                newCar.id=car.id;
                newCar.imageURL = url;
                imageLoader.persistentImageProvider = newCar;
            } else if (!imageLoader.persistentImageProvider && b) {
                newCar=new Car();
                newCar.id=car.id;
                newCar.imageURL = url;
                imageLoader.persistentImageProvider = newCar;
            }

            if (car && car.packet && car.packet.lock) {
                if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
                    carLockIcon.visible=true;
                    carUnlockIcon.visible=false;
                }
                else{
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=false;
                }
            } else if(car && car.packet) {
                if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=true;
                }
                else{
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=false;
                }
            } else{
                carLockIcon.visible=false;
                carUnlockIcon.visible=false;
            }
        }
        label.visible=false;
        if(imageLoader){
            resampleCar(imageLoader);
        }
/*
        if(_backgroundContainer){
            updateAnimationList();
        }*/
    }

        public function get carBackground():CarBackground {
            return _carBackground;
        }

        public function set carBackground(value:CarBackground):void {
            _carBackground = value;
        }
    }
}