package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.brands.BrandScreenCarChooser;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeLeaderboardItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeLeaderboardUserRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeScreenItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusInChallenge;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeUserAvatar;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ColorUtil;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.tunings.editing.ValueStepper;
    import com.utils.FontSelector;
    import com.utils.NumberUtils;
    import com.utils.ObjectUtils;
    import com.utils.Tracker;

    import feathers.controls.ButtonState;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.List;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.ToggleButton;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.controls.text.TextFieldTextEditor;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.FeathersControl;
    import feathers.core.ITextEditor;
    import feathers.core.ITextRenderer;
    import feathers.core.ToggleGroup;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalLayout;

    import flash.geom.Rectangle;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.core.Starling;

    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.filters.ColorMatrixFilter;
    import starling.textures.Texture;

    public class ChallengesLeaderboardScreen extends ChallengesScreen {
        private var _challengesList:ExtendedList;
        private var _usersList:List;
        private var _topUsersLabel:ExtendedLabel;
        private var _userLabelGroup:LayoutGroup;
        private var _userAvatar:ChallengeUserAvatar;
        private var _userRankLabel:Label;
        private var _messageLabel:Label;
        private var _userRankValue:String = "";
        private var _leaderboardData:Object;
        //
        private var _periodSelector:ValueStepper;
        private var _challengeTypeSelector:ValueStepper;
        private var _carTypeSelector:ValueStepper;
        private var _borderList:Vector.<Quad>;
        //
        public static const CHALLENGE_SELECTED:String = "ChallengesLeaderboardScreen::challengeSelected";
        public static const USER_SELECTED:String = "ChallengesLeaderboardScreen::userSelected";
        public static const LOAD_LEADERBOARD:String = "ChallengesLeaderboardScreen::loadLeaderboard";
        private static const ITEM_WIDTH:Number = 56.5;

        private static var CHALLENGE_TYPE_DESIGN:String = "design";
        private static var CHALLENGE_TYPE_TIME_TRIAL:String = "trial";

        private static var CHALLENGE_PERIOD_THIS_WEEK:String = "cw";
        private static var CHALLENGE_PERIOD_LAST_WEEK:String = "lw";
        private static var CHALLENGE_PERIOD_ALL_TIME:String = "all";

        public static var LAST_SELECTED_SECTION:String = "all";
        public static var LAST_SELECTED_TYPE:String = CHALLENGE_TYPE_DESIGN;
        public static var LAST_SELECTED_PERIOD:String = CHALLENGE_PERIOD_LAST_WEEK;
        //
        public function ChallengesLeaderboardScreen(name:String = 'ChallengesListScreen') {
            super(name);
        }

        override protected function initialize():void {
            super.initialize();
            _toggleGroup.selectedItem = _leaderboardBtn;


            _challengesList = new ExtendedList();
            var listLayout:HorizontalLayout = new HorizontalLayout();
            listLayout.gap = 1;

            _challengesList.height = contentHeight / 2;
            _challengesList.paddingTop = 0;
            _challengesList.layout = listLayout;
            _challengesList.verticalScrollPolicy = ScrollPolicy.OFF;
            _challengesList.horizontalScrollPolicy = ScrollPolicy.ON;
            _challengesList.addEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, challengeSelectedHandler);
            addChild(_challengesList);
            _challengesList.itemRendererFactory = function():IListItemRenderer{
                var renderer:ChallengeLeaderboardItemRenderer = new ChallengeLeaderboardItemRenderer();
                renderer.isWideScreen = Constants.isWideScreen;
                renderer.isUltraWideScreen = Constants.isUltraWideScreen;
                renderer.owner = _challengesList;
                renderer.setSize((contentWidth - 3) / 4, _challengesList.height);
                return renderer;
            }


            createUsersList();

            _topUsersLabel = new ExtendedLabel();
            _topUsersLabel.paddingLeft = _usersList.paddingLeft;
            _topUsersLabel.textRendererFactory = LabelFactory.challengeLeaderboardTitle2Label;
            _topUsersLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));
            _topUsersLabel.defaultIcon.scale = 0.7 * ButtonFactory.assetManager.scaleFactor / 2;
            _topUsersLabel.labelOffsetX = 3;
            _topUsersLabel.labelOffsetY = 2;
            _topUsersLabel.text = "TOP TUNERS";
            addChild(_topUsersLabel);
            var colorFilter:ColorMatrixFilter = new ColorMatrixFilter();
            colorFilter.tint(0x535353, 1);
            _topUsersLabel.defaultIcon.filter = colorFilter;
            _topUsersLabel.validate();

            if(UsersProxy.loggedInUser() && user){
                createUserLabel();
            }

            //
            _periodSelector = new ValueStepper();
            _periodSelector.incrementIconColor = _periodSelector.decrementIconColor = 0x535353;
            _periodSelector.setIcons("arrow_left_challenge", "arrow_right_challenge");
            _periodSelector.width = 110;
            _periodSelector.height = 15;
            _periodSelector.addEventListener(Event.CHANGE, periodStepperChangeHandler);
            _periodSelector.setTextEditorFactory(valueStepperLabel,0,0,0);
            addChild(_periodSelector);
            _periodSelector.iconScale = 0.75;
            _periodSelector.data = [{title:"LAST WEEK", value:CHALLENGE_PERIOD_LAST_WEEK}, {title:"THIS WEEK", value:CHALLENGE_PERIOD_THIS_WEEK}, {title:"ALL TIME", value:CHALLENGE_PERIOD_ALL_TIME}];
            //
            _challengeTypeSelector = new ValueStepper();
            _challengeTypeSelector.incrementIconColor = _challengeTypeSelector.decrementIconColor = 0x535353;
            _challengeTypeSelector.setIcons("arrow_left_challenge", "arrow_right_challenge");
            _challengeTypeSelector.width = 190;
            _challengeTypeSelector.height = 15;
            _challengeTypeSelector.addEventListener(Event.CHANGE, challengeTypeStepperChangeHandler);
            _challengeTypeSelector.setTextEditorFactory(valueStepperLabel,0,0,0);
            addChild(_challengeTypeSelector);
            _challengeTypeSelector.iconScale = 0.75;
            _challengeTypeSelector.data = [{title:"DESIGN CHALLENGES", value:CHALLENGE_TYPE_DESIGN}, {title:"TIME-TRIAL CHALLENGES", value:CHALLENGE_TYPE_TIME_TRIAL}];
            //
            _carTypeSelector = new ValueStepper();
            _carTypeSelector.incrementIconColor = _carTypeSelector.decrementIconColor = 0x535353;
            _carTypeSelector.setIcons("arrow_left_challenge", "arrow_right_challenge");
            _carTypeSelector.width = 105;
            _carTypeSelector.height = 15;
            _carTypeSelector.addEventListener(Event.CHANGE, carTypeStepperChangeHandler);
            _carTypeSelector.setTextEditorFactory(valueStepperLabel,0,0,0);
            _carTypeSelector.iconScale = 0.75;
            addChild(_carTypeSelector);

            //
            _messageLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratRegular(), 11, 0x424242, TextFormatAlign.LEFT, false);
            _messageLabel.text = "There are no users in this category at the moment";
            _messageLabel.visible = false;
            addChild(_messageLabel);

            //
            var carTypeList:Array = [{title:"ALL CARS", value:"all"}];
            var i:int;
            for(i = 0; i < BrandScreenCarChooser.BODY_TYPE_LABELS.length; i++){
                carTypeList.push({title:BrandScreenCarChooser.BODY_TYPE_LABELS[i].name, value:BrandScreenCarChooser.BODY_TYPE_LABELS[i].value});
            }
            _carTypeSelector.data = carTypeList;
            //
            _borderList = new Vector.<Quad>(4);
            for(i = 0; i < _borderList.length; i++){
                _borderList[i] = createSeparator();
                addChild(_borderList[i]);
            }
        }

        private static function createSeparator():Quad{
            var quad:Quad = new Quad(1,1, 0xcccccc);
            quad.width = 0.5;
            quad.height = 16;
            return quad;
        }

        public static function  valueStepperLabel():ITextEditor {
            var textRenderer:TextFieldTextEditor = new TextFieldTextEditor();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 10, 0x535353);
            textRenderer.textFormat.bold = false;
            textRenderer.isEditable = false;
            textRenderer.isSelectable = false;
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            return textRenderer;
        }

        private function periodStepperChangeHandler(event:Event):void{
            if(_periodSelector.selectedItem.value){
                LAST_SELECTED_PERIOD = _periodSelector.selectedItem.value;
                setLeaderboardData(LAST_SELECTED_PERIOD, LAST_SELECTED_TYPE, LAST_SELECTED_SECTION);
            }
        }

        private function challengeTypeStepperChangeHandler(event:Event):void{
            if(_challengeTypeSelector.selectedItem.value){
                LAST_SELECTED_TYPE = _challengeTypeSelector.selectedItem.value;
                setLeaderboardData(LAST_SELECTED_PERIOD, LAST_SELECTED_TYPE, LAST_SELECTED_SECTION);
            }
        }

        private function carTypeStepperChangeHandler(event:Event):void{
            if(_carTypeSelector.selectedItem.value){
                LAST_SELECTED_SECTION = _carTypeSelector.selectedItem.value;
                setLeaderboardData(LAST_SELECTED_PERIOD, LAST_SELECTED_TYPE, LAST_SELECTED_SECTION);
            }
        }

        override public function updateUserData(value:Object):void{
            super .updateUserData(value);
            if(UsersProxy.loggedInUser() && user && !_userLabelGroup){
                createUserLabel();
                invalidate(FeathersControl.INVALIDATION_FLAG_SIZE);
            }
        }

        private function createUserLabel():void{
            _userLabelGroup = new LayoutGroup();
            _userLabelGroup.width = 65;
            _userAvatar ||= new ChallengeUserAvatar(user, 51);
            _userAvatar.x = (_userLabelGroup.width - _userAvatar.width) / 2;
            _userLabelGroup.addChild(_userAvatar);
            var groupLayout:VerticalLayout = new VerticalLayout();
            groupLayout.horizontalAlign = HorizontalAlign.CENTER;
            groupLayout.paddingTop = 9;
            _userLabelGroup.layout = groupLayout;
            addChild(_userLabelGroup);


            var frameTxt:Texture = ButtonFactory.assetManager.getTexture("border_line");
            var frameImg:LayoutImage = new LayoutImage(frameTxt);
            frameImg.scale9Grid = new Rectangle(frameTxt.width/2 - 0.5, frameTxt.height / 2 - 0.5, 1, 1);
            frameImg.color = 0xfba500;
            frameImg.includeInLayout = false;
            frameImg.width = _userLabelGroup.width;
            frameImg.height = 83;
            _userLabelGroup.addChild(frameImg);

            var _yourRankLabel:Label = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 9, 0x535353, TextFormatAlign.CENTER, false, false);
            _yourRankLabel.text = "YOUR RANK";
            _yourRankLabel.paddingTop = 0;
            _userLabelGroup.addChild(_yourRankLabel);

            _userRankLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratBold(), 7.5, 0x535353, TextFormatAlign.CENTER, false, false);
            _userRankLabel.paddingTop = -1;
            _userLabelGroup.addChild(_userRankLabel);
            _userRankLabel.text = _userRankValue;
        }

        private function updateOrientation():void{
            if(!stage || !_challengesList){
                return;
            }
            _challengesList.orientation = stage.starling.nativeStage.orientation;
        }

        override protected function initTopMenuContainer():void{
            super .initTopMenuContainer();
            title.text = 'LEADERBOARD';
        }

        private function challengeSelectedHandler(event:Event):void {
            dispatchEvent(new Event(CHALLENGE_SELECTED, false, event.data));
        }

        private function userSelectedHandler(event:Event):void {
            var eventData:Object = {challengeType: _challengeTypeSelector.selectedItem.value};
            if(eventData.challengeType == CHALLENGE_TYPE_TIME_TRIAL || _carTypeSelector.selectedItem.value == "all"){
                eventData = ObjectUtils.merge(event.data, eventData);
            }else{
                eventData.bodyType = _carTypeSelector.selectedItem.value;
                eventData = ObjectUtils.merge(event.data, eventData);
            }
            dispatchEvent(new Event(USER_SELECTED, false, eventData));
        }

        override protected function draw():void {
            super.draw();
            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
            if (sizeInvalid) {
                _challengesList.width = actualWidth;
                _challengesList.x = (actualWidth - _challengesList.width) / 2;
                _challengesList.y = marginTop;

                updateListPosition();
            }
        }

        private function updateListPosition():void{
            var avatarWidth:Number = _userLabelGroup?_userLabelGroup.width:0;
            var userListWidth:Number;
            var num:int;
            if(Device.isIphoneXVariant()){
                _topUsersLabel.x = _challengesList.x + _sideMargin;
                userListWidth = _challengesList.width - 2 * _sideMargin - avatarWidth + 10 - (_usersList.paddingLeft + _usersList.paddingRight);
                num = int(userListWidth/(ITEM_WIDTH + 10));
                _usersList.width = num * ITEM_WIDTH + (num - 1) * 10 + (_usersList.paddingLeft + _usersList.paddingRight);
            }else{
                userListWidth = _challengesList.width - avatarWidth + 10 - (_usersList.paddingLeft + _usersList.paddingRight);
                _topUsersLabel.x = _challengesList.x;
                num = int(userListWidth/(ITEM_WIDTH + 10));
                _usersList.width = num * ITEM_WIDTH + (num - 1) * 10 + (_usersList.paddingLeft + _usersList.paddingRight);
            }

            _topUsersLabel.y = _challengesList.y + _challengesList.height + 8;
            _usersList.x = (actualWidth - _usersList.width - avatarWidth) / 2;
            _usersList.y = _topUsersLabel.y + _topUsersLabel.height + 7;

            if(_userLabelGroup){
                _userLabelGroup.x = _usersList.x + _usersList.width - _usersList.paddingRight;
                _userLabelGroup.y = _usersList.y;
            }

            _periodSelector.x = _topUsersLabel.x + _topUsersLabel.width + 10;
            _periodSelector.y = _topUsersLabel.y + (_topUsersLabel.height - _periodSelector.height) / 2 + 1;

            _challengeTypeSelector.x = _periodSelector.x + _periodSelector.width;
            _challengeTypeSelector.y = _topUsersLabel.y + (_topUsersLabel.height - _challengeTypeSelector.height) / 2 + 1;

            _carTypeSelector.x = _challengeTypeSelector.x + _challengeTypeSelector.width;
            _carTypeSelector.y = _topUsersLabel.y + (_topUsersLabel.height - _carTypeSelector.height) / 2 + 1;

            _borderList[0].x = _topUsersLabel.x + _topUsersLabel.width + 12;
            _borderList[0].y = _topUsersLabel.y + (_topUsersLabel.height - _borderList[0].height) / 2 + 1;

            _borderList[1].x = _periodSelector.x + _periodSelector.width;
            _borderList[1].y = _borderList[0].y;

            _borderList[2].x = _challengeTypeSelector.x + _challengeTypeSelector.width;
            _borderList[2].y = _borderList[0].y;

            _borderList[3].x = _carTypeSelector.x + _carTypeSelector.width;
            _borderList[3].y = _borderList[0].y;

            _messageLabel.x = (_borderList[3].x - _topUsersLabel.x - _messageLabel.width) / 2;
            _messageLabel.y = _topUsersLabel.y + (actualHeight - (_topUsersLabel.y + _topUsersLabel.height) - marginBottom) / 2;
        }

        public function loadLeaderboard(object:Object, challengeType:String, period:String):void{
            if(object){
                _leaderboardData ||= {};
                if(object.hasOwnProperty("lastChallengesTop") && object.lastChallengesTop.length){
                    _leaderboardData.lastChallengesTop = object.lastChallengesTop;
                    delete object.lastChallengesTop;
                }
                _leaderboardData[period] ||= {};
                _leaderboardData[period][challengeType] = object.leaderboard;
                if(object.leaderboardByType){
                    _leaderboardData[period][challengeType].leaderboardByType = object.leaderboardByType;
                }

                if(_challengesList && _challengesList.dataProvider == null && _leaderboardData.lastChallengesTop){
                    _challengesList.dataProvider = new ListCollection(_leaderboardData.lastChallengesTop);
                }

                setLeaderboardData(period, challengeType, LAST_SELECTED_SECTION);

                if(onComplete != null){
                    onComplete();
                    onComplete = null;
                }
            }
        }

        private function setLeaderboardData(challengePeriod:String, challengeType:String, bodyTypeValue:String = null):void{
            if(!_leaderboardData || !_leaderboardData.hasOwnProperty(challengePeriod) || !_leaderboardData[challengePeriod].hasOwnProperty(challengeType)){
                dispatchEventWith(LOAD_LEADERBOARD, false, {challengeType: challengeType, challengePeriod: challengePeriod});
                return;
            }
            setPeriodLabel(challengePeriod);
            setChallengeTypeLabel(challengeType);
            setCarTypeLabel(bodyTypeValue);

            if(_leaderboardData[challengePeriod][challengeType].activeUserPosition){
                _userRankValue = _leaderboardData[challengePeriod][challengeType].activeUserPosition.toString();
                if(_userRankLabel){
                    _userRankLabel.text = _userRankValue;
                    _userRankLabel.validate();
                }
            }

            var ratings:Array;
            var activeUserPosition:String;
            var activeUserRating:String;
            var i:int;
            if(challengeType == CHALLENGE_TYPE_TIME_TRIAL){
                ratings = _leaderboardData[challengePeriod][challengeType]["ratings"];
                activeUserPosition = _leaderboardData[challengePeriod][challengeType]["activeUserPosition"] || "";
                activeUserRating = _leaderboardData[challengePeriod][challengeType]["activeUserRating"] || "";
                for(i = 0; i < ratings.length; i++){
                    ratings[i]["challengeNewSystemRatings"] = ratings[i]["challengeNewSystemTrials"];
                }
                if(_userAvatar){
                    _userAvatar.ratingLabelBackgroundColor = 0xff6501;
                }
                //
                _carTypeSelector.isEnabled = false;
            }else if(challengeType == CHALLENGE_TYPE_DESIGN){
                _carTypeSelector.isEnabled = true;
                if(_userAvatar){
                    _userAvatar.ratingLabelBackgroundColor = 0xfba500;
                }
                if(bodyTypeValue == "all"){
                    ratings = _leaderboardData[challengePeriod][challengeType]["ratings"];
                    activeUserPosition = _leaderboardData[challengePeriod][challengeType]["activeUserPosition"] || "";
                    activeUserRating = _leaderboardData[challengePeriod][challengeType]["activeUserRating"] || "";
                }else{
                    ratings = ObjectUtils.getValue(_leaderboardData, challengePeriod + "." + challengeType + ".leaderboardByType." + bodyTypeValue + ".ratings", []);
                    activeUserPosition = ObjectUtils.getValue(_leaderboardData, challengePeriod + "." + challengeType + ".leaderboardByType." + bodyTypeValue + ".activeUserPosition", "");
                    activeUserRating = ObjectUtils.getValue(_leaderboardData, challengePeriod + "." + challengeType + ".leaderboardByType." + bodyTypeValue + ".activeUserRating", "");
                    for(i = 0; i < ratings.length; i++){
                        ratings[i].challengeNewSystemRatings = ratings[i].challengeNewRatings[bodyTypeValue].rating;
                    }
                }
            }
            _messageLabel.visible = (ratings.length == 0);
            var collection:ListCollection = new ListCollection(ratings);
            //
            setUserListData(collection, activeUserPosition, activeUserRating);
        }

        private function setPeriodLabel(value:String):void{
            if(_periodSelector.selectedItem.value != value){
                _periodSelector.removeEventListener(Event.CHANGE, periodStepperChangeHandler);
                _periodSelector.selectedItem = value;
                _periodSelector.addEventListener(Event.CHANGE, periodStepperChangeHandler);
            }
        }

        private function setChallengeTypeLabel(value:String):void{
            if(_challengeTypeSelector.selectedItem.value != value){
                _challengeTypeSelector.removeEventListener(Event.CHANGE, challengeTypeStepperChangeHandler);
                _challengeTypeSelector.selectedItem = value;
                _challengeTypeSelector.addEventListener(Event.CHANGE, challengeTypeStepperChangeHandler);
            }
        }

        private function setCarTypeLabel(value:String):void{
            if(value && _carTypeSelector.selectedItem.value != value){
                _carTypeSelector.removeEventListener(Event.CHANGE, carTypeStepperChangeHandler);
                _carTypeSelector.selectedItem = value;
                _carTypeSelector.addEventListener(Event.CHANGE, carTypeStepperChangeHandler);
            }
        }

        private function setUserListData(collection:ListCollection, activeUserPosition:String = "", activeUserRating:String = ""):void{
            if(!_usersList){
                createUsersList();
            }

            _usersList.dataProvider = collection;

            if(_userRankLabel){
                _userRankLabel.text = activeUserPosition;
            }
            if(_userAvatar){
                if(activeUserRating && activeUserRating.length){
                    activeUserRating = NumberUtils.toFixed(parseFloat(activeUserRating),2);
                }
                _userAvatar.setRatingLabel(activeUserRating);
            }
        }

        private function createUsersList():void{
            _usersList = new List();
            var listLayout:HorizontalLayout = new HorizontalLayout();
            listLayout.gap = 10;
            listLayout.useVirtualLayout = true;

            _usersList.height = 90;
            _usersList.paddingLeft = _usersList.paddingRight = 15;
            _usersList.paddingTop = 10;
            _usersList.layout = listLayout;
            _usersList.verticalScrollPolicy = ScrollPolicy.OFF;
            _usersList.horizontalScrollPolicy = ScrollPolicy.ON;
            _usersList.addEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, userSelectedHandler);
            _usersList.snapToPages = false;
            addChild(_usersList);

            _usersList.setItemRendererFactoryWithID("design", designItemFactory);
            _usersList.setItemRendererFactoryWithID("timetrial", timeTrialItemFactory);
            _usersList.factoryIDFunction = function(item:Object):String {
                if(_challengeTypeSelector.selectedItem.value == CHALLENGE_TYPE_TIME_TRIAL){
                    return "timetrial";
                }
                return "design";
            };
        }

        private function designItemFactory():IListItemRenderer{
            var renderer:ChallengeLeaderboardUserRenderer = new ChallengeLeaderboardUserRenderer();
            renderer.owner = _usersList;
            renderer.setSize(ITEM_WIDTH, _usersList.height);
            return renderer;
        }

        private function timeTrialItemFactory():IListItemRenderer{
            var renderer:ChallengeLeaderboardUserRenderer = new ChallengeLeaderboardUserRenderer();
            renderer.owner = _usersList;
            renderer.setSize(ITEM_WIDTH, _usersList.height);
            renderer.designRatingColor = 0xff6501;
            return renderer;
        }

        override public function ShowDone():void {
            if(onComplete == null){
                BannerStackedScreen.showAdBanner();
            }
            Tracker.trackEvent("Challenges Screen", "open");
        }

        override protected function screen_removedFromStageHandler(event:Event):void{
            super.screen_removedFromStageHandler(event);
            BannerStackedScreen.hideAdBanner(true);
        }

        override public function dispose():void {
            super.dispose();
            if (_challengesList) {
                _challengesList.removeFromParent(true);
                _challengesList = null;
            }
            if (_usersList) {
                _usersList.removeFromParent(true);
                _usersList = null;
            }
            if (_topUsersLabel) {
                _topUsersLabel.removeFromParent(true);
                _topUsersLabel = null;
            }
        }
    }
}
