package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.UIColors;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengesLoadBar;
    import com.despark.tuning3d.view.components.challenges.controls.UserLevelAvatar;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedToggleButton;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.MainMenuButton;
    import com.despark.tuning3d.view.components.core.MainMenuView;
    import com.despark.tuning3d.view.components.core.StackedBannerScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.gallery.GalleryLabelFactory;
    import com.distriqt.AdsManager;
    import com.utils.DisplayObjectUtil;
    import com.utils.ObjectUtils;
    import com.utils.Tracker;

    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.ToggleButton;
    import feathers.core.FeathersControl;
    import feathers.core.ToggleGroup;
    import feathers.layout.AnchorLayoutData;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.RelativePosition;
    import feathers.layout.VerticalAlign;

    import flash.events.StageOrientationEvent;

    import flash.geom.Point;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;

    public class ChallengesScreen extends StackedBannerScreen{
        private var _backgroundQuad:Quad;
        private var _bottomMenuContainer:LayoutGroup;
        protected var _menuButton:MainMenuButton;

        private var _topButtonPadding:Number = 0;
        private var _loadBar:ChallengesLoadBar;

        protected var title:Label;
        private var _messageField:Label;
        //
        protected var _toggleGroup:ToggleGroup;
        protected var _leaderboardBtn:ToggleButton;
        protected var _homeBtn:ToggleButton;
        protected var _votingBtn:ToggleButton;
        protected var _userGarageBtn:ToggleButton;
        protected var _notificationsBtn:ToggleButton;

        private var _notificationImage:LayoutImage;
        private var _userAvatar:UserLevelAvatar;
        private static const INVALIDATION_USER_AVATAR:String = "invalidateUserAvatar";
        private var _onComplete:Function;
        public function ChallengesScreen(name : String = 'ChallengesScreen') {
            super(name);
            _loadBar = new ChallengesLoadBar();
            _loadBar.name = '_loadBar';
        }

        override protected function initialize():void {
            super.initialize();
            setSize(stage.stageWidth, stage.stageHeight);
            showLoadBar();

            _backgroundQuad = new Quad(1, 1, UIColors.WHITE);
            addChildAt(_backgroundQuad, 0);

            _bottomMenuContainer = new LayoutGroup();
            _bottomMenuContainer.backgroundSkin = new Quad(1, 1, HEADER_BG_COLOR);
            var bottomMenuLayout:HorizontalLayout = new HorizontalLayout();
            bottomMenuLayout.verticalAlign = VerticalAlign.BOTTOM;
            bottomMenuLayout.horizontalAlign = HorizontalAlign.CENTER;
            bottomMenuLayout.paddingBottom = 13;
            _bottomMenuContainer.layout = bottomMenuLayout;
            addChild(_bottomMenuContainer);
            //
            _toggleGroup = new ToggleGroup();
            _leaderboardBtn = createBottomToggle("challenges_leaderboard_icon");

            _homeBtn = createBottomToggle("challenges_home_icon");
            _homeBtn.paddingBottom = -2;

            _votingBtn = createBottomToggle("challenges_likes_icon");

            _userGarageBtn = createBottomToggle("challenges_users_icon");

            _notificationsBtn = createBottomToggle("challenges_mails_icon");
            _notificationsBtn.paddingBottom = -1;

            _toggleGroup.addItem(_leaderboardBtn);
            _toggleGroup.addItem(_homeBtn);
            _toggleGroup.addItem(_votingBtn);
            _toggleGroup.addItem(_userGarageBtn);
            _toggleGroup.addItem(_notificationsBtn);
            _toggleGroup.addEventListener(Event.CHANGE, toggleChangeHandler);

            _bottomMenuContainer.addChild(_leaderboardBtn);
            _bottomMenuContainer.addChild(_homeBtn);
            _bottomMenuContainer.addChild(_votingBtn);
            _bottomMenuContainer.addChild(_userGarageBtn);
            _bottomMenuContainer.addChild(_notificationsBtn);
            addChild(_bottomMenuContainer);

            _notificationImage = new LayoutImage(assetManager.getTexture('notification'));
            _notificationImage.name = 'notification';
            _notificationImage.visible = false;
            _notificationImage.touchable = false;
            _notificationImage.width = _notificationImage.height = 4.5;
            _notificationImage.includeInLayout = false;
            addChild(_notificationImage);
            updateNotifications();
            //
            AdsManager.$.checkRewardAds(function(status:Boolean):void{
                updateRewardsAdsStatus();
            });
            //
            var arr:Vector.<ToggleButton> = new <ToggleButton>[_leaderboardBtn, _homeBtn, _votingBtn, _userGarageBtn, _notificationsBtn];
            var cWidth:Number = 0;
            for(var i:int = 0; i < arr.length; i++){
                arr[i].validate();
                cWidth += arr[i].width;
            }

            bottomMenuLayout.gap = (actualWidth / 3 - cWidth + 20) / (arr.length - 1);
        }

        override public function updateUserData(result:Object):void{
            super .updateUserData(result);

            if(user && user.id){
                _userAvatar ||= new UserLevelAvatar(35);
                if(!_userAvatar.parent){
                    addChild(_userAvatar);
                }
                _userAvatar.update(user);
                invalidate(INVALIDATION_USER_AVATAR);
            }else{
                if(_userAvatar){
                    _userAvatar.removeFromParent(true);
                    _userAvatar = null;
                }
            }
        }

        override protected function feathersControl_addedToStageHandler(event:Event):void{
            super .feathersControl_addedToStageHandler(event);
            if(Device.isIphoneXVariant()){
                Starling.current.nativeStage.addEventListener(StageOrientationEvent.ORIENTATION_CHANGE, nativeStage_orientationChangeHandler);
            }
            Starling.juggler.delayCall(updateLoadBar, 0.25);
        }

        private function updateLoadBar():void{
            if(_loadBar && _loadBar.stage){
                _loadBar.setSize(contentWidth, contentHeight);
                _loadBar.x = 0;
                _loadBar.y = marginTop;
                _loadBar.parent.setChildIndex(_loadBar, _loadBar.parent.numChildren);
                _loadBar.visible = contentWidth;
                _loadBar.validate();
            }
        }

        private function toggleChangeHandler(event:Event):void {
            switch (_toggleGroup.selectedItem) {
                case _leaderboardBtn:
                    if(!(this is ChallengesLeaderboardScreen)){
                        dispatchEvent(new Event(CHALLENGES_LEADERBOARD_SELECTED, false, {"tabMenu": true}));
                    }
                    break;
                case _homeBtn:
                    if(!(this is ChallengesListScreen)){
                        dispatchEvent(new Event(CHALLENGES_HOME_SELECTED, false, {"tabMenu": true}));
                    }
                    break;
                case _votingBtn:
                    if(!(this is ChallengesVotingScreen)){
                        dispatchEvent(new Event(CHALLENGES_RATE_SELECTED, false, {"tabMenu": true}));
                    }
                    break;
                case _userGarageBtn:
                    if(!(this is ChallengesUserTuningsScreen) || !ChallengesUserTuningsScreen(this).userGarage){
                        dispatchEvent(new Event(CHALLENGES_GARAGE_SELECTED, false, {"tabMenu": true}));
                    }
                    break;
                case _notificationsBtn:
                    if(!(this is ChallengesNotificationsScreen)){
                        dispatchEvent(new Event(CHALLENGES_NOTIFICATIONS_SELECTED, false, {"tabMenu": true}));
                    }
                    break;
            }
        }


        public function restoreScreen():void{
            if(this is ChallengesListScreen){
                _toggleGroup.selectedItem = _homeBtn;
            }else if(this is ChallengesLeaderboardScreen){
                _toggleGroup.selectedItem = _leaderboardBtn;
            }else if(this is ChallengesVotingScreen){
                _toggleGroup.selectedItem = _votingBtn;
            }
        }


        override protected function initTopMenuContainer():void {
            super.initTopMenuContainer();

            _ratingLabel.paddingRight = user?40:10;

            var menuButtonLayoutData:AnchorLayoutData = new AnchorLayoutData();
            menuButtonLayoutData.verticalCenter = 0;
            _menuButton = ButtonFactory.menuBigButton("a_back_button", HEADER_BG_COLOR);
            _menuButton.layoutData = menuButtonLayoutData;
            _menuButton.width = 35;
            _menuButton.height = 35;
            _menuButton.addEventListener(Event.TRIGGERED, mainMenuButtonHandler);
            _topMenuContainer.addChild(_menuButton);

            menuButtonLayoutData.left = _sideMargin;
            if(Device.isIphoneXVariant()){
                menuButtonLayoutData.left += 6;
            }

            var titleLayData:AnchorLayoutData = new AnchorLayoutData();
            titleLayData.verticalCenter = 0;
            titleLayData.leftAnchorDisplayObject = _menuButton;
            titleLayData.left = 7.5;
            title = new Label();
            title.layoutData = titleLayData;
            title.paddingTop = -3;
            title.textRendererFactory = GalleryLabelFactory.screenTitle;
            _topMenuContainer.addChild(title);
            if(Constants.isWideScreen){
                title.visible = false;
            }

            var messageLayData:AnchorLayoutData = new AnchorLayoutData();
            messageLayData.verticalCenter = 0;
            messageLayData.horizontalCenter = 0;
            _messageField = new Label();
            _messageField.layoutData = titleLayData;
            _messageField.textRendererFactory = GalleryLabelFactory.messageLabel;
            _messageField.text = "You don't have any challenges at this moment";
            _messageField.visible = false;
            addChild(_messageField);
        }

        public function updateNotifications() :void{
            if(_notificationImage){
                _notificationImage.visible = Constants.UNREAD_CHALLENGE_NOTIFICATIONS > 0;
                _notificationImage.x = _bottomMenuContainer.x + _notificationsBtn.x + _notificationsBtn.width - 3 * _notificationImage.width / 2;
                _notificationImage.y = _bottomMenuContainer.y + _notificationsBtn.y - _notificationImage.height / 2;
            }
        }

        private function mainMenuButtonHandler(event:Event):void {
            homeButtonTriggeredHandler();
        }

        override public function homeButtonTriggeredHandler(event:Event = null):void{
            dispatchEventWith(StackedScreen.HOME_SELECTED);
        }

        override protected function draw():void {
            super.draw();

            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);

            if (!mainMenuView) {
                mainMenuView = new MainMenuView();
                mainMenuView.y = _topButtonPadding;
                mainMenuView.name = 'mainMenuView';

                mainMenuView.width = MainMenuView.MENU_WIDTH;
                mainMenuView.height = actualHeight - _topButtonPadding;
                mainMenuView.parentActualWidth = actualWidth;
                mainMenuView.parentActualHeight = actualHeight;

                mainMenuView.addEventListener(MainMenuView.START_SHOW, menuOpenHandler);
                mainMenuView.addEventListener(Event.CLOSE, menuCloseHandler);
            }

            if (sizeInvalid) {
                if (mainMenuView) {
                    mainMenuView.y = _topButtonPadding;

                    mainMenuView.width = MainMenuView.MENU_WIDTH;
                    mainMenuView.height = actualHeight - _topButtonPadding;
                    mainMenuView.parentActualWidth = actualWidth;
                    mainMenuView.parentActualHeight = actualHeight - _topButtonPadding;
                }

                _topMenuContainer.width = actualWidth;
                _topMenuContainer.validate();

                _bottomMenuContainer.width = actualWidth;
                _bottomMenuContainer.height = marginBottom;
                _bottomMenuContainer.y = actualHeight - marginBottom;
                _bottomMenuContainer.validate();

                _backgroundQuad.width = actualWidth;
                _backgroundQuad.height = actualHeight - _topButtonPadding;
            }
            if(_userAvatar && (isInvalid(INVALIDATION_USER_AVATAR) || sizeInvalid)){
                _userAvatar.x = actualWidth - (_userAvatar.width / 2 + _sideMargin);
                _userAvatar.y = marginTop / 2 - 1;
                _ratingLabel.paddingRight = 30 + _sideMargin;
                _userAvatar.addEventListener(TouchEvent.TOUCH, avatarTouchHandler);
            }else if(!_userAvatar){
                _ratingLabel.paddingRight = _sideMargin;
            }
        }

        private function avatarTouchHandler(event:TouchEvent):void {
            if(event.getTouch(_userAvatar, TouchPhase.ENDED)){
                dispatchEventWith(USER_LEVEL_INFO_SELECTED);
            }
        }

        public static function get marginBottom():Number{
            return 32.5;
        }

        override public function ShowDone():void {
            if(onComplete == null){
                BannerStackedScreen.showAdBanner();
            }
            Tracker.trackEvent("Challenges Screen", "open");
        }

        private function menuOpenHandler(event:Event):void {
            BannerStackedScreen.hideAdBanner(false);
        }

        private function menuCloseHandler(event:Event):void {
            BannerStackedScreen.showAdBanner();
        }

        override protected function screen_removedFromStageHandler(event:Event):void{
            super.screen_removedFromStageHandler(event);
            BannerStackedScreen.hideAdBanner(true);
            if(Device.isIphoneXVariant()){
                Starling.current.nativeStage.removeEventListener(StageOrientationEvent.ORIENTATION_CHANGE, nativeStage_orientationChangeHandler);
            }
        }

        protected function nativeStage_orientationChangeHandler(event:StageOrientationEvent):void {

        }

        override public function dispose():void {
            super.dispose();

            DisplayObjectUtil.disposeMesh(_backgroundQuad);
            _backgroundQuad = null;
            if (_loadBar) {
                _loadBar.removeFromParent();
                _loadBar = null;
            }

            if (_topMenuContainer) {
                _topMenuContainer.removeFromParent(true);
                _topMenuContainer = null;
            }
            if (_bottomMenuContainer) {
                _bottomMenuContainer.removeFromParent(true);
                _bottomMenuContainer = null;
            }
            if (_menuButton) {
                _menuButton.removeFromParent(true);
                _menuButton = null;
            }
            if (title) {
                title.removeFromParent(true);
                title = null;
            }
            if (_messageField) {
                _messageField.removeFromParent(true);
                _messageField = null;
            }
            if (_toggleGroup) {
                _toggleGroup.removeAllItems();
                _toggleGroup = null;
            }
            if (_leaderboardBtn) {
                _leaderboardBtn.removeFromParent(true);
                _leaderboardBtn = null;
            }
            if (_homeBtn) {
                _homeBtn.removeFromParent(true);
                _homeBtn = null;
            }
            if (_votingBtn) {
                _votingBtn.removeFromParent(true);
                _votingBtn = null;
            }
            if (_userGarageBtn) {
                _userGarageBtn.removeFromParent(true);
                _userGarageBtn = null;
            }
            if (_notificationsBtn) {
                _notificationsBtn.removeFromParent(true);
                _notificationsBtn = null;
            }
            if (_notificationImage) {
                _notificationImage.removeFromParent(true);
                _notificationImage = null;
            }
        }

        //
        private static function createBottomToggle(textureId:String):ToggleButton{
            var button:ExtendedToggleButton = new ExtendedToggleButton();
            button.customHitArea = new Point(7, 15);
            var defaultIcon:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
            defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
            defaultIcon.color = 0x494949;
            button.defaultIcon = defaultIcon;
            var selectedIcon:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
            selectedIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
            selectedIcon.color = 0xFFFFFE;
            button.defaultSelectedIcon = selectedIcon;
            button.isQuickHitAreaEnabled = true;
            button.paddingLeft = button.paddingRight = 10;
            return button;
        }

        public function get onComplete():Function {
            return _onComplete;
        }

        public function set onComplete(value:Function):void {
            _onComplete = value;
        }
    }
}
