package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeScreenItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengesTuningItemRenderer;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.utils.FontSelector;
    import com.utils.ObjectUtils;

    import feathers.controls.Label;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.FeathersControl;
    import feathers.data.ListCollection;
    import feathers.layout.Direction;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.TiledRowsLayout;
    import feathers.layout.VerticalAlign;

    import flash.events.StageOrientationEvent;
    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import starling.events.Event;
    import starling.textures.Texture;

    public class ChallengesTuningsListScreen extends ChallengesScreen {
        public var challengeId:String;
        private var _tuningsList:ExtendedList;
        private var _messageLabel:Label;
        private var _limit:int = 12;
        private var _skip:int = 0;
        private var _isLoading:Boolean = false;
        private var _userBuild:ChallengesTuningItemRenderer;
        private var _layoutGap:Number = 0.75;
        private var _featureRules:Object;
        private var _challengeData:Object;
        public function ChallengesTuningsListScreen(challengeData:Object = null, featureRules:Object = null) {
            super("ChallengesTuningsListScreen");
            _challengeData = challengeData;
            this.challengeId = _challengeData._id;
            _featureRules = featureRules;
        }

        override protected function initialize():void {
            super.initialize();
            _toggleGroup.selectedItem = _leaderboardBtn;
            _tuningsList = new ExtendedList();

            var listLayout : TiledRowsLayout = new TiledRowsLayout();
            listLayout.gap = _layoutGap;
            listLayout.padding = 0;
            listLayout.useSquareTiles = false;
            listLayout.paging = Direction.HORIZONTAL;
            listLayout.requestedRowCount = 2;
            listLayout.requestedColumnCount = 3;
            listLayout.horizontalAlign = HorizontalAlign.CENTER;
            listLayout.verticalAlign = VerticalAlign.MIDDLE;
            listLayout.useVirtualLayout = true;
            _tuningsList.width = contentWidth;
            _tuningsList.height = contentHeight;

            _tuningsList.minimumPageThrowVelocity = 1;
            _tuningsList.snapToPages = true;

            _tuningsList.layout = listLayout;
            _tuningsList.verticalScrollPolicy = ScrollPolicy.OFF;
            _tuningsList.horizontalScrollPolicy = ScrollPolicy.ON;
            _tuningsList.addEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, challengeSelectedHandler);
            _tuningsList.addEventListener(Event.SCROLL, tuningListScrollHandler);
            if(Device.isIphoneXVariant()){
                updateOrientation();
            }
            addChild(_tuningsList);
            _tuningsList.setItemRendererFactoryWithID("regular", regularItemRenderer);
            _tuningsList.setItemRendererFactoryWithID("blank", blankItemRenderer);
            _tuningsList.factoryIDFunction = function(item:Object):String {
                if(item.hasOwnProperty("blank") && item.blank){
                    return "blank";
                }
                return "regular";
            };
            //
            dispatchEventWith(ChallengeEvent.REQUEST_CHALLENGE_TUNING_LIST);
        }

        private function regularItemRenderer():IListItemRenderer{
            var renderer:ChallengesTuningItemRenderer = new ChallengesTuningItemRenderer();
            renderer.owner = _tuningsList;
            renderer.setSize((contentWidth - 2 * _layoutGap)/3, (contentHeight - _layoutGap) / 2);
            return renderer;
        }

        private function blankItemRenderer():IListItemRenderer{
            var renderer:ListItemRenderer = new ListItemRenderer();
            renderer.owner = _tuningsList;
            renderer.setSize((contentWidth - 2 * _layoutGap)/3, (contentHeight - _layoutGap) / 2);
            return renderer;
        }

        private function tuningListScrollHandler(event:Event):void {
            if(!isLoading && !_tuningsList.isScrolling && _tuningsList.horizontalPageIndex >= _tuningsList.horizontalPageCount - 1){
                dispatchEventWith(ChallengeEvent.REQUEST_CHALLENGE_TUNING_LIST);
            }
            //
            if(Device.isIphoneXVariant()){
                if(_tuningsList.dataProvider){
                    _tuningsList.orientationUpdateCallback();
                }
            }
        }

        override protected function initTopMenuContainer():void{
            super .initTopMenuContainer();
            title.text = 'TOP DESIGNS';
        }

        private function challengeSelectedHandler(event:Event):void {
            var data:Object = ObjectUtils.clone(event.data);
            data.tunings = dataList;
            dispatchEvent(new Event(ChallengeEvent.TUNING_SELECTED, false, data));
        }

        private function get dataList():Array{
            if(_tuningsList && _tuningsList.dataProvider){
                var result:Array = [];
                for(var i:int = 0; i < _tuningsList.dataProvider.length; i++){
                    result.push(_tuningsList.dataProvider.getItemAt(i));
                }
                return result;
            }
            return [];
        }


        public function loadTunings(data:Array):void{
            if(_tuningsList){
                for(var i:int = 0; i < data.length; i++){
                    var itemData:Object = {stats: data[i].challengeNewSystem};
                    itemData.tuning = TuningsProxy.parseTuning(data[i], true);
                    itemData.featureRules = _featureRules;
                    itemData.stats.challengeHostId = _challengeData;
                    if(data[i].hasOwnProperty("timeTrialsPercent")){
                        itemData.stats["timeTrialsPercent"] = data[i]["timeTrialsPercent"];
                    }
                    if(data[i].hasOwnProperty("timeTrialsScore")){
                        itemData.stats["timeTrialsScore"] = data[i]["timeTrialsScore"];
                    }
                    if(data[i].hasOwnProperty("timeTrialsSeconds")){
                        itemData.stats["timeTrialsSeconds"] = data[i]["timeTrialsSeconds"];
                    }
                    data[i] = itemData;
                }
                if(_userBuild){
                    if(data.length > limit / 2){
                        data.insertAt(limit / 2, {blank:true});
                    }
                    if(data.length == limit + 1){
                        data.insertAt(data.length, {blank:true});
                    }
                }
                if(_tuningsList.dataProvider){
                    _tuningsList.dataProvider.addAll(new ListCollection(data));
                }else{
                    _tuningsList.dataProvider = new ListCollection(data);
                }
                _skip = _tuningsList.dataProvider.length;
                if(_userBuild){
                    _skip -= Math.floor(2 * _skip / (_limit + 2));
                }
            }
            if(_tuningsList && _tuningsList.dataProvider.length == 0){
                _messageLabel ||= LabelFactory.createSimpleLabel(FontSelector.getMontserratRegular(), 12, 0x424242, TextFormatAlign.LEFT, false);
                _messageLabel.text = "There are no submitted challenges at the moment.";
                addChild(_messageLabel);
                _messageLabel.validate();
                _messageLabel.x = (contentWidth - _messageLabel.width) / 2;
                _messageLabel.y = marginTop + (contentHeight - _messageLabel.height) / 2;
            }
        }

        override protected function draw():void {
            super.draw();
            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
            if (sizeInvalid) {
                _tuningsList.width = contentWidth;
                _tuningsList.x = 0;
                _tuningsList.y = marginTop;
            }
        }

        override public function dispose():void {
            super.dispose();
            if (_mainMenuView) {
                _mainMenuView.removeFromParent(true);
                _mainMenuView = null;
            }
            if (_tuningsList) {
                _tuningsList.removeFromParent(true);
                _tuningsList = null;
            }
            if (_messageLabel) {
                _messageLabel.removeFromParent(true);
                _messageLabel = null;
            }
        }

        override protected function nativeStage_orientationChangeHandler(event:StageOrientationEvent):void {
            super .nativeStage_orientationChangeHandler(event);
            updateOrientation();
        }

        private function updateOrientation():void{
            if(!stage || !_tuningsList){
                return;
            }
            _tuningsList.orientation = stage.starling.nativeStage.orientation;
        }

        public function get limit():int {
            return _limit;
        }

        public function get skip():int {
            return _skip;
        }

        public function set isLoading(isLoading:Boolean):void {
            _isLoading = isLoading;
        }

        public function get isLoading():Boolean {
            return _isLoading;
        }

        public function loadUserBuild(userBuild:Object):void {
            if(userBuild){
                _limit = 10;
                _userBuild = new ChallengesTuningItemRenderer();
                _userBuild.owner = _tuningsList;
                _userBuild.setSize(contentWidth / 3, contentHeight / 2);
                var itemData:Object = {stats: userBuild.challengeNewSystem};
                if(userBuild.hasOwnProperty("timeTrialsPercent")){
                    itemData.stats["timeTrialsPercent"] = userBuild["timeTrialsPercent"];
                }
                if(userBuild.hasOwnProperty("timeTrialsScore")){
                    itemData.stats["timeTrialsScore"] = userBuild["timeTrialsScore"];
                }
                if(userBuild.hasOwnProperty("timeTrialsSeconds")){
                    itemData.stats["timeTrialsSeconds"] = userBuild["timeTrialsSeconds"];
                }
                itemData.tuning = TuningsProxy.parseTuning(userBuild, true);
                itemData.featureRules = _featureRules;
                ObjectUtils.setValue(itemData, "stats.challengeHostId", _challengeData);
                addChild(_userBuild);
                _userBuild.x = _tuningsList.x + _tuningsList.width - _userBuild.width;
                _userBuild.y = _tuningsList.y + _tuningsList.height - _userBuild.height;
                _userBuild.data = itemData;


                var frameTxt:Texture = ButtonFactory.assetManager.getTexture("border_line");
                var frameImg:LayoutImage = new LayoutImage(frameTxt);
                frameImg.scale9Grid = new Rectangle(frameTxt.width/2 - 0.5, frameTxt.height / 2 - 0.5, 1, 1);
                frameImg.color = 0xfba500;
                frameImg.includeInLayout = false;
                frameImg.width = _userBuild.width;
                frameImg.height = _userBuild.height;
                _userBuild.addChild(frameImg);
            }
        }
    }
}
