package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeScreenItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengesUserTuningItemRenderer;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.utils.FontSelector;
    import com.utils.ObjectUtils;
    import com.utils.ObjectUtils;
    import com.utils.ObjectUtils;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.FeathersControl;
    import feathers.data.ListCollection;
    import feathers.layout.Direction;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.TiledRowsLayout;
    import feathers.layout.VerticalAlign;

    import flash.events.StageOrientationEvent;
    import flash.text.TextFormatAlign;

    import starling.events.Event;

    public class ChallengesUserTuningsScreen extends ChallengesScreen {
        private var _tuningsList:ExtendedList;
        public var garageOwner:User;
        private var _userGarage:Boolean;
        private var _closeButton:Button;
        private var _messageLabel:Label;
        private var _limit:int = 12;
        private var _skip:int = 0;
        private var _isLoading:Boolean = false;
        private var _challengeType:String;
        private var _bodyType:String;
        public function ChallengesUserTuningsScreen(userData:Object, userGarage:Boolean = false) {
            super("ChallengesTuningsListScreen");
            garageOwner = new User(userData);
            _userGarage = userGarage;
            if(userData.hasOwnProperty("bodyType")){
                _bodyType = userData.bodyType;
            }
            if(userData.hasOwnProperty("challengeType")){
                _challengeType = userData.challengeType;
            }else{
                _challengeType = "all";
            }
        }

        override protected function initialize():void {
            super.initialize();
            if(_userGarage){
                _toggleGroup.selectedItem = _userGarageBtn;
            }else{
                _toggleGroup.selectedItem = _leaderboardBtn;
            }
            _tuningsList = new ExtendedList();

            var listLayout : TiledRowsLayout = new TiledRowsLayout();
            listLayout.gap = 0.75;
            listLayout.padding = 0;
            listLayout.useSquareTiles = false;
            listLayout.paging = Direction.HORIZONTAL;
            listLayout.requestedRowCount = 2;
            listLayout.requestedColumnCount = 3;
            listLayout.horizontalAlign = HorizontalAlign.CENTER;
            listLayout.verticalAlign = VerticalAlign.MIDDLE;
            listLayout.useVirtualLayout = true;

            _tuningsList.width = contentWidth;
            _tuningsList.height = contentHeight;

            _tuningsList.minimumPageThrowVelocity = 1;
            _tuningsList.snapToPages = true;

            _tuningsList.layout = listLayout;
            _tuningsList.verticalScrollPolicy = ScrollPolicy.OFF;
            _tuningsList.horizontalScrollPolicy = ScrollPolicy.ON;
            _tuningsList.addEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, challengeSelectedHandler);
            _tuningsList.addEventListener(Event.SCROLL, tuningListScrollHandler);
            if(Device.isIphoneXVariant()){
                updateOrientation();
            }
            addChild(_tuningsList);
            _tuningsList.itemRendererFactory = function():IListItemRenderer{
                var renderer:ChallengesUserTuningItemRenderer = new ChallengesUserTuningItemRenderer();
                renderer.owner = _tuningsList;
                renderer.setSize((contentWidth - 2 * listLayout.gap)/3, (contentHeight - listLayout.gap) / 2);
                renderer.userGarage = userGarage;
                return renderer;
            }
            //
            dispatchEventWith(ChallengeEvent.REQUEST_CHALLENGE_TUNING_LIST, false, {bodyType:_bodyType, challengeType: _challengeType});
        }

        override protected function initTopMenuContainer():void{
            super .initTopMenuContainer();
            if(_userGarage){
                title.text = 'MY DESIGNS';
            }else{
                title.text = 'TOP TUNERS';
            }
        }

        private function tuningListScrollHandler(event:Event):void {
            if(!isLoading && !_tuningsList.isScrolling && _tuningsList.horizontalPageIndex >= _tuningsList.horizontalPageCount - 1){
                dispatchEventWith(ChallengeEvent.REQUEST_CHALLENGE_TUNING_LIST, false, {bodyType:_bodyType, challengeType: _challengeType});
            }
            if(Device.isIphoneXVariant()){
                if(_tuningsList.dataProvider){
                    _tuningsList.orientationUpdateCallback();
                }
            }
        }

        private function challengeSelectedHandler(event:Event):void {
            var data:Object = ObjectUtils.clone(event.data);
            data.user = garageOwner;
            data.tunings = dataList.concat();
            data.index = data.tunings.indexOf(event.data);
            if(!userGarage){
                var rating:Number = event.data.stats.rating;
                var isTimeTrial:Boolean = ObjectUtils.getValue(event.data.stats, "challengeHostId.timeTrials", false);
                if(rating == -1 && !isTimeTrial){
                    return;
                }
                for(var i:int = data.tunings.length - 1; i > -1; i--){
                    var obj:Object = data.tunings[i];
                    if(obj.stats.rating == -1){
                        data.tunings.removeAt(i);
                    }
                }
            }
            dispatchEvent(new Event(ChallengeEvent.TUNING_SELECTED, false, data));
        }

        private function get dataList():Array{
            if(_tuningsList && _tuningsList.dataProvider){
                var result:Array = [];
                for(var i:int = 0; i < _tuningsList.dataProvider.length; i++){
                    result.push(_tuningsList.dataProvider.getItemAt(i));
                }
                return result;
            }
            return [];
        }

        public function loadTunings(data:Array):void{
            var result:Array = [];
            if(_tuningsList){
                for(var i:int = 0; i < data.length; i++){
                    var itemData:Object = {stats: data[i].challengeNewSystem};
                    itemData.tuning = TuningsProxy.parseTuning(data[i], true);
                    itemData.stats.challengeHostId.timeTrials = ObjectUtils.getValue(data[i], "isTimeTrial", false);
                    itemData.stats.challengeHostId.timeTrialsPercent = ObjectUtils.getValue(data[i], "timeTrialsPercent", 0);
                    itemData.stats.challengeHostId.timeTrialsSeconds = ObjectUtils.getValue(data[i], "timeTrialsSeconds", 0);
                    itemData.stats.challengeHostId.timeTrialsScore = ObjectUtils.getValue(data[i], "timeTrialsScore", 0);
                    if(!_userGarage){
                        itemData.user = garageOwner;
                    }
                    result.push(itemData);
                }
                if(_tuningsList.dataProvider){
                    _tuningsList.dataProvider.addAll(new ListCollection(result));
                }else{
                    _tuningsList.dataProvider = new ListCollection(result);
                }
                _skip = _tuningsList.dataProvider.length;
            }
            if(!_tuningsList || !_tuningsList.dataProvider || _tuningsList.dataProvider.length == 0){
                _messageLabel ||= LabelFactory.createSimpleLabel(FontSelector.getMontserratRegular(), 14, 0x424242, TextFormatAlign.LEFT, false);
                if(_userGarage){
                    _messageLabel.text = "You don't have any submitted challenges at the moment.";
                }else{
                    _messageLabel.text = "This user doesn't have any submitted challenges at the moment.";
                }
                addChild(_messageLabel);
                _messageLabel.validate();
                _messageLabel.x = (contentWidth - _messageLabel.width) / 2;
                _messageLabel.y = marginTop + (contentHeight - _messageLabel.height) / 2;
            }else{
                if(_messageLabel){
                    _messageLabel.removeFromParent(true);
                    _messageLabel = null;
                }
            }
            if(onComplete != null){
                onComplete();
                onComplete = null;
            }
        }

        override protected function draw():void {
            super.draw();
            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
            if (sizeInvalid) {
                _tuningsList.width = contentWidth;
                _tuningsList.x = 0;
                _tuningsList.y = marginTop;

                if(_closeButton){
                    _closeButton.x = title.x + title.width;
                    _closeButton.y = (marginTop - _closeButton.height) / 2;
                }
            }
        }

        override public function dispose():void {
            super.dispose();
            if (_tuningsList) {
                _tuningsList.removeFromParent(true);
                _tuningsList = null;
            }
            if (_messageLabel) {
                _messageLabel.removeFromParent(true);
                _messageLabel = null;
            }
            garageOwner = null;
        }

        public function get userGarage():Boolean {
            return _userGarage;
        }

        override protected function nativeStage_orientationChangeHandler(event:StageOrientationEvent):void {
            super .nativeStage_orientationChangeHandler(event);
            updateOrientation();
        }

        protected function updateOrientation():void{
            if(!stage || !_tuningsList){
                return;
            }
            _tuningsList.orientation = stage.starling.nativeStage.orientation;
            if(_tuningsList.dataProvider){
                _tuningsList.orientationUpdateCallback();
            }
        }

        public function get limit():int {
            return _limit;
        }

        public function get skip():int {
            return _skip;
        }

        public function get isLoading():Boolean {
            return _isLoading;
        }

        public function set isLoading(value:Boolean):void {
            _isLoading = value;
        }
    }
}
