package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.model.challenges.ChallengeAlertEvent;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusInVoting;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeVoteCarView;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeVoteResultView;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengesTuningSideSelector;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.ExtentedButton;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.LayoutQuad;
    import com.despark.tuning3d.view.components.core.StackedBannerScreen;
    import com.despark.tuning3d.view.components.tunings.TuningItemRenderer;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.core.FeathersControl;
    import feathers.layout.FlowLayout;

    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.filters.ColorMatrixFilter;

    public class ChallengesVotingScreen extends ChallengesScreen {
        public var challengeId:String;
        private var _challenges:Array;

        private var _titleLabel:Label;
        private var _statusLabel:ExtendedLabel;
        private var _descriptionLabel:Label;
        private var _carTypeLabel:ExtendedLabel;
        private var _costLabel:Label;
        private var _costValueLabel:ExtendedLabel;
        private var _rewardLabel:Label;
        private var _rewardValueLabel:ExtendedLabel;
        private var _labelContainer:LayoutGroup;
        private var _voteButton:Button;
        private var _equalButton:Button;

        private var _data:Object;
        private var _challengeData:ChallengeData;
        private var _carView:ChallengeVoteCarView;
        private var _voteResultView:ChallengeVoteResultView;
        private var tuningOne:Tuning;
        private var tuningTwo:Tuning;

        private var tuningOneView:TuningItemRenderer;
        private var tuningTwoView:TuningItemRenderer;

        private var countLabel:Label;
        private var pairBackground:Quad;
        private var _lastVotedObject:Object;
        private var _challengeIndex:int;
        private var _votedTuningCount:int;

        private var _sideSelector:ChallengesTuningSideSelector;
        private var _rulesButton:Button;

        private static var SIDE_MENU_WIDTH:Number = 220;
        private static var SIDE_MENU_PADDING:Number = 23;
        public function ChallengesVotingScreen(challengeId:String = null) {
            super("ChallengesVotingScreen");
            this.challengeId = challengeId;
        }

        override protected function initialize():void {
            super.initialize();
            setSizeInternal(stage.stageWidth, stage.stageHeight, false);
            SIDE_MENU_WIDTH = stage.stageWidth * 0.36;

            _toggleGroup.selectedItem = _votingBtn;
            _labelContainer = new LayoutGroup();
            _labelContainer.backgroundSkin = new Quad(1,1,0xFFFFFF);
            var labelLayout:FlowLayout = new FlowLayout();
            labelLayout.paddingLeft = labelLayout.paddingRight = SIDE_MENU_PADDING;
            labelLayout.paddingTop = SIDE_MENU_PADDING - 5;
            labelLayout.paddingBottom = 45;
            labelLayout.verticalGap = 7;
            _labelContainer.layout = labelLayout;
            _labelContainer.width = SIDE_MENU_WIDTH;
            addChild(_labelContainer);

            _rulesButton = new ExtentedButton();
            _rulesButton.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getMontserratRegular(), 7.5, 0x535353, TextFormatAlign.LEFT, false);
            _rulesButton.label = "Rules";
            _rulesButton.includeInLayout = false;
            if(_rulesButton is ExtentedButton){
                ExtentedButton(_rulesButton).hitAreaPadding = 15;
            }
            _rulesButton.addEventListener(Event.TRIGGERED, infoTriggeredHandler);

            _statusLabel = new ChallengeStatusInVoting();
            _statusLabel.paddingLeft = 8;
            _statusLabel.paddingRight = 10;
            _statusLabel.paddingBottom = 15;
            _statusLabel.scale = 1.15;
            _statusLabel.text = "IN VOTING";
            _statusLabel.includeInLayout = true;

            var topBorder:LayoutQuad = new LayoutQuad(1, 1, 0x535353);
            topBorder.width = _labelContainer.width - 2 * SIDE_MENU_PADDING;
            topBorder.height = 0.5;

            _titleLabel = new Label();
            _titleLabel.width = _labelContainer.width - 2 * SIDE_MENU_PADDING;
            _titleLabel.wordWrap = true;
            _titleLabel.paddingTop = -8;
            _titleLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 21, 0x494949, TextFormatAlign.LEFT, true);

            _descriptionLabel = new Label();
            _descriptionLabel.width = _labelContainer.width - 2 * SIDE_MENU_PADDING;
            _descriptionLabel.wordWrap = true;
            _descriptionLabel.textRendererFactory = LabelFactory.challengeListDescLabel;
            _descriptionLabel.paddingBottom = 5;

            _carTypeLabel = new ExtendedLabel();
            _carTypeLabel.textRendererFactory = LabelFactory.challengeListLocLabel;
            _carTypeLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("care_btn"));
            _carTypeLabel.defaultIcon.width = _carTypeLabel.defaultIcon.height = 14;
            _carTypeLabel.labelOffsetX = 4;
            _carTypeLabel.paddingTop = -10;
            var colorFilter1:ColorMatrixFilter = new ColorMatrixFilter();
            colorFilter1.tint(0x535353, 1);
            _carTypeLabel.defaultIcon.filter = colorFilter1;

            _costLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x424242, TextFormatAlign.LEFT, false);
            _costLabel.text = "Cost";
            _costLabel.paddingRight = 3;
            _costValueLabel = new ExtendedLabel();
            _costValueLabel.textRendererFactory = LabelFactory.challengeListDescLabel;
            _costValueLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_key_icon"));
            _costValueLabel.defaultIcon.color = 0x535353;
            _costValueLabel.defaultIcon.width = _costValueLabel.defaultIcon.height = 12;
            _costValueLabel.labelOffsetX = 2;
            _costValueLabel.paddingRight = 0;

            _rewardLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x424242, TextFormatAlign.LEFT, false);
            _rewardLabel.text = "Earn up to";
            _rewardLabel.paddingLeft = 5;
            _rewardLabel.paddingRight = 3;
            _rewardValueLabel = new ExtendedLabel();
            _rewardValueLabel.textRendererFactory = LabelFactory.challengeListDescLabel;
            _rewardValueLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_earn_icon"));
            _rewardValueLabel.defaultIcon.color = 0x535353;
            _rewardValueLabel.defaultIcon.width = _costValueLabel.defaultIcon.height = 12;
            _rewardValueLabel.labelOffsetX = 2;

            var bottomBorder:LayoutQuad = new LayoutQuad(1, 1, 0x535353);
            bottomBorder.width = _labelContainer.width - 2 * SIDE_MENU_PADDING;
            bottomBorder.height = 0.5;

            _voteButton = ButtonFactory.defaultButton("VOTE", 0x267596);
            _voteButton.labelFactory = LabelFactory.challengeVoteBtnLabel;
            _voteButton.width = 75;
            _voteButton.height = 23;
            _voteButton.includeInLayout = false;
            _voteButton.addEventListener(Event.TRIGGERED, voteTriggeredHandler);

            _sideSelector = new ChallengesTuningSideSelector();
            _sideSelector.toggleGroup.addEventListener(Event.CHANGE, viewChangeHandler);
            _sideSelector.visible = false;

            _carView = new ChallengeVoteCarView();
            _carView.setSize(actualWidth, contentHeight);
            addChild(_carView);

            _labelContainer.addChild(_rulesButton);
            _labelContainer.addChild(_statusLabel);
            _labelContainer.addChild(topBorder);
            _labelContainer.addChild(_titleLabel);
            _labelContainer.addChild(_carTypeLabel);
            _labelContainer.addChild(_descriptionLabel);
            _labelContainer.addChild(bottomBorder);
            addChild(_labelContainer);
            addChild(_voteButton);

            addChild(_sideSelector);
        }


        override protected function initTopMenuContainer():void{
            super .initTopMenuContainer();
            title.text = 'VOTING';
        }

        private function infoTriggeredHandler(event:Event):void {
            var statsDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.HOW_TO_VOTE);
            addChild(statsDialog);
            statsDialog.open();
        }

        private function viewChangeHandler(event:Event):void {
            tuningOneView.addNextFrame();
            tuningTwoView.addNextFrame();

            tuningOneView.page = _sideSelector.toggleGroup.selectedIndex;
            tuningTwoView.page = _sideSelector.toggleGroup.selectedIndex;
        }

        private function voteTriggeredHandler(event:Event):void {
            if(UsersProxy.loggedInUser()){
                _voteButton.isEnabled = false;
            }
            dispatchEventWith(ChallengeEvent.LOAD_CHALLENGE_PAIR, false, _data);
        }

        override protected function draw():void {
            super.draw();
            var dataInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_DATA);
            if(dataInvalid){
                if(_data){
                    _data.carName = ChallengesListScreen.getCarName(_data.car);
                    _titleLabel.text = _data.name;
                    _descriptionLabel.text = _data.description;
                    _carTypeLabel.text = _data.carName;
                    _costValueLabel.text = _data.cost;
                    _rewardValueLabel.text = _data.reward;
                    _carView.data = _data;
                }
            }
            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
            if (sizeInvalid || dataInvalid) {
                _rewardLabel.width = Number.NaN;
                //
                _titleLabel.validate();
                _rulesButton.validate();
                _statusLabel.validate();
                _descriptionLabel.validate();
                _carTypeLabel.validate();
                _costLabel.validate();
                _costValueLabel.validate();
                _rewardLabel.validate();
                _rewardValueLabel.validate();

                if(_labelContainer){
                    _labelContainer.validate();
                    _labelContainer.x = actualWidth - (_labelContainer.width + SIDE_MENU_PADDING);
                    _labelContainer.y = marginTop + (contentHeight - _labelContainer.height) / 2;

                    _voteButton.x = _labelContainer.x + (_labelContainer.width - _voteButton.width) / 2;
                    _voteButton.y = _labelContainer.y + _labelContainer.height - (_voteButton.height + 11);
                }

                _rulesButton.x = _labelContainer.width - _rulesButton.width - 10;
                _rulesButton.y = 7;

                _carView.setSize(actualWidth, contentHeight);
                _carView.y = marginTop;

                var btnMargin:Number = 8;
                if(_sideSelector){
                    _sideSelector.x = actualWidth - (_sideSelector.width + btnMargin);
                    _sideSelector.y = contentHeight + marginTop - (_sideSelector.height + btnMargin);
                }
            }
        }

        override public function dispose():void {
            super.dispose();
            if (_titleLabel) {
                _titleLabel.removeFromParent(true);
                _titleLabel = null;
            }
            if (_statusLabel) {
                _statusLabel.removeFromParent(true);
                _statusLabel = null;
            }
            if (_descriptionLabel) {
                _descriptionLabel.removeFromParent(true);
                _descriptionLabel = null;
            }
            if (_carTypeLabel) {
                _carTypeLabel.removeFromParent(true);
                _carTypeLabel = null;
            }
            if (_costLabel) {
                _costLabel.removeFromParent(true);
                _costLabel = null;
            }
            if (_costValueLabel) {
                _costValueLabel.removeFromParent(true);
                _costValueLabel = null;
            }
            if (_rewardLabel) {
                _rewardLabel.removeFromParent(true);
                _rewardLabel = null;
            }
            if (_rewardValueLabel) {
                _rewardValueLabel.removeFromParent(true);
                _rewardValueLabel = null;
            }
            if (_labelContainer) {
                _labelContainer.removeFromParent(true);
                _labelContainer = null;
            }

            if (_voteButton) {
                _voteButton.removeFromParent(true);
                _voteButton = null;
            }

            if (_equalButton) {
                _equalButton.removeFromParent(true);
                _equalButton = null;
            }
            if (_carView) {
                _carView.removeFromParent(true);
                _carView = null;
            }
            if (_voteResultView) {
                _voteResultView.removeFromParent(true);
                _voteResultView = null;
            }

            if (_voteResultView) {
                _voteResultView.removeFromParent(true);
                _voteResultView = null;
            }

            if (tuningOneView) {
                tuningOneView.removeFromParent(true);
                tuningOneView = null;
            }

            if (tuningTwoView) {
                tuningTwoView.removeFromParent(true);
                tuningTwoView = null;
            }

            if (countLabel) {
                countLabel.removeFromParent(true);
                countLabel = null;
            }


            if (pairBackground) {
                pairBackground.removeFromParent(true);
                pairBackground = null;
            }

            if (_sideSelector) {
                _sideSelector.removeFromParent(true);
                _sideSelector = null;
            }

            if (_rulesButton) {
                _rulesButton.removeFromParent(true);
                _rulesButton = null;
            }
            challengeId = null;
            _challenges = null;
            _data = null;
            tuningOne = null;
            tuningTwo = null;
            _lastVotedObject = null;
        }

        public function loadChallenges(result:Object):void {
            if(result){
                _challenges = result as Array;
                if(_challenges && _challenges.length){
                    _challengeIndex = 0;
                    _votedTuningCount = 0;
                    _data = _challenges[_challengeIndex];
                    _challengeData = new ChallengeData(_data);
                    invalidate(FeathersControl.INVALIDATION_FLAG_DATA);
                }
            }
            if(onComplete != null){
                onComplete();
                onComplete = null;
            }
        }

        public function displayChallengePair(value:Object):void{
            removeVoteResultView();

            pairBackground ||= new Quad(1, 1, StackedBannerScreen.HEADER_BG_COLOR);
            pairBackground.width = contentWidth;
            pairBackground.height = contentHeight;
            pairBackground.y = marginTop;
            addChild(pairBackground);

            tuningOne = TuningsProxy.parseTuning(value.tuningOne, true);
            tuningTwo = TuningsProxy.parseTuning(value.tuningTwo, true);
            var isCustomFrame:Boolean = _challengeData.featuresRules.customAngle;

            var contentScale:Number = isCustomFrame? 1.15: 1.4;
            if(!Constants.isUltraWideScreen){
                contentScale *= Constants.TUNING_SCALE_FACTOR;
            }
            var leftOffset:Number = isCustomFrame? 0: 35;

            tuningOneView ||= new TuningItemRenderer();
            tuningOneView.maxFrameCount = 1;
            tuningOneView.topOffset = 0;
            tuningOneView.leftOffset = leftOffset;
            tuningOneView.contentScale = contentScale;
            tuningOneView.isSliding = false;
            tuningOneView.data = tuningOne;
            tuningOneView.setSize(contentWidth / 2 - 0.75, contentHeight);
            tuningOneView.y = marginTop;

            addChild(tuningOneView);
            tuningOneView.pageIndicator.visible = false;
            tuningOneView.fitContent = true;
            tuningOneView.addEventListener(TouchEvent.TOUCH, tuningTouchHandler);

            tuningTwoView ||= new TuningItemRenderer();
            tuningTwoView.maxFrameCount = 1;
            tuningTwoView.topOffset = 0;
            tuningTwoView.leftOffset = leftOffset;
            tuningTwoView.contentScale = contentScale;
            tuningTwoView.isSliding = false;
            tuningTwoView.data = tuningTwo;
            tuningTwoView.setSize(contentWidth / 2 - 0.75, contentHeight);
            tuningTwoView.x = contentWidth / 2 + 0.75;
            tuningTwoView.y = marginTop;
            addChild(tuningTwoView);
            tuningTwoView.pageIndicator.visible = false;
            tuningTwoView.fitContent = true;
            tuningTwoView.addEventListener(TouchEvent.TOUCH, tuningTouchHandler);

            if(countLabel == null){
                var textFormat:TextFormat = new TextFormat();
                textFormat.leading = 0;
                countLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansSemiBold(), 13, 0x4a4a4a, TextFormatAlign.CENTER, false, false, textFormat);
                var countLabelBg:Image = new Image(ButtonFactory.assetManager.getTexture("challenges_vote_count_bg"));
                countLabelBg.scale = ButtonFactory.assetManager.scaleFactor / 2;
                countLabel.backgroundSkin = countLabelBg;
                countLabel.setSize(countLabelBg.width + 2, countLabelBg.height);
                countLabel.paddingTop = -2;
                countLabel.paddingBottom = 2;
                countLabel.x = (actualWidth - countLabel.width) / 2;
                countLabel.y = marginTop;
                addChild(countLabel);
            }else{
                setChildIndex(countLabel, numChildren - 1);
            }

            _equalButton = ButtonFactory.simpleButtonByTextureName("vote_equal_icon", true);
            _equalButton.addEventListener(Event.TRIGGERED, equalTriggeredHandler);
            addChild(_equalButton);
            _equalButton.validate();
            _equalButton.x = (contentWidth - _equalButton.width) / 2;
            _equalButton.y = marginTop + (contentHeight - _equalButton.width) / 2;

            tuningOneView.visible = tuningTwoView.visible = true;
            pairBackground.visible = true;
            _equalButton.visible = _equalButton.isEnabled = true;
            countLabel.visible = true;

            countLabel.text = (_votedTuningCount + 1).toString() + "/5";

            tuningOneView.touchable = tuningTwoView.touchable = true;
            _sideSelector.visible = true;
            _sideSelector.toggleGroup.selectedIndex = 0;
            setChildIndex(_sideSelector, numChildren - 1);

            if(_carView){
                _carView.visible = false;
            }
        }

        private function equalTriggeredHandler(event:Event):void {
            _equalButton.isEnabled = false;
            var voteObj:Object = getVotingParams(true, true);
            if(voteObj){
                tuningOneView.touchable = tuningTwoView.touchable = false;
                _lastVotedObject = voteObj;
                dispatchEventWith(ChallengeEvent.VOTE_FOR_CHALLENGE, false, _lastVotedObject);
            }
        }

        private function getVotingParams(vote1:Boolean, vote2:Boolean):Object{
            return {tuning_one: {tuning_id: tuningOne.id, voted: vote1}, tuning_two: {tuning_id: tuningTwo.id, voted: vote2}, challenge_id: _data._id};
        }

        public function showVoteResult(value:Object):void{
            _voteResultView ||= new ChallengeVoteResultView();
            addChild(_voteResultView);
            _voteResultView.loadData(value, _lastVotedObject);
            _voteResultView.setSize(contentWidth, contentHeight + 2);
            _voteResultView.y = marginTop - 1;
            _voteResultView.x = 0;
            _equalButton.visible = false;
            _votedTuningCount ++;
            Starling.juggler.delayCall(function ():void {
                if(_votedTuningCount >= 5){
                    startNewChallengeVote();
                }else{
                    dispatchEventWith(ChallengeEvent.LOAD_CHALLENGE_PAIR, false, _data);
                }
            }, 2);
        }

        public function challengeVotingFailedHandler(messageCloseHandler:Function = null):void{
            var alertView:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.CHALLENGE_NOT_AVAILABLE, null, stage);
            alertView.addEventListener(ChallengeAlertEvent.CLOSE_TRIGGERED, function (event:Event):void {
                removeCurrentAndLoadNextChallenge();
                if(messageCloseHandler != null){
                    Starling.juggler.delayCall(messageCloseHandler, 0.25);
                }
            });
            alertView.open();
        }

        public function removeCurrentAndLoadNextChallenge():void{
            _challenges.removeAt(_challengeIndex);
            startNewChallengeVote(false, true);
        }

        public function startNewChallengeVote(loadNext:Boolean = true, skipVoting:Boolean = false):void{
            if(!skipVoting){
                dispatchEventWith(ChallengeEvent.ADD_CHALLENGE_KEY, false, _data);
            }
            if(loadNext){
                if(_challengeIndex < _challenges.length - 1){
                    _challengeIndex++;
                }else{
                    _challengeIndex = 0;
                }
            }

            if(!_challenges.length){
                dispatchEventWith(StackedBannerScreen.CHALLENGES_HOME_SELECTED);
                return;
            }

            removeVoteResultView();
            if(tuningOneView && tuningTwoView && pairBackground && countLabel){
                tuningOneView.visible = tuningTwoView.visible = false;
                pairBackground.visible = false;
                countLabel.visible = false;
            }


            _sideSelector.visible = false;
            _votedTuningCount = 0;
            _data = _challenges[_challengeIndex];
            _challengeData = new ChallengeData(_data);
            invalidate(FeathersControl.INVALIDATION_FLAG_DATA);

            if(_equalButton){
                _equalButton.visible = false;
            }
            _carView.visible = true;
            _labelContainer.visible = true;
            _voteButton.visible = _voteButton.isEnabled = true;
        }

        private function tuningTouchHandler(event:TouchEvent):void {
            if(!tuningOneView.touchable  || !tuningTwoView.touchable){
                return;
            }
            var tuning1Touch:Touch = event.getTouch(tuningOneView, TouchPhase.ENDED);
            var tuning2Touch:Touch = event.getTouch(tuningTwoView, TouchPhase.ENDED);
            var voteObj:Object;
            if(tuning1Touch && tuning2Touch){
                equalTriggeredHandler(null);
                event.stopImmediatePropagation();
                return;
            }else if(tuning1Touch){
                voteObj = getVotingParams(true, false);
            }else if(tuning2Touch){
                voteObj = getVotingParams(false, true);
            }
            if(voteObj){
                tuningOneView.touchable = tuningTwoView.touchable = false;
                _lastVotedObject = voteObj;
                dispatchEventWith(ChallengeEvent.VOTE_FOR_CHALLENGE, false, _lastVotedObject);
            }
        }

        private function removeVoteResultView():void{
            if(_voteResultView){
                Starling.juggler.tween(_voteResultView, 0.25, {alpha: 0, onComplete:function():void{
                        if(_voteResultView){
                            _voteResultView.removeFromParent(true);
                            _voteResultView = null;
                        }
                    }});
            }
        }

        public function get votedTuningCount():int {
            return _votedTuningCount;
        }
    }
}
