package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.LayoutQuad;
    import com.utils.DateParser;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;

    import flash.text.TextFormatAlign;

    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.events.Event;
    import starling.events.TouchEvent;
    import starling.filters.ColorMatrixFilter;

    public class ChallengeInfoView extends Sprite {
        private var _data:Object;
        private var titleLabel:Label;
        private var _timeLabel:ExtendedLabel;
        private var _carTypeLabel:ExtendedLabel;
        //
        private var _detailsLabel:Label;
        private var _descTitleLabel:Label;
        private var _descLabel:Label;

        private var _reqTitleLabel:Label;
        private var _reqLabel:Label;


        private var _costLabel:Label;
        private var _costValueLabel:ExtendedLabel;
        private var _earnLabel:Label;

        private var _rewardValue1Label:ExtendedLabel;
        private var _rewardValue2Label:ExtendedLabel;
        private var _rewardValue3Label:ExtendedLabel;

        private var _keyValue1Label:ExtendedLabel;
        private var _keyValue2Label:ExtendedLabel;
        private var _keyValue3Label:ExtendedLabel;

        private var _headerBorder:LayoutQuad;
        private var _bottomBorder:LayoutQuad;
        private var _earnBorder1:LayoutQuad;
        private var _earnBorder2:LayoutQuad;
        private var _enterButton:Button;
        //
        public static const ENTER_CHALLENGE:String = "EnterChallengePopup:enter_challenge";
        public static const ENTER_CHALLENGE_VOTING:String = "EnterChallengePopup:enter_challenge_voting";
        public static const ENTER_CHALLENGE_GALLERY:String = "EnterChallengePopup:enter_challenge_gallery";
        private var _challengeData:ChallengeData;
        private var _user:User;
        private var _initWidth:Number;
        private var _initHeight:Number;
        private var _background:Quad;
        public function ChallengeInfoView(data:Object, initWidth:Number, initHeight:Number) {
            super();
            _data = data;
            _user = _data.user;
            _challengeData = new ChallengeData(_data);
            _initWidth = initWidth;
            _initHeight = initHeight;
            initialize();
            draw();
        }

        private function initialize():void{
            _background = new Quad(1,1,0xFFFFFF);
            addChild(_background);
            titleLabel = new Label();
            titleLabel.textRendererFactory = LabelFactory.challengeListTitleLabel;
            addChild(titleLabel);
            titleLabel.text = _data.name;
            titleLabel.touchable = false;


            var currentDate:Date = new Date();
            var timeLabelBgColor:uint;
            var enterLabel:String;

            if(_challengeData.status == ChallengeData.IN_CHALLENGE){
                if(_timeLabel && !(_timeLabel is ChallengeStatusInChallenge)){
                    _timeLabel.removeFromParent(true);
                    _timeLabel = null;
                }
                if(!_timeLabel){
                    _timeLabel = new ChallengeStatusInChallenge(challengeData.isTimeTrial?ChallengeStatusInChallenge.TIME_TRIAL_COLOR:ChallengeStatusInChallenge.MAIN_COLOR);
                }else{
                    Quad(_timeLabel.backgroundSkin).color = challengeData.isTimeTrial?ChallengeStatusInChallenge.TIME_TRIAL_COLOR:ChallengeStatusInChallenge.MAIN_COLOR;
                }
                _timeLabel.text = DateParser.getInChallengeLabel( _challengeData.finishDate.time - currentDate.time);
                enterLabel = "ENTER CHALLENGE";
                timeLabelBgColor = 0x0da506;
            }else if(_challengeData.status == ChallengeData.IN_VOTING){
                if(_timeLabel && !(_timeLabel is ChallengeStatusInVoting)){
                    _timeLabel.removeFromParent(true);
                    _timeLabel = null;
                }
                if(!_timeLabel){
                    _timeLabel = new ChallengeStatusInVoting();
                }
                _timeLabel.text = DateParser.getInChallengeLabel( _challengeData.votingFinishDate.time - currentDate.time);
                enterLabel = "GO TO VOTING";
                timeLabelBgColor = 0x267596;
            }else if(_challengeData.status == ChallengeData.FINISHED){
                if(_timeLabel && !(_timeLabel is ChallengeStatusFinished)){
                    _timeLabel.removeFromParent(true);
                    _timeLabel = null;
                }
                if(!_timeLabel){
                    _timeLabel = new ChallengeStatusFinished();
                }
                _timeLabel.text = "FINISHED";
                enterLabel = "GO TO LEADERBOARD";
                timeLabelBgColor = 0x3c4142;
            }
            _timeLabel.paddingLeft = 5;
            _timeLabel.paddingRight = 5;
            addChild(_timeLabel);

            _carTypeLabel = new ExtendedLabel();
            _carTypeLabel.textRendererFactory = LabelFactory.challengeListLocLabel;
            _carTypeLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("care_btn"));
            _carTypeLabel.defaultIcon.width = _carTypeLabel.defaultIcon.height = 14;
            _carTypeLabel.labelOffsetX = 4;
            var colorFilter1:ColorMatrixFilter = new ColorMatrixFilter();
            colorFilter1.tint(0x535353, 1);
            _carTypeLabel.defaultIcon.filter = colorFilter1;
            _carTypeLabel.text = _data.carName;
            addChild(_carTypeLabel);


            _headerBorder = new LayoutQuad(1, 1, 0x686868);
            _headerBorder.height = 0.5;
            addChild(_headerBorder);

            _bottomBorder = new LayoutQuad(1, 1, 0x686868);
            _bottomBorder.height = 0.5;
            addChild(_bottomBorder);

            _detailsLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratSemiBold(), 9, 0x535353, TextFormatAlign.LEFT, false);
            addChild(_detailsLabel);
            _detailsLabel.text = "Details";


            _costLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x424242, TextFormatAlign.LEFT, false);
            _costLabel.text = "Cost";

            _costValueLabel = new ExtendedLabel();
            _costValueLabel.textRendererFactory = LabelFactory.challengeListDescLabel;
            _costValueLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_key_icon"));
            _costValueLabel.defaultIcon.color = 0x535353;
            _costValueLabel.defaultIcon.height = 9;
            _costValueLabel.defaultIcon.scale = _costValueLabel.defaultIcon.scaleY;
            _costValueLabel.labelOffsetX = 2;
            _costValueLabel.paddingLeft = 5;
            _costValueLabel.text = _data.cost;
            addChild(_costLabel);
            addChild(_costValueLabel);


            _earnLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x424242, TextFormatAlign.LEFT, false);
            _earnLabel.text = "Earn";
            _earnLabel.paddingRight = 4;
            addChild(_earnLabel);


            if(_challengeData.rewardRules && _challengeData.rewardRules[0] && _challengeData.rewardRules[0].hasOwnProperty("max") && _challengeData.rewardRules[0].hasOwnProperty("reward")){
                _keyValue1Label = createKeysLabel(_challengeData.rewardRules[0].max);
                addChild(_keyValue1Label);
                _keyValue1Label.validate();

                _rewardValue1Label = createRewardsLabel(_challengeData.rewardRules[0].reward);
                addChild(_rewardValue1Label);
                _rewardValue1Label.validate();
            }

            _earnBorder1 = new LayoutQuad(1, 1, 0x686868);
            _earnBorder1.width = 0.5;
            _earnBorder1.height = 12;
            addChild(_earnBorder1);

            if(_challengeData.rewardRules && _challengeData.rewardRules[1] && _challengeData.rewardRules[1].hasOwnProperty("max") && _challengeData.rewardRules[1].hasOwnProperty("reward")) {
                _keyValue2Label = createKeysLabel(_challengeData.rewardRules[1].max)
                addChild(_keyValue2Label);
                _keyValue2Label.validate();

                _rewardValue2Label = createRewardsLabel(_challengeData.rewardRules[1].reward);
                addChild(_rewardValue2Label);
                _rewardValue2Label.validate();
            }

            _earnBorder2 = new LayoutQuad(1, 1, 0x686868);
            _earnBorder2.width = 0.5;
            _earnBorder2.height = 12;
            addChild(_earnBorder2);

            if(_challengeData.rewardRules && _challengeData.rewardRules[2] && _challengeData.rewardRules[2].hasOwnProperty("max") && _challengeData.rewardRules[2].hasOwnProperty("reward")) {
                _keyValue3Label = createKeysLabel(_challengeData.rewardRules[2].max)
                addChild(_keyValue3Label);
                _keyValue3Label.validate();

                _rewardValue3Label = createRewardsLabel(_challengeData.rewardRules[2].reward);
                addChild(_rewardValue3Label);
                _rewardValue3Label.validate();
            }

            _costValueLabel.text = _data.cost;
            _descTitleLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratSemiBold(), 9, 0x535353, TextFormatAlign.LEFT, false);
            addChild(_descTitleLabel);
            _descTitleLabel.text = "Description";

            _descLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x535353, TextFormatAlign.LEFT, true);
            addChild(_descLabel);
            _descLabel.text = _challengeData.description;

            if(_challengeData.requirements){
                _reqTitleLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratSemiBold(), 9, 0x535353, TextFormatAlign.LEFT, false);
                addChild(_reqTitleLabel);
                _reqTitleLabel.text = "Requirements";

                _reqLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x535353, TextFormatAlign.LEFT, true);
                addChild(_reqLabel);
                _reqLabel.text = _challengeData.requirements;
            }


            // 0x046bcb
            _enterButton = ButtonFactory.defaultButton(enterLabel, timeLabelBgColor);
            _enterButton.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getMontserratSemiBold(), 12, 0xFFFFFF, TextFormatAlign.CENTER, false);
            _enterButton.width = 155;
            _enterButton.height = 30;
            _enterButton.addEventListener(Event.TRIGGERED, enterButtonTriggeredHandler);
            addChild(_enterButton);


            titleLabel.validate();
            _timeLabel.validate();
            _detailsLabel.validate();
            _carTypeLabel.validate();
            _earnLabel.validate();
            if(_rewardValue1Label){
                _rewardValue1Label.validate();
            }
            _costLabel.validate();
            _costValueLabel.validate();
            _descTitleLabel.validate();
            _descLabel.validate();

            if(_challengeData.requirements){
                _reqTitleLabel.validate();
                _reqLabel.validate();
            }

            addEventListener(TouchEvent.TOUCH, touchHandler);
        }

        private function touchHandler(event:TouchEvent):void {

        }

        private static function createRewardsLabel(label:String):ExtendedLabel{
            var result:ExtendedLabel = new ExtendedLabel();
            result.textRendererFactory = LabelFactory.challengeListDescLabel;
            result.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));

            result.defaultIcon.height = 8;
            result.defaultIcon.scale = result.defaultIcon.scaleY;
            result.labelOffsetX = 2;

            result.paddingTop = 1;
            result.labelOffsetY = -1;

            result.text = label;
            return result;
        }

        private static function createKeysLabel(label:String):ExtendedLabel{
            var result:ExtendedLabel = new ExtendedLabel();
            result.textRendererFactory = LabelFactory.challengeListDescLabel;
            result.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));

            result.defaultIcon.height = 9;
            result.defaultIcon.scale = result.defaultIcon.scaleY;
            result.labelOffsetX = 2;
            result.paddingRight = 2;

            result.paddingTop = -1;
            result.labelOffsetY = 1;

            result.text = parseFloat(label).toFixed(1) + " =";
            return result;
        }

        private function enterButtonTriggeredHandler(event:Event):void {
            if(_challengeData.status == ChallengeData.IN_CHALLENGE){
                dispatchEvent(new Event(ENTER_CHALLENGE, false, _data));
            }else if(_challengeData.status == ChallengeData.IN_VOTING){
                dispatchEvent(new Event(ENTER_CHALLENGE_VOTING, false, _data));
            }else if(_challengeData.status == ChallengeData.FINISHED){
                dispatchEvent(new Event(ENTER_CHALLENGE_GALLERY, false, _data));
            }
        }

        private function draw():void{
            //
            _background.width = _initWidth;
            _background.height = _initHeight;
            //
            var sMargin:Number = 18;
            var sx:Number = sMargin;
            var sy:Number = 4;
            var bGap:Number = 8;
            var sGap:Number = 4;

            titleLabel.x = sx;
            titleLabel.y = sy;

            _timeLabel.x = sx;
            _timeLabel.y = titleLabel.y + titleLabel.height + sGap + 5;

            _carTypeLabel.x = _timeLabel.x + _timeLabel.width + sGap + 6;
            _carTypeLabel.y = _timeLabel.y;

            _headerBorder.width = _initWidth - 2 * sMargin;
            _headerBorder.x = sx;
            _headerBorder.y = _carTypeLabel.y + _carTypeLabel.height + sGap + 3;

            _detailsLabel.x = sx;
            _detailsLabel.y = _headerBorder.y + sGap + 2;

            _costLabel.x = sx;
            _costLabel.y = _detailsLabel.y + _detailsLabel.height + sGap - 2;

            _costValueLabel.x = _costLabel.x + _costLabel.width;
            _costValueLabel.y = _costLabel.y;

            _earnLabel.x = sx;
            _earnLabel.y = _costLabel.y + _costLabel.height + sGap - 2;

            if(_keyValue1Label){
                _keyValue1Label.x = _earnLabel.x + _earnLabel.width;
                _keyValue1Label.y = _earnLabel.y;
            }

            if(_rewardValue1Label){
                _rewardValue1Label.x = _keyValue1Label.x + _keyValue1Label.width;
                _rewardValue1Label.y = _earnLabel.y;

                _earnBorder1.x = int(_rewardValue1Label.x + _rewardValue1Label.width + sGap);
                _earnBorder1.y = _rewardValue1Label.y + (_rewardValue1Label.height - _earnBorder1.height) / 2;
            }

            if(_keyValue2Label){
                _keyValue2Label.x = _earnBorder1.x + _earnBorder1.width + sGap;
                _keyValue2Label.y = _earnLabel.y;
            }

            if(_rewardValue2Label){
                _rewardValue2Label.x = _keyValue2Label.x + _keyValue2Label.width;
                _rewardValue2Label.y = _earnLabel.y;

                _earnBorder2.x = int(_rewardValue2Label.x + _rewardValue2Label.width + sGap);
                _earnBorder2.y = _rewardValue2Label.y + (_rewardValue2Label.height - _earnBorder2.height) / 2;
            }

            if(_keyValue3Label){
                _keyValue3Label.x = _earnBorder2.x + _earnBorder2.width + sGap;
                _keyValue3Label.y = _earnLabel.y;
            }

            if(_rewardValue3Label){
                _rewardValue3Label.x = _keyValue3Label.x + _keyValue3Label.width;
                _rewardValue3Label.y = _earnLabel.y;
            }



            _descTitleLabel.x = sx;
            if(_earnLabel){
                _descTitleLabel.y = _earnLabel.y + _earnLabel.height + sGap + 2;
            }

            _descLabel.width = _headerBorder.width;
            _descLabel.x = sx;
            _descLabel.y = _descTitleLabel.y + _descTitleLabel.height;
            _descLabel.validate();

            if(_challengeData.requirements){
                _reqTitleLabel.x = sx;
                _reqTitleLabel.y = _descLabel.y + _descLabel.height + sGap + 2;

                _reqLabel.width = _headerBorder.width;
                _reqLabel.x = sx;
                _reqLabel.y = _reqTitleLabel.y + _reqTitleLabel.height;
            }

            _enterButton.x = (_initWidth - _enterButton.width) / 2;
            _enterButton.y = _initWidth - (_enterButton.height + bGap);

            _bottomBorder.width = _initWidth - 2 * sMargin;
            _bottomBorder.x = sx;
            _bottomBorder.y = _enterButton.y - _bottomBorder.height - bGap;
        }

        override public function dispose():void{
            super .dispose();
            removeEventListener(TouchEvent.TOUCH, touchHandler);

            _data = null
            if(_timeLabel){
                _timeLabel.removeFromParent(true);
                _timeLabel = null;
            }
            if(_carTypeLabel){
                _carTypeLabel.removeFromParent(true);
                _carTypeLabel = null;
            }
            if(_detailsLabel){
                _detailsLabel.removeFromParent(true);
                _detailsLabel = null;
            }
            if(_descTitleLabel){
                _descTitleLabel.removeFromParent(true);
                _descTitleLabel = null;
            }
            if(_descLabel){
                _descLabel.removeFromParent(true);
                _descLabel = null;
            }
            if(_reqTitleLabel){
                _reqTitleLabel.removeFromParent(true);
                _reqTitleLabel = null;
            }
            if(_reqLabel){
                _reqLabel.removeFromParent(true);
                _reqLabel = null;
            }
            if(_costLabel){
                _costLabel.removeFromParent(true);
                _costLabel = null;
            }
            if(_costValueLabel){
                _costValueLabel.removeFromParent(true);
                _costValueLabel = null;
            }
            if(_earnLabel){
                _earnLabel.removeFromParent(true);
                _earnLabel = null;
            }
            if(_rewardValue1Label){
                _rewardValue1Label.removeFromParent(true);
                _rewardValue1Label = null;
            }
            if(_rewardValue2Label){
                _rewardValue2Label.removeFromParent(true);
                _rewardValue2Label = null;
            }
            if(_rewardValue3Label){
                _rewardValue3Label.removeFromParent(true);
                _rewardValue3Label = null;
            }
            if(_keyValue1Label){
                _keyValue1Label.removeFromParent(true);
                _keyValue1Label = null;
            }
            if(_keyValue2Label){
                _keyValue2Label.removeFromParent(true);
                _keyValue2Label = null;
            }
            if(_keyValue3Label){
                _keyValue3Label.removeFromParent(true);
                _keyValue3Label = null;
            }
            if(_headerBorder){
                _headerBorder.removeFromParent(true);
                _headerBorder = null;
            }
            if(_bottomBorder){
                _bottomBorder.removeFromParent(true);
                _bottomBorder = null;
            }
            if(_earnBorder1){
                _earnBorder1.removeFromParent(true);
                _earnBorder1 = null;
            }
            if(_earnBorder2){
                _earnBorder2.removeFromParent(true);
                _earnBorder2 = null;
            }
            if(_enterButton){
                _enterButton.removeFromParent(true);
                _enterButton = null;
            }
        }

        public function get user():User {
            return _user;
        }

        public function get challengeData():ChallengeData {
            return _challengeData;
        }
    }
}
