package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarBackground;
    import com.despark.tuning3d.model.cars.CarsParser;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.LayoutQuad;
    import com.utils.FontSelector;

    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.layout.FlowLayout;

    import flash.geom.Rectangle;

    import flash.text.TextFormat;

    import flash.text.TextFormatAlign;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.textures.Texture;
    import starling.textures.Texture;

    public class ChallengeItemRenderer extends ChallengeScreenItemRenderer {
    protected var _labelContainer:LayoutGroup;
    protected var _titleLabel:Label;
    protected var challengeImage:ImageLoaderWithStates;
    protected var actionLabel:Label;
    protected var background:LayoutQuad;
    protected var imageMask:LayoutQuad;
    private var _challengeData:ChallengeData;
    private var _isVotingImage:Boolean = false;
    private var _challengeId:String;
    public function ChallengeItemRenderer() {
        super ();
        imageMask = new LayoutQuad(1, 1, 0xFF0000);
        imageMask.includeInLayout = false;
        background = new LayoutQuad(1, 1, 0x626262);
    }

    override protected function draw():void {
        super.draw();
        var dataInvalid:Boolean = isInvalid(INVALIDATION_FLAG_DATA);
        var sizeInvalid:Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
        if(dataInvalid) {
            commitData();
            if(_titleLabel){
                _titleLabel.validate();
            }
        }
        if(sizeInvalid){
            updateImageLayout();
        }
        autoSizeIfNeeded();
    }

    protected function autoSizeIfNeeded():Boolean {
        var needsWidth:Boolean = isNaN(explicitWidth);
        var needsHeight:Boolean = isNaN(explicitHeight);
        if(!needsWidth || !needsHeight){
            return false;
        }
        var newWidth:Number = explicitWidth;
        var newHeight:Number = explicitHeight;
        if(isNaN(newWidth) || isNaN(newWidth)){
            return saveMeasurements(0, 0, 0, 0);
        }
        return saveMeasurements(newWidth, newHeight, newWidth, newHeight);
    }

        override public function set data(value:Object):void {
            super .data = value;
            if(_data){
                if(!_challengeData || _challengeData.id != _data._id){
                    _challengeData = new ChallengeData(_data);
                    if(!(_data.car is Car)){
                        _challengeData.car = (new CarsParser().parseObject(_data.car, "") as Car|| new Car());
                        _challengeData.car.isChallenge = true;
                    }
                }
            }else{
                _challengeData = null;
            }
        }

    protected function commitData():void{
        if(_data){
            if(_data._id == _challengeId){
                return;
            }
            _challengeId = _data._id;
            if(_titleLabel){
                _titleLabel.text = _challengeData.name;
            }

            if(challengeImage){
                challengeImage.removeFromParent(true);
            }

            challengeImage = new ImageLoaderWithStates();
            challengeImage.overrideDispose = false;
            var carBackground:CarBackground = new CarBackground();
            if(_data && _data.images){
                if(isVotingImage){
                    carBackground.mainUrl = _data.images.voting||"";
                }else{
                    carBackground.mainUrl = _data.images.challenge||"";
                }
            }else{
                carBackground.mainUrl = "";
            }

            carBackground.showThumbnailUrl = false;

            challengeImage.includeInLayout = false;
            challengeImage.persistentImageProvider = carBackground;
            challengeImage.completeHandler = backgroundCompleteHandler;
            challengeImage.alpha = 0;
            challengeImage.mask = imageMask;
            _labelContainer.addChildAt(challengeImage, _labelContainer.getChildIndex(background) + 1);
            updateImageLayout();
        }
    }

    private function backgroundCompleteHandler(loader:ImageLoaderWithStates):void{
        if(challengeImage.image){
            var bgRatio:Number = challengeImage.image.height / challengeImage.image.width;
            if(actualHeight/actualWidth > bgRatio){
                challengeImage.height = imageMask.height;
                challengeImage.width = imageMask.height / bgRatio;
            }else{
                challengeImage.width = actualWidth;
                challengeImage.height = actualWidth * bgRatio;
            }
            challengeImage.image.color = 0xFF0000;
            Starling.juggler.tween(challengeImage, .2, {alpha: 1});
            updateImageLayout();
        }
    }

    protected function updateImageLayout():void{
        if(challengeImage && challengeImage.image){
            challengeImage.x = imageMask.x + (imageMask.width - challengeImage.width) / 2;
            challengeImage.y = imageMask.y + (imageMask.height - challengeImage.height) / 2 - (isVotingImage? 20: 0);
        }

        if(actionLabel){
            actionLabel.validate();
            actionLabel.x = actualWidth - (actionLabel.width + 15);
            actionLabel.y = actualHeight - (actionLabel.height + 5);
        }
    }

    override protected function initialize():void{
        super .initialize();
        imageMask.width = actualWidth;
        imageMask.height = actualHeight;

        background.width = imageMask.width;
        background.height = imageMask.height;

        _labelContainer = new LayoutGroup();
        var labelLayout:FlowLayout = new FlowLayout();
        _labelContainer.layout = labelLayout;
        _titleLabel = new Label();

        _labelContainer.width = actualWidth;
        addChild(_labelContainer);

        _labelContainer.addChild(imageMask);
        _labelContainer.addChild(background);
        _labelContainer.addChild(_titleLabel);
        mask = imageMask;

        actionLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansSemiBold(), 16, 0xFFFFFF, TextFormatAlign.CENTER, false);
    }

        override public function dispose():void{
            super .dispose();
            if(_labelContainer){
                _labelContainer.removeFromParent(true);
                _labelContainer = null;
            }
            if(_titleLabel){
                _titleLabel.removeFromParent(true);
                _titleLabel = null;
            }
            if(challengeImage){
                challengeImage.removeFromParent();
                challengeImage.sepcialDispose();
                challengeImage = null;
            }
            if(actionLabel){
                actionLabel.removeFromParent(true);
                actionLabel = null;
            }
            if(background){
                background.removeFromParent(true);
                background = null;
            }

            if(imageMask){
                imageMask.removeFromParent(true);
                imageMask = null;
            }
            _challengeData = null;
        }

        protected function get isVotingImage():Boolean {
            return _isVotingImage;
        }

        protected function set isVotingImage(value:Boolean):void {
            _isVotingImage = value;
        }
    }
}