package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.utils.FontSelector;

    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.layout.FlowLayout;

    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import starling.display.Image;
    import starling.textures.Texture;

    public class ChallengeLeaderboardUserRenderer extends ChallengeScreenItemRenderer {
    protected var _labelContainer:LayoutGroup;
    protected var _titleLabel:Label;
    protected var _rankLabel:Label;
    private var _avatar:ChallengeUserAvatar;
    private var _user:User;
    private var _rating:String;
    private var _designRatingColor:uint = 0xfba500;
    public static const ITEM_SELECTED:String = 'ChallengeLeaderboardUserRenderer:itemSelected';
    public function ChallengeLeaderboardUserRenderer() {
        super ();
    }

    override protected function draw():void {
        super.draw();
        var dataInvalid:Boolean = isInvalid(INVALIDATION_FLAG_DATA);
        if(dataInvalid) {
            commitData();
            _titleLabel.width = actualWidth;
            _titleLabel.validate();
            //
            _rankLabel.x = _avatar.x;
            _rankLabel.y = _avatar.y;
        }
        autoSizeIfNeeded();
    }



    protected function autoSizeIfNeeded():Boolean {
        var needsWidth:Boolean = isNaN(explicitWidth);
        var needsHeight:Boolean = isNaN(explicitHeight);
        if(!needsWidth || !needsHeight){
            return false;
        }
        var newWidth:Number = explicitWidth;
        var newHeight:Number = explicitHeight;
        return saveMeasurements(newWidth, newHeight, newWidth, newHeight);
    }

    protected function commitData():void{
        if(_data){
            _user = new User(_data);
            _rating = _data.averageRating;
            if(_avatar){
                _avatar.removeFromParent(true);
            }
            _avatar = new ChallengeUserAvatar(_user, 50);
            _avatar.includeInLayout = true;
            _avatar.ratingLabelBackgroundColor = _designRatingColor;
            _labelContainer.addChildAt(_avatar, 0);

            _titleLabel.text = _user.nameToShow;
            _titleLabel.validate();

            if(index != -1){
                _rankLabel.text = (index + 1).toString();
            }
        }
    }

    override protected function initialize():void{
        super .initialize();
        _labelContainer = new LayoutGroup();
        _labelContainer.layout = new FlowLayout();
        _titleLabel = new Label();
        _titleLabel.paddingTop = 8;
        _titleLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getUnicode(), 7.5, 0x535353, TextFormatAlign.CENTER, true, false, null, false, true);

        _rankLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 7.5, 0x535353, TextFormatAlign.CENTER, false);
        _rankLabel.includeInLayout = false;
        var rankBgTxt:Texture = ButtonFactory.assetManager.getTexture("circle_button_bg");
        Image.bindScale9GridToTexture(rankBgTxt, new Rectangle(rankBgTxt.width/2 - 0.5, 0, 1, rankBgTxt.height));
        var rankBg:Image = new Image(rankBgTxt);
        rankBg.color = 0xFEFFFF;
        _rankLabel.paddingTop = 1;
        _rankLabel.paddingBottom = -1;
        _rankLabel.backgroundSkin = rankBg;
        _rankLabel.height = 14;
        _rankLabel.minWidth = 14;
        _rankLabel.width = 14;
        //

        _labelContainer.width = actualWidth;
        _labelContainer.addChild(_rankLabel);
        _labelContainer.addChild(_titleLabel);
        addChild(_labelContainer);
    }

    override public function dispose():void{
        super.dispose();

        if(_labelContainer){
            _labelContainer.removeFromParent(true);
            _labelContainer = null;
        }
        if(_titleLabel){
            _titleLabel.removeFromParent(true);
            _titleLabel = null;
        }
        if(_rankLabel){
            _rankLabel.removeFromParent(true);
            _rankLabel = null;
        }
        if(_avatar){
            _avatar.removeFromParent(true);
            _avatar = null;
        }
        _user = null;
    }

        public function get avatar():ChallengeUserAvatar {
            return _avatar;
        }

        public function set designRatingColor(value:uint):void {
            _designRatingColor = value;
        }
    }
}