package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarsParser;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.core.UIStrings;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.LayoutQuad;
    import com.greensock.TweenLite;
    import com.greensock.easing.Linear;
    import com.utils.DateParser;
    import com.utils.FontSelector;
    import com.utils.StringUtils;

    import feathers.controls.Label;
    import feathers.layout.AnchorLayout;

    import flash.display.StageOrientation;
    import flash.geom.Rectangle;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.animation.Transitions;

    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.display.Sprite3D;
    import starling.filters.ColorMatrixFilter;
    import starling.textures.Texture;

    public class ChallengeListItemRenderer extends ChallengeItemRenderer {
    private var _carTypeLabel:ExtendedLabel;

    private var _costLabel:Label;
    private var _costValueLabel:ExtendedLabel;
    private var _rewardLabel:Label;
    private var _rewardValueLabel:ExtendedLabel;

    private var _statusLabel:ChallengeStatusLabel;
    private var _status:String;

    private var borderLeft:LayoutQuad;
    private var borderRight:LayoutQuad;
    private var challengeId:String;

    private var _container:Sprite;
    private var _infoView:ChallengeInfoView;
    private var _carIcon:Image;
    public var titlePadding:Number = 15;

    public function ChallengeListItemRenderer() {
        super ();
    }

    override protected function draw():void {
        super.draw();
        var dataInvalid:Boolean = isInvalid(INVALIDATION_FLAG_DATA);
        if(dataInvalid) {
            if(owner is ExtendedList && ExtendedList(owner).orientationUpdate){
                updateOrientationDependantView();
                return;
            }
            commitData();
            //
            _rewardLabel.width = Number.NaN;
            //
            _titleLabel.validate();
            _carTypeLabel.validate();
            _costLabel.validate();
            _costValueLabel.validate();
            _rewardLabel.validate();
            _rewardValueLabel.validate();
            if(_statusLabel){
                _statusLabel.validate();
            }

            var textFormat:TextFormat = new TextFormat();
            if(_data && _data.carName){
                textFormat.leading = -5;
            }else{
                textFormat.leading = 7;
            }

            if(_statusLabel){
                _statusLabel.x = 0;
                _statusLabel.y = 10;

                if(_titleLabel){
                    _titleLabel.x = 0;
                    _titleLabel.y = _statusLabel.y + _statusLabel.height;
                    _titleLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.AlumniSansHtml, 21, 0xFFFFFF, TextFormatAlign.LEFT, false, true, textFormat);
                }
            }

            updateOrientationDependantView();

            if(borderLeft){
                borderLeft.width = 0.25;
                borderLeft.height = actualHeight;
                borderRight.width = 0.25;
                borderRight.height = actualHeight;
                borderRight.x = actualWidth - borderRight.width;
            }
        }
        autoSizeIfNeeded();
    }

    private function updateOrientationDependantView():void{
        var statusLeftPadding:Number = 9;
        if(owner is ExtendedList && ExtendedList(owner).orientation == StageOrientation.ROTATED_RIGHT && isFirstVisibleItem()){
            if(_statusLabel){
                _statusLabel.x = 0;
                _statusLabel.y = 10;
                _statusLabel.paddingLeft = titlePadding + _statusLabel.defaultPaddingLeft + statusLeftPadding - 20;
            }
            if(_titleLabel){
                _titleLabel.paddingLeft = titlePadding + 24;
                _titleLabel.x = 0;
                _titleLabel.y = _statusLabel.y + _statusLabel.height;
            }
        }else{
            if(_statusLabel){
                _statusLabel.x = 0;
                _statusLabel.y = 10;
                _statusLabel.paddingLeft = _statusLabel.defaultPaddingLeft + titlePadding + statusLeftPadding - 20;
            }
            if(_titleLabel){
                _titleLabel.paddingLeft = titlePadding;
                _titleLabel.x = 0;
                _titleLabel.y = _statusLabel.y + _statusLabel.height;
            }
        }

        if(_carIcon){
            _carIcon.x = _titleLabel.paddingLeft + 1;
            _carIcon.y = _titleLabel.y + 21 + (15 - _carIcon.height);
        }
    }

    override protected function commitData():void{
        super.commitData();
        if(_data){
            if(_data._id == challengeId){
                return;
            }
            //
            if(_infoView){
                _infoView.removeFromParent(true);
            }
            /*if(_container){
                _container.pivotX = 0;
                _container.x = 0;
                _container.skewY = 0;
                _container.width = actualWidth;
                _container.height = actualHeight;
                _container.visible = true;

                imageMask.width = actualWidth;
                imageMask.height = actualHeight;
                _container.mask = imageMask;
                mask = null;
            }*/

            challengeId = _data._id;
            //
            var challengeData:ChallengeData = new ChallengeData(_data);
            _data.carName = ChallengeData.initCarName(_data.carName);
            _titleLabel.width = NaN;
            _titleLabel.text = "<font>" + StringUtils.truncate(_data.name,  27) + "</font>";
            if(challengeData.isTimeTrial){
                if(!_carIcon || _carIcon.texture != ButtonFactory.assetManager.getTexture("challenge_time_trails")){
                    if(_carIcon){
                        _carIcon.removeFromParent(true);
                    }
                    _carIcon = new Image(ButtonFactory.assetManager.getTexture("challenge_time_trails"));
                }
                _carIcon.color = 0xFEFFFF;
                _carIcon.height = 13;
                _carIcon.scale = _carIcon.scaleY;
                addChild(_carIcon);
                _titleLabel.text = _titleLabel.text + "\n<textformat blockindent='18'><font size='12'>TIME TRIALS</font></textformat>";
            }else if(_data.carName && _data.carName.length){
                if(!_carIcon || _carIcon.texture != ButtonFactory.assetManager.getTexture("care_btn")){
                    if(_carIcon){
                        _carIcon.removeFromParent(true);
                    }
                    _carIcon = new Image(ButtonFactory.assetManager.getTexture("care_btn"));
                }
                _carIcon.color = 0xFEFFFF;
                _carIcon.height = 15;
                _carIcon.scale = _carIcon.scaleY;
                addChild(_carIcon);
                _titleLabel.text = _titleLabel.text + "\n<textformat blockindent='20'><font size='12'>" + _data.carName + "</font></textformat>";
            }else{
                if(_carIcon){
                    _carIcon.removeFromParent(true);
                    _carIcon = null;
                }
            }
            _titleLabel.height = 40;
            _carTypeLabel.text = _data.carName;
            _costValueLabel.text = _data.cost;
            if(_data.rewardRules && _data.rewardRules.length){
                _rewardValueLabel.text = _data.rewardRules[_data.rewardRules.length - 1].reward;
            }else if(_data.reward) {
                _rewardValueLabel.text = _data.reward;
            }

            if(!(_data.car is Car)){
                _data.car = (new CarsParser().parseObject(_data.car, "") as Car||new Car());
                _data.car.model = ChallengeData.initCarName(_data.car.model);
                _data.car.isChallenge = true;
            }

            var currentDate:Date = new Date();
            var finishDate:Date = DateParser.parseDate(_data.finishDate);
            var votingStartDate:Date = DateParser.parseDate(_data.votingStartDate);
            var votingFinishDate:Date = DateParser.parseDate(_data.votingFinishDate);
            //
            if(finishDate.time > currentDate.time){
                if(_status != ChallengeData.IN_CHALLENGE){
                    _status = ChallengeData.IN_CHALLENGE;
                }
                _costLabel.visible = _costValueLabel.visible = _rewardLabel.visible = _rewardValueLabel.visible = true;
            }else if(votingStartDate.time < currentDate.time && votingFinishDate.time > currentDate.time){
                if(_status != ChallengeData.IN_VOTING){
                    _status = ChallengeData.IN_VOTING;
                }
                _costLabel.visible = _costValueLabel.visible = _rewardLabel.visible = _rewardValueLabel.visible = false;
            }else{
                if(_status != ChallengeData.FINISHED){
                    _status = ChallengeData.FINISHED;
                }
                _costLabel.visible = _costValueLabel.visible = _rewardLabel.visible = _rewardValueLabel.visible = false;
            }
            var statusBackground:Image;
            switch(_status){
                 case ChallengeData.IN_CHALLENGE:
                     statusBackground = new Image(ButtonFactory.assetManager.getTexture("time_label_in_challenge"));
                     if(_statusLabel && !(_statusLabel is ChallengeStatusInChallenge)){
                         _statusLabel.removeFromParent(true);
                         _statusLabel = null;
                     }
                         var statusColor:uint;
                         if(challengeData.isTimeTrial){
                             statusColor = ChallengeStatusInChallenge.TIME_TRIAL_COLOR;
                         }else if(challengeData.isFromGarage){
                             statusColor = ChallengeStatusInChallenge.FROM_GARAGE_COLOR;
                         }else{
                             statusColor = ChallengeStatusInChallenge.MAIN_COLOR;
                         }
                     if(!_statusLabel){
                         _statusLabel = new ChallengeStatusInChallenge(statusColor);
                     }else{
                         Quad(_statusLabel.backgroundSkin).color = statusColor;
                     }
                     _statusLabel.text = "ACTIVE " + DateParser.getInChallengeLabel(finishDate.time - currentDate.time);
                     break;
                 case ChallengeData.IN_VOTING:
                     if(_statusLabel && !(_statusLabel is ChallengeStatusInVoting)){
                         _statusLabel.removeFromParent(true);
                         _statusLabel = null;
                     }
                     if(!_statusLabel){
                         _statusLabel = new ChallengeStatusInVoting();
                     }
                     _statusLabel.text = "IN VOTING " + DateParser.getInChallengeLabel(votingFinishDate.time - currentDate.time);
                     break;
                 case ChallengeData.FINISHED:
                     if(_statusLabel && !(_statusLabel is ChallengeStatusFinished)){
                         _statusLabel.removeFromParent(true);
                         _statusLabel = null;
                     }
                     if(!_statusLabel){
                         _statusLabel = new ChallengeStatusFinished();
                     }
                     _statusLabel.text = "FINISHED";
                     break;
             }

            if(_statusLabel){
                _statusLabel.includeInLayout = false;
                _statusLabel.paddingRight = 10;
                _statusLabel.scale = 1.15;
                _labelContainer.addChild(_statusLabel);
            }
            if(_status == ChallengeData.IN_CHALLENGE){
                actionLabel.text = "ENTER";
            }else if(_status == ChallengeData.IN_VOTING){
                actionLabel.text = "DETAILS";
            }else if(_status == ChallengeData.FINISHED){
                actionLabel.text = "DETAILS";
            }
            actionLabel.validate();
        }
    }


    override protected function updateImageLayout():void{
        super .updateImageLayout();
        actionLabel.x = (actualWidth - actionLabel.width) / 2;
        actionLabel.y = actualHeight - actionLabel.height - 10;
        //
        _costLabel.visible = _costValueLabel.visible = _rewardLabel.visible = _rewardValueLabel.visible = (_status == ChallengeData.IN_CHALLENGE);
        if(_status == ChallengeData.IN_CHALLENGE){
            _costLabel.x = 10;
            _costLabel.y = actionLabel.y + (actionLabel.height - _costLabel.height) / 2;
            _costValueLabel.x = _costLabel.x + _costLabel.width;
            _costValueLabel.y = _costLabel.y + 1;

            _rewardValueLabel.x = actualWidth - (_rewardValueLabel.width + 10);
            _rewardValueLabel.y = actionLabel.y + (actionLabel.height - _rewardLabel.height) / 2 + 1;
            _rewardLabel.x = _rewardValueLabel.x - _rewardLabel.width;
            _rewardLabel.y = _rewardValueLabel.y - 1;
        }
    }

    override protected function initialize():void{
        super .initialize();
        background.color = 0x424242;
        var badgeTxt:Texture = ButtonFactory.assetManager.getTexture("button_border");
        Image.bindScale9GridToTexture(badgeTxt, new Rectangle(badgeTxt.width/3, 0, badgeTxt.width/3, badgeTxt.height));
        var badgeBg:Image = new Image(badgeTxt);
        badgeBg.color = 0xFFFFFE;

        actionLabel.backgroundSkin = badgeBg;
        actionLabel.includeInLayout = false;
        actionLabel.height = 18;
        actionLabel.paddingBottom = 1.25;
        actionLabel.paddingTop = -1.25;
        actionLabel.paddingLeft = actionLabel.paddingRight = 8;

        _labelContainer.addChild(actionLabel);
        if(_titleLabel){
            _titleLabel.removeFromParent(true);
        }

        var textFormat:TextFormat = new TextFormat();
        textFormat.leading = -5;
        _titleLabel = LabelFactory.createSimpleLabel(FontSelector.AlumniSansHtml, 20, 0xFFFFFF, TextFormatAlign.LEFT, false, true, textFormat);
        _titleLabel.minWidth = actualWidth * 9 / 10;
        _titleLabel.maxWidth = actualWidth * 9.5 / 10;
        _titleLabel.backgroundSkin = new Quad(1, 1, 0x000000);
        _titleLabel.backgroundSkin.alpha = 0.65;
        _titleLabel.paddingLeft = _titleLabel.paddingRight = 14;
        _titleLabel.paddingBottom = 3;
        _titleLabel.includeInLayout = false;

        _carTypeLabel = new ExtendedLabel();
        _carTypeLabel.textRendererFactory = LabelFactory.challengeListLocLabel;
        _carTypeLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("care_btn"));
        _carTypeLabel.defaultIcon.width = _carTypeLabel.defaultIcon.height = 14;
        _carTypeLabel.labelOffsetX = 4;
        var colorFilter1:ColorMatrixFilter = new ColorMatrixFilter();
        colorFilter1.tint(0x535353, 1);
        _carTypeLabel.defaultIcon.filter = colorFilter1;

        _costLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0xFFFFFF, TextFormatAlign.LEFT, false);
        _costLabel.text = "Cost";
        _costLabel.paddingRight = 3;
        _costLabel.paddingLeft = -0.5;
        _costLabel.includeInLayout = false;

        _costValueLabel = new ExtendedLabel();
        _costValueLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getMontserratBold(), 9, 0xFFFFFF, TextFormatAlign.LEFT, false);
        _costValueLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_key_icon"));
        _costValueLabel.defaultIcon.color = 0x2bc703;
        _costValueLabel.defaultIcon.width = _costValueLabel.defaultIcon.height = 10;
        _costValueLabel.labelOffsetX = 2;
        _costValueLabel.includeInLayout = false;
        _costValueLabel.paddingRight = 4;
        _costValueLabel.iconOffsetY = -4;

        _costValueLabel.labelYOffset = -1;
        _costValueLabel.paddingTop = -1;

        var border:LayoutQuad = new LayoutQuad(1, 1, 0x535353);
        border.width = 0.75;
        border.height = 10;

        _rewardLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0xFFFFFF, TextFormatAlign.LEFT, false);
        _rewardLabel.text = "Earn*";
        _rewardLabel.paddingLeft = 4;
        _rewardLabel.paddingRight = 3;
        _rewardLabel.includeInLayout = false;

        _rewardValueLabel = new ExtendedLabel();
        _rewardValueLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getMontserratBold(), 9, 0xFFFFFF, TextFormatAlign.LEFT, false);;
        _rewardValueLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_earn_icon"));
        _rewardValueLabel.defaultIcon.color = 0x14ade3;
        _rewardValueLabel.defaultIcon.width = _rewardValueLabel.defaultIcon.height = 10;
        _rewardValueLabel.labelOffsetX = 2;
        _rewardValueLabel.labelYOffset = -1;
        _rewardValueLabel.paddingTop = -1;
        _rewardValueLabel.includeInLayout = false;

        _labelContainer.width = actualWidth;
        _labelContainer.height = actualHeight;
        _labelContainer.addChild(_titleLabel);
        _labelContainer.addChild(_carTypeLabel);
        _labelContainer.addChild(_costLabel);
        _labelContainer.addChild(_costValueLabel);
        _labelContainer.addChild(border);
        _labelContainer.addChild(_rewardLabel);
        _labelContainer.addChild(_rewardValueLabel);
        borderLeft = new LayoutQuad(1,1,0xFFFFFF);

        borderLeft.includeInLayout = false;
        borderRight = new LayoutQuad(1,1,0xFFFFFF);
        borderRight.includeInLayout = false;

        _container = new Sprite();
        _container.addChild(_labelContainer);
        _container.addChild(borderLeft);
        _container.addChild(borderRight);
        addChild(_container);
        /*if(imageMask){
            _container.mask = imageMask;
            mask = null;
        }*/
    }

    public function openInfo(info:Object):void{
        if(_infoView){
            _infoView.removeFromParent(true);
        }
        _container.width = actualWidth;
        _container.pivotX = actualWidth / 2;
        _container.x = _container.pivotX;
        _container.visible = true;

        var duration:Number = 0.25;
        _infoView = new ChallengeInfoView(info, actualWidth, actualHeight);
        _infoView.pivotX = actualWidth / 2;
        _infoView.x = _infoView.pivotX;
        addChild(_infoView);
        _infoView.visible = false;
        TweenLite.to(_container, duration, {skewY: Math.PI / 6, width: 0, ease: Linear.easeIn, onComplete:function():void{
                _infoView.skewY = -Math.PI / 6;
                _infoView.width = 0;
                _infoView.visible = true;
                _container.visible = false;
                TweenLite.to(_infoView, duration, {skewY: 0, width: actualWidth, ease: Linear.easeOut});
            }});
    }

    override public function dispose():void{
        super .dispose();
        if(_carTypeLabel){
            _carTypeLabel.removeFromParent(true);
            _carTypeLabel = null;
        }

        if(_costLabel){
            _costLabel.removeFromParent(true);
            _costLabel = null;
        }
        if(_costValueLabel){
            _costValueLabel.removeFromParent(true);
            _costValueLabel = null;
        }
        if(_rewardLabel){
            _rewardLabel.removeFromParent(true);
            _rewardLabel = null;
        }
        if(_rewardValueLabel){
            _rewardValueLabel.removeFromParent(true);
            _rewardValueLabel = null;
        }
        if(_statusLabel){
            _statusLabel.removeFromParent(true);
            _statusLabel = null;
        }
        if(borderLeft){
            borderLeft.removeFromParent(true);
            borderLeft = null;
        }
        if(borderRight){
            borderRight.removeFromParent(true);
            borderRight = null;
        }
        if(_container) {
            _container.removeFromParent(true);
            _container = null;
        }
    }
}
}