package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarsParser;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.LayoutQuad;
    import com.greensock.TweenLite;
    import com.greensock.easing.Linear;
    import com.utils.DateParser;
    import com.utils.FontSelector;

    import feathers.controls.Label;
    import feathers.layout.AnchorLayout;

    import flash.display.StageOrientation;
    import flash.geom.Rectangle;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.animation.Transitions;

    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.display.Sprite3D;
    import starling.filters.ColorMatrixFilter;
    import starling.textures.Texture;

    public class ChallengeModelsListItemRenderer extends ChallengeItemRenderer {
    private var _carTypeLabel:ExtendedLabel;


    private var borderLeft:LayoutQuad;
    private var borderRight:LayoutQuad;
    private var challengeId:String;

    private var _container:Sprite;
    private var _infoView:ChallengeInfoView;
    public var titlePadding:Number = 15;

    public function ChallengeModelsListItemRenderer() {
        super ();
    }

    override protected function draw():void {
        super.draw();
        var dataInvalid:Boolean = isInvalid(INVALIDATION_FLAG_DATA);
        if(dataInvalid) {
            if(owner is ExtendedList && ExtendedList(owner).orientationUpdate){
                updateOrientationDependantView();
                return;
            }
            commitData();
            //
            _titleLabel.validate();
            _carTypeLabel.validate();
            var textFormat:TextFormat = new TextFormat();
            textFormat.leading = -7;
            if(_titleLabel){
                _titleLabel.x = 0;
                _titleLabel.y = 10;
                _titleLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 21, 0xFFFFFF, TextFormatAlign.LEFT, false, true, textFormat);
            }

            updateOrientationDependantView();

            if(borderLeft){
                borderLeft.width = 0.25;
                borderLeft.height = actualHeight;
                borderRight.width = 0.25;
                borderRight.height = actualHeight;
                borderRight.x = actualWidth - borderRight.width;
            }
        }
        autoSizeIfNeeded();
    }

    private function updateOrientationDependantView():void{
        if(owner is ExtendedList && ExtendedList(owner).orientation == StageOrientation.ROTATED_RIGHT && isFirstVisibleItem()){
            if(_titleLabel){
                _titleLabel.paddingLeft = titlePadding + 24;
                _titleLabel.x = 0;
                _titleLabel.y = 10;
            }
        }else{
            if(_titleLabel){
                _titleLabel.paddingLeft = titlePadding;
                _titleLabel.x = 0;
                _titleLabel.y = 10;
            }
        }
    }

    override protected function commitData():void{
        super.commitData();
        if(_data){
            if(_data._id == challengeId){
                return;
            }
            //
            if(_infoView){
                _infoView.removeFromParent(true);
            }

            challengeId = _data._id;
            //
            _data.carName = ChallengeData.initCarName(_data.carName);
            _titleLabel.text = '<font>' + _data.carName + '</font>';
            _carTypeLabel.text = _data.carName;
            if(!(_data.car is Car)){
                _data.car = (new CarsParser().parseObject(_data.car, "") as Car||new Car());
                _data.car.model = ChallengeData.initCarName(_data.car.model);
                _data.car.isChallenge = true;
            }

            actionLabel.text = "SELECT";
            actionLabel.validate();
        }
    }


    override protected function updateImageLayout():void{
        super .updateImageLayout();
        actionLabel.x = (actualWidth - actionLabel.width) / 2;
        actionLabel.y = actualHeight - actionLabel.height - 10;
    }

    override protected function initialize():void{
        super .initialize();
        background.color = 0x424242;
        var badgeTxt:Texture = ButtonFactory.assetManager.getTexture("button_border");
        Image.bindScale9GridToTexture(badgeTxt, new Rectangle(badgeTxt.width/3, 0, badgeTxt.width/3, badgeTxt.height));
        var badgeBg:Image = new Image(badgeTxt);
        badgeBg.color = 0xFFFFFE;

        actionLabel.backgroundSkin = badgeBg;
        actionLabel.includeInLayout = false;
        actionLabel.height = 18;
        actionLabel.paddingBottom = 1.25;
        actionLabel.paddingTop = -1.25;
        actionLabel.paddingLeft = actionLabel.paddingRight = 8;

        _labelContainer.addChild(actionLabel);
        if(_titleLabel){
            _titleLabel.removeFromParent(true);
        }

        _titleLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansSemiBold(), 20, 0xFFFFFF, TextFormatAlign.LEFT, false, true);
        _titleLabel.minWidth = actualWidth * 9 / 10;
        _titleLabel.maxWidth = actualWidth * 9.5 / 10;
        _titleLabel.backgroundSkin = new Quad(1, 1, 0x000000);
        _titleLabel.backgroundSkin.alpha = 0.65;
        _titleLabel.paddingLeft = _titleLabel.paddingRight = 14;
        _titleLabel.paddingBottom = 8;
        _titleLabel.includeInLayout = false;

        _carTypeLabel = new ExtendedLabel();
        _carTypeLabel.textRendererFactory = LabelFactory.challengeListLocLabel;
        _carTypeLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("care_btn"));
        _carTypeLabel.defaultIcon.width = _carTypeLabel.defaultIcon.height = 14;
        _carTypeLabel.labelOffsetX = 4;
        var colorFilter1:ColorMatrixFilter = new ColorMatrixFilter();
        colorFilter1.tint(0x535353, 1);
        _carTypeLabel.defaultIcon.filter = colorFilter1;

        _labelContainer.width = actualWidth;
        _labelContainer.height = actualHeight;
        _labelContainer.addChild(_titleLabel);
        _labelContainer.addChild(_carTypeLabel);
        borderLeft = new LayoutQuad(1,1,0xFFFFFF);

        borderLeft.includeInLayout = false;
        borderRight = new LayoutQuad(1,1,0xFFFFFF);
        borderRight.includeInLayout = false;

        _container = new Sprite();
        _container.addChild(_labelContainer);
        _container.addChild(borderLeft);
        _container.addChild(borderRight);
        addChild(_container);
    }

    override public function dispose():void{
        super .dispose();
        if(_carTypeLabel){
            _carTypeLabel.removeFromParent(true);
            _carTypeLabel = null;
        }
        if(borderLeft){
            borderLeft.removeFromParent(true);
            borderLeft = null;
        }
        if(borderRight){
            borderRight.removeFromParent(true);
            borderRight = null;
        }
        if(_container) {
            _container.removeFromParent(true);
            _container = null;
        }
    }
}
}