package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.feed.ChallengeNotificationData;
    import com.despark.tuning3d.model.challenges.feed.ChallengeWonNoteData;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.MultilingualButton;
    import com.utils.FontSelector;

    import feathers.controls.Label;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.RelativePosition;

    import flash.display.BitmapData;
    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import starling.display.DisplayObject;
    import starling.display.DisplayObjectContainer;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.EventDispatcher;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.filters.DropShadowFilter;
    import starling.textures.Texture;

    public class ChallengeNotificationItemRenderer extends ListItemRenderer {
    private var _eventDispatcher:EventDispatcher;
    private var _challengeNoteData:ChallengeNotificationData;
    private var _icon:Image;
    private var _iconTailRight:Quad;
    private var _background:Quad;
    private var _bgBottomBorder:Quad;
    private var _contentBorder:Image;
    private var _dateLabel:DateLabel;
    private var _claimBtn:MultilingualButton;
    private var _claimLabel:Label;

    private static var borderTexture:Texture;
    public var isWide:Boolean = false;
    public var isUltraWide:Boolean = false;
    public function ChallengeNotificationItemRenderer() {
        super();
        createBorderTexture();
    }

    override protected function draw():void {
        super.draw();
        updateView();
    }

    public function updateView():void{
        if(_background) {
            _background.width = actualWidth;
            _background.height = actualHeight - 4;
            _background.y = (actualHeight - _background.height) / 2;
        }

        if(_bgBottomBorder) {
            _bgBottomBorder.width = actualWidth;
            _bgBottomBorder.height = 0.8;
            _bgBottomBorder.y = _background.y + _background.height;
        }

        var leftMargin:Number = Device.isIphoneXVariant()?90:60;
        var rightMargin:Number = Device.isIphoneXVariant()?45:12;
        var iconRightMargin:Number = 16;
        if(_icon) {
            _icon.x = leftMargin - (_icon.width + iconRightMargin);
            _icon.y = (actualHeight - _icon.height) / 2;

            if(_contentBorder){
                _contentBorder.width = actualWidth - (leftMargin + rightMargin);
                _contentBorder.height = actualHeight - 28;
                _contentBorder.x = leftMargin;
                _contentBorder.y = (actualHeight - _contentBorder.height)/2;
            }

            if(_iconTailRight == null){
                _iconTailRight = new Quad(1,1, 0xe8e8e8);
                _iconTailRight.width = iconRightMargin;
                _iconTailRight.height = 1;
                addChild(_iconTailRight);
                _iconTailRight.x = leftMargin - _iconTailRight.width;
                _iconTailRight.y = _icon.y + (_icon.height - _iconTailRight.height) / 2;
            }else if(!contains(_iconTailRight)){
                addChild(_iconTailRight);
            }
            setChildIndex(_iconTailRight, numChildren - 1);

            if (_challengeNoteData.tuningView) {
                _challengeNoteData.tuningView.height = _contentBorder.height;
                if(_challengeNoteData.tuningView is DisplayObjectContainer){
                    validateLayoutGroup(_challengeNoteData.tuningView as DisplayObjectContainer);
                }else{
                    _challengeNoteData.tuningView.scale = _challengeNoteData.tuningView.scaleY;
                }
                _challengeNoteData.tuningView.x = _contentBorder.x + _contentBorder.width - _challengeNoteData.tuningView.width;
                _challengeNoteData.tuningView.y = _contentBorder.y + (_contentBorder.height - _challengeNoteData.tuningView.height) / 2;
            }
        }

        if(isUltraWide){
            if(_dateLabel){
                _dateLabel.x = _contentBorder.x + _contentBorder.width - 160;
                _dateLabel.y = (actualHeight - _dateLabel.height) / 2;
            }
        }


        if(_claimBtn){
            if(isUltraWide){
                _claimBtn.x = _contentBorder.x + _contentBorder.width - (_claimBtn.width + 180);
                _claimBtn.y = (actualHeight - _claimBtn.height) / 2;

                if(_claimLabel){
                    _claimLabel.x = _claimBtn.x + (_claimBtn.width - _claimLabel.width) / 2;
                    _claimLabel.y = (actualHeight - _claimLabel.height) / 2;
                }
            }else{
                _claimBtn.x = _contentBorder.x + _contentBorder.width - (_claimBtn.width + 110);
                _claimBtn.y = actualHeight / 2 - 4 * _claimBtn.height / 5;

                if(_claimLabel){
                    _claimLabel.x = _claimBtn.x + (_claimBtn.width - _claimLabel.width) / 2;
                    _claimLabel.y = _claimBtn.y + _claimBtn.height - _claimLabel.height;
                }

                if(_dateLabel){
                    _dateLabel.x = _claimBtn.x + (_claimBtn.width - _dateLabel.width) / 2;
                    _dateLabel.y = _claimBtn.y + _claimBtn.height + 3;
                }
            }
        }

        _challengeNoteData.leftMargin = leftMargin;
        _challengeNoteData.layout(actualWidth, actualHeight);
    }

    private static function validateLayoutGroup(group:DisplayObjectContainer):void{
        for(var i:int = 0; i < group.numChildren; i++){
            var child:DisplayObject = group.getChildAt(i);
            if(child is DisplayObjectContainer){
                validateLayoutGroup(child as DisplayObjectContainer);
            }else if(child.hasOwnProperty("validate")){
                child["validate"]();
            }
            if(group.hasOwnProperty("validate")){
                group["validate"]();
            }
        }
    }

    override protected function commitData():void {
        super.commitData();
        if(!data){
            return;
        }
        if(_challengeNoteData && (_challengeNoteData != data || _challengeNoteData.disposed)){
            while (numChildren) {
                removeChildAt(0);
            }
            removeComponents();
            _challengeNoteData = data  as ChallengeNotificationData;
            _challengeNoteData.holder = this;
            _challengeNoteData.createView(actualWidth, actualHeight);
            invalidate();
        }
        if(!_challengeNoteData) {
            _challengeNoteData = data as ChallengeNotificationData;
            _challengeNoteData.holder = this;
        }

        if(!_bgBottomBorder){
            _bgBottomBorder = new Quad(1, 1, 0x929091);
            _bgBottomBorder.filter = new DropShadowFilter(1, Math.PI);
        }
        if(!_bgBottomBorder.parent){
            addChild(_bgBottomBorder);
        }
        if(!_background) {
            _background = new Quad(1, 1, 0xFFFFFF);
        }
        if(!_background.parent){
            addChildAt(_background, getChildIndex(_bgBottomBorder)+1);
        }
        if(!_icon) {
            _icon = new Image(ButtonFactory.assetManager.getTexture(_challengeNoteData.iconTextureId));
            _icon.scale = _icon.texture.scale/2;
        }
        if(!_icon.parent) {
            addChildAt(_icon, getChildIndex(_background) + 1);
        }
        if(!_contentBorder) {
            _contentBorder = new Image(borderTexture);
            _contentBorder.scale9Grid = new Rectangle(_contentBorder.width / 4, _contentBorder.height / 4, _contentBorder.width / 2, _contentBorder.height / 2);
        }
        if(!_contentBorder.parent){
            addChildAt(_contentBorder, getChildIndex(_icon)+1);
        }

        if(!_dateLabel) {
            _dateLabel = new DateLabel();
        }
        if(!_dateLabel.parent){
            addChild(_dateLabel);
        }
        if(!_claimBtn){
            _claimBtn = ButtonFactory.defaultButton("CLAIM " + (_data.source.diamond * Constants.UNIT_MULTIPLIER).toString(), 0xff0000, "button_gradient");
            _claimBtn.horizontalAlign = HorizontalAlign.CENTER;
            _claimBtn.width = 72;
            _claimBtn.height = 20;
            _claimBtn.includeInLayout = false;
            _claimBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 16, 0xFFFFFF, TextFormatAlign.LEFT, false);
            _claimBtn.addEventListener(Event.TRIGGERED, claimSelectedHandler);
            addChild(_claimBtn);
            _claimBtn.validate();

            var rewardsIcon:Image = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_btn_icon"));
            rewardsIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
            rewardsIcon.color = 0xFFFFFE;

            _claimBtn.defaultIcon = rewardsIcon;
            _claimBtn.iconPosition = RelativePosition.RIGHT;
            _claimBtn.iconOffsetX = 5;
            _claimBtn.labelOffsetY = -0.5;
        }else{
            if(!_claimBtn.parent){
                addChild(_claimBtn);
            }
            _claimBtn.label = "CLAIM " + (_data.source.diamond * Constants.UNIT_MULTIPLIER).toString();
        }

        _claimBtn.visible = (_data.source.hasOwnProperty("claimed") && !_data.source.claimed);

        if(!_claimLabel){
            _claimLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 13, 0x979797, TextFormatAlign.CENTER, false);
            _claimLabel.text = "CLAIMED";
            _claimLabel.width = _claimBtn.width;
            _claimLabel.touchable = false;
            addChild(_claimLabel);
        }else{
            if(!_claimLabel.parent){
                addChild(_claimLabel);
            }
        }

        _claimLabel.visible = !_claimBtn.visible;

        if(_challengeNoteData.messageLabel && !_challengeNoteData.messageLabel.parent) {
            addChild(_challengeNoteData.messageLabel);
        }
        if(_challengeNoteData.tuningView && !_challengeNoteData.tuningView.parent){
            addChild(_challengeNoteData.tuningView);
        }
        if(_challengeNoteData.challengeLabel && !_challengeNoteData.challengeLabel.parent){
            addChild(_challengeNoteData.challengeLabel);
        }
        if(_challengeNoteData.avatar && !_challengeNoteData.avatar.parent){
            addChild(_challengeNoteData.avatar);
        }
        if(_challengeNoteData.buttonGroup && !_challengeNoteData.buttonGroup.parent){
            addChild(_challengeNoteData.buttonGroup);
        }
        _dateLabel.time = _challengeNoteData.time;
        updateView();
        if(_challengeNoteData && _challengeNoteData.disposed){
            _challengeNoteData.createView(actualWidth, actualHeight);
            invalidate();
        }
    }

    override public function dispose():void {
        super.dispose();
        disposeComponents();
        _eventDispatcher = null;
    }

    override protected function touchHandler(event:TouchEvent):void{
        super .touchHandler(event);
        var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
        var touchClaim:Touch = event.getTouch(_claimBtn, TouchPhase.ENDED);
        if(touch && !owner.isScrolling && !touchClaim){
            if(_eventDispatcher){
                if(_challengeNoteData is ChallengeWonNoteData){
                    _eventDispatcher.dispatchEventWith(ChallengeEvent.TUNING_SELECTED, false, _challengeNoteData.source);
                }
            }
        }
    }

    private function claimSelectedHandler(event:Event):void {
        _challengeNoteData.source.claimed = true;
        if(_eventDispatcher){
            _eventDispatcher.dispatchEventWith(ChallengeEvent.CHALLENGE_CLAIM_REWARD, false, _challengeNoteData.source);
        }
        _claimBtn.visible = false;
        _claimLabel.visible = true;
    }

    private function disposeComponents():void{
        if(_icon){
            _icon.removeFromParent(true);
            _icon = null;
        }
        if(_background){
            _background.removeFromParent(true);
            _background = null;
        }
        if(_bgBottomBorder){
            _bgBottomBorder.removeFromParent(true);
            _bgBottomBorder = null;
        }
        if(_contentBorder){
            _contentBorder.removeFromParent(true);
            _contentBorder = null;
        }
        if(_dateLabel){
            _dateLabel.removeFromParent(true);
            _dateLabel = null;
        }
        if(_challengeNoteData){
            _challengeNoteData.dispose();
            _challengeNoteData = null;
        }
        if(_claimBtn){
            _claimBtn.removeFromParent(true);
            _claimBtn = null;
        }
        if(_claimLabel){
            _claimLabel.removeFromParent(true);
            _claimLabel = null;
        }
    }

    private function removeComponents():void{
        if(_icon){
            _icon.removeFromParent(true);
            _icon = null;
        }
        if(_challengeNoteData){
            _challengeNoteData.dispose();
            _challengeNoteData = null;
        }
    }

    public function get eventDispatcher():EventDispatcher {
        return _eventDispatcher;
    }

    public function set eventDispatcher(value:EventDispatcher):void {
        _eventDispatcher = value;
    }

    private static function createBorderTexture():void{
        if(!borderTexture) {
            var bmd:BitmapData = new BitmapData(20, 20, true, 0xFFe6e6e6);
            bmd.fillRect(new Rectangle(1, 1, 18, 18), 0xFFFFFFFF);
            borderTexture = Texture.fromBitmapData(bmd, false, false, 2);
        }
    }
}
}
