package com.despark.tuning3d.view.components.challenges.controls {
import com.despark.tuning3d.model.challenges.ChallengeData;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.utils.DateParser;
import com.utils.FontSelector;

import feathers.controls.Label;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;

import fl.motion.MatrixTransformer;

import flash.events.TimerEvent;
import flash.geom.Matrix;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.utils.Timer;

import starling.display.Image;
import starling.display.Quad;
import starling.display.Sprite;
import starling.filters.DropShadowFilter;

public class ChallengeTimerLabel extends Sprite {
    public var background:Image;
    private var label1:Label;
    private var label2:Label;
    private var label3:Label;
    private var _timer:Timer;
    private var _timeLeft:Number;
    private var finishedBackground:Quad;
    private static const SECOND:Number = 1000;
    private static const MINUTE:Number = SECOND * 60;
    private static const HOUR:Number = MINUTE * 60;
    public var isFinished:Boolean;
    public function ChallengeTimerLabel() {
        isFinished = false;
        label1 = new Label();
        label2 = new Label();
        label3 = new Label();

        background = new Image(ButtonFactory.assetManager.getTexture("challenge_clock_background"));
        background.scale = ButtonFactory.assetManager.scaleFactor / 2;
        label1.textRendererFactory = labelRenderer;
        label2.textRendererFactory = labelRenderer;
        label3.textRendererFactory = labelRenderer;
        addChild(background);
        addChild(label1);
        addChild(label2);
        addChild(label3);
        label1.text = "00";
        label2.text = "00";
        label3.text = "00";
        setLabelDimensions();
    }

    private function setLabelDimensions():void{
        label1.width = label2.width = label3.width = 30;
        label1.height = label2.height = label3.height = 22;
        var timerBorder:Number = 6;
        label1.x = timerBorder;
        label2.x = label1.x + label1.width + timerBorder/2;
        label3.x = label2.x + label2.width + timerBorder/2;
        label1.y = label2.y = label3.y = (background.height - label1.height) / 2 - .5;
    }

    public function setTime(value:String):void{
        if(_timer && _timer.running){
            _timer.stop();
        }
        isFinished = false;
        if(finishedBackground){
            finishedBackground.removeFromParent();
        }
        label2.visible = label3.visible = background.visible = true;
        label1.textRendererFactory = labelRenderer;
        setLabelDimensions();
        _timeLeft = new Date().time - DateParser.parseDate(value).time;
        _timeLeft = ChallengeData.DURATION - _timeLeft;
        if(_timeLeft>0){
            _timer ||= new Timer(1000);
            if(!_timer.hasEventListener(TimerEvent.TIMER)){
                _timer.addEventListener(TimerEvent.TIMER, timerHandler);
            }
            updateTime();
            _timer.start();
        }else {
            finished = true;
        }
    }

    private function timerHandler(event:TimerEvent):void {
        updateTime();
    }

    public function set finished(value:Boolean):void{
        if(value){
            if(_timer){
                _timer.stop();
                _timer.removeEventListener(TimerEvent.TIMER, timerHandler);
                _timer = null;
            }
            label2.visible = false;
            label3.visible = false;
            background.visible = false;
            label1.textRendererFactory = labelFinishedRenderer;
            label1.width = width;
            label1.x = 0;
            label1.text = "FINISHED";
            finishedBackground ||= new Quad(1,1,0xFFFFFF);
            finishedBackground.width = width - 30;
            finishedBackground.height = height - 10;
            finishedBackground.x = (width - finishedBackground.width)/2;
            finishedBackground.y = 5;
            addChildAt(finishedBackground, 0);
            finishedBackground.filter ||= new DropShadowFilter(0,0, 0, 0.5, 0.2);
            isFinished = true;
        }else{
            isFinished = false;
        }
    }

    public function get fininshed():Boolean{
        return isFinished;
    }

    private function updateTime():void{
        if(_timeLeft<=0){
            finished = true;
            return;
        }
        label1.text = getPair(int(_timeLeft/HOUR));
        var mn:Number = (_timeLeft%HOUR);
        label2.text = getPair(int(mn/MINUTE));
        var sec:Number = int((mn%MINUTE)/SECOND);
        label3.text = getPair(sec);
        _timeLeft -= SECOND;
    }

    override public function dispose():void {
        if(_timer){
            _timer.stop();
            _timer.removeEventListener(TimerEvent.TIMER, timerHandler);
            _timer = null;
        }
        super.dispose();
        if(background){
            background.removeFromParent(true);
            background = null;
        }
        if(label1){
            label1.removeFromParent(true);
            label1 = null;
        }
        if(label2){
            label2.removeFromParent(true);
            label2 = null;
        }
        if(finishedBackground){
            finishedBackground.removeFromParent(true);
            finishedBackground = null;
        }
    }

    private static function getPair(num:int):String{
        var str:String = num.toString();
        return str.length==1?"0"+str:str;
    }

    private static function labelRenderer() :ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(true, true), 18, 0x242424);
        textRenderer.textFormat.align = TextFormatAlign.CENTER;
        textRenderer.textFormat.italic = true;
        textRenderer.textFormat.bold = true;
        textRenderer.embedFonts = true;
        var m:Matrix = new Matrix();
        MatrixTransformer.setSkewX(m, 10);
        textRenderer.transformationMatrix = m;
        return textRenderer;
    }

    private static function labelFinishedRenderer() :ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(false, false), 16, 0x242424);
        textRenderer.textFormat.align = TextFormatAlign.CENTER;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        return textRenderer;
    }

}
}
