package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.NumberUtils;

    import feathers.controls.Label;

    import flash.geom.Rectangle;

    import starling.display.Image;
    import starling.textures.Texture;

    public class ChallengeUserAvatar extends UserCircularAvatar {
        private var _ratingLabel:Label;
        private var _ratingLabelBackgroundColor:uint = 0xfba500;
        public function ChallengeUserAvatar(user:User, size:int, showBorder:Boolean = false, borderSize:Number = 0) {
            super(user, size, showBorder, borderSize);
        }

        override protected function initialize():void{
            super .initialize();
            var ratingTxt:Texture = ButtonFactory.assetManager.getTexture("challenge_user_rating_bg");
            Image.bindScale9GridToTexture(ratingTxt, new Rectangle(ratingTxt.width/2 - 0.5, 0, 1, ratingTxt.height));
            var ratingBg:Image = new Image(ratingTxt);
            ratingBg.color = _ratingLabelBackgroundColor;
            _ratingLabel = new Label();
            _ratingLabel.paddingTop = 0;

            _ratingLabel.paddingLeft = -2;
            _ratingLabel.paddingRight = -1;

            _ratingLabel.backgroundSkin = ratingBg;
            _ratingLabel.width = 20;
            _ratingLabel.height = 12;
            _ratingLabel.includeInLayout = false;
            _ratingLabel.textRendererFactory = LabelFactory.challengeUserRatingLabel;
            _ratingLabel.text = NumberUtils.toFixed(user.rating,2);
            addChild(_ratingLabel);
        }

        public function set ratingLabelBackgroundColor(value:uint):void{
            _ratingLabelBackgroundColor = value;
            if(_ratingLabel && _ratingLabel.backgroundSkin){
                Image(_ratingLabel.backgroundSkin).color = _ratingLabelBackgroundColor;
            }
        }

        override protected function draw():void{
            super.draw();
            if(_ratingLabel){
                _ratingLabel.x = width / 2 + Math.sin(Math.PI / 4) * width / 2 - _ratingLabel.width / 2 + 1;
                _ratingLabel.y = height / 2 + Math.sin(Math.PI / 4) * height / 2 - _ratingLabel.height / 2 + 1;
            }
        }

        public function setRatingLabel(value:String):void{
            if(_ratingLabel){
                _ratingLabel.visible = (value && value.length);
                _ratingLabel.text = value;
            }
        }
    }
}
