package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.despark.tuning3d.view.components.tunings.TuningRowsFHDItemRenderer;
    import com.utils.DateParser;
    import com.utils.FontSelector;
    import com.utils.NumberUtils;
    import com.utils.ObjectUtils;

    import feathers.controls.Label;

    import flash.display.StageOrientation;
    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import starling.display.Image;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;

    public class ChallengesTuningItemRenderer extends TuningRowsFHDItemRenderer {
    protected var _avatar:UserCircularAvatar;
    protected var _usernameLabel:Label;
    protected var _ratingLabel:ExtendedLabel;
    protected var _timeTrialLabel:ChallengeStatusTimeTrial;
    protected var _challengeData:ChallengeData;
    private static const labels:Object = {sec:["s"], min:["m"], hr:["h"], day:["d"]};
    public function ChallengesTuningItemRenderer() {
        super ();
        if(!hasEventListener(TouchEvent.TOUCH, touchHandler)) {
            addEventListener(TouchEvent.TOUCH, touchHandler);
        }
    }

    override protected function initialize():void{
        super .initialize();
        if(nameLabel){
            nameLabel.removeFromParent(true);
            nameLabel = null;
        }
        if(nameBg){
            nameBg.removeFromParent(true);
            nameBg = null;
        }

        _usernameLabel = new Label();
        _usernameLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getUnicode(), 8, 0xFFFFFF, TextFormatAlign.CENTER, true, false, null, false, true);
        addChild(_usernameLabel);

        createRatingLabel();
    }

    protected function createRatingLabel():void{
        _ratingLabel = new ExtendedLabel();
        var rateBg:Image = new Image(ButtonFactory.assetManager.getTexture("challenge_tuning_rating_bg"));
        rateBg.scale9Grid = new Rectangle(rateBg.texture.frameWidth / 3,0, rateBg.texture.frameWidth / 3, rateBg.texture.frameHeight);
        _ratingLabel.backgroundSkin = rateBg;
        _ratingLabel.width = 55;
        _ratingLabel.height = 22;
        _ratingLabel.textRendererFactory = LabelFactory.challengeTuningsListRatingLabel;


        var ratingIcon:LayoutImage = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));
        ratingIcon.scale = 0.6 * ButtonFactory.assetManager.scaleFactor / 2;
        _ratingLabel.defaultIcon = ratingIcon;
        _ratingLabel.labelOffsetX = 5;
        _ratingLabel.paddingLeft = 12;

        addChild(_ratingLabel);
    }

    override protected function commitData():void{
        super .commitData();
        if(_data) {
            var challengeSourceData:Object = ObjectUtils.getProperty(_data, "stats.challengeHostId");
            if(!(challengeSourceData is String)){
                challengeSourceData = ObjectUtils.merge(challengeSourceData, _data.stats);
                _challengeData = new ChallengeData(challengeSourceData);
            }
        }
        if(_challengeData && _challengeData.isTimeTrial && _ratingLabel){
            _ratingLabel.removeFromParent();
            _timeTrialLabel ||= new ChallengeStatusTimeTrial();
            addChild(_timeTrialLabel);
            _timeTrialLabel.text = _challengeData.timeTrialsPercent.toString() + "% / " + DateParser.getInChallengeLabel(_challengeData.timeTrialsSeconds * 1000, labels, 2, false);
        }else if(_ratingLabel){
            addChild(_ratingLabel);
        }
        if(tuning && tuning.userObj){
            var user:User = new User(tuning.userObj);
            if(_avatar){
                _avatar.user = user;
            }else{
                _avatar = new UserCircularAvatar(user, 26, true, 1);
            }
            addChild(_avatar);
            if(_usernameLabel){
                if(index > -1){
                    _usernameLabel.text = (index + 1).toString() + ". " + user.nameToShow;
                }else{
                    _usernameLabel.text = user.nameToShow;
                }
                _usernameLabel.validate();
            }
            if(_ratingLabel){
                _ratingLabel.text = NumberUtils.toFixed(parseFloat(_data.stats.rating),2);
            }
        }
    }

    override protected function layout():void{
        super .layout();
        if(!_data){
            return;
        }
        if(imageLoader) {
            if(!_challengeData && _data.featureRules && !_data.featureRules.customFrame){
                imageLoader.height = actualHeight * 1.35;
                imageLoader.width = actualWidth * 1.35;
                imageLoader.x = Math.min((actualWidth - imageLoader.width) / 2 + imageLoader.width * 0.05, 0);
            }else if(_challengeData && _challengeData.featuresRules && !_challengeData.featuresRules.customAngle){
                imageLoader.height = actualHeight * 1.45;
                imageLoader.width = actualWidth * 1.45;
                imageLoader.x = Math.min((actualWidth - imageLoader.width) / 2 + imageLoader.width * 0.1, 0);
            }else{
                imageLoader.x = (actualWidth - imageLoader.width) / 2;
            }
            imageLoader.y = (actualHeight - imageLoader.height) / 2;
        }
    }

    override protected function draw():void{
        super .draw();
        var gap:Number = 5;
        var xMargin:Number = 12;
        if(_avatar){
            _avatar.x = xMargin;
            _avatar.y = actualHeight - (_avatar.height + gap + 1);
            if(_usernameLabel){
                _usernameLabel.x = _avatar.x + _avatar.width + gap;
                _usernameLabel.y = _avatar.y + (_avatar.height - _usernameLabel.height) / 2;
            }
        }
        if(_timeTrialLabel && _timeTrialLabel.parent){
            _timeTrialLabel.x = 0;
            _timeTrialLabel.y = gap;
        }else if(_ratingLabel && _ratingLabel.parent){
            _ratingLabel.x = 0;
            _ratingLabel.y = gap;
            _ratingLabel.backgroundSkin.y = 0;
            _ratingLabel.backgroundSkin.x = 0;
        }
    }

        override public function updateOrientationDependantView():void{
            var xMargin:Number = 12;
            if(owner is ExtendedList && ExtendedList(owner).orientation == StageOrientation.ROTATED_RIGHT && isLeftSideItem()){
                var notchSize:Number = 20;
                if(_ratingLabel){
                    _ratingLabel.width = 55 + notchSize;
                    _ratingLabel.paddingLeft = 12 + notchSize;
                }
                if(_avatar){
                    if(_avatar && rowIndex == 1){
                        _avatar.x = xMargin + notchSize;
                    }else{
                        _avatar.x = xMargin;
                    }
                }
            }else{
                if(_ratingLabel){
                    _ratingLabel.width = 55;
                    _ratingLabel.paddingLeft = 12;
                }
                if(_avatar){
                    _avatar.x = xMargin;
                }
            }
            if(_usernameLabel && _avatar){
                var gap:Number = 5;
                _usernameLabel.x = _avatar.x + _avatar.width + gap;
            }
        }

    public function isLeftSideItem():Boolean{
        if(data && owner && owner.dataProvider){
            var numRows:int = Math.round(owner.pageHeight / actualHeight);
            var numCells:int = Math.round(owner.pageWidth / actualWidth);
            var numPerPage:int = numRows * numCells;
            if(owner.horizontalPageIndex < owner.horizontalPageCount - 1 || owner.dataProvider.length % numPerPage == 0) {
                var firstIndex:int = owner.horizontalPageIndex * numPerPage;
                if (index == firstIndex) {
                    return true;
                }
                if(numRows > 1 && index == firstIndex + numCells){
                    return true;
                }
            }else if(index == owner.dataProvider.length - (owner.dataProvider.length % numPerPage)){
                return true;
            }else if(index == owner.dataProvider.length - (owner.dataProvider.length % numPerPage) + numCells){
                return true;
            }
        }
        return false;
    }

    public function get rowIndex():int{
        if(owner){
            return Math.round(y / actualHeight) + 1;
        }
        return 0;
    }

    public function get cellIndex():int{
        if(owner){
            return Math.round(x / actualWidth) + 1;
        }
        return 0;
    }

    override public function get tuning():Tuning{
        if(data && data.hasOwnProperty("tuning")){
            return data.tuning as Tuning;
        }
        return null;
    }

    override protected function touchHandler(event : TouchEvent):void {
        try {
            const touches : Vector.<Touch> = event.getTouches(this);
            for each(var touch : Touch in touches) {
                if (touch.phase == TouchPhase.ENDED && !owner.isScrolling) {
                    owner.dispatchEvent(new Event(ChallengeScreenItemRenderer.ITEM_SELECTED, false, _data));
                    return;
                }
            }
        } catch(e:Error) {

        }
    }
}
}