package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedToggleButton;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.ScreenComponentFactory;

    import feathers.controls.LayoutGroup;
    import feathers.controls.ToggleButton;
    import feathers.core.ToggleGroup;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.RelativePosition;
    import feathers.layout.VerticalAlign;

    import flash.geom.Rectangle;

    import starling.display.Image;

    import starling.textures.Texture;

    public class ChallengesTuningSideSelector extends LayoutGroup {
        private var _frontViewButton:ExtendedToggleButton;
        private var _rearViewButton:ExtendedToggleButton;
        private var _toggleGroup:ToggleGroup;
        public function ChallengesTuningSideSelector() {
            super();
            _frontViewButton = ScreenComponentFactory.createRadio("FRONT", "front", false, null, null, true, 11, 11, 0x4a4a4a, 0x4a4a4a) as ExtendedToggleButton;
            _rearViewButton = ScreenComponentFactory.createRadio("REAR", "rear", false, null, null, true, 11, 11, 0x4a4a4a, 0x4a4a4a) as ExtendedToggleButton;

            _frontViewButton.hitAreaPadding = 10;
            _frontViewButton.hitAreaPaddingRight = 0;

            _rearViewButton.hitAreaPadding = 10;
            _rearViewButton.hitAreaPaddingLeft = -15;

            _toggleGroup = new ToggleGroup();
            _toggleGroup.addItem(_frontViewButton);
            _toggleGroup.addItem(_rearViewButton);
        }

        override protected function initialize():void{
            super .initialize();

            layout = new HorizontalLayout();

            var sideBgTexture:Texture = ButtonFactory.assetManager.getTexture("button");
            var region:Rectangle = new Rectangle();
            region.width = sideBgTexture.frameHeight;
            region.height = sideBgTexture.frameHeight;

            var frontBg:Image = new Image(Texture.fromTexture(sideBgTexture, region, null, false, 1.25));
            frontBg.scale9Grid = new Rectangle(frontBg.width / 2, frontBg.height / 2);
            frontBg.color = 0xFEFFFF;
            frontBg.alpha = 0.85;

            region.x = sideBgTexture.width - region.width;
            var rearBg:Image = new Image(Texture.fromTexture(sideBgTexture, region, null, false, 1.25));
            rearBg.scale9Grid = new Rectangle(rearBg.width / 2,rearBg.height / 2);
            rearBg.color = 0xFEFFFF;
            rearBg.alpha = 0.85;

            _frontViewButton.iconPosition = RelativePosition.RIGHT;
            _rearViewButton.iconPosition = RelativePosition.RIGHT;

            _frontViewButton.horizontalAlign = HorizontalAlign.CENTER;
            _rearViewButton.horizontalAlign = HorizontalAlign.CENTER;

            _frontViewButton.verticalAlign = VerticalAlign.MIDDLE;
            _rearViewButton.verticalAlign = VerticalAlign.MIDDLE;

            _frontViewButton.defaultSkin = frontBg;
            _rearViewButton.defaultSkin = rearBg;

            _rearViewButton.height = _frontViewButton.height = 16;

            _frontViewButton.iconOffsetX = 9.5;
            _frontViewButton.iconOffsetY = -0.5;
            _frontViewButton.labelOffsetY = -1;

            _frontViewButton.paddingTop = 2;
            _frontViewButton.paddingRight = 10;
            _frontViewButton.paddingBottom = 1;
            _frontViewButton.paddingLeft = 5;

            _rearViewButton.iconOffsetX = 10;
            _rearViewButton.iconOffsetY = -0.5;
            _rearViewButton.labelOffsetY = -1;

            _rearViewButton.paddingTop = 2;
            _rearViewButton.paddingRight = 18;
            _rearViewButton.paddingBottom = 1;

            _frontViewButton.labelFactory = LabelFactory.challengeTuningViewLabel;
            _rearViewButton.labelFactory = LabelFactory.challengeTuningViewLabel;

            _toggleGroup.isSelectionRequired = true;

            addChild(_frontViewButton);
            addChild(_rearViewButton);
            _frontViewButton.validate();
            _rearViewButton.validate();
        }

        public function get toggleGroup():ToggleGroup {
            return _toggleGroup;
        }
    }
}
