package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.DateParser;
    import com.utils.DateParser;
    import com.utils.NumberUtils;
    import com.utils.ObjectUtils;

    import feathers.controls.Label;

    import flash.display.StageOrientation;

    import starling.display.Quad;

    public class ChallengesUserTuningItemRenderer extends ChallengesTuningItemRenderer {
    private var _titleLabel:Label;
    private var _statusLabel:ChallengeStatusLabel;
    private var _userGarage:Boolean;
    private var _background:Quad;
    private var iconLoader:ImageLoaderWithStates;
    private static const labels:Object = {sec:["s"], min:["m"], hr:["h"], day:["d"]};
    public function ChallengesUserTuningItemRenderer() {
        super ();
    }

    override protected function initialize():void{
        super .initialize();
        if(nameBg){
            nameBg.removeFromParent(true);
            nameBg = null;
        }
        _titleLabel = new Label();
        _titleLabel.minWidth = actualWidth / 2;
        _titleLabel.paddingLeft = _titleLabel.paddingRight = 12;
        _titleLabel.wordWrap = false;
        _titleLabel.includeInLayout = false;
        _titleLabel.textRendererFactory = LabelFactory.challengeLeaderboardTitleLabel;
        _titleLabel.backgroundSkin = new Quad(1, 1, 0x000000);
        _titleLabel.backgroundSkin.alpha = 0.75;
        addChild(_titleLabel);
    }

    override protected function commitData():void{
        super .commitData();
        if(_data){
            if(_background){
                _background.removeFromParent(true);
                _background = null;
            }
            if(iconLoader){
                iconLoader.removeFromParent(true);
                iconLoader = null;
            }
            imageLoader.visible = true;
            if(_challengeData.isTimeTrial){
                if(_statusLabel is ChallengeStatusInVoting){
                    _statusLabel.removeFromParent(true);
                    _statusLabel = null;
                }
                _statusLabel ||= new ChallengeStatusTimeTrial();
                _statusLabel.visible = true;
            }else{
                if(_statusLabel is ChallengeStatusTimeTrial){
                    _statusLabel.removeFromParent(true);
                    _statusLabel = null;
                }
                if(_challengeData.status == ChallengeData.IN_VOTING || _challengeData.status == ChallengeData.IN_CHALLENGE){
                    _statusLabel ||= new ChallengeStatusInVoting();
                }else if(_statusLabel){
                    _statusLabel.removeFromParent(true);
                    _statusLabel = null;
                }
            }
            if(_statusLabel){
                addChild(_statusLabel);
            }

            if(_data.user){
                if(_avatar){
                    _avatar.user = _data.user;
                }else{
                    _avatar = new UserCircularAvatar(_data.user, 26, true, 1);
                }
                _usernameLabel.text = _avatar.user.nameToShow;
                _usernameLabel.validate();
            }else if(_avatar){
                _avatar.removeFromParent(true);
                _avatar = null;
                if(_usernameLabel){
                    _usernameLabel.removeFromParent(true);
                    _usernameLabel = null;
                }
            }

            if(_challengeData.isTimeTrial){
                if(_ratingLabel){
                    _ratingLabel.removeFromParent(true);
                    _ratingLabel = null;
                }
            }else{
                if(!_ratingLabel){
                    createRatingLabel();
                }
                _ratingLabel.text = NumberUtils.toFixed(parseFloat(_data.stats.rating),2);
            }

            if(_titleLabel){
                _titleLabel.text = tuning.name;
            }
            if(_statusLabel){
                if(_statusLabel is ChallengeStatusTimeTrial){
                    _statusLabel.text = _challengeData.timeTrialsPercent.toString() + "% / " + DateParser.getInChallengeLabel(_challengeData.timeTrialsSeconds * 1000, labels, 2, false);
                }else{
                    _statusLabel.text = DateParser.getInChallengeLabel(_challengeData.votingFinishDate.time - new Date().time).toString();
                }
            }
            if(!_userGarage && _challengeData.status != ChallengeData.FINISHED && !_challengeData.isTimeTrial){
                imageLoader.visible = false;
                //
                if(!_background){
                    _background = new Quad(1,1,0x252525);
                    _background.width = actualWidth;
                    _background.height = actualHeight;
                    addChildAt(_background, 0);
                }
                iconLoader = new ImageLoaderWithStates();
                iconLoader.bitmapScale = 1;
                iconLoader.scale = 1 / 0.4;
                iconLoader.persistentImageProvider = Car.BlankCar;
                iconLoader.completeHandler = defaultImageLoadComplete;
                addChildAt(iconLoader, 1);
            }
        }
    }

    private function defaultImageLoadComplete(loaderWithStates:ImageLoaderWithStates):void{
        loaderWithStates.width = actualWidth - 20;
        loaderWithStates.height = actualHeight - 20;
        loaderWithStates.x = (actualWidth - loaderWithStates.width) / 2;
        loaderWithStates.y = (actualHeight - loaderWithStates.height) / 2 + 20;
        validate();
    }

    override protected function draw():void{
        super .draw();
        var gap:Number = 10;
        if(_titleLabel){
            _titleLabel.textRendererFactory = LabelFactory.challengeLeaderboardTitleLabel;
            _titleLabel.validate();
            _titleLabel.x = 0;
        }

        if(_titleLabel){
            _titleLabel.y = gap;
        }

        if(_timeTrialLabel){
            _timeTrialLabel.removeFromParent(true);
            _timeTrialLabel = null;
        }

        if(_ratingLabel){
            _ratingLabel.y = _titleLabel.height + _titleLabel.y - 3;
            _ratingLabel.visible = (_challengeData.status == ChallengeData.FINISHED);
        }

        if(_statusLabel){
            _statusLabel.validate();
            _statusLabel.x = 0;
            _statusLabel.y = _titleLabel.y + _titleLabel.height;
            _statusLabel.visible = (_challengeData.isTimeTrial || _challengeData.status != ChallengeData.FINISHED);
        }

    }

        override public function updateOrientationDependantView():void{
            super .updateOrientationDependantView();
            if(owner is ExtendedList && ExtendedList(owner).orientation == StageOrientation.ROTATED_RIGHT && isLeftSideItem()){
                var notchSize:Number = 20;
                if(_statusLabel){
                    _statusLabel.paddingLeft = _statusLabel.defaultPaddingLeft + notchSize;
                }
                if(_titleLabel && rowIndex == 1){
                    _titleLabel.paddingLeft = 12;
                }else{
                    _titleLabel.paddingLeft = 12 + notchSize;
                }
            }else if(_statusLabel && _titleLabel){
                _statusLabel.paddingLeft = _statusLabel.defaultPaddingLeft;
                _titleLabel.paddingLeft = 12;
            }
        }

        public function set userGarage(userGarage:Boolean):void {
            _userGarage = userGarage;
        }
    }
}