package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.view.components.challenges.modals.StartChallengePopup;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.gallery.SearchInput;
    import com.utils.TextureUtil;

    import feathers.controls.Button;
    import feathers.controls.Label;

    import feathers.core.FeathersControl;
    import feathers.data.ListCollection;

    import flash.geom.Rectangle;

    import starling.animation.Transitions;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;

    public class SearchBox extends FeathersControl{
    private var _searchResult:SearchResultList;
    private var background:Quad;
    private var _searchResultCollection:ListCollection;
    private var _defaultCollection:ListCollection;
    private var _searchField:SearchInput;
    private var _searchProvider:Function;
    private var _customSearchProvider:Function;
    public var isWaiting:Boolean = false;
    private var _backButton:Button;
    private var targetY:Number;
    private var _bannerSize:Number = 0;
    private var _challengeLogo:Image;
    private var logoId:String;
    private var boxTypeId:String;
    private var boxTypeLabel:Label;
    private var boxTypeText:String;
    private var boxTypeIcon:Image;
    public function SearchBox(logoId:String, boxTypeId:String, boxTypeText:String) {
        _searchResult = new SearchResultList();
        _searchField = new SearchInput(true);
        this.logoId = logoId;
        this.boxTypeId = boxTypeId;
        this.boxTypeText = boxTypeText;
        searchProvider = null;
    }

    public function get length():int{
        _defaultCollection ||= _searchResult.dataProvider as ListCollection;
        if(_defaultCollection){
            return _defaultCollection.length;
        }
        return 0;
    }

    override protected function initialize():void {
        super.initialize();
        background = new Quad(1,1,0xFFFFFF);
        addChild(background);

        _challengeLogo ||= new Image(ButtonFactory.assetManager.getTexture(logoId));
        _challengeLogo.scale = ButtonFactory.assetManager.scaleFactor / 2;
        addChild(_challengeLogo);

        addChild(_searchResult);
        addChild(_searchField);
        var fieldBg:Image = new Image(ButtonFactory.assetManager.getTexture("active_label_bg"));
        var size:Number = StartChallengePopup.INPUT_SIZE;
        TextureUtil.scaleImage(fieldBg, 2*(size/fieldBg.texture.height)/ButtonFactory.assetManager.scaleFactor);
        var rectSize:Number = fieldBg.texture.height/2-0.5;
        fieldBg.scale9Grid = new Rectangle(rectSize, rectSize, fieldBg.texture.width - 2*rectSize, 1);
        _searchField.height = fieldBg.height;
        _searchField.addEventListener(Event.CHANGE, searchFieldChangeHandler);
        //
        _backButton = new Button();
        var clsImg:Image = new Image(ButtonFactory.assetManager.getTexture("back_button_clean"));
        _backButton.defaultSkin = clsImg;
        _backButton.setSize(27, 27);

        _backButton.x = actualWidth - (_backButton.width + 7);
        _backButton.y = 7;
        addChild(_backButton);
        _backButton.addEventListener(Event.TRIGGERED, backButtonTriggerHandler);

        boxTypeIcon = new Image(ButtonFactory.assetManager.getTexture(boxTypeId));
        boxTypeIcon.color = 0x7e7e7e;
        if(boxTypeIcon.width > boxTypeIcon.height){
            boxTypeIcon.width = 16;
            boxTypeIcon.scale = boxTypeIcon.scaleX;
        }else{
            boxTypeIcon.height = 12;
            boxTypeIcon.scale = boxTypeIcon.scaleY;
        }
        addChild(boxTypeIcon);

        boxTypeLabel = new Label();
        boxTypeLabel.textRendererFactory = LabelFactory.searchBoxTypeLabel;
        boxTypeLabel.text = boxTypeText;
        addChild(boxTypeLabel);
        boxTypeLabel.validate();
    }

        private function backButtonTriggerHandler(event:Event):void {
            close();
        }

    private function searchFieldChangeHandler(event:Event):void {
        Starling.juggler.removeDelayedCalls(commitSearch);
        Starling.juggler.delayCall(commitSearch, 1);
    }

    private function commitSearch():void{
        _searchResult.loadingItems = 0;
        if(_searchField.text.length){
            if(_customSearchProvider!=null){
                _defaultCollection ||= _searchResult.dataProvider as ListCollection;
                _customSearchProvider.apply(null, [_searchField.text]);
                return;
            }
            if(_searchProvider!=null) {
                _defaultCollection ||= _searchResult.dataProvider as ListCollection;
                _searchResultCollection = new ListCollection(_searchProvider.apply(null, [_searchField.text, _defaultCollection]));
                _searchResult.dataProvider = _searchResultCollection;
            }
        }else if(_defaultCollection){
            _searchResult.dataProvider = _defaultCollection;
        }
    }

    override protected function draw():void {
        super.draw();
        if(background){
            background.width = actualWidth;
            background.height = actualHeight;
        }

        if(_challengeLogo){
            _challengeLogo.x = (actualWidth - _challengeLogo.width) / 2;
            _challengeLogo.y = 20;
            _bannerSize = (_challengeLogo.height + 1.75 * _challengeLogo.y);
        }

        _searchField.width = actualWidth - 54;
        _searchField.x = actualWidth - (_searchField.height + 17);
        _searchField.y = _bannerSize - (_searchField.height + 10);
        _searchResult.width = actualWidth;
        _searchResult.height = actualHeight - _bannerSize;
        _searchResult.x = 0;
        _searchResult.y = _bannerSize;

        boxTypeIcon.x = 15;
        boxTypeIcon.y = _bannerSize - boxTypeIcon.height - 10;
        boxTypeLabel.x = boxTypeIcon.x + boxTypeIcon.width + 3;
        boxTypeLabel.y = boxTypeIcon.y + (boxTypeIcon.height - boxTypeLabel.height) / 2;
    }

    public function open(animate:Boolean = true):void{
        if(y != 0 && !Starling.juggler.containsTweens(this)){
            y = actualHeight;
            targetY = 0;
            applyState(animate);
        }
    }

    public function close(animate:Boolean = true):void{
        if(y != actualHeight && !Starling.juggler.containsTweens(this)) {
            y = 0;
            targetY = actualHeight;
            applyState(animate);
        }
    }

    public function toggle(animate:Boolean = true):void{
        if(targetY == actualHeight){
            targetY = 0;
        }else{
            targetY = actualHeight;
        }
        if(Starling.juggler.containsTweens(this)){
            Starling.juggler.removeTweens(this);
        }
        applyState(animate);
    }

    private function applyState(animate:Boolean):void{
        if(parent && parent.getChildIndex(this) < parent.numChildren -1){
            parent.setChildIndex(this, parent.numChildren - 1);
        }
        visible = true;
        if(animate){
            Starling.juggler.tween(this, 0.5, {y: targetY, transition: Transitions.EASE_OUT});
        }else{
            y = targetY;
        }
    }

    public function get searchResult():SearchResultList {
        return _searchResult;
    }

    public function set searchProvider(value:Function):void {
        if(value == null){
            _searchProvider = function (str:String = null, collection:ListCollection = null):Array {
                return null;
            };
            return;
        }
        _searchProvider = value;
    }

    public function set customSearchProvider(value:Function):void {
        _customSearchProvider = value;
    }

    public function get searchField():SearchInput {
        return _searchField;
    }

    override public function dispose():void{
        super .dispose();
        if(_challengeLogo){
            _challengeLogo.removeFromParent(true);
            _challengeLogo = null;
        }
    }

        public function get isOpen():Boolean {
            return targetY == 0;
        }

        public function get bannerSize():Number {
            return _bannerSize;
        }
    }
}
