package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.utils.FontSelector;

    import feathers.controls.Label;
    import feathers.core.FeathersControl;

    import flash.display.BitmapData;
    import flash.display.CapsStyle;
    import flash.display.JointStyle;
    import flash.display.LineScaleMode;
    import flash.display.Shape;
    import flash.text.TextFormatAlign;

    import starling.display.Image;
    import starling.textures.Texture;
    import starling.utils.MathUtil;

    public class UserLevelAvatar extends FeathersControl {
        private var _levelProgress:Image;
        private var _levelLabel:Label;
        private var _avatarDot:Image;
        private var _border:Image;
        private var _size:Number;
        public function UserLevelAvatar(size:int) {
            super();
            _size = size;
            pivotX = _size / 2;
            pivotY = _size / 2;
            width = height = size;
        }

        override protected function initialize():void{
            super .initialize();
            _border = new Image(ButtonFactory.assetManager.getTexture('avatar_border'));
            _border.width = _border.height = size;
            _border.color = 0x343434;
            addChild(_border);

            _avatarDot = new Image(ButtonFactory.assetManager.getTexture('avatar_background'));
            _avatarDot.height = 6;
            _avatarDot.scale = _avatarDot.scaleY;
            _avatarDot.color = 0xf9ae2c;
            addChild(_avatarDot);
        }

        public function update(value:User):void{
            var progress:Number = (value.submittedChallenges.length - value.levelData.minChallengeCount) / (value.levelData.nextMinChallengeCount - value.levelData.minChallengeCount);
            var toAngle:Number = 2 * Math.PI * progress;
            var startAngle:Number = Math.PI / 2;
            var angle:Number = 0;
            var r:Number =  5 * (_size / 2 + 4);
            var s:Number = 5 * _size / 2;
            var shape:Shape = new Shape();
            shape.graphics.beginFill(0x000000, 0.75);
            shape.graphics.drawCircle(r, r, s - 5);
            shape.graphics.endFill();

            var fromX:Number = r + s * Math.cos(angle + startAngle);
            var fromY:Number = r + s * Math.sin(angle + startAngle);
            shape.graphics.lineStyle(s / 8, 0xf8a719, 1, true, LineScaleMode.NORMAL, CapsStyle.SQUARE, JointStyle.ROUND);
            shape.graphics.moveTo(fromX, fromY);
            while (angle < toAngle){
                angle += 1 / 16;
                shape.graphics.lineTo(r + s * Math.cos(angle + startAngle), r + s * Math.sin(angle + startAngle));
            }
            var bd:BitmapData = new BitmapData(2 * r, 2 * r, true, 0x00000000);
            bd.draw(shape, null, null, null, null, true);
            if(_levelProgress){
                _levelProgress.removeFromParent(true);
            }
            _levelProgress = new Image(Texture.fromBitmapData(bd));
            _levelProgress.width = _size + 6;
            _levelProgress.height = _size + 6;
            addChildAt(_levelProgress, MathUtil.clamp(getChildIndex(_avatarDot), 0, numChildren));
            _levelProgress.x = (_size - _levelProgress.width) / 2;
            _levelProgress.y = (_size - _levelProgress.height) / 2;

            _border.x = (_size - _border.width) / 2;
            _border.y = (_size - _border.height) / 2;

            _levelLabel ||= LabelFactory.createSimpleLabel(FontSelector.getAlumniSansSemiBold(), 10, 0xFFFFFF, TextFormatAlign.CENTER, false, true);
            _levelLabel.text = '<textformat leading="-7"><font size="19">' + value.levelData.level.toString() + '</font>\nLEVEL<textformat>';
            addChildAt(_levelLabel, MathUtil.clamp(getChildIndex(_avatarDot), 0, numChildren));

            bd.dispose();
            bd = null;
            shape = null;
        }

        override protected function draw():void{
            super .draw();
            if(_levelLabel){
                _levelLabel.validate();
                _levelLabel.x = (_size - _levelLabel.width) / 2;
                _levelLabel.y = (_size - _levelLabel.height) / 2 - 1;
            }

            if(_avatarDot){
                _avatarDot.x = (_size - _avatarDot.width) / 2;
                _avatarDot.y = _size - (_avatarDot.height + 1) / 2;
            }
        }

        public function get size():Number {
            return _size;
        }
    }
}
