package com.despark.tuning3d.view.components.challenges.modals {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.cars.Car;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.model.friend.Friend;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeViewUserLabel;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.IMainMenuHolder;
import com.despark.tuning3d.view.components.core.MainMenuView;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.core.SimpleExtendedButton;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.timeline.TimelineGarage.TimelineAddFriendButton;
import com.despark.tuning3d.view.components.timeline.TimelineGarage.TimelineTuningViewItemRenderer;
import com.despark.tuning3d.view.components.tunings.ITuningListMenuVIew;
import com.despark.tuning3d.view.components.tunings.TuningsListMenuView;
import com.despark.tuning3d.view.components.tunings.TuningsListScreen;

import feathers.controls.Button;
import feathers.controls.List;
import feathers.controls.ScrollBarDisplayMode;
import feathers.controls.ScrollPolicy;
import feathers.controls.renderers.IListItemRenderer;
import feathers.core.FeathersControl;
import feathers.data.ListCollection;
import feathers.layout.AnchorLayout;
import feathers.layout.AnchorLayoutData;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;

import starling.core.Starling;
import starling.display.Image;
import starling.display.Image;
import starling.display.Sprite;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class ChallengeNotificationTuningScreen extends PopUp implements Multilingual, IMainMenuHolder, ITuningListMenuVIew {
    private var _mainMenuView : MainMenuView;
    private var _languages:Array;
    private var _selectedLanguage:Language;
    private var _selectedTuning:Tuning;
    private var _tuningList:List;
    private var _snapshots : Array;
    private var pageIndicatorContainer:Sprite;
    private var tuningsListMenuView:TuningsListMenuView;
    private var _challengesList:Object;
    private const _gap:Number = 6;
    private var _selectedUser:User;
    private var _currentUser:User;
    private var _userLabel:ChallengeViewUserLabel;
    private var _container:MainContainer;
    private var _closeHandler:Function;
    public function ChallengeNotificationTuningScreen() {
        var usersProxy:UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
        _container = new MainContainer();
        _currentUser = usersProxy.currentUser;
        _snapshots = [];
    }

    override protected function initialize():void {
        super.initialize();
        setSizeInternal(stage.stageWidth, stage.stageHeight, false);

        if(contentBackgroundImage){
            contentBackgroundImage.removeFromParent();
        }

        background.color = 0xFFFFFF;
        background.alpha = 1;
        _container.setSize(actualWidth, actualHeight);

        var layout : HorizontalLayout = new HorizontalLayout();
        layout.gap = 0;
        layout.horizontalAlign = HorizontalAlign.CENTER;
        layout.verticalAlign = VerticalAlign.MIDDLE;

        _tuningList = new List();
        _tuningList.minimumPageThrowVelocity = 1;
        _tuningList.dataProvider = new ListCollection([]);
        _tuningList.layout = layout;
        _tuningList.snapToPages = true;
        _tuningList.hasElasticEdges = false;
        _tuningList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
        _tuningList.horizontalScrollPolicy = ScrollPolicy.ON;
        _tuningList.addEventListener(Event.SCROLL, listScrollHandler);
        _tuningList.itemRendererFactory = slideItemRendererFactory;
        _container.addChild(_tuningList);

        tuningsListMenuView = new TuningsListMenuView(this);
        tuningsListMenuView.addEventListener(TuningsListMenuView.CAR_SELECTED, onCarSelected);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_SELECTED, onTuningSelected);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_SHARE, onTuningShare);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_DELETE, onTuningDelete);
        tuningsListMenuView.addEventListener(TuningsListMenuView.START_CHALLENGE, challengeHandler);
        _container.addChild(tuningsListMenuView);
        if(background.parent && background.parent == this){
            addChildAt(_container, getChildIndex(background)+1);
        }

        var closeIcon:Image = new Image(ButtonFactory.assetManager.getTexture('garage_close'));
        closeIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        closeButton.width = closeIcon.width;
        closeButton.height = closeIcon.height;
        closeButton.defaultIcon = closeIcon;
        _container.addChild(closeButton);
    }

    private function challengeHandler(event:Event):void{
        var tuning:Tuning = (selectedTuning && !Constants.selectedUser)?selectedTuning:null;
        var data:Object  = ChallengeEvent.getData(_currentUser, tuning, Constants.selectedUser, null);
        dispatchEvent(new Event(ChallengeEvent.CHALLENGE_REQUEST, false, data));
    }

    protected function onCarSelected(event:Event):void {
        dispatchEventWith(TuningsListMenuView.CAR_SELECTED);
    }

    override protected function closeButtonTriggeredHandler(event:Event):void {
        super.closeButtonTriggeredHandler(event);
        if(_closeHandler != null){
            _closeHandler.call();
        }
    }

    protected function onTuningSelected(event:Event):void {
        _closeHandler = null;
        closeButtonTriggeredHandler(null);
        dispatchEventWith(StackedScreen.TUNING_SELECTED);
        getSocialData();
    }

    protected function onTuningShare(event:Event):void{
        dispatchEventWith(TuningsListMenuView.TUNING_SHARE);
    }

    protected function onTuningDelete(event:Event):void{
        dispatchEventWith(TuningsListMenuView.TUNING_DELETE, false, event.data);
    }

    public function set likes(value:int) : void {
        tuningsListMenuView.likes = value;
    }

    override protected function draw():void {
        super.draw();
        if (pageIndicatorContainer) {
            pageIndicatorContainer.x = (actualWidth - pageIndicatorContainer.width) / 2;
            pageIndicatorContainer.y = (actualHeight - tuningsListMenuView.height)-(pageIndicatorContainer.height + 9);
        }
        if(_tuningList){
            _tuningList.setSize(actualWidth, actualHeight);
            initPagingIcons();
        }

        if(closeButton){
            if(!closeButton.parent){
                addChild(closeButton);
            }
            closeButton.visible = true;
            closeButton.x = actualWidth - (closeButton.width + 9);
            closeButton.y = 9;
        }

        if(tuningsListMenuView) {
            tuningsListMenuView.width = actualWidth;
            tuningsListMenuView.height = 50;
            tuningsListMenuView.y = actualHeight - tuningsListMenuView.height;
            tuningsListMenuView.parentHeight = actualHeight;
        }

        if(_userLabel){
            _userLabel.x = _gap;
            _userLabel.y = _gap;
        }
    }

    private function listScrollHandler(event:Event) : void {
        initPagingIcons();
    }

    private function clearPagePageIndicators():void{
        if(pageIndicatorContainer){
            while (pageIndicatorContainer.numChildren){
                pageIndicatorContainer.removeChildAt(0, true);
            }
        }
    }

    public function initPagingIcons() : void {
        var size:int = 15;
        var lineCount:int = int((actualWidth-230) / size);
        if (!pageIndicatorContainer) {
            pageIndicatorContainer = new Sprite();
            _container.addChild(pageIndicatorContainer);
            pageIndicatorContainer.touchable = false;
        }

        if (pageIndicatorContainer.numChildren > 0) {
            var item  : Sprite;
            for (var i : int = 0; i < pageIndicatorContainer.numChildren; i++) {
                item = (pageIndicatorContainer.getChildAt(i) as Sprite);
                item.getChildAt(1).visible = (_tuningList.horizontalPageIndex == i);
                item.x = (i % lineCount) * size;
                item.y = int(i / lineCount) * size;
            }
        } else {
            var stateContainer:Sprite;
            var state1Image : Image;
            var state2Image : Image;
            for (var ii:int = 0; ii < _tuningList.dataProvider.length; ii++) {
                stateContainer = new Sprite();
                state1Image = new Image(ButtonFactory.assetManager.getTexture("pageIndicator"));
                state2Image = new Image(ButtonFactory.assetManager.getTexture("pageIndicatorSelected"));
                state1Image.width = state1Image.height = 9;
                state2Image.width = state2Image.height = 9;
                stateContainer.addChild(state1Image);
                stateContainer.addChild(state2Image);
                if (ii != 0) {
                    stateContainer.getChildAt(1).visible = false;
                }
                stateContainer.x = (ii % lineCount) * size;
                stateContainer.y = int(ii / lineCount) * size;
                pageIndicatorContainer.addChild(stateContainer);
            }
        }
    }

    private function slideItemRendererFactory() : IListItemRenderer {
        var renderer :TimelineTuningViewItemRenderer = new TimelineTuningViewItemRenderer(null);
        var item_w:Number=actualWidth;
        var item_h:Number=actualHeight;
        renderer.setSize(item_w, item_h);
        return renderer;
    }

    private function labelTouchHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(_userLabel, TouchPhase.ENDED);
        if(touch){
            dispatchEvent(new Event(StackedScreen.USER_DETAILS_SELECTED, false, {user: _selectedUser}));
        }
    }

    public function selectTuning(tuning:Tuning, user:User):void{
        _selectedTuning = tuning;
        _selectedUser = user;
        _userLabel ||= new ChallengeViewUserLabel();
        _selectedTuning.car ||= Car.jsonToCar(_selectedTuning.carObj);

        _userLabel ||= new ChallengeViewUserLabel();
        if(!_userLabel.hasEventListener(TouchEvent.TOUCH)){
            _userLabel.addEventListener(TouchEvent.TOUCH, labelTouchHandler);
        }
        _userLabel.num = "";
        _userLabel.user = _selectedUser;
        if((_currentUser && _selectedUser && _selectedUser.id != _currentUser.id) || !_currentUser){
            Constants.selectedUser = _selectedUser;
        }else{
            Constants.selectedUser = null;
        }

        if(isInitialized) {
            clearPagePageIndicators();
            _snapshots = getTuningSnapshots(_selectedTuning);
            _tuningList.dataProvider = new ListCollection(_snapshots);
             initPagingIcons();

            tuningsListMenuView.currentTuning = _selectedTuning;
            tuningsListMenuView.setCarName();
            tuningsListMenuView.currentTuningID = _selectedTuning.id;
            if (_selectedTuning.car) {
                tuningsListMenuView.currentCarID = _selectedTuning.car.id;
            }

            if(_userLabel){
                _container.addChild(_userLabel);
            }
            invalidate();
            validate();
        }else{
            Starling.juggler.delayCall(function ():void {
                selectTuning(tuning, user);
            }, 0.1);
        }
    }

    private static function getTuningSnapshots(tuning:Tuning):Array{
        return (tuning.fullHdSnapshots && tuning.fullHdSnapshots.length)?tuning.fullHdSnapshots:tuning.snapshots;
    }

    private function getSocialData() : void {
        if (!Constants.selectedUser) {
            return;
        }
        dispatchEventWith(TuningsListScreen.GET_SOCIAL_DATA);
    }

    public function set comments(value:Array) : void {
        tuningsListMenuView.initComments = value;
    }

    public function getVisibleTuningURL():String {
        return _snapshots[_tuningList.horizontalPageIndex];
    }

    public function set languages(value:Array):void{
        _languages=value;
    }

    public function get languages():Array{
        return _languages;
    }

    public function brandsScreenButtonTriggeredHandler(event:Event = null):void {
        dispatchEventWith(StackedScreen.BRANDS_SELECTED);
    }

    public function homeButtonTriggeredHandler(event:Event = null):void {
        dispatchEventWith(StackedScreen.HOME_SELECTED);
    }

    public function enterRowsState():void {

    }

    public function set mainMenuView(value : MainMenuView) : void {
        _mainMenuView = value;
        _mainMenuView.owner = this;
    }

    public function get mainMenuView():MainMenuView {
        return _mainMenuView;
    }

    public function set selectedLanguage(value:Language):void {
        _selectedLanguage = value;
    }

    public function get selectedLanguage():Language {
        return _selectedLanguage;
    }

    public function dispatchExternalEvent(msg:String):void {
        if (mainMenuView) {
            mainMenuView.hide();
        }
        dispatchEventWith(msg);
    }

    public function updateWithLanguage(language:Language):void {
        if (mainMenuView) {
            mainMenuView.updateWithLanguage(language);
        }
    }

    override public function dispose():void {
        super.dispose();
        if(_mainMenuView){
            _mainMenuView.removeFromParent(true);
            _mainMenuView = null;
        }
        if(_tuningList){
            _tuningList.removeFromParent(true);
            _tuningList = null;
        }
        if(pageIndicatorContainer){
            clearPagePageIndicators();
            pageIndicatorContainer.removeFromParent(true);
            pageIndicatorContainer = null;
        }
        if(tuningsListMenuView){
            tuningsListMenuView.removeFromParent(true);
            tuningsListMenuView = null;
        }
        if(_userLabel){
            _userLabel.removeFromParent(true);
            _userLabel = null;
        }
        if(_container){
            _container.removeFromParent(true);
            _container = null;
        }
        _languages = null;
        _selectedLanguage = null;
        _selectedTuning = null;
        _snapshots = null;
        _challengesList = null;
        _selectedUser = null;
        _currentUser = null;
    }

    public function get selectedTuning():Tuning {
        return _selectedTuning;
    }

    public function set closeHandler(value:Function):void {
        _closeHandler = value;
    }
}
}

import feathers.core.FeathersControl;

class MainContainer extends FeathersControl{
    public function MainContainer(){
        super ();
    }
}