package com.despark.tuning3d.view.components.challenges.modals {
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.UIStrings;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusInVoting;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusLabel;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusTimeTrial;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeTuningView;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengesTuningSideSelector;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.timeline.components.TimelineLoadBar;
    import com.despark.tuning3d.view.components.tunings.ShareView;
    import com.utils.DateParser;
    import com.utils.FontSelector;
    import com.utils.NumberUtils;
    import com.utils.ObjectUtils;

    import feathers.controls.Button;
    import feathers.controls.Label;

    import flash.display.StageOrientation;
    import flash.events.StageOrientationEvent;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;

    public class ChallengeTuningViewPopup extends PopUp {
        private var _tuningView:ChallengeTuningView;
        private var _onClose:Function;
        private var _data:Object;
        private var _nextButton:Button;
        private var _prevButton:Button;
        private var _shareButton:Button;
        private var _shareView:ShareView;
        private var _loaderBar:TimelineLoadBar;
        private var _ratingLabel:ExtendedLabel;
        private var _votingLabel:ChallengeStatusLabel;
        private var _nameLabel:Label;

        private var _sideSelector:ChallengesTuningSideSelector;
        private var currentIndex:int;

        private var _challengeData:ChallengeData;

        private static const MARGIN:int = 15;
        private static const labels:Object = {sec:["s"], min:["m"], hr:["h"], day:["d"]};
        public function ChallengeTuningViewPopup(data:Object = null) {
            super();
            _data = data;
            var challengeHostId:* = ObjectUtils.getValue(_data, "stats.challengeHostId", {});
            if(challengeHostId is String){
                challengeHostId = {_id:challengeHostId};
            }
            challengeHostId = ObjectUtils.merge(challengeHostId, _data.stats);
            _challengeData = new ChallengeData(challengeHostId);
            if(_data.tuning && _data.tunings){
                currentIndex = getObjectIndex(_data.tuning, _data.tunings);
            }else{
                currentIndex = 0;
            }

            _tuningView = new ChallengeTuningView(data);
            _tuningView.addEventListener(ChallengeTuningView.INDEX_CHANGED, tuningPageChangeHandler);

        }

        private function tuningPageChangeHandler(event:Event):void {
            _sideSelector.toggleGroup.selectedIndex = event.data.page;
        }

        override protected function initialize():void {
            super.initialize();
            setSizeInternal(stage.stageWidth, stage.stageHeight, false);
            var closeIcon:Image = new Image(ButtonFactory.assetManager.getTexture('garage_close'));
            closeIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
            closeButton.width = closeIcon.width;
            closeButton.height = closeIcon.height;
            closeButton.defaultIcon = closeIcon;

            background.color = 0xFFFFFF;
            background.alpha = 1;
            contentBackgroundImage.visible = false;

            _prevButton = ButtonFactory.simpleButtonByTextureName("slide_lt_arr", true, 0xFF0000, false);
            _nextButton = ButtonFactory.simpleButtonByTextureName("slide_lt_arr", true, 0xFF0000, false, true);

            _prevButton.addEventListener(Event.TRIGGERED, prevButtonHandler);
            _nextButton.addEventListener(Event.TRIGGERED, nextButtonHandler);
            _prevButton.alpha = 1;
            _nextButton.alpha = 1;

            if(!_data.hasOwnProperty("tunings") || !_data.tunings.length){
                _nextButton.visible = _prevButton.visible = false;
            }

            restoreColor();

            /*_shareButton = ButtonFactory.circleButton('a_arrow_right_button');
            ExtentedButton(_shareButton).color = 0x000000;
            _shareButton.addEventListener(Event.TRIGGERED, shareButtonTriggeredHandler);

            _shareButton.width = 35;
            _shareButton.height = 35;*/

            _shareView = new ShareView();
            _shareView.color = 0x000000;
            _shareView.setSize(35, (35 * 3) + (7.5 * 2));
            _shareView.addEventListener(ShareView.SHARE_SELECTED, onShareSelected);

            var textFormat:TextFormat = new TextFormat();
            textFormat.leading = -5;
            _nameLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansSemiBold(), 20, 0xFFFFFF, TextFormatAlign.LEFT, false, true, textFormat);
            _nameLabel.minWidth = 160;
            _nameLabel.backgroundSkin = new Quad(1, 1, 0x000000);
            _nameLabel.backgroundSkin.alpha = 0.65;
            _nameLabel.paddingLeft = _nameLabel.paddingRight = 14;
            _nameLabel.paddingBottom = 3;
            _nameLabel.text = '<font>' + _data.tuning.name + '</font><br><font size="12">' + UIStrings.getCarName(_data.tuning.car)+ '</font>';


            _ratingLabel = new ExtendedLabel();
            _ratingLabel.backgroundSkin = new Image(ButtonFactory.assetManager.getTexture("challenge_tuning_rating_bg"));
            _ratingLabel.width = 0.85 * 48.5 / 0.6;
            _ratingLabel.height = 0.85 * 22 / 0.6;
            _ratingLabel.textRendererFactory = LabelFactory.challengeTuningPopupRatingLabel;
            if(_data.stats && _data.stats.rating){
                _ratingLabel.text = NumberUtils.toFixed(parseFloat(_data.stats.rating),2);
            }else if(_data.rating){
                _ratingLabel.text = NumberUtils.toFixed(parseFloat(_data.rating),2);
            }
            _ratingLabel.visible = !_challengeData.isTimeTrial && (!_ratingLabel.text || _ratingLabel.text.indexOf('-1') == -1);
            if(!_ratingLabel.visible){
                if(_challengeData.isTimeTrial){
                    _votingLabel = new ChallengeStatusTimeTrial();
                    _votingLabel.text = _challengeData.timeTrialsPercent.toString() + "% / " + DateParser.getInChallengeLabel(_challengeData.timeTrialsSeconds * 1000, labels, 2, false);
                    _votingLabel.scale = 1.25;
                }else{
                    _votingLabel = new ChallengeStatusInVoting();
                    var currentDate:Date = new Date();
                    var votingFinishDate:Date = DateParser.parseDate(_data.stats.challengeHostId.votingFinishDate);
                    _votingLabel.text = "RESULTS IN " + DateParser.getInChallengeLabel(votingFinishDate.time - currentDate.time);
                    _votingLabel.scale = 1.25;
                }
            }

            var ratingIcon:LayoutImage = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));
            ratingIcon.scale = 0.8 * ButtonFactory.assetManager.scaleFactor / 2;
            _ratingLabel.defaultIcon = ratingIcon;
            _ratingLabel.labelOffsetX = 5;
            _ratingLabel.labelOffsetY = 1;
            _ratingLabel.paddingLeft = 12;
            _ratingLabel.paddingBottom = 2;

            _sideSelector = new ChallengesTuningSideSelector();
            _sideSelector.toggleGroup.addEventListener(Event.CHANGE, viewChangeHandler);

            _tuningView.contentScale = Constants.TUNING_SCALE_FACTOR;
            addChild(_tuningView);
            addChild(closeButton);
            //addChild(_shareButton);

            addChild(_prevButton);
            addChild(_nextButton);

            addChild(_nameLabel);
            addChild(_ratingLabel);
            if(_votingLabel){
                addChild(_votingLabel);
            }

            addChild(_sideSelector);
            _sideSelector.validate();

            _loaderBar ||= new TimelineLoadBar();
            _loaderBar.name = '_timelineLoadBar';
            _loaderBar.setSize(60, 60);
            _loaderBar.x = Constants.STAGE_WIDTH / 2 - _loaderBar.width / 2;
            _loaderBar.y = Constants.STAGE_HEIGHT / 2 - _loaderBar.height / 2;
            _loaderBar.visible = false;
            addChild(_loaderBar);


            if(Device.isIphoneXVariant()){
                updateOrientation();
                Starling.current.nativeStage.addEventListener(StageOrientationEvent.ORIENTATION_CHANGE, nativeStage_orientationChangeHandler);
            }
        }

        protected function nativeStage_orientationChangeHandler(event:StageOrientationEvent):void {
            updateOrientation();
        }

        private function updateOrientation():void{
            var marginSize:Number = 24;
            if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_RIGHT){
                _prevButton.x = marginSize;
                _nextButton.x = actualWidth - _nextButton.width;
            }else if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_LEFT){
                _prevButton.x = 0;
                _nextButton.x = actualWidth - (_nextButton.width + marginSize);
            }else{
                _prevButton.x = 0;
                _nextButton.x = actualWidth - _nextButton.width;
            }
        }

        private function viewChangeHandler(event:Event):void {
            _tuningView.page = _sideSelector.toggleGroup.selectedIndex;
        }

        private function shareButtonTriggeredHandler(event:Event):void {
            if (_shareView.parent) {
                SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
                _shareView.removeFromParent();
                return;
            }
            SoundManager.playSound(SoundManager.PART_MENU);
            addChild(_shareView);
            _shareView.x = _shareButton.x - (_shareView.width - _shareButton.width);
            _shareView.y = _shareButton.y - (_shareView.height + _shareButton.height + 7.5);
        }

        override public function dispose():void {
            super.dispose();
            if(Device.isIphoneXVariant()){
                Starling.current.nativeStage.removeEventListener(StageOrientationEvent.ORIENTATION_CHANGE, nativeStage_orientationChangeHandler);
            }

            if (_tuningView) {
                _tuningView.removeFromParent(true);
                _tuningView = null;
            }
            if (_prevButton) {
                _prevButton.removeFromParent(true);
                _prevButton = null;
            }
            if (_nextButton) {
                _nextButton.removeFromParent(true);
                _nextButton = null;
            }
            if (_shareView) {
                _shareView.removeFromParent(true);
                _shareView = null;
            }
            if (_shareButton) {
                _shareButton.removeFromParent(true);
                _shareButton = null;
            }
            if (_loaderBar) {
                _loaderBar.removeFromParent(true);
                _loaderBar = null;
            }
        }

        private function nextButtonHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            if(currentIndex < _data.tunings.length - 1){
                currentIndex++;
                var data:Object = _data.tunings[currentIndex];
                data.user = _data.user;
                updateTuningData(data);
            }
        }

        private function prevButtonHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART_BACK);
            if(currentIndex > 0){
                currentIndex--;
                var data:Object = _data.tunings[currentIndex];
                data.user = _data.user;
                updateTuningData(data);
            }
        }

        private static function getObjectIndex(tuning:Object, dataList:Array):int{
            for(var i:int = 0; i < dataList.length; i++){
                if(tuning == dataList[i].tuning){
                    return i;
                }
            }
            return 0;
        }

        private function updateTuningData(challengeData:Object):void{
            if(_tuningView){
                var childIndex:int = getChildIndex(_tuningView);
                _tuningView.removeEventListeners();
                _tuningView.removeFromParent(true);
            }
            _tuningView = new ChallengeTuningView(challengeData);
            _tuningView.contentScale = Constants.TUNING_SCALE_FACTOR;
            _tuningView.addEventListener(ChallengeTuningView.INDEX_CHANGED, tuningPageChangeHandler);
            addChildAt(_tuningView, childIndex);

            if(_ratingLabel){
                if(challengeData.stats && challengeData.stats.rating){
                    _ratingLabel.text = NumberUtils.toFixed(parseFloat(challengeData.stats.rating),2);
                }else if(challengeData.rating){
                    _ratingLabel.text = NumberUtils.toFixed(parseFloat(challengeData.rating),2);
                }
                _ratingLabel.visible = (!_ratingLabel.text || _ratingLabel.text.indexOf('-1') == -1);
                if(_votingLabel){
                    _votingLabel.visible = !_ratingLabel.visible;
                }
            }

            if(!_ratingLabel.visible){
                _votingLabel ||= new ChallengeStatusInVoting();
                var currentDate:Date = new Date();
                var votingFinishDate:Date = DateParser.parseDate(challengeData.stats.challengeHostId.votingFinishDate);
                _votingLabel.text = "RESULTS IN " + DateParser.getInChallengeLabel(votingFinishDate.time - currentDate.time);
                _votingLabel.scale = 1.25;
                addChild(_votingLabel);
            }
            if(_nameLabel){
                _nameLabel.text = '<font>' + challengeData.tuning.name + '</font><br><font size="12">' + UIStrings.getCarName(challengeData.tuning.car)+ '</font>';
            }

            _sideSelector.toggleGroup.removeEventListener(Event.CHANGE, viewChangeHandler);
            _sideSelector.toggleGroup.selectedIndex = 0;
            _sideSelector.toggleGroup.addEventListener(Event.CHANGE, viewChangeHandler);

            invalidate();
        }

        private function restoreColor():void {
            Image(_nextButton.defaultSkin).color = 0xFF0000;
            Image(_prevButton.defaultSkin).color = 0xFF0000;
        }

        override protected function draw():void {
            super.draw();
            if (_tuningView) {
                _tuningView.width = stage.stageWidth;
                _tuningView.height = stage.stageHeight;
            }
            closeButton.x = stage.stageWidth - (closeButton.width + MARGIN);
            closeButton.y = MARGIN;

            if (_nextButton) {
                _nextButton.validate();
                _nextButton.x = stage.stageWidth - _nextButton.width;
                _nextButton.y = (stage.stageHeight - _nextButton.height) / 2;
            }
            if (_prevButton) {
                _prevButton.validate();
                _prevButton.x = 0;
                _prevButton.y = (stage.stageHeight - _prevButton.height) / 2;
            }
            if (_shareButton) {
                _shareButton.x = stage.stageWidth - (_shareButton.width + MARGIN);
                _shareButton.y = stage.stageHeight - (_shareButton.height + MARGIN);
            }

            if (_nameLabel) {
                _nameLabel.validate();
                _nameLabel.x = 0;
                _nameLabel.y = stage.stageHeight - (_nameLabel.height + MARGIN);
                if (_ratingLabel) {
                    _ratingLabel.x = 0;
                    _ratingLabel.y = _nameLabel.y - _ratingLabel.height + 5;
                }
                if(_votingLabel){
                    _votingLabel.x = 0;
                    _votingLabel.y = _nameLabel.y - _votingLabel.height;
                }
            }

            if(_sideSelector){
                _sideSelector.x = (actualWidth - _sideSelector.width) / 2;
                _sideSelector.y = _nameLabel.y + (_nameLabel.height - _sideSelector.height) / 2;
            }

            if(Device.isIphoneXVariant()){
                updateOrientation();
            }
        }

        private function onShareSelected(event:Event):void {
            _shareView.removeFromParent();
        }

        override protected function closeButtonTriggeredHandler(event:Event):void {
            super.closeButtonTriggeredHandler(event);
            SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            if (_onClose != null) {
                _onClose.call();
            }
        }

        public function get tuningView():ChallengeTuningView {
            return _tuningView;
        }
    }
}
