package com.despark.tuning3d.view.components.challenges.modals {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.challenges.ChallengesListScreen;
    import com.despark.tuning3d.view.components.challenges.ChallengesScreen;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeModelsListItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusFinished;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusInChallenge;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusInVoting;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.despark.tuning3d.view.components.core.LayoutQuad;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedBannerScreen;
    import com.distriqt.AdsManager;
    import com.utils.DateParser;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.layout.FlowLayout;
    import feathers.layout.VerticalAlign;

    import flash.text.TextFormatAlign;

    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.filters.ColorMatrixFilter;
    import starling.textures.Texture;

    public class EnterChallengePopup extends PopUp {
        private var _data:Object;
        private var _timeLabel:ExtendedLabel;
        private var _carTypeLabel:ExtendedLabel;
        //
        private var _detailsLabel:Label;
        private var _descTitleLabel:Label;
        private var _descLabel:Label;

        private var _reqTitleLabel:Label;
        private var _reqLabel:Label;


        private var _costLabel:Label;
        private var _costValueLabel:ExtendedLabel;
        private var _earnLabel:Label;
        private var _earnValueLabel:ExtendedLabel;


        private var _rewardValueLabels:Vector.<ExtendedLabel>;
        private var _keyValueLabels:Vector.<ExtendedLabel>;
        private var _earnBorders:Vector.<LayoutQuad>;
        private var _earnBorder:LayoutQuad;

        private var _enterButton:Button;
        //
        private var _carsList:LayoutGroup;
        //
        public static const ENTER_CHALLENGE:String = "EnterChallengePopup:enter_challenge";
        public static const ENTER_CHALLENGE_GARAGE:String = "EnterChallengePopup:enter_challenge_garage";
        public static const ENTER_CHALLENGE_VOTING:String = "EnterChallengePopup:enter_challenge_voting";
        public static const ENTER_CHALLENGE_GALLERY:String = "EnterChallengePopup:enter_challenge_gallery";
        private var _challengeData:ChallengeData;
        private var _user:User;
        public function EnterChallengePopup(data:Object) {
            super();
            _data = data;
            _user = _data.user;
            _challengeData = new ChallengeData(_data);
            paddingTop = StackedBannerScreen.marginTop;
        }

        override protected function initialize():void{
            super .initialize();

            SoundManager.playSound(SoundManager.PART_MENU);

            contentBackgroundImage.scale9Grid = null;
            contentBackgroundImage.texture = Texture.fromColor(1, 1, 0xFFFFFF);
            addChild(contentBackgroundImage);
            setSizeInternal(stage.stageWidth, stage.stageHeight - (StackedBannerScreen.marginTop + ChallengesScreen.marginBottom), false);

            var colorFilter:ColorMatrixFilter = new ColorMatrixFilter();
            colorFilter.tint(0x424242, 1);
            closeButton.defaultIcon.width = 9;
            closeButton.defaultIcon.scale = closeButton.defaultIcon.scaleX;
            closeButton.defaultIcon.filter = colorFilter;
            addChild(closeButton);

            titleLabel.textRendererFactory = LabelFactory.challengeListTitleLabel;
            addChild(titleLabel);
            titleLabel.text = _data.name;
            titleLabel.touchable = false;



            var currentDate:Date = new Date();
            var timeLabelBgColor:uint;
            var enterLabel:String;
            var labels:Object = {sec:["SECOND", "SECONDS"], min:["MINUTE", "MINUTES"], hr:["HOUR", "HOURS"], day:["DAY", "DAYS"]};

            if(_challengeData.status == ChallengeData.IN_CHALLENGE){
                if(_timeLabel && !(_timeLabel is ChallengeStatusInChallenge)){
                    _timeLabel.removeFromParent(true);
                    _timeLabel = null;
                }

                if(challengeData.isTimeTrial){
                    timeLabelBgColor = ChallengeStatusInChallenge.TIME_TRIAL_COLOR;
                }else if(challengeData.isFromGarage){
                    timeLabelBgColor = ChallengeStatusInChallenge.FROM_GARAGE_COLOR;
                }else{
                    timeLabelBgColor = ChallengeStatusInChallenge.MAIN_COLOR;
                }

                if(!_timeLabel){
                    _timeLabel = new ChallengeStatusInChallenge(timeLabelBgColor);
                }else{
                    Quad(_timeLabel.backgroundSkin).color = timeLabelBgColor;
                }
                _timeLabel.text = DateParser.getInChallengeLabel( _challengeData.finishDate.time - currentDate.time, labels);
                if(_challengeData.isTimeTrial){
                    enterLabel = "START CHALLENGE";
                }else if(_challengeData.isFromGarage){
                    enterLabel = "GO TO GARAGE";
                }else if(_challengeData.cars){
                    enterLabel = "CHOOSE YOUR MODEL";
                }else{
                    enterLabel = "ENTER CHALLENGE";
                }

            }else if(_challengeData.status == ChallengeData.IN_VOTING){
                if(_timeLabel && !(_timeLabel is ChallengeStatusInVoting)){
                    _timeLabel.removeFromParent(true);
                    _timeLabel = null;
                }
                if(!_timeLabel){
                    _timeLabel = new ChallengeStatusInVoting();
                }
                _timeLabel.text = DateParser.getInChallengeLabel( _challengeData.votingFinishDate.time - currentDate.time);
                enterLabel = "GO TO VOTING";
                timeLabelBgColor = 0x267596;
            }else if(_challengeData.status == ChallengeData.FINISHED){
                if(_timeLabel && !(_timeLabel is ChallengeStatusFinished)){
                    _timeLabel.removeFromParent(true);
                    _timeLabel = null;
                }
                if(!_timeLabel){
                    _timeLabel = new ChallengeStatusFinished();
                }
                _timeLabel.text = "FINISHED";
                enterLabel = "GO TO LEADERBOARD";
                timeLabelBgColor = 0x3c4142;
            }
            _timeLabel.paddingLeft = 16;
            _timeLabel.paddingRight = 24;
            _timeLabel.scale = 1.15;
            addChild(_timeLabel);

            _carTypeLabel = new ExtendedLabel();
            _carTypeLabel.textRendererFactory = LabelFactory.challengeListLocLabel;
            if(challengeData.isTimeTrial){
                _carTypeLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_time_trails"));
                _carTypeLabel.defaultIcon.color = 0x535353;
                _carTypeLabel.labelOffsetX = 1;
            }else{
                _carTypeLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("care_btn"));
                var colorFilter1:ColorMatrixFilter = new ColorMatrixFilter();
                colorFilter1.tint(0x535353, 1);
                _carTypeLabel.defaultIcon.filter = colorFilter1;
                _carTypeLabel.labelOffsetX = 4;
            }

            _carTypeLabel.defaultIcon.width = _carTypeLabel.defaultIcon.height = 14;
            if(challengeData.isTimeTrial){
                _carTypeLabel.text = "TIME TRIALS";
                addChild(_carTypeLabel);
            }else if(_data.carName && _data.carName.length){
                _carTypeLabel.text = _data.carName;
                addChild(_carTypeLabel);
            }

            _detailsLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratSemiBold(), 9, 0x535353, TextFormatAlign.LEFT, false);
            addChild(_detailsLabel);
            _detailsLabel.text = "Details";

            _earnLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x424242, TextFormatAlign.LEFT, false);
            _earnLabel.text = "Earn up to";
            _earnLabel.paddingRight = 4;
            addChild(_earnLabel);

            _earnValueLabel = createRewardsLabel(_challengeData.rewardRules[_challengeData.rewardRules.length - 1].reward);
            addChild(_earnValueLabel);
            _earnValueLabel.validate();

            _earnBorder = new LayoutQuad(1, 1, 0x686868);
            _earnBorder.width = 0.5;
            _earnBorder.height = 12;
            addChild(_earnBorder);

            _costLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x424242, TextFormatAlign.LEFT, false);
            _costLabel.text = "Cost";

            _costValueLabel = new ExtendedLabel();
            _costValueLabel.textRendererFactory = LabelFactory.challengeListDescLabel;
            _costValueLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_key_icon"));
            _costValueLabel.defaultIcon.color = 0x2fc703;
            _costValueLabel.defaultIcon.height = 9;
            _costValueLabel.defaultIcon.scale = _costValueLabel.defaultIcon.scaleY;
            _costValueLabel.labelOffsetX = 2;
            _costValueLabel.paddingLeft = 5;
            _costValueLabel.text = _data.cost;
            addChild(_costLabel);
            addChild(_costValueLabel);


            _rewardValueLabels = new Vector.<ExtendedLabel>(_challengeData.rewardRules.length);
            _keyValueLabels = new Vector.<ExtendedLabel>(_challengeData.rewardRules.length);
            _earnBorders = new Vector.<LayoutQuad>(_challengeData.rewardRules.length - 1);
            for(var i:int = 0; i < _challengeData.rewardRules.length; i++){
                if(i > 0){
                    _earnBorders[i - 1] = new LayoutQuad(1, 1, 0x686868);
                    _earnBorders[i - 1].width = 0.5;
                    _earnBorders[i - 1].height = 12;
                    addChild(_earnBorders[i - 1]);
                }
                _keyValueLabels[i] = createKeysLabel(_challengeData.rewardRules[i].max, challengeData.isTimeTrial);
                addChild(_keyValueLabels[i]);
                _keyValueLabels[i].validate();

                _rewardValueLabels[i] = createRewardsLabel(_challengeData.rewardRules[i].reward);
                addChild(_rewardValueLabels[i]);
                _rewardValueLabels[i].validate();
            }

            _costValueLabel.text = _data.cost;
            _descTitleLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratSemiBold(), 9, 0x535353, TextFormatAlign.LEFT, false);
            addChild(_descTitleLabel);
            _descTitleLabel.text = "Description";

            _descLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x535353, TextFormatAlign.LEFT, true);
            addChild(_descLabel);
            _descLabel.text = _challengeData.description;

            if(_challengeData.requirements){
                _reqTitleLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratSemiBold(), 9, 0x535353, TextFormatAlign.LEFT, false);
                addChild(_reqTitleLabel);
                _reqTitleLabel.text = "Requirements";

                _reqLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 9, 0x535353, TextFormatAlign.LEFT, true);
                addChild(_reqLabel);
                _reqLabel.text = _challengeData.requirements;
            }


            // 0x046bcb
            _enterButton = ButtonFactory.defaultButton(enterLabel, timeLabelBgColor);
            _enterButton.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getMontserratSemiBold(), 12, 0xFFFFFF, TextFormatAlign.CENTER, false);
            if(_challengeData.cars){
                _enterButton.width = 170;
            }else{
                _enterButton.width = 155;
            }
            _enterButton.height = 30;
            _enterButton.addEventListener(Event.TRIGGERED, enterButtonTriggeredHandler);
            addChild(_enterButton);


            titleLabel.validate();
            _timeLabel.validate();
            _detailsLabel.validate();
            _carTypeLabel.validate();
            _earnLabel.validate();
            if(_rewardValueLabels && _rewardValueLabels[0]){
                _rewardValueLabels[0].validate();
            }
            _costLabel.validate();
            _costValueLabel.validate();
            _descTitleLabel.validate();
            _descLabel.validate();

            if(_challengeData.requirements){
                _reqTitleLabel.validate();
                _reqLabel.validate();
            }

            addEventListener(TouchEvent.TOUCH, touchHandler);
        }

        private function touchHandler(event:TouchEvent):void {
            var cnTouch:Touch = event.getTouch(contentBackgroundImage, TouchPhase.ENDED);
            var bgTouch:Touch = event.getTouch(background, TouchPhase.ENDED);
            if(bgTouch && !cnTouch){
                closeButtonTriggeredHandler(null);
            }
        }

        private static function createRewardsLabel(label:String):ExtendedLabel{
            var result:ExtendedLabel = new ExtendedLabel();
            result.textRendererFactory = LabelFactory.challengeListDescLabel;
            result.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));

            result.defaultIcon.height = 8;
            result.defaultIcon.scale = result.defaultIcon.scaleY;
            result.labelOffsetX = 2;

            result.paddingTop = 1;
            result.labelOffsetY = -1;

            result.text = label;
            return result;
        }

        private static function createKeysLabel(label:String, isTimeTrial:Boolean = false):ExtendedLabel{
            var result:ExtendedLabel = new ExtendedLabel();
            result.textRendererFactory = LabelFactory.challengeListDescLabel;
            if(!isTimeTrial){
                result.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));

                result.defaultIcon.height = 9;
                result.defaultIcon.scale = result.defaultIcon.scaleY;
                result.labelOffsetX = 2;
                result.paddingRight = 2;

                result.paddingTop = -1;
                result.labelOffsetY = 1;

                result.text = parseFloat(label).toFixed(1) + " =";
            }else{
                result.text = parseFloat(label) + "% =";
                result.paddingRight = 2;
            }

            return result;
        }

        private function enterButtonTriggeredHandler(event:Event):void {
            if(_challengeData.status == ChallengeData.IN_CHALLENGE){
                if(_challengeData.isFromGarage){
                    var eventData:Object = {isGarage: true, challengeData:_data};
                    dispatchEvent(new Event(ENTER_CHALLENGE, false, eventData));
                }else if(_challengeData.cars){
                    showCarChooser(_challengeData.cars);
                }else{
                    dispatchEvent(new Event(ENTER_CHALLENGE, false, _data));
                }
            }else if(_challengeData.status == ChallengeData.IN_VOTING){
                dispatchEvent(new Event(ENTER_CHALLENGE_VOTING, false, _data));
            }else if(_challengeData.status == ChallengeData.FINISHED){
                dispatchEvent(new Event(ENTER_CHALLENGE_GALLERY, false, _data));
            }
        }

        private function showCarChooser(cars:Vector.<Car>):void {
            AdsManager.$.bannerHide();
            if(!_carsList){
                _carsList = new LayoutGroup();
                _carsList.width = actualWidth;
                _carsList.height = actualHeight;
                var listBg:Quad = new Quad(1,1,0x000000);
                var headerLabel:Label = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 12, 0xFFFFFF, TextFormatAlign.CENTER, false, false);
                headerLabel.paddingTop = 15;
                headerLabel.width = actualWidth;
                headerLabel.height = StackedBannerScreen.marginTop;
                headerLabel.backgroundSkin = new Quad(1,1,0x242424);
                headerLabel.text = "CHOOSE ONE OF THE VEHICLES";
                _carsList.backgroundSkin = listBg;
                _carsList.addChild(headerLabel);
                var listLayout:FlowLayout = new FlowLayout();
                listLayout.verticalAlign = VerticalAlign.MIDDLE;
                listLayout.gap = 0;
                //
                var closeButton:Button = ButtonFactory.menuBigButton("a_back_button", 0x242424);
                closeButton.width = 35;
                closeButton.height = 35;
                closeButton.includeInLayout = false;
                closeButton.x = 20;
                closeButton.y = (headerLabel.height - closeButton.height) / 2;
                closeButton.addEventListener(Event.TRIGGERED, closeCarsListHandler);
                _carsList.addChild(closeButton);
                //
                _carsList.layout = listLayout;
                for(var i:int = 0; i < cars.length; i++){
                    var btn:ChallengeModelsListItemRenderer = new ChallengeModelsListItemRenderer();
                    btn.name = i.toString();
                    var carName:String = ChallengesListScreen.getCarName(cars[i]);
                    var btnData:Object = {car:cars[i], _id: cars[i].id, carName: carName, images:{challenge: _data.carsImages?_data.carsImages[i]:""}};
                    btn.data = btnData;
                    btn.width = actualWidth / 3;
                    btn.height = actualHeight;
                    var btnListener:Function = function(event:TouchEvent):void{
                        var touch:Touch = event.getTouch(event.currentTarget as ChallengeModelsListItemRenderer, TouchPhase.ENDED);
                        if(touch){
                            var index:int = parseInt(ChallengeModelsListItemRenderer(event.currentTarget).name);
                            _data.car = _challengeData.cars[index];
                            dispatchEvent(new Event(ENTER_CHALLENGE, false, _data));
                        }
                    };
                    btn.addEventListener(TouchEvent.TOUCH, btnListener);
                    _carsList.addChild(btn);
                }
            }
            addChild(_carsList);
            _carsList.validate();
            _carsList.x = (actualWidth - _carsList.width) / 3;
            _carsList.y = -StackedBannerScreen.marginTop;
        }

        private function closeCarsListHandler(event:Event):void{
            if(_carsList){
                _carsList.removeFromParent(true);
                _carsList = null;
            }
            AdsManager.$.bannerShow(BannerScreen.HOME_SCREEN);
        }

        override protected function draw():void{
            super.draw();
            background.width = actualWidth;
            background.height = actualHeight;

            contentBackgroundImage.width = 240;
            contentBackgroundImage.height = actualHeight;
            contentBackgroundImage.x = actualWidth - contentBackgroundImage.width;
            contentBackgroundImage.y = 0;

            var sMargin:Number = 18;
            var sx:Number = contentBackgroundImage.x + sMargin;
            var sy:Number = 10;
            var bGap:Number = 8;
            var sGap:Number = 4;

            closeButton.x = actualWidth - (closeButton.width + closeButton.hitAreaPadding);
            closeButton.y = closeButton.hitAreaPadding;

            _timeLabel.x = contentBackgroundImage.x;
            _timeLabel.y = sy;

            titleLabel.x = sx;
            titleLabel.y = _timeLabel.y + _timeLabel.height;

            _carTypeLabel.x = sx;
            _carTypeLabel.y = titleLabel.y + titleLabel.height + sGap + 2;

            _detailsLabel.x = sx;
            if(_carTypeLabel.parent){
                _detailsLabel.y = _carTypeLabel.y + _carTypeLabel.height + sGap + 2;
            }else{
                _detailsLabel.y = titleLabel.y + titleLabel.height + sGap + 2;
            }

            _earnLabel.x = sx;
            _earnLabel.y = _detailsLabel.y + _detailsLabel.height;

            _earnValueLabel.x = _earnLabel.x + _earnLabel.width;
            _earnValueLabel.y = _earnLabel.y;

            _earnBorder.x = _earnValueLabel.x + _earnValueLabel.width + sGap;
            _earnBorder.y = _earnValueLabel.y + (_earnValueLabel.height - _earnBorder.height) / 2;

            _costLabel.x = _earnBorder.x + _earnBorder.width + sGap;
            _costLabel.y = _earnValueLabel.y;

            _costValueLabel.x = _costLabel.x + _costLabel.width;
            _costValueLabel.y = _costLabel.y;

            for(var i:int = 0; i < _keyValueLabels.length; i++){
                if(i == 0){
                    _keyValueLabels[i].x = _earnLabel.x;
                }else{
                    _keyValueLabels[i].x = _earnBorders[i - 1].x + _earnBorders[i - 1].width + sGap;
                }
                _keyValueLabels[i].y = _earnLabel.y + _earnLabel.height + sGap;

                _rewardValueLabels[i].x = _keyValueLabels[i].x + _keyValueLabels[i].width;
                _rewardValueLabels[i].y = _earnLabel.y + _earnLabel.height + sGap;

                if(i < _earnBorders.length){
                    _earnBorders[i].x = int(_rewardValueLabels[i].x + _rewardValueLabels[i].width + sGap);
                    _earnBorders[i].y = _rewardValueLabels[i].y + (_rewardValueLabels[i].height - _earnBorders[i].height) / 2;
                }
            }

            _descTitleLabel.x = sx;
            if(_keyValueLabels){
                _descTitleLabel.y = _keyValueLabels[0].y + _keyValueLabels[0].height + sGap + 2;
            }

            _descLabel.width = contentBackgroundImage.width - 2 * sMargin;
            _descLabel.x = sx;
            _descLabel.y = _descTitleLabel.y + _descTitleLabel.height;
            _descLabel.validate();

            if(_challengeData.requirements){
                _reqTitleLabel.x = sx;
                _reqTitleLabel.y = _descLabel.y + _descLabel.height + sGap + 2;

                _reqLabel.width = contentBackgroundImage.width - 2 * sMargin;
                _reqLabel.x = sx;
                _reqLabel.y = _reqTitleLabel.y + _reqTitleLabel.height;
            }

            _enterButton.x = contentBackgroundImage.x + (contentBackgroundImage.width - _enterButton.width) / 2;
            _enterButton.y = contentBackgroundImage.y + contentBackgroundImage.height - (_enterButton.height + bGap);
        }

        override protected function closeButtonTriggeredHandler(event:Event):void{
            super .closeButtonTriggeredHandler(event);
            SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        }

        override public function dispose():void{
            super .dispose();
            removeEventListener(TouchEvent.TOUCH, touchHandler);

            _data = null
            if(_timeLabel){
                _timeLabel.removeFromParent(true);
                _timeLabel = null;
            }
            if(_carTypeLabel){
                _carTypeLabel.removeFromParent(true);
                _carTypeLabel = null;
            }
            if(_detailsLabel){
                _detailsLabel.removeFromParent(true);
                _detailsLabel = null;
            }
            if(_descTitleLabel){
                _descTitleLabel.removeFromParent(true);
                _descTitleLabel = null;
            }
            if(_descLabel){
                _descLabel.removeFromParent(true);
                _descLabel = null;
            }
            if(_reqTitleLabel){
                _reqTitleLabel.removeFromParent(true);
                _reqTitleLabel = null;
            }
            if(_reqLabel){
                _reqLabel.removeFromParent(true);
                _reqLabel = null;
            }
            if(_costLabel){
                _costLabel.removeFromParent(true);
                _costLabel = null;
            }
            if(_costValueLabel){
                _costValueLabel.removeFromParent(true);
                _costValueLabel = null;
            }
            if(_earnLabel){
                _earnLabel.removeFromParent(true);
                _earnLabel = null;
            }

            if(_rewardValueLabels){
                while (_rewardValueLabels.length){
                    _rewardValueLabels[0].removeFromParent(true);
                    _rewardValueLabels[0] = null;
                    _rewardValueLabels.shift();
                }
                _rewardValueLabels = null;
            }

            if(_keyValueLabels){
                while (_keyValueLabels.length){
                    _keyValueLabels[0].removeFromParent(true);
                    _keyValueLabels[0] = null;
                    _keyValueLabels.shift();
                }
                _keyValueLabels = null;
            }

            if(_earnBorders){
                while (_earnBorders.length){
                    _earnBorders[0].removeFromParent(true);
                    _earnBorders[0] = null;
                    _earnBorders.shift();
                }
                _earnBorders = null;
            }

            if(_enterButton){
                _enterButton.removeFromParent(true);
                _enterButton = null;
            }
            if(_carsList){
                _carsList.removeFromParent(true);
                _carsList = null;
            }
        }

        public function get user():User {
            return _user;
        }

        public function get challengeData():ChallengeData {
            return _challengeData;
        }
    }
}
