package com.despark.tuning3d.view.components.core {
    import com.despark.tuning3d.model.cars.CarBackgroundAnimatedObject;
    import com.despark.tuning3d.model.core.Constants;

    import feathers.controls.ImageLoader;

    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.events.SecurityErrorEvent;
    import flash.filesystem.File;
    import flash.net.URLLoader;
    import flash.net.URLLoaderDataFormat;
    import flash.net.URLRequest;

    import starling.textures.TextureAtlas;

    public class BackgroundAnimationLoader extends ImageLoader {
        private var _atlasData:Object;
        private var _animatedObjectData:CarBackgroundAnimatedObject;
        private var _textureAtlas:TextureAtlas;
        private var _cameraIndex:int;
        public static const ATLAS_COMPLETE:String = 'atlasComplete';
        public function BackgroundAnimationLoader() {
            super();
        }

        override protected function initialize():void{
            super .initialize();
            var manualCacheURL:String = getManualCacheURL(_atlasData.imagePath);
            if(manualCacheURL){
                source = manualCacheURL;
            }else{
                source = _atlasData.imagePath;
            }
        }

        override protected function loader_completeHandler(event:Event):void{
            super .loader_completeHandler(event);
            var manualCacheURL:String = getManualCacheURL(_atlasData.xmlPath);
            if(manualCacheURL){
                source = manualCacheURL;
            }else{
                source = _atlasData.xmlPath;
            }
        }

        override protected function rawDataLoader_completeHandler(event:Event):void{
            var xmlData:XML = XML(this.urlLoader.data);
            _textureAtlas = new TextureAtlas(_texture, xmlData);
            this.urlLoader.removeEventListener(Event.COMPLETE, rawDataLoader_completeHandler);
            this.urlLoader.removeEventListener(IOErrorEvent.IO_ERROR, rawDataLoader_errorHandler);
            this.urlLoader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, rawDataLoader_errorHandler);
            this.urlLoader = null;

            dispatchEventWith(ATLAS_COMPLETE);
        }

        override protected function commitData():void{
            var sourceURL:String = this.source as String;
            if(source.toLowerCase().lastIndexOf("xml") == sourceURL.length - 3) {
                if(this.loader) {
                    this.loader = null;
                }
                if(!this.urlLoader) {
                    this.urlLoader = new URLLoader();
                    this.urlLoader.dataFormat = URLLoaderDataFormat.TEXT;
                }
                this.urlLoader.addEventListener(Event.COMPLETE, rawDataLoader_completeHandler);
                this.urlLoader.addEventListener(IOErrorEvent.IO_ERROR, rawDataLoader_errorHandler);
                this.urlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, rawDataLoader_errorHandler);
                this.urlLoader.load(new URLRequest(sourceURL));
                return;
            }
            super .commitData();
        }

        private function getManualCacheURL(url: String) : String{
            if(url == null) {
                return null;
            }
            url = url.split('com')[1];
            var file : File = File.applicationDirectory.resolvePath(Constants.MANUAL_CACHE_PATH + url);
            if(file.exists && !file.isDirectory){
                return file.url;
            } else{
                return null;
            }
        }

        public function get textureAtlas():TextureAtlas {
            return _textureAtlas;
        }

        public function get animatedObjectData():CarBackgroundAnimatedObject {
            return _animatedObjectData;
        }

        public function set animatedObjectData(value:CarBackgroundAnimatedObject):void {
            _animatedObjectData = value;
        }

        public function setCamera(value:int):void {
            _cameraIndex = value;
            if(_cameraIndex == 0){
                _atlasData = {
                    imagePath: _animatedObjectData.cam1.imagePath + ".png",
                    xmlPath: _animatedObjectData.cam1.imagePath + ".xml"
                };
            }else if(_cameraIndex == 1){
                _atlasData = {
                    imagePath: _animatedObjectData.cam2.imagePath + ".png",
                    xmlPath: _animatedObjectData.cam2.imagePath + ".xml"
                };
            }
        }

        public function get cameraIndex():int {
            return _cameraIndex;
        }
    }
}
