package com.despark.tuning3d.view.components.core {
	import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.model.cars.CarBackgroundAnimatedObject;
	import com.despark.tuning3d.view.components.cars.Background3D;
	import com.greensock.TweenLite;
	import com.utils.BitmapDataUtils;

	import feathers.core.FeathersControl;

	import flash.display.BitmapData;
	import flash.events.PermissionEvent;
	import flash.geom.Matrix;
	import flash.media.Camera;
	import flash.permissions.PermissionStatus;
	import flash.utils.setTimeout;

	import starling.animation.Tween;

	import starling.core.Starling;

	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.events.Event;
	import starling.textures.Texture;
	import starling.utils.MathUtil;

	public class BackgroundContainer extends FeathersControl {
		
		public static const EVENT_CAMERA_ERROR:String = "EVENT_CAMERA_ERROR";
		public static const ANIMATIONS_LOAD_COMPLETE:String = "animationLoadComplete";

		private var _background:Image;
		private var _backgroundFromCamera:Image = null;
		private var _scale:Number = 1;
		
		private var _handleCameraTexture:Texture = null;
		private var _camera:Camera;
		private var _textureList:Vector.<Texture>;
		private var _moderateSize:Boolean = true;
		private var _defaultTexture:Texture;
		private var _background3D:Background3D;
		private var _ARBackgroundData:BitmapData;
		//
		private var _animationList:Vector.<Vector.<BackgroundAnimationClip>>;
		private var _carBackground:CarBackground;
		private var _loadedAnimationCount:int = 0;
		public function BackgroundContainer() {
			super();
            _textureList = new Vector.<Texture>();
		}

		private function onPermissionStatusChange(event:PermissionEvent):void{
			if(event.status == PermissionStatus.GRANTED){
				if(!_camera){
					startCamera();
				}else{
					setCameraMode();
				}
			} else {
				dispatchEventWith(EVENT_CAMERA_ERROR, false);
				_camera = null;
			}
		}
		
		private function setCameraMode():Boolean{
			if(_camera){
				_camera.setMode(1920, 1080, 30);
				_camera.setQuality(0, 100);
				_handleCameraTexture = Texture.fromCamera(_camera,1, function(texture:Texture):void{
					_backgroundFromCamera = new Image(texture)
					addChild(_backgroundFromCamera);
					invalidate();
				});
				return true;
			}
			return false;
		}
		
		
		public function startCamera():Boolean{
			_camera = Camera.getCamera();
			if(Camera.permissionStatus==PermissionStatus.UNKNOWN || Camera.permissionStatus==PermissionStatus.DENIED){
				_camera.addEventListener(PermissionEvent.PERMISSION_STATUS, onPermissionStatusChange);
				Camera.permissionManager.requestPermission();
				//_camera.requestPermission();
				return false;
			}
			
			return setCameraMode();
		}
		
		public function stopCamera(getBitmapData:Boolean):BitmapData{
			
			if(!_camera) return null;
			
			var bitmapData:BitmapData = null;
			if(getBitmapData){
				
				bitmapData = new BitmapData(_camera.width, _camera.height, false);
				_camera.drawToBitmapData(bitmapData);
				
				if(_camera.width!=1920 || _camera.height!=1080){
					var scaleX:Number = 1920/_camera.width;
					var scaleY:Number = 1080/_camera.height;
					bitmapData = BitmapDataUtils.scaleBitmapDataXY(bitmapData,scaleX,scaleY, true);
				}
			
				var texture:Texture = Texture.fromBitmapData(bitmapData);
				updateBackground(texture);
				
			}
			
			
			if(_handleCameraTexture){
				
				_handleCameraTexture.root.attachCamera(null);
				_handleCameraTexture.dispose();
				_handleCameraTexture = null;
			}
			
			if(_backgroundFromCamera){
				
				_backgroundFromCamera.removeFromParent(true);
				_backgroundFromCamera = null;
			}
			
			
			
			_camera = null;
			
			return bitmapData;
			
		}

		public function uploadCustomBackground(bitmapData:BitmapData):BitmapData{
			if(bitmapData.width!=CarBackground.DEFAULT_WIDTH || bitmapData.height!=CarBackground.DEFAULT_HEIGHT){
				var scaledBitmapData:BitmapData = new BitmapData(CarBackground.DEFAULT_WIDTH, CarBackground.DEFAULT_HEIGHT, false);
				var m:Matrix = new Matrix();
				var sc:Number;
				var tx:Number = 0;
				var ty:Number = 0;
				if(bitmapData.width/bitmapData.height >= CarBackground.DEFAULT_WIDTH/CarBackground.DEFAULT_HEIGHT){
					sc = CarBackground.DEFAULT_HEIGHT / bitmapData.height;
					tx = (CarBackground.DEFAULT_WIDTH - bitmapData.width * sc)/ 2;
				}else{
					sc = CarBackground.DEFAULT_WIDTH / bitmapData.width;
					ty = (CarBackground.DEFAULT_HEIGHT - bitmapData.height * sc) / 2;
				}
				m.scale(sc, sc);
				m.tx = tx;
				m.ty = ty;
				scaledBitmapData.draw(bitmapData, m);
				bitmapData.dispose();
				bitmapData = scaledBitmapData;
			}
			var texture:Texture = Texture.fromBitmapData(bitmapData);
			updateBackground(texture);
			return bitmapData;
		}
		
		
		override protected function initialize():void{
			super.initialize();
			if(_defaultTexture == null){
				_defaultTexture = Texture.fromBitmapData(new BitmapData(1920, 1080,false, 0xd4d2de));
			}
			_background ||= new Image(_defaultTexture);
			addChild(_background);
		}
		
		override public function dispose():void{
			super.dispose();
			var i:int;
			if(_background){
				if(_background.texture){
                    _background.texture.dispose();
                    _background.texture = null;
				}
                _background.removeFromParent(true);
                _background = null;
			}
			if(_textureList){
				for(i = 0; i < _textureList.length; i++){
                    _textureList[i].dispose();
                    _textureList[i] = null;
				}
                _textureList = null;
			}

			if(_backgroundFromCamera){
				_backgroundFromCamera.removeFromParent(true);
				_backgroundFromCamera = null;
			}

			if(_handleCameraTexture){
				_handleCameraTexture.dispose();
				_handleCameraTexture = null;
			}

			if(_defaultTexture){
				_defaultTexture.dispose();
				_defaultTexture = null;
			}

			if(_background3D){
				_background3D.removeFromParent(true);
				_background3D = null;
			}
			if(_ARBackgroundData){
				_ARBackgroundData.dispose();
				_ARBackgroundData = null;
			}
			if(_animationList){
				for(i = 0; i < _animationList.length; i++){
					if(_animationList[i]){
						for(var j:int = 0; j < _animationList[i].length; j++){
							if(_animationList[i][j]){
								_animationList[i][j].removeFromParent(true);
								_animationList[i][j] = null;
							}
						}
						_animationList[i] = null;
					}
				}
				_animationList = null;
			}

		}
		
		public function updateBackground(texture:Texture, disposeOld:Boolean = true):void{
			if(_background){
				if(_background.texture!=texture){
					if(disposeOld && _background.texture){
						_background.texture.dispose();
					}else if(_background.texture){
                        _textureList.push(_background.texture);
					}
					_background.texture = texture;
					_background.readjustSize();

					if(_defaultTexture){
						_defaultTexture.dispose();
						_defaultTexture = null;
					}

					invalidate(INVALIDATION_FLAG_ALL);
				}
			}
		}
		
		public function get texture():Texture{
			if(_background && _background.texture) return _background.texture;
			else return null;
		}
		
		public function set myScale(value:Number):void{
			_scale = value;
			this.invalidate(INVALIDATION_FLAG_SIZE);
		}

		override protected function draw():void{
			super.draw();
			if(this.isInvalid(INVALIDATION_FLAG_SIZE)){
				if(moderateSize){
					_background.width = actualWidth * _scale;
					_background.height = actualHeight * _scale;
				}

				_background.x = (this.actualWidth - _background.width) / 2;
				_background.y = ( this.actualHeight-_background.height) / 2;

				if(_backgroundFromCamera){
					_backgroundFromCamera.width = _background.width;
					_backgroundFromCamera.height = _background.height;
				
					_backgroundFromCamera.x = _background.x;
					_backgroundFromCamera.y = _background.y;
				}
			}
		}

		public function get background():Image {
			return _background;
		}

		public function get moderateSize():Boolean {
			return _moderateSize;
		}

		public function set moderateSize(value:Boolean):void {
			_moderateSize = value;
		}

		public function get textureScale():Number{
			if(background && background.texture && background.texture.width){
				return background.width / background.texture.nativeWidth;
			}
			return 1;
		}

		public function set background3D(value:Background3D):void {
			if(_background3D && _background3D != value){
				_background3D.removeFromParent(true);
			}
			_background3D = value;
			_background3D.includeInLayout = false;
			addChild(_background3D);
		}

		public function get background3D():Background3D {
			return _background3D;
		}

		public function getARBackgroundData():BitmapData {
			return _ARBackgroundData.clone();
		}

		public function set ARBackgroundData(value:BitmapData):void {
			_ARBackgroundData = value;
		}

		public function createAnimatedObjects(animatedObjects:Vector.<CarBackgroundAnimatedObject>, cameraIndex:int):void{
			if(!animatedObjects || !animatedObjects.length){
				clearAnimatedObjects();
				return;
			}
			_animationList ||= new Vector.<Vector.<BackgroundAnimationClip>>(2);
			_animationList[0] ||= new Vector.<BackgroundAnimationClip>(animatedObjects.length);
			_animationList[1] ||= new Vector.<BackgroundAnimationClip>(animatedObjects.length);
			//
			var i:int;
			for(i = numChildren - 1; i >= 0; i--){
				if(getChildAt(i) is BackgroundAnimationClip){
					removeChildAt(i);
				}
			}

			if(_animationList[cameraIndex] && _animationList[cameraIndex].length == animatedObjects.length){
				var animationsCreated:Boolean = true;
				for(i = 0; i < _animationList[cameraIndex].length; i++){
					if(_animationList[cameraIndex][i]){
						var childIndex:int = MathUtil.clamp(getChildIndex(background) + _animationList[cameraIndex][i].loader.animatedObjectData.layerPosition, 0, numChildren);
						childIndex = getAnimationChildIndex(_animationList[cameraIndex][i], childIndex);
						addChildAt(_animationList[cameraIndex][i], childIndex);

						if(_animationList[cameraIndex][i].loader.animatedObjectData.animationType == CarBackgroundAnimatedObject.BLINKING){
							Starling.juggler.removeTweens(_animationList[cameraIndex][i]);
							lightAnimationRequest(_animationList[cameraIndex][i]);
						}else if(_animationList[cameraIndex][i].loader.animatedObjectData.animationType == CarBackgroundAnimatedObject.MOVING){
							Starling.juggler.removeTweens(_animationList[cameraIndex][i]);
							movingAnimationRequest(_animationList[cameraIndex][i]);
						}
						continue;
					}
					animationsCreated = false;
				}
				if(animationsCreated){
					return;
				}
			}

			if(animatedObjects && animatedObjects.length){
				for(i = 0; i < animatedObjects.length; i++){
					var animationLoader:BackgroundAnimationLoader = new BackgroundAnimationLoader();
					animationLoader.textureScale = textureScale;
					animationLoader.animatedObjectData = animatedObjects[i];
					animationLoader.setCamera(cameraIndex);
					animationLoader.visible = false;
					if(animatedObjects[i].animationType == CarBackgroundAnimatedObject.NONE){
						animationLoader.addEventListener(BackgroundAnimationLoader.ATLAS_COMPLETE, noneLoadComplete);
					}else if(animatedObjects[i].animationType == CarBackgroundAnimatedObject.MOVING){
						animationLoader.addEventListener(BackgroundAnimationLoader.ATLAS_COMPLETE, movingLoadComplete);
					}else if(animatedObjects[i].animationType == CarBackgroundAnimatedObject.FRAME_BY_FRAME){
						animationLoader.addEventListener(BackgroundAnimationLoader.ATLAS_COMPLETE, frameByFrameLoadComplete);
					}else if(animatedObjects[i].animationType == CarBackgroundAnimatedObject.BLINKING){
						animationLoader.addEventListener(BackgroundAnimationLoader.ATLAS_COMPLETE, blinkingLoadComplete);
					}
					parent.addChild(animationLoader);
				}
			}
		}

		private function getAnimationChildIndex(animation:BackgroundAnimationClip, index:int = 0):int{
			for(var i:int = 0; i < numChildren; i++){
				if(getChildAt(i) is BackgroundAnimationClip){
					var mc:BackgroundAnimationClip = getChildAt(i) as BackgroundAnimationClip;
					if(mc.loader.animatedObjectData.layerPosition < animation.loader.animatedObjectData.layerPosition){
						index = i + 1;
					}
				}else if(getChildAt(i) == background && animation.loader.animatedObjectData.layerPosition > 0){
					index = i + 1;
				}
			}
			return index;
		}

		public function clearAnimatedObjects():void{
			if(_animationList && _animationList.length){
				for(var i:int = 0; i < _animationList.length; i++){
					while(_animationList[i].length){
						var mc:BackgroundAnimationClip = _animationList[i].pop();
						if(mc){
							Starling.juggler.removeTweens(mc);
							mc.removeFromParent(true);
						}
					}
				}
				_animationList = null;
			}
		}

		private function frameByFrameLoadComplete(event:Event):void {
			var atlasLoader:BackgroundAnimationLoader = event.currentTarget as BackgroundAnimationLoader;
			var frames:Vector.<Texture> = atlasLoader.textureAtlas.getTextures();
			var animation:BackgroundAnimationClip = new BackgroundAnimationClip(frames, atlasLoader.animatedObjectData.frameRate);
			animation.loader = atlasLoader;
			animation.width = background.width;
			animation.scale = animation.scaleX;
			if(atlasLoader.cameraIndex == 0){
				animation.x = background.x + atlasLoader.animatedObjectData.cam1.x * animation.scale;
				animation.y = background.y + atlasLoader.animatedObjectData.cam1.y * animation.scale;
			}else if(atlasLoader.cameraIndex == 1){
				animation.x = background.x + atlasLoader.animatedObjectData.cam2.x * animation.scale;
				animation.y = background.y + atlasLoader.animatedObjectData.cam2.y * animation.scale;
			}

			var childIndex:int = MathUtil.clamp(getChildIndex(background) + atlasLoader.animatedObjectData.layerPosition, 0, numChildren);
			childIndex = getAnimationChildIndex(animation, childIndex);
			addChildAt(animation, childIndex);
			animation.play();
			Starling.juggler.add(animation);
			atlasLoader.removeFromParent();
			//
			if(atlasLoader.animatedObjectData.delay){
				animation.setFrameDuration(animation.numFrames - 1, atlasLoader.animatedObjectData.delay);
			}
			//
			var animIndex:int = atlasLoader.animatedObjectData.layerPosition - 1;
			if(animIndex >= 0 && animIndex < _animationList[atlasLoader.cameraIndex].length && !_animationList[atlasLoader.cameraIndex][animIndex]){
				_animationList[atlasLoader.cameraIndex][animIndex] = animation;
			}
			addLoadedAnimation();
		}

		private function noneLoadComplete(event:Event):void {
			var atlasLoader:BackgroundAnimationLoader = event.currentTarget as BackgroundAnimationLoader;
			var frames:Vector.<Texture> = atlasLoader.textureAtlas.getTextures();
			var animation:BackgroundAnimationClip = new BackgroundAnimationClip(frames, atlasLoader.animatedObjectData.frameRate);
			animation.loader = atlasLoader;
			animation.width = background.width;
			animation.scale = animation.scaleX;
			animation.x = background.x;
			animation.y = background.y;

			var childIndex:int = MathUtil.clamp(getChildIndex(background) + atlasLoader.animatedObjectData.layerPosition, 0, numChildren);
			childIndex = getAnimationChildIndex(animation, childIndex);
			addChildAt(animation, childIndex);

			atlasLoader.removeFromParent();
			_animationList[atlasLoader.cameraIndex][atlasLoader.animatedObjectData.index] = animation;
			addLoadedAnimation();
		}

		private function movingLoadComplete(event:Event):void {
			var atlasLoader:BackgroundAnimationLoader = event.currentTarget as BackgroundAnimationLoader;
			var frames:Vector.<Texture> = atlasLoader.textureAtlas.getTextures();
			var animation:BackgroundAnimationClip = new BackgroundAnimationClip(frames, atlasLoader.animatedObjectData.frameRate);
			animation.loader = atlasLoader;
			animation.width = background.width;
			animation.scale = animation.scaleX;
			animation.x = background.x;
			animation.y = background.y;

			var childIndex:int = MathUtil.clamp(getChildIndex(background) + atlasLoader.animatedObjectData.layerPosition, 0, numChildren);
			childIndex = getAnimationChildIndex(animation, childIndex);
			addChildAt(animation, childIndex);

			atlasLoader.removeFromParent();
			movingAnimationRequest(animation);
			_animationList[atlasLoader.cameraIndex][atlasLoader.animatedObjectData.index] = animation;
			addLoadedAnimation();
		}

		private function blinkingLoadComplete(event:Event):void {
			var atlasLoader:BackgroundAnimationLoader = event.currentTarget as BackgroundAnimationLoader;
			var frames:Vector.<Texture> = atlasLoader.textureAtlas.getTextures();
			var animation:BackgroundAnimationClip = new BackgroundAnimationClip(frames, atlasLoader.animatedObjectData.frameRate);
			animation.loader = atlasLoader;
			animation.width = background.width;
			animation.scale = animation.scaleX;
			animation.x = background.x;
			animation.y = background.y;

			var childIndex:int = MathUtil.clamp(getChildIndex(background) + atlasLoader.animatedObjectData.layerPosition, 0, numChildren);
			childIndex = getAnimationChildIndex(animation, childIndex);
			addChildAt(animation, childIndex);

			atlasLoader.removeFromParent();
			lightAnimationRequest(animation);
			_animationList[atlasLoader.cameraIndex][atlasLoader.animatedObjectData.index] = animation;
			addLoadedAnimation();
		}

		private function addLoadedAnimation():void{
			_loadedAnimationCount++;
			if(_loadedAnimationCount == 1){ // _carBackground.animatedObjects.length
				dispatchEventWith(ANIMATIONS_LOAD_COMPLETE);
			}
		}

		private function movingAnimationRequest(image:BackgroundAnimationClip):void{
			var atlasLoader:BackgroundAnimationLoader = image.loader;
			if(!image.stage){
				return;
			}
			if(atlasLoader.cameraIndex == 0){
				image.x = background.x + (atlasLoader.animatedObjectData.cam1.startX + image.texture.frame.x) * image.scale;
				image.y = background.y + (atlasLoader.animatedObjectData.cam1.startY + image.texture.frame.y) * image.scale;
			}else if(atlasLoader.cameraIndex == 1){
				image.x = background.x + (atlasLoader.animatedObjectData.cam2.startX + image.texture.frame.x) * image.scale;
				image.y = background.y + (atlasLoader.animatedObjectData.cam2.startY + image.texture.frame.y) * image.scale;
			}
			var tween:Tween = new Tween(image, 60 / atlasLoader.animatedObjectData.speed);
			tween.repeatCount = 0;
			if(atlasLoader.cameraIndex == 0){
				tween.moveTo(background.x + (atlasLoader.animatedObjectData.cam1.endX + image.texture.frame.x) * image.scale,
						background.y + (atlasLoader.animatedObjectData.cam1.endY + image.texture.frame.y) * image.scale);
			}else if(atlasLoader.cameraIndex == 1){
				tween.moveTo(background.x + (atlasLoader.animatedObjectData.cam2.endX + image.texture.frame.x) * image.scale,
						background.y + (atlasLoader.animatedObjectData.cam2.endY + image.texture.frame.y) * image.scale);
			}
			Starling.juggler.removeTweens(image);
			Starling.juggler.add(tween);
		}


		private function lightAnimationRequest(image:BackgroundAnimationClip, delayedCall:Boolean = true, count:int = 0):void{
			if(!image.stage){
				return;
			}
			var atlasLoader:BackgroundAnimationLoader = image.loader;
			if(count == 0){
				count = Math.ceil(Math.random() * (atlasLoader.animatedObjectData.blinkCount.max - atlasLoader.animatedObjectData.blinkCount.min)) + atlasLoader.animatedObjectData.blinkCount.min;
			}
			if(!image.stage){
				return;
			}
			var tween1:Tween = new Tween(image, 0.05 +  Math.random() * 0.01);
			if(delayedCall){
				tween1.delay = Math.random() * (atlasLoader.animatedObjectData.frequency.max - atlasLoader.animatedObjectData.frequency.min) + atlasLoader.animatedObjectData.frequency.min;
			}
			tween1.onComplete = function():void{
				if(!image.stage){
					return;
				}
				var tween2:Tween = new Tween(image, 0.05 +  Math.random() * 0.01);
				tween2.onComplete = function():void{
					count--;
					if(count == 0){
						lightAnimationRequest(image);
					}else{
						lightAnimationRequest(image, false, count);
					}
				};
				tween2.fadeTo(1);
				Starling.juggler.add(tween2);
			};
			tween1.fadeTo(0.01 + Math.random() * 0.99);
			Starling.juggler.add(tween1);
		}

		public function get carBackground():CarBackground {
			return _carBackground;
		}

		public function set carBackground(value:CarBackground):void {
			if(value != _carBackground){
				clearAnimatedObjects();
			}
			_carBackground = value;
		}

		public function get animationList():Vector.<Vector.<BackgroundAnimationClip>> {
			return _animationList;
		}
	}
}