package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.model.core.UIStrings;
	import com.despark.tuning3d.view.components.auth.AuthScreen;
	import com.utils.FontSelector;

	import feathers.controls.Label;
	import feathers.core.FeathersControl;

	import flash.display.Bitmap;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.animation.Transitions;
	import starling.animation.Tween;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.EnterFrameEvent;
	import starling.textures.Texture;

	public class BlockerView extends FeathersControl {
		private var quad:Quad=null;
		private var spinner:Sprite=null;
		private var _logoImage:Image;
		private static var _current:BlockerView;
		//
		public static const BLACK_SCREEN:String = "blackScreen";
		public static const LOGO_SCREEN:String = "logoScreen";
		private var _blockType:String;
		private var _onHidingComplete:Function;
		private var _tipsLabel:Label;
		//
		private static const tipsList:Array = [
			"Come back daily for more prizes!",
			"Don’t forget to consider your cars stance after applying wide fenders or body kits.",
			"More than 300 legendary vehicles available, explore them all!",
			"Do you want to see your favorite car models in 3DTuning? Visit 'Cars Schedule' and cast your vote now!",
			"Stay updated with the latest tuning news and official posts in Timeline.",
			"Comment and Like millions of tuning posts and join the community!",
			"Try different tuning styles for the same vehicle!",
			"Explore the Tuning Gallery and find inspiration from millions of amazing builds!",
			"For feedback, queries & suggestions, please contact us at <u>info@3dtuning.com</u>",
			"Show off your amazing tuning builds from your Garage with cool videos!",
			"We are regularly adding new cars & parts. Visit Daily News not to miss any update!",
			"Follow 3DTuning on Facebook & join the community!",
			"Follow 3DTuning on Instagram & join the community!",
			"Become Friends with other tuners on 3DTuning and follow their activities.",
			"Don't miss out on the latest content and optimizations. Update your app from the Store now!",
			"Add some flair to your masterpiece with photo-filter effects and make it stand out!",
			"“Zoom in” mode enables faster loading of tuning parts, try it out.",
			"Any in-app purchase turns off all types of ads in 3DT.",
			"Purchased pack grants unlimited access to all current and future models in that pack.",
			"Manufactures, Specs & more info available under ‘I’ icon.",
			"New Tuning Challenge is available every 4 hours."
		]

		private static var tipsIndexList:Array;
		//
		public function BlockerView(blockType:String = null) {
			super();
			_blockType = blockType;
		}
		
		public function hide(delay:Number = 0):void{
			if(_blockType == BLACK_SCREEN || _blockType == LOGO_SCREEN){
				var tween: Tween = new Tween(_current, 0.35, Transitions.EASE_IN);
				tween.onComplete = function():void{
					removeFromParent(true);
					if(_onHidingComplete != null){
						_onHidingComplete();
						_onHidingComplete = null;
					}
				};
				tween.fadeTo(0);
				if(delay > 0){
					Starling.juggler.delayCall(function ():void {
						Starling.juggler.add(tween);
					}, delay);
				}else{
					Starling.juggler.add(tween);
				}
			}else{
				removeFromParent(true);
			}
		}
		
		public static function show(blockType:String = null, onCompleteHandler:Function = null):BlockerView {
			if(_current){
				if(_current.stage){
					return null;
				}
				hide();
			}
			_current = new BlockerView(blockType);
			_current.width = Starling.current.stage.width;
			_current.height = Starling.current.stage.height;
			Starling.current.stage.addChild(_current);
			_current.alpha = 0;
			var tween: Tween = new Tween(_current, 0.35, Transitions.EASE_IN_OUT);
			tween.onComplete = onCompleteHandler;
			tween.fadeTo(1.0);

			Starling.juggler.add(tween);
			Starling.juggler.removeDelayedCalls(hide);
			Starling.juggler.delayCall(hide, 5);
			return _current;
		}

		public static function hide():void{
			if(_current){
				_current.hide();
				_current = null;
			}
		}
		
		private function enterFrameHandler(event : EnterFrameEvent) : void {
			if(spinner && spinner.visible){
				spinner.rotation+=Math.PI / 18;
			}
		}
		
		public function addImage(image:Image):void{
			addChildAt(image, getChildIndex(quad)+1);
		}
		
		
		protected override function initialize():void{
			super.initialize();
			quad = new Quad(1,1,0x000000);
			if(_blockType == BLACK_SCREEN || _blockType == LOGO_SCREEN){
				quad.alpha = 1;
			}else {
				quad.alpha = 0.3;
			}
			addChild(quad);

			if(_blockType == LOGO_SCREEN){
				var img : Bitmap = new AuthScreen.LOGO_CLASS_3();
				_logoImage = new Image(Texture.fromBitmapData(img.bitmapData));
				_logoImage.width = actualWidth / 3.5;
				_logoImage.scale = _logoImage.scaleX;
				_logoImage.y = 35;
				_logoImage.x = (actualWidth - _logoImage.width) / 2;
				_logoImage.color = 0xFF0000;
				addChild(_logoImage);

				var textFormat:TextFormat = new TextFormat();
				textFormat.bold = true;
				_tipsLabel = LabelFactory.createSimpleLabel(FontSelector.MontserratHtml, 12, 0xFFFFFF, TextFormatAlign.CENTER, true, true, textFormat);
				_tipsLabel.text = getRandomTip();
				_tipsLabel.width = actualWidth / 2;
				addChild(_tipsLabel);
				_tipsLabel.validate();
			}

			if(!_blockType || _blockType == LOGO_SCREEN){
				var spinnerSize:Number = (_blockType == LOGO_SCREEN)? 30: 40;
				spinner = new Sprite();
				var spinnerTexture:Texture = ButtonFactory.assetManager.getTexture("spinner");
				var spinnerImage:Image = new Image(spinnerTexture);
				spinnerImage.width = spinnerSize;
				spinnerImage.height = spinnerSize;
				spinnerImage.x = -spinnerSize / 2;
				spinnerImage.y = -spinnerSize / 2;
				spinner.addChild(spinnerImage);
				addChild(spinner);
				addEventListener(EnterFrameEvent.ENTER_FRAME, enterFrameHandler);
			}
		}

		private static function generateTipsIndexList():Array{
			var result:Array = tipsIndexList || [];
			for(var i:int = 0; i < tipsList.length; i++){
				result.push(i);
			}
			return result;
		}

		private static function getRandomTip():String{
			if(!tipsIndexList || !tipsIndexList.length){
				tipsIndexList = generateTipsIndexList();
			}
			var index:int = Math.floor(Math.random() * tipsIndexList.length);
			var result:String = tipsList[index];
			tipsIndexList.removeAt(index);
			return result;
		}
		
		protected override function draw():void{
			super.draw();
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			if(sizeInvalid){
				quad.x=0;
				quad.y=0;
				quad.width=actualWidth;
				quad.height=actualHeight;
				if(spinner){
					spinner.x = actualWidth/2;
					if(_blockType == LOGO_SCREEN){
						spinner.y = actualHeight - 45;
						if(_tipsLabel){
							_tipsLabel.x = (actualWidth - _tipsLabel.width) / 2;
							_tipsLabel.y = spinner.y - (spinner.height / 2 + _tipsLabel.height + 15);
						}
					}else{
						spinner.y = actualHeight / 2;
					}
				}
			}
		}

		public function set onHidingComplete(value:Function):void {
			_onHidingComplete = value;
		}

		override public function dispose():void{
			super .dispose();
			if(quad){
				quad.removeFromParent(true);
				quad = null;
			}
			if(spinner){
				spinner.removeFromParent(true);
				spinner = null;
			}
			if(_logoImage){
				_logoImage.removeFromParent(true);
				_logoImage = null;
			}
			if(_tipsLabel){
				_tipsLabel.removeFromParent(true);
				_tipsLabel = null;
			}
		}
	}
}