package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.settings.SettingsProxy;
	import com.greensock.easing.Quad;
	import com.utils.FontSelector;
	
	import flash.geom.Point;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import feathers.controls.ImageLoader;
	import feathers.controls.Label;
	import feathers.controls.List;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.controls.text.BitmapFontTextRenderer;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	import feathers.core.ITextRenderer;
	import feathers.text.BitmapFontTextFormat;
	
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.text.TextField;
	import starling.textures.TextureSmoothing;
	
	public class BrandListItemRenderer extends FeathersControl implements IListItemRenderer
	{
		public var fontSize : int;
		
		protected var label:MultilingualLabel;
		//protected var imageLoader : ImageLoaderWithStates;
		protected var _index : int = -1;
		protected var _owner:List;
		protected var _data : Object;
		protected var _isSelected:Boolean;
		protected var useBitmapFont : Boolean;
		
		
		private var touchPointID : int;
		private var selectedByTouching : Boolean;
		
		private static const HELPER_POINT : Point = new Point();
		
		protected var _textColor : uint;
		protected var _bold : Boolean;
		
		private var settingsProxy : SettingsProxy;
		private var lastLang : Language;
		
		protected var overflowMultiLang : Boolean = false;
		
		public function BrandListItemRenderer() 
		{
			super();
			
			fontSize  = 7;
			textColor = 0x636365;
			
			addEventListener(TouchEvent.TOUCH, touchHandler);   
		}
		
		protected var _factoryID:String;
		public function get factoryID():String{
			return _factoryID;
		}
		
		
		public function set factoryID(value:String):void{
			_factoryID = value;
		}
		
		public function set textColor(value : uint) : void
		{
			_textColor = value;
			
			if (label) {
				label.textFormat.color = value;
				label.invalidate();
			}
		}
		
		public function set bold(value : Boolean) : void
		{
			_bold = value;
			
			if (label){
				label.textFormat.font = FontSelector.getFont(_bold);     
				label.textFormat.bold = _bold;
				label.invalidate(); 
			}
		}
		
		public function get textColor() : uint
		{
			return _textColor;
		}
		
		public override function dispose() : void 
		{
			removeEventListener(TouchEvent.TOUCH, touchHandler);   
			/*if (imageLoader){
				imageLoader.removeFromParent(true);
			}
			
			imageLoader = null;*/
			settingsProxy = null;
			lastLang=null;
			
			super.dispose();
			
			_owner = null;
			_data = null;
		}
		
		public function get index():int
		{
			return _index;
		}
		
		public function set index(value : int) : void
		{
			if (_index == value){
				return;
			}
			
			_index = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get owner() : List
		{
			return List(_owner);
		}
		
		public function set owner(value : List) : void
		{
			if (_owner == value){
				return;
			}
			_owner = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get data() : Object
		{
			return _data;
		}
		
		public function set data(value : Object) : void
		{
			if (_data == value){
				return;
			}
			
			_data = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get isSelected() : Boolean
		{
			return _isSelected;
		}
		
		public function set isSelected(value : Boolean) : void
		{
			if (_isSelected == value)
			{
				if (_isSelected && selectedByTouching) {
					dispatchEventWith("itemSelectedAgain", true);
				}
				return;
			}
			
			_isSelected = value;
			invalidate(INVALIDATION_FLAG_SELECTED);
			dispatchEventWith(Event.CHANGE);
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			//settingsProxy  = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			
			this.lastLang = AlertView.language;
			label = new MultilingualLabel();
			
			if (!overflowMultiLang){
				label.textFormat = new TextFormat(FontSelector.getFont(_bold), fontSize, textColor);
			}
			else {
				label.textFormat = new TextFormat("NotoSans", fontSize, textColor);
			}
			
			label.textFormat.align = TextFormatAlign.LEFT;
			label.textFormat.bold = _bold;   
			addChild(label);
			
			if (!overflowMultiLang) {
				label.updateWithLanguage(AlertView.language);  
			}
			
			/*imageLoader = new ImageLoaderWithStates();
			imageLoader.overrideDispose = true;
			addChild(imageLoader);  */
		}
		
		override protected function draw():void
		{
			var dataInvalid : Boolean = isInvalid(INVALIDATION_FLAG_DATA);
			var selectionInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SELECTED);
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			
			if (lastLang != AlertView.language && !overflowMultiLang)
			{  
				lastLang = AlertView.language;
				if (label){
					label.updateWithLanguage(AlertView.language);
				}
				sizeInvalid = true;  
			}
			
			
			if (dataInvalid) {
				commitData();
			}
			
			if (selectionInvalid) {
				commitSelection();
			}
			
			var autoSizeNeeded : Boolean = autoSizeIfNeeded();
			if (autoSizeNeeded || sizeInvalid){
				layout();
			}
			///
			layout();
		}
		
		public function get useEmbeddedFont() : Boolean 
		{
			return Language.supportsEmbeddedFont;
		}
		
		protected function autoSizeIfNeeded():Boolean
		{
			return false;
		}
		
		protected function commitData() : void
		{
		}
		
		protected function commitSelection() : void
		{
		//	if (imageLoader) imageLoader.selected = isSelected;
		}
		
		protected function layout():void
		{
			label.width  = actualWidth;
			label.height = actualHeight;
			label.height = NaN;
			label.width  = NaN;
			label.validate();
			label.y = actualHeight - label.height;
			
		/*	if (imageLoader){
				imageLoader.width  = actualWidth;
				imageLoader.height = actualHeight - (label.text ? label.height : 0);
			}*/
		}
		
		protected function touchHandler(event : TouchEvent):void
		{
			try
			{
				const touches : Vector.<Touch> = event.getTouches(this);
				for each(var touch : Touch in touches)
				{
					if (touch.phase == TouchPhase.ENDED)
					{
						touch.getLocation(stage, HELPER_POINT);
						const isInBounds : Boolean = contains(stage.hitTest(HELPER_POINT));
						if (isInBounds) 
						{
							selectedByTouching = true;
							isSelected = true;
							selectedByTouching = false;         
						}
						return;
					}					
				}
			}
			catch(e:Error) {
				//trace(this + "->touchHandler error:"+e.message+" stack:"+e.getStackTrace());
			}
		}
	}
}