package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.settings.SettingsProxy;
	import com.greensock.easing.Quad;
	import com.utils.FontSelector;
	
	import flash.geom.Point;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import feathers.controls.ImageLoader;
	import feathers.controls.Label;
	import feathers.controls.List;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.controls.text.BitmapFontTextRenderer;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	import feathers.core.ITextRenderer;
	import feathers.text.BitmapFontTextFormat;
	
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.text.TextField;
	import starling.textures.TextureSmoothing;
	
	public class CarListItemRenderer extends FeathersControl implements IListItemRenderer
	{
		public var fontSize : int;
		
		protected var label :MultilingualLabel;
		protected var tlabel:MultilingualLabel;
		protected var ylabel:MultilingualLabel;
		
		protected var imageLoader : ImageLoaderWithStates;
		
		protected var _index : int = -1;
		protected var _owner:List;
		protected var _data : Object;
		protected var _isSelected : Boolean;
		protected var useBitmapFont : Boolean;
		
		private var touchPointID : int;
		private var selectedByTouching : Boolean;
		private static const HELPER_POINT : Point = new Point();
		
		protected var _textColor : uint;
		protected var _bold : Boolean;
		
		private var settingsProxy : SettingsProxy;
		private var lastLang : Language;
		
		protected var overflowMultiLang:Boolean = false;
		
		public var labelHeight : Number = 20;
				
		public function CarListItemRenderer() 
		{
			super();
			
			
			if (Constants.SCALE_FACTOR == 4) {
				fontSize = Constants.isIOS ? 13 : 13.5;
			}
			else if (Constants.SCALE_FACTOR == 2){
				fontSize = Constants.isIOS ? 14 : 15;
			}
			else {
				fontSize = 15;
			}
			//fontSize  = 15;
			
			textColor = 0x58595A;
			
			addEventListener(TouchEvent.TOUCH, touchHandler);   
		}
		
		protected var _factoryID:String;
		public function get factoryID():String{
			return _factoryID;
		}
		
		/**
		 * @private
		 */
		public function set factoryID(value:String):void{
			
			_factoryID = value;
		}
		
		public function set textColor(value : uint) : void
		{
			_textColor = value;
			
			if (label)
			{
				label.textFormat.color  = value;
				tlabel.textFormat.color = value;
				ylabel.textFormat.color = value;
				
				label.invalidate();
				tlabel.invalidate();
				ylabel.invalidate();
			}
		}
		
		public function set bold(value : Boolean) : void
		{
			_bold = value;
			
			if (label){
				label.textFormat.font = FontSelector.getFont(_bold);     
				label.textFormat.bold = _bold;
				label.invalidate(); 
			}
		}
		
		public function get textColor() : uint {
			return _textColor;
		}
		
		public override function dispose() : void 
		{
			removeEventListener(TouchEvent.TOUCH, touchHandler);   
			if (imageLoader){
				imageLoader.removeFromParent(true);
			}
			
			imageLoader = null;
			settingsProxy = null;
			lastLang = null;
			
			super.dispose();
			
			_owner = null;
			_data  = null;
		}
		
		public function get index():int{
			return _index;
		}
		
		public function set index(value : int) : void
		{
			if (_index == value) return;
			
			_index = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get owner() : List{
			return List(_owner);
		}
		
		public function set owner(value : List) : void
		{
			if (_owner == value){
				return;
			}
			_owner = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get data() : Object{
			return _data;
		}
		
		public function set data(value : Object) : void
		{
			if (_data == value) return;
			
			_data = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get isSelected() : Boolean{
			return _isSelected;
		}
		
		public function set isSelected(value : Boolean) : void
		{
			if (_isSelected == value)
			{
				if (_isSelected && selectedByTouching){
					dispatchEventWith("itemSelectedAgain", true);
				}
				return;
			}
			
			_isSelected = value;
			invalidate(INVALIDATION_FLAG_SELECTED);
			dispatchEventWith(Event.CHANGE);
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			//settingsProxy  = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			
			this.lastLang = AlertView.language;
			label  = new MultilingualLabel();
			tlabel = new MultilingualLabel();
			ylabel = new MultilingualLabel();
			
			if (!overflowMultiLang)
			{
				label.textFormat  = new TextFormat(FontSelector.getFont(_bold), fontSize, textColor);
				tlabel.textFormat = new TextFormat(FontSelector.getFont(_bold), fontSize, textColor);
				ylabel.textFormat = new TextFormat(FontSelector.getFont(_bold), fontSize, textColor);
			}
			else
			{
				label.textFormat  = new TextFormat("NotoSans", fontSize, textColor);
				tlabel.textFormat = new TextFormat("NotoSans", fontSize, textColor);
				ylabel.textFormat = new TextFormat("NotoSans", fontSize, textColor);
			}
			
			label.textFormat.align = TextFormatAlign.LEFT;
			label.textFormat.bold  = _bold;   
			
			tlabel.textFormat.align = TextFormatAlign.LEFT;
			ylabel.textFormat.align = TextFormatAlign.RIGHT;
			
			if (!overflowMultiLang) {
				label.updateWithLanguage(AlertView.language);  
				tlabel.updateWithLanguage(AlertView.language);  
				ylabel.updateWithLanguage(AlertView.language);  
			}
			
			imageLoader = new ImageLoaderWithStates();
			imageLoader.overrideDispose = true;
			addChild(imageLoader);  
			
			addChild(label);
			addChild(tlabel);
			addChild(ylabel);
		}
		
		override protected function draw():void
		{
			var dataInvalid : Boolean = isInvalid(INVALIDATION_FLAG_DATA);
			var selectionInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SELECTED);
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			
			if (lastLang != AlertView.language && !overflowMultiLang)
			{  
				lastLang = AlertView.language;
				if (label) 
				{
					label.updateWithLanguage(AlertView.language);
					tlabel.updateWithLanguage(AlertView.language);
					ylabel.updateWithLanguage(AlertView.language);
				}
				sizeInvalid = true;  
			}
			
			if (dataInvalid) commitData();
			
			if (selectionInvalid) commitSelection();
			
			var autoSizeNeeded : Boolean = autoSizeIfNeeded();
			if (autoSizeNeeded || sizeInvalid){
				layout();
			}
		}
		
		private function getTextRenderer():ITextRenderer
		{
			var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(), fontSize, textColor);
				textRenderer.textFormat.align = TextFormatAlign.LEFT;
				textRenderer.embedFonts = "NotoSans";
				textRenderer.textFormat.leftMargin = 0;
				textRenderer.textFormat.italic = false;
				textRenderer.textFormat.bold = true;
				textRenderer.embedFonts = true;
			return textRenderer;
		}
		
		public function get useEmbeddedFont() : Boolean 
		{
			return Language.supportsEmbeddedFont;
		}
		
		protected function autoSizeIfNeeded():Boolean
		{
			return false;
		}
		
		protected function commitData() : void
		{
		}
		
		protected function commitSelection() : void
		{
			if (imageLoader) imageLoader.selected = isSelected;
			
			textColor = isSelected ? 0xA80808 : 0x58595A;
			label.textRendererFactory = getTextRenderer;
			
			
		}
		
		public var pading : Number = 3;
		protected function layout():void
		{
			label.height = tlabel.height = ylabel.height = labelHeight;
			label.y = ylabel.y = pading;
			tlabel.y = label.y + label.height - pading;
			ylabel.x = actualWidth - 25;
			tlabel.width = actualWidth - 25;
			ylabel.width = 22;
			label.width = actualWidth - 40;
			
			label.width = NaN;
			label.validate();
			
			tlabel.width = NaN;
			tlabel.validate();
			
			ylabel.width = NaN;
			ylabel.validate();
			
			if (imageLoader){
				imageLoader.width  = actualWidth ;
				imageLoader.height = actualHeight;
			}
		}
		
		protected function touchHandler(event : TouchEvent):void
		{
			try
			{
				const touches : Vector.<Touch> = event.getTouches(this);
				for each(var touch : Touch in touches)
				{
					if (touch.phase == TouchPhase.ENDED)
					{
						touch.getLocation(stage, HELPER_POINT);
						const isInBounds : Boolean = contains(stage.hitTest(HELPER_POINT));
						
						if (isInBounds) {
							selectedByTouching = true;
							isSelected = true;
							selectedByTouching = false;         
						}
						return;
					}					
				}
			}
			catch(e:Error) {
				//trace(this + "->touchHandler("+e.message+")"+e.getStackTrace());
			}
		}
	}
}