package com.despark.tuning3d.view.components.core {
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.core.ITextBaselineControl;
    import feathers.core.IToolTip;
    import feathers.layout.AnchorLayoutData;
    import feathers.layout.BaseLinearLayout;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.ILayout;
    import feathers.layout.RelativePosition;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.geom.Point;

    import flash.geom.Rectangle;

    import starling.display.DisplayObject;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;

    [Event(name="triggered",type="starling.events.Event")]

    public class ExtendedLabel extends LayoutGroup implements ITextBaselineControl, IToolTip{
        private var _defaultIcon:LayoutImage;
        private var _label:Label;
        private var _labelXOffset:Number = 0;
        private var _labelYOffset:Number = 0;
        private var _labelPaddingRight:Number = 0;
        private var _iconOffsetX:Number = 0;
        private var _iconOffsetY:Number = 0;
        private var _layoutObject:ILayout;
        private var _hitAreaPadding:Point;
        private var _iconPosition:String;
        public function ExtendedLabel() {
            super();
            _label = new Label();
            layout = new HorizontalLayout();
            verticalAlign = VerticalAlign.MIDDLE;
            touchGroup = true;
            addEventListener(TouchEvent.TOUCH, touchHandler);
        }

        override public function set layout(value:ILayout):void{
            var currentLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
            _layoutObject = value;
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                baseLayout.gap = _labelXOffset;
                if(currentLayout){
                    baseLayout.horizontalAlign = currentLayout.horizontalAlign;
                    baseLayout.verticalAlign = currentLayout.verticalAlign;
                    baseLayout.paddingTop = currentLayout.paddingTop;
                    baseLayout.paddingBottom = currentLayout.paddingBottom;
                    baseLayout.paddingLeft = currentLayout.paddingLeft;
                    baseLayout.paddingRight = currentLayout.paddingRight;
                }
            }
            super .layout = _layoutObject;
        }

        override protected function draw():void{
            super .draw();
        }

        override protected function initialize():void{
            super .initialize();
            addChild(_label);
            if(_defaultIcon && !_defaultIcon.parent){
                addChild(_defaultIcon);
            }
        }

        public function set horizontalAlign(value:String):void{
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                baseLayout.horizontalAlign = value;
            }
        }

        public function set verticalAlign(value:String):void{
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                baseLayout.verticalAlign = value;
            }
        }

        public function get defaultIcon():LayoutImage {
            return _defaultIcon;
        }

        public function set defaultIcon(value:LayoutImage):void {
            if(_defaultIcon){
                _defaultIcon.removeFromParent(true);
                _defaultIcon = null;
            }
            _defaultIcon = value;
            if(_defaultIcon){
                addChildAt(_defaultIcon, 0);
                iconOffsetX = _iconOffsetX;
                iconOffsetY = _iconOffsetY;
            }
        }

        public function get baseline():Number {
            return 0;
        }

        public function get text():String {
            if(_label) {
                return _label.text;
            }
            return '';
        }

        public function set text(value:String):void {
            if(_label) {
                _label.text = value;
            }
        }

        public function set textRendererFactory(value:Function):void{
            if(_label){
                _label.textRendererFactory = value;
            }
        }

        public function set paddingLeft(value:Number):void{
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                baseLayout.paddingLeft = value;
            }
        }

        public function get paddingLeft():Number {
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                return baseLayout.paddingLeft;
            }
            return 0;
        }

        public function set paddingRight(value:Number):void{
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                baseLayout.paddingRight = value;
            }
        }

        public function get paddingRight():Number {
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                return baseLayout.paddingRight;
            }
            return 0;
        }

        public function set paddingTop(value:Number):void{
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                baseLayout.paddingTop = value;
            }
        }

        public function get paddingTop():Number {
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                return baseLayout.paddingTop;
            }
            return 0;
        }


        public function set paddingBottom(value:Number):void{
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                baseLayout.paddingBottom = value;
                layout = _layoutObject;
            }
        }

        public function get paddingBottom():Number {
            if(_layoutObject is BaseLinearLayout){
                var baseLayout:BaseLinearLayout = _layoutObject as BaseLinearLayout;
                return baseLayout.paddingBottom;
            }
            return 0;
        }

        public function set labelOffsetX(value:Number):void{
            if(_label){
                if(value >= 0){
                    _label.paddingRight = 0;
                    _label.paddingLeft = value;
                }else{
                    _label.paddingRight = Math.abs(value);
                    _label.paddingLeft = 0;
                }
            }
        }

        public function set labelOffsetY(value:Number):void{
            if(_label){
                _label.paddingTop = value;
            }
        }

        public function set iconOffsetX(value:Number):void{
            _iconOffsetX = value;
            if(_defaultIcon){
                var layoutData:AnchorLayoutData = new AnchorLayoutData();
                layoutData.left = value;
                _defaultIcon.layoutData = layoutData;
            }
        }

        public function set iconOffsetY(value:Number):void{
            _iconOffsetY = value;
            if(_defaultIcon){
                var layoutData:AnchorLayoutData = new AnchorLayoutData();
                layoutData.top = value;
                _defaultIcon.layoutData = layoutData;
            }
        }


        public function get labelOffsetX():Number {
            if(_layoutObject is HorizontalLayout){
                var hLayout:HorizontalLayout = _layoutObject as HorizontalLayout;
                return hLayout.gap;
            }else if(_layoutObject is HorizontalLayout){
                var vLayout:VerticalLayout = _layoutObject as VerticalLayout;
                return vLayout.gap;
            }
            return 0;
        }

        private function touchHandler(event:TouchEvent):void{
            var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
            if(touch){
                dispatchEvent(new Event(Event.TRIGGERED));
            }
        }

        public override function hitTest(localPoint : Point) : DisplayObject {
            if (!visible || !touchable) {
                return null;
            }

            var bounds : Rectangle = getBounds(this);
            if(_hitAreaPadding){
                bounds.inflate(_hitAreaPadding.x, _hitAreaPadding.y);
                if (bounds.containsPoint(localPoint)) {
                    return this;
                }
            }
            return null;
        }

        public function set hitAreaPadding(value:Point):void {
            _hitAreaPadding = value;
        }

        public function set labelPaddingRight(value:int):void {
            _labelPaddingRight = value;
            if(_label){
                _label.paddingRight = _labelPaddingRight;
            }
        }

        public function get iconPosition():String {
            return _iconPosition;
        }

        public function set iconPosition(value:String):void {
            _iconPosition = value;
            if((_iconPosition == RelativePosition.TOP || _iconPosition == RelativePosition.BOTTOM) && _layoutObject is HorizontalLayout){
                layout = new VerticalLayout();
            }else if((_iconPosition == RelativePosition.LEFT || _iconPosition == RelativePosition.RIGHT) && _layoutObject is VerticalLayout){
                layout = new HorizontalLayout();
            }
        }

        public function get labelYOffset():Number {
            return _labelYOffset;
        }

        public function set labelYOffset(value:Number):void {
            _labelYOffset = value;
            if(_label){
                if(_labelYOffset > 0){
                    _label.paddingTop = _labelYOffset;
                    _label.paddingBottom = 0;
                }else if(_labelYOffset < 0){
                    _label.paddingTop = 0;
                    _label.paddingBottom = -_labelYOffset;
                }else{
                    _label.paddingTop = 0;
                    _label.paddingBottom = 0;
                }
            }
        }

        public function get labelXOffset():Number {
            return _labelXOffset;
        }

        public function set labelXOffset(value:Number):void {
            _labelXOffset = value;
            if(_label){
                _label.paddingLeft = _labelXOffset;
            }
        }
    }
}
