package com.despark.tuning3d.view.components.core {
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.despark.tuning3d.view.core.HomeScreenPopUp;
import com.utils.FontSelector;
import com.utils.RegExpUtil;

import feathers.controls.Label;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;

import flash.net.URLRequest;

import flash.net.navigateToURL;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.events.Event;

import starling.events.Touch;

import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class HomeNewsItemRenderer extends ListItemRenderer {
    private var _timeLabel:Label;
    private var _carNameLabel:Label;
    private var _msgLabel:Label;
    private var _carNameTop:Boolean;
    private var _homeScreen:HomeScreenPopUp;
    private var _screenMode:String;
    public function HomeNewsItemRenderer() {
        super();
    }

    override protected function initialize():void {
        super.initialize();
        _timeLabel = new Label();
        _timeLabel.textRendererFactory = textTimeRenderer;
        addChild(_timeLabel);

        _carNameLabel = new Label();
        _carNameLabel.textRendererFactory = textCarNameRenderer;
        addChild(_carNameLabel);

        _msgLabel = new Label();
        _msgLabel.textRendererFactory = textMsgRenderer;
        addChild(_msgLabel);
    }


    override protected function draw():void {
        super.draw();
        var secX:Number = 22;
        var secY:Number = 12;
        if(_carNameLabel){
            _carNameLabel.x = _carNameTop? 0: secX;
            _carNameLabel.y = _carNameTop? secY: 0;
        }

        if(_msgLabel){
            _msgLabel.x = _carNameTop? secX: 0;
            _msgLabel.y = _carNameTop? 0: secY;
        }
    }

    override protected function commitData():void {
        super.commitData();
        if(data && isInitialized){
            if(data.hasOwnProperty('date')){
                _timeLabel.text = RegExpUtil.removeHtmlTags(data.date);
            }

            if(data.hasOwnProperty('text')){
                var str:String = data.text;
                var carNamePattern:RegExp = /(<strong>(?P<content>.*)<\/strong>)/gsm;
                var cn:* = carNamePattern.exec(str);
                var carName:String = cn.content;
                _carNameLabel.text = carName;
                _carNameTop = cn.index;

                var pattern:RegExp = /((?P<plain1>.*)<br>)?<strong>.*<\/strong>(<br>(?P<plain2>.*))?/gsm;
                var r:* = pattern.exec(str);
                _msgLabel.text = r.plain1?r.plain1:r.plain2?r.plain2:'';
            }
            if(!hasEventListener(TouchEvent.TOUCH, touchEventHandler)){
                addEventListener(TouchEvent.TOUCH, touchEventHandler);
            }
            invalidate(INVALIDATION_FLAG_SIZE);
        }
    }

    override public function dispose():void {
        super.dispose();
        removeEventListener(TouchEvent.TOUCH, touchEventHandler);
        if(_timeLabel){
            _timeLabel.removeFromParent(true);
            _timeLabel = null;
        }
        if(_carNameLabel){
            _carNameLabel.removeFromParent(true);
            _carNameLabel = null;
        }
        if(_msgLabel){
            _msgLabel.removeFromParent(true);
            _msgLabel = null;
        }
    }

    private function touchEventHandler(event:TouchEvent):void{
        var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
        if(touch && !owner.isScrolling){
            if(data.hasOwnProperty('carId') && data.carId && data.carId.length > 0 && _homeScreen){
                _homeScreen.dispatchEvent(new Event(StackedScreen.TUNING_SELECTED, false, {carId: data.carId, screenMode: _screenMode}));
            }
        }
    }

    public static function  textTimeRenderer ():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getMontserratSemiBold(), LabelFactory.HOME_NEWS_SMALL, 0xde0000);
        //textRenderer.textFormat.letterSpacing = -.35;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = true;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

        public static function  textCarNameRenderer ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratSemiBold(), LabelFactory.HOME_NEWS_SMALL, TimelineLabelFactory.COLOR_TEXT_WHITE);
            // textRenderer.textFormat.letterSpacing = -.35;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            return textRenderer;
        }

        public static function  textMsgRenderer ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), LabelFactory.HOME_NEWS_SMALL, TimelineLabelFactory.COLOR_TEXT_WHITE);
            // textRenderer.textFormat.letterSpacing = -.35;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            return textRenderer;
        }

        public function set homeScreen(value:HomeScreenPopUp):void {
            _homeScreen = value;
        }

    public function set screenMode(value:String):void {
        _screenMode = value;
    }
}
}
