package com.despark.tuning3d.view.components.core {
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.layout.HorizontalAlign;

    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.filters.DropShadowFilter;
    import starling.textures.Texture;

    public class HomeScreenButton extends LayoutGroup{
    private var _mainImageTexture : Texture;
    private var _topButton:Button;
    private var border:LayoutImage;
    private var maskQuad:Quad;
    private var buttonLabel:Label;
    private var label:String;
    private var labelBackgroundColor:uint;
    public var isNarrowScreen:Boolean = false;


    public function HomeScreenButton(mainImageTexture : Texture, label:String, labelBackgroundColor:uint) {
        _mainImageTexture = mainImageTexture;
        this.label = label;
        this.labelBackgroundColor = labelBackgroundColor;
    }

    override protected function initialize():void {
        super.initialize();

        initTopButton();
        _topButton.addEventListener(Event.TRIGGERED, onButtonSelected);

        var borderTexture:Texture = Texture.fromTexture(ButtonFactory.assetManager.getTexture("timeline_feed_border"), null, null, null, 2 / ButtonFactory.assetManager.scaleFactor);
        border = new LayoutImage(borderTexture);
        border.scale9Grid = new Rectangle(border.texture.frameWidth / 3,border.texture.frameHeight / 3,border.texture.frameWidth / 3,border.texture.frameHeight / 3);
        border.touchable = false;
        border.includeInLayout = false;
       // addChild(border);

        maskQuad = new Quad(1,1,0x000000);
        addChild(maskQuad);

        _topButton.mask = maskQuad;

        buttonLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 30, 0xFFFFFF, TextFormatAlign.CENTER, false);
        buttonLabel.text = label;

        var labelBackground:Quad = new Quad(1,1,labelBackgroundColor);
        buttonLabel.backgroundSkin = labelBackground;
        //buttonLabel.minWidth = 3 * width / 5;
        buttonLabel.paddingRight = buttonLabel.paddingLeft = 10;
        buttonLabel.paddingTop = buttonLabel.paddingBottom = -2;
        buttonLabel.touchable = false;
        addChild(buttonLabel);
    }

    private function initTopButton():void {
        var defaultIcon : Image = new Image(_mainImageTexture);
        var downIcon : Image = new Image(_mainImageTexture);
        defaultIcon.color = 0xFF0000;
        defaultIcon.alpha = 0;
        downIcon.color = 0xFF0000;
        downIcon.alpha = 0;
        defaultIcon.filter = new DropShadowFilter(0, 0, 0, 0.5, 1.5, 0.32);
        //
        _topButton = new Button();
        _topButton.defaultSkin = defaultIcon;
        _topButton.downSkin = downIcon;
        _topButton.horizontalAlign = HorizontalAlign.CENTER;
        _topButton.addEventListener(Event.TRIGGERED, onButtonSelected);
        addChild(_topButton);

        width = Math.max(stage.stageWidth - 270, 330);
        height = isNarrowScreen?175:179;

        if(width / height > _topButton.defaultSkin.width/ _topButton.defaultSkin.height){
            _topButton.width = width;
            _topButton.height = width * _topButton.defaultSkin.height/ _topButton.defaultSkin.width;
        }else{
            _topButton.height = height;
            _topButton.width = height * _topButton.defaultSkin.width/ _topButton.defaultSkin.height;
        }

        _topButton.x = (width - _topButton.width) / 2;
        _topButton.y = (height - _topButton.height) / 2;
    }

    override protected function draw():void{
        super .draw();
        if(border){
            border.width = width;
            border.height = height;
        }

        if(maskQuad){
            maskQuad.width = width;
            maskQuad.height = height;
        }

        if(buttonLabel){
            buttonLabel.x = (actualWidth - buttonLabel.width) / 2;
            buttonLabel.y = height - (buttonLabel.height);
        }
    }

    private function onButtonSelected(event:Event):void {
        dispatchEventWith(Event.TRIGGERED);
    }
}
}
