package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.core.AppMode;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.friend.FriendProxy;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.LanguagesProxy;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.brands.BrandsScreen;
	import com.despark.tuning3d.view.components.languages.LanguageItemRenderer;

	import feathers.controls.Button;
	import feathers.controls.LayoutGroup;
	import feathers.controls.List;
	import feathers.controls.PickerList;
	import feathers.controls.ToggleButton;
	import feathers.controls.popups.DropUpPopUpContentManager;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.data.ListCollection;
	import feathers.layout.HorizontalAlign;
	import feathers.layout.HorizontalLayout;
	import feathers.layout.VerticalAlign;
	import feathers.layout.VerticalLayout;

	import flash.events.TimerEvent;
	import flash.geom.Rectangle;
	import flash.utils.Timer;

	import starling.animation.Transitions;
	import starling.animation.Tween;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.textures.Texture;
	import starling.utils.AssetManager;

	public class MainMenuView extends StackedScreenNavigator implements Multilingual {
		public static var assetManager : AssetManager;
		public static var language : Language;

		public var garageThumbMode:Boolean = false;

		private static var pickerButton2 : MultilingualButton = null;

		public var image : Sprite;
		public var isOpened : Boolean = false;

		private var languageTimer:Timer;

		private var _text : String;
		private var _parentActualWidth:Number;
		private var _parentActualHeight:Number;

		private var backButton : MultilingualButton;

		public var soundSwitch : ToggleButton;
		public var pickerButton : MultilingualButton;
		public var toggleButton : MainMenuButton;
		private var userButtonContainer:LayoutGroup;

		private var _languages : Array;
		private var _languageList:List = null;

		public var languagePickerList : PickerList;
		public var restoreButton : Button;

		private var verticalOffset   : Number = 13;
		private var horizontalOffset : Number = 15;

		private var itemCollection : Array;

		public var _language : Language = null;
		public var _selectedLanguage : Language;
		public var languageChangedHandlerDisabled : Boolean;

		private var _signedIN : Boolean = false;


		private var _owner : IMainMenuHolder;
		private var completeShowCallback : Function = null;

		private var _friendProxy:FriendProxy;
		public static const MENU_WIDTH:int = 205;
		public static const START_SHOW:String = 'startShow';
		public  function MainMenuView() {
			super();

			if(Device.isIphoneXVariant()){
				horizontalOffset+=17;
			}

			this.addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
            _friendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
			visible = false;
		}

        public function get customUserName() : String {
            var str : String = selectedLanguage.translate("mob_garage","myGarage").toUpperCase();
            if (Constants.selectedUser && Constants.selectedUser.username){
				trace("customUserName:", Constants.selectedUser.username + " " + str.split(" ")[1]);
            }

            return str;
        }

		public function set owner(value : IMainMenuHolder)  : void {
			this._owner = value;
		}

		public function get owner():IMainMenuHolder{
			return this._owner;
		}

		public function get userSignedIn():Boolean{
			return (ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy).currentUser != null;
		}

		private function onAddedToStage(event:Event) : void {
			this.removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);

			language = owner.selectedLanguage;
			languageChangedHandlerDisabled = true;

			hide();
		}

		public function get signedIN() : Boolean {
			_signedIN=userSignedIn;
			return userSignedIn;
		}

		public function get selectedLanguage() : Language {
			var ln:Language = _selectedLanguage||AlertView.language;
			return ln;
		}
		public function set selectedLanguage(value :Language):void{
			_selectedLanguage = value;
		}

		public function get languages() : Array {
			return _languages;
		}

		public function set languages(value:Array):void{
			_languages = value;
			invalidate("languages");
		}

		private function addElement(item:*,index:int=-1) : void
		{
			if (!itemCollection) {
				itemCollection = [];
			}

			for (var i:int = 0; i < itemCollection; i++) {
				if (itemCollection[i].name == item.name) {
					return;
				}
			}
			index != -1 ? addChildAt(item,index) : addChild(item);
		}

		public function get text() : String {
			return _text;
		}
		public function set text(value:String):void{
			_text = value;
			invalidate("text");
		}

		public function set parentActualWidth(value:Number):void{
			_parentActualWidth = value;
		}
		public function get parentActualWidth() : Number{
			return _parentActualWidth;
		}

		public function set parentActualHeight(value:Number):void{
			_parentActualHeight = value;
		}
		public function get parentActualHeight():Number{
			return _parentActualHeight;
		}

		public function restoreButtonTriggeredHandler(event:Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
            if (userSignedIn || Constants.isIOS) {
                owner.dispatchExternalEvent(StackedScreen.USER_RESTORE_SELECTED);
            } else {
                owner.dispatchExternalEvent(BrandsScreen.LOGIN_SELECTED);
            }
		}

		public function updateStates() : void {
			if (languagePickerList){
				languagePickerList.width = 30;
				languagePickerList.height = 30;
			}

			if (signedIN) {
				soundSwitch.visible = false;
                restoreButton.visible = true;
				updateWithLanguage(AlertView.language);
			}else{
				soundSwitch.visible = true;
                restoreButton.visible = true;
			}
			invalidate();
		}

		private function updateItemsLanguage() : void {
			invalidate();
		}

		public function updateWithLanguage(language:Language) : void {
			if(!language){
				return;
			}
			_language = language;

			if (backButton){
				backButton.updateWithLanguage(language);
			}

			if (pickerButton){
				pickerButton.updateWithLanguage(language);
			}

			if (pickerButton2){
				pickerButton2.updateWithLanguage(language);
			}
		}

		private function closeLangMenuButtonTriggeredHandler(event:Event):void{
			if (pickerButton2){
				pickerButton2.alpha = 1.0;
				if (pickerButton2.disabledSkin) {
					pickerButton2.disabledSkin.alpha = 1.0;
				}
			}
			closeLanguagePickerList();
		}

		public function closeLanguagePickerList() : void{
			if (languagePickerList && languagePickerList.popUpContentManager) {
				languagePickerList.popUpContentManager.close();
			}
			languageChangedHandlerDisabled = false;
		}

		private function getElement(target:*,str:String) :*{
			if (!target || !target.numChildren) return null;

			for (var i:int = 0; i < target.numChildren; i++){
				if (target.getChildAt(i).name == str) return target.getChildAt(i);
			}
			return null;
		}

		private function onTouchOverlay(event:Event) : void {
			SoundManager.playSound(SoundManager.MENU_CLOSE);
			hide(true);
		}

		public function hide(animation:Boolean=false, time:Number=.1):void
		{
			BrandsScreen.adsBlocker = false;
			if (languagePickerList) {
				closeLanguagePickerList();
			}

			var target :* = owner;
			if (animation) {
				var tween : Tween = new Tween(target, .35, Transitions.EASE_IN_OUT);
					tween.moveTo(0,0);
					tween.onComplete = completeHide;
				Starling.juggler.add(tween);
			} else {
				target.x = 0;
				completeHide();
			}
		}

		private function completeHide() : void {
			dispatchEventWith(Event.CLOSE);
			isOpened = false;
		}

		private function modeUpdate():void{
			if(AppMode.self.mode == AppMode.MODE_HOME){

			}else if(AppMode.self.mode == AppMode.MODE_TUNING) {

            }
		}

		public function show(animation:Boolean=false, complete:Function=null):void {
			SoundManager.playSound(SoundManager.MENU);

            modeUpdate();
			updateStates();
			completeShowCallback = complete;
			dispatchEventWith(START_SHOW);
			var target :* = owner;
			if (animation) {
				var tween : Tween = new Tween(target, .35, Transitions.EASE_IN_OUT);
				tween.moveTo(actualWidth,0);
				tween.onComplete = completeShow;
				Starling.juggler.add(tween);
			} else {
				target.x = actualWidth;
				completeShow();
			}
		}

		private function completeShow():void {
			dispatchEventWith(Event.OPEN);

			if (completeShowCallback) {
				completeShowCallback();
				completeShowCallback = null;
			}
			isOpened = true;
		}



		private  function onCheckForLanguage(event:TimerEvent):void {
			if (!owner.languages) return;

			languages = owner.languages;
			if (languages && languages.length > 0) {
				languageTimer.stop();
				languageTimer.removeEventListener(TimerEvent.TIMER, onCheckForLanguage);
				languageTimer = null;

				if (owner.selectedLanguage) {
					//trace("onCheckForLanguage2");
					selectedLanguage = owner.selectedLanguage;
					if (selectedLanguage && languages) {
						var index : int = -1;
						for (var i : int = 0; i < languages.length; i++){
							if (selectedLanguage.code == languages[i].code) {
								index = i;
								break;
							}
						}
					}

					setLanguageCollection(languages);
					languagePickerList.selectedIndex = index;
					invalidate();
				}
			}
		}

		private function setLanguageCollection(lnArr:Array):void{
			languagePickerList.removeEventListener(Event.CHANGE,languagePickerListChangeHandler);
			languagePickerList.dataProvider = new ListCollection(lnArr);
			languagePickerList.addEventListener(Event.CHANGE,languagePickerListChangeHandler);
		}

		public function languagePickerListChangeHandler(event :Event) : void {
			if (languageChangedHandlerDisabled) return;

			try {
				if (pickerButton2) pickerButton2.label = LanguageItemRenderer.getShortname(languagePickerList.selectedItem as Language);
			}
			catch(error:Error){

			}

			if (languagePickerList && languagePickerList.selectedItem && selectedLanguage != languagePickerList.selectedItem as Language) {
				selectedLanguage = languagePickerList.selectedItem as Language;
				language = selectedLanguage;

				Language.selectedLanguage = owner.selectedLanguage = selectedLanguage;
				owner.dispatchExternalEvent(BrandsScreen.LANGUAGE_CHANGED);
				owner.updateWithLanguage(owner.selectedLanguage);
			}

			closeLanguagePickerList()
		}

		protected override function initialize() : void {
			super.initialize();

			languageChangedHandlerDisabled = true;
			this.x = -actualWidth;

			var wdth : Number = actualWidth - 2 * horizontalOffset;
			if(Device.isIphoneXVariant()){
				wdth+=horizontalOffset-5;
			}
            var layout : VerticalLayout = new VerticalLayout();
            layout.gap = 0;
            layout.hasVariableItemDimensions = true;
            layout.horizontalAlign = HorizontalAlign.LEFT;
            layout.verticalAlign = VerticalAlign.TOP;
			if (!LanguageItemRenderer.SHORT_LABEL) {
				LanguageItemRenderer.SHORT_LABEL={};
				LanguageItemRenderer.SHORT_LABEL['ru-RU'] = 'RUS';
				LanguageItemRenderer.SHORT_LABEL['en-US'] = 'ENG';
			}

			languagePickerList = new PickerList();

			languagePickerList.addEventListener(Event.CLOSE,function(event:Event) : void{});
			languagePickerList.addEventListener(Event.CHANGE,languagePickerListChangeHandler);
			languagePickerList.addEventListener(Event.OPEN,function(event:Event) : void {
				if (pickerButton2) {
					pickerButton2.label = LanguageItemRenderer.getShortname(languagePickerList.selectedItem as Language);
				}});
			languagePickerList.labelField = "name";
			languagePickerList.popUpContentManager = new DropUpPopUpContentManager();
			(languagePickerList.popUpContentManager as DropUpPopUpContentManager).gap = -15;

			pickerButton = ButtonFactory.circleButton("emptyButton") as ExtentedButton;


			languageChangedHandlerDisabled = false;
			languagePickerList.buttonFactory = function() : Button {return pickerButton;};
			languagePickerList.labelFunction = function(item:Object) : String {
				return LanguageItemRenderer.getShortname(item as Language);
			};
			languagePickerList.listFactory = function() : List {
				var texture:Texture = assetManager.getTexture("bgOfShareMenu");
				if(pickerButton2 && pickerButton2.parent) {
					pickerButton2.removeFromParent(true);
				}

				pickerButton2 = ButtonFactory.circleButton("emptyButton", 1.0) as ExtentedButton;
				pickerButton2.width  = languagePickerList.width;
				pickerButton2.height = languagePickerList.height;
				pickerButton2.addEventListener(Event.TRIGGERED,closeLangMenuButtonTriggeredHandler);
				var list : List = new List();
				_languageList = list;

				var layout : VerticalLayout = new VerticalLayout();
				layout.useVirtualLayout = false;
				list.layout = layout;
				list.setSize(languagePickerList.width, 105);
				list.verticalScrollPosition = 0;
				list.paddingTop = 15;
				list.paddingBottom = 15;
				list.backgroundSkin = new Image(texture);
				(list.backgroundSkin as Image).scale9Grid = new Rectangle(0, 30, texture.frameWidth, 24); //tt возможно не правильно нужно для вертикальной позиции задать
				list.backgroundSkin.y = 0;
				list.itemRendererFactory = function() : IListItemRenderer {
					var renderer :	LanguageItemRenderer = new LanguageItemRenderer();
					renderer.setSize(languagePickerList.width, languagePickerList.height * 35 / 40);
					return renderer;
				};
				return list;
			};



			if (!_languages && !languageTimer) {
				languageTimer = new Timer(40, 10);
				languageTimer.addEventListener(TimerEvent.TIMER, onCheckForLanguage);
				languageTimer.start();
			}

			restoreButton = ButtonFactory.circleDefaultButton("restore", 0.6, 0x0756B0, 0);
			restoreButton.scale = ButtonFactory.getMainMenuProfileButtonSize() / restoreButton.width;
			restoreButton.name = "profileButton";
			restoreButton.addEventListener(Event.TRIGGERED, restoreButtonTriggeredHandler);
			restoreButton.visible = true;

			if(!soundSwitch){
				createSoundSwitchButton();
			}
			soundSwitch.visible = false;

            userButtonContainer ||= new LayoutGroup();
			var buttonLayout:HorizontalLayout = new HorizontalLayout();
            buttonLayout.paddingTop = 3 * horizontalOffset / 2;
            buttonLayout.horizontalAlign = HorizontalAlign.LEFT;
            userButtonContainer.layout = buttonLayout;

            userButtonContainer.addChild(restoreButton);
            userButtonContainer.addChild(languagePickerList);
			userButtonContainer.addChild(soundSwitch);
			if (languagePickerList) {
				trace("languagePickerList.dp", languagePickerList.dataProvider);
				languagePickerList.width  = 30;
				languagePickerList.height = 30;
			}

			toggleButton = ButtonFactory.menuBigButton("a_back_button");
			toggleButton.visible=false;
			toggleButton.name = "toggleButton";
			toggleButton.width = toggleButton.height = 35;
			addElement(toggleButton);
			updateStates();
			_selectedLanguage = AlertView.language;
		}

		private function createSoundSwitchButton():void {
			soundSwitch = new ToggleButton();
			soundSwitch.defaultSkin = new Quad(1,1,0xFF0000);
			soundSwitch.defaultSkin.alpha = 0;
			var defaultIcon:Image = new Image(ButtonFactory.assetManager.getTexture("sound_on_icon"));
			defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
			soundSwitch.defaultIcon = defaultIcon;
			var defaultSelectedIcon:Image = new Image(ButtonFactory.assetManager.getTexture("sound_off_icon"));
			defaultSelectedIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
			soundSwitch.defaultSelectedIcon = defaultSelectedIcon;
			soundSwitch.width = soundSwitch.height = ButtonFactory.getMainMenuProfileButtonSize();
			soundSwitch.addEventListener(Event.CHANGE, soundChangeHandler);
			soundSwitch.isSelected = SoundManager.isMute;
		}

		private function soundChangeHandler(event:Event):void {
			SoundManager.soundMute(soundSwitch.isSelected);
			if(!soundSwitch.isSelected){
				SoundManager.playSound(SoundManager.SELECT_PART);
			}
		}


		override protected function draw() : void {
			super.draw();

			if (isInvalid(INVALIDATION_FLAG_SIZE)){
				toggleButton.x = actualWidth + horizontalOffset / 2;
				toggleButton.y = verticalOffset / 2;

				if(AlertView.language){

					if(pickerButton){
						pickerButton.updateWithLanguage(AlertView.language);
					}

					if(pickerButton2){
						pickerButton2.updateWithLanguage(AlertView.language);
					}
				}

				if (languagePickerList) {

					var facade : ApplicationFacade = ApplicationFacade.getInstance();
					var langProxy:LanguagesProxy= facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;

					if(langProxy.defaultLanguageCode=="en-US"){
						languagePickerList.visible = false;
					}

					if(!languagePickerList.dataProvider){


						if(langProxy.languages && AlertView.language){

							var index : int = -1;
							if (AlertView.language /*&& isInvalid("languages") &&*/ && langProxy.languages)
							{

								for (var i : int = 0; i < langProxy.languages.length; i++){
									if (AlertView.language.code == langProxy.languages[i].code) {
										index = i;
										break;
									}
								}
							}

							setLanguageCollection(langProxy.languages);
							languagePickerList.selectedIndex = index;
						}
					}
					languagePickerList.width = 30;
					languagePickerList.height = 30;
					languagePickerList.validate();
				}

				if(languagePickerList && languagePickerList.selectedItem)
				if (pickerButton2) {
					pickerButton2.label = LanguageItemRenderer.getShortname(languagePickerList.selectedItem as Language);
				}

				if(userButtonContainer){
					var btnCount:int = -1;
					var btnWidth:Number = 0;
					var j:int;
					for(j = 0; j < userButtonContainer.numChildren; j++){
						if(userButtonContainer.getChildAt(j).visible){
							btnWidth += userButtonContainer.getChildAt(j).width;
                            btnCount ++;
						}
					}

                    if(btnCount >= 1) {
                        var btnGap:Number = (userButtonContainer.width - btnWidth) / btnCount;
						var btnX:Number = 0;
                        for(j = 0; j < userButtonContainer.numChildren; j++){
                            if(userButtonContainer.getChildAt(j).visible){
                                userButtonContainer.getChildAt(j).x = btnX;
                                btnX += userButtonContainer.getChildAt(j).width + btnGap;
                            }
                        }
                    }else{
                        for(j = 0; j < userButtonContainer.numChildren; j++){
                            if(userButtonContainer.getChildAt(j).visible){
                                userButtonContainer.getChildAt(j).x = 0;
                                break;
                            }
                        }
					}
				}
			}
		}


	}
}
