package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.assets.AssetsProxy;
	import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.parts.PartMaterial;
	import com.despark.tuning3d.view.components.core.ExtendedLabel;
	import com.greensock.TweenLite;
	import com.utils.FontSelector;

	import feathers.controls.Label;

	import flash.geom.Rectangle;
	import flash.text.TextFormatAlign;

	import starling.display.Image;
import starling.display.Quad;
import starling.textures.Texture;

public class MaterialItemRenderer extends ListItemRenderer {
		public var language : Language;
		private var quad : Quad;
		private var HELPER_RECTANGLE : Rectangle = new Rectangle();
		public static const WIDTH:Number = 50;
		public static const HEIGHT:Number = 50;
		private var selectionSign:Image;
		private var iconId:String;
		private var iconImage:Image;
		private var assetsProxy : AssetsProxy;
	//
		private var _level:ExtendedLabel;
		override public function dispose():void{
			super.dispose();
			HELPER_RECTANGLE = null;
			language = null;
			data = null;
			//
			if(quad){
				quad.removeFromParent(true);
				quad = null;
			}
			if(selectionSign){
				selectionSign.removeFromParent(true);
				selectionSign = null;
			}
			if(iconImage){
				iconImage.removeFromParent(true);
				iconImage = null;
			}
			if(assetsProxy && iconId){
				//assetsProxy.removeTexture(iconId);
				iconId = null;
				assetsProxy = null;
			}
		}
				
		public function MaterialItemRenderer() {
			super();
			fontSize = 9;
			_textColor = 0xffffff;
			_bold = false;
		}
		
		
		protected override function initialize() : void {
			super.initialize();
			quad = new Quad( 1,  1, 0xffffff);
			quad.alpha=0;
			addChild(quad);
			
			label.paddingBottom = 0;
			label.paddingTop = 0;
			label.paddingLeft = 0;
			label.paddingRight = 0;
			imageLoader.visible = false;
		}
		
		public function get material() :PartMaterial {
			return data as PartMaterial;
		}
		
		protected override function commitData() : void {
			super.commitData();
			if(!language){
				return;
			}
			var text : String = material.label;
			label.text = text;
			label.width = NaN;
			label.height = NaN;
			label.validate();
			//
			if(iconId != material.name.toLowerCase()){
				iconId = material.name.toLowerCase();
				assetsProxy = ApplicationFacade.getInstance().retrieveProxy(AssetsProxy.NAME) as AssetsProxy;
				assetsProxy.loadMaterialIcon(iconId, function():void{
					if(iconImage){
						iconImage.removeFromParent(true);
					}
					iconImage = new Image(ButtonFactory.assetManager.getTexture(iconId));
					addChild(iconImage);
					iconImage.scale = (65 / 100) * ButtonFactory.assetManager.scaleFactor / 2;
					iconImage.color = 0xFF0000;
					//
					iconImage.x = (actualWidth - iconImage.width) / 2;
					iconImage.y = 12 + (actualHeight - 50) / 2;

					if(label){
						label.validate();
						label.x = (WIDTH - label.width) / 2;
						label.y = iconImage.y + iconImage.height + 1;
					}
					locateSelectIcon();
					updateLevelLabel();
				});
			}

			if(material.unlockLevel == -1){
				isEnabled = touchable = true;
				if(_level){
					_level.removeFromParent(true);
					_level = null;
				}
			}else{
				//isEnabled = touchable = false;
				if(!_level){
					_level = new ExtendedLabel();
					_level.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansSemiBold(), 10, 0xFFFFFF, TextFormatAlign.CENTER, false);
					_level.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("material_lock_icon"));
					_level.defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
					_level.labelOffsetX = 2;
					_level.paddingLeft = 6.5;
					var bg:Quad = new Quad(1,1,0x000000);
					bg.alpha = 0.6;
					_level.backgroundSkin = bg;
				}

				_level.text = "LEVEL " + material.unlockLevel.toString();
				addChild(_level);
				updateLevelLabel();
			}
		}

	private function updateLevelLabel():void{
		if(_level && iconImage){
			_level.visible = true;
			_level.width = iconImage.width;
			_level.height = iconImage.height;
			_level.x = iconImage.x + (iconImage.width - _level.width) / 2;
			_level.y = iconImage.y + (iconImage.height - _level.height) / 2;
			setChildIndex(_level, numChildren - 1);
		}

	}

		override protected function draw():void{
			super.draw();
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			if(sizeInvalid) {
				quad.width = actualWidth;
				quad.height = actualHeight;
			}
		}

		protected override function layout() : void {
			super.layout();
			locateSelectIcon();

			label.width = NaN;
			label.height = NaN;
			label.validate();
			label.x = (WIDTH - label.width) / 2;
			if(iconImage){
				label.y = iconImage.y + iconImage.height + 1;
			}

			updateLevelLabel();
		}

		
		protected override function autoSizeIfNeeded() : Boolean {
			var needsWidth : Boolean = isNaN(explicitWidth);
			var needsHeight : Boolean = isNaN(explicitHeight);
			
			if(!needsWidth && !needsHeight) {
				return false;
			}

			locateSelectIcon();
			
			label.width = NaN;
			label.height = NaN;
			label.validate();
			label.x = (WIDTH - label.width) / 2;
			if(iconImage){
				label.y = iconImage.y + iconImage.height + 1;
			}
			return setSizeInternal(WIDTH, owner.height, false);
		}

		protected override function commitSelection() :void {
			super.commitSelection();
			textColor = (material.unlockLevel == -1?0xFFFFFF: 0x999999);
			if(isSelected && !selectionSign){
				selectionSign = new Image(ButtonFactory.assetManager.getTexture("material_select"));
				addChild(selectionSign);
				selectionSign.alpha = 0;
				TweenLite.to(selectionSign, 0.75, {"alpha": 1});
			}else if(!isSelected && selectionSign){
				selectionSign.removeFromParent(true);
				selectionSign = null;
			}
			locateSelectIcon();
		}

		private function locateSelectIcon():void{
			if(iconImage){
				iconImage.scale = (65 / 100) * ButtonFactory.assetManager.scaleFactor / 2;
				iconImage.x = (WIDTH - iconImage.width) / 2;
				iconImage.y = 12 + (actualHeight - 50) / 2;
			}
			if(selectionSign){
				selectionSign.parent.setChildIndex(selectionSign, selectionSign.parent.numChildren);
				selectionSign.scale = ButtonFactory.assetManager.scaleFactor / 2;
				selectionSign.x = (WIDTH - selectionSign.width) / 2;
				if(iconImage){
					selectionSign.y = iconImage.y + (iconImage.height - selectionSign.height) / 2;
				}
			}
		}
		
		public override function get useEmbeddedFont() : Boolean {
			if(!language){
				return null;
			}
			return language.supportsEmbeddedFont;
		}
	}
}



