package com.despark.tuning3d.view.components.core
{
	import com.utils.FontSelector;
	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.media.StageWebView;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import feathers.controls.ImageLoader;
	import feathers.controls.Label;
	import feathers.controls.List;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.controls.text.BitmapFontTextRenderer;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	import feathers.text.BitmapFontTextFormat;
	
	import starling.core.Starling;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.text.TextField;
	
	public class PageItemRenderer extends FeathersControl implements IListItemRenderer
	{
		public var fontSize : int;
		
		protected var label : Label;
		//protected var stageWebView : StageWebView;
		protected var imageLoader : ImageLoaderWithStates;
		protected var _index : int = -1;
		protected var _owner:List;
		protected var _data : Object;
		protected var _isSelected:Boolean;
		protected var useBitmapFont : Boolean;
		
		private var touchPointID : int;
		private var selectedByTouching : Boolean;
		private static const HELPER_POINT : Point = new Point();
		
		private var _textColor : uint;
		private var _bold : Boolean;
		
		public function PageItemRenderer() 
		{
			super();
			
			fontSize = 7;
			textColor = 0x2D3327;
			
			addEventListener(TouchEvent.TOUCH, touchHandler);
		}
		
		protected var _factoryID:String;
		public function get factoryID():String{
			return _factoryID;
		}
		
		
		public function set factoryID(value:String):void{
			
			_factoryID = value;
		}
		
		public function set textColor(value : uint) : void
		{
			_textColor = value;
			
			if(label)
			{
				label.textRendererProperties.textFormat = getTextFormat();
			}
		}
		
		public function set bold(value : Boolean) : void
		{
			_bold = value;
			
			if(label)
			{
				label.textRendererProperties.textFormat = getTextFormat();
			}
		}
		
		public function getTextFormat() : Object
		{
			if(useBitmapFont)
			{
				return getBitmapFontTextFormat();
			}
			
			return getTextFieldTextFormat();
		}
		
		public function getBitmapFontTextFormat() : BitmapFontTextFormat
		{
			var bitmapFontTextFormat : BitmapFontTextFormat = new BitmapFontTextFormat(
				TextField.getBitmapFont("openSans"), fontSize, textColor);
			
			bitmapFontTextFormat.align = "center";
			
			return bitmapFontTextFormat;
			
		}
		
		public function getTextFieldTextFormat() : TextFormat
		{
			var textFormat : TextFormat = new TextFormat(FontSelector.getFont(_bold), fontSize, _textColor);
			textFormat.align = TextFormatAlign.CENTER;
			textFormat.bold=_bold;
			
			return textFormat;
		}
		
		public function get textColor() : uint
		{
			return _textColor;
		}
		
		public override function dispose() : void 
		{
			//stageWebView.dispose();
			imageLoader.dispose();
			super.dispose();
		}
		
		public function get index():int
		{
			return _index;
		}
		
		public function set index(value : int) : void
		{
			if(_index == value)
			{
				return;
			}
			
			_index = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get owner() : List
		{
			return List(_owner);
		}
		
		public function set owner(value : List) : void
		{
			if(_owner == value)
			{
				return;
			}
			_owner = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get data() : Object
		{
			return _data;
		}
		
		public function set data(value : Object) : void
		{
			if(_data == value)
			{
				return;
			}
			
			_data = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		
		public function get isSelected() : Boolean
		{
			return _isSelected;
		}
		
		public function set isSelected(value : Boolean) : void
		{
			if(_isSelected == value)
			{
				if(_isSelected && selectedByTouching)
				{
					dispatchEventWith("itemSelectedAgain", true);
				}
				
				return;
			}
			
			_isSelected = value;
			invalidate(INVALIDATION_FLAG_SELECTED);
			dispatchEventWith(Event.CHANGE);
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			
			//stageWebView = new StageWebView();
			
			
			
			label = new Label();
			//addChild(label);
			
			if(useBitmapFont)
			{
				label.textRendererFactory = function():Object
				{
					var textRenderer:BitmapFontTextRenderer = new BitmapFontTextRenderer();
					
					textRenderer.textFormat = getTextFormat() as BitmapFontTextFormat;
				
					textRenderer.useSeparateBatch = false;
					
					return textRenderer;
				}
			}
			else
			{
				label.textRendererFactory = function():Object
				{
					var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
					
					textRenderer.textFormat = getTextFormat() as TextFormat;
					textRenderer.embedFonts = FontSelector.embedFonts();
					
					return textRenderer;
				}
			}
			
			imageLoader = new ImageLoaderWithStates();
			imageLoader.overrideDispose = true;
			addChild(imageLoader);
		}
		
		override protected function draw():void
		{
			var dataInvalid : Boolean = isInvalid(INVALIDATION_FLAG_DATA);
			var selectionInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SELECTED);
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);

			
			
			if(dataInvalid)
			{
				commitData();
			}
			
			if(selectionInvalid) 
			{
				commitSelection();
			}
			
			var autoSizeNeeded : Boolean = autoSizeIfNeeded();
			
			if(autoSizeNeeded || sizeInvalid)
			{
				layout();
			}
			
			
		}
		
		public function get useEmbeddedFont() : Boolean 
		{
			return true;
		}
		
		protected function autoSizeIfNeeded():Boolean
		{
			return false;
		}
		
		protected function commitData() : void
		{
			label.text = data.testText;
			
			if(data.texture){
				imageLoader.source = data.texture;
			}
			
			//if(data.htmlText){
				//stageWebView.loadString(data.htmlText);
			//}
			//stageWebView.loadURL("https://itunes.apple.com/us/app/ibooks/id364709193");
			//if(data.htmlText){
			//	stageWebView.loadString(data.htmlText);
			//}
		}
		
		protected function commitSelection() : void
		{
			imageLoader.selected = isSelected;
		}
		
		protected function layout():void
		{
			//label.width = actualWidth;
			//label.height = 13;
			//label.y = actualHeight - label.height;
			
			//imageLoader.x = 0;
			//imageLoader.y = 0;
			imageLoader.width = actualWidth;
			imageLoader.height = actualHeight;// - (label.text ? label.height : 0);
			
			///*
			
			
			
			
			//	*/
				/*
			if(text){
				stageWebView.loadString(text);
			}else if(url){
				stageWebView.loadURL(url);
			}*/
		}
		
		protected function touchHandler(event : TouchEvent):void
		{
			const touches : Vector.<Touch> = event.getTouches(this);
			

			for each(var touch : Touch in touches)
			{
				if(touch.phase == TouchPhase.ENDED)
				{
					
					touch.getLocation(stage, HELPER_POINT);
					
					const isInBounds : Boolean = contains(stage.hitTest(HELPER_POINT));
					
					if(isInBounds)
					{
						selectedByTouching = true;
						isSelected = true;
						selectedByTouching = false;
					}
					
					//AlertView.show('fuck' + isInBounds + " " + HELPER_POINT.x + " " + HELPER_POINT.y, -1, false);
					
					return;
				}
			}
		}
	}
}