package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.packages.Purchase;
	import com.despark.tuning3d.view.components.brands.PacketItemRenderer;
	import com.despark.tuning3d.view.components.brands.ParchaseItemRenderer;
	import com.utils.Console;
	import com.utils.FontSelector;

	import feathers.controls.List;
	import feathers.controls.ScrollBarDisplayMode;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.core.FeathersControl;
	import feathers.data.ListCollection;
	import feathers.layout.VerticalAlign;
	import feathers.layout.VerticalLayout;

	import flash.geom.Rectangle;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.display.Image;
	import starling.events.Event;
	import starling.textures.Texture;
	import starling.utils.AssetManager;

	public class PurchaseView extends FeathersControl implements Multilingual
	{
		
		
		public static var assetManager : AssetManager;   
		public static var language : Language;
		
		
		private var backgroundImage:Image;
		

		
		private var label : MultilingualLabel;
		private var _text : String;
		
		private var mainList : List;
		
		private var _dataProvider:Array;
		
		public function PurchaseView() {
			super();
		}
		
		public function set dataProvider(value:Array):void{
			_dataProvider = value;
			text = "PURCHASE LIST";
			Console.log("PURCHASE LIST");
			if(mainList.dataProvider){
				mainList.dataProvider.removeAll();
			}
			if(_dataProvider && _dataProvider.length){
				mainList.dataProvider = new ListCollection(_dataProvider);
			}else{
				mainList.dataProvider = new ListCollection([]);
			}
		}
		
		public function get dataProvider():Array{
			return _dataProvider;
		}
		
		public function updateWithLanguage(language:Language):void
		{
			

			if(label)
				label.updateWithLanguage(language);
		}
		
		protected override function initialize() : void {
			super.initialize();
			var backgroundTexture : Texture = assetManager.getTexture("controlBackground");
			backgroundImage = new Image(backgroundTexture);
			backgroundImage.scale9Grid = new Rectangle(16, 24, 8, 6);
			backgroundImage.alpha = 1.0;
			addChild(backgroundImage);

			label = new MultilingualLabel();
			label.textFormat = new TextFormat(FontSelector.getFont(),  10, 0xc1c3c3 );
			label.textFormat.align = TextFormatAlign.CENTER;
			label.wordWrap = true;

			addChild(label);
			
			var mainListLayout :VerticalLayout = new VerticalLayout();
			mainListLayout.paddingTop = 5;
			mainListLayout.useVirtualLayout = false;
			mainListLayout.scrollPositionVerticalAlign=VerticalAlign.TOP;

			mainList = new List();
			mainList.name = "PurchaseViewList";
			mainList.clipContent = true;
			mainList.layout = mainListLayout;
			mainList.scrollBarDisplayMode = ScrollBarDisplayMode.FIXED;
			mainList.addEventListener(PacketItemRenderer.TOUCH_PACKET_INFO , touchPacketInfo );
			mainList.addEventListener(Event.CHANGE, changePacketItem);
			mainList.itemRendererFactory = function() : IListItemRenderer {
				var renderer:ParchaseItemRenderer = new ParchaseItemRenderer();
				renderer.setSize(162, 23);
				return renderer;
			};
			addChild(mainList);
			
		}
		
	
		protected function changePacketItem(event:Event):void{
			if(mainList.selectedIndex<0)  
				return;
			mainList.selectedIndex=-1;
		}
		
		protected function touchPacketInfo(event:Event):void{
			
		}
		
		public function get text():String {
			return _text;
		}
		
		public function set text(value:String):void {
			_text = value;
			invalidate("text");
		}
		
		private function backButtonTriggeredHandler(event : Event) : void {
			hide();
			this.dispatchEventWith(Event.CLOSE);
		}
		
		
		public function hide():void{
			this.visible=false;
		}
		
		public function show():void{
			text = "PURCHASE LIST";
			this.visible=true;
			this.dispatchEventWith(Event.OPEN);
		}
		
		override protected function draw() : void {
			super.draw();
			if(isInvalid("text")) {
				if(PackagesView.language) {
					label.updateWithLanguage(PackagesView.language);
				}else {
					var l:Language = Language.createInstance("en-US");
					label.updateWithLanguage(l);
				}
				label.text = text;
				label.visible = false;
			}

			if(isInvalid(INVALIDATION_FLAG_SIZE)) {
				label.x = 0;
				label.width = actualWidth;
				label.y = 5;

				backgroundImage.x = 0;
				backgroundImage.width = actualWidth;
				backgroundImage.height = 120;

				mainList.x = 0;
				mainList.y = 25;
				mainList.width = actualWidth;
				mainList.height = 80;
			}
		}
    }
}