package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.utils.FontSelector;

	import feathers.controls.Button;
	import feathers.controls.TextInput;
	import feathers.controls.text.StageTextTextEditor;
	import feathers.core.FeathersControl;
	import feathers.core.ITextEditor;
	import feathers.events.FeathersEventType;

	import flash.geom.Rectangle;

	import starling.display.Image;
	import starling.events.Event;
	import starling.textures.Texture;
	import starling.utils.AssetManager;

	public class TextInputExtended extends FeathersControl implements Multilingual
	{
		public static var assetManager : AssetManager;
		
		public var fun_enterPress : Function; 
		public var textInput : TextInput;
		public var clearButton : Button;
		
		private var titleLabel : MultilingualLabel;
		private var _title     : String;
		private var _password  : Boolean;
		private var _text : String;
		
		public var owner : SearchView;
		
		private var _multiline:Boolean=false;
		private var _search:Boolean=false;
		
		public var _owner :SearchView;
		
		//private var _testQuad:Quad;
		
		public var isIOS :Boolean = false;
		
		public function TextInputExtended(multiline:Boolean=false, search:Boolean=false,own:SearchView=null)
		{
			if (own) _owner = own;
			
			_multiline = multiline;
			_search = search;
			
			super();

			//setSizeInternal(actualWidth, actualHeight, false);
			
			this.addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		protected function onEnterFrame(event:Event):void 
		{
			if (!Device.isIos() && !Constants.IS_EMULATOR){
//				MeasureKeyboard.getInstance().setKeyboardAdjustDefault();
			}
		}
		
		override public function set width(value:Number):void
		{  
			if (textInput) {
				textInput.width = value;
			}
			else {
				super.width = value;
			}
		}
		
		public function get password():Boolean {
			return _password;
		}
		public function set password(value : Boolean) : void
		{
			_password = value;
			invalidate("password");
		}
		
		public function get title() : String {
			return _title;
		}
		public function set title(value : String) : void
		{
			_title = value;
			invalidate("title");
		}
		
		public function get text() : String {
			return textInput.text;
		}
		public function set text(value : String) : void{
			_text = value;
			invalidate("text");
		}
		
		protected override function initialize() : void
		{
			super.initialize();
			
			isIOS = Device.isIos();
			
			
			//_testQuad = new Quad(50,18,0xff0000);
			//addChild(_testQuad);
			
			textInput = new TextInput();
			
			var texName:String = "textField";
			if (_search){
				texName = "textFieldWhite";
			}
				
			if (!_search){
				var texture  : Texture = assetManager.getTexture(texName);
				textInput.backgroundSkin = new Image(texture);
				(textInput.backgroundSkin as Image).scale9Grid = new Rectangle(5, 5, 5, 5);
			}
			
			textInput.addEventListener(Event.CHANGE, textInputChangeHandler);
			textInput.addEventListener(FeathersEventType.ENTER, enterPress);
			
			if (!isIOS)
			{
				textInput.textEditorFactory = function() : ITextEditor {
					var editor:StageTextTextEditor = new StageTextTextEditor();
						editor.fontSize = 15;
						editor.color = 0x58595b;
					return editor;
				}
			}
			else
			{
				textInput.textEditorFactory = function() : ITextEditor{
					var editor : StageTextTextEditor = new StageTextTextEditor();
						editor.fontFamily = FontSelector.getFont();
						editor.fontSize = 15;
						editor.color = 0x58595b;
					return editor;
				}
			}
			textInput.alpha = 0.5;
			addChild(textInput);      
			
			if (!isIOS) {
				clearButton = ButtonFactory.circleButton("clearInput");
				clearButton.addEventListener(Event.TRIGGERED, clearTextTriggeredHandler);
				clearButton.width  = 16;
				clearButton.height = 16;
				clearButton.visible = false;
				addChild(clearButton);
			}
		}
		
		protected function clearTextTriggeredHandler(event:Event):void
		{
			textInput.text = _text = "";
			
			dispatchEventWith(Event.CHANGE);
			
			if (_owner){
				_owner.onSearchToggle();
			}
		}
		
		public function updateWithLanguage(language : Language) : void {
			if (titleLabel) titleLabel.updateWithLanguage(language);
		}
		
		protected override function draw() : void
		{
			super.draw();
			
			if (isInvalid(INVALIDATION_FLAG_SIZE))
			{
				
				//_testQuad.width = 50;
				//_testQuad.height = actualHeight;
				
				//trace("actualWidth:", actualWidth, "actualHeight", actualHeight)
				
				textInput.height = 18;		
				textInput.width = actualWidth;
				
				textInput.y=(actualHeight-textInput.height)/2;
				
				
				if (clearButton){
					clearButton.y =  (actualHeight - clearButton.height) / 2;
				}
			}
			
			if (isInvalid("title") && titleLabel){
				titleLabel.text = title;
			}
			
			if (isInvalid("text")){
				textInput.text = _text;
			}
		}
		
		private function textInputChangeHandler(event:Event) : void
		{	
			_text = textInput.text;
			if (clearButton){
				clearButton.visible = text.length > 0 ? true : false;
			}
			
			if (isIOS) {

			}

			dispatchEventWith(Event.CHANGE);
		}
		
		private function enterPress(event:Object=null) : void
		{
			if (fun_enterPress)	{
				fun_enterPress();
			}
		}
	}
}


