package com.despark.tuning3d.view.components.downloads
{
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.MultilingualButton;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.despark.tuning3d.view.components.core.PopUp;
	import com.despark.tuning3d.view.components.downloads.DownloadsDefCarSlotView;
	import com.despark.tuning3d.view.components.downloads.DownloadsPartSlotView;
	import com.utils.FontSelector;
	
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import feathers.controls.Button;
	import feathers.controls.Label;
	import feathers.controls.ScrollContainer;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	
	import starling.core.Starling;
	import starling.display.Quad;
	import starling.events.Event;

	public class DownloadsPopUp extends PopUp implements Multilingual
	{//cleanCacheButton
		public static const CLEAN_CACHE_SELECTED : String = "cleanCacheSelected";
		public static const DOWNLOAD_SELECTED : String = "DownloadSelected";
		public static const DELETE_SELECTED : String = "DeleteSelected";
		public static const MIN_WIDTH : int = 300;
		
		public var totalValueLabel : Label;
		public var car : Car;
		public var partSlotsForDownload : Array;
		
		private var currentLanguage : Language;
		private var container : ScrollContainer;
		private var totalLabel : MultilingualLabel;
		private var cleanCacheButton : MultilingualButton;
		private var downloadButton : MultilingualButton;
		private var deleteButton : MultilingualButton;
		private var intializingLabel : MultilingualLabel;
		private var downloadingView : FeathersControl;
		private var partSlotViews : Array;
		
		private var _saveDefCar : Boolean;
		
		private var defCarView : DownloadsDefCarSlotView;

		public function set saveDefCar(value : Boolean) : void
		{
			_saveDefCar = value;
			
			updateDefCarView();
			
		}
		
		protected function updateDefCarView() : void
		{
			
			if(defCarView){
				defCarView.checked(_saveDefCar);
			}
		}
		
		public function DownloadsPopUp()
		{
			super()
						
			setSizeInternal(Starling.current.stage.width - 100, Starling.current.stage.height - 100, false);
			
			if(width <= MIN_WIDTH)
			{
				setSizeInternal(Starling.current.stage.width - 40, Starling.current.stage.height - 40, false);
			}
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			currentLanguage = language;
			
			title = language.translate("mob_downloads", "customDownload").toUpperCase();
			titleLabel.updateWithLanguage(language);
			
			intializingLabel.updateWithLanguage(language);
			
			totalLabel.text = language.translate("mob_downloads", "total");
			totalLabel.updateWithLanguage(language);
			
			cleanCacheButton.updateWithLanguage(language);
			cleanCacheButton.label = "   CLEAN CACHE"//language.translate("mob_downloads", "cleanCache").toUpperCase();
			
			downloadButton.updateWithLanguage(language);
			downloadButton.label = language.translate("mob_downloads", "download").toUpperCase();
			
			deleteButton.updateWithLanguage(language);
			deleteButton.label = language.translate("mob_downloads", "delete").toUpperCase();
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			
			container = new ScrollContainer();
			container.visible = false;
			container.paddingTop = container.paddingBottom = 0;
			container.paddingLeft = container.paddingRight = 10;
			addChild( container );
			
			intializingLabel = new MultilingualLabel();
			intializingLabel.height = 20;
			intializingLabel.textFormat = new TextFormat(FontSelector.getFont(), 12, 0xFFFFFF);
			intializingLabel.textFormat.align = TextFormatAlign.CENTER;
			addChild(intializingLabel);
			
			totalLabel = new MultilingualLabel();
			totalLabel.visible = false;
			totalLabel.height = 20;
			totalLabel.textFormat = new TextFormat(FontSelector.getFont(), 12, 0xFFFFFF);
			addChild(totalLabel);
				
			totalValueLabel = new MultilingualLabel();
			totalValueLabel.visible = false;
			totalValueLabel.height = 20;
			totalValueLabel.width = 100;
			addChild(totalValueLabel);
			
			totalValueLabel.textRendererFactory = function() : Object
			{
				var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
				
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 12, 0x989C9D);
				textRenderer.embedFonts =FontSelector.embedFonts();
				
				return textRenderer;
			}
				
				//cleanCacheButton
			cleanCacheButton = ButtonFactory.cleanCacheButton(null, 0xed1c24);
			cleanCacheButton.visible = false;
			cleanCacheButton.addEventListener(Event.TRIGGERED, cleanCacheButtonTriggeredHandler);
			addChild(cleanCacheButton);
			
			downloadButton = ButtonFactory.defaultButton(null, 0xED3F25);
			downloadButton.visible = false;
			downloadButton.addEventListener(Event.TRIGGERED, downloadButtonTriggeredHandler);
			addChild(downloadButton);
			
			deleteButton = ButtonFactory.defaultButton(null, 0xB2BDC1);
			deleteButton.addEventListener(Event.TRIGGERED, deleteButtonTriggeredHandler);
			deleteButton.visible = false;
			addChild(deleteButton);
			
			contentBackgroundImage.visible = false;
		}
	
		
		override protected function draw() : void
		{
			super.draw();
			
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
				container.x = contentBackgroundImage.x;
				container.y = contentBackgroundImage.y;
				container.width = contentBackgroundImage.width;
				container.height = contentBackgroundImage.height;
				
				totalLabel.x = contentBackgroundImage.x - 3;
				totalLabel.y = contentBackgroundImage.y + contentBackgroundImage.height + 15;
				totalLabel.width = 50;
				
				totalValueLabel.x = totalLabel.x + totalLabel.width;
				totalValueLabel.y = totalLabel.y;
				
				cleanCacheButton.x = actualWidth - cleanCacheButton.width - 25;
				cleanCacheButton.y = totalLabel.y + (downloadButton.height - cleanCacheButton.height)/2;
				
				downloadButton.x = cleanCacheButton.x - downloadButton.width - 5;
				downloadButton.y = totalLabel.y;
				
				deleteButton.x = downloadButton.x - deleteButton.width - 5;
				deleteButton.y = downloadButton.y;
				
				intializingLabel.width = actualWidth;
				intializingLabel.y = int((actualHeight - intializingLabel.height) / 2);
			}
			
			if(isInvalid("partSlotViews") && partSlotsForDownload)
			{
				if(!partSlotViews)
				{
					createPartSlotViews();
				}
				else 
				{
					refreshPartSlotViews();
				}
			}
		}
		
		private function createPartSlotViews() : void
		{
			const verticalGap : int = 0;
			const horizontalGap : int = 20;
			const itemHeight : int = 22;
			const itemsPerColumn : int = int( (container.height - 2 * container.paddingTop + verticalGap) / (itemHeight + verticalGap) );
			
			var initialY : int =  int((container.height - 2 * container.paddingTop - ((itemHeight + verticalGap) * (itemsPerColumn - 1) + itemHeight)) / 2);
			
			var currentX : int = 0;
			var currentY : int = 0;
			var partSlotIndex : int = 0;
			
			container.removeChildren();
			partSlotViews = [];
			
			var primalsCount : int = 0;
			
			for each(var slot : PartSlot in partSlotsForDownload)
			{
				if(!slot.isPrimal)
				{
					primalsCount++;
				}
			}
			
			{//для загрузки машины с параметрами по умолчанию
				primalsCount++;
				defCarView  = new DownloadsDefCarSlotView();
				
				defCarView.updateWithLanguage(currentLanguage);
				defCarView.assetManager = assetManager;
				defCarView.x = currentX;
				defCarView.y = currentY;
				defCarView.width = 130;
				defCarView.height = itemHeight;
				
				updateDefCarView();
				
				
				
				//partSlotViews.push(defCarView);// попробуем отключить refresh
				container.addChild(defCarView);
				
				
				{
					if(currentY == 0)
					{
						defCarView.height = itemHeight + initialY;
						defCarView.bias = initialY;
					}
					currentY += defCarView.height + verticalGap;
				}
				
				
				partSlotIndex++;
				
				
				
				
			}
						
			for each(var partSlot : PartSlot in partSlotsForDownload)
			{
				if(!partSlot.isPrimal && partSlot.possibleParts.length>0)  //if(!partSlot.isPrimal && partSlot.sizeInMegabytes>0)
				{
					var tunedPartView : DownloadsPartSlotView = new DownloadsPartSlotView();
					
					tunedPartView.updateWithLanguage(currentLanguage);
					tunedPartView.partSlot = partSlot;
					tunedPartView.assetManager = assetManager;
					tunedPartView.x = currentX;
					tunedPartView.y = currentY;
					tunedPartView.width = 140;
					tunedPartView.height = itemHeight;
										
					partSlotViews.push(tunedPartView);
					container.addChild(tunedPartView);
					
					if((partSlotIndex + 1) % itemsPerColumn == 0 && partSlotIndex > 0 && partSlotIndex < primalsCount - 1) 
					{
						currentY = 0;
						currentX += tunedPartView.width + horizontalGap / 2;
						
						var separatorY : int = initialY + int((tunedPartView.height - 11)/2)
						
						var separator : Quad = new Quad(1, container.height - 2 * separatorY, 0xDADBDB);
						separator.x = currentX;
						separator.y = separatorY;
						container.addChild( separator );
						
						currentX += horizontalGap / 2;
						
						tunedPartView.height = itemHeight + initialY;
						tunedPartView.bias = -initialY;
					}
					else 
					{
						if(currentY == 0)
						{
							tunedPartView.height = itemHeight + initialY;
							tunedPartView.bias = initialY;
						}
						currentY += tunedPartView.height + verticalGap;
					}
					
					partSlotIndex++;
				}
			}
		}
		
		public function setDownloadButtonEnabled(value : Boolean) : void
		{
			downloadButton.alpha = value ? 1.0 : 0.5;
			downloadButton.touchable = value;
		}
		
		public function setDeleteButtonEnabled(value : Boolean) : void
		{
			deleteButton.alpha = value ? 1.0 : 0.5;
			deleteButton.touchable = value;
		}
		
		private function showPartSlotsForSelection(value : Boolean) : void
		{
			container.visible = value;
			cleanCacheButton.visible = value;
			downloadButton.visible = value;
			deleteButton.visible = value;
			totalLabel.visible = false;//value;
			totalValueLabel.visible = false;//value;
			contentBackgroundImage.visible = value;
			closeButton.visible = value;
		}
		
		public function enterSelectionState() : void
		{
			showPartSlotsForSelection(true);
			intializingLabel.visible = false;
			
			if(downloadingView)
			{
				removeChild(downloadingView);
				downloadingView = null;
			}
			
			invalidate("partSlotViews");
		}
		
		public function enterIntializationState() : void
		{
			if(downloadingView)
			{
				removeChild(downloadingView);
				downloadingView = null;
			}
			
			showPartSlotsForSelection(false);
			intializingLabel.visible = true;
			intializingLabel.text = currentLanguage.translate("mob_downloads", "initializing").toUpperCase();
		}
		
		public function enterPreparingState() : void
		{	
			showPartSlotsForSelection(false);
			intializingLabel.visible = true;
			
			intializingLabel.text = currentLanguage.translate("mob_downloads", "preparingForDownload").toUpperCase();
		}
		
		public function enterDeletionState() : void
		{
			showPartSlotsForSelection(false);
			intializingLabel.visible = true;
			intializingLabel.text = currentLanguage.translate("mob_downloads", "deleting").toUpperCase();
		}
		
		public function enterDownloadingState(view : FeathersControl) : void
		{
			showPartSlotsForSelection(false);
			intializingLabel.visible = false;
			
			downloadingView = view;
			addChild(downloadingView);
			
			downloadingView.width = actualWidth;
			downloadingView.height = actualHeight;
		}
		
		public function refreshPartSlotViews() : void
		{
			for each(var tunedPartView : DownloadsPartSlotView in partSlotViews)
			{
				tunedPartView.refresh();
			}
		}
		//
		private function cleanCacheButtonTriggeredHandler(event : Event) : void
		{
			dispatchEventWith(CLEAN_CACHE_SELECTED);
		}
		
		
		private function downloadButtonTriggeredHandler(event : Event) : void
		{
			dispatchEventWith(DOWNLOAD_SELECTED);
		}
		
		private function deleteButtonTriggeredHandler(event : Event) : void
		{
			dispatchEventWith(DELETE_SELECTED);
		}
	}
}