package com.despark.tuning3d.view.components.friends {
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.friend.Friend;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.challenges.controls.SearchBox;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.MultilingualLabel;
import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.gallery.GalleryLabelFactory;
    import com.despark.tuning3d.view.components.gallery.SearchInput;
    import com.distriqt.AdsManager;
    import com.utils.DevHelper;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.ButtonGroup;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
import feathers.controls.List;
import feathers.controls.Radio;
import feathers.controls.ScrollBar;
import feathers.controls.ScrollBarDisplayMode;
import feathers.controls.ScrollPolicy;
import feathers.controls.renderers.IListItemRenderer;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;
import feathers.core.ToggleGroup;
import feathers.data.ListCollection;
import feathers.layout.AnchorLayout;
import feathers.layout.AnchorLayoutData;
import feathers.layout.Direction;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;
import feathers.layout.VerticalLayout;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.core.Starling;
import starling.display.Image;

import starling.display.Quad;
import starling.display.Sprite;
import starling.events.Event;

public class FriendsListPopUp extends PopUp {

    private static const SEARCH_IDLE_TIME : Number = 1.2;//sec

    public static const SEARCH_FRIENDS : String = "SearchFriends";
    public static const LOAD_MORE_FRIENDS: String = 'LoadMoreFriends';
    public static const PENDING_FRIENDS_READ : String = "PendingFriendsRead";
    public static const LOAD_MORE_PENDINGS: String = 'LoadMorePENDINGS';

    protected static const STATE_FRIENDS : String = 'stateFriends';
    protected static const STATE_REQUESTS : String = 'stateRequests';
    protected static const STATE_SEARCH_TYPING : String = 'stateSearching'; // typing
    protected static const STATE_SEARCH_WAITING_RESPONSE : String = 'stateSearchWaitingResponse'; // sent request. waiting for response
    protected static const STATE_SEARCH_IDLE : String = 'stateSearchIdle'; // search has been showed

    private var _state : String;
    private var _prevSearchText : String = "";
    private var _gap : int = 10;
    private var _totalFriends : int = 0;

    private var _screenTitle:Label;
    private var _searchInput:SearchInput;
    private var _topContainer:LayoutGroup;
    private var _alertView:AlertView;
    private var _friendsList:List;

    private var _searchResultCollection : ListCollection;
    private var _friendsListCollection : ListCollection;
    private var _requestedCollection:ListCollection;

    private var _group : ToggleGroup;
    private var _friendsButton : Radio;
    private var _requestsButton : Radio;

    public static var sentData   : Array = [];
    public static var pendingData : Array = [];

    private var _unreadRequests:Boolean = false;
    private var _pendingFriendsUnreadIcon:Image;
    private var _showBannerAfterExit:Boolean;
    private var _bannerY:Number;
    private var _bannerScreen:String;

    private var _messageLabel : Label;
    private var _currentFriend:Friend;
    private var _onComplete:Function;
    public function FriendsListPopUp(data:Object = null) {
        super();
        _showBannerAfterExit = (data && data.hasOwnProperty("showBannerAfterExit") && data.showBannerAfterExit);
        _bannerScreen = (data && data.hasOwnProperty("bannerScreen"))?data["bannerScreen"]:BannerScreen.BANNER_SCREEN;
        _bannerY = AdsManager.bannerY;
    }

    override protected function initialize() : void {
        super.initialize();

        setSizeInternal(this.stage.stageWidth*1, this.stage.stageHeight, false);
        background.color = 0xFFFFFF;
        background.alpha = 1;
        contentBackgroundImage.visible = false;

        _topContainer = new LayoutGroup();
        _topContainer.layout = new AnchorLayout();
        var topBg:Quad = new Quad(1,1, 0x184161);
        topBg.width = actualWidth;
        topBg.height = BannerStackedScreen.BANNER_SIZE;
        _topContainer.addChild(topBg);
        addChild(_topContainer);


        /**
         * Buttons
         */
        var layData : AnchorLayoutData = new AnchorLayoutData();
        layData.left = _gap;
        layData.top = BannerStackedScreen.BANNER_SIZE + (BannerStackedScreen.TOOLBAR_SIZE - 21) / 2 - 1;
        var layout : HorizontalLayout = new HorizontalLayout();
        layout.paddingLeft = layout.paddingRight = layout.paddingTop = layout.gap = 1;
        var buttonsContainer : LayoutGroup = new LayoutGroup();
        buttonsContainer.backgroundSkin = new Quad(1,1,0xffffff);
        buttonsContainer.layoutData = layData;
        buttonsContainer.layout = layout;
        _topContainer.addChild(buttonsContainer);

        _screenTitle = new Label();
        _screenTitle.textRendererFactory = GalleryLabelFactory.screenTitle;
        _screenTitle.text = "FRIENDS";
        _screenTitle.validate();
        _screenTitle.paddingBottom = 3;
        addChild(_screenTitle);

        _group = new ToggleGroup();
        _group.addEventListener(Event.CHANGE, buttonSelectedHandler);
        _group.isSelectionRequired = true;

        _friendsButton = ButtonFactory.friendsRadioButton(120,21, 0x184161, 0xFFFFFC, "timeline_friends_activities_icon");
        buttonsContainer.addChild(_friendsButton);
        _friendsButton.toggleGroup = _group;
        _friendsButton.isEnabled = true;

        _requestsButton = ButtonFactory.friendsRadioButton(120,21, 0x184161, 0xFFFFFC, "add_friend_icon");
        buttonsContainer.addChild(_requestsButton);
        _requestsButton.toggleGroup = _group;
        _requestsButton.isEnabled = true;

        /**
         * Close Button
         */
        layData = new AnchorLayoutData();
        layData.top = (BannerStackedScreen.BANNER_SIZE - closeButton.height) / 2;
        layData.right = _gap;

        _topContainer.addChild(closeButton);
        closeButton.layoutData = layData;

        /**
         * SearchView
         */


        _searchInput = new SearchInput();
        _searchInput.addEventListener(Event.CHANGE, onSearchTextChange);
        _topContainer.addChild(_searchInput);
        _searchInput.validate();
        layData = new AnchorLayoutData();
        layData.top = BannerStackedScreen.BANNER_SIZE + (BannerStackedScreen.TOOLBAR_SIZE - _searchInput.height) / 2;
        layData.right = _gap;
        _searchInput.layoutData = layData;

        _messageLabel = new MultilingualLabel();
        _messageLabel.includeInLayout = false;
        _messageLabel.visible = false;
        _messageLabel.height = 14;
        _messageLabel.textRendererFactory = LabelFactory.notificationsMessageField;
        _messageLabel.text = "Your Friends list is empty at this moment";
        _messageLabel.width = actualWidth;
        addChild(_messageLabel);

        initializeFriendsList();

        this._state = STATE_FRIENDS;

        updateTitle();
    }

    override protected function closeButtonTriggeredHandler(event:Event):void{
        super .closeButtonTriggeredHandler(event);
        SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        Starling.juggler.delayCall(function():void{
            if(_showBannerAfterExit){
                if(_bannerScreen == BannerScreen.BANNER_SCREEN){
                    BannerStackedScreen.showAdBanner();
                }else if(_bannerScreen == BannerScreen.HOME_SCREEN){
                    AdsManager.$.setBannerPosition(_bannerY, BannerScreen.HOME_SCREEN);
                    AdsManager.$.bannerShow(BannerScreen.HOME_SCREEN);
                }
            }
        }, 0.5);
    }

    private function buttonSelectedHandler(event:Event):void {
        if(!_friendsList || !_friendsListCollection || !_requestedCollection){
            return;
        }

        SoundManager.playSound(SoundManager.SELECT_PART);
        if(_group.selectedIndex == 0){
            this.changeState(STATE_FRIENDS);
        }

        if(_group.selectedIndex == 1){
            this.changeState(STATE_REQUESTS);
        }
    }

    override public function transitionComplete():void{
        super .transitionComplete();
        BannerStackedScreen.showAdBanner();
    }

    private function initializeFriendsList() : void {
        var layout : VerticalLayout = new VerticalLayout();
        layout.verticalAlign = VerticalAlign.TOP;
        layout.horizontalAlign = HorizontalAlign.CENTER;
        layout.useVirtualLayout = false;
        layout.gap = 0;
        layout.paddingLeft = 0;

        _friendsListCollection = new ListCollection([]);
        _requestedCollection = new ListCollection([]);
        _searchResultCollection = new ListCollection([]);
        _friendsList = new List();
        _friendsList.dataProvider = _friendsListCollection;
        _friendsList.layout = layout;
        _friendsList.width = actualWidth;
        _friendsList.height = actualHeight;
        _friendsList.backgroundSkin = new Quad(1,1,0xfeffff);

        _friendsList.verticalScrollBarFactory = verticalScrollBarFactory;
        _friendsList.scrollBarDisplayMode = ScrollBarDisplayMode.FLOAT;
        _friendsList.horizontalScrollPolicy = ScrollPolicy.OFF;
        _friendsList.verticalScrollPolicy = ScrollPolicy.ON;
        _friendsList.addEventListener(Event.SCROLL, friendsListScrollHandler);
        _friendsList.itemRendererFactory = friendItemRenderer;

        addChildAt(_friendsList,1);
    }

    private function hideAlert() : void{
        if(_alertView && _alertView.parent)
        {
            _alertView.hide();
        }

        _alertView = null;
    }

    public function close() : void{
        hideAlert();
        dispatchEventWith(CLOSE_BUTTON_SELECTED);
    }

    private function friendItemRenderer() : IListItemRenderer {
        var imageWidth:Number  = actualWidth - 2*_gap;
        var imageHeight:Number = 50;

        var renderer : FriendItemRenderer = new FriendItemRenderer();
        renderer.setSize(imageWidth, imageHeight);
        renderer.eventDispatcher = this;
        return renderer;
    }

    private function friendsListScrollHandler(event:Event=null) : void {
        if (_friendsList.verticalScrollPosition == _friendsList.maxVerticalScrollPosition && _friendsList.dataProvider.length > 9){

            switch (this._state){
                case STATE_FRIENDS:
                    showAlert();
                    this.dispatchEventWith(LOAD_MORE_FRIENDS);
                    break;
                case STATE_REQUESTS:
                    showAlert();
                    this.dispatchEventWith(LOAD_MORE_PENDINGS);
                    break;
                case STATE_SEARCH_IDLE:
                    showAlert();
                    this.dispatchEventWith(SEARCH_FRIENDS, false, {
                        skip:_searchResultCollection.length,
                        text:_searchInput.text
                    });
                    break;
            }
        }
    }

    public function showAlert():void{
        if(!_alertView){
            _alertView = AlertView.show('loading...',8);
        }
    }

    public function setSearchResult(value : Array) : void{
        hideAlert();
        _searchResultCollection = new ListCollection(value);
        _friendsList.dataProvider = _searchResultCollection;

        this.changeState(STATE_SEARCH_IDLE);

        _messageLabel.text = "No search results found";
        _messageLabel.visible = (_friendsList.dataProvider.length == 0);
    }

    public function set pendings(value : Array) : void{
        var filteredValue:Array = [];
        for(var i:int = 0; i < value.length; i++){
            _currentFriend = value[i] as Friend;
            if(!_requestedCollection.data.some(containsUser)){
                filteredValue.push(_currentFriend);
            }
        }
        _requestedCollection.addAll(new ListCollection(filteredValue));
        if(_state == STATE_REQUESTS){
            _messageLabel.text = "You don't have friend requests at this moment";
            _messageLabel.visible = (value.length == 0);
        }
    }

    private function containsUser(element:*, index:int, arr:Array):Boolean{
        var friend:Friend = element as Friend;
        return (friend.user.id == _currentFriend.user.id);
    }

    public function appendSearchResult(value : Array) : void{
        hideAlert();

        if(_state == STATE_SEARCH_IDLE){
            _searchResultCollection.addAll(new ListCollection(value));
            this.changeState(STATE_SEARCH_IDLE);
        }
    }


    private function changeState(newState:String) : void{
        _messageLabel.visible = false;
        switch (newState){

            case STATE_REQUESTS:
                    _friendsList.dataProvider = _requestedCollection;
                    dispatchEventWith(PENDING_FRIENDS_READ, false, {friendList:_requestedCollection.data});
                    unreadRequests = false;

                _messageLabel.text = "You don't have friend requests at this moment";
                _messageLabel.visible = (_friendsList.dataProvider.length == 0);
                break;

            case STATE_FRIENDS:
                _friendsList.dataProvider = _friendsListCollection;
                _messageLabel.text = "Your Friends list is empty at this moment";
                _messageLabel.visible = (_friendsList.dataProvider.length == 0);
                break;
            case STATE_SEARCH_TYPING:
                Starling.juggler.delayCall(onSearchTextChange, SEARCH_IDLE_TIME);
                    _friendsList.dataProvider = _searchResultCollection;

                break;
            case STATE_SEARCH_WAITING_RESPONSE:
                showAlert();
                this.dispatchEventWith(SEARCH_FRIENDS, false, {
                    skip:0,
                    text:_searchInput.text
                });

                break;
            case STATE_SEARCH_IDLE:
                if(_searchInput.text.length == 0){
                    if(_group.selectedIndex == 0){
                        this.changeState(STATE_FRIENDS);
                    }else if(_group.selectedIndex == 1){
                        this.changeState(STATE_REQUESTS);
                    }
                }
                if(_prevSearchText != _searchInput.text){
                    this.changeState(STATE_SEARCH_TYPING);
                }
                break;

        }

        _state = newState;

    }


    protected function onSearchTextChange(event:Event=null):void {
        switch (this._state){
            case STATE_FRIENDS:
            case STATE_REQUESTS:
                  if(_searchInput.text.length > 0){
                      this.changeState(STATE_SEARCH_TYPING);
                  }

                break;
            case STATE_SEARCH_TYPING:
                    if(_searchInput.text.length == 0){
                        if(_group.selectedIndex == 0){
                            this.changeState(STATE_FRIENDS);
                        }else if(_group.selectedIndex == 1){
                            this.changeState(STATE_REQUESTS);
                        }
                    }
                    else if(_prevSearchText == _searchInput.text){
                        this.changeState(STATE_SEARCH_WAITING_RESPONSE);
                    }
                    else if(event == null) {
                        Starling.juggler.delayCall(onSearchTextChange, SEARCH_IDLE_TIME);
                    }
                break;
            case STATE_SEARCH_WAITING_RESPONSE:

                break;
            case STATE_SEARCH_IDLE:
                if(_searchInput.text.length == 0){
                    if(_group.selectedIndex == 0){
                        this.changeState(STATE_FRIENDS);
                    }else if(_group.selectedIndex == 1){
                        this.changeState(STATE_REQUESTS);
                    }
                }
                else if(_prevSearchText != _searchInput.text){
                    this.changeState(STATE_SEARCH_TYPING);
                }

                break;
        }

        _prevSearchText = _searchInput.text;
    }

    public function get friends() : Array{
        return _friendsListCollection.data as Array;
    }

    public function set total(value : int) : void{
        this._totalFriends = value;
        updateTitle();
    }

    public function get total() : int{
        return this._totalFriends;
    }

    public function removePending(friend:Friend):void{
        if(_requestedCollection.getItemIndex(friend) >= 0){
            _requestedCollection.removeItem(friend);
        }
    }

    public function removeFriend(friend:Friend):void{
        if(_friendsListCollection.getItemIndex(friend) >= 0){
            _friendsListCollection.removeItem(friend);
            _totalFriends --;
            updateTitle();
        }
    }

    public function addFriend(friend:Friend):void{
        if(_friendsListCollection.getItemIndex(friend) < 0){
            _friendsListCollection.addItemAt(friend, 0);
            _totalFriends ++;
            updateTitle();
        }
    }

    public function set friends(value:Array) : void {
        hideAlert();
        _friendsListCollection.addAll(new ListCollection(value));
        _friendsList.invalidate(INVALIDATION_FLAG_ALL);
        updateTitle();

        _messageLabel.text = "Your Friends list is empty at this moment";
        _messageLabel.visible = (_friendsList.dataProvider.length == 0);
        //
        if(_onComplete != null){
            _onComplete();
            _onComplete = null;
        }
    }


    private function updateTitle() : void{
        this._friendsButton.label = "Friends (" +  _totalFriends +")";
        this._requestsButton.label = "Requested";
    }


    override protected function draw() : void {
        super.draw();

        _topContainer.width = actualWidth;
        _topContainer.x = 0;
        _topContainer.y = 0;
        _topContainer.validate();

        _friendsList.x = 0;
        _friendsList.y = BannerStackedScreen.BANNER_SIZE + BannerStackedScreen.TOOLBAR_SIZE;
        _friendsList.width = actualWidth;
        _friendsList.height = actualHeight - (BannerStackedScreen.BANNER_SIZE + BannerStackedScreen.TOOLBAR_SIZE);

        if(_messageLabel){
            _messageLabel.x = (actualWidth - _messageLabel.width) / 2;
            _messageLabel.y = (actualHeight - _messageLabel.height) / 2;
        }

        if(_screenTitle){
            _screenTitle.x = _gap;
            _screenTitle.y = (BannerStackedScreen.BANNER_SIZE - _screenTitle.height) / 2;
        }
    }

    private function verticalScrollBarFactory() : ScrollBar {
        var scrollbar:ScrollBar = new ScrollBar();
        scrollbar.direction = Direction.VERTICAL;//ScrollBar.DIRECTION_VERTICAL;
        scrollbar.thumbFactory = function() : Button {
            var button : Button = new Button();
            var sprite : Sprite = new Sprite();
            var back : Quad = new Quad(8, 25, 0xffffff);
            back.alpha = 0;
            var quad : Quad = new Quad(4, 25, 0xC1C3C2);
            quad.x = 12;
            sprite.addChild(back);
            sprite.addChild(quad);
            button.defaultSkin = sprite;

            return button;
        };
        return scrollbar;
    }

    override public function dispose():void {
        super.dispose();
        if(pendingData){
            pendingData.length = 0;
        }
        if(sentData){
            sentData.length = 0;
        }

        if(_group){
            _group.removeAllItems();
            _group = null;
        }
        if(_friendsListCollection){
            _friendsListCollection.removeAll();
            _friendsListCollection = null;
        }
        if(_requestedCollection){
            _requestedCollection.removeAll();
            _requestedCollection = null;
        }

        if(_searchResultCollection){
            _searchResultCollection.removeAll();
            _searchResultCollection = null;
        }

        if(_screenTitle){
            _screenTitle.removeFromParent(true);
            _screenTitle = null;
        }

        if(_messageLabel){
            _messageLabel.removeFromParent(true);
            _messageLabel = null;
        }

        if(_alertView){
            _alertView.removeFromParent(true);
            _alertView = null;
        }

        if(_topContainer){
            _topContainer.removeFromParent(true);
            _topContainer = null;
        }

        if(_pendingFriendsUnreadIcon){
            _pendingFriendsUnreadIcon.removeFromParent(true);
            _pendingFriendsUnreadIcon = null;
        }

        if(_searchInput){
            _searchInput.removeFromParent(true);
            _searchInput = null;
        }

        if(_friendsButton){
            _friendsButton.removeFromParent(true);
            _friendsButton = null;
        }
        if(_requestsButton){
            _requestsButton.removeFromParent(true);
            _requestsButton = null;
        }

        if (_friendsList) {
            _friendsList.removeFromParent(true);
            _friendsList = null;
        }
    }

    public function get unreadRequests():Boolean {
        return _unreadRequests;
    }

    public function set unreadRequests(value:Boolean):void {
        _unreadRequests = value;
        if(_requestsButton) {
            if (value) {
                _pendingFriendsUnreadIcon ||= new Image(ButtonFactory.assetManager.getTexture("notification"));
                _pendingFriendsUnreadIcon.width = _pendingFriendsUnreadIcon.height = 5;
                _pendingFriendsUnreadIcon.x = _requestsButton.width-(_pendingFriendsUnreadIcon.width+5);
                _pendingFriendsUnreadIcon.y = (_requestsButton.height-_pendingFriendsUnreadIcon.height)/2;
                _requestsButton.addChild(_pendingFriendsUnreadIcon);
            } else if(_pendingFriendsUnreadIcon){
                _pendingFriendsUnreadIcon.removeFromParent();
            }
        }
    }

    public function set onComplete(onComplete:Function):void {
        _onComplete = onComplete;
    }
}
}