package com.despark.tuning3d.view.components.gallery {
    import com.despark.tuning3d.model.core.UIColors;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.greensock.TweenLite;

import feathers.controls.Label;

import feathers.controls.TextInput;

import feathers.core.FeathersControl;
import feathers.events.FeathersEventType;

import flash.geom.Rectangle;

import starling.core.Starling;
    import starling.display.DisplayObjectContainer;

    import starling.display.Image;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
    import starling.utils.ScaleMode;

    public class SearchInput extends FeathersControl {
    private var searchInput:TextInput;
    private var searchInputPlaceholder:Label;
    private var searchBackground:Image;
    private var tween:TweenLite;
    private var _isOpen:Boolean;
    private static const MAX_WIDTH:int = 150;
    private static var SEARCH_LABEL:String = 'search';
    private var _isFixed:Boolean;
    private var _showOnFilled:Boolean;
    private var _expandedWidth:Number = 0;
    private var _size:Number;
    private var _iconTxt:String;
    private var _iconScaleMode:String;
    private var _iconColor:uint;
    public function SearchInput(isFixed:Boolean = false, size:Number = 0, iconTxt:String = null, iconScaleMode:String = null, iconColor:int = -1, showOnFilled:Boolean = false) {
        super();
        _isFixed = isFixed;
        _size = size;
        _iconTxt = iconTxt||"src";
        _iconScaleMode = iconScaleMode||ScaleMode.SHOW_ALL;
        _iconColor = (iconColor==-1?UIColors.UI_COMPONENT_GRAY: iconColor);
        _showOnFilled = showOnFilled;
    }


    override protected function initialize():void {
        super.initialize();
        _size ||= 22;
        width = height = _size;
        _isOpen = false;

        searchBackground = new Image(ButtonFactory.assetManager.getTexture("searchBg"));
        searchBackground.color = UIColors.UI_COMPONENT_GRAY;
        searchBackground.scale = ButtonFactory.assetManager.scaleFactor / 2;
        searchBackground.scale9Grid = new Rectangle(searchBackground.texture.frameWidth / 12, 0, 10 * searchBackground.texture.frameWidth / 12, searchBackground.texture.frameHeight);

        addChild(searchBackground);
        searchBackground.width = _size;
        searchBackground.height = _size;

        searchInput = new TextInput();
        searchInput.textEditorFactory = GalleryLabelFactory.searchFieldLabel;


        var searchIcon:Image = new Image(ButtonFactory.assetManager.getTexture(_iconTxt));
        if(_iconScaleMode == ScaleMode.SHOW_ALL){
            searchIcon.height = searchBackground.height - 6;
            searchIcon.scale = searchIcon.scaleY;
        }else{
            searchIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        }

        searchIcon.x = (searchBackground.width - searchIcon.width) / 2;
        searchIcon.y = (searchBackground.height - searchIcon.height) / 2;
        searchIcon.color = _iconColor;

        addChild(searchIcon);
        searchInput.width = expandedWidth - 25;
        searchInput.height = searchIcon.height;
        searchInput.x = 5;
        searchInput.y = searchBackground.y + (searchBackground.height - searchInput.height)/2;
        searchInput.addEventListener(FeathersEventType.FOCUS_IN, searchFieldFocusHandler);
        searchInput.addEventListener(FeathersEventType.FOCUS_OUT, searchFieldFocusHandler);
        searchInput.addEventListener(Event.CHANGE, searchInputChangeHandler);
        searchInput.visible = false;
        addChild(searchInput);

        searchInputPlaceholder = new Label();
        searchInputPlaceholder.textRendererFactory = GalleryLabelFactory.searchPlaceholderDefaultLabel;
        searchInputPlaceholder.text = SEARCH_LABEL;
        searchInputPlaceholder.height = searchInput.height;
        searchInputPlaceholder.validate();
        addChild(searchInputPlaceholder);
        searchInputPlaceholder.visible = false;
        searchInputPlaceholder.touchable = searchInputPlaceholder.isEnabled = false;

        if(_isFixed){
            open();
        }
    }

    override protected function feathersControl_addedToStageHandler(event:Event):void {
        super.feathersControl_addedToStageHandler(event);
        if(!_isFixed){
            stage.addEventListener(TouchEvent.TOUCH, touchHandler);
        }
    }

    override protected function feathersControl_removedFromStageHandler(event:Event):void {
        super.feathersControl_addedToStageHandler(event);
        stage.removeEventListener(TouchEvent.TOUCH, touchHandler);
    }

    private function touchHandler(event:TouchEvent):void{
        var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
        var touchEnd:Touch = event.getTouch(stage, TouchPhase.ENDED);
        if(touch && !_isOpen){
            open();
        }else if(!touch && _isOpen && touchEnd && (!_showOnFilled || (_showOnFilled && !text.length))){
            close();
        }
    }

    private function enableUserInputs(value:Boolean, object:DisplayObjectContainer):void{
        if(object && object != this){
            for(var i:int = 0; i < object.numChildren; i++){
                if(object.getChildAt(i) is DisplayObjectContainer && DisplayObjectContainer(object.getChildAt(i)).contains(this)){
                    enableUserInputs(value, DisplayObjectContainer(object.getChildAt(i)));
                }else{
                    object.getChildAt(i).touchable = value;
                }
            }
        }
    }

    public function close():void{
        if(!tween){
            //SoundManager.playSound(SoundManager.SELECT_PART_BACK);
            searchInput.visible = false;
            searchInputPlaceholder.visible = false;
            tween = TweenLite.to(searchBackground, 0.25, {width:_size, onUpdate:function():void{
                    searchBackground.x = (_size - searchBackground.width);
                }, onComplete:function():void{
                    tween = null;
                    _isOpen = false;
                    enableUserInputs(true, stage);
                }});
        }
    }

    public function open():void{
        if(_isFixed){
            searchBackground.width = expandedWidth;
            searchBackground.x = (_size - searchBackground.width);
            tween = null;
            _isOpen = true;
            searchInput.x = _size - searchBackground.width + 5;
            searchInput.visible = true;
            searchInputPlaceholder.x = _size - searchBackground.width + 10;
            searchInputPlaceholder.y = 2;
            if(text == "" || !text){
                searchInputPlaceholder.visible = true;
            }
        }else if(!tween){
            if(isInitialized){
                //SoundManager.playSound(SoundManager.SELECT_PART);
            }
            tween = TweenLite.to(searchBackground, 0.25, {width: expandedWidth, onUpdate:function():void{
                    searchBackground.x = (_size - searchBackground.width);
                }, onComplete:function():void{
                    tween = null;
                    _isOpen = true;
                    searchInput.x = _size - searchBackground.width + 5;
                    searchInput.visible = true;
                    if(!_isFixed){
                        enableUserInputs(text.length, stage);
                        searchInput.setFocus();
                    }
                    searchInputPlaceholder.x = _size - searchBackground.width + 10;
                    searchInputPlaceholder.y = 2;
                    if(text == "" || !text){
                        searchInputPlaceholder.visible = true;
                    }
                }});
        }
    }

    private function searchFieldFocusHandler(event:Event):void {
        if(event.type == FeathersEventType.FOCUS_IN && searchInput.text == ''){
            SoundManager.playSound(SoundManager.SELECT_PART);
            searchInputPlaceholder.visible = false;
        }else if(event.type == FeathersEventType.FOCUS_OUT && searchInput.text == ''){
            if(isInitialized){
                SoundManager.playSound(SoundManager.SELECT_PART_BACK);
            }
            searchInputPlaceholder.visible = true;
        }
    }

    private function searchInputChangeHandler(event:Event):void {
        searchInputPlaceholder.visible = (searchInput.text.length == 0);
        dispatchEvent(new Event(event.type, event.bubbles, event.data));
        enableUserInputs(text.length, stage);
    }

    public function set text(value:String):void{
        searchInput.text = value;
    }

    public function get text():String{
        return searchInput.text;
    }

    public function set expandedWidth(value:Number):void {
        _expandedWidth = value;
    }

    public function get expandedWidth():Number {
        return _expandedWidth>0?_expandedWidth:MAX_WIDTH;
    }

    override public function dispose():void{
        super .dispose();
        if(stage && stage.hasEventListener(TouchEvent.TOUCH, touchHandler)){
            stage.removeEventListener(TouchEvent.TOUCH, touchHandler);
        }

        if(searchInput){
            searchInput.removeFromParent(true);
            searchInput = null;
        }
        if(searchInputPlaceholder){
            searchInputPlaceholder.removeFromParent(true);
            searchInputPlaceholder = null;
        }
        if(searchBackground){
            searchBackground.removeFromParent(true);
            searchBackground = null;
        }
        if(tween){
            tween.kill();
            tween = null;
        }
    }
}
}
