package com.despark.tuning3d.view.components.levels {
    import com.despark.tuning3d.model.core.UIStrings;
    import com.despark.tuning3d.model.levels.LevelData;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.utils.FontSelector;

    import feathers.controls.Label;

    import flash.text.TextFormatAlign;

    import starling.display.Quad;

    public class LevelDataItemRenderer extends ListItemRenderer {
        public var _levelLabel:Label;
        public var levelIcon:Label;
        public var _levelDesc:Label;
        public var _border:Quad;

        private var _levelIndex:int = -1;
        private var _imageList:Vector.<LevelItemImage>;
        private var _plusList:Vector.<Label>;
        public function LevelDataItemRenderer() {
            super();
        }

        override protected function initialize():void{
            super.initialize();
            _border = new Quad(1,1, 0x535353);
            _border.width = explicitWidth;
            _border.height = 0.5;
            _border.x = 20;
            _border.y = 0;
            addChild(_border);

            _levelLabel = new Label();
            _levelLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 13, 0xFFFFFF, TextFormatAlign.LEFT, false);
            _levelLabel.backgroundSkin = new Quad(1,1, 0x0da506);
            _levelLabel.paddingLeft = _levelLabel.paddingRight = 5;
            _levelLabel.paddingTop = _levelLabel.paddingBottom = -1;
            _levelLabel.y = 8;
            addChild(_levelLabel);

            levelIcon = LabelFactory.createSimpleLabel(FontSelector.MontserratHtml, 8, 0xFFFFFF, TextFormatAlign.LEFT, false, true);
            addChild(levelIcon);

            _levelDesc = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 13, 0xFFFFFF, TextFormatAlign.LEFT, false);
            _levelDesc.paddingLeft = _levelDesc.paddingRight = 1;
            _levelDesc.paddingTop = _levelDesc.paddingBottom = -1;
            addChild(_levelDesc);
            _levelDesc.y = _levelLabel.y;

            if(imageLoader){
                imageLoader.removeFromParent(true);
                imageLoader = null;
            }
        }

        override protected function commitData():void{
            super .commitData();
            if(levelData){
                //
                if(_levelIndex == levelData.level){
                    return;
                }
                _levelIndex = levelData.level;
                //
                var levelUnlockedList:Array = [];
                var i:int;
                var count:int = 0;
                if(levelData.unlockedData.materials && levelData.unlockedData.materials.length){
                    count += levelData.unlockedData.materials.length;
                    for(i = 0; i < levelData.unlockedData.materials.length; i++){
                        levelUnlockedList.push({name:"material", "value":levelData.unlockedData.materials[i]});
                    }
                }
                if(levelData.unlockedData.components && levelData.unlockedData.components.length){
                    count += levelData.unlockedData.components.length;
                    for(i = 0; i < levelData.unlockedData.components.length; i++){
                        levelUnlockedList.push(levelData.unlockedData.components[i]);
                    }
                }
                if(levelData.unlockedData.limit){
                    count++;
                    levelUnlockedList.push({name:"limit", "value": levelData.unlockedData.limit});
                }
                //
                if(_levelLabel){
                    if(levelData.level < LevelData.currentUserLevelData.level){
                        Quad(_levelLabel.backgroundSkin).color = 0x0da506;
                        levelIcon.text = UIStrings.getUnlockIconChar(8);
                    }else if(levelData.level == LevelData.currentUserLevelData.level){
                        Quad(_levelLabel.backgroundSkin).color = 0x0da506;
                        levelIcon.text = UIStrings.getCheckIconChar(8);
                    }else{
                        Quad(_levelLabel.backgroundSkin).color = 0xf8a007;
                        levelIcon.text = UIStrings.getLockIconChar(8);
                    }

                    _levelLabel.text = "LEVEL " + levelData.level.toString();
                    levelIcon.validate();
                    if(levelIcon){
                        _levelLabel.paddingLeft = 18;
                        _levelLabel.paddingRight = 5;
                    }
                }

                if(_levelLabel){
                    _levelLabel.validate();
                    if(levelData.level == LevelData.currentUserLevelData.level){
                        _levelDesc.text = "/ YOUR CURRENT LEVEL";
                    }else{
                        _levelDesc.text = "/ " + levelData.minChallengeCount.toString() + " Challenges required";
                    }
                    _levelDesc.x = _levelLabel.x + _levelLabel.width + 5;
                }
                //
                if(_imageList){
                    while (_imageList.length){
                        var img:LevelItemImage = _imageList.pop();
                        if(img){
                            img.removeFromParent(true);
                        }
                    }
                }else{
                    _imageList = new Vector.<LevelItemImage>();
                }//
                if(_plusList){
                    while (_plusList.length){
                        var lbl:Label = _plusList.pop();
                        if(lbl){
                            lbl.removeFromParent(true);
                        }
                    }
                }else{
                    _plusList = new Vector.<Label>();
                }
                for(i = 0; i < levelUnlockedList.length; i++){
                    var levelImage:LevelItemImage = new LevelItemImage(LevelData.currentUserLevelData, LevelData.levelDataList[_levelIndex], levelUnlockedList[i]);
                    addChild(levelImage);
                    levelImage.x = actualWidth / 2 - 60 * (levelUnlockedList.length - 1)/ 2 + i * 60 - 14;
                    levelImage.y = 32;
                    _imageList.push(levelImage);
                    //
                    if(i < levelUnlockedList.length - 1){
                        var plusLabel:Label = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 11, levelImage.borderColor, TextFormatAlign.LEFT, false, false);
                        addChild(plusLabel);
                        plusLabel.text = "+";
                        plusLabel.validate();
                        plusLabel.x = levelImage.x + levelImage.borderWidth + (60 - levelImage.borderWidth - plusLabel.width) / 2;
                        plusLabel.y = levelImage.y + (levelImage.borderHeight - plusLabel.height) / 2;
                        plusLabel.alpha = levelImage.alpha;
                        _plusList.push(plusLabel);
                    }
                }
            }
        }

        override protected function draw():void {
            super.draw();
            var dataInvalid:Boolean = isInvalid(INVALIDATION_FLAG_DATA);
            if(dataInvalid) {
                commitData();
                //
                if(levelIcon){
                    levelIcon.x = 5;
                    levelIcon.y = _levelLabel.y + (_levelLabel.height - levelIcon.height) / 2;
                }
            }
            autoSizeIfNeeded();
        }

        override protected function autoSizeIfNeeded():Boolean {
            var needsWidth:Boolean = isNaN(explicitWidth);
            var needsHeight:Boolean = isNaN(explicitHeight);
            if(!needsWidth || !needsHeight){
                return false;
            }
            var newWidth:Number = explicitWidth;
            var newHeight:Number = explicitHeight;
            if(isNaN(newWidth) || isNaN(newWidth)){
                return saveMeasurements(0, 0, 0, 0);
            }
            return saveMeasurements(newWidth, newHeight, newWidth, newHeight);
        }

        public function get levelData():LevelData{
            return data as LevelData;
        }

        override public function dispose():void{
            super.dispose();
            if(_levelLabel){
                _levelLabel.removeFromParent(true);
                _levelLabel = null;
            }
            if(_levelDesc){
                _levelDesc.removeFromParent(true);
                _levelDesc = null;
            }
            if(_border){
                _border.removeFromParent(true);
                _border = null;
            }
            if(_imageList){
                while (_imageList.length){
                    var levelImage:LevelItemImage = _imageList.pop();
                    if(levelImage){
                        levelImage.removeFromParent(true);
                    }
                }
                _imageList = null;
            }
        }
    }
}
