package com.despark.tuning3d.view.components.levels {
    import com.despark.tuning3d.model.core.UIStrings;
    import com.despark.tuning3d.model.levels.LevelData;
    import com.despark.tuning3d.model.parts.PartMaterial;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.utils.FontSelector;
    import com.utils.TextureUtil;

    import feathers.controls.Label;

    import feathers.core.FeathersControl;

    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import starling.display.Image;
    import starling.textures.Texture;

    public class LevelItemImage extends FeathersControl {
        private var _iconImage:Image;
        private var _border:Image;
        private var _levelUpData:Object;
        private var _levelData:LevelData;
        private var _check:Label;
        private var _label:Label;
        private var _userLevelData:LevelData;
        private var _borderColor:uint;
        public function LevelItemImage(userLevelData:LevelData, levelData:LevelData, levelUpData:Object) {
            super();
            _userLevelData = userLevelData;
            _levelUpData = levelUpData;
            _levelData = levelData;
        }

        override protected function initialize():void{
            super .initialize();
            if(_levelUpData.name == "material"){
                _iconImage = new Image(getIconTexture(_levelUpData.value));
            }else if(_levelUpData.name == "limit"){
                _iconImage = new Image(getIconTexture("level_key_limit"));
            }else if(_levelUpData.name == "aerography_packs"){
                _iconImage = new Image(getIconTexture("racing_stripe"));
            }else if(_levelUpData.name == "vinyl_decals"){
                _iconImage = new Image(getIconTexture("vinyl_big"));
            }
            _iconImage.color = 0xFF0000;
            if(_levelData.level == _userLevelData.level){
                _borderColor = 0x0da506;
            }else{
                _borderColor = 0xFEFFFF;
            }

            _border = new Image(ButtonFactory.assetManager.getTexture("reward_list_item_border"));
            _border.texture = TextureUtil.scaleTexture(_border.texture, 0.5);
            _border.scale = ButtonFactory.assetManager.scaleFactor / 2;
            _border.scale9Grid = new Rectangle(_border.texture.frameWidth / 2 - 0.5,_border.texture.frameHeight / 2 - 0.5, 1, 1);
            _border.width = 45;
            _border.height = 50;
            _border.color = _borderColor;
            addChild(_border);

            var checkSize:int = 6;
            var yDiff:int = 1;
            _check = LabelFactory.createSimpleLabel(FontSelector.MontserratHtml, checkSize, _borderColor, TextFormatAlign.LEFT, false, true);
            addChild(_check);
            if(_levelData.level > _userLevelData.level){
                _check.text = UIStrings.getLockIconChar(checkSize + 1, _borderColor.toString(16));
                yDiff = -1;
                alpha = 0.35;
            }else{
                _check.text = UIStrings.getCheckIconChar(checkSize, _borderColor.toString(16));
            }
            _check.validate();
            _check.x = _border.x + _border.width - _check.width;
            _check.y = _border.y + yDiff;

            _label = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 10, 0xFFFFFF, TextFormatAlign.CENTER, false, false);
            var iconSize:Number;
            if(_levelUpData.name == "material"){
                _label.text = PartMaterial.getLabel(_levelUpData.value) + " Finish";
                iconSize = 40;
            }else if(_levelUpData.name == "limit"){
                _label.text = "Key Limit " + _levelData.maxKeys.toString();
                iconSize = 40;
            }else if(_levelUpData.name == "aerography_packs" && _levelData.components.hasOwnProperty("aerography_packs")){
                _label.text = "Racing Stripe #" + _levelData.components["aerography_packs"].toString();
                iconSize = 40;
            }else if(_levelUpData.name == "vinyl_decals" && _levelData.components.hasOwnProperty("vinyl_decals")){
                _label.text = "Vinyl #" + _levelData.components["vinyl_decals"].toString();
                iconSize = 40;
            }
            addChild(_label);
            _label.validate();
            _label.x = (_border.width - _label.width) / 2;
            _label.y = _border.y + _border.height + 3;

            if(_iconImage){
                _iconImage.pivotX = _iconImage.width / 2;
                _iconImage.pivotY = _iconImage.height / 2;
                _iconImage.width = iconSize;
                _iconImage.scale = _iconImage.scaleX;
                _iconImage.x = _border.width / 2;
                _iconImage.y = _border.height / 2;
                addChild(_iconImage);
            }
        }

        public function get borderWidth():Number{
            if(_border){
                return _border.width;
            }
            return 0;
        }

        public function get borderHeight():Number{
            if(_border){
                return _border.height;
            }
            return 0;
        }

        private static function getIconTexture(name:String):Texture{
            return ButtonFactory.assetManager.getTextureAtlas("level_icons").getTexture(name + ".png");
        }

        override public function dispose():void{
            super.dispose();

            if(_iconImage){
                _iconImage.removeFromParent(true);
                _iconImage = null;
            }

            if(_border){
                _border.removeFromParent(true);
                _border = null;
            }

            if(_check){
                _check.removeFromParent(true);
                _check = null;
            }

            if(_label){
                _label.removeFromParent(true);
                _label = null;
            }

            _levelUpData = null;
            _levelData = null;
            _userLevelData = null;
        }

        public function get borderColor():uint {
            return _borderColor;
        }
    }
}
