package com.despark.tuning3d.view.components.levels {
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.UIStrings;
    import com.despark.tuning3d.model.levels.LevelData;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.challenges.controls.UserLevelAvatar;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedBannerScreen;
    import com.distriqt.AdsManager;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.List;
    import feathers.controls.ScrollBar;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.data.ListCollection;
    import feathers.layout.Direction;
    import feathers.layout.VerticalLayout;

    import flash.text.TextFormatAlign;

    import starling.animation.Tween;
    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.events.Event;

    public class UserLevelInfoPopup extends PopUp {
        private var _mainContainer:LayoutGroup;
        private var _levelInfoList:List;
        private var _levelInfoListCover:Quad;
        private var _userAvatar:UserLevelAvatar;
        private var _currentUser:User;
        private var _levelRewardsLabel:Label;
        private var _challengesLabel:Label;
        private static const ITEM_HEIGHT:Number = 105;
        public function UserLevelInfoPopup(currentUser:User) {
            super();
            _currentUser = currentUser;
        }

        override protected function initialize():void{
            super.initialize();
            setSizeInternal(stage.stageWidth, stage.stageHeight, false);
            background.color = 0x000000;
            background.alpha = 0.65;

            _mainContainer = new LayoutGroup();
            _mainContainer.backgroundSkin = new Quad(1,1,0x111111);
            _mainContainer.setSize(216, actualHeight);
            _mainContainer.x = actualWidth - _mainContainer.width;
            addChild(_mainContainer);

            if(contentBackgroundImage){
                contentBackgroundImage.removeFromParent(true);
                contentBackgroundImage = null;
            }

            var closeIcon:Image;
            closeIcon = new Image(ButtonFactory.assetManager.getTexture('details_close'));
            var closeDownIcon:Image = new Image(closeIcon.texture);
            closeIcon.color = 0xFEFFFF;
            closeDownIcon.color = 0xCCCCCC;
            closeIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
            closeDownIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
            closeDownIcon.alpha = 0.6;

            closeButton.width = closeIcon.width;
            closeButton.height = closeIcon.height;
            closeButton.defaultIcon = closeIcon;
            closeButton.downIcon = closeDownIcon;

            _userAvatar = new UserLevelAvatar(35);
            _mainContainer.addChild(_userAvatar);
            _userAvatar.x = 30;
            _userAvatar.y = StackedBannerScreen.marginTop / 2;
            _userAvatar.update(_currentUser);

            _levelRewardsLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 21, 0xFFFFFF, TextFormatAlign.LEFT, false, false);
            _levelRewardsLabel.text = "LEVEL REWARDS";
            _mainContainer.addChild(_levelRewardsLabel);
            _levelRewardsLabel.validate();
            _levelRewardsLabel.x = _userAvatar.x + _userAvatar.size / 2 + 10;
            _levelRewardsLabel.y = _userAvatar.y - _userAvatar.size / 2;

            _challengesLabel = LabelFactory.createSimpleLabel(FontSelector.AlumniSansHtml, 13, 0xFFFFFF, TextFormatAlign.LEFT, false, true);
            _challengesLabel.text = UIStrings.getCarIconChar(13, "#FFFFFF") + '  <b><font color="#f8a007">' +_currentUser.submittedChallenges.length.toString()+ '</font> / ' + _currentUser.levelData.nextMinChallengeCount + '</b>';
            _mainContainer.addChild(_challengesLabel);
            _challengesLabel.validate();
            _challengesLabel.x = _levelRewardsLabel.x;
            _challengesLabel.y = _userAvatar.y + _userAvatar.height / 2 - _challengesLabel.height + 5;


            var listLayout : VerticalLayout = new VerticalLayout();
            listLayout.useVirtualLayout = true;
            listLayout.gap = 0;
            listLayout.paddingLeft = 0;
            listLayout.paddingRight = 0;
            listLayout.paddingBottom = 5;

            listLayout.useVirtualLayout = true;
            listLayout.afterVirtualizedItemCount = 3;
            listLayout.beforeVirtualizedItemCount = 3;

            _levelInfoList = new ExtendedList();
            _levelInfoList.verticalScrollBarFactory = function ():ScrollBar {
                var scrollbar:ScrollBar = new ScrollBar();
                scrollbar.direction = Direction.VERTICAL;
                scrollbar.thumbFactory = function ():Button {
                    var button:Button = new Button();
                    var sprite:Sprite = new Sprite();
                    var back:Quad = new Quad(10, 25, 0xffffff);
                    back.alpha=0;
                    var quad:Quad = new Quad(3, 25, 0x535353);
                    sprite.addChild(back);
                    sprite.addChild(quad);
                    button.defaultSkin = sprite;
                    return button;
                }
                return scrollbar;
            }
            _levelInfoList.padding = 0;
            _levelInfoList.paddingRight = -30;
            _levelInfoList.includeInLayout = false;
            _levelInfoList.layout = listLayout;
            _levelInfoList.verticalScrollPolicy = ScrollPolicy.ON;
            _levelInfoList.horizontalScrollPolicy = ScrollPolicy.OFF;
            _levelInfoList.setSize(_mainContainer.width - 33, actualHeight - (StackedBannerScreen.marginTop + 2));
            _levelInfoList.y = StackedBannerScreen.marginTop + 2;
            _levelInfoList.x = 0;
            _levelInfoList.itemRendererFactory = function(): IListItemRenderer{
                var renderer:LevelDataItemRenderer = new LevelDataItemRenderer();
                renderer.owner = _levelInfoList;
                renderer.width = _levelInfoList.width;
                renderer.height = ITEM_HEIGHT;
                return renderer;
            }
            _mainContainer.addChild(_levelInfoList);
            _levelInfoListCover = new Quad(1,1,0x111111);
            _levelInfoListCover.width = _levelInfoList.width + 15;
            _levelInfoListCover.height = _levelInfoList.height + 4;
            _levelInfoListCover.x = _levelInfoList.x;
            _levelInfoListCover.y = _levelInfoList.y - 2;
            _mainContainer.addChild(_levelInfoListCover);
            addChild(closeButton);


        }

        override protected function closeButtonTriggeredHandler(event : Event):void{
            super .closeButtonTriggeredHandler(event);
            AdsManager.$.bannerShow(BannerScreen.BANNER_SCREEN);
        }

        override protected function draw():void{
            super.draw();
            closeButton.y = _levelRewardsLabel.y;
        }

        private function scrollToCurrentLevel():void{
            for(var i:int = 0; i < _levelInfoList.dataProvider.length; i++){
                var levelData:Object = _levelInfoList.dataProvider.getItemAt(i);
                if(levelData.level == LevelData.currentUserLevelData.level){
                    var index:int = _levelInfoList.dataProvider.getItemIndex(levelData);
                    if(index != -1){
                        var position:Number = ITEM_HEIGHT * index;
                        _levelInfoList.verticalScrollPosition = position;
                    }
                    break;
                }
            }
        }

        public function transitionCompleteHandler():void{
            _levelInfoList.dataProvider = new ListCollection(LevelData.levelDataArray);
            scrollToCurrentLevel();

            var tween:Tween = new Tween(_levelInfoListCover, 0.25);
            tween.fadeTo(0);
            tween.delay = 0.01;
            tween.onComplete = function():void{
                _levelInfoListCover.removeFromParent(true);
                _levelInfoListCover = null;
            }
            Starling.juggler.add(tween);
        }
    }
}
