package com.despark.tuning3d.view.components.mailbox {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.UIColors;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.Multilingual;
    import com.despark.tuning3d.model.mails.MailsProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.IMainMenuHolder;
    import com.despark.tuning3d.view.components.core.MainMenuView;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.gallery.*;
    import com.despark.tuning3d.view.components.mailbox.mails.MailLabelItemRenderer;
    import com.despark.tuning3d.view.components.timeline.components.TimelineLoadBar;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.Tracker;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.List;
    import feathers.controls.ScrollBar;
    import feathers.controls.ScrollBarDisplayMode;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.ToggleButton;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.ToggleGroup;
    import feathers.data.ListCollection;
    import feathers.layout.Direction;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.display.StageOrientation;

    import flash.utils.setTimeout;

    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.events.Event;
    import starling.textures.Texture;
    import starling.utils.MathUtil;

    public class MailsScreen extends StackedScreen implements Multilingual, IMainMenuHolder {
        private var _mainMenuView:MainMenuView;
        private var _selectedLanguage:Language;
        private var _languages:Array;
        private var menuButton:Button;
        private var _background:Quad;
        private var _bannerBackground:Image;
        private var _loadingBar: TimelineLoadBar;
        private var titleLabel:Label;
        private var _mailsProxy:MailsProxy;

        private var viewToggleGroup:ToggleGroup;
        private var viewLayout:LayoutGroup;

        private var backgroundButton:ToggleButton;

        private var searchInput:SearchInput;

        private var _mailsList : List;

        private static const TOOLBAR_SIZE:int = 31;
        public static const MAIL_ITEM_SELECTED:String = "mailItemSelected";
        public static const SEND_INSTANT_MAIL:String = "sendInstantMail";
        //
        private var _data:Array;
        private var _friends:Array;
        //
        private var _limit:int = 25;
        private var _skip:int = 0;
        private var _query:String;
        private var _userSignedIn:Boolean;

        private var _isLoading:Boolean;
        private var _noMoreMails:Boolean;

        private var _noResultLabel:Label;
        private static var NO_RESULT_LABEL:String = 'No results found';
        private static var NO_MAIL_LABEL:String = 'Currently, you don\'t have any mails in your mailbox';
        public static const CONTENT_MAX_WIDTH:Number = 568;
        public static const MARGIN_SIZE:Number = 48;


        public function MailsScreen() {
            super("MailsScreen");
        }

        override public function dispose():void {
            super.dispose();

            if(viewToggleGroup){
                viewToggleGroup.removeEventListeners();
            }

            if(_mainMenuView) {
                _mainMenuView.removeFromParent(true);
                _mainMenuView = null;
            }

            if(menuButton) {
                menuButton.removeFromParent(true);
                menuButton = null;
            }

            if(_background) {
                _background.removeFromParent(true);
                _background = null;
            }

            if(_bannerBackground) {
                _bannerBackground.removeFromParent(true);
                _bannerBackground = null;
            }

            if(_loadingBar) {
                _loadingBar.removeFromParent(true);
                _loadingBar = null;
            }

            if(titleLabel) {
                titleLabel.removeFromParent(true);
                titleLabel = null;
            }

            if(backgroundButton){
                backgroundButton.removeFromParent(true);
                backgroundButton = null;
            }

            if(searchInput){
                searchInput.removeFromParent(true);
                searchInput = null;
            }

            if(_mailsList){
                _mailsList.dataProvider = null;
                _mailsList.removeFromParent(true);
                _mailsList = null;
            }

            if(_noResultLabel){
                _noResultLabel.removeFromParent(true);
                _noResultLabel = null;
            }
            _data = null;
            _friends = null;
            //
            var excludeUsers:Array = UsersProxy.loggedInUser()?[UsersProxy.loggedInUser().id]:null;
            UserCircularAvatar.disposeStoredAvatarTextures(excludeUsers);
        }

        public function refreshMails(showLoadBar:Boolean = true):void{
            _loadingBar.visible = showLoadBar;
            loadContent(refreshMailsCompleteHandler, true, showLoadBar);
        }

        public function addMails(value:Array):void{
            if(value){
                mailsLoadComplete(value);
            }
            _isLoading = false;
            if(_loadingBar){
                _loadingBar.visible = _isLoading;
            }
            touchable = true;
        }

        public function updateWithLanguage(language : Language) : void {
            if (mainMenuView){
                mainMenuView.updateWithLanguage(language);
            }
        }

        override protected function initialize() : void {
            super.initialize();
            _noMoreMails = false;
            // _timestamp = new Date().time/1000;

            _background = new Quad(1, 1, 0xFFFFFF);
            addChild(_background);

            _bannerBackground = new Image(Texture.fromColor(1,1, UIColors.BANNER_BACKGROUND_COLOR));
            addChild(_bannerBackground);

            titleLabel = new Label();
            titleLabel.textRendererFactory = GalleryLabelFactory.screenTitle;
            titleLabel.text = "MESSAGES";
            titleLabel.validate();
            titleLabel.paddingBottom = 3;
            addChild(titleLabel);

            menuButton = ButtonFactory.menuBigButton("a_back_button", 0x23405e);
            menuButton.width = 35;
            menuButton.height = 35;
            menuButton.addEventListener(Event.TRIGGERED, onMenuButton);
            addChild(menuButton);

            menuButton.x = menuButton.y = (BannerStackedScreen.BANNER_SIZE - menuButton.height) / 2;

            var layout : HorizontalLayout = new HorizontalLayout();
            layout.gap = 0;
            layout.horizontalAlign = HorizontalAlign.CENTER;
            layout.verticalAlign = VerticalAlign.MIDDLE;
            layout.useVirtualLayout = true;
            initializeMailsList();
            //_limit = 10;
            createSearchView();

            _loadingBar ||= new TimelineLoadBar();
            _loadingBar.name = '_timelineLoadBar';
            _loadingBar.setSize(60, 60);
            _loadingBar.x = Constants.STAGE_WIDTH / 2 - _loadingBar.width / 2;
            _loadingBar.y = Constants.STAGE_HEIGHT / 2 - _loadingBar.height / 2;
            _loadingBar.visible = false;
            addChild(_loadingBar);

            loadContent(refreshMailsCompleteHandler);
        }

        private function initializeMailsList() : void {
            var layout : VerticalLayout = new VerticalLayout();
            layout.verticalAlign = VerticalAlign.TOP;
            layout.horizontalAlign = HorizontalAlign.CENTER;
            layout.useVirtualLayout = false;
            layout.gap = 0;
            layout.paddingLeft = 0;

            _mailsList = new List();
            _mailsList.isSelectable = true;
            _mailsList.dataProvider = new ListCollection();
            _mailsList.layout = layout;
            _mailsList.width = actualWidth;
            _mailsList.height = actualHeight - BannerStackedScreen.BANNER_SIZE;
            _mailsList.backgroundSkin = new Quad(1,1,0xfeffff);

            _mailsList.verticalScrollBarFactory = verticalScrollBarFactory;
            _mailsList.scrollBarDisplayMode = ScrollBarDisplayMode.FLOAT;
            _mailsList.horizontalScrollPolicy = ScrollPolicy.OFF;
            _mailsList.verticalScrollPolicy = ScrollPolicy.ON;
            _mailsList.addEventListener(Event.SCROLL, mailsListScrollHandler);
            _mailsList.itemRendererFactory = function() : IListItemRenderer {
                var renderer : MailLabelItemRenderer = new MailLabelItemRenderer();
                var item_w:Number = _mailsList.width;
                var item_h:Number = MailLabelItemRenderer.ITEM_HEIGHT;
                renderer.setSize(item_w, item_h);
                renderer.eventDispatcher = this;
                renderer.marginLeft = renderer.marginRight = MathUtil.clamp((actualWidth - MailsScreen.CONTENT_MAX_WIDTH) / 2, 0, MailsScreen.MARGIN_SIZE);
                return renderer;
            };
            _mailsList.addEventListener(Event.CHANGE, itemSelectHandler);
            addChildAt(_mailsList,1);
        }

        private function itemSelectHandler(event:Event):void {
            if(_mailsList.selectedItem){
                SoundManager.playSound(SoundManager.PART_MENU);
                _mailsList.selectedItem.seen = true;
                _mailsList.dataProvider.updateItemAt(_mailsList.selectedIndex);
                dispatchEventWith(MAIL_ITEM_SELECTED, false, {"data": _mailsList.selectedItem});
                _mailsList.selectedItem = null;
            }
        }

        private function verticalScrollBarFactory() : ScrollBar {
            var scrollbar:ScrollBar = new ScrollBar();
            scrollbar.direction = Direction.VERTICAL;
            scrollbar.thumbFactory = function() : Button {
                var button : Button = new Button();
                var sprite : Sprite = new Sprite();
                var back : Quad = new Quad(8, 25, 0xffffff);
                back.alpha = 0;
                var quad : Quad = new Quad(4, 25, 0xC1C3C2);
                quad.x = 12;
                sprite.addChild(back);
                sprite.addChild(quad);
                button.defaultSkin = sprite;

                return button;
            };
            return scrollbar;
        }

        public function setMargins():void{
            if(!stage){
                return;
            }
            var marginSize:Number = 24;
            if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_RIGHT){
                _mailsList.width = actualWidth - marginSize;
                _mailsList.paddingLeft = marginSize;
                _mailsList.paddingRight = 0;
                _mailsList.dataProvider.updateAll();
            }else if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_LEFT){
                _mailsList.width = actualWidth - marginSize;
                _mailsList.paddingLeft = 0;
                _mailsList.paddingRight = marginSize;
                _mailsList.dataProvider.updateAll();
            }else{
                _mailsList.width = actualWidth - 2 * marginSize;
                _mailsList.paddingLeft = marginSize;
                _mailsList.paddingRight = marginSize;
                _mailsList.dataProvider.updateAll();
            }
        }

        private function mailsListScrollHandler(event:Event):void {
            if (_mailsList.verticalScrollPosition > _mailsList.maxVerticalScrollPosition + MailLabelItemRenderer.ITEM_HEIGHT) {
                loadContent(mailsLoadComplete, false, true);
            }else if (_mailsList.verticalScrollPosition < -MailLabelItemRenderer.ITEM_HEIGHT) {
                refreshMails();
            }
        }

        // *******
        private var _onComplete:Function;
        private function loadContent(completeHandler:Function, refresh:Boolean = true, showLoadBar:Boolean = true):void{
            if(refresh){
                _noMoreMails = false;
            }
            if(_isLoading || _noMoreMails){
                return;
            }
            if(_noResultLabel){
                _noResultLabel.visible = false;
            }
            _isLoading = true;
            if(_loadingBar){
                _loadingBar.visible = showLoadBar;
            }
            touchable = false;

            if(refresh || !_friends){
                _skip = 0;
            }else if(_friends){
                _skip = _friends.length;
            }
            _mailsProxy ||= ApplicationFacade.getInstance().retrieveProxy(MailsProxy.NAME) as MailsProxy;
            _mailsProxy.loadMails(_skip, _limit + 1);
        }

        public function get userSignedIn():Boolean{
            return _userSignedIn;
        }

        public function set userSignedIn(value:Boolean):void {
            _userSignedIn = value;
            if (!owner)	return;
            invalidate("userSignedIn");
        }
        private function refreshMailsCompleteHandler(value:Array):void{
            clearContent();
            setTimeout(function():void{
                mailsLoadComplete(value);
            }, 100);
        }

        private function mailsLoadComplete(value:Array):void{
            _noResultLabel.visible = false;
            if(value){
                if(value.length <= _limit){
                    _noMoreMails = true;
                }else{
                    _noMoreMails = false;
                    value.pop();
                }
            }

            if(_skip == 0){
                _friends = [];
            }else{
                _friends ||= [];
            }
            _friends = _friends.concat(value);
            if(_mailsList.dataProvider == null || _skip == 0){
                if(_mailsList.dataProvider){
                    _mailsList.dataProvider.removeAll();
                }
                _mailsList.dataProvider = new ListCollection(value);
            }else{
                _mailsList.dataProvider.addAll(new ListCollection(value));
            }
            if(_friends.length == 0){
                _noResultLabel.text = NO_MAIL_LABEL;
                _noResultLabel.visible = true;
            }
            touchable = true;
            if(_onComplete != null){
                _onComplete();
                _onComplete = null;
            }
        }

        public function dispatchExternalEvent(msg:String):void {
            if (mainMenuView) {
                mainMenuView.hide();
            }
            dispatchEventWith(msg);
        }

        override public function ShowDone():void {
            Tracker.trackEvent("Mails Screen", "open");
        }

        override public function transitionComplete():void{
            super .transitionComplete();
            BannerStackedScreen.showAdBanner();
        }

        private function menuOpenHandler(event:Event):void {
            BannerStackedScreen.hideAdBanner(false);
        }

        private function menuCloseHandler(event:Event):void {
            BannerStackedScreen.showAdBanner();
        }

        override protected function screen_removedFromStageHandler(event:Event):void{
            super.screen_removedFromStageHandler(event);
            BannerStackedScreen.hideAdBanner(true);
        }

        override protected function draw() : void {
            super.draw();
            if (!mainMenuView){
                mainMenuView = new MainMenuView();
                mainMenuView.name = "mainMenuView";
                mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight);
                mainMenuView.parentActualWidth = actualWidth;
                mainMenuView.parentActualHeight = actualHeight;
                addChild(mainMenuView);

                mainMenuView.addEventListener(MainMenuView.START_SHOW, menuOpenHandler);
                mainMenuView.addEventListener(Event.CLOSE, menuCloseHandler);
            }

            if(_background){
                _background.width = actualWidth;
                _background.height = actualHeight;
            }

            if(_bannerBackground){
                _bannerBackground.width = actualWidth;
                _bannerBackground.height = BannerStackedScreen.BANNER_SIZE;
            }

            if(titleLabel){
                titleLabel.x = 50;
                titleLabel.y = (BannerStackedScreen.BANNER_SIZE - titleLabel.height)/2;
            }

            if(viewLayout){
                viewLayout.x = actualWidth - viewLayout.width - 7.5;
                viewLayout.y = (BannerStackedScreen.BANNER_SIZE - viewLayout.height)/2;
            }

            if(searchInput){
                searchInput.x = actualWidth - searchInput.width - 10;
                searchInput.y = (BannerStackedScreen.BANNER_SIZE - searchInput.height) / 2;
            }

            if(_mailsList){
                _mailsList.width = actualWidth;
                _mailsList.height = actualHeight - BannerStackedScreen.BANNER_SIZE;
                _mailsList.x = 0;
                _mailsList.y = actualHeight - _mailsList.height;
            }

            if(_noResultLabel){
                _noResultLabel.x = (actualWidth - _noResultLabel.width) / 2;
                if(_mailsList){
                    _noResultLabel.y = _mailsList.y + (_mailsList.height - _noResultLabel.height) / 2;
                }else{
                    _noResultLabel.y = (actualHeight - _noResultLabel.height) / 2;
                }
            }
        }

        private function clearContent():void{
            if(_mailsList.dataProvider){
                _mailsList.dataProvider.removeAll();
            }
            _mailsList.dataProvider = null;
            _data = _friends = null;
        }

        private function createSearchView():void{
            searchInput = new SearchInput(true);
            searchInput.expandedWidth = 120;
            searchInput.x = 5;
            searchInput.addEventListener(Event.CHANGE, searchInputChangeHandler);

            _noResultLabel = new Label();
            _noResultLabel.textRendererFactory = GalleryLabelFactory.sortByTitleLabel;
            _noResultLabel.text = NO_RESULT_LABEL;

            addChild(searchInput);
            addChild(_noResultLabel);
            _noResultLabel.visible = false;
        }

        private function searchInputChangeHandler(event:Event):void {
            /*Starling.juggler.removeDelayedCalls(startSearch);
            Starling.juggler.delayCall(startSearch, 0.1);*/
            startSearch();
        }

        private function startSearch():void{
            _noResultLabel.visible = false;
            if(searchInput.text == ""){
                _mailsList.dataProvider = new ListCollection(_friends);
                if(_friends.length == 0){
                    _noResultLabel.text = NO_MAIL_LABEL;
                    _noResultLabel.visible = true;
                }
                return;
            }
            var patStr:String = searchInput.text.replace(/[^\S\r\n]+/, ",");
            var ws:Array = patStr.split(",");
            patStr = "";
            var i:int;
            for(i = 0; i < ws.length; i++){
                patStr += "(?=.*" + ws[i] + ")";
            }
            var pattern:RegExp = new RegExp(patStr, "gsmi");
            var result:Array = [];
            for(i = 0; i < _friends.length; i++){
                var obj:Object = _friends[i];
                if(pattern.test(obj.name) || pattern.test(obj.username)){
                    result.push(obj);
                }
            }
            //
            _mailsList.dataProvider = new ListCollection(result);
            if(result.length == 0){
                _noResultLabel.text = NO_RESULT_LABEL;
                _noResultLabel.visible = true;
            }
        }

        private function onMenuButton(event:Event):void{
            homeButtonTriggeredHandler();
        }

        public function set mainMenuView(value : MainMenuView) :void {
            _mainMenuView = value;
            _mainMenuView.owner = this;
        }

        public function get mainMenuView():MainMenuView {
            return _mainMenuView;
        }

        public function set selectedLanguage(value:Language):void {
            _selectedLanguage =value;
        }

        public function get selectedLanguage():Language {
            return _selectedLanguage;
        }

        public function set languages(value:Array):void{
            _languages = value;
        }
        public function get languages():Array{
            return _languages;
        }

        public function brandsScreenButtonTriggeredHandler(event:Event = null):void{
            dispatchEventWith(StackedScreen.BRANDS_SELECTED);
        }

        public function homeButtonTriggeredHandler(event:Event = null):void {
            dispatchEventWith(StackedScreen.HOME_SELECTED);
        }

        public function enterRowsState():void{

        }

        public function set onComplete(onComplete:Function):void {
            _onComplete = onComplete;
        }
    }
}
