package com.despark.tuning3d.view.components.news {
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.MultilingualLabel;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.core.HomeScreenPopUp;
    import com.distriqt.AdsManager;
    import com.utils.DisplayObjectUtil;
    import com.utils.FileUtil;

    import feathers.controls.Label;
    import feathers.controls.List;
    import feathers.controls.ScrollBarDisplayMode;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.filesystem.File;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;

    public class NewsPopUp extends PopUp{


    public static const MORE_NEWS : String = "MoreNews";
    private var _newsList:List;
    private var _news : Array;
    private var _showBannerAfterExit:Boolean;
    private var _bannerScreen:String;
    private var _messageLabel : Label;

    private var _data:Object;
    private var _homeScreenPopup:HomeScreenPopUp;
    public function NewsPopUp(object:Object = null, homeScreenPopUp:HomeScreenPopUp = null) {
        super();
        _showBannerAfterExit = (object && object.hasOwnProperty("showBannerAfterExit") && object.showBannerAfterExit);
        _bannerScreen = (object && object.hasOwnProperty("bannerScreen"))?object["bannerScreen"]:BannerScreen.BANNER_SCREEN;
        _data = object.data;
        _homeScreenPopup = homeScreenPopUp;
        AdsManager.$.showBannerAfterComplete = false;
    }

    override protected function initialize():void {
        super.initialize();

        setSizeInternal(this.stage.stageWidth*1, this.stage.stageHeight,false);
        background.color = 0x555555;
        background.alpha = 0.8;
        contentBackgroundImage.visible = false;

        var layout : VerticalLayout = new VerticalLayout();
        layout.gap = 0;
        layout.horizontalAlign = HorizontalAlign.LEFT;
        layout.verticalAlign = VerticalAlign.TOP;

        _news = [];

        data = _data;


        _newsList = new List();
        _newsList.dataProvider = new ListCollection(_news);
        _newsList.layout = layout;
        _newsList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
        _newsList.horizontalScrollPolicy = ScrollPolicy.OFF;
        _newsList.verticalScrollPolicy = ScrollPolicy.ON;
        _newsList.backgroundSkin = new Quad(1,1,0xe8e8e8);
        _newsList.addEventListener(Event.SCROLL, listScrollHandler);


        _newsList.width = actualWidth;
        _newsList.height = actualHeight;
        _newsList.x = 0;
        _newsList.y = 0;

        rendererWidth = _newsList.width -_newsList.paddingLeft - _newsList.paddingRight;

        _newsList.itemRendererFactory =  newsItemRendererFactory;
        addChild(_newsList);
        addChild(closeButton);
        closeButton.defaultSkin = new Image( assetManager.getTexture('model_close'));
        closeButton.defaultSkin.alpha = 0.6;
        closeButton.setSize(36, 36);

        _messageLabel = new MultilingualLabel();
        _messageLabel.includeInLayout = false;
        _messageLabel.visible = false;
        _messageLabel.height = 14;
        _messageLabel.textRendererFactory = LabelFactory.notificationsMessageField;
        _messageLabel.text = "We don't have any news at this moment";
        _messageLabel.width = actualWidth;
        addChild(_messageLabel);
    }

    override protected function closeButtonTriggeredHandler(event:Event):void{
        super .closeButtonTriggeredHandler(event);
        SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        Starling.juggler.delayCall(function():void{
            AdsManager.$.showBannerAfterComplete = true;
            if(_showBannerAfterExit){
                if(_bannerScreen == BannerScreen.BANNER_SCREEN){
                    BannerStackedScreen.showAdBanner();
                }else if(_bannerScreen == BannerScreen.HOME_SCREEN){
                    AdsManager.$.bannerShow(BannerScreen.HOME_SCREEN);
                }
            }
        }, 0.5);
    }

    private function listScrollHandler(event:Event):void {
        if(!_newsList || !_news || _news.length == 0 || _newsList.dataProvider.length == 0) {
            return;
        }

        if(_newsList.verticalScrollPosition >= _newsList.maxVerticalScrollPosition-30/* && _notifications.length<MAX_COUNT*/) {
            dispatchEventWith(MORE_NEWS);
        }
    }

    override protected function draw():void {
        super.draw();

        if(isInvalid(INVALIDATION_FLAG_SIZE)) {
            _newsList.width = actualWidth;
            _newsList.height = actualHeight;
            _newsList.x = 0;
            _newsList.y = 0;

            rendererWidth = _newsList.width -_newsList.paddingLeft - _newsList.paddingRight;

            if(closeButton){
                closeButton.x = actualWidth - (45 + closeButton.width) / 2;
                closeButton.y = 8;
            }

            if(_messageLabel){
                _messageLabel.x = (actualWidth - _messageLabel.width) / 2;
                _messageLabel.y = (actualHeight - _messageLabel.height) / 2;
            }
        }
    }

    public function set news (value : Array) : void {
        if(_news && _news.length > 0) {
            var newListCollection : ListCollection = new ListCollection();
            for(var i : int = 0; i < value.length; i++) {
                newListCollection.addItem(value[i]);// its automatically adds in feed array too
            }
            _newsList.dataProvider.addAll(newListCollection);
        }  else {
            _news = value;
            if(_newsList) {
                _newsList.dataProvider = new ListCollection(value);
                _newsList.validate();
            }
        }
        _messageLabel.visible = (_news.length == 0);
    }

    private var rendererWidth : Number;

    private function newsItemRendererFactory() : IListItemRenderer {
        var renderer : NewsPopupItemRenderer = new NewsPopupItemRenderer();
        renderer.setSize(rendererWidth, 80);
        renderer.eventDispatcher = this;
        renderer.homeScreen = _homeScreenPopup;
        return renderer;
    }

    override public function dispose():void {
        super.dispose();
        if(_newsList) {
            _newsList.removeEventListener(Event.SCROLL, listScrollHandler);
            _newsList.dataProvider = null;
            _newsList.removeFromParent(true);
            _newsList = null;
        }

        if(_messageLabel){
            _messageLabel.removeFromParent(true);
            _messageLabel = null;
        }

        _news = null;
        _data = null;
        _homeScreenPopup = null;
    }

    public function close():void {
        dispatchEventWith(CLOSE_BUTTON_SELECTED);
    }

        public function updateView():void{
            _newsList.dataProvider = new ListCollection(_news);
        }

        public function set data(value:Object):void {
            var newsCacheFile:File = File.applicationStorageDirectory.resolvePath("news_feed.json");
            var newsCacheContent:Object;
            if(newsCacheFile.exists){
                var newsCacheStr:String = FileUtil.getFileContent(newsCacheFile);
                if(newsCacheStr && newsCacheStr.length){
                    newsCacheContent = JSON.parse(newsCacheStr);
                }else{
                    newsCacheContent = {};
                }
            }else{
                newsCacheContent = {};
            }
            var newsCount:int = 0;

            _news = [];
            _data = value;
            var feed:NewsFeedData;
            var i:int;
            if(_data){
                for(var feedType:String in _data){
                    if(NewsFeedData.isAllowedType(feedType)){
                        for(i = 0; i < _data[feedType].length; i++){
                            feed = new NewsFeedData(_data[feedType][i], feedType);
                            _news.push(feed);
                        }
                    }
                }
            }

            for(i = 0; i < _news.length; i++){
                feed = _news[i] as NewsFeedData;
                if(feed && !newsCacheContent.hasOwnProperty(feed.uniqueId)){
                    newsCacheContent[feed.uniqueId] = true;
                    newsCount ++;
                }
            }
            FileUtil.putFileContent(newsCacheFile, JSON.stringify(newsCacheContent));
            _news.sortOn(["updateDate"], [Array.NUMERIC | Array.DESCENDING]);
        }
    }
}
