package com.despark.tuning3d.view.components.news {
    import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.despark.tuning3d.view.core.HomeScreenPopUp;

    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.layout.FlowLayout;
    import feathers.layout.HorizontalAlign;

    import flash.display.BitmapData;
    import flash.geom.Rectangle;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.textures.Texture;

    public class NewsPopupItemRenderer extends ListItemRenderer{
    private var _gap:int;
    private static const _vGap:Number = 2.5;
    private var _bgQuad:Quad;

    private var _bottomLineQuad:Quad;

    private var _dateLabel:Label;
    private var _carNameLabel:Label;
    private var _text1Label:Label;
    private var _text2Label:Label;
    private var _avatar : UserCircularAvatar;

    private var _eventDispatcher : *;
    private var _labelsContainer:LayoutGroup;

    private var _iconImage:Image;
    private var _iconName:String;
    private var _borderImage:Image;
    private var _iconBorder:Image;
    private var _homeScreen:HomeScreenPopUp;
    private static var _borderTexture:Texture;

    public function set eventDispatcher(value:*) : void{
        _eventDispatcher = value;
    }
    public function get eventDispatcher() : *{
        return _eventDispatcher;
    }

    public function NewsPopupItemRenderer() {
    }

    override protected function initialize():void {
        super.initialize();

        _gap = 10;
        _bottomLineQuad = new Quad(1, 1, 0x9a9899);
        addChild(_bottomLineQuad);

        _bgQuad = new Quad(1,1,0xffffff);
        addChild(_bgQuad);

        var labelLayout:FlowLayout = new FlowLayout();
        labelLayout.horizontalAlign = HorizontalAlign.LEFT;
        labelLayout.gap = 5;
        _labelsContainer = new LayoutGroup();
        _labelsContainer.layout = labelLayout;
        addChild(_labelsContainer);

        _dateLabel = new Label();
        _dateLabel.textRendererFactory = LabelFactory.newsDateLabel;
        _labelsContainer.addChild(_dateLabel);

        _carNameLabel = new Label();
        _carNameLabel.textRendererFactory = LabelFactory.newsCarNameLabel;
        _labelsContainer.addChild(_carNameLabel);

        _text1Label = new Label();
        _text1Label.textRendererFactory = LabelFactory.newsTextLabel;
        _labelsContainer.addChild(_text1Label);

        _text2Label = new Label();
        _text2Label.textRendererFactory = LabelFactory.newsTextLabel;
        _labelsContainer.addChild(_text2Label);

        _labelsContainer.y = (actualHeight - _labelsContainer.height)/2;

        addEventListener(TouchEvent.TOUCH, touchHandler);
    }


    public function get feedData():NewsFeedData {
        return _data as NewsFeedData;
    }

    override protected function commitData():void {
        super.commitData();
        update();
    }


    override protected function commitSelection():void {
    }

    private function update():void {
        if(!feedData) return;
        _carNameLabel.text = feedData.carFullName + " -";
        _dateLabel.text = feedData.date + " |";
        _text1Label.text = feedData.textLine1;
        _text2Label.text = feedData.textLine2;
        if(_text1Label.text.length){
            _labelsContainer.addChild(_text1Label);
        }else{
            _text1Label.removeFromParent();
        }
        if(_text2Label.text.length){
            _labelsContainer.addChild(_text2Label);
        }else{
            _text2Label.removeFromParent();
        }

        _dateLabel.validate();
        _carNameLabel.validate();
        _text1Label.validate();
        _text2Label.validate();
        switch(feedData.feedType) {
            case NewsFeedData.TUNING_NEWS:

                break;
            case NewsFeedData.TRUCK_NEWS:

                break;
            case NewsFeedData.MUSCLE_NEWS:

                break;
        }

        updateLabelPosition();
        updateIcon();
    }

    private function updateIcon():void {
        if(feedData.iconTextureName && feedData.iconTextureName != _iconName){
            _iconBorder ||= new Image(ButtonFactory.assetManager.getTexture("icon_border"));
            _iconBorder.scale = ButtonFactory.assetManager.scaleFactor / 2;
            addChild(_iconBorder);

            _iconName = feedData.iconTextureName;
            if(_iconImage){
                _iconImage.removeFromParent(true);
            }
            _iconImage = new Image(ButtonFactory.assetManager.getTexture(_iconName));
            _iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
            addChild(_iconImage);
            _borderTexture ||= function ():Texture {
                var bd:BitmapData = new BitmapData(20, 20, true, 0xFFe0e0e0);
                bd.fillRect(new Rectangle( 1, 1, 18, 18), 0xFFFFFFFF);
                var txt:Texture = Texture.fromBitmapData(bd);
                Image.bindScale9GridToTexture(txt, new Rectangle(5,5,10,10));
                return txt;
            }();
            _borderImage ||= new Image(_borderTexture);
            _borderImage.alpha = 0.6;
            _borderImage.width = actualWidth - 135;
            _borderImage.height = _bgQuad.height - 2*_gap;
            if(!_borderImage.parent){
                addChildAt(_borderImage, getChildIndex(_bgQuad)+1);
            }
            invalidate();
        }
    }

    private function updateLabelPosition():void {
        _labelsContainer.validate();
        _labelsContainer.y = (actualHeight - _labelsContainer.height)/2;
    }

    override protected function draw():void {
        super.draw();

        _bgQuad.width = actualWidth;
        _bgQuad.height = actualHeight - 2*_vGap;
        _bgQuad.y = _vGap;

        _bottomLineQuad.y = _bgQuad.y + _bgQuad.height;
        _bottomLineQuad.width = actualWidth;
        _bottomLineQuad.height = 0.5;
        if(_borderImage){
            _borderImage.height = _bgQuad.height - 2*_gap;
            _borderImage.x = 90;
            _borderImage.y = _bgQuad.y + _gap;
        }

        if(_iconBorder){
            _iconBorder.x = _borderImage.x - _iconBorder.width;
            _iconBorder.y = (actualHeight - _iconBorder.height) / 2;
        }

        if(_iconImage){
            _iconImage.x = _iconBorder.x + (_iconBorder.width - 10 - _iconImage.width) / 2;
            _iconImage.y = (actualHeight-_iconImage.height)/2;
        }

        if(_labelsContainer){
            _labelsContainer.width = _borderImage.width - 20;
            _labelsContainer.validate();
            _labelsContainer.x = _borderImage.x + 10;
            _labelsContainer.y = (actualHeight - _labelsContainer.height)/2;
        }
        updateLabelPosition();
    }


    private function touchHandler(event:TouchEvent) : void {
        var touch : Touch = event.touches[0] as Touch;
        if (touch.phase == TouchPhase.ENDED && !this.owner.isScrolling){
            if(feedData && feedData.carId && feedData.carId.length) {
                SoundManager.playSound(SoundManager.PART_MENU);
                AppMode.self.setModeValues(feedData.projectType);
                _homeScreen.dispatchEvent(new Event(StackedScreen.TUNING_SELECTED, false, {carId: data.carId, screenMode: feedData.feedType}));
            }
        }
    }

    override public function dispose():void {
        super.dispose();
        if(_avatar) {
            _avatar.removeFromParent(true);
            _avatar = null;
        }
        if(imageLoader){
            imageLoader.removeFromParent(true);
            imageLoader = null;
        }
        if(_iconImage){
            _iconImage.removeFromParent(true);
            _iconImage = null;
        }
        if(_borderImage){
            _borderImage.removeFromParent(true);
            _borderImage = null;
        }
    }

        public function set homeScreen(value:HomeScreenPopUp):void {
            _homeScreen = value;
        }
    }
}
