package com.despark.tuning3d.view.components.parts
{
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.ListItemRenderer;
	import com.despark.tuning3d.view.components.tunings.colors.TuningDiskSettingsView;
	import com.utils.FontSelector;
	
	import flash.geom.Rectangle;
	import flash.text.TextFormat;
	
	import feathers.controls.Label;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	import feathers.core.ITextRenderer;
	
	import starling.display.Image;
	import starling.display.Quad;
	
	public class CamberItemRenderer extends ListItemRenderer
	{
		
		protected var quad_back:Quad;
		
		
		private var iconUp:Image=null;
		private var iconDown:Image=null;
		private var iconSelectedBackground:Image=null;
		
		
		private var labelUp:Label;
		private var labelDown:Label;
		
		private var labelTextColorUp:uint=0xFFFFFF;
		private var labelTextColorDown:uint=0x242424;

		public function CamberItemRenderer()
		{
			super();
		}
		
		public function get item() : Object
		{
			return data as Object;
		}
		
		protected override function initialize():void 
		{
			quad_back=new Quad(actualWidth, actualHeight, 0xffffff);
			quad_back.visible=true;
			quad_back.alpha=0;
			
			addChildAt(quad_back, 0);
			
			super.initialize();
			
			this._isQuickHitAreaEnabled=true;
			this._hitArea=new Rectangle(0,0,0,0);
		}
		
		protected override function commitData() : void 
		{
			super.commitData();

			var lastChildIndex:int;
			if(quad_back)
			lastChildIndex=getChildIndex(quad_back);
			else
			lastChildIndex=numChildren-1;
			
			if(item.icon_up!=null){
				
				if(!iconUp){
				
					iconUp=new Image(ButtonFactory.assetManager.getTexture(item.icon_up));
					iconUp.visible=false;
					iconUp.width=18;
					iconUp.height=18;
					
					
					lastChildIndex++;
					
					addChildAt(iconUp,lastChildIndex);
				
				}
			}
			
			if(item.icon_down!=null){
				
				if(!iconDown){
					iconDown=new Image(ButtonFactory.assetManager.getTexture(item.icon_down));
					iconDown.visible=false;
					iconDown.width=18;
					iconDown.height=18;

					lastChildIndex++;
					
					addChildAt(iconDown,lastChildIndex);     
				}
			}
			
			if(item.selected_background!=null){
				
				if(!iconSelectedBackground){
					iconSelectedBackground=new Image(ButtonFactory.assetManager.getTexture(item.selected_background));
					iconSelectedBackground.visible=false;
					iconSelectedBackground.width=18;
					iconSelectedBackground.height=18;
					
					lastChildIndex++;
					
					addChildAt(iconSelectedBackground, lastChildIndex);
				}
			}
			
			if(item.title!=null){
				
				if(!labelUp){
					labelUp=new Label();
					labelUp.visible=false;
					labelUp.textRendererFactory=createLableUp;
					labelUp.text=item.title;
					labelUp.validate();
					
					lastChildIndex++;
					addChildAt(labelUp, lastChildIndex);
				}
				
				if(!labelDown){
					labelDown=new Label();
					labelDown.visible=false;
					labelDown.textRendererFactory=createLableDown;
					labelDown.text=item.title;
					labelDown.validate();
					
					lastChildIndex++;
					
					addChildAt(labelDown, lastChildIndex);
				}
			}
			updateVisible();
		}
		
		private function updateVisible():void{
			
			if(iconUp){
				
				iconUp.visible=!isSelected;
			}
			
			if(iconDown){
				
				iconDown.visible=isSelected;
			}
			
			if(iconSelectedBackground){
				
				iconSelectedBackground.visible=isSelected;
			}
			
			if(labelUp){
				
				labelUp.visible=!isSelected;
			}
			
			if(labelDown){
				
				labelDown.visible=isSelected;
			}
		}
		
		private function createLableUp():ITextRenderer{
			
			var label:TextFieldTextRenderer=new TextFieldTextRenderer();
			
			label.textFormat=new TextFormat(FontSelector.getFont(true), 11, labelTextColorUp );
			label.textFormat.bold=true;
			
			label.embedFonts=FontSelector.embedFonts();
			
			
			return label ;
		}
		
		private function createLableDown():ITextRenderer{
			
			var label:TextFieldTextRenderer=new TextFieldTextRenderer();
			
			label.textFormat=new TextFormat(FontSelector.getFont(true), 11, labelTextColorDown , true);
			label.textFormat.bold=true;
			
			label.embedFonts=FontSelector.embedFonts();
			
			return label ;
		}
		
		protected override function commitSelection() :void
		{
			super.commitSelection();
			
			updateVisible();
		}
		
		protected override function layout():void
		{
			quad_back.width=actualWidth;
			quad_back.height=actualHeight;
			
			
			_hitArea.x=-2.5;
			_hitArea.y=-2.5;
			
			_hitArea.width=actualWidth+5;
			_hitArea.height=actualWidth+5;
		
			updateVisible();
			
			
			if(label)
				label.visible=false;
			
			if(imageLoader)
				imageLoader.visible=false;
			
			
			if(iconUp){
				
				iconUp.x=(TuningDiskSettingsView.ITEM_WIDTH-iconUp.width)/2-.5;
				iconUp.y=(TuningDiskSettingsView.ITEM_HEIGHT-iconUp.height)/2;
			}
			
			
			if(iconDown){
				
				iconDown.x=(TuningDiskSettingsView.ITEM_WIDTH-iconDown.width)/2-.5;
				iconDown.y=(TuningDiskSettingsView.ITEM_HEIGHT-iconDown.height)/2;
			}
			
			if(iconSelectedBackground){
				
				iconSelectedBackground.x=(TuningDiskSettingsView.ITEM_WIDTH-iconSelectedBackground.width)/2-.5;
				iconSelectedBackground.y=(TuningDiskSettingsView.ITEM_HEIGHT-iconSelectedBackground.height)/2;
			}
			
			if(labelUp){
				
				labelUp.validate();
				labelUp.x=(TuningDiskSettingsView.ITEM_WIDTH-labelUp.width)/2-.5;
				labelUp.y=(TuningDiskSettingsView.ITEM_HEIGHT-labelUp.height)/2;
			}
			
			if(labelDown){
				
				labelUp.validate();
				labelDown.x=(TuningDiskSettingsView.ITEM_WIDTH-labelDown.width)/2-.5;
				labelDown.y=(TuningDiskSettingsView.ITEM_HEIGHT-labelDown.height)/2;
			}
			
		}
		
		protected override function draw():void{
			
			super.draw();
			var sizeInvalid : Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
			
			if(sizeInvalid){
			
				quad_back.width=actualWidth;
				quad_back.height=actualHeight;
				
				
				if(label)
					label.visible=false;
				
				if(imageLoader)
					imageLoader.visible=false;
				
			
				if(iconUp){
				
					iconUp.x=(TuningDiskSettingsView.ITEM_WIDTH-iconUp.width)/2-.5;
					iconUp.y=(TuningDiskSettingsView.ITEM_HEIGHT-iconUp.height)/2;
				}
				
				
				if(iconDown){
					
					iconDown.x=(TuningDiskSettingsView.ITEM_WIDTH-iconDown.width)/2-.5;
					iconDown.y=(TuningDiskSettingsView.ITEM_HEIGHT-iconDown.height)/2;
				}
				
				if(iconSelectedBackground){
					
					iconSelectedBackground.x=(TuningDiskSettingsView.ITEM_WIDTH-iconSelectedBackground.width)/2-.5;
					iconSelectedBackground.y=(TuningDiskSettingsView.ITEM_HEIGHT-iconSelectedBackground.height)/2;
				}
				
				if(labelUp){
					
					labelUp.x=(TuningDiskSettingsView.ITEM_WIDTH-labelUp.width)/2-.5;
					labelUp.y=(TuningDiskSettingsView.ITEM_HEIGHT-labelUp.height)/2;
				}
				
				if(labelDown){
					
					labelDown.x=(TuningDiskSettingsView.ITEM_WIDTH-labelDown.width)/2-.5;
					labelDown.y=(TuningDiskSettingsView.ITEM_HEIGHT-labelDown.height)/2;
				}
			
			}
			
			updateVisible();
		}
		
	}
}