package com.despark.tuning3d.view.components.parts {
	import com.despark.tuning3d.model.disks.Disk;
	import com.despark.tuning3d.model.disks.DiskBrand;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.utils.FontSelector;

	import feathers.controls.Label;

	import flash.text.TextFormatAlign;

	import starling.core.Starling;
	import starling.utils.MathUtil;

	public class PartDiskTurn5Renderer extends PartDiskItemRenderer {
		private var priceLabel:Label;
		private static const _truncated_name:Object = {};
		public function PartDiskTurn5Renderer() {
			super();
			fontSize = LabelFactory.TURN5_PRICE_SIZE;
			textColor = 0xFFFFFF;
			_bold = true;
		}

		protected override function initialize():void {
			super .initialize();

			label.paddingLeft = label.paddingRight = 3;

			priceLabel = new Label();
			priceLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getMontserratRegular(), LabelFactory.TURN5_PRICE_SIZE, 0xFFFFFF, TextFormatAlign.LEFT, true);
			priceLabel.wordWrap = true;
			priceLabel.paddingLeft = priceLabel.paddingRight = 3;
			addChild(priceLabel);

			if(label2){
				addChild(label2);
			}
		}

		protected override function commitData() : void {
			super.commitData();
			label.wordWrap = true;
			label.textFormat.align = TextFormatAlign.LEFT;

			if (part && part.name != 'banner') {
				setSize(220, 50);
			}

			if(part && _truncated_name[part.name]){
				label.text = _truncated_name[part.name];
			}
			radiusBtnLayout();

			if(part){
				priceLabel.text = "Starting at: " + (part.priceString?part.priceString:"");
				priceLabel.visible = true;
				priceLabel.validate();
				Starling.juggler.delayCall(layout, 0.25);
			}else{
				priceLabel.text = "";
				priceLabel.visible = false;
			}

			if(partDisk && partDisk.manufacturer){
				var diskBrand:DiskBrand = Disk.getDiskBrandById(partDisk.manufacturer);
				if (diskBrand && diskBrand.id != "factory") {
					label2.text = diskBrand.name;
				} else {
					label2.text = "";
				}
			}
		}

		override protected function commitSelection():void{
			super .commitSelection();
			radiusBtnLayout();
		}

		override protected function draw():void{
			super.draw();
			radiusBtnLayout();
		}
		   
		override protected function layout():void{
			super.layout();
			if(_isDisposed){
				return;
			}
			label2.visible = (label2.text && label2.text.length > 0);
			radiusBtnLayout();
			//
			label.width = actualWidth - imageLoader.width - 10 - (btnInfo.visible?(btnInfo.width + 10):0);
			if(label2.visible){
				label.height = 12.5;
			}else{
				label.height = 25;
			}

			label.x = imageLoader.x + imageLoader.width + 3;
			label2.x = label.x - 2;

			var contentHeight:Number = 24;
			if(priceLabel.text && priceLabel.text.length){
				priceLabel.validate();
				priceLabel.x = label.x;
				contentHeight += priceLabel.height + 1;
			}else{
				contentHeight = label.height;
			}
			label.y = (actualHeight - contentHeight) / 2 - (label.height - 25);
			label2.y = label.y - label2.height - 2;
			if(priceLabel.text && priceLabel.text.length){
				if(label2.visible){
					priceLabel.y = label.y + label.height + 1;
				}else{
					priceLabel.y = label.y + label.height + (label.height < 13? 13: 1);
				}
			}
		}

		override public function set bold(value : Boolean) : void {
			super .bold = value;
			if (priceLabel) {
				priceLabel.textRendererFactory = LabelFactory.createTextRendererFactory(value?FontSelector.getMontserratBold():FontSelector.getMontserratRegular(), LabelFactory.TURN5_PRICE_SIZE, 0xFFFFFF, TextFormatAlign.LEFT, true);
				priceLabel.invalidate();
			}
		}

		override public function dispose():void{
			super .dispose();
			if(priceLabel){
				priceLabel.removeFromParent(true);
				priceLabel = null;
			}
		}

		private function radiusBtnLayout():void{
			if(btnContainer){
				btnContainer.visible = false;
			}
			if(btnContainer && btnContainer.visible){
				btnContainer.x = 0;
				imageLoader.x = btnContainer.x + btnContainer.width + 3;
			}
		}
	}
}

