package com.despark.tuning3d.view.components.parts
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.utils.FontSelector;
	
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import feathers.controls.Label;
	import feathers.controls.text.BitmapFontTextRenderer;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	import feathers.core.ITextRenderer;
	import feathers.text.BitmapFontTextFormat;
	
	import starling.events.Event;
	import starling.text.TextField;

	public class TunedPartView extends FeathersControl implements Multilingual
	{
		private var nameLabel : MultilingualLabel;
		private var valueLabel : Label;
		private var _partSlot : PartSlot;
		private var currentLanguage : Language;
		
		public function TunedPartView()
		{
			super();
		}
		
		public function get partSlot() : PartSlot
		{
			return _partSlot;
		}

		public function set partSlot(value : PartSlot):void
		{
			_partSlot = value;
			invalidate("partSlot");
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			currentLanguage = language;
		}

		override protected function initialize() : void
		{
			super.initialize();
			
			nameLabel = new MultilingualLabel();
			nameLabel.textFormat = new TextFormat(FontSelector.getFont(), 7, 0x2D3327);
			nameLabel.textFormat.align = TextFormatAlign.LEFT;
			addChild(nameLabel);
			
				
			valueLabel = new Label();
			addChild(valueLabel);
			
			valueLabel.textRendererFactory = function() : ITextRenderer
			{
				/*var textRenderer : BitmapFontTextRenderer = new BitmapFontTextRenderer();
				
				textRenderer.textFormat = new BitmapFontTextFormat(TextField.getBitmapFont("openSans"), 7, 0x8C8F90);
				textRenderer.textFormat.horizontalAlign = "right";
				textRenderer.useSeparateBatch = false;*/
				
				var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
				
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 7, 0x8C8F90);
				textRenderer.textFormat.align = TextFormatAlign.RIGHT;
				
				textRenderer.embedFonts=FontSelector.embedFonts();
				
				return textRenderer;
			}
		}
		
		override protected function draw() : void
		{
			super.draw();
			
			if(isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE))
			{
				nameLabel.width = int(actualWidth / 2);
				nameLabel.height = actualHeight;
			
				valueLabel.x = nameLabel.x + nameLabel.width;
				valueLabel.width = actualWidth - valueLabel.x;
			}
			
			if(isInvalid("partSlot"))
			{
				if(currentLanguage){
					nameLabel.text = currentLanguage.translate("componentTypes", partSlot.name).toUpperCase();
					nameLabel.updateWithLanguage(currentLanguage);
				}
				
				
				if(partSlot.selectedPart)
				{
					if(partSlot.selectedPart.name.indexOf("{custom}") >= 0) 
					{
						var text:String = partSlot.selectedPart.name;
						text = text.replace("{custom}",AlertView.language.translate("componentNames", "custom").toUpperCase());
						
						valueLabel.text = text;//currentLanguage.translate("componentNames", "custom").toUpperCase();
					}
					else if(partSlot.selectedPart.name.indexOf("{factory}") >= 0)
					{
						
						var text2:String = partSlot.selectedPart.name;
						text2 = text2.replace("{factory}",AlertView.language.translate("componentNames", "factory").toUpperCase());
						valueLabel.text = text2;//currentLanguage.translate("componentNames", "factory").toUpperCase();
					}
					else 
					{
						valueLabel.text = partSlot.selectedPart.name.toUpperCase();
					}	
				}
				else 
				{
					valueLabel.text = "N/A";
				}
			}
		}
	}
}