package com.despark.tuning3d.view.components.rewards {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.packages.PackagesParser;
    import com.despark.tuning3d.view.components.challenges.controls.ImageLabel;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.utils.FontSelector;

    import feathers.controls.Label;

    import flash.text.TextFormatAlign;

    import starling.animation.Tween;
    import starling.core.Starling;

    public class RewardsCarIdLabel extends RewardsLabel {
        private var _carId:String;
        private var _shineImg:ShineImage;
        private var _shineImg2:ShineImage;
        private var _carLoader:ImageLoaderWithStates;
        private var _car:Car;
        private var _carNameLabel:Label;
        private var _durLabel:ImageLabel;
        public function RewardsCarIdLabel(carId:String) {
            super();
            _rewardType = RewardType.CAR_ID;
            _carId = carId;
            _car = PackagesParser.allCarsList[_carId];
        }
        
        override protected function initialize():void{
            super .initialize();
            _shineImg = new ShineImage(ButtonFactory.assetManager.getTexture("lights_effect_2_lines"));
            _shineImg.isRotating = false;
            _shineImg.setColor(0xf4ac04);
            _shineImg.dir = -1.9;
            _shineImg.scale = ButtonFactory.assetManager.scaleFactor / 2;
            _shineImg.colorChangingInterval = 10;
            addChild(_shineImg);
            _shineImg.y = 25;

            _shineImg2 = new ShineImage(ButtonFactory.assetManager.getTexture("lights_effect_2"));
            _shineImg2.isRotating = true;
            _shineImg2.rotationSpeed = Math.PI / 360;
            _shineImg2.setColor(0xf4ac04);
            _shineImg2.dir = 1.5;
            _shineImg2.scale = 1.25 * ButtonFactory.assetManager.scaleFactor / 2;
            _shineImg2.colorChangingInterval = 10;
            addChild(_shineImg2);
            _shineImg2.y = 25;

            if(car){
                _carLoader = new ImageLoaderWithStates();
                if(car.imageURL && car.imageURL.indexOf("side.png") != -1){
                    car.imageURL = car.imageURL.replace("side.png", "side_2.png");
                }
                _carLoader.height = 60;
                _carLoader.persistentImageProvider = car;
                _carLoader.completeHandler = carLoadCompleteHandler;
                _carLoader.alpha = 0;
                addChild(_carLoader);

                _carNameLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansBold(), 21, 0xf4ac04, TextFormatAlign.LEFT, false, false);
                addChild(_carNameLabel);
                var carName:String = car.displayName.toUpperCase();
                if(carName.length > 23){
                    carName = carName.substr(0, 22) + "…";
                }
                _carNameLabel.text = carName;
                _carNameLabel.validate();
                _carNameLabel.x = -_carNameLabel.width / 2;
                _carNameLabel.y = -43;
            }

            _durLabel = new ImageLabel();
            _durLabel.color = 0x1399f4;
            _durLabel.text = "@ 24h";
            _durLabel.scale = 0.45 * ButtonFactory.assetManager.scaleFactor / 2;
            addChild(_durLabel);
            _durLabel.x = -_durLabel.width / 2;
            _durLabel.y = 50;
        }

        override public function dispose():void{
            super .dispose();
            if(_shineImg){
                _shineImg.removeFromParent(true);
                _shineImg = null;
            }
            if(_carLoader){
                _carLoader.removeFromParent(true);
                _carLoader = null;
            }
            if(_durLabel){
                _durLabel.removeFromParent(true);
                _durLabel = null;
            }
            if(_carNameLabel){
                _carNameLabel.removeFromParent(true);
                _carNameLabel = null;
            }
        }

        private function carLoadCompleteHandler(loader:ImageLoaderWithStates):void {
            _carLoader.image.color = 0xFF0000;
            _carLoader.width = _carLoader.height * _carLoader.image.width / _carLoader.image.height;
            _carLoader.x = -_carLoader.width / 2;
            _carLoader.y = _shineImg.y -_carLoader.height / 2 - 10;
            var tween:Tween = new Tween(_carLoader, 1);
            tween.fadeTo(1);
            Starling.juggler.add(tween);
        }

        public function get car():Car {
            return _car;
        }

        public function set car(value:Car):void {
            _car = value;
        }
    }
}
