package com.despark.tuning3d.view.components.rewards {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.rewards.DailyRewardsData;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.utils.FontSelector;
    import com.utils.TextureUtil;

    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;

    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import starling.animation.Tween;
    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.filters.ColorMatrixFilter;

    public class RewardsListItem extends LayoutGroup {
        public static const STATUS_COMPLETE:String = "f";
        public static const STATUS_CURRENT:String = "c";
        public static const STATUS_INACTIVE:String = "i";

        public static const CLAIM_REWARDS:String = "claimRewards";

        private var _data:DailyRewardsData;
        private var _status:String;
        private var _borderImage:Image;
        private var _borderActiveImage:Image;
        private var _iconImage:Image;
        private var _iconBottomOffset:Number = 0;
        private var _indexLabel:Label;
        private var _infoLabel:Label;
        private var _shineImg:ShineImage;
        private var _car:Car;
        private var _carLoader:ImageLoaderWithStates;
        private var innerTween:Tween;


        public static const ITEM_WIDTH:Number = 37;
        public static const ITEM_HEIGHT:Number = 42;
        public function RewardsListItem() {
            super();
            _status = STATUS_INACTIVE;
            touchGroup = true;
            addEventListener(TouchEvent.TOUCH, touchEventHandler);
        }

        private function touchEventHandler(event:TouchEvent):void {
            if(_status == STATUS_CURRENT && event.getTouch(this, TouchPhase.ENDED)){
                dispatchEventWith(CLAIM_REWARDS, false, _data);
            }
        }

        override protected function initialize():void {
            super .initialize();
            _borderImage = new Image(ButtonFactory.assetManager.getTexture("reward_list_item_border"));
            _borderImage.texture = TextureUtil.scaleTexture(_borderImage.texture, 0.5);
            _borderImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
            _borderImage.scale9Grid = new Rectangle(_borderImage.texture.frameWidth / 2 - 0.5,_borderImage.texture.frameHeight / 2 - 0.5, 1, 1);
            _borderImage.width = ITEM_WIDTH;
            _borderImage.height = ITEM_HEIGHT;
            updateStatus();
        }

        public function updateStatus():void{
            if(_indexLabel){
                _indexLabel.removeFromParent(true);
                _indexLabel = null;
            }
            if(_iconImage){
                _iconImage.removeFromParent(true);
                _iconImage = null;
            }
            if(_infoLabel){
                _infoLabel.removeFromParent(true);
                _infoLabel = null;
            }
            if(_carLoader){
                _carLoader.removeFromParent(true);
                _carLoader = null;
            }
            if(_shineImg){
                _shineImg.removeFromParent(true);
                _shineImg = null;
            }
            if(_borderActiveImage){
                _borderActiveImage.removeFromParent(true);
                _borderActiveImage = null;
            }
            if(innerTween){
                Starling.juggler.remove(innerTween);
                innerTween = null;
            }
            switch (_status) {
                case STATUS_COMPLETE:
                    _borderImage.filter = null;
                    _borderImage.color = 0x19d300;
                    _borderImage.alpha = 1;

                    _indexLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 9, 0x19d300, TextFormatAlign.CENTER, false);
                    _iconImage = new Image(ButtonFactory.assetManager.getTexture("reward_check_sign"));
                    _iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
                    _iconImage.color = 0x19d300;
                    break;
                case STATUS_CURRENT:
                    _borderImage.filter = null;
                    _borderImage.color = 0xFEFFFF;
                    _borderImage.alpha = 1;
                    _indexLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 9, 0xFFFFFF, TextFormatAlign.CENTER, false);
                    _shineImg = new ShineImage(ButtonFactory.assetManager.getTexture("lights_effect"));
                    _shineImg.setColor(0x1189fb);
                    _shineImg.dir = 1;
                    _shineImg.width = 40;
                    _shineImg.scale = _shineImg.scaleX;
                    _shineImg.rotationSpeed = Math.PI / 360;
                    _shineImg.rotation = 2 * Math.PI * Math.random();
                    addChild(_shineImg);
                    createResourceIcon();

                    _borderActiveImage = new Image(_borderImage.texture);
                    _borderActiveImage.scale9Grid = new Rectangle(_borderActiveImage.texture.frameWidth / 2 - 0.5,_borderActiveImage.texture.frameHeight / 2 - 0.5, 1, 1);

                    _borderActiveImage.pivotX = _borderActiveImage.width / 2;
                    _borderActiveImage.pivotY = _borderActiveImage.height / 2;

                    _borderActiveImage.width = ITEM_WIDTH + 5;
                    _borderActiveImage.height = ITEM_HEIGHT + 5;

                    _borderActiveImage.x = ITEM_WIDTH / 2;
                    _borderActiveImage.y = ITEM_HEIGHT / 2;

                    innerTween = new Tween(_borderActiveImage, 3);
                    innerTween.repeatCount = 0;
                    innerTween.onUpdate = function():void{
                        var pr:Number = innerTween.currentTime / innerTween.totalTime;
                        if(pr < 0.25){
                            _borderActiveImage.alpha = 4 * pr;
                        }else if(pr < 0.5){
                            _borderActiveImage.alpha = 1 - 4 * (pr - 0.25);
                        }else if(pr < 0.75){
                            _borderActiveImage.alpha = 4 * (pr - 0.5);
                        }else{
                            _borderActiveImage.alpha = 1 - 4 * (pr - 0.75);
                        }
                        _borderImage.alpha = 1 - _borderActiveImage.alpha;
                    }
                    Starling.juggler.add(innerTween);
                    break;
                case STATUS_INACTIVE:
                    _borderImage.color = 0xFEFFFF;
                    _borderImage.alpha = 1;
                    _indexLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 9, 0xffb400, TextFormatAlign.CENTER, false);
                    createResourceIcon();
                    _borderImage.filter = disabledFilter();
                    _indexLabel.filter = disabledFilter();
                    _infoLabel.filter = disabledFilter();
                    if(_iconImage){
                        _iconImage.filter = disabledFilter();
                    }else if(_carLoader){
                        _carLoader.filter = disabledFilter();
                    }
                    break;

            }

            addChild(_borderImage);
            addChild(_indexLabel);
            if(_iconImage){
                addChild(_iconImage);
            }
            if(_data){
                _indexLabel.text = _data.day.toString();
            }

            if(_borderActiveImage){
                addChild(_borderActiveImage);
            }
        }

        private static function disabledFilter():ColorMatrixFilter{
            var colorFilter:ColorMatrixFilter = new ColorMatrixFilter();
            colorFilter.tint(0x000000, 0.5);
            return colorFilter;
        }

        private function createResourceIcon():void{
            var labelColor:uint = (_status == STATUS_CURRENT? 0xffb400: 0xFFFFFF);
            _infoLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 9, labelColor, TextFormatAlign.CENTER, false);
            addChild(_infoLabel);
            if(data.diamonds && data.keys == 0){
                _iconImage = new Image(ButtonFactory.assetManager.getTextureAtlas("resource_icons").getTexture("collect_diamond_big.png"));
                _iconImage.width = 25;
                _infoLabel.text = "x" + data.diamonds.toString();
                _iconBottomOffset = -3;
                _iconImage.scale = _iconImage.scaleX;
            }else if(data.diamonds == 0 && data.keys){
                _iconImage = new Image(ButtonFactory.assetManager.getTextureAtlas("resource_icons").getTexture("collect_key_big.png"));
                _iconImage.width = 25;
                _infoLabel.text = "x" + data.keys.toString();
                _iconBottomOffset = -3;
                _iconImage.scale = _iconImage.scaleX;
            }else if(data.diamonds && data.keys){
                _iconImage = new Image(ButtonFactory.assetManager.getTextureAtlas("resource_icons").getTexture("collect_diamond_key_big.png"));
                _iconImage.width = 25;
                _infoLabel.text = "x" + data.diamonds.toString() + "/x" + data.keys.toString();
                _iconBottomOffset = -3;
                _iconImage.scale = _iconImage.scaleX;
            }else if(data.carId){
                if(_shineImg){
                    _shineImg.setColor(0xf4ac04);
                }
                _infoLabel.text = "24h";
                if(car){
                    _carLoader = new ImageLoaderWithStates();
                    if(car.imageURL && car.imageURL.indexOf("side.png") != -1){
                        car.imageURL = car.imageURL.replace("side.png", "side_2.png");
                    }
                    _carLoader.height = 2 * _borderImage.height / 3;
                    _carLoader.persistentImageProvider = car;
                    _carLoader.completeHandler = carLoadCompleteHandler;
                    _carLoader.visible = false;
                    addChild(_carLoader);

                    var _contentMask:Quad = new Quad(1, 1);
                    _contentMask.width = _borderImage.width;
                    _contentMask.height = _borderImage.height;
                    addChild(_contentMask);
                    _carLoader.mask = _contentMask;
                }
            }else if(data.carType){
                if(data.carType == "ALL"){
                    _iconImage = new Image(ButtonFactory.assetManager.getTextureAtlas("resource_icons").getTexture("all_cars.png"));
                }else{
                    _iconImage = new Image(ButtonFactory.assetManager.getTextureAtlas("resource_icons").getTexture("all_" + data.carType.toLowerCase() + ".png"));
                }
                _iconImage.height = 16;
                _iconImage.scale = _iconImage.scaleY;
                _infoLabel.text = "24h";
                _iconBottomOffset = -3;
            }else if(data.rewardsMultiplier){
                _iconImage = new Image(ButtonFactory.assetManager.getTextureAtlas("resource_icons").getTexture("x2.png"));
                _iconImage.width = 22;
                _iconImage.scale = _iconImage.scaleX;
                _infoLabel.text = "24h";
                _iconBottomOffset = -2;
            }
        }

        private function carLoadCompleteHandler(loader:ImageLoaderWithStates):void {
            loader.visible = true;
        }

        override protected function draw():void{
            super .draw();
            if(_indexLabel){
                _indexLabel.validate();
                _indexLabel.x = _borderImage.width - _indexLabel.width - 2;
                _indexLabel.y = -2;
            }

            if(_iconImage){
                _iconImage.x = (_borderImage.width - _iconImage.width) / 2;
                _iconImage.y = (_borderImage.height - _iconImage.height) / 2 + _iconBottomOffset;
                if(_shineImg){
                    _shineImg.x = _iconImage.x + _iconImage.width / 2;
                    _shineImg.y = _iconImage.y + _iconImage.height / 2;
                }
            }else if(_carLoader){
                _carLoader.y = 5;
                _carLoader.x = 5;
                if(_shineImg){
                    _shineImg.x = _borderImage.width / 2;
                    _shineImg.y = _borderImage.height / 2;
                }
            }


            if(_infoLabel){
                _infoLabel.x = (_borderImage.width - _infoLabel.width) / 2;
                _infoLabel.y = (_borderImage.height - _infoLabel.height) - 1;
            }
        }

        override public function dispose():void{
            super .dispose();
            if(_indexLabel){
                _indexLabel.removeFromParent(true);
                _indexLabel = null;
            }
            if(_iconImage){
                _iconImage.removeFromParent(true);
                _iconImage = null;
            }
            if(_infoLabel){
                _infoLabel.removeFromParent(true);
                _infoLabel = null;
            }
            if(_carLoader){
                _carLoader.removeFromParent(true);
                _carLoader = null;
            }
            if(_shineImg){
                _shineImg.removeFromParent(true);
                _shineImg = null;
            }
            if(_borderActiveImage){
                _borderActiveImage.removeFromParent(true);
                _borderActiveImage = null;
            }
            if(innerTween){
                Starling.juggler.remove(innerTween);
                innerTween = null;
            }
        }

        public function get data():DailyRewardsData {
            return _data;
        }

        public function set data(value:DailyRewardsData):void {
            _data = value;
            if(_data && _indexLabel){
                _indexLabel.text = _data.day.toString();
            }
        }

        public function get status():String {
            return _status;
        }

        public function set status(value:String):void {
            _status = value;
        }

        public function get car():Car {
            return _car;
        }

        public function set car(value:Car):void {
            _car = value;
        }
    }
}
