package com.despark.tuning3d.view.components.rewards {
    import com.despark.tuning3d.view.components.challenges.controls.ImageLabel;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ColorUtil;

    import flash.events.TimerEvent;

    import flash.utils.Timer;

    import starling.animation.Tween;
    import starling.core.Starling;

    import starling.display.BlendMode;

    import starling.display.Image;

    public class RewardsResourceLabel extends RewardsLabel {
        protected var _numField:ImageLabel;
        protected var _rewardIcon:Image;
        protected var _rewardValue:int;
        protected var _shineImg:ShineImage;
        protected var _labelTimer:Timer;

        protected var _currentValue:int = 0;
        protected var _color:int = 0;
        private var _numDefaultScale:Number;
        public function RewardsResourceLabel(value:int = 0) {
            super();
            _rewardValue = value;
        }
        
        override protected function initialize():void{
            super .initialize();
            _shineImg = new ShineImage(ButtonFactory.assetManager.getTexture("lights_effect"));
            _shineImg.setColor(0x1189fb);
            _shineImg.scale = 0.6 * ButtonFactory.assetManager.scaleFactor / 2;
            _shineImg.rotation = 20 * Math.PI / 180;
            addChild(_shineImg);
            //
            _numField = new ImageLabel();
            addChild(_numField);
            _numField.text = "0";
            _numField.validate();
            _numDefaultScale = _numField.scale = ButtonFactory.assetManager.scaleFactor / 2;
            _numField.visible = false;
        }

        override protected function draw():void{
            super .draw();
            _numField.x = (actualWidth - _numField.width) / 2;
            _numField.y = _rewardIcon.height / 2;
        }

        override public function startLabelAnimation():void{
            super.startLabelAnimation();
            var repeatCount:int = 10;
            _numField.visible = true;
            _labelTimer = new Timer(125, repeatCount);
            _labelTimer.addEventListener(TimerEvent.TIMER, labelTimerHandler);
            _labelTimer.addEventListener(TimerEvent.TIMER_COMPLETE, labelTimerCompleteHandler);
            if(_rewardValue == 0){
                labelTimerHandler(null);
                return;
            }
            _labelTimer.start();
        }

        private function labelTimerHandler(event:TimerEvent):void{
            _currentValue += Math.ceil(_rewardValue / _labelTimer.repeatCount);
            if(_currentValue > _rewardValue || _labelTimer.currentCount == _labelTimer.repeatCount){
                _currentValue = _rewardValue;
            }
            _numField.text = _currentValue.toString();
            _numField.validate();
            _numField.scale = _numDefaultScale * 1.25;
            _numField.x = (actualWidth - _numField.width) / 2;
            Starling.juggler.removeTweens(_numField);
            var tween:Tween = new Tween(_numField, 0.125);
            tween.scaleTo(_numDefaultScale);
            tween.onUpdate = function():void{
                if(_numField){
                    _numField.x = -_numField.width / 2;
                }
            }
            Starling.juggler.add(tween);
            //
            if(_currentValue == _rewardValue){
                labelTimerCompleteHandler(null);
            }
        }

        private function labelTimerCompleteHandler(event:TimerEvent):void{
            _labelTimer.stop();
            _labelTimer.removeEventListener(TimerEvent.TIMER, labelTimerHandler);
            _labelTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, labelTimerCompleteHandler);
            _labelTimer = null;
        }

        override public function dispose():void{
            super .dispose();

            if(_labelTimer){
                _labelTimer.stop();
                _labelTimer.removeEventListener(TimerEvent.TIMER, labelTimerHandler);
                _labelTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, labelTimerCompleteHandler);
                _labelTimer = null;
            }

            if(_numField){
                Starling.juggler.removeTweens(_numField);
                _numField.removeFromParent(true);
                _numField = null;
            }
            if(_rewardIcon){
                _rewardIcon.removeFromParent(true);
                _rewardIcon = null;
            }
            if(_shineImg){
                _shineImg.removeFromParent(true);
                _shineImg = null;
            }
        }

        public function get color():int {
            return _color;
        }

        public function get rewardValue():int {
            return _rewardValue;
        }
    }
}
