package com.despark.tuning3d.view.components.rewards {
    import com.despark.tuning3d.view.components.core.ColorUtil;

    import flash.events.TimerEvent;

    import flash.utils.Timer;

    import starling.display.BlendMode;
    import starling.display.Image;
    import starling.events.Event;
    import starling.textures.Texture;

    public class ShineImage extends Image {
        protected var _animationTimer:Timer;
        protected var _defaultColor:Object;
        protected var _hue:Number;
        private var _dir:Number;
        private var _colorChangingInterval:int = 10;
        private var _isRotating:Boolean = true;
        private var _rotationSpeed:Number = Math.PI / 720;

        public function ShineImage(texture:Texture) {
            super(texture);
            pivotX = width / 2;
            pivotY = height / 2;
            blendMode = BlendMode.SCREEN;

            _animationTimer = new Timer(60);
            _animationTimer.addEventListener(TimerEvent.TIMER, timerHandler);

            _hue = Math.random() * 20 - 10;
            _dir = 2;
            addEventListener(Event.ADDED_TO_STAGE, onAddedToStage)
            addEventListener(Event.REMOVED_FROM_STAGE, onRemovedFromStage);
        }

        public function setColor(value:uint):void{
            color = value;
            _defaultColor = ColorUtil.HexToHSV(color);
        }

        private function onRemovedFromStage(event:Event):void {
            _animationTimer.stop();
        }

        private function onAddedToStage(event:Event):void {
            _animationTimer.start();
        }

        override public function dispose():void{
            super .dispose();
            if(_animationTimer){
                _animationTimer.stop();
                _animationTimer.removeEventListener(TimerEvent.TIMER, timerHandler);
                _animationTimer = null;
            }
            _defaultColor = null;
        }

        private function timerHandler(event:TimerEvent):void{
            if(!stage){
                dispose();
                return;
            }
            color = ColorUtil.HSVtoHEX(_defaultColor.h + _hue, _defaultColor.s, _defaultColor.v);
            _hue += _dir;
            if(_hue >= _colorChangingInterval){
                _dir = -Math.abs(_dir);
            }else if(_hue <= -_colorChangingInterval){
                _dir = Math.abs(_dir);
            }

            if(_isRotating){
                rotation += _rotationSpeed;
            }
        }

        public function set colorChangingInterval(value:int):void {
            _colorChangingInterval = value;
        }

        public function set dir(value:Number):void {
            _dir = value;
        }

        public function set isRotating(value:Boolean):void {
            _isRotating = value;
        }

        public function set rotationSpeed(value:Number):void {
            _rotationSpeed = value;
        }

        public function get rotationSpeed():Number {
            return _rotationSpeed;
        }
    }
}
