package com.despark.tuning3d.view.components.timeline.ItemRenderers.Abstract {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.brands.BrandsScreen;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.LabelFactory;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.TimelineScreen;
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.utils.DateParser;

import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.core.FeathersControl;
import feathers.layout.AnchorLayout;
import feathers.layout.AnchorLayoutData;
import feathers.layout.FlowLayout;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.HorizontalLayoutData;
import feathers.layout.HorizontalSpinnerLayout;
import feathers.layout.TiledColumnsLayout;
import feathers.layout.TiledRowsLayout;
import feathers.layout.VerticalAlign;
import feathers.layout.VerticalLayout;
import feathers.layout.VerticalLayoutData;

import flash.geom.Point;
import flash.geom.Rectangle;

import flash.utils.Dictionary;
import flash.utils.clearTimeout;
import flash.utils.setTimeout;

import org.as3commons.lang.Assert;

import starling.core.Starling;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class
ATimelineBaseItemRenderer extends ListItemRenderer{
    public static var iconsMap : Dictionary;
    public static var nameMap: Dictionary;
    protected static var tracer : Tracer;

    public var itemWidth : int;

    protected var _gap:int;
    protected var _bigGap:int;
    protected var _avatarXpos:int;
    protected var _avatarYpos:int;
    protected var _avatarSize:int;

    protected var _bottomLineQuad:Quad;
    protected var _bgQuad:Quad;
    protected var _topContainer :LayoutGroup;
    protected var _feedTitleGroup :LayoutGroup;
    protected var _dateGroup :LayoutGroup;
    protected var _dateIconGroup :LayoutGroup;
    protected var _mainContainer :LayoutGroup;
    protected var _dataContainer :LayoutGroup;
    protected var _stageGroup :LayoutGroup;
    protected var _bottomContainer :LayoutGroup;
    protected var _border:Image;
    protected var _infoContainer :LayoutGroup;
    protected var _nameLabel :FeathersControl;
    protected var _friendNameLabel :FeathersControl;
    protected var _dateLabel :Label;
    protected var _descriptionLabel :FeathersControl;

    protected var _postIcon :Image;

    protected var _eventDispatcher : *;

    public static const MIN_MARGIN:int = 20;
    public static const INFO_WIDTH:int = 50;
    public static const MAX_WIDTH:int = 540;
    public static var CONTENT_DATA_WIDTH:int;
    private var _invalidateTimeout:int;
    /**
     * Abstract base class for feed item renderer
     */
    public function ATimelineBaseItemRenderer() {
        if(!nameMap) {
            tracer = new Tracer("TimelineItemRenderer");
            nameMap = new Dictionary();
            nameMap[FeedData.TYPE_TEXT] = "post_icon";
            nameMap[FeedData.TYPE_NEWBORN] = "post_icon";
            nameMap[FeedData.TYPE_TUNING_LIKE] = "post_like_icon";
            nameMap[FeedData.TYPE_NEW_FRIEND] = "post_friend_icon";
            nameMap[FeedData.TYPE_NEW_TUNING] = "post_tuning_icon";
            nameMap[FeedData.TYPE_TUNING_COMMENT] = "post_icon";
            // challenges
            nameMap[FeedData.TYPE_CHALLENGE_ANSWER] = "post_challenge_accept_icon";
            nameMap[FeedData.TYPE_CHALLENGE_WIN] = "post_challenge_win_icon";

            iconsMap = new Dictionary();
            iconsMap[FeedData.TYPE_NEWBORN] = AlertView.assetManager.getTexture(nameMap[FeedData.TYPE_NEWBORN]);
            iconsMap[FeedData.TYPE_TEXT] = AlertView.assetManager.getTexture(nameMap[FeedData.TYPE_TEXT]);
            iconsMap[FeedData.TYPE_TUNING_LIKE] = AlertView.assetManager.getTexture(nameMap[FeedData.TYPE_TUNING_LIKE]);
            iconsMap[FeedData.TYPE_NEW_FRIEND] = AlertView.assetManager.getTexture(nameMap[FeedData.TYPE_NEW_FRIEND]);
            iconsMap[FeedData.TYPE_NEW_TUNING] = AlertView.assetManager.getTexture(nameMap[FeedData.TYPE_NEW_TUNING]);
            iconsMap[FeedData.TYPE_TUNING_COMMENT] = AlertView.assetManager.getTexture(nameMap[FeedData.TYPE_TUNING_COMMENT]);

            iconsMap[FeedData.TYPE_CHALLENGE_ANSWER] = AlertView.assetManager.getTexture(nameMap[FeedData.TYPE_CHALLENGE_ANSWER]);
            iconsMap[FeedData.TYPE_CHALLENGE_WIN] = AlertView.assetManager.getTexture(nameMap[FeedData.TYPE_CHALLENGE_WIN]);
        }
    }

    public function set eventDispatcher(value:*) : void{
        _eventDispatcher = value;
    }

    public function get eventDispatcher() : *{
        return _eventDispatcher;
    }

    public function get feedData():FeedData {
        return _data as FeedData;
    }

    protected override function initialize():void {
        super.initialize();
        initAllComponents();
    }

    protected function initAllComponents():void {
        _gap = 10;
        _bigGap = 18;
        _avatarXpos = Math.max((itemWidth - MAX_WIDTH) / 2, MIN_MARGIN); // 20 // max width 640
        CONTENT_DATA_WIDTH ||= Math.min(MAX_WIDTH, itemWidth - 4 * _gap) - 2 * INFO_WIDTH;//int(Math.min(MAX_WIDTH, itemWidth - 2 * MIN_MARGIN) - (INFO_WIDTH + 2 * _gap));
        _avatarYpos = 0;
        _avatarSize = 30;

        _bgQuad = new Quad(1,1,0xffffff);
        addChild(_bgQuad);

        var stageLayout : VerticalLayout = new VerticalLayout();
        stageLayout.gap = 0;
        stageLayout.padding = _gap;
        stageLayout.paddingLeft = (_gap - INFO_WIDTH) / 2 + _gap;
        stageLayout.paddingBottom = _bigGap;

        _stageGroup = new LayoutGroup();
        _stageGroup.layout = stageLayout;
        addChild(_stageGroup);

        var topContainerLayout : HorizontalLayout = new HorizontalLayout();
        topContainerLayout.gap = _gap;
        topContainerLayout.paddingLeft = _avatarXpos;
        topContainerLayout.horizontalAlign = HorizontalAlign.LEFT;

        _topContainer = new LayoutGroup();
        _topContainer.layout = topContainerLayout;

        var mainLayout:VerticalLayout = new VerticalLayout();
        mainLayout.gap = 5;
        mainLayout.paddingTop = 0;

        _mainContainer = new LayoutGroup();
        _mainContainer.layout = mainLayout;

        _dataContainer = new LayoutGroup();
        var dataContainerLayout:VerticalLayout = new VerticalLayout();
        dataContainerLayout.paddingTop = _bigGap;


        var titleLayout:FlowLayout = new FlowLayout();
        titleLayout.gap = 5;
        titleLayout.padding = 0;
        titleLayout.paddingTop = 3;
        titleLayout.paddingBottom = 8;
        titleLayout.rowVerticalAlign = VerticalAlign.MIDDLE;
        _feedTitleGroup  = new LayoutGroup();
        _feedTitleGroup.layout = titleLayout;

        var dateLayout : HorizontalLayout = new HorizontalLayout();
        dateLayout.gap = 5;
        dateLayout = new HorizontalLayout();
        dateLayout.paddingTop = 2;
        dateLayout.verticalAlign = VerticalAlign.MIDDLE;
        _dateGroup = new LayoutGroup();
        _dateGroup.layout = dateLayout;

        var group2Layout:VerticalLayout = new VerticalLayout();
        group2Layout.horizontalAlign = HorizontalAlign.CENTER;
        _dateIconGroup = new LayoutGroup();
        _dateIconGroup.layout = group2Layout;
        _dateIconGroup.width = NaN;
        _dateIconGroup.addChild(_dateGroup);

        _infoContainer = new LayoutGroup();
        _infoContainer.width = INFO_WIDTH;

        _stageGroup.addChild(_mainContainer);
        _stageGroup.addChild(_topContainer);

        _mainContainer.addChild(_feedTitleGroup);
        _mainContainer.addChild(_dataContainer);

        _infoContainer.addChild(_dateIconGroup);

        _topContainer.addChild(_infoContainer);
        _topContainer.addChild(_mainContainer);

        _bottomLineQuad = new Quad(1,2,0x939090);

        addChild(_bottomLineQuad);

        initIcon();
        initLabels();
    }

    protected function initLabels():void {
        if(Constants.isIOS) {
            _nameLabel = LabelFactory.usernameLabelLargeIos() as FeathersControl;
        }else{
            _nameLabel = new Label();
            Label(_nameLabel).textRendererFactory = LabelFactory.nameLabelRendererLargeAndroid;
            Label(_nameLabel).wordWrap = true;
        }
        _nameLabel = LabelFactory.createLabelLarge();
        _nameLabel.addEventListener(TouchEvent.TOUCH,onCreatorGarageSelectHandler);
        _feedTitleGroup.addChild(_nameLabel);

        _descriptionLabel = LabelFactory.createLabelTimelineDesc();
        _feedTitleGroup.addChild(_descriptionLabel);


        _friendNameLabel = LabelFactory.createLabelLarge();
        _friendNameLabel.addEventListener(TouchEvent.TOUCH,onFriendGarageSelectHandler);
        _feedTitleGroup.addChild(_friendNameLabel);

        /*var _dateIcon : Image = new Image(AlertView.assetManager.getTexture("dateIcon"));
        _dateIcon.width = _dateIcon.height = 11;
        _dateGroup.addChild(_dateIcon);*/

        _dateLabel = new Label();
        _dateLabel.textRendererFactory = TimelineLabelFactory.postDateLabelRenderer;
        _dateLabel.wordWrap = true;
        _dateLabel.width = _avatarSize + _gap / 2;
        _dateGroup.addChild(_dateLabel);
    }

    protected function onCreatorGarageSelectHandler(event:TouchEvent) : void {
        var touch : Touch = event.touches[0] as Touch;
        if (touch.phase == TouchPhase.ENDED && !TimelineScreen.SCROLL_FLAG){
            Constants.selectedUser =  feedData.creator;
            checkForUserAndDispatch();
        }
    }

    protected function onFriendGarageSelectHandler(event:TouchEvent) : void {
        var touch : Touch = event.touches[0] as Touch;
        if (touch.phase == TouchPhase.ENDED && !TimelineScreen.SCROLL_FLAG){
            var tmpUser : User = feedData.feedType == FeedData.TYPE_NEW_FRIEND ?  feedData.newFriend : feedData.owner;
            Constants.selectedUser =  tmpUser;
            checkForUserAndDispatch();
        }
    }

    public function checkForUserAndDispatch() : void {
        if (Constants.selectedUser && Constants.selectedUser.id == (ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy).currentUser.id){
            Constants.selectedUser = null;
        }

        if (eventDispatcher) eventDispatcher.dispatchEventWith(StackedScreen.USER_DETAILS_SELECTED);
    }

    override protected function layout():void {
        super.layout();
        updateSize();
    }

    override protected function autoSizeIfNeeded():Boolean{
        var needsWidth : Boolean = isNaN(explicitWidth);
        var needsHeight : Boolean = true;//isNaN(explicitHeight);

        if(!needsWidth && !needsHeight) {
            return false;
        }

        _topContainer.height = NaN;
        _topContainer.validate();

        _mainContainer.height = NaN;
        _mainContainer.validate();

        _stageGroup.width = stage.width;
        _stageGroup.height = NaN;
        _stageGroup.validate();


        _stageGroup.width = stage.width;
        _stageGroup.height = NaN;
        _stageGroup.validate();

        var newWidth : Number = _stageGroup.width;//explicitWidth;
        var newHeight : Number = _stageGroup.height;//measureHeight();//explicitHeight;
        var dh:Number = 0;
        var autoSize:Boolean = setSizeInternal(newWidth, newHeight+dh, false);

        if(autoSize) {
            updateSize();
        }
        return autoSize;
    }

    protected function updateSize():void{
        _bottomLineQuad.y = actualHeight - _bottomLineQuad.height;
        _bottomLineQuad.width = actualWidth;

        _bgQuad.width = actualWidth;
        _bgQuad.height = actualHeight;
        _feedTitleGroup.width = contentWidth;

        if(_dateLabel.width == _avatarSize + _gap / 2 && _dateLabel.text.length && _dateLabel.text.indexOf(' ')==-1){
            _dateLabel.width = _avatarSize + _bigGap;
        }
    }

    protected function update():void {
        Assert.isTrue(false, "ATimelineCommentLikeItemRenderer");
    }


    override protected function commitData():void {
        super.commitData();
        update();
    }

    protected function customAutosize():void {
        this.invalidate(FeathersControl.INVALIDATION_FLAG_SIZE);
    }

    protected function initIcon() : void {
        Assert.isTrue(false); // need to override this function
    }

    protected function layoutIcon():void{
        _postIcon.height= _avatarSize;
        _postIcon.scale = _postIcon.scaleY;
        _postIcon.scale9Grid = new Rectangle(0,0, _postIcon.texture.frameWidth * (1 - _avatarSize / _postIcon.width), _postIcon.texture.frameHeight);
        _postIcon.width = _avatarSize;
        // _topContainer.addChildAt(_postIcon,0);
        _dateIconGroup.addChildAt(_postIcon,0);
    }

    override public function dispose():void {
        super.dispose();
        if(_bottomLineQuad){
            _bottomLineQuad.removeFromParent(true);
            _bottomLineQuad = null;
        }

        if(_bgQuad){
            _bgQuad.removeFromParent(true);
            _bgQuad = null;
        }

        if(_topContainer){
            _topContainer.removeFromParent(true);
            _topContainer = null;
        }

        if(_feedTitleGroup){
            _feedTitleGroup.removeFromParent(true);
            _feedTitleGroup = null;
        }

        if(_dateGroup){
            _dateGroup.removeFromParent(true);
            _dateGroup = null;
        }

        if(_dateIconGroup){
            _dateIconGroup.removeFromParent(true);
            _dateIconGroup = null;
        }

        if(_mainContainer){
            _mainContainer.removeFromParent(true);
            _mainContainer = null;
        }

        if(_dataContainer){
            _dataContainer.removeFromParent(true);
            _dataContainer = null;
        }

        if(_stageGroup){
            _stageGroup.removeFromParent(true);
            _stageGroup = null;
        }

        if(_bottomContainer){
            _bottomContainer.removeFromParent(true);
            _bottomContainer = null;
        }

        if(_border){
            _border.removeFromParent(true);
            _border = null;
        }

        if(_infoContainer){
            _infoContainer.removeFromParent(true);
            _infoContainer = null;
        }

        if(_nameLabel){
            _nameLabel.removeFromParent(true);
            _nameLabel = null;
        }

        if(_friendNameLabel){
            _friendNameLabel.removeFromParent(true);
            _friendNameLabel = null;
        }

        if(_dateLabel){
            _dateLabel.removeFromParent(true);
            _dateLabel = null;
        }

        if(_descriptionLabel){
            _descriptionLabel.removeFromParent(true);
            _descriptionLabel = null;
        }

        if(_postIcon){
            _postIcon.removeFromParent(true);
            _postIcon = null;
        }
    }

    protected function get contentWidth(): Number{
        return CONTENT_DATA_WIDTH;
    }

    protected function get tuningImageY(): Number {
        return _feedTitleGroup.height;
    }

    public function invalidateItem():void{
        if(_invalidateTimeout != -1){
            clearTimeout(_invalidateTimeout);
        }
        _invalidateTimeout = setTimeout(function():void{
            height = NaN;
            invalidate(INVALIDATION_FLAG_SIZE);
            _invalidateTimeout = -1;
        }, 10);
    }

    public function invalidateDataContainer():void{
        _mainContainer.height = _dataContainer.height = _infoContainer.height = NaN;
    }
}
}
