package com.despark.tuning3d.view.components.timeline.ItemRenderers {
import com.despark.tuning3d.view.components.core.LabelFactory;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Abstract.ATimelineCommentLikeItemRender;
import com.despark.tuning3d.view.components.timeline.TimelineScreen;
import com.despark.tuning3d.view.components.timeline.components.TimelineFeedImageHolder;
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.utils.DateParser;
import com.utils.DisplayObjectUtil;
import com.utils.ObjectUtils;

import feathers.controls.Label;
import feathers.core.FeathersControl;

import starling.display.Image;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class FeedNewTuningItemRenderer extends ATimelineCommentLikeItemRender{

    protected var _carNameLabel : FeathersControl;
    protected var _carImageContainer : TimelineFeedImageHolder;

    public function FeedNewTuningItemRenderer() {
        super();
    }

    protected override function initAllComponents():void {
        super.initAllComponents();
        _carImageContainer ||= new TimelineFeedImageHolder();
        _dataContainer.addChildAt(_carImageContainer, 0);
        _dataContainer.addEventListener(TouchEvent.TOUCH, onTuningImageTouchHandler);
    }

    private function onTuningImageTouchHandler(event:TouchEvent):void {
        if(_carImageContainer && _carImageContainer.parent) {
            var touch : Touch = event.getTouch(_carImageContainer, TouchPhase.ENDED);
            if (touch && !this.owner.isScrolling) {
                this.owner.dispatchEventWith(FeedData.TUNING_CAR_IMAGE_SELECTED,false, feedData);
            }
        }
    }


    protected override function initLabels():void {
        super.initLabels();
        _carNameLabel = LabelFactory.createLabelTimelineCarName();
        //_carNameLabel = new Label();
        //_carNameLabel.textRendererFactory = TimelineLabelFactory.carNameLabelRenderer;
        _carNameLabel.addEventListener(TouchEvent.TOUCH,onCarSelectHandler);
        _feedTitleGroup.addChild(_carNameLabel);
    }


    protected override function update():void {
        if(!feedData) {
            return;
        }
        _dataContainer.height = _infoContainer.height = _mainContainer.height = NaN;

        _nameLabel["text"] = feedData.nameLabel;
        _nameLabel.visible = true;

        _descriptionLabel["text"] = feedData.descriptionLabel;
        _descriptionLabel.visible = true;

        _friendNameLabel.visible = false;
        _friendNameLabel.removeFromParent();

        _carNameLabel["text"] = feedData.getCarFullName();
        _carNameLabel.visible = true;

        _dateLabel.text = DateParser.getDateAppropriateText(feedData.availableDate);
        _dateLabel.visible = true;

        _bottomContainer.validate();
        invalidateLikeCommentsViews();
        loadImage();
    }

    public function loadImage():void{
        _bottomContainer.validate();
        _feedTitleGroup.validate();
        setImagePlaceHolder();
        //
        var w : Number = contentWidth;
        var h : Number = stage.stageHeight - tuningImageY;
        feedData.setParent(_carImageContainer, w, h, customAutosize, _bottomContainer.height + _gap);
    }

    private function setImagePlaceHolder():void{
        if(!feedData){
            return;
        }
        var iWidth:Number;
        var iHeight:Number;
        var ratio:Number;
        if(ObjectUtils.hasProperty(feedData.tuning, 'size.width') && ObjectUtils.hasProperty(feedData.tuning, 'size.height')) {
            iWidth = feedData.tuning.size.width;
            iHeight = feedData.tuning.size.height;
        }else{
            iWidth = 1440;
            iHeight = 900;
        }
        ratio = feedData.width / iWidth;
        iWidth *= ratio;
        iHeight *= ratio;
        _carImageContainer.width = iWidth;
        _carImageContainer.height = iHeight;
        // invalidateItem();
        layoutInfoContainer();
    }

    override protected function customAutosize():void {
        _carImageContainer.invalidate(INVALIDATION_FLAG_SIZE);
        _carImageContainer.validate();
        _mainContainer.invalidate(INVALIDATION_FLAG_SIZE);
        _mainContainer.validate();
        super.customAutosize();
    }

    override public function dispose():void {
        _carImageContainer.removeEventListener(TouchEvent.TOUCH, onTuningImageTouchHandler);
        super.dispose();
        DisplayObjectUtil.dispose(_carImageContainer, true, true);
        _carImageContainer.dispose();
        _carImageContainer = null;

        if(_carNameLabel){
            _carNameLabel.removeFromParent(true);
            _carNameLabel = null;
        }
    }

    override protected function draw():void {
        super.draw();
        if(_carImageContainer && _infoContainer){
            var h:Number = _bigGap;
            for(var i:int = 0; i < _infoContainer.numChildren; i++){
                h += _infoContainer.getChildAt(i).height;
            }
            _infoContainer.height = Math.max(_dataContainer.height + _feedTitleGroup.height + 2 * _gap + 2, h);
            if(_socialContainer && _socialContainer.parent == _infoContainer){
                _socialContainer.y = _infoContainer.height - _socialContainer.height - _gap;
            }
        }
    }

    private function onCarSelectHandler(event:TouchEvent) : void {
        var touch : Touch = event.touches[0] as Touch;
        if (touch.phase == TouchPhase.ENDED && !TimelineScreen.SCROLL_FLAG){
            this.owner.dispatchEventWith(FeedData.TUNING_CAR_IMAGE_SELECTED,false, feedData);
        }
    }

    override protected function initIcon() : void{
        _postIcon = new Image(iconsMap[FeedData.TYPE_NEW_TUNING]);
        layoutIcon();
    }
}
}
