package com.despark.tuning3d.view.components.timeline.components {
import com.utils.RegExpUtil;

import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.layout.VerticalLayout;

import starling.display.Image;
import starling.display.Quad;
import starling.textures.Texture;

public class TimelineURLSnippet extends LayoutGroup{
    private var _data:Object;
    private var _descriptionField:Label;
    private var _titleField:Label;
    private var _urlField:Label;
    private var _image:Image;
    private var _imagePlaceHolder:Quad;
    private var _vLayout:VerticalLayout;
    private static const _padding:Number = 5;
    public function TimelineURLSnippet(data:Object) {
        _vLayout = new VerticalLayout();
        _vLayout.paddingTop = _vLayout.paddingBottom = _padding;
        layout = _vLayout;
        //
        _imagePlaceHolder = new Quad(1, 1, 0xFFFFFF);
        _urlField = new Label();
        _titleField = new Label();
        _descriptionField = new Label();
        //
        _imagePlaceHolder.visible = false;
        _titleField.visible = false;
        _descriptionField.visible = false;
        //
        _urlField.paddingLeft = _urlField.paddingRight = _padding;
        _urlField.textRendererFactory = TimelineLabelFactory.snippetDescRenderer;

        _titleField.paddingLeft = _titleField.paddingRight = _padding;
        _titleField.wordWrap = true;
        _titleField.textRendererFactory = TimelineLabelFactory.snippetTitleRenderer;

        _descriptionField.paddingLeft = _descriptionField.paddingRight = _padding;
        _descriptionField.wordWrap = true;
        _descriptionField.textRendererFactory = TimelineLabelFactory.snippetDescRenderer;
        //
        addChild(_imagePlaceHolder);
        addChild(_urlField);
        addChild(_titleField);
        addChild(_descriptionField);
        //
        var bg:Quad = new Quad(1,1,0xEEEEEE);
        backgroundSkin = bg;
        //
        update(data);
    }

    public function update(data:Object):void {
        if(_data == data.meta){
            return;
        }
        _data = data.meta;
        if(_image){
            _image.removeFromParent(true);
            _image = null;
        }
        height = NaN;
        /*if(imageUrl) {
            _vLayout.paddingTop = 0;
            if (_data.hasOwnProperty('size')) {
                _imagePlaceHolder.width = _data.size.width;
                _imagePlaceHolder.height = _data.size.height;
                _imagePlaceHolder.visible = true;
            }
        }else{
            _imagePlaceHolder.visible = false;
        }*/
        //
        _urlField.text = RegExpUtil.getDomain(data.meta.url).toUpperCase();
        if(_data.hasOwnProperty("title")){
            _titleField.text = _data.title;
            _titleField.visible = true;
        }else{
            _titleField.visible = false;
        }

        if(_data.hasOwnProperty("description")){
            _descriptionField.height = NaN;
            _descriptionField.text = _data.description;
            _descriptionField.visible = true;
        }else{
            _descriptionField.visible = false;
        }
    }

    public function get url():String{
        if(_data && _data.url){
            return _data.url;
        }
        return null;
    }


    override public function dispose():void {
        super.dispose();
        if(_descriptionField){
            _descriptionField.removeFromParent(true);
            _descriptionField = null;
        }
        if(_titleField){
            _titleField.removeFromParent(true);
            _titleField = null;
        }
        if(_urlField){
            _urlField.removeFromParent(true);
            _urlField = null;
        }
        if(_image){
            /*_image.texture.dispose();
            _image.texture = null;*/
            _image.removeFromParent(true);
            _image = null;
        }
        if(_imagePlaceHolder){
            _imagePlaceHolder.removeFromParent(true);
            _imagePlaceHolder = null;
        }
    }

    override public function set width(value:Number):void {
        super.width = value;
        if(_urlField){
            _urlField.width = value - 2 * _padding;
        }

        if(_descriptionField){
            _descriptionField.width = value - 2 * _padding;
        }

        if(_titleField){
            _titleField.width = value - 2 * _padding;
        }

        if(_image){
            _image.width = value;
            _image.scale = _image.scaleX;
        }

        if(_imagePlaceHolder && (_imagePlaceHolder.width == 0 || isNaN(_imagePlaceHolder.width))){
            _imagePlaceHolder.width = value;
        }
    }

    public function addImage(texture:Texture):void {
        return;
        //
        if(_imagePlaceHolder){
            _imagePlaceHolder.removeFromParent(true);
            _imagePlaceHolder = null;
        }
        //
        _image = new Image(texture);
        _image.width = width;
        _image.scale = _image.scaleX;
        _data.size = {width: _image.width, height: _image.height};
        addChildAt(_image, 0);
        height = NaN;
        invalidate(INVALIDATION_FLAG_SIZE);
    }

    public function get imageUrl():String{
        if(_data.image && RegExpUtil.isAllowedImageURL(_data.image)){
            return _data.image;
        }
        if(_data.logo && RegExpUtil.isAllowedImageURL(_data.logo)){
            return _data.logo;
        }
        return null;
    }
}
}
