package com.despark.tuning3d.view.components.timeline.components {
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.utils.FontSelector;
    import com.utils.ImageUtils;

    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.FeathersControl;
    import feathers.core.ITextRenderer;
    import feathers.layout.AnchorLayout;
    import feathers.layout.AnchorLayoutData;

    import flash.display.Bitmap;
    import flash.display.Loader;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.net.URLRequest;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;
    import flash.utils.Dictionary;

    import starling.display.Canvas;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.extensions.TextureMaskStyle;
    import starling.filters.BlurFilter;
    import starling.filters.GlowFilter;
    import starling.textures.Texture;
    import starling.utils.Color;

    public class UserCircularAvatar extends FeathersControl {

    private var _user : User;
    protected var _size : int;
    private var _loader:Loader;

    private static var avatarMap : Dictionary = new Dictionary();
    private static const _instanceList:Object = {};
    private var _loadComplete:Function;
    private var _border:Image;
    private var _avatarImage:Image
    private var _avatarImageMask:Image
    private var _borderSize:Number;
    private var _showBorder:Boolean;

    public function UserCircularAvatar(user : User, size : int , showBorder:Boolean = false, borderSize:Number = 0) {
        _user = user;
        _size = size;
        _showBorder = showBorder;
        _borderSize = borderSize;
        loadComplete = null;
        _instanceList[_user.id] ||= new Vector.<UserCircularAvatar>();
        _instanceList[_user.id].push(this);
    }


    private function apply(): void {
        if(_showBorder) {
            _border ||= new Image(ButtonFactory.assetManager.getTexture('avatar_background'));
            _border.width = _border.height = _size;
            _border.color = 0xFEFFFFF;
            addChildAt(_border, 0);
        }else if(_border){
            _border.removeFromParent(true);
            _border = null;
        }

        if(_user){
            if(_user.avatarPath) {
                loadImage(_user.avatarPath);
            } else {
                var defaultAvatar:Sprite = UserCircularAvatar.generateAvatarWithName(_user.username?_user.username:_user.name,_size - 2 * _borderSize,Color.GRAY);
                defaultAvatar.x = defaultAvatar.y = _borderSize;
                addChild(defaultAvatar);
                _loadComplete.call();
            }
        }
    }

    override protected function initialize():void {
        super.initialize();
        this.setSize(_size, _size);
        this.apply();
    }

    private function createImage():void{
        if(_showBorder) {
            _border ||= new Image(ButtonFactory.assetManager.getTexture('avatar_background'));
            _border.width = _border.height = _size;
            _border.color = 0xFEFFFFF;
            addChildAt(_border, 0);
        }else if(_border){
            _border.removeFromParent(true);
            _border = null;
        }

        if(_avatarImage){
            _avatarImage.removeFromParent(true);
            _avatarImage = null;
        }
        _avatarImage = new Image(avatarMap[_user.id]);
        if(_border && _border.parent == this){
            addChildAt(_avatarImage, getChildIndex(_border) + 1);
        }else{
            addChildAt(_avatarImage, 0);
        }
        _avatarImage.width = _size - 2 * _borderSize;
        _avatarImage.height = _size - 2* _borderSize;
        _avatarImage.x = _avatarImage.y = _borderSize;

        if(_user.socialNetwork == UsersProxy.typeOfSN_GS){
            if(!_avatarImageMask){
                _avatarImageMask = new Image(ButtonFactory.assetManager.getTexture('avatar_background'));
                _avatarImageMask.width = _avatarImage.width;
                _avatarImageMask.height = _avatarImage.height;
                _avatarImageMask.x = _avatarImage.x;
                _avatarImageMask.y = _avatarImage.y;
            }
            var style:TextureMaskStyle = new TextureMaskStyle(0.5);
            _avatarImageMask.style = style;

            addChildAt(_avatarImageMask, getChildIndex(_avatarImage) + 1);
            _avatarImage.mask = _avatarImageMask;
        }else if(_avatarImageMask){
            _avatarImageMask.removeFromParent(true);
            _avatarImageMask = null;
        }


        setSize(_size, _size);
    }

    public function loadImage(url:String = null) : void {
        if(avatarMap[_user.id] != null) {
            createImage();
            return;
        }
        if(url.indexOf("http") == - 1) url = "http:" + url;
        if(_loader) {
            _loader.unloadAndStop(false);
            _loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, onLoadComplete);
            _loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
            try{
                _loader.close();
                _loader = null;
            }catch (e:Error) {
                _loader = null;
            }
        }

        _loader = new Loader();
        _loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
        _loader.contentLoaderInfo.addEventListener(Event.COMPLETE, onLoadComplete);
        _loader.load(new URLRequest(url));
    }

    private function onIOErrorHandler(event:IOErrorEvent):void {
        _loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, onLoadComplete);
        _loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
        _loader = null;
        //
        var defaultAvatar:Sprite = UserCircularAvatar.generateAvatarWithName(_user.username?_user.username:_user.name,_size - 2 * _borderSize,Color.GRAY);
        defaultAvatar.x = defaultAvatar.y = _borderSize;
        this.addChild(defaultAvatar);
        _loadComplete.call();
    }

    public function onLoadComplete(event:Event) : void{
        if(event){
            avatarMap[_user.id] = Texture.fromBitmap(ImageUtils.roundCorners(event.currentTarget.loader.content as Bitmap));
        }

        if(_loader){
            _loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, onLoadComplete);
            _loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
            _loader = null;
        }

        createImage();

        _loadComplete.call();
    }

    public static function setIconTexture(userId:String, texture:Texture, dispatchCallback:Boolean = true):void{
        avatarMap[userId] = texture;
        if(dispatchCallback && _instanceList[userId]){
            var instanceList:Vector.<UserCircularAvatar> = _instanceList[userId];
            for(var i:int = 0; i < instanceList.length; i++){
                instanceList[i].clearAll();
                instanceList[i].onLoadComplete(null);
            }
        }
    }

    public function get user() : User{
        return _user;
    }

    public function set user(value:User):void {
        if(value.id == _user.id) {
            return;
        }
        clearAll();
        //
        _user = value;
        apply();
    }

    private static function getColorByname(name: String): uint {
        if (!name || name.length === 0) {
            return Color.GRAY;
        }
        var hashStr: Number = 0;
        for (var i: int = 0; i< name.length; i++) {
             var charCode: Number = name.charCodeAt(i);
            hashStr = ((hashStr << 5) - hashStr) + charCode;
            hashStr |= 0;
        }

        var bg: String = '0000' + Math.abs(hashStr).toString(16);
        bg = bg.substr(bg.length - 6);

        var bint: int = parseInt(bg, 16);
        var r: int = (bint >> 16) & 255;
        var g: int = (bint >> 8) & 255;
        var b: int = bint & 255;
        return rgbToHex(r, g, b);
    }

    private static function rgbToHex(r: int, g: int, b: int): uint {
        var hex:uint = r << 16 | g << 8 | b;
        return hex;
    }

    public static function generateAvatarWithName(name : String, size : int, color : uint = 0xacacac) : Sprite {
        color = getColorByname(name);
        var canvas:Canvas = new Canvas();
        canvas.beginFill(color);
        canvas.drawCircle(size/2, size/2, size/2);
        canvas.endFill();


        var layData : AnchorLayoutData = new AnchorLayoutData();
        layData.verticalCenter = 0;
        layData.horizontalCenter = 0;

        var container : LayoutGroup = new LayoutGroup();
        container.layout = new AnchorLayout();
        container.addChild(new Quad(size, size, color));
        container.getChildAt(0).mask = canvas;

        var label  : Label = new Label();
        label.layoutData = layData;
        label.textRendererFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), size*0.6, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        if(name == null){
            name = "?";
        }
        container.addChild(label);
        label.text = name.charAt(0).toLocaleUpperCase();

        return container;
    }

    public static function disposeStoredAvatarTextures(excludeUsers:Array = null):void {
        if(!excludeUsers){
            excludeUsers = [];
        }
        for (var userId:String in avatarMap) {
            if(excludeUsers.indexOf(userId) == -1){
                var t : Texture = avatarMap[userId] as Texture;
                if(t){
                    t.dispose();
                }
                avatarMap[userId] = null;
                delete avatarMap[userId];
            }
        }
    }

    override public function dispose():void {
        super.dispose();
        if(_user && _instanceList[_user.id]){
            var index:int = _instanceList[_user.id].indexOf(this);
            if(index != -1){
                var instanceList:Vector.<UserCircularAvatar> = _instanceList[_user.id];
                instanceList.removeAt(index);
            }
        }
        if(_loader) {
            try {
                _loader.unloadAndStop(false);
                _loader.close();
            }catch (e:Error) {

            }
            _loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, onLoadComplete);
            _loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
            _loader= null;
        }
        clearAll();
    }

    public function set loadComplete(value:Function):void {
        if(value == null){
            _loadComplete = function (...args):void {

            };
            return;
        }
        _loadComplete = value;
    }

        private function clearAll():void {
            this.removeChildren(0, -1, true);
            //
            _border = null;
            _avatarImage = null;
            _avatarImageMask = null;
        }
    }
}
