package com.despark.tuning3d.view.components.tunings {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.GridPageIndicator;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.distriqt.extension.application.Application;
    import com.utils.Console;
    import com.utils.DisplayObjectUtil;

import feathers.controls.List;
import feathers.controls.PageIndicator;
import feathers.controls.PageIndicatorInteractionMode;
import feathers.controls.ScrollBarDisplayMode;
import feathers.controls.ScrollPolicy;
import feathers.controls.renderers.IListItemRenderer;
import feathers.data.ListCollection;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;

    import flash.geom.Point;

    import starling.core.Starling;
import starling.display.DisplayObject;
import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.utils.MathUtil;

    public class TuningItemRenderer extends ListItemRenderer {
    private var _pageIndicator:PageIndicator;
    private var slidesList:List;

    private var clipRectQuad:Quad;
    private var challengeIcon:Image;
    public var clipBackground:Boolean = false;
    private var _overrideDispose:Boolean = true;
    private var _indicatorMargin:Number = 100;
    private var _fitContent:Boolean = false;
    private var scrollStarted:Boolean = false;
    private var rightEdgeDragged:Boolean = false;
    private var leftEdgeDragged:Boolean = false;
    private var _hasElasticEdges:Boolean = false;
    private var _pageChangeDelay:Number = 0.5;
    private var _topOffset:Number = -25;
    private var _leftOffset:Number = 0;
    private var _isSliding:Boolean = true;

    public static const LEFT_EDGE_DRAGGED:String = "leftEdgeDragged";
    public static const RIGHT_EDGE_DRAGGED:String = "rightEdgeDragged";
    private var _contentScale:Number = 1;
    private var _maxFrameCount:int = -1;
    //
    private var _zoomImage:Image;
    private var _defaultZoom:Number;
    public static const ZOOM_MODE_START:String = 'zoomModeStart';
    public static const ZOOM_MODE_END:String = 'zoomModeEnd';

    public function TuningItemRenderer(){
        super ();
    }

    public function get tuning():Tuning {
        return data as Tuning;
    }

    public function startZoomMode():void{
        var renderer:TuningSlideFHDItemRenderer = slidesList.itemToItemRenderer(slidesList.dataProvider.getItemAt(slidesList.horizontalPageIndex)) as TuningSlideFHDItemRenderer;
        if(!_zoomImage && renderer){
            _zoomImage = renderer.getImageClone();
            if(_zoomImage){
                _defaultZoom = _zoomImage.scale;
                stage.addChild(_zoomImage);
                _zoomImage.addEventListener(TouchEvent.TOUCH, zoomTouchEventHandler);
            }
            isEnabled = touchable = false;
            dispatchEvent(new Event(ZOOM_MODE_START));
        }
    }

    public function endZoomMode():void{
        if(_zoomImage){
            Starling.juggler.removeTweens(_zoomImage);
            _zoomImage.removeEventListeners();
            _zoomImage.removeFromParent(true);
            _zoomImage = null;
            isEnabled = touchable = true;
            dispatchEvent(new Event(ZOOM_MODE_END));
        }
    }

        private function zoomTouchEventHandler(event:TouchEvent):void {
            var touch:Touch = event.getTouch(_zoomImage, TouchPhase.ENDED);
            if(touch && touch.tapCount == 2){
                endZoomMode();
            }else if(event.touches.length > 1){
                zoom(event.touches);
            }
        }

        public function zoom(touches:Vector.<Touch>):void {
            try {
                var _zoomScale:Number = _zoomImage.scale;

                var touchA:Touch = touches[0];
                var touchB:Touch = touches[1];

                var currentPosA:Point = touchA.getLocation(parent);
                var previousPosA:Point = touchA.getPreviousLocation(parent);
                var currentPosB:Point = touchB.getLocation(parent);
                var previousPosB:Point = touchB.getPreviousLocation(parent);

                var currentVector:Point = currentPosA.subtract(currentPosB);
                var previousVector:Point = previousPosA.subtract(previousPosB);
                var sizeDiff:Number = currentVector.length / previousVector.length;
                _zoomScale *= sizeDiff;
                _zoomScale = MathUtil.clamp(_zoomScale, _defaultZoom * 0.5, _defaultZoom * 5);

                var prevWidth:Number = _zoomImage.width;
                var prevHeight:Number = _zoomImage.height;

                var nextWidth:Number = _zoomScale * prevWidth / _zoomImage.scale;
                var nextHeight:Number = _zoomScale * prevHeight / _zoomImage.scale;
                if(nextWidth < stage.stageWidth){
                    nextWidth = stage.stageWidth;
                    _zoomScale = _zoomImage.scale * nextWidth / prevWidth;
                    nextHeight = _zoomScale * prevHeight / _zoomImage.scale;
                }

                if(nextHeight < stage.stageHeight){
                    nextHeight = stage.stageHeight;
                    _zoomScale = _zoomImage.scale * nextHeight / prevHeight;
                    nextWidth = _zoomScale * prevWidth / _zoomImage.scale;
                }


                var prevPos:Point = previousPosA.add(previousPosB);
                var currPos:Point = currentPosA.add(currentPosB);

                var widthDiff:Number = nextWidth - prevWidth;
                var heightDiff:Number = nextHeight - prevHeight;
                var xMove:Number = (currPos.x - prevPos.x) / 2;
                var yMove:Number = (currPos.y - prevPos.y) / 2;
                var toX:Number = MathUtil.clamp(_zoomImage.x - widthDiff / 2 + xMove, stage.stageWidth - nextWidth, 0);
                var toY:Number = MathUtil.clamp(_zoomImage.y - heightDiff / 2 + yMove, stage.stageHeight - nextHeight, 0);
                //
                var isBeginning:Boolean = false;
                for each(var touch:Touch in touches){
                    if(touch.phase == TouchPhase.BEGAN){
                        isBeginning = true;
                        break;
                    }
                }
                if(!isBeginning && (_zoomImage.width == stage.stageWidth || _zoomImage.height == stage.stageHeight) && Math.round(_zoomImage.x - toX) == 0  && Math.round(_zoomImage.y - toY) == 0){
                    endZoomMode();
                    return;
                }
                _zoomImage.scale = _zoomScale;
                _zoomImage.x = toX;
                _zoomImage.y = toY;
            } catch (e:Error) {
                _zoomImage = touches[0].target as Image;
                endZoomMode();
            }
        }

        public function drag(touches:Vector.<Touch>):void {
            try {
                var touchA:Touch = touches[0];
                var prevPos:Point = touchA.getPreviousLocation(parent);
                var currPos:Point = touchA.getLocation(parent);

                var xMove:Number = (currPos.x - prevPos.x) / 2;
                var yMove:Number = (currPos.y - prevPos.y) / 2;
                var toX:Number = MathUtil.clamp(_zoomImage.x + xMove, stage.stageWidth, 0);
                var toY:Number = MathUtil.clamp(_zoomImage.y + yMove, stage.stageHeight, 0);
                _zoomImage.x = toX;
                _zoomImage.y = toY;
            } catch (e:Error) {
                _zoomImage = touches[0].target as Image;
                endZoomMode();
            }
        }

    protected override function initialize():void {
        clipRectQuad = new Quad(1, 1);
        super.initialize();
        overrideDispose = _overrideDispose;
        this.label.visible = false;
        var layout:HorizontalLayout = new HorizontalLayout();
        layout.gap = 0;
        layout.horizontalAlign = HorizontalAlign.CENTER;
        layout.verticalAlign = VerticalAlign.MIDDLE;
        slidesList = new List();
        slidesList.minimumPageThrowVelocity = 1;
        slidesList.dataProvider = new ListCollection();

        slidesList.layout = layout;
        slidesList.snapToPages = true;
        slidesList.hasElasticEdges = _hasElasticEdges;
        slidesList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
        slidesList.horizontalScrollPolicy = (_isSliding?ScrollPolicy.ON:ScrollPolicy.OFF);
        slidesList.verticalScrollPolicy = ScrollPolicy.OFF;
        slidesList.addEventListener(Event.SCROLL, slidesListScrollHandler);
        addEventListener(Event.CHANGE, slidesListChangeHandler);
        slidesList.factoryIDFunction = function(item:Object, index:int):String {
            var tuningItem:Tuning = item.tuning as Tuning;
            if(tuningItem && tuningItem.hdQuality) {
                return "fhd";
            }
            return "cs";
        };
        slidesList.setItemRendererFactoryWithID("cs", itemCSRendererFactory);
        slidesList.setItemRendererFactoryWithID("fhd", itemFHDRendererFactory);

        addChild(slidesList);

        _pageIndicator = new GridPageIndicator();
        _pageIndicator.width = actualWidth - 2 * _indicatorMargin;
        _pageIndicator.normalSymbolFactory = function ():DisplayObject {
            var img:Image = new Image(AlertView.assetManager.getTexture("pageIndicator"));
            img.width = 9;
            img.height = 9;
            return img;
        };

        _pageIndicator.selectedSymbolFactory = function ():DisplayObject {
            var img:Image = new Image(AlertView.assetManager.getTexture("pageIndicatorSelected"));
            img.width = 9;
            img.height = 9;
            return img;
        };

        _pageIndicator.gap = 6;

        _pageIndicator.horizontalAlign = HorizontalAlign.CENTER;
        _pageIndicator.verticalAlign = VerticalAlign.MIDDLE;


        _pageIndicator.selectedIndex = -1;
        _pageIndicator.pageCount = 0;
        _pageIndicator.interactionMode = PageIndicatorInteractionMode.PRECISE;
        _pageIndicator.addEventListener(Event.CHANGE, pageChangeHandler);
        this.addChild(_pageIndicator);

        challengeIcon = new Image(ButtonFactory.assetManager.getTexture("gone_to_challenge_lable"));
        challengeIcon.width = actualWidth/4;
        challengeIcon.scale = challengeIcon.scaleX;
        this.addChild(challengeIcon);
    }

    private function itemCSRendererFactory():IListItemRenderer {
        var renderer:TuningSlideItemRenderer = new TuningSlideItemRenderer();
        var item_w:Number = (actualWidth);
        var item_h:Number = (actualHeight);
        renderer.overrideDispose = _overrideDispose;
        renderer.clipBackground = clipBackground;
        renderer.setSize(item_w, item_h);
        renderer.fitContent = fitContent;
        return renderer;
    }

    private function itemFHDRendererFactory():IListItemRenderer {
        var renderer:TuningSlideFHDItemRenderer = new TuningSlideFHDItemRenderer();
        renderer.topOffset = _topOffset;
        renderer.leftOffset = _leftOffset;
        renderer.contentScale = _contentScale;
        var item_w:Number = actualWidth;
        var item_h:Number = actualHeight;
        renderer.overrideDispose = _overrideDispose;
        renderer.clipBackground = clipBackground;
        renderer.setSize(item_w, item_h);
        renderer.fitContent = fitContent;
        return renderer;
    }

    public function set hasElasticEdges(value:Boolean):void{
        _hasElasticEdges = value;
        if(slidesList){
            slidesList.hasElasticEdges = _hasElasticEdges;
        }
    }

    private function pageChangeHandler(event:Event):void {
        slidesList.removeEventListener(Event.SCROLL, slidesListScrollHandler);
        slidesList.scrollToPageIndex(_pageIndicator.selectedIndex, 0, _pageChangeDelay);
        restoreScrollHandling();
        dispatchEventWith(Event.CHANGE, false);
    }

    override public function setSize(w:Number, h:Number):void{
        super .setSize(w, h);
        if(slidesList){
            slidesList.width = actualWidth;
            slidesList.height = actualHeight;
            slidesList.validate();
            slidesList.scrollToPageIndex(slidesList.horizontalPageIndex, 0);
        }
    }

    private function restoreScrollHandling():void{
        if(slidesList.isScrolling){
            Starling.juggler.delayCall(restoreScrollHandling, 0.03);
        }else if(!slidesList.hasEventListener(Event.SCROLL, slidesListScrollHandler)){
            slidesList.addEventListener(Event.SCROLL, slidesListScrollHandler);
        }
    }

    private function postToTimelineButtonHandler(event:Event):void {
        if (!data) {
            return;
        }
        AlertView.show('Are you sure you want to share this tuning on 3DTuning\' feed', -1, false, function (yesSelected:Boolean):void {
            if (yesSelected) {
                var timelineProxy : TimelineProxy = ApplicationFacade.getInstance().retrieveProxy(TimelineProxy.NAME) as TimelineProxy;
                timelineProxy.postToTimeline(tuning.id);
                /*var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
                var snapshotIndex:int = (slidesList.horizontalPageIndex>-1?slidesList.horizontalPageIndex:0);
                var urls:Array = [{url: tuning.snapshots[snapshotIndex]}];*/
                /*for(var i:int = 0; i < tuning.snapshots.length; i++){
                    urls.push({url: tuning.snapshots[i]});
                }*/
                //timelineProxy.newPost('', usersProxy.currentUser, urls);
            }
        });
    }

    protected function slidesListScrollHandler(event:Event):void {
        if(!scrollStarted && isScrolling){
            scrollStarted = true;
            SoundManager.playSound(SoundManager.SELECT_PART);
        }else if(!isScrolling){
            scrollStarted = false;
        }
        setPageIndicatorPage(slidesList.horizontalPageIndex);
        if(_hasElasticEdges){
            if(slidesList.horizontalPageIndex == slidesList.dataProvider.length - 1 && slidesList.horizontalScrollPosition > slidesList.maxHorizontalScrollPosition){
                rightEdgeDragged = true;
                leftEdgeDragged = false;
            }else if(slidesList.horizontalPageIndex == 0 && slidesList.horizontalScrollPosition < slidesList.minHorizontalScrollPosition){
                rightEdgeDragged = false;
                leftEdgeDragged = true;
            }
            if(!scrollStarted){
                if(rightEdgeDragged){
                    rightEdgeDragged = false;
                    dispatchEventWith(RIGHT_EDGE_DRAGGED);
                }else if(leftEdgeDragged){
                    leftEdgeDragged = false;
                    dispatchEventWith(LEFT_EDGE_DRAGGED);
                }
            }
        }
    }

    protected function slidesListChangeHandler(event:Event):void {
        var renderer:TuningSlideFHDItemRenderer = slidesList.itemToItemRenderer(slidesList.dataProvider.getItemAt(slidesList.horizontalPageIndex)) as TuningSlideFHDItemRenderer;
        if(renderer){
            renderer.updatePositions();
        }
    }

    private function setPageIndicatorPage(value:int):void{
        _pageIndicator.removeEventListener(Event.CHANGE, pageChangeHandler);
        if(_pageIndicator.selectedIndex != value){
            _pageIndicator.selectedIndex = value;
            dispatchEventWith(Event.CHANGE, false);
        }
        _pageIndicator.addEventListener(Event.CHANGE, pageChangeHandler);
    }

    protected override function draw():void {
        super.draw();
        var sizeInvalid:Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
        if (sizeInvalid) {
            if (!this.mask) {
                clipRectQuad.x = 0;
                clipRectQuad.width = actualWidth;
                if(clipBackground){
                    addChild(clipRectQuad);
                    clipRectQuad.height = actualHeight + 50;
                    clipRectQuad.y = -25;
                }else{
                    clipRectQuad.height = actualHeight;
                    clipRectQuad.y = 0;
                }
                this.mask = clipRectQuad;
            }else if(clipBackground){
                if(mask.height != actualHeight + 50){
                    mask.height = actualHeight + 50;
                }
                if(mask.y != -25){
                    mask.y = -25;
                }
                if(!mask.parent){
                    addChild(mask);
                }
            }

            _pageIndicator.validate();

            _pageIndicator.x = (actualWidth - _pageIndicator.width) / 2;
            _pageIndicator.y = (actualHeight - 10 - _pageIndicator.height);

            imageLoader.width = actualWidth * 0.75;
            imageLoader.height = actualHeight * 0.75;


            slidesList.width = this.actualWidth;
            slidesList.height = this.actualHeight;
        }
        if(data && Tuning(data).isChallenging && challengeIcon){

            challengeIcon.visible = true;
        }else if(data && !Tuning(data).isChallenging && challengeIcon){

            challengeIcon.visible = false;
        }

    }


    public function get isScrolling():Boolean{
        if(!slidesList){
            return false;
        }
        return slidesList.isScrolling;
    }

    protected override function commitData():void {
        super.commitData();


        if (!tuning) return;

        var hdsize:Boolean = false;
        hdsize = tuning.hdsize;


        var list:Array = [];

        if (tuning.snapshots && tuning.snapshots.length == 0) {
            trace("no snapshots");
        }

        var ln:int = tuning.snapshots.length;
        if(_maxFrameCount > 0){
            ln = _maxFrameCount;
        }
        for(var i:int = 0; i < ln; i++) {
            var url:String = tuning.snapshots[i];
            if(url){
                list.push({
                    url: url,
                    hdsize: hdsize,
                    tuning: tuning,
                    index: list.length
                });
            }
        }

        if (list.length <= 0) {

            list.push({
                url: tuning.imageURL,
                hdsize: hdsize,
                tuning: tuning,
                index: list.length
            });
        }
        slidesList.dataProvider = new ListCollection(list);
        _pageIndicator.pageCount = tuning.snapshots.length;
        _pageIndicator.selectedIndex = 0;
        _pageIndicator.validate();
        if(_pageIndicator.x == 0){
            _pageIndicator.x = (actualWidth - _pageIndicator.width) / 2;
        }
        _pageIndicator.y = (actualHeight - 15 - _pageIndicator.height);
    }

    public function addNextFrame():void{
        if(tuning.snapshots.length > slidesList.dataProvider.length){
            var url:String = tuning.snapshots[slidesList.dataProvider.length];
            var frameData:Object = {
                url: url,
                hdsize: tuning.hdsize,
                tuning: tuning,
                index: slidesList.dataProvider.length
            };
            slidesList.dataProvider.addItem(frameData);
        }
    }

    override public function dispose():void {
        if(_pageIndicator){
            DisplayObjectUtil.dispose(_pageIndicator);
            _pageIndicator.dispose();
            _pageIndicator = null;
        }
        if(slidesList){
            slidesList.dataProvider = null;
            slidesList.itemRendererFactory = null;
            slidesList.dispose();
        }
        if(clipRectQuad){
            clipRectQuad.dispose();
            clipRectQuad = null;
        }
        DisplayObjectUtil.dispose(this);
        if (challengeIcon) {
            challengeIcon.removeFromParent(true);
            challengeIcon = null;
        }
        super.dispose();
    }

    public function set page(value:int):void{
        if(slidesList && slidesList.dataProvider && value >= 0 && value < slidesList.dataProvider.length){
            slidesList.scrollToPageIndex(value, 0, 0);
        }
    }

    public function get pageIndicator():PageIndicator {
        return _pageIndicator;
    }

    public function get selectedIndex():Number {
        return slidesList.horizontalPageIndex;
    }

    public function set overrideDispose(overrideDispose:Boolean):void {
        if(imageLoader){
            imageLoader.overrideDispose = overrideDispose;
        }
        _overrideDispose = overrideDispose;
    }

    public function set indicatorMargin(value:Number):void {
        _indicatorMargin = value;
    }

    public function set fitContent(value:Boolean):void {
        _fitContent = value;
    }

    public function get fitContent():Boolean {
        return _fitContent;
    }

    public function set pageChangeDelay(value:Number):void {
        _pageChangeDelay = value;
    }

    public function set topOffset(value:Number):void {
        _topOffset = value;
        if(slidesList && slidesList.dataProvider && slidesList.dataProvider.length){
            for(var i:int = 0; i < slidesList.dataProvider.length; i++){
                var renderer:TuningSlideFHDItemRenderer = slidesList.itemToItemRenderer(slidesList.dataProvider.getItemAt(i)) as TuningSlideFHDItemRenderer;
                if(renderer){
                    renderer.topOffset = _topOffset;
                }
            }
        }
    }

    public function set leftOffset(value:Number):void {
        _leftOffset = value;
        if(slidesList && slidesList.dataProvider && slidesList.dataProvider.length){
            for(var i:int = 0; i < slidesList.dataProvider.length; i++){
                var renderer:TuningSlideFHDItemRenderer = slidesList.itemToItemRenderer(slidesList.dataProvider.getItemAt(i)) as TuningSlideFHDItemRenderer;
                if(renderer){
                    renderer.leftOffset = _leftOffset;
                }
            }
        }
    }

    public function set isSliding(value:Boolean):void {
        _isSliding = value;
    }

    public function set contentScale(value:Number):void {
        _contentScale = value;
        if(slidesList && slidesList.dataProvider && slidesList.dataProvider.length){
            for(var i:int = 0; i < slidesList.dataProvider.length; i++){
                var renderer:TuningSlideFHDItemRenderer = slidesList.itemToItemRenderer(slidesList.dataProvider.getItemAt(i)) as TuningSlideFHDItemRenderer;
                if(renderer){
                    renderer.contentScale = _contentScale;
                }
            }
        }
    }

    public function set maxFrameCount(value:int):void {
        _maxFrameCount = value;
    }
}
}