package com.despark.tuning3d.view.components.tunings
{
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.tunings.Tuning;
	import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusInVoting;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
	import com.despark.tuning3d.view.components.core.ListItemRenderer;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.utils.DateParser;
	import com.utils.DisplayObjectUtil;
	import com.utils.FontSelector;

	import flash.filesystem.File;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.core.Starling;
	import starling.display.Quad;
	import starling.textures.TextureSmoothing;

	public class TuningRowsItemRenderer extends ListItemRenderer {
		private var _scale:Number=0.75;
        private var _posY:Number=0;
		protected var nameBg:Quad;
		private var clipRectQuad:Quad;
		protected var nameLabel:MultilingualLabel;
		private var backgroundImage:ImageLoaderWithStates=null;
		public var pageContentCount:int = 1;
		//
		private var leftMargin:Number = 0;
		private var rightMargin:Number = 0;
		private var topMargin:Number = 0;
		private var bottomMargin:Number = 0;
		private var marginSize:Number = 0.75;
		//
		private var itemIndex:int = 0;
		private var carTransform:Object;
		private var _imageWidth:Number = 0;
		private var _imageHeight:Number = 0;
		//
		private var inChallengeBadge:ChallengeStatusInVoting;
		public function get tuning() : Tuning {
			return data as Tuning;
		}
		
		protected override function initialize() : void {
			autoResize = false;
			backgroundImage = new ImageLoaderWithStates();
			backgroundImage.overrideDispose = true;
			backgroundImage.visible = false;
			backgroundImage.completeHandler = function():void{
				var bgRatio:Number = backgroundImage.currentTexture.width / backgroundImage.currentTexture.height;
				var actualRatio:Number = actualWidth / actualHeight;
				if(actualRatio < bgRatio){
					if(backgroundImage.height != actualHeight){
						backgroundImage.height = actualHeight;
						backgroundImage.width = actualHeight * bgRatio;
					}
				}else{
					if(backgroundImage.width != actualWidth){
						backgroundImage.width = actualWidth;
						backgroundImage.height = actualWidth / bgRatio;
					}
				}

				backgroundImage.x = (actualWidth - backgroundImage.width)/2;
				backgroundImage.y = (actualHeight - backgroundImage.height)/2;
			};
			addChild(backgroundImage);
			
			super.initialize();
			imageLoader.overrideDispose = true;
			imageLoader.visible = false;

			label.visible = false;
			nameBg = new Quad(1, 1, 0x000000);
			nameBg.alpha = 0.4;
			addChild(nameBg);



			nameLabel = new MultilingualLabel();
			nameLabel.textFormat = new TextFormat(FontSelector.getFont(false), 10, 0xffffff);
			nameLabel.textFormat.align = TextFormatAlign.LEFT;
			if(Device.isIphoneXVariant()){
				nameLabel.textFormat.align = TextFormatAlign.CENTER;
			}
			nameLabel.textFormat.bold = false;
			addChild(nameLabel);
			nameLabel.updateWithLanguage(AlertView.language);
			clipRectQuad = new Quad(1,1,0x000000);
			addChild(clipRectQuad);
		}
		

        override public function dispose():void {
            DisplayObjectUtil.disposeMesh(nameBg);
			nameBg = null;
            DisplayObjectUtil.disposeMesh(clipRectQuad);
			clipRectQuad = null;
            if(nameLabel)
				nameLabel.dispose();
			nameLabel = null;
            if(backgroundImage)
				backgroundImage.dispose();
			backgroundImage = null;
			super.dispose();


        }

        protected override function draw() : void {
			super.draw();
			var dataInvalid : Boolean = isInvalid(INVALIDATION_FLAG_DATA);
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			clipRectQuad.width = actualWidth - (leftMargin + rightMargin);
			clipRectQuad.height = actualHeight - (topMargin + bottomMargin);
			clipRectQuad.x = rightMargin;
			clipRectQuad.y = topMargin;
			mask = clipRectQuad;

			if(sizeInvalid && nameBg){
				nameBg.width = actualWidth;
				nameBg.height = 20;
				nameBg.x = 0;
				nameBg.y = actualHeight - nameBg.height;
			}

			if(dataInvalid && nameLabel){
				if(tuning){
					nameLabel.text=tuning.name;
				}
				nameLabel.height=NaN;
				nameLabel.width=actualWidth-10;
				nameLabel.validate();
				nameLabel.x=5;
				nameLabel.y=(actualHeight-nameBg.height)+((20-nameLabel.height)/2);
			}
		}
		
		protected function completeLoadImage(imageWidth:Number, imageHeight:Number):void{
			_imageWidth = imageWidth;
			_imageHeight = imageHeight;
			layout();
			Starling.current.juggler.delayCall(layout, 0.1);
		}

		protected function completeLoadDefaultImage(imageWidth:Number, imageHeight:Number):void{
			_imageWidth = imageWidth;
			_imageHeight = imageHeight;
			imageLoader.width = _imageWidth * actualScale * 0.45;
			imageLoader.height = _imageHeight * actualScale * 0.45;

			imageLoader.x = (actualWidth - imageLoader.width) / 2;
			imageLoader.y = (actualHeight - imageLoader.height) / 2;
		}

		override protected function layout():void{
			if(imageLoader) {
				var bgScale:Number = 1;
				var imgScale:Number;
				if (backgroundImage && backgroundImage.image) {
					bgScale = backgroundImage.image.scale;
				}
				if (bgScale == 1) {
					bgScale = actualScale;
					Starling.current.juggler.delayCall(layout, 0.1);
					imageLoader.visible = false;
				} else {
					imageLoader.visible = true;
				}
				imgScale = bgScale * carTransform.zoomLevel;
				imageLoader.width = _imageWidth * imgScale;
				imageLoader.height = _imageHeight * imgScale;
				var ds:Number = 0.06 * (ButtonFactory.assetManager.scaleFactor - 1.5) / 2.5 + 0.24;
				imageLoader.x = (actualWidth - imageLoader.width) / 2 + carTransform.pan.x * imgScale + tuningDx * ds;
				imageLoader.y = (actualHeight - imageLoader.height) / 2 + carTransform.pan.y * imgScale + tuningDy * ds;
			}
		}

		private function get actualScale():Number{
			if(stage){
				if(actualWidth/actualHeight < stage.stageWidth / stage.stageHeight){
					return actualWidth / stage.stageWidth;
				}
				return actualHeight / stage.stageHeight;
			}
			return 0.3;
		}

		public function get tuningDx():Number{
			var dx:Number = customOffset.left;
			return dx;
		}

		public function get tuningDy():Number{
			var dy:Number = customOffset.bottom;
			return dy;
		}

		private function get customOffset():Object{
			if(tuning){
				if(tuning.carObj && tuning.carObj.custom_offset){
					return tuning.carObj.custom_offset;
				}
				if(tuning.car && tuning.car.custom_offset){
					return tuning.car.custom_offset
				}
				return {bottom:0, left: 0};
			}
			return {bottom:0, left: 0};
		}

		private function updateBackground(carBackground:CarBackground):void{
			if(backgroundImage.persistentImageProvider && backgroundImage.persistentImageProvider is CarBackground && CarBackground(backgroundImage.persistentImageProvider).mainUrl == carBackground.mainUrl){
				backgroundImage.visible = true;
				return;
			}
			backgroundImage.persistentImageProvider = carBackground;
			backgroundImage.visible = true;
		}
		
		protected override function commitData() : void{
			super.commitData();
			if(tuning){
				if(tuning.carTransform && tuning.carTransform.length){
					carTransform = tuning.carTransform[0];
				}else{
					carTransform = {zoomLevel:1, pan:{x:0, y:0}};
				}
				//
				itemIndex = owner.dataProvider.getItemIndex(data) % pageContentCount;
				if(pageContentCount == 1 || pageContentCount == 6){
					leftMargin = rightMargin = topMargin = bottomMargin = 0;
				}else if(pageContentCount == 4){
					if(itemIndex % 2 == 0){
						leftMargin = marginSize;
						rightMargin = 0;
					}else if(itemIndex % 2 == 1){
						leftMargin = 0;
						rightMargin = marginSize;
					}

					if(int(itemIndex / 2) == 0){
						topMargin = 0;
						bottomMargin = 1;
					}else if(int(itemIndex / 2) == 1){
						topMargin = marginSize;
						bottomMargin = 0;
					}
				}else if(pageContentCount == 9){
					if(itemIndex % 3 == 0){
						leftMargin = marginSize;
						rightMargin = 0;
					}else if(itemIndex % 3 == 1){
						leftMargin = marginSize;
						rightMargin = marginSize;
					}else{
						leftMargin = 0;
						rightMargin = marginSize;
					}

					if(int(itemIndex / 3) == 0){
						topMargin = 0;
						bottomMargin = marginSize;
					}else if(int(itemIndex / 3) == 1){
						topMargin = marginSize;
						bottomMargin = marginSize;
					}else{
						topMargin = marginSize;
						bottomMargin = 0;
					}
				}
				//
				if(tuning.inChallenge){
					inChallengeBadge ||= new ChallengeStatusInVoting();
					inChallengeBadge.text = "AVAILABLE FOR CHALLENGE IN " + DateParser.getInChallengeLabel(tuning.inChallenge.time - new Date().time);
					addChild(inChallengeBadge);
				}else if(inChallengeBadge){
					inChallengeBadge.removeFromParent(true);
					inChallengeBadge = null;
				}
			}else{
				carTransform = {zoomLevel:1, pan:{x:0, y:0}};
			}
			var cameraIndex:int;
			var carBackground:CarBackground;
			if(tuning && tuning.backgroundId && tuning.backgroundId!=CarBackground.DEFAULT_ID && tuning.backgroundsMap && tuning.backgroundsMap[tuning.backgroundId]){
				carBackground = tuning.backgroundsMap[tuning.backgroundId];
				cameraIndex = 0;
				if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0 && carBackground.imagePathSecondCam && carBackground.imagePathSecondCam.length>0){
					cameraIndex = tuning.customSnapshotsCamera[0];
				}
				carBackground.showThumbnailUrl=false;
				carBackground.selectedCamera = cameraIndex;
				backgroundImage.smoothing = TextureSmoothing.BILINEAR;
				updateBackground(carBackground);
				_scale=0.75;
				_posY=0.025;
			}else if(tuning){
				var randomSplashUrl:String;
				if(tuning && tuning.carChooserBgName){
					randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/"+tuning.carChooserBgName.replace('.jpg', '') + ".jpg").url;
					if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0){
						cameraIndex = tuning.customSnapshotsCamera[0];
						if(tuning.carObj.defaultBkgMobile[cameraIndex]){
							randomSplashUrl = tuning.carObj.defaultBkgMobile[cameraIndex];
						}
					}
					carBackground = new CarBackground();
					carBackground.mainUrl = randomSplashUrl;
					carBackground.isLocalFile = true;
					carBackground.showThumbnailUrl = false;
					updateBackground(carBackground);
				}else if(tuning && tuning.carObj && tuning.carObj.defaultBkgMobile && tuning.carObj.defaultBkgMobile.length){
					if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0){
						cameraIndex = tuning.customSnapshotsCamera[0];
						if(tuning.carObj.defaultBkgMobile[cameraIndex]){
							randomSplashUrl = tuning.carObj.defaultBkgMobile[cameraIndex];
						}
					}
					randomSplashUrl ||= tuning.carObj.defaultBkgMobile[0];
					carBackground = new CarBackground();
					carBackground.mainUrl = randomSplashUrl;
					carBackground.isLocalFile = true;
					carBackground.showThumbnailUrl = false;
					updateBackground(carBackground);
				}else{
					randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/car_chooser.jpg").url;
					carBackground = new CarBackground();
					carBackground.mainUrl = randomSplashUrl;
					carBackground.isLocalFile = true;
					carBackground.showThumbnailUrl = false;
					updateBackground(carBackground);
				}
				backgroundImage.visible=true;

				_scale=0.85;

				_posY=0.05;
			}

			if(tuning){
				if(imageLoader.isLoaded && imageLoader.persistentImageProvider && imageLoader.persistentImageProvider is Car && Car(imageLoader.persistentImageProvider).imageURL == tuning.imageURL){
					return;
				}
				var car:Car = new Car();
				car.imageURL = tuning.imageURL;

				imageLoader.recieveImageSize = completeLoadImage;
				imageLoader.bitmapScale = 0.4;
				imageLoader.errorHandler = imageLoaderErrorHandler;
				imageLoader.persistentImageProvider = car;
			}
		}

		private function imageLoaderErrorHandler():void{
			var url:String = "http://data.3dtuning.com/ui-elements/intro_page/image-archived.jpg";
			var imageProvider:Car = imageLoader.persistentImageProvider as Car;
			imageProvider.imageURL = url;
			imageLoader.recieveImageSize = completeLoadDefaultImage;
			imageLoader.persistentImageProvider = imageProvider;
		}
	}
}
