package com.despark.tuning3d.view.components.tunings
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.challenges.ChallengeData;
	import com.despark.tuning3d.model.challenges.ChallengeEvent;
	import com.despark.tuning3d.model.core.BannerScreen;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.core.Tracer;
	import com.despark.tuning3d.model.core.UIColors;
	import com.despark.tuning3d.model.core.UIStrings;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.LanguagesProxy;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.tunings.Tuning;
	import com.despark.tuning3d.model.tunings.TuningsProxy;
	import com.despark.tuning3d.model.users.User;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.auth.UserDetailsPopup;
	import com.despark.tuning3d.view.components.brands.BrandScreenCarChooser;
	import com.despark.tuning3d.view.components.challenges.controls.ChallengeViewUserLabel;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.BannerStackedScreen;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.IMainMenuHolder;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.core.MainMenuView;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.despark.tuning3d.view.components.core.ScreenComponentFactory;
	import com.despark.tuning3d.view.components.core.StackedScreen;
	import com.despark.tuning3d.view.components.core.StackedScreenNavigator;
	import com.despark.tuning3d.view.components.gallery.GalleryLabelFactory;
	import com.despark.tuning3d.view.components.gallery.SearchInput;
	import com.despark.tuning3d.view.components.gallery.data.RequestPeriod;
	import com.despark.tuning3d.view.components.gallery.data.RequestSort;
	import com.despark.tuning3d.view.components.garage.GarageSettingControl;
	import com.despark.tuning3d.view.components.timeline.components.TimelineLoadBar;
	import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
	import com.distriqt.AdsManager;
	import com.greensock.TweenLite;
	import com.utils.DisplayObjectUtil;
	import com.utils.FileUtil;
	import com.utils.FontSelector;
	import com.utils.ObjectUtils;
	import com.utils.Tracker;

	import feathers.controls.Button;
	import feathers.controls.ButtonState;
	import feathers.controls.Label;
	import feathers.controls.LayoutGroup;
	import feathers.controls.List;
	import feathers.controls.Radio;
	import feathers.controls.ScrollBarDisplayMode;
	import feathers.controls.ScrollPolicy;
	import feathers.controls.ToggleButton;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.core.FeathersControl;
	import feathers.core.ToggleGroup;
	import feathers.data.ListCollection;
	import feathers.layout.Direction;
	import feathers.layout.HorizontalAlign;
	import feathers.layout.HorizontalLayout;
	import feathers.layout.TiledRowsLayout;
	import feathers.layout.VerticalAlign;
	import feathers.layout.VerticalLayout;

	import flash.filesystem.File;
	import flash.geom.Rectangle;
	import flash.text.TextFormat;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.DropShadowFilter;
	import starling.text.TextFormat;
	import starling.textures.Texture;
	import starling.utils.Color;

	public class TuningsListScreen extends StackedScreen implements Multilingual, IMainMenuHolder, ITuningListMenuVIew
	{
		private static var k:int = 0;

        public static const GET_SOCIAL_DATA : String = "GetSocialData";
        public static const UPDATE_FEED     : String = "UpdateFeed";
        public static const CAR_SELECTED    : String = "CarSelected";
		public static const TUNING_SELECTED : String = "TuningSelected";
		public static const TUNING_SHARE    : String = "TuningShare";
        public static const TUNING_DELETE   : String = "TUNING_DELETE";
		public static const EDITOR_SELECT   : String = "EDITOR_SELECT";
		public static const BUTTONS_HIDE   : String = "BUTTONS_HIDE";
		public static const BUTTONS_SHOW   : String = "BUTTONS_SHOW";

		private static const BANNER_SIZE:int = 47;
		private static const TOOLBAR_SIZE:int = 2; //31;


		//public var pageReached : int = -1;
		private var _mainMenuView : MainMenuView;
		//private var frameImage : Scale9Image;
		//private var topBarQuad : Quad;
		private var _backgroundImage:Image;
		private var _headerBackground:Image;
		private var _backgroundQuad : Quad;
		private var topBarHeight : int;
		//private var titleLabel : MultilingualLabel;
		private var tuningsRowsList : List;
		private var tuningsList : List;
        private var _tuningsListCollection : ListCollection;
		private var noTuningsLabel : MultilingualLabel;
		private var _onComplete : Function;
		private var titleLabel : MultilingualLabel;
		//private var refreshed : Boolean;

        private var tuningsListMenuView:TuningsListMenuView;
		
		private static var _selectedPage:int=-1;
		private var _selPage:int=-1;
		
		//private var pages:PageIndicator;


		private var _settingsButton:Button;
		private var _nextButton:Button;
		private var _prevButton:Button;
        private var closeButton:Button;
		private var pagesLabel:MultilingualLabel;
		private var _closeHandler:Function;
		
		public var _stateRows:Boolean=true;
		
		private var menuButton:Button;

		private var _selectedLanguage:Language;
		private var _languages:Array;
		private var _gap:int = 10;
		public var enableBackward:Boolean = false;
        private var _challengesList:Object;

		private var _loadingBar: TimelineLoadBar;
        public var tuningsIsLoading: Boolean = false;
        private var _menuTween:TweenLite;
        private var _showBanner:ToggleButton;
        private var _userLabel:ChallengeViewUserLabel;
        private var _userAvatar:UserCircularAvatar;
        private var _tuningRendererList:Vector.<TuningItemRenderer>;
        private var _currentUser:User;

		private var rowCount:int = 3;
		private var colCount:int = 2;
		private var _countPerPage:int = rowCount * colCount;
		private var viewToggleGroup:ToggleGroup;
		private var viewLayout:LayoutGroup;

		private var showByButton:ToggleButton;
		private var showByLayout:LayoutGroup;
		private var sortByLayout:LayoutGroup;
		private var showByMenu:Sprite;
		private var showByGroup:LayoutGroup;
		private var sortByGroup:LayoutGroup;
		private var showByToggleGroup:ToggleGroup;
		private var sortByToggleGroup:ToggleGroup;

		private var sortByLabel:Label;

		private var searchInput:SearchInput;
		private var _noResultLabel:Label;

		private var showByBorderList:Array;
		private var _period:String;
		private var _sort:String;
		private var _query:String;
		private var _sortIconTextures:Object;

		private var settingsControl:GarageSettingControl;
		private var isSettingOpen:Boolean = false;
		//
		private static var filterSettings:Object;
		private var settingsFile:File;
		public var postToTimeline:Boolean = false;
		public var carType:String;
		public var carId:String;
		public var enterChallenge:Boolean = false;
		public var _selectedChallengeTuning:Tuning;
		public var challengeData:Object;
		private var _topButtonPadding:Number=0;
		private var _tuningView:TuningItemRenderer;
		private var _newBuildBtn:Button;
		public static const CREATE_NEW_BUILD:String = 'createNewBuild';
		public function TuningsListScreen() {
			super("TuningsListScreen");
			k++;
            _challengesList = {};
            var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
            _currentUser = usersProxy.currentUser;
		}

		override protected function initialize() : void {
			super.initialize();
			//
			if(!filterSettings) {
				filterSettings = {
					sortBy:RequestSort.DATE
				};
				settingsFile = File.applicationStorageDirectory.resolvePath("garage_settings.json");
				if (settingsFile.exists) {
					var contentStr:String = FileUtil.getFileContent(settingsFile);
					var contentObject:Object;
					if(contentStr && contentStr.length){
						contentObject = JSON.parse(contentStr);
					}else{
						contentObject = {};
					}
					ObjectUtils.mergeInto(contentObject, filterSettings);
				}
			}
			sortBy = filterSettings.sortBy;
			//
			topBarHeight = 0;//; DeviceCapabilities.isTablet(Starling.current.nativeStage) ? 35 : 40;

			_tuningRendererList = new <TuningItemRenderer>[];
			_backgroundQuad = new Quad(1, 1, Color.rgb(255, 255, 255));
			addChild(_backgroundQuad);


			_backgroundImage = new Image(Texture.fromColor(600,400));
			addChild(_backgroundImage);

			_headerBackground = new Image(Texture.fromColor(1,1,UIColors.BANNER_BACKGROUND_COLOR));
			addChild(_headerBackground);


			titleLabel = new MultilingualLabel();
			//titleLabel.height = 13;
			titleLabel.textFormat = new flash.text.TextFormat( FontSelector.getMontserratMedium(), GalleryLabelFactory.SIZE_BIG, 0xFFFFFF, true);
			titleLabel.textFormat.align = TextFormatAlign.LEFT;
			addChild(titleLabel);
			titleLabel.updateWithLanguage(AlertView.language);

			pagesLabel = new MultilingualLabel();
			pagesLabel.textFormat = new flash.text.TextFormat( FontSelector.getMontserratMedium(), 12, UIColors.WHITE);
			pagesLabel.textFormat.align = TextFormatAlign.CENTER;
			pagesLabel.height = 13;
			addChild(pagesLabel);
			pagesLabel.updateWithLanguage(AlertView.language);

			if (!tuningsRowsList) {
				initializeTuningsRowsList();
			}

			if (!tuningsList) {
				initializeTuningsList();
			}

			noTuningsLabel = new MultilingualLabel();
			noTuningsLabel.includeInLayout = false;
			noTuningsLabel.visible = false;
			noTuningsLabel.height = 13;
			noTuningsLabel.textFormat = new flash.text.TextFormat( FontSelector.getMontserratSemiBold(), 11, 0x424242);
			noTuningsLabel.textFormat.align = TextFormatAlign.CENTER;
			addChild(noTuningsLabel);

			tuningsListMenuView = new TuningsListMenuView(this);

			tuningsListMenuView.addEventListener(TuningsListMenuView.CAR_SELECTED, onCarSelected);
			tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_SELECTED, onTuningSelected);
			tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_SHARE, onTuningShare);
			tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_DELETE, onTuningDelete);
			tuningsListMenuView.addEventListener(TuningsListMenuView.START_CHALLENGE, challengeHandler);
			tuningsListMenuView.addEventListener(TuningsListMenuView.POST_TO_TIMELINE, postToTimelineButtonHandler);
			tuningsListMenuView.addEventListener(TuningsListMenuView.FILTER_SELECTED, filterButtonHandler);

			tuningsListMenuView.addEventListener(TuningsListMenuView.COMMENT_SELECTED, commentSelectHandler);
			tuningsListMenuView.addEventListener(TuningsListMenuView.LIKE_SELECTED, likeSelectHandler);

			tuningsListMenuView.deleteAvailable = true;
			tuningsListMenuView.owner = this;
			tuningsListMenuView.width = Starling.current.stage.stageWidth;
			tuningsListMenuView.height = 50;
			tuningsListMenuView.requireLogin = (_currentUser == null);
			addChild(tuningsListMenuView);

			_nextButton = ButtonFactory.arrowButton("slide_arr_lt");
			_nextButton.rotation = Math.PI;
			_nextButton.width = 28;
			_nextButton.height = 43;
			_nextButton.addEventListener(Event.TRIGGERED, onNextButtonHandler);
			_nextButton.alpha = 1;
			addChild(_nextButton);


			menuButton = ButtonFactory.menuBigButton("a_back_button", 0x23405e);
			menuButton.width = 35;
			menuButton.height = 35;
			menuButton.addEventListener(Event.TRIGGERED, onMenuButton);
			addChild(menuButton);

			_prevButton = ButtonFactory.arrowButton("slide_arr_lt");
			_prevButton.width = 28;
			_prevButton.height = 43;
			_prevButton.addEventListener(Event.TRIGGERED, onPrevButtonHandler);
			_prevButton.alpha = 1;
			addChild(_prevButton);

			menuButton.x = menuButton.y = (BANNER_SIZE - menuButton.height) / 2;
			if(Device.isIphoneXVariant()){
				menuButton.x += 6;
			}

			_loadingBar ||= new TimelineLoadBar();
			_loadingBar.name = '_timelineLoadBar';
			_loadingBar.setSize(60, 60);
			_loadingBar.x = Constants.STAGE_WIDTH / 2 - _loadingBar.width / 2;
			_loadingBar.y = Constants.STAGE_HEIGHT / 2 - _loadingBar.height / 2;
			_loadingBar.visible = false;
			addChild(_loadingBar);

			_showBanner = ButtonFactory.createTuningMenuHideButton();
			_showBanner.isSelected = true;
			_showBanner.addEventListener(Event.CHANGE, showBannerChangeHandler);
			addChild(_showBanner);

			enterRowsState();
			//

			var img:Image;
			_settingsButton = new Button();
			img = new Image(ButtonFactory.assetManager.getTexture("garage_settings"));
			_settingsButton.defaultSkin = img;
			_settingsButton.width = _settingsButton.height = 35;
			_settingsButton.addEventListener(Event.TRIGGERED, settingButtonTriggeredHandler);
			addChild(_settingsButton);


			settingsControl = new GarageSettingControl();
			createSortBySection();
			addChild(settingsControl);
			settingsControl.visible = false;
			settingsControl.addEventListener(Event.CLOSE, settingsControlCloseHandler);

			//
			if(Constants.selectedUser && (!_currentUser || Constants.selectedUser.id != _currentUser.id)){
				_userAvatar = new UserCircularAvatar(Constants.selectedUser, 34, true, 1);
				_userAvatar.user = Constants.selectedUser;
				addChild(_userAvatar);
				if(menuButton){
					menuButton.visible = true;
				}
				_userAvatar.addEventListener(TouchEvent.TOUCH, labelTouchHandler);
				titleLabel.addEventListener(TouchEvent.TOUCH, labelTouchHandler);
			}
			//
			//addEventListener(TouchEvent.TOUCH, tuningItemTouchHandler);

			_newBuildBtn = new Button();
			var defaultImg:Image = new Image(ButtonFactory.assetManager.getTexture("create_new_build_btn"));
			defaultImg.color = 0xFF0000;
			defaultImg.scale = ButtonFactory.assetManager.scaleFactor / 2;
			_newBuildBtn.defaultSkin = defaultImg;

			var selectedImg:Image = new Image(defaultImg.texture);
			selectedImg.color = 0xFF0000;
			selectedImg.alpha = 0.75;
			selectedImg.width = defaultImg.width;
			selectedImg.height = defaultImg.height;
			_newBuildBtn.downSkin = selectedImg;

			_newBuildBtn.width = defaultImg.height;
			_newBuildBtn.height = defaultImg.height;
			_newBuildBtn.paddingRight = 25;
			addChild(_newBuildBtn);
			_newBuildBtn.addEventListener(Event.TRIGGERED, newBuildButtonHandler);
		}

		private function newBuildButtonHandler(event:Event):void {
			dispatchEventWith(CREATE_NEW_BUILD);
		}
		
	    public function set tuningsListCollection(value:ListCollection):void{
			if(!_tuningsListCollection || _tuningsListCollection.length == 0){
				hideLoader();
			}
			_tuningsListCollection=value;
			invalidate("tuningsListCollection");
		}
		public function get tuningsListCollection():ListCollection{
			return _tuningsListCollection;
		}

		public function get selectedSocial():String{
			if(postToTimeline){
				return "timeline";
			}
			return tuningsListMenuView.selectedSocial;
		}

        public function getVisibleTuningURL() : String{
            var tuning : Tuning = getSelectedTuning();
            return tuning.fullHdSnapshots[tuning.tuningVisibleSnapShotIndex];
        }

		override public function ShowDone() : void {
			Tracer.print("TuningsListScreen.ShowDone");
			Starling.juggler.delayCall(function () : void{
				var facade : ApplicationFacade = ApplicationFacade.getInstance();
				Starling.juggler.delayCall(function () :void{
					if(!mainMenuView)
							return;
					mainMenuView.updateStates();
					
					var langProxy :LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
					mainMenuView.languages = langProxy.languages;
					
					mainMenuView.updateWithLanguage(AlertView.language);
				}, 2);
				
				
            }, 1);
			
			if (tuningsList && this._tuningsListCollection){
				tuningsList.dataProvider=_tuningsListCollection;
				tuningsList.scrollToDisplayIndex(_selectedPage);
			}
			//BannerStackedScreen.showAdBanner();
			Tracker.trackEvent("Garage Screen", "open");
		}

		override protected function screen_removedFromStageHandler(event:Event):void{
			super.screen_removedFromStageHandler(event);
			BannerStackedScreen.hideAdBanner(true);
		}

		private function createViewSection():void{
			viewLayout = new LayoutGroup();
			var hLayout:HorizontalLayout = new HorizontalLayout();
			hLayout.horizontalAlign = HorizontalAlign.RIGHT;
			hLayout.verticalAlign = VerticalAlign.MIDDLE;
			viewLayout.layout = hLayout;
			viewToggleGroup = new ToggleGroup();

			var toggleButton:ToggleButton;
			var defaultIcon:Image;
			var selectedIcon:Image;

			toggleButton = new ToggleButton();
			toggleButton.paddingLeft = toggleButton.paddingRight = 2.5;
			toggleButton.name = "4";
			defaultIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_4"));
			defaultIcon.color = 0xFEFFFF;
			defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
			toggleButton.defaultIcon = defaultIcon;


			selectedIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_4"));
			selectedIcon.color = 0x51A1F2;
			selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
			toggleButton.defaultSelectedIcon = selectedIcon;
			toggleButton.isSelected = true;
			viewToggleGroup.addItem(toggleButton);
			viewLayout.addChild(toggleButton);

			toggleButton = new ToggleButton();
			toggleButton.paddingLeft = toggleButton.paddingRight = 2.5;
			toggleButton.name = "9";
			defaultIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_9"));
			defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
			defaultIcon.color = 0xFEFFFF;
			toggleButton.defaultIcon = defaultIcon;

			selectedIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_9"));
			selectedIcon.color = 0x51A1F2;
			selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
			toggleButton.defaultSelectedIcon = selectedIcon;
			viewToggleGroup.addItem(toggleButton);
			viewLayout.addChild(toggleButton);

			toggleButton = new ToggleButton();
			toggleButton.paddingLeft = toggleButton.paddingRight = 2.5;
			toggleButton.name = "1";
			defaultIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_1"));
			defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
			defaultIcon.color = 0xFEFFFF;
			toggleButton.defaultIcon = defaultIcon;

			selectedIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_1"));
			selectedIcon.color = 0x51A1F2;
			selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
			toggleButton.defaultSelectedIcon = selectedIcon;
			viewToggleGroup.addItem(toggleButton);
			viewLayout.addChild(toggleButton);

			viewToggleGroup.addEventListener(Event.CHANGE, viewChangeHandler);
			addChild(viewLayout);
		}

		private function labelTouchHandler(event:TouchEvent):void {
			var touch:Touch = event.getTouch(_userLabel, TouchPhase.ENDED);
			var touch2:Touch = event.getTouch(_userAvatar, TouchPhase.ENDED);
			var touch3:Touch = event.getTouch(titleLabel, TouchPhase.ENDED);
			if(touch || touch2 || touch3){
				dispatchEvent(new Event(StackedScreen.USER_DETAILS_SELECTED, false, {user: Constants.selectedUser, options: [UserDetailsPopup.TIMELINE, UserDetailsPopup.EMAILS, UserDetailsPopup.CHALLENGE]}));
			}
		}

		private function createShowBySection():void{
			showByBorderList = [];
			var border:Quad;
			showByLayout = new LayoutGroup();
			var hLayout:HorizontalLayout = new HorizontalLayout();
			hLayout.horizontalAlign = HorizontalAlign.RIGHT;
			hLayout.verticalAlign = VerticalAlign.MIDDLE;
			hLayout.gap = 0;
			hLayout.paddingTop = 3;
			showByLayout.layout = hLayout;


			showByButton = ScreenComponentFactory.createDropDownButton("gallery_clock", "show");
			showByLayout.addChild(showByButton);
			showByButton.addEventListener(Event.CHANGE, showByChangeHandler);
			showByButton.labelFactory = GalleryLabelFactory.garagePeriodLabel;
			showByButton.paddingLeft = showByButton.gap = 5;

			showByButton.width = 135;

			border = createBorder();
			showByBorderList.push(border);
			border.height = showByButton.defaultSkin.height;
			showByLayout.addChildAt(border, 0);

			border = createBorder();
			showByBorderList.push(border);
			border.height = showByButton.defaultSkin.height;
			showByLayout.addChild(border);
			showByLayout.validate();

			addChild(showByLayout);


			showByMenu = new Sprite();
			var menuBackground:Image = new Image(ButtonFactory.assetManager.getTexture("menu_background"));
			menuBackground.scale9Grid = new Rectangle(menuBackground.width/3, menuBackground.height/3, menuBackground.width/3, menuBackground.height/3);
			menuBackground.width = showByLayout.width;
			menuBackground.height = 190;
			menuBackground.filter = new DropShadowFilter(2, Math.PI/4, 0, 0.2);
			menuBackground.y = -1;

			var menuTopBorder:Quad = new Quad( 1, 1, 0xFFFFFF);
			menuTopBorder.width = menuBackground.width + 10;
			menuTopBorder.height = 5;
			menuTopBorder.x = -5;
			menuTopBorder.y = -5;

			showByMenu.addChild(menuBackground);
			showByMenu.addChild(menuTopBorder);

			showByGroup = new LayoutGroup();
			var layout:VerticalLayout = new VerticalLayout();
			layout.gap = 5;
			layout.paddingLeft = layout.paddingRight = 5;
			layout.paddingTop = 6;
			showByGroup.layout = layout;
			showByToggleGroup = new ToggleGroup();

			var radio:ToggleButton;
			_period = RequestPeriod.THIS_YEAR;

			var itemWidth:Number = menuBackground.width - 10;
			radio = ScreenComponentFactory.createRadio("today", RequestPeriod.TODAY);
			radio.width = itemWidth;
			showByGroup.addChild(radio);
			showByToggleGroup.addItem(radio);
			if(_period == RequestPeriod.TODAY){
				showByToggleGroup.selectedItem = radio;
			}


			radio = ScreenComponentFactory.createRadio("yesterday", RequestPeriod.YESTERDAY);
			showByGroup.addChild(createRadioBorder(itemWidth));
			radio.width = itemWidth;
			showByGroup.addChild(radio);
			showByToggleGroup.addItem(radio);
			if(_period == RequestPeriod.YESTERDAY){
				showByToggleGroup.selectedItem = radio;
			}

			radio = ScreenComponentFactory.createRadio("this week", RequestPeriod.THIS_WEEK);
			showByGroup.addChild(createRadioBorder(itemWidth));
			radio.width = itemWidth;
			showByGroup.addChild(radio);
			showByToggleGroup.addItem(radio);
			if(_period == RequestPeriod.THIS_WEEK){
				showByToggleGroup.selectedItem = radio;
			}


			radio = ScreenComponentFactory.createRadio("this month", RequestPeriod.THIS_MONTH);
			showByGroup.addChild(createRadioBorder(itemWidth));
			radio.width = itemWidth;
			showByGroup.addChild(radio);
			showByToggleGroup.addItem(radio);
			if(_period == RequestPeriod.THIS_MONTH){
				showByToggleGroup.selectedItem = radio;
			}


			radio = ScreenComponentFactory.createRadio("last month", RequestPeriod.LAST_MONTH);
			showByGroup.addChild(createRadioBorder(itemWidth));
			radio.width = itemWidth;
			showByGroup.addChild(radio);
			showByToggleGroup.addItem(radio);
			if(_period == RequestPeriod.LAST_MONTH){
				showByToggleGroup.selectedItem = radio;
			}


			radio = ScreenComponentFactory.createRadio("this year", RequestPeriod.THIS_YEAR);
			showByGroup.addChild(createRadioBorder(itemWidth));
			radio.width = itemWidth;
			showByGroup.addChild(radio);
			showByToggleGroup.addItem(radio);
			if(_period == RequestPeriod.THIS_YEAR){
				showByToggleGroup.selectedItem = radio;
			}


			radio = ScreenComponentFactory.createRadio("show all", RequestPeriod.ALL);
			radio.width = itemWidth;
			showByGroup.addChild(createRadioBorder(itemWidth));
			showByGroup.addChild(radio);
			showByToggleGroup.addItem(radio);
			if(_period == RequestPeriod.ALL){
				showByToggleGroup.selectedItem = radio;
			}

			showByGroup.validate();
			menuBackground.height = showByGroup.height + 5;

			showByMenu.addChild(showByGroup);
			addChild(showByMenu);
			showByMenu.visible = false;

			showByChangeHandler(null);
			showByToggleGroup.addEventListener(Event.CHANGE, showByGroupChangeHandler);
		}

		private function createSortBySection():void{
			_sortIconTextures = {};
			_sortIconTextures[RequestSort.VIEWS] = "gallery_eye";
			_sortIconTextures[RequestSort.LIKES] = "gallery_like";
			_sortIconTextures[RequestSort.COMMENTS] = "gallery_comments";
			_sortIconTextures[RequestSort.DATE] = "gallery_date";


			sortByGroup = new LayoutGroup();
			sortByGroup.width = settingsControl.width;

			var layout:VerticalLayout = new VerticalLayout();
			layout.gap = 10;
			layout.paddingTop = 10;
			layout.horizontalAlign = HorizontalAlign.CENTER;
			sortByGroup.layout = layout;
			sortByToggleGroup = new ToggleGroup();

			var radio:ToggleButton;
			var itemWidth:Number = settingsControl.width - 40;

			sortByLabel = new Label();
			sortByLabel.textRendererFactory = LabelFactory.garageSettingLabel;
			sortByLabel.text = "SORT BY";
			sortByLabel.height = 27;
			sortByGroup.addChild(sortByLabel);
			sortByLabel.paddingTop = 4;
			sortByLabel.paddingBottom = 5;
			sortByGroup.addChild(createRadioBorder(itemWidth - 18));


			radio = ScreenComponentFactory.createRadio("Likes", RequestSort.LIKES, true, _sortIconTextures[RequestSort.LIKES], _sortIconTextures[RequestSort.LIKES], true, 0, 0, 0xFEFFFF);
			radio.width = itemWidth;
			sortByGroup.addChild(radio);
			sortByToggleGroup.addItem(radio);
			if(_sort == RequestSort.LIKES){
				sortByToggleGroup.selectedItem = radio;
			}

			radio = ScreenComponentFactory.createRadio("Comments", RequestSort.COMMENTS, true, _sortIconTextures[RequestSort.COMMENTS], _sortIconTextures[RequestSort.COMMENTS], true, 0, 0, 0xFEFFFF);
			radio.width = itemWidth;
			sortByGroup.addChild(radio);
			sortByToggleGroup.addItem(radio);
			if(_sort == RequestSort.COMMENTS){
				sortByToggleGroup.selectedItem = radio;
			}


			radio = ScreenComponentFactory.createRadio("Views", RequestSort.VIEWS, true, _sortIconTextures[RequestSort.VIEWS], _sortIconTextures[RequestSort.VIEWS], true, 0, 0, 0xFEFFFF);
			radio.width = itemWidth;
			sortByGroup.addChild(radio);
			sortByToggleGroup.addItem(radio);
			if(_sort == RequestSort.VIEWS){
				sortByToggleGroup.selectedItem = radio;
			}


			radio = ScreenComponentFactory.createRadio("Date", RequestSort.DATE, true, _sortIconTextures[RequestSort.DATE], _sortIconTextures[RequestSort.DATE], true, 0, 0, 0xFEFFFF);
			radio.width = itemWidth;
			sortByGroup.addChild(radio);
			sortByToggleGroup.addItem(radio);
			if(_sort == RequestSort.DATE){
				sortByToggleGroup.selectedItem = radio;
			}

			sortByGroup.validate();
			settingsControl.addChild(sortByGroup);

			sortByToggleGroup.addEventListener(Event.CHANGE, sortByGroupChangeHandler);
			sortBy = sortByToggleGroup.selectedItem.name;
		}

		private function createSearchView():void{
			searchInput = new SearchInput();
			searchInput.x = 5;
			searchInput.addEventListener(Event.CHANGE, searchInputChangeHandler);
			addChild(searchInput);
		}

		private function createMessageField():void{
			_noResultLabel = new Label();
			_noResultLabel.textRendererFactory = GalleryLabelFactory.sortByTitleLabel;
			_noResultLabel.text = UIStrings.NO_RESULT_LABEL;

			addChild(_noResultLabel);
			_noResultLabel.visible = false;
		}

		private function searchInputChangeHandler(event:Event):void {
			Starling.juggler.removeDelayedCalls(startSearch);
			Starling.juggler.delayCall(startSearch, 1);
		}

		private function startSearch():void{
			_query = searchInput.text;
			// TODO: reload content
		}

		private static function createBorder():Quad{
			var border:Quad = new Quad(1, 1, 0xCCCCCC);
			border.width = 1;
			return border;
		}

		private static function createRadioBorder(width:Number):Quad{
			var border:Quad = new Quad(1, 1, 0xCCCCCC);
			border.height = 1;
			border.width = width;
			return border;
		}

		private function viewChangeHandler(event:Event):void {
			if(viewToggleGroup.selectedItem){
				_countPerPage = parseInt(viewToggleGroup.selectedItem.name);
				if(_countPerPage){
					tuningsRowsList.dataProvider = null;
					Starling.juggler.delayCall(function():void{
						tuningsRowsList.dataProvider = tuningsListCollection;
						tuningsRowsList.setRequiresRedraw();
					}, 0.1);

				}
			}
		}

		private function showByChangeHandler(event:Event):void {
			if(showByButton.isSelected){
				SoundManager.playSound(SoundManager.PART_MENU);
			}else{
				SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
			}
			showByMenu.visible = showByButton.isSelected;
			var str:String = Radio(showByToggleGroup.selectedItem).label;
			showByButton.label = str.charAt(0).toUpperCase() + str.slice(1);
			updateShowByBorders();
			if(showByMenu.visible){
				showByMenu.parent.setChildIndex(showByMenu, showByMenu.parent.numChildren - 1);
				showByMenu.x = showByMenu.x;
				showByMenu.y = BANNER_SIZE + TOOLBAR_SIZE;

			}
			if(showByButton.isSelected){
				Starling.juggler.delayCall(function():void{
					stage.addEventListener(TouchEvent.TOUCH, stageTouchHandler);
				}, .1);
			}
		}

		private function sortByChangeHandler(event:Event):void {
			updateShowByBorders();
		}

		private function showByGroupChangeHandler(event:Event):void {
			if(showByToggleGroup.selectedItem){
				SoundManager.playSound(SoundManager.SELECT_PART);
				_period = showByToggleGroup.selectedItem.name;
				reloadContent();
				if(showByButton.isSelected){
					Starling.juggler.delayCall(function():void{
						showByButton.isSelected = false;
					}, 0.1);
				}
			}
		}

		private function sortByGroupChangeHandler(event:Event):void {
			if(sortByToggleGroup.selectedItem){
				SoundManager.playSound(SoundManager.SELECT_PART);
				showLoader();
				sortBy = sortByToggleGroup.selectedItem.name;
				reloadContent();
				settingsFile ||= File.applicationStorageDirectory.resolvePath("garage_settings.json");
				FileUtil.putFileContent(settingsFile, JSON.stringify(filterSettings));
			}
		}

		private function reloadContent():void {
			tuningsListCollection.removeAll();
			var usersProxy:UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
			var tuningsProxy:TuningsProxy = ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
			tuningsProxy.loadTunings(Constants.selectedUser ? Constants.selectedUser : usersProxy.currentUser, tuningsRowsList.dataProvider.length, sortBy, period, carType, carId, enterChallenge);
		}

		private function updateShowByBorders():void{
			if(showByBorderList && showByButton){
				if(showByButton.isSelected){
					showByBorderList[0].alpha = showByBorderList[1].alpha = 0;
				}else{
					showByBorderList[0].alpha = showByBorderList[1].alpha = 1;
				}
			}
		}

		private function stageTouchHandler(event:TouchEvent):void {
			if(!stage){
				event.currentTarget.removeEventListener(event.type, stageTouchHandler);
				return;
			}
			var menuTouch:Touch;
			var buttonTouch:Touch;
			//
			var button:ToggleButton;
			if(showByMenu && showByMenu.visible){
				menuTouch = event.getTouch(showByMenu);
				buttonTouch = event.getTouch(showByButton);
				button = showByButton;
			}
			//
			var touchBegan:Touch = event.getTouch(stage, TouchPhase.BEGAN);
			if(touchBegan && !menuTouch && !buttonTouch){
				if(button){
					button.isSelected = false;
				}
				stage.removeEventListener(event.type, stageTouchHandler);
				event.stopImmediatePropagation();
			}
		}

        /**
		 * Garage screen dispose
         */
		override public function dispose():void {
			super.dispose();
			if(tuningsListMenuView) {
                tuningsListMenuView.removeFromParent(true);
                tuningsListMenuView = null;
            }

            if(_mainMenuView){
				_mainMenuView.dispose();
				_mainMenuView = null;
			}
            DisplayObjectUtil.disposeMesh(_backgroundImage);
			_backgroundImage = null;
            DisplayObjectUtil.disposeMesh(_backgroundQuad);
			_backgroundQuad = null;
            if(tuningsRowsList){
                tuningsRowsList.removeFromParent();
                tuningsRowsList.dataProvider = null;
				tuningsRowsList.setItemRendererFactoryWithID("cs", null);
				tuningsRowsList.setItemRendererFactoryWithID("fhd", null);
                tuningsRowsList.dispose();
                tuningsRowsList = null;
			}
            if(tuningsList){
                tuningsList.removeFromParent();
                tuningsList.dataProvider = null;
                tuningsList.itemRendererFactory = null;
                tuningsList.dispose();
                tuningsList = null;
			}
            _tuningsListCollection = null;
            if(noTuningsLabel){
                noTuningsLabel.removeFromParent(true);
				noTuningsLabel = null;
			}
            if(titleLabel){
                titleLabel.removeFromParent(true);
				titleLabel = null;
			}

            if(_nextButton){
                _nextButton.removeFromParent(true);
				_nextButton = null;
			}
            if(_prevButton){
                _prevButton.removeFromParent(true);
				_prevButton = null;
			}
            if(pagesLabel){
				pagesLabel.removeFromParent(true);
				pagesLabel = null;
			}
            if(menuButton){
				menuButton.removeFromParent(true);
				menuButton = null;
			}

			if (_loadingBar) {
				_loadingBar.removeFromParent(true);
				_loadingBar = null;
			}

            if(_userLabel){
                _userLabel.removeFromParent(true);
                _userLabel = null;
            }
		}

		public function updateWithLanguage(language : Language) : void {
			if(noTuningsLabel){
				noTuningsLabel.updateWithLanguage(language);
				var msgId:String;
				if(enterChallenge){
					if(carId){
						msgId = "youGarageNoSuchCar";
					}else{
						msgId = "youGarageNoSuchModel";
					}
				}else if((_currentUser && Constants.selectedUser && Constants.selectedUser.id != _currentUser.id) || !_currentUser){
					msgId = "userGarageIsEmpty";
				}else{
					msgId = "yourGarageIsEmpty";
				}
				var msg:String = language.translate("mob_garage", msgId).toUpperCase();
				if(enterChallenge){
					msg = msg.replace(/\[carType]/i, BrandScreenCarChooser.getLabel(carType));
				}
				noTuningsLabel.text = msg;
			}
		
			if (mainMenuView){
				mainMenuView.updateWithLanguage(language);
			}
		}

        public function get topButtonPadding():Number {
            return _topButtonPadding;
        }

        public function set topButtonPadding(value:Number):void {
            if(_topButtonPadding != value) {
                _topButtonPadding = value;
                invalidate("ads");
                invalidate(INVALIDATION_FLAG_SIZE);
                draw();
            }
        }

        private function likeSelectHandler(event:Event):void {
            dispatchEvent(new Event(TuningsListMenuView.LIKE_SELECTED));
        }

        private function commentSelectHandler(event:Event):void {
            dispatchEvent(new Event(TuningsListMenuView.COMMENT_SELECTED));
        }

		public function dispatchExternalEvent(msg:String):void {
			if (mainMenuView) {
				mainMenuView.hide();
			}
			
			dispatchEventWith(msg);
		}
		
		private function onMenuButton(event:Event):void{
			homeButtonTriggeredHandler();
		}

		public function enterOneState():void {
			if(postToTimeline){
				postToTimelineButtonHandler(null);
				return;
			}

			SoundManager.playSound(SoundManager.PART_MENU);
			BannerStackedScreen.hideAdBanner(false);

			tuningsListMenuView.visible=true;
			this._nextButton.visible=true;
			this._prevButton.visible=true;
			
			this.tuningsRowsList.visible=false;
			this.tuningsList.visible=true;
			
			_backgroundImage.visible=true;
			
			this.titleLabel.visible=false;
			this.pagesLabel.visible=false;
			
			_stateRows = false;
			if (mainMenuView){
				mainMenuView.garageThumbMode = false;
			}
            if(closeButton){
                closeButton.visible = true;
            }
			if(menuButton){
				menuButton.visible = false;
			}
            activateCloseButton();
			this.invalidate();
            tuningsList.height = actualHeight + 50;
            if(_showBanner){
                _showBanner.visible = true;
            }
			if(_userLabel){
				_userLabel.visible = true;
			}

			if(_userAvatar){
				_userAvatar.visible = false;
			}

			if(settingsControl && isSettingOpen){
				settingButtonTriggeredHandler(null);
			}

			if(_settingsButton){
				_settingsButton.visible = false;
			}

			if(_newBuildBtn){
				_newBuildBtn.visible = false;
			}

            menuAnimationUpdate();
            Starling.juggler.delayCall(function():void{
                tuningsListMenuView.currentTuning = getSelectedTuning();
                tuningsListMenuView.setCarName();
			}, .03);
		}

		public function enterRowsState():void {
			if(isInitialized){
				SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
			}
			_backgroundImage.visible=false;
			
			tuningsListMenuView.visible=false;

			this._nextButton.visible=false;
			this._prevButton.visible=false;
			
			this.tuningsRowsList.visible=true;
			tuningsRowsList.scrollToDisplayIndex(_selPage);
			this.tuningsList.visible=false;
			
			this.titleLabel.visible=true;
			this.pagesLabel.visible=true;
			
			_stateRows=true;
			
			if (mainMenuView){
				mainMenuView.garageThumbMode = true;
			}
            Tracer.print("enterRowsState()");
			this.invalidate();
            menuButton.visible = true;
			if(closeButton){
				closeButton.visible = enableBackward;
			}
			if(_showBanner){
				_showBanner.visible = false;
			}
			if(_userLabel){
				_userLabel.visible = false;
			}

			if(_userAvatar){
				_userAvatar.visible = true;
			}

			if(_settingsButton){
				_settingsButton.visible = true;
			}

			if(_newBuildBtn){
				_newBuildBtn.visible = true;
			}

			tuningsListMenuView.enterNormalState();

			if(!enterChallenge && onComplete == null){
				Starling.juggler.delayCall(BannerStackedScreen.showAdBanner, 0.25);
			}
		}

		private function zoomModeEndHandler(event:Event):void {
			touchable = true;
			_tuningView.removeEventListener(TuningItemRenderer.ZOOM_MODE_START, zoomModeStartHandler);
			_tuningView.removeEventListener(TuningItemRenderer.ZOOM_MODE_END, zoomModeEndHandler);
			_tuningView = null;
		}

		private function zoomModeStartHandler(event:Event):void {
			touchable = false;
		}

		private function tuningItemTouchHandler(event:TouchEvent):void {
			if(!_stateRows && event.touches && event.touches.length > 1){
				_tuningView = selectedTuningRenderer;
				if(_tuningView){
					_tuningView.addEventListener(TuningItemRenderer.ZOOM_MODE_START, zoomModeStartHandler);
					_tuningView.addEventListener(TuningItemRenderer.ZOOM_MODE_END, zoomModeEndHandler);
					_tuningView.startZoomMode();
					_tuningView.zoom(event.touches);
				}

			}
		}
		
		protected function onTuningDelete(event:Event):void{
			dispatchEventWith(TUNING_DELETE, false, event.data);
		}
		
		protected function onTuningShare(event:Event):void{
			
			dispatchEventWith(TUNING_SHARE);
		}

        private function filterButtonHandler(event:Event):void {
            if (!selectedTuning) {
                return;
            }
            var userName:String = selectedTuning.userObj.username;
            var shortId:String = selectedTuning.shortId;
            var car:Car = selectedTuning.car;

			var renderer:TuningItemRenderer = tuningsList.itemToItemRenderer(tuningsList.dataProvider.getItemAt(tuningsList.horizontalPageIndex)) as TuningItemRenderer;
			var index:int = renderer.selectedIndex;
            var urlToSend:String = car.getEditorLink(userName, shortId, index, selectedTuning.hdQuality);
            var obj:Object = {
                url: urlToSend,
                tuning: selectedTuning,
				isChallenge: false
            };

            ApplicationFacade.getInstance().sendNotification(Notifications.DISPLAY_TUNING_EDITOR_POPUP, obj);
        }

        private function postToTimelineButtonHandler(event:Event):void {
            dispatchEvent(new Event(TuningsListScreen.TUNING_SHARE));
        }
		
		protected function onPrevButtonHandler(event:Event):void {
			SoundManager.playSound(SoundManager.SELECT_PART_BACK);
			if(_selPage-1<0) {
				return;
			}
			
			_selPage-=1;
			tuningsList.scrollToDisplayIndex(_selPage, 0.3)
		}
		
		protected function onNextButtonHandler(event:Event):void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			if(_selPage+1>=tuningsList.horizontalPageCount) {
				return;
			}
			
			_selPage+=1;
			tuningsList.scrollToDisplayIndex(_selPage, 0.3)
		}

		private var _selectedTuningID : String;
		private var isScrolling:Boolean = false;
		public function set selectedTuningID(value:String) : void{
            _selectedTuningID = value;
		}

        public function get selectedTuningID() : String{
            return _selectedTuningID;
        }

        private function getSocialData() : void {
            if (!Constants.selectedUser) {
                return;
            }
			dispatchEventWith(GET_SOCIAL_DATA);
		}

        public function set comments(value:Array) : void
        {
            tuningsListMenuView.initComments = value;
        }

        public function set likes(value:int) : void
        {
            tuningsListMenuView.likes = value;
        }

		protected function onTuningSelected(event:Event):void {
			_selectedPage = _tuningsListCollection.getItemIndex(getSelectedTuning());
			dispatchEventWith(TUNING_SELECTED);
            getSocialData();
		}

        protected function onCarSelected(event:Event):void
        {
            dispatchEventWith(CAR_SELECTED, false, event.data);
        }


		public function cleanupMemory():void
		{
			tuningsList.dataProvider=null;
			tuningsRowsList.dataProvider=null;
			Tracer.print("cleanUPmemory", "TUNING LIST SCREEN");
		}
		   
		public function beginScreenShot(resultHandler : Function = null) : void {
			_nextButton.visible = false;
			_prevButton.visible = false;
			tuningsListMenuView.visible = false;
			menuButton.visible = false;
			_showBanner.visible = false;

			pagesLabel.visible = false;
			closeButton.visible = false;
			var renderer:TuningItemRenderer = tuningsList.itemToItemRenderer(tuningsList.dataProvider.getItemAt(tuningsList.horizontalPageIndex)) as TuningItemRenderer;
			renderer.pageIndicator.visible = false;

			tuningsList.dispatchEventWith(TuningsListScreen.BUTTONS_HIDE);
			//showLoader();
			resultHandler();
		}


		public function showLoader():void {
			if(_loadingBar){
				_loadingBar.visible = true;
			}
		}

		public function hideLoader():void {
			if(_loadingBar){
				_loadingBar.visible = false;
			}
		}

		public function endScreenShot( ) : void{
			_nextButton.visible = true;
			_prevButton.visible = true;
			tuningsListMenuView.visible = true;
			menuButton.visible = true;
			_showBanner.visible = true;

			pagesLabel.visible = true;
			if(closeButton){
				closeButton.visible = enableBackward || !_stateRows;
			}
			var renderer:TuningItemRenderer = tuningsList.itemToItemRenderer(tuningsList.dataProvider.getItemAt(tuningsList.horizontalPageIndex)) as TuningItemRenderer;
			renderer.pageIndicator.visible = true;
			tuningsList.dispatchEventWith(TuningsListScreen.BUTTONS_SHOW);
//			hideLoader(); TODO: check

		}
		
		private function initializeTuningsRowsList() : void {
		   	var layout : TiledRowsLayout = new TiledRowsLayout();
			layout.paging = Direction.HORIZONTAL;
			layout.useSquareTiles = false;
			
			layout.useVirtualLayout = true;
			//layout.manageVisibility = true; //tt
			layout.gap = 0;
			layout.horizontalGap=0;
			layout.verticalGap=0;
			layout.horizontalAlign = HorizontalAlign.CENTER;
			layout.verticalAlign = VerticalAlign.MIDDLE;
			tuningsListCollection = new ListCollection();
			
			tuningsRowsList = new List();
			tuningsRowsList.minimumPageThrowVelocity=1;
			tuningsRowsList.dataProvider = tuningsListCollection;
			tuningsRowsList.layout = layout;
			tuningsRowsList.snapToPages = true;
			tuningsRowsList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
			tuningsRowsList.horizontalScrollPolicy = ScrollPolicy.ON;
			tuningsRowsList.addEventListener(Event.SCROLL, tuningsRowsListScrollHandler);
			tuningsRowsList.addEventListener(Event.CHANGE, tuningsRowsListChangeHandler);
			tuningsRowsList.factoryIDFunction = function(item:Object, index:int):String {
				var tuningItem:Tuning = item as Tuning;
				if(tuningItem && tuningItem.hdQuality) {
					return "fhd";
				}
				return "cs";
			};
			tuningsRowsList.setItemRendererFactoryWithID("cs", itemCSRendererFactory);
			tuningsRowsList.setItemRendererFactoryWithID("fhd", itemFHDRendererFactory);

			addChild(tuningsRowsList);
		}

		private function itemCSRendererFactory() : IListItemRenderer {
			var renderer : TuningRowsItemRenderer = new TuningRowsItemRenderer();
			renderer.pageContentCount = rowCount * colCount;
			var item_w:Number = (actualWidth / rowCount);
			var item_h:Number = (actualHeight - (BANNER_SIZE + TOOLBAR_SIZE)) / colCount;
			renderer.setSize(item_w, item_h);
			return renderer;
		}

		private function itemFHDRendererFactory() : IListItemRenderer {
			var renderer : TuningRowsFHDItemRenderer = new TuningRowsFHDItemRenderer();
			renderer.pageContentCount = rowCount * colCount;
			var item_w:Number = (actualWidth / rowCount);
			var item_h:Number = (actualHeight - (BANNER_SIZE + TOOLBAR_SIZE)) / colCount;
			renderer.setSize(item_w, item_h);
			return renderer;
		}

		private function tuningsRowsListScrollHandler(event:Event):void{
			if(!isScrolling && tuningsRowsList.isScrolling){
				isScrolling = true;
				SoundManager.playSound(SoundManager.SELECT_PART);
			}else if(!tuningsRowsList.isScrolling){
				isScrolling = false;
			}
			invalidate("tuningsRowsScroll");
		}
		
		private function tuningsRowsListChangeHandler(event:Event):void {
			if(enterChallenge){
				if(tuningsRowsList && tuningsRowsList.dataProvider){
					_selectedChallengeTuning = tuningsRowsList.dataProvider.getItemAt(tuningsRowsList.selectedIndex) as Tuning;
				}
				if(_selectedChallengeTuning){
					_selectedChallengeTuning.challengeNewSystemCar = true;
					_selectedChallengeTuning.car.isChallenge = true;
					tuningsRowsList.removeEventListener(Event.CHANGE, tuningsRowsListChangeHandler);
					var params:Object = {isEditing: true, challengeData: challengeData};
					dispatchEventWith(CAR_SELECTED, false, params);
				}
				return;
			}
			if(tuningsRowsList.selectedIndex<0) {
				return;
			}
			_selPage = tuningsRowsList.selectedIndex;
			tuningsList.scrollToDisplayIndex(_selPage, 0);
			this.enterOneState();
			tuningsRowsList.selectedIndex=-1;

		}
		private function initializeTuningsList() : void {
			var layout : HorizontalLayout = new HorizontalLayout();
			layout.gap = 0;
			layout.horizontalAlign = HorizontalAlign.CENTER;
			layout.verticalAlign = VerticalAlign.MIDDLE;
			layout.useVirtualLayout = true;
			tuningsListCollection = new ListCollection();
						
			tuningsList = new List();
			tuningsList.visible=false;
			tuningsList.minimumPageThrowVelocity=1;
			tuningsList.dataProvider = tuningsListCollection;
			tuningsList.layout = layout;
			tuningsList.snapToPages = true;
			tuningsList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
			tuningsList.horizontalScrollPolicy = ScrollPolicy.ON;
			tuningsList.verticalScrollPolicy = ScrollPolicy.OFF;
			tuningsList.addEventListener(Event.SCROLL, tuningsListScrollHandler);
			tuningsList.itemRendererFactory = function() : IListItemRenderer {
				var renderer : TuningItemRenderer = new TuningItemRenderer();
				renderer.scale = Constants.TUNING_SCALE_FACTOR;
				var item_w:Number = actualWidth / renderer.scale;
				var item_h:Number = (actualHeight + 50) / renderer.scale;
				renderer.setSize(item_w, item_h);
                renderer.overrideDispose = true;
                _tuningRendererList.push(renderer);
				return renderer;
			};
            addChild(tuningsList);
		}
		
		public function get loadedTuningsCount() : int {
			return tuningsListCollection.length;
		}
		
		public function get pagesCount() : int {
			return tuningsList.horizontalPageCount;
		}
		
		public function get selectedTuning() : Tuning {
			return getSelectedTuning();//tuningsList.selectedItem as Tuning;
		}
		
		public function get selectedIndex() : int {
			return _tuningsListCollection.getItemIndex(getSelectedTuning());//tuningsList.selectedIndex;
		}

		public function get selectedTuningFrameIndex() : int {
            var renderer:TuningItemRenderer = tuningsList.itemToItemRenderer(tuningsList.dataProvider.getItemAt(tuningsList.horizontalPageIndex)) as TuningItemRenderer;
            return renderer.selectedIndex;
		}

		public function get selectedTuningRenderer() : TuningItemRenderer {
            return tuningsList.itemToItemRenderer(tuningsList.dataProvider.getItemAt(tuningsList.horizontalPageIndex)) as TuningItemRenderer;
		}
		
		public function get addedTunings() : ListCollection {
			return tuningsListCollection;
		}
		
		private function addElement(item:*,index:int=-1) : void {
			for (var i:int = 0; i < numChildren; i++) {
				if (getChildAt(i).name == item.name) return;
			}
			index == -1 ? addChild(item) : addChildAt(item,index);
		}

        override protected function draw() : void {
			super.draw();
			
			if (!mainMenuView){
				mainMenuView = new MainMenuView();
				mainMenuView.name = "mainMenuView";
				mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight);
				mainMenuView.parentActualWidth = actualWidth;
				mainMenuView.parentActualHeight = actualHeight;
				addElement(mainMenuView,this.numChildren-1);

				mainMenuView.addEventListener(MainMenuView.START_SHOW, menuOpenHandler);
				mainMenuView.addEventListener(Event.CLOSE, menuCloseHandler);
			}
			var hasCloseButton:Boolean = (closeButton && closeButton.visible);
			var sizeInvalid : Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
			
			if(sizeInvalid) {
				_backgroundQuad.width = actualWidth;
				_backgroundQuad.height = actualHeight;
				_backgroundImage.width = actualWidth;
				_backgroundImage.height = actualHeight;


				_headerBackground.width = actualWidth;
				_headerBackground.height = BANNER_SIZE;

				tuningsList.x = 0;
				tuningsList.width = actualWidth;// - 10;
				tuningsList.height = actualHeight + 50;// - topBarHeight - 11;
				
				var imageWidth:Number=actualWidth /3;
				//var imageHeight:Number=imageWidth/1.54918032786885;
				var imageHeight:Number=actualHeight * 0.42 - _topButtonPadding/2;
				
				tuningsRowsList.x = 0;
				tuningsRowsList.y = BANNER_SIZE + TOOLBAR_SIZE;
				tuningsRowsList.width = actualWidth;
				tuningsRowsList.height = actualHeight - (BANNER_SIZE + TOOLBAR_SIZE);

				menuButton.x = menuButton.y = (BANNER_SIZE - menuButton.height) / 2;
				if(Device.isIphoneXVariant()){
					menuButton.x += 6;
				}
				
				if(Device.isIphoneXVariant()){
					menuButton.x+=6;
				}

				if(enterChallenge){
					titleLabel.text = "Choose your model";
				}else{
					titleLabel.text = Constants.selectedUser ? Constants.selectedUser.nameToShow: AlertView.language.translate("mob_garage", "myGarage").toUpperCase();
				}

				var str : String = titleLabel.text;
				if (str.indexOf(" ") != -1){
					str = str.split(" ")[0];
				}

                if (str.indexOf("_") != -1){
                    str = str.split("_")[0];
                }

				if (str.length > 8){
					str.substr(0,8);
				}

				// mainMenuView.updateGaragLabel(str + " " + AlertView.language.translate("mob_garage", "myGarage").toUpperCase().split(" ")[1]);

				titleLabel.validate();
				if(titleLabel){
					if(enterChallenge){
						titleLabel.x = (actualWidth - titleLabel.width) / 2;
					}else{
						titleLabel.x = menuButton.x + menuButton.width + 9;
					}
					titleLabel.y = (BANNER_SIZE - titleLabel.height)/2;
				}
				if(!enterChallenge){
					titleLabel.width = 150;
				}

				noTuningsLabel.width = tuningsList.width;
				noTuningsLabel.y = int((tuningsList.height - noTuningsLabel.height) / 2);

				tuningsListMenuView.width = (actualWidth||Starling.current.stage.stageWidth);
				tuningsListMenuView.height = 50;
				tuningsListMenuView.y = actualHeight - tuningsListMenuView.height;
                tuningsListMenuView.parentHeight = actualHeight;
                tuningsListMenuView.setCarName();
				tuningsListMenuView.invalidate(INVALIDATION_FLAG_SIZE);
				
				_nextButton.x = actualWidth-_nextButton.width/2;
				_nextButton.y = (actualHeight)/2;
				
				_prevButton.x = _prevButton.width/2;
				_prevButton.y = (actualHeight)/2;


				if (mainMenuView){
					mainMenuView.y =_topButtonPadding;
					mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight -_topButtonPadding);
					mainMenuView.parentActualWidth = actualWidth;
					mainMenuView.parentActualHeight = actualHeight - _topButtonPadding;
				}

			}

			if(viewLayout){
				viewLayout.x = actualWidth - viewLayout.width - 7.5;
				viewLayout.y = (BANNER_SIZE - viewLayout.height)/2;
			}

			if(_settingsButton){
				_settingsButton.x = actualWidth - (_settingsButton.width + 15 + (hasCloseButton?(closeButton.width + 5):0));
				_settingsButton.y = (BANNER_SIZE - _settingsButton.height) / 2;
				pagesLabel.x = actualWidth - (pagesLabel.width + _settingsButton.width + (hasCloseButton?(closeButton.width + 5):0) + 35) - (searchInput? (searchInput.width + 10): 0);
			}

			if(_newBuildBtn){
				_newBuildBtn.x = actualWidth - _newBuildBtn.width;
				_newBuildBtn.y = actualHeight - (_newBuildBtn.height + 24);
			}



			if(!isSettingOpen && settingsControl){
				settingsControl.x = actualWidth;
			}

			if(showByLayout){
				showByLayout.x = 135;
				showByLayout.y = BANNER_SIZE + (TOOLBAR_SIZE - (showByLayout.height + HorizontalLayout(showByLayout.layout).padding)) / 2;
				if(showByMenu){
					showByMenu.x = showByLayout.x;
					showByMenu.y = BANNER_SIZE + TOOLBAR_SIZE;
				}
			}

			if(searchInput){
				searchInput.x = actualWidth - searchInput.width - 10;
				searchInput.y = BANNER_SIZE + (TOOLBAR_SIZE - searchInput.height) / 2;
			}
			
			if (this.isInvalid("tuningsListCollection")){
				tuningsList.dataProvider=_tuningsListCollection;
				tuningsRowsList.dataProvider=_tuningsListCollection;
				tuningsList.scrollToDisplayIndex(_selPage);
			}

			if (this.isInvalid("ads") || sizeInvalid) {
				tuningsList.x = 0;
				tuningsList.width = actualWidth;
				tuningsList.height = actualHeight + 50;
				
                menuButton.y = _topButtonPadding + 6;
				titleLabel.y = (BANNER_SIZE - titleLabel.height) / 2 - 2;
			}

			if(isInvalid("tuningsRowsScroll") || isInvalid("tuningsListCollection") || sizeInvalid || isInvalid("ads")) {

				pagesLabel.text = (tuningsRowsList.horizontalPageIndex + 1) + "/" + tuningsRowsList.horizontalPageCount;
				pagesLabel.updateWithLanguage(AlertView.language);
				pagesLabel.width = NaN;
				pagesLabel.height = 13;
				pagesLabel.validate();

				pagesLabel.x = actualWidth - (pagesLabel.width + _settingsButton.width + (hasCloseButton?(closeButton.width + 5):0) + 35) - (searchInput? (searchInput.width + 10): 0);
				pagesLabel.y = (BANNER_SIZE - pagesLabel.height) / 2;

				if (mainMenuView) {
					mainMenuView.y = _topButtonPadding;
					mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight - _topButtonPadding);
					mainMenuView.parentActualWidth = actualWidth;
					mainMenuView.parentActualHeight = actualHeight - _topButtonPadding;
				}


				if (tuningsRowsList.horizontalPageIndex + 1 === tuningsRowsList.horizontalPageCount && !tuningsIsLoading && tuningsRowsList.horizontalPageCount != 1) {
					tuningsIsLoading= true;
					var usersProxy:UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
					var tuningsProxy:TuningsProxy = ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
					tuningsProxy.loadTunings(Constants.selectedUser ? Constants.selectedUser : usersProxy.currentUser, tuningsRowsList.dataProvider.length, sortBy, period, carType, carId, enterChallenge);
				}
			}
            if(_userLabel){
                _userLabel.x = _gap;
                _userLabel.y = _gap;
            }
			if(_userAvatar){
				_userAvatar.x = _gap;
				_userAvatar.y = (BannerStackedScreen.BANNER_SIZE - _userAvatar.height) / 2;
			}
            menuAnimationUpdate();
		}

		private function menuOpenHandler(event:Event):void {
			BannerStackedScreen.hideAdBanner(false);
		}

		private function menuCloseHandler(event:Event):void {
			BannerStackedScreen.showAdBanner();
		}

		private function challengeHandler(event:Event):void{
            var user : User  = (ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy).currentUser;
			var tuning:Tuning = (!_stateRows && selectedTuning && !Constants.selectedUser)?selectedTuning:null;
			var data:Object  = ChallengeEvent.getData(user, tuning, Constants.selectedUser, null);
            dispatchEvent(new Event(ChallengeEvent.CHALLENGE_REQUEST, false, data));
		}

        public function activateCloseButton(closeHandler:Function = null): void {
            _closeHandler = closeHandler;
            if(!closeButton){
                closeButton = ButtonFactory.screenCloseButton();
                closeButton.addEventListener(Event.TRIGGERED, closeButtonTriggeredHandler);
            }
			if(!closeButton.parent){
				addChild(closeButton);
			}

			closeButton.visible = enableBackward || !_stateRows;
			closeButton.x = actualWidth-closeButton.width-_gap;
			closeButton.y = _gap;
        }

        private function closeButtonTriggeredHandler(event:Event):void {
			if(_stateRows){
				Constants.selectedUser = null;
                if(_closeHandler != null){
                    _closeHandler.call();
				}else{
					dispatchEventWith(StackedScreenNavigator.BACK_TO_SCREEN);
				}
			}else{
                enterRowsState();
			}
        }
		
		public function removeTuning():void{
			this.tuningsList.dataProvider.removeItem(this.getSelectedTuning());
		}

		public function updateTuning():void{
			tuningsList.dataProvider.updateItemAt(tuningsList.horizontalPageIndex);
            Starling.juggler.delayCall(menuAnimationUpdate, .01);
		}

        public function homeButtonTriggeredHandler(event:Event = null):void {
			if(enterChallenge){
				dispatchEventWith(StackedScreen.CHALLENGES_SELECTED);
			}else{
				dispatchEventWith(StackedScreen.HOME_SELECTED);
			}
        }

        public function brandsScreenButtonTriggeredHandler(event : Event=null) : void
		{
			dispatchEventWith(StackedScreen.BRANDS_SELECTED);
		}
		
		public function getSelectedTuning():Tuning {
			if(enterChallenge && _selectedChallengeTuning){
				return _selectedChallengeTuning;
			}
			if(!tuningsList) {
				return null;
            }
			if(!this._tuningsListCollection){
				return null;
            }
            if(tuningsList.horizontalPageIndex<0){
				return null;
            }
			if(tuningsList.horizontalPageIndex>=_tuningsListCollection.length){
				return null;
            }
			var tuning:Tuning = _tuningsListCollection.getItemAt(tuningsList.horizontalPageIndex) as Tuning;
			if (!tuning){
				return null;
            }
			return tuning;
		}
		
		private function tuningsListScrollHandler(event : Event) : void {
			var tuning:Tuning = getSelectedTuning();
			if (this._stateRows) {
				tuningsListMenuView.visible=false;
				if(!tuning) {
					tuningsListMenuView.visible=false;
					return;
				}
			} else {
                if(!tuning){
                    tuningsListMenuView.visible = false;
                    return;
                }else{
                    tuningsListMenuView.visible = true;
                }
            }
			
			if (tuningsList.horizontalPageIndex - 1 < 0 ) {
				this._prevButton.alpha=0.5;
            } else {
				_prevButton.alpha=1;
            }
			if (tuningsList.horizontalPageIndex + 1 >= this.tuningsList.horizontalPageCount) {
				this._nextButton.alpha=0.5;
            } else {
                _nextButton.alpha = 1;
            }
			_selPage = tuningsList.horizontalPageIndex;
			
			selectedTuningID = tuning.id;
            tuningsListMenuView.currentTuning = tuning;
            tuningsListMenuView.currentTuningID = _selectedTuningID;
            var user:User = new User(tuning.userObj);
            if((_currentUser && user && user.id != _currentUser.id) || !_currentUser){
                Constants.selectedUser = user;
                _userLabel ||= new ChallengeViewUserLabel();
				_userLabel.num = "";
                _userLabel.user = user;
                addChild(_userLabel);
				_userLabel.visible = !_stateRows;

				if(_userAvatar){
					_userAvatar.visible = _stateRows;
				}
				menuButton.visible = false;
				if(!_userLabel.hasEventListener(TouchEvent.TOUCH, labelTouchHandler)){
					_userLabel.addEventListener(TouchEvent.TOUCH, labelTouchHandler);
				}
            }else{
                Constants.selectedUser = null;
                if(_userLabel){
                    _userLabel.removeFromParent(true);
                    _userLabel = null;
				}
            }
            if (tuning.car) {
                tuningsListMenuView.currentCarID = tuning.car.id;
            }
            if (!tuningsList.isScrolling){
                getSocialData();
            }
			if (tuningsList.horizontalPageIndex + 1 === tuningsList.horizontalPageCount && !tuningsIsLoading && tuningsList.horizontalPageCount != 1) {
				tuningsIsLoading= true;
				var usersProxy:UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
				var tuningsProxy:TuningsProxy = ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
				tuningsProxy.loadTunings(Constants.selectedUser ? Constants.selectedUser : usersProxy.currentUser, tuningsList.dataProvider.length, sortBy, period, carType, carId, enterChallenge);
			}
			Starling.juggler.delayCall(function ():void {
                if(tuningsListMenuView){
					tuningsListMenuView.setCarName();
                }
            }, .03);
            menuAnimationUpdate();
		}

        private function showBannerChangeHandler(event:Event):void {
            if(_menuTween){
                _menuTween.kill();
            }
            if(_showBanner.isSelected){
                _menuTween = new TweenLite(tuningsListMenuView, 0.3, {y: actualHeight - tuningsListMenuView.height, onUpdate:menuAnimationUpdate});
            }else{
                _menuTween = new TweenLite(tuningsListMenuView, 0.3, {y: actualHeight, onUpdate:menuAnimationUpdate});
            }
        }

        private function menuAnimationUpdate():void{
            _showBanner.x = (actualWidth - _showBanner.width) / 2;
            _showBanner.y = tuningsListMenuView.y - (_showBanner.height - 10);
            if(tuningsList){
                tuningsList.y = -(actualHeight - tuningsListMenuView.y) / 2;
            }
            tuningsListMenuView.updateLabelPositions();
            updatePageIndicatorPosition();
        }

        private function updatePageIndicatorPosition():void{
			if(_tuningRendererList.length) {
                for (var i:int = _tuningRendererList.length - 1; i >= 0; i--) {
                    if (!_tuningRendererList[i].pageIndicator) {
                        _tuningRendererList.removeAt(i);
                    } else {
						var itemScale:Number = Constants.TUNING_SCALE_FACTOR;
                        _tuningRendererList[i].pageIndicator.y = (_showBanner.y - (_tuningRendererList[i].pageIndicator.height + 5) - tuningsList.y) / itemScale;
                    }
                }
            }
        }
		

		public function set mainMenuView(value : MainMenuView) :void
		{
			this._mainMenuView = value;
            this._mainMenuView.owner = this;
		}

        public function get mainMenuView():MainMenuView {
            return this._mainMenuView;
        }

        public function set selectedLanguage(value:Language):void {
			this._selectedLanguage =value;
        }

        public function get selectedLanguage():Language {
            return this._selectedLanguage;
        }

        public function showNoTuningsMessage(value:Boolean) : void
		{
			if (noTuningsLabel) {
				noTuningsLabel.visible = value;
			}

			if(onComplete != null){
				onComplete();
				onComplete = null;
			}
		}

		override public function transitionComplete():void{
			BannerStackedScreen.showAdBanner();
		}
		
		public function set languages(value:Array):void{
			_languages=value;
		}
		public function get languages():Array{
			return _languages;
		}

        public function appendChallenges(challenges:Array):void{
            _challengesList ||= {};
            for(var i:int = 0;  i < challenges.length; i++){
                _challengesList[challenges[i]] = true;
            }
			for(var j:int = 0; j < tuningsList.dataProvider.length; j++){
				var tuning:Tuning = tuningsList.dataProvider.getItemAt(j) as Tuning;
				tuning.isChallenging = _challengesList[tuning.id];
			}
            var tuningsListCollection:ListCollection;
			if(tuningsList && tuningsList.dataProvider){
                tuningsList.dataProvider.updateItemAt(tuningsList.horizontalPageIndex);
				Starling.juggler.delayCall(menuAnimationUpdate, .01);
			}
			if(tuningsRowsList && tuningsRowsList.dataProvider){
				if(_stateRows){
					_selPage = (tuningsRowsList.horizontalPageIndex + 1) * _countPerPage - 1;
				}else{
					_selPage = tuningsList.horizontalPageIndex;
				}
                tuningsListCollection = tuningsRowsList.dataProvider as ListCollection;
                tuningsRowsList.dataProvider = null;
                tuningsRowsList.dataProvider = tuningsListCollection;
				tuningsRowsList.scrollToDisplayIndex(_selPage);
			}
        }

        public function get challengesList():Object {
            return _challengesList;
        }

		public function set requireLogin(value:Boolean):void{
			if(tuningsListMenuView){
				tuningsListMenuView.requireLogin = value;
			}
		}

		public function setTuningIsChallenge(tuningId: String, status: Boolean): void {
			for(var j:int = 0; j < tuningsList.dataProvider.length; j++){
				var tuning:Tuning = tuningsList.dataProvider.getItemAt(j) as Tuning;
				if (tuning.id == tuningId) {
					tuning.isChallenging = status;
				}
			}
		}

		public function get sortBy():String {
			return _sort;
		}

		public function set sortBy(value:String):void {
			_sort = value;
			filterSettings.sortBy = value;
		}

		public function get period():String {
			return _period;
		}

		private function settingButtonTriggeredHandler(event:Event):void {
			_settingsButton.visible = isSettingOpen;
			if(closeButton){
				closeButton.visible = isSettingOpen && (enableBackward || !_stateRows);
			}
			if(event){
				if(isSettingOpen){
					SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
				}else{
					SoundManager.playSound(SoundManager.PART_MENU);
				}
			}
			pagesLabel.visible = isSettingOpen;
			if(isSettingOpen){
				TweenLite.to(settingsControl, 0.3, {x:actualWidth, onComplete:function ():void {
						settingsControl.visible = false;
						isSettingOpen = false;
						if(_stateRows){
							AdsManager.$.bannerShow(BannerScreen.BANNER_SCREEN);
						}
					}});
			}else{
				settingsControl.visible = true;
				AdsManager.$.bannerHide();
				TweenLite.to(settingsControl, 0.3, {x:(actualWidth - settingsControl.width), onComplete:function ():void {
						isSettingOpen = true;
					}});
			}
		}

		private function settingsControlCloseHandler(event:Event):void {
			settingButtonTriggeredHandler(null);
		}

		public function updateCurrentTuning(tuning:Tuning):void{
			for(var i:int = 0; i < tuningsList.dataProvider.length; i++){
				var dataTuning:Tuning = tuningsList.dataProvider.getItemAt(i) as Tuning;
				if(dataTuning && dataTuning.id == tuning.id){
					tuningsList.dataProvider.setItemAt(tuning, i);
					tuningsList.dataProvider.updateItemAt(i);
					break;
				}
			}
		}

		public function get onComplete():Function {
			return _onComplete;
		}

		public function set onComplete(value:Function):void {
			_onComplete = value;
		}
	}
}