package com.despark.tuning3d.view.components.tunings.colors {
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.ExtentedButton;

	import feathers.controls.Button;
	import feathers.core.FeathersControl;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.textures.Texture;

	public class EffectMenu extends FeathersControl {
		public static const TOGGLE_STATE:String="toogle_state";
		public static const TOGGLE_STATE_ERROR:String="toogle_state_error";
		private var backgroundImage:Image;
		private var closeButton:Button;
		private var frontLightButton:Button;
		private var diskNeonButton:Button;
		
		private var neonButton:Button;
		
		private var _car:Car;
		
		private var _frontLightEnabled:Boolean=false;
		private var _diskNeonEnabled:Boolean=false;
		private var _neonEnabled:Boolean=false;

		public function EffectMenu() {
			super();
		}
		
		override protected function initialize():void{
			super.initialize();
			var backgroundTexture:Texture=AlertView.assetManager.getTexture("a_effect_bg");
			backgroundImage=new Image(backgroundTexture);
			addChild(backgroundImage);

			frontLightButton=ButtonFactory.circleButton("a_front_light_effect");
			frontLightButton.name = "front_light";
			frontLightButton.width = 78/2;
			frontLightButton.height = 43/2;
			frontLightButton.addEventListener(Event.TRIGGERED, onToggleState);
			(frontLightButton.defaultSkin as Image).color=0xfefefe;
			(frontLightButton.downSkin as Image).color=0xfefefe;
			addChild(frontLightButton);
			
			diskNeonButton=ButtonFactory.circleButton("a_disk_neon_effect");
			diskNeonButton.name = "disk_neon";
			diskNeonButton.width = 50/2;
			diskNeonButton.height = 50/2;
			diskNeonButton.addEventListener(Event.TRIGGERED, onToggleState);
			(diskNeonButton.defaultSkin as Image).color=0xfefefe;
			(diskNeonButton.downSkin as Image).color=0xfefefe;
			addChild(diskNeonButton);
			
			
			neonButton=ButtonFactory.circleButton("a_neon_effect");
			neonButton.name = "neon";
			neonButton.width = 129/2;
			neonButton.height = 41/2;
			neonButton.addEventListener(Event.TRIGGERED, onToggleState);
			(neonButton.defaultSkin as Image).color=0xfefefe;
			(neonButton.downSkin as Image).color=0xfefefe;
			addChild(neonButton);
			
			closeButton=ButtonFactory.circleButton("a_close_effect_menu");
			(closeButton as ExtentedButton ).hitAreaPadding=30;
			closeButton.width = 19/2;
			closeButton.height = 20/2;
			closeButton.addEventListener(Event.TRIGGERED, onCloseHandler);
			addChild(closeButton);
		}
		
		public function get frontLightEnabled():Boolean{
			return _frontLightEnabled;
		}
		
		public function set frontLightEnabled(value:Boolean):void{
			_frontLightEnabled=value;
			if(_frontLightEnabled){
				(frontLightButton.defaultSkin as Image).color = 0x0f86ff;
				(frontLightButton.downSkin as Image).color = 0x0f86ff;
			}else{
				(frontLightButton.defaultSkin as Image).color = 0xfefefe;
				(frontLightButton.downSkin as Image).color = 0xfefefe;
			}
		}
		
		
		public function get diskNeonEnabled():Boolean{
			return _diskNeonEnabled;
		}

		public function set diskNeonEnabled(value:Boolean):void{
			_diskNeonEnabled=value;
			if(_diskNeonEnabled){
				(diskNeonButton.defaultSkin as Image).color = 0x0f86ff;
				(diskNeonButton.downSkin as Image).color = 0x0f86ff;
			}else{
				(diskNeonButton.defaultSkin as Image).color = 0xfefefe;
				(diskNeonButton.downSkin as Image).color = 0xfefefe;
			}
		}
		
		public function get neonEnabled():Boolean{
			return _neonEnabled;
		}
		
		public function set neonEnabled(value:Boolean):void{
			_neonEnabled=value;
			if(_neonEnabled){
				(neonButton.defaultSkin as Image).color=0x0f86ff;
				(neonButton.downSkin as Image).color=0x0f86ff;
			}else{
				(neonButton.defaultSkin as Image).color=0xfefefe;
				(neonButton.downSkin as Image).color=0xfefefe;
			}
		}
		
		public function set car (value:Car):void{
			_car=value;
			invalidate("car");
		}
		
		public function isPossibleRearLight():Boolean{
			if(_car){
				var rearLightPartSlot:PartSlot=_car.getPossiblePartSlot("rear_lights");
				if(rearLightPartSlot && rearLightPartSlot.selectedPart &&  rearLightPartSlot.selectedPart.states && rearLightPartSlot.selectedPart.states["lights"]){
					return true;
				}
			}
			return false;
		}

		public function isPossibleLight():Boolean{
			var result:Boolean = false;
			if(_car && _car.possiblePartSlots){
                for(var k:int = 0; k < _car.possiblePartSlots.length; k++){
                    var lightPartSlot:PartSlot = _car.possiblePartSlots[k] as PartSlot;
                    if(lightPartSlot && lightPartSlot.selectedPart && lightPartSlot.selectedPart.states && lightPartSlot.selectedPart.states["lights"]){
                        result = true;
						break;
                    }
                }
			}
			return result;
		}

		public function isLightsOn():Boolean{
			var result:Boolean = false;
			if(_car){
                for(var k:int = 0; k < _car.possiblePartSlots.length; k++){
                    var lightPartSlot:PartSlot = _car.possiblePartSlots[k] as PartSlot;
                    if(lightPartSlot && lightPartSlot.selectedPart && lightPartSlot.selectedPart.states && lightPartSlot.selectedPart.states["lights"]){
                        result = lightPartSlot.selectedPart.selectedState!=null;
						break;
                    }
                }
			}
			return result;
		}

		public function isNeonOn():Boolean{
			var result:Boolean = false;
			if(_car){
                for(var k:int = 0; k < Car.CAR_DISKS_SLOTS.length; k++){
                    var partSlot:PartSlot = _car.getPossiblePartSlot(Car.CAR_DISKS_SLOTS[k]) as PartSlot;
                    if(partSlot && partSlot.selectedPart && partSlot.selectedPart.selectedState!=null && partSlot.selectedPart.selectedState.nameOfState.indexOf("neon")!=-1){
                        result = true;
						break;
                    }
                }
			}
			return result;
		}
		
		public function isPossibleFrontLight():Boolean{
			if(_car){
				var frontLightPartSlot:PartSlot=_car.getPossiblePartSlot("front_lights");
				if(frontLightPartSlot && frontLightPartSlot.selectedPart &&  frontLightPartSlot.selectedPart.states &&  frontLightPartSlot.selectedPart.states['lights']){
					return true;
				}
			}
			return false;
		}
		
		public function isPossibleDiskNeon():Boolean{
			if(_car){
				for each(var partSlotName:String in Car.CAR_DISKS_SLOTS){
					var disksPartSlot:PartSlot=_car.getPossiblePartSlot(partSlotName);
					if(disksPartSlot && disksPartSlot.selectedPart &&  disksPartSlot.selectedPart.states){
                        var selectedStateName:String;
						if(disksPartSlot.selectedPart.selectedState){
							selectedStateName = disksPartSlot.selectedPart.selectedState.nameOfState;
							if(selectedStateName.indexOf('neon') == -1){
                                if(selectedStateName.indexOf('trr') != -1){
                                    selectedStateName = selectedStateName.replace('trr', 'neon-trr');
								}else{
                                    selectedStateName = selectedStateName + '-neon';
								}
							}
						}else{
                            selectedStateName = 'neon';
                        }
						if(selectedStateName.indexOf('neon') != -1 && disksPartSlot.selectedPart.states[selectedStateName]){
							return true;
						}
					}
				}
			}
			return false;
		}

		public function isPossibleNeon():Boolean{
			if(_car){
				var neonPartSlot:PartSlot=_car.getPossiblePartSlot("neon");
				if(neonPartSlot && neonPartSlot.possibleParts && neonPartSlot.possibleParts.length>0){
					return true;
				}
			}
			return false;
		}
		
		public function doToggleState(stateName:String):void{
			var currentValue:Boolean;
			if(stateName=="front_light"){
				if(!isPossibleFrontLight() && !isPossibleLight()){
					dispatchEventWith(TOGGLE_STATE_ERROR, false, {stateName:stateName});
					return;
				}
				SoundManager.playSound(SoundManager.SELECT_PART);
				frontLightEnabled=!frontLightEnabled;
				currentValue=frontLightEnabled;
			}else if(stateName=="disk_neon"){
				if(!isPossibleDiskNeon()){
					dispatchEventWith(TOGGLE_STATE_ERROR, false, {stateName:stateName});
					return;
				}
				diskNeonEnabled=!diskNeonEnabled;
				currentValue=diskNeonEnabled;
			}else if(stateName=="neon"){
				if(!isPossibleNeon()){
					dispatchEventWith(TOGGLE_STATE_ERROR, false, {stateName:stateName});
					return;
				}
				neonEnabled=!neonEnabled;
				currentValue=neonEnabled;
			}
			this.dispatchEventWith(TOGGLE_STATE, false, {stateName:stateName, newValue:currentValue});
		}
		
		protected function onToggleState(event:Event):void{
			var stateName:String=(event.currentTarget as Button).name;
			this.doToggleState(stateName);
		}
		
		protected function onCloseHandler(event:Event):void{
			this.visible=false;
		}
		
		private var closeContainer:Sprite;
		private var backAlpha : Number = 0;
		
		override protected function draw():void {
			super.draw();
			var hGap:Number=38/2;
			
			if(isInvalid(INVALIDATION_FLAG_SIZE)){
				backgroundImage.width=actualWidth;
				backgroundImage.height=actualHeight;
				
				backgroundImage.x=0;
				backgroundImage.y=0;
				
				closeButton.x=actualWidth-closeButton.width-6;
				closeButton.y=4;

				frontLightButton.x=actualWidth-frontLightButton.width-(96/2);
				frontLightButton.y=(actualHeight-frontLightButton.height)/2;
				
				diskNeonButton.x=frontLightButton.x-hGap-diskNeonButton.width;
				diskNeonButton.y=(actualHeight-diskNeonButton.height)/2;
				
				neonButton.x=diskNeonButton.x-hGap-neonButton.width;
				neonButton.y=(actualHeight-neonButton.height)/2;
			}
			
			if (!closeContainer) {
				closeContainer = new Sprite();
				addChild(closeContainer);
				
				var tx : Number = closeButton.x - 4;
				var wt:Number = (closeButton.width + 8);
				var ht:Number = closeButton.height + 8;
				
				var skn : Quad = new Quad(wt,ht,0xff0000);
				skn.alpha = backAlpha;
				var cl:Button = new Button();
				cl.defaultSkin = skn;
				cl.x = tx;
				cl.y = 0;
				cl.addEventListener(Event.TRIGGERED, onCloseHandler);
				closeContainer.addChild(cl);
			}
			
			if(isInvalid("car")){
				if(!_car){
					return;
				}
				frontLightEnabled = isLightsOn();
				var diskNeonNames:Array = ["disks_front", "disks_rear", "disks"];//TODO sync
				diskNeonEnabled = isNeonOn();
				for each (var partSlotName:String in diskNeonNames){
					var diskPartSlot:PartSlot=_car.getPossiblePartSlot(partSlotName);
					if(diskPartSlot && diskPartSlot.selectedPart && 
						diskPartSlot.selectedPart.selectedState && 
						diskPartSlot.selectedPart.selectedState.nameOfState.indexOf("neon")>=0){
						diskNeonEnabled=true;
						break;
					}
				}
				var neonPartSlot:PartSlot=_car.getPossiblePartSlot("neon");
				neonEnabled = (neonPartSlot && neonPartSlot.selectedPart);
				if(frontLightButton){
					frontLightButton.visible = (isPossibleFrontLight() || isPossibleFrontLight());
				}

				diskNeonButton.visible = isPossibleDiskNeon();
				neonButton.visible = isPossibleNeon();
			}
		}
	}
}



