package com.despark.tuning3d.view.components.tunings.colors
{
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.cars.Car;
import com.despark.tuning3d.model.cars.CarBackground;
import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.model.core.ImagePicker;
import com.despark.tuning3d.model.core.http.HttpClient;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.BackgroundItemRenderer;
import com.despark.tuning3d.view.components.core.BlockerView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.CustomAlertView;
    import com.despark.tuning3d.view.components.core.DisposableList;
import com.despark.tuning3d.view.components.core.ExtentedButton;
import com.despark.tuning3d.view.components.core.FillColorItemRenderer;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.utils.FontSelector;

import feathers.layout.VerticalAlign;

import flash.display.Bitmap;

import flash.display.BitmapData;

import flash.events.IOErrorEvent;
    import flash.filesystem.File;
    import flash.filesystem.FileMode;
    import flash.filesystem.FileStream;
    import flash.net.FileFilter;
    import flash.net.URLLoaderDataFormat;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import feathers.controls.Button;
import feathers.controls.List;
import feathers.controls.renderers.IListItemRenderer;
import feathers.core.FeathersControl;
import feathers.data.ListCollection;
import feathers.layout.HorizontalLayout;

import flash.utils.ByteArray;

import ru.inspirit.net.MultipartURLLoader;

import starling.core.Starling;

import starling.display.Image;
import starling.display.Quad;
import starling.display.Sprite;
import starling.events.Event;
import starling.textures.Texture;

import uk.soulwire.utils.display.DisplayUtils;

public class TuningBackgroundMenuView extends FeathersControl implements Multilingual
{
    public static const ON_CLOSE:String="ON_CLOSE";
    public static const BACKGROUND_SELECTED:String="BACKGROUND_SELECTED";
    public static const BACKGROUND_UNSELECTED:String="BACKGROUND_UNSELECTED";
    public static const BACKGROUND_UPLOAD:String="BACKGROUND_UPLOAD";
    public static const BACKGROUND_UPLOAD_COMPLETE:String="BACKGROUND_UPLOAD_COMPLETE";
    public static const SNAPSHOT_CAMERA_SELECTED:String="SNAPSHOT_CAMERA_SELECTED";
    public static const BACKGROUND_DELETE : String = "BACKGROUND_DELETE";
    public static const FILL_COLOR_SELECTED:String="FILL_COLOR_SELECTED";
    public static const UPDATE_UPLOADED_IMAGES:String="UPDATE_UPLOADED_IMAGES";
    private var _language:Language;

    private var _quad:Quad;
    private var _closeButton:Button;
    private var _deleteButton:Button;
    private var _deleteButtonSeporator:Image;

    private var _fillColorButton:Button;
    private var _fillColorButtonSeporator:Image;


    private var _uploadButton:Button;
    private var _uploadButtonSeporator:Image;

    private var _allowedFormatLabel:MultilingualButton;
    private var _allowedFormatLabelSeporator:Image;

    private var _backgroundList:List;

    private var _fillColorList:List;

    private var _carBackgrounds:Array;

    private var _car:Car;

    private var _selectedBackground:CarBackground = null;
    private var _selectedFillColor:Object;

    private var _deleteBackgroundImage:Image;
    private var _fillColorBackgroundImage:Image;


    private var _areSureWantToDeleteLabel:MultilingualButton;
    private var _cancelLabel:MultilingualButton;

    private var _imagePicker:ImagePicker;
    public function TuningBackgroundMenuView()
    {
        super();
    }

    public function updateWithLanguage(language:Language):void
    {
        _language=language;
    }

    public function get uploadButton():Button{

        return _uploadButton;
    }

    public function set carBackgrounds(value:Array):void{

        _carBackgrounds=value;
        invalidate("carBackgrounds");
    }

    public function addBackground(carBackground:CarBackground):void{
        var hasListener:Boolean = _backgroundList.hasEventListener(Event.CHANGE, backgroundListChangeHandler);
        if(hasListener){
            _backgroundList.removeEventListener(Event.CHANGE, backgroundListChangeHandler);
        }

        if(_backgroundList.dataProvider){
            _backgroundList.dataProvider.unshift(carBackground);
            _backgroundList.horizontalScrollPosition = 0;
            _backgroundList.selectedItem = carBackground;
        }

        if(hasListener){
            _backgroundList.addEventListener(Event.CHANGE, backgroundListChangeHandler);
        }
    }

    public function removeBackground(carBackground:CarBackground, updateBackground:Boolean = true):void {
        var hasListener:Boolean = _backgroundList.hasEventListener(Event.CHANGE, backgroundListChangeHandler);
        if(!updateBackground && hasListener){
            _backgroundList.removeEventListener(Event.CHANGE, backgroundListChangeHandler);
        }

        if(_backgroundList.dataProvider && _backgroundList.dataProvider.contains(carBackground)){
            var removedIndex:int = _backgroundList.dataProvider.getItemIndex(carBackground);
            _backgroundList.dataProvider.removeItem(carBackground);
            if(_backgroundList.dataProvider.length > removedIndex){
                _backgroundList.selectedIndex = removedIndex;
            }else if(_backgroundList.dataProvider.length){
                _backgroundList.selectedIndex = removedIndex - 1;
            }
        }

        if(!updateBackground && hasListener){
            _backgroundList.addEventListener(Event.CHANGE, backgroundListChangeHandler);
        }
    }

    public function get carBackgrounds():Array{

        return _carBackgrounds;
    }



    public function set car(value:Car):void{

        _car=value;
        invalidate("car");
    }

    public function get car():Car{

        return _car;
    }

    public function get selectedBackground():CarBackground{

        return _selectedBackground;
    }

    public function get selectedFillColor():Object{

        return _selectedFillColor;
    }

    public function enterUploadingState():void{


    }

    public function enterFillColorState():void{

        _fillColorBackgroundImage.visible=true;
        _fillColorList.visible=true;

    }


    public function enterNormalState():void{

        _fillColorBackgroundImage.visible=false;
        _fillColorList.visible=false;

        _deleteBackgroundImage.visible=false;
        this._areSureWantToDeleteLabel.visible=false;
        this._cancelLabel.visible=false;
    }

    public function enterDeleteState():void{

        _deleteBackgroundImage.visible=true;
        this._areSureWantToDeleteLabel.visible=true;
        this._cancelLabel.visible=true;
    }


    override public function set visible(value:Boolean):void {
        super.visible = value;
        if(_backgroundList)
            _backgroundList.visible = value;
    }

    override protected function initialize() : void {
        super.initialize();
        _quad=new Quad(1,1,0x000000);
        _quad.alpha=0;
        addChild(_quad);

        _imagePicker = new ImagePicker();
        _deleteButtonSeporator=createSeporator();
        addChild(_deleteButtonSeporator);

        _fillColorButton=ButtonFactory.circleButton("a_color_bg_button");
        //_deleteButton.name="rear_light";
        _fillColorButton.width=44/2;
        _fillColorButton.height=44/2;
        _fillColorButton.addEventListener(Event.TRIGGERED, onFillColorHandler);
        //(_deleteButton.defaultSkin as Image).color=0xfefefe;
        //(_deleteButton.downSkin as Image).color=0xfefefe;
        // addChild(_fillColorButton);
        //_fillColorButtonSeporator=createSeporator();
        //addChild(_fillColorButtonSeporator);


        _uploadButton=ButtonFactory.circleButton("a_upload_bg_button");
        //_deleteButton.name="rear_light";
        _uploadButton.width=35/2;
        _uploadButton.height=35/2;
        _uploadButton.addEventListener(Event.TRIGGERED, uploadPhotoButtonHandler);

        //_uploadButton.visible=false;
        //(_deleteButton.defaultSkin as Image).color=0xfefefe;
        //(_deleteButton.downSkin as Image).color=0xfefefe;
        addChild(_uploadButton);

        _uploadButtonSeporator=createSeporator();
        addChild(_uploadButtonSeporator);
        _uploadButtonSeporator.visible=false;

        _allowedFormatLabel=new MultilingualButton();
        _allowedFormatLabel.textFormat = new TextFormat(FontSelector.getFont(true), 8, 0xFFFFFF );
        _allowedFormatLabel.textFormat.align = TextFormatAlign.LEFT;
        _allowedFormatLabel.textFormat.bold=false;
        _allowedFormatLabel.paddingLeft=8;
        _allowedFormatLabel.paddingRight=8;
        _allowedFormatLabel.wordWrap=true;
        _allowedFormatLabel.label="Allowed format\nJPEG, PNG,\nmin. size 1920x1080";

        _allowedFormatLabel.defaultSkin = new Quad(1,1,0x000000);
        _allowedFormatLabel.defaultSkin.alpha=0;
        _allowedFormatLabel.downSkin=new Quad(1,1,0x000000);
        _allowedFormatLabel.downSkin.alpha=0.0;

        _allowedFormatLabel.updateWithLanguage(AlertView.language);
        addChild(_allowedFormatLabel);
        //addChild(moreColorosButton);

        _allowedFormatLabelSeporator=createSeporator();
        addChild(_allowedFormatLabelSeporator);
        //a_fill_color_button


        var backgroundListLayout : HorizontalLayout = new HorizontalLayout();
        //backgroundListLayout.manageVisibility = false; //tt
        backgroundListLayout.useVirtualLayout = false;
        backgroundListLayout.gap=2.5;
        backgroundListLayout.paddingLeft=2.5;
        backgroundListLayout.paddingRight=2.5;

        _backgroundList = new DisposableList();
        //materialList.visible = false;
        _backgroundList.snapScrollPositionsToPixels = true;
        _backgroundList.hasElasticEdges=false;
        _backgroundList.clipContent = true;
        _backgroundList.layout = backgroundListLayout;
        _backgroundList.addEventListener("itemSelectedAgain", backgroundListItemSelectedAgain);
        _backgroundList.addEventListener(Event.CHANGE, backgroundListChangeHandler);
        _backgroundList.addEventListener(Event.SCROLL, backgroundListScrollHandler);
        _backgroundList.itemRendererFactory = function() : IListItemRenderer {
            var renderer :BackgroundItemRenderer = new BackgroundItemRenderer();
            //renderer.language = AlertView.language;
            renderer.setSize(41, 41);
            //renderer.partSlotNameWidthsMap = partSlotNameWidthsMap;
            return renderer;
        };

        addChild(_backgroundList);
        var backgroundTexture:Texture=AlertView.assetManager.getTexture("a_effect_bg");
        _deleteBackgroundImage=new Image(backgroundTexture);
        _deleteBackgroundImage.visible=false;
        addChild(_deleteBackgroundImage);

        _deleteButton = ButtonFactory.deleteButton();
        _deleteButton.addEventListener(Event.TRIGGERED, onDeleteHandler);

        //(_deleteButton.defaultSkin as Image).color=0xfefefe;
        //(_deleteButton.downSkin as Image).color=0xfefefe;
        addChild(_deleteButton);

        _areSureWantToDeleteLabel=new MultilingualButton();
        _areSureWantToDeleteLabel.visible=false;
        _areSureWantToDeleteLabel.textFormat = new TextFormat(FontSelector.getFont(false), 11, 0xFF0000 );
        _areSureWantToDeleteLabel.textFormat.align = TextFormatAlign.LEFT;
        _areSureWantToDeleteLabel.textFormat.bold=false;
        _areSureWantToDeleteLabel.paddingLeft=8;
        _areSureWantToDeleteLabel.paddingRight=8;
        _areSureWantToDeleteLabel.wordWrap=true;
        _areSureWantToDeleteLabel.label="Are sure want to delete?";

        _areSureWantToDeleteLabel.defaultSkin = new Quad(1,1,0x000000);
        _areSureWantToDeleteLabel.defaultSkin.alpha=0;
        _areSureWantToDeleteLabel.downSkin=new Quad(1,1,0x000000);
        _areSureWantToDeleteLabel.downSkin.alpha=0.0;

        _areSureWantToDeleteLabel.updateWithLanguage(AlertView.language);
        addChild(_areSureWantToDeleteLabel);

        _cancelLabel=new MultilingualButton();
        _cancelLabel.visible=false;
        _cancelLabel.textFormat = new TextFormat(FontSelector.getFont(false), 11, 0xFFFFFF );
        _cancelLabel.textFormat.align = TextFormatAlign.LEFT;
        _cancelLabel.textFormat.bold=false;
        _cancelLabel.paddingLeft=8;
        _cancelLabel.paddingRight=8;
        _cancelLabel.wordWrap=true;
        _cancelLabel.label="Cancel";
        _cancelLabel.defaultSkin = new Quad(1,1,0x000000);
        _cancelLabel.defaultSkin.alpha=0;
        _cancelLabel.downSkin=new Quad(1,1,0x000000);
        _cancelLabel.downSkin.alpha=0.3;
        _cancelLabel.updateWithLanguage(AlertView.language);
        _cancelLabel.addEventListener(Event.TRIGGERED, onDeleteCancel);
        addChild(_cancelLabel);
        //addChild(moreColorosButton);

        _fillColorBackgroundImage=new Image(backgroundTexture);
        _fillColorBackgroundImage.visible=false;
        addChild(_fillColorBackgroundImage);

        var fillColorListLayout : HorizontalLayout = new HorizontalLayout();
        //fillColorListLayout.manageVisibility = false; //tt
        fillColorListLayout.useVirtualLayout = false;
        fillColorListLayout.verticalAlign = VerticalAlign.MIDDLE;
        fillColorListLayout.gap=9;
        fillColorListLayout.paddingLeft=8;
        fillColorListLayout.paddingRight=8;

        _fillColorList = new List();
        _fillColorList.visible=false;
        //materialList.visible = false;
        _fillColorList.snapScrollPositionsToPixels = true;
        _fillColorList.hasElasticEdges=false;
        _fillColorList.clipContent = true;
        _fillColorList.layout = backgroundListLayout;
        _fillColorList.addEventListener("itemSelectedAgain", fillColorListItemSelectedAgain);
        _fillColorList.addEventListener(Event.CHANGE, fillColorListChangeHandler);
        //_fillColorList.addEventListener(Event.SCROLL, fillColorListScrollHandler);
        _fillColorList.itemRendererFactory = function() : IListItemRenderer {
            var renderer:FillColorItemRenderer = new FillColorItemRenderer();
            //renderer.language = AlertView.language;
            renderer.setSize(20, 20);
            //renderer.partSlotNameWidthsMap = partSlotNameWidthsMap;
            return renderer;
        };

        _fillColorList.dataProvider=new ListCollection([{color:0x3c8256}, {color:0x301de6}, {color:0x829cdc}, {color:0x626262}, {color:0xc48e54}, {color:0xa1bdbe}, {color:0x275c7b}]);
        addChild(_fillColorList);

        _closeButton=ButtonFactory.circleButton("a_close_effect_menu");
        (_closeButton as ExtentedButton).hitAreaPadding = 15;
        _closeButton.width = 10;
        _closeButton.height = 10;
        _closeButton.addEventListener(Event.TRIGGERED, onCloseHandler);
        addChild(_closeButton);
    }

    private var closeContainer:Sprite;
    private var backAlpha : Number = 0;

    override protected function draw() : void {
        super.draw();
        var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
        if(sizeInvalid) {
            //11
            _quad.width=actualWidth;
            _quad.height=actualHeight;

            _deleteBackgroundImage.width=actualWidth;
            _deleteBackgroundImage.height=actualHeight;

            _fillColorBackgroundImage.width=actualWidth;
            _fillColorBackgroundImage.height=actualHeight;

            _closeButton.x = actualWidth - _closeButton.width - 3;
            _closeButton.y = 3;


            _deleteButton.x = actualWidth - _deleteButton.width - 25; // was 35 :)
            _deleteButton.y = (actualHeight - _deleteButton.height) / 2;

            _deleteButtonSeporator.x = _deleteButton.x - 13;
            _deleteButtonSeporator.y = (actualHeight - _deleteButtonSeporator.height) / 2;



            /*_fillColorButton.x=_deleteButtonSeporator.x-_fillColorButton.width-7;
            _fillColorButton.y=(actualHeight-_fillColorButton.height)/2;*/

            /*_fillColorButtonSeporator.x=_fillColorButton.x-8.5;
            _fillColorButtonSeporator.y=(actualHeight-_fillColorButtonSeporator.height)/2;*/


            _uploadButton.x = _deleteButtonSeporator.x-_uploadButton.width-9;
            _uploadButton.y = (actualHeight-_uploadButton.height)/2;

            _uploadButtonSeporator.x=_uploadButton.x-10.5;
            _uploadButtonSeporator.y=(actualHeight-_uploadButtonSeporator.height)/2;



            _allowedFormatLabel.width=101;
            _allowedFormatLabel.x = (_uploadButton.visible?_uploadButtonSeporator.x:_deleteButtonSeporator.x)-_allowedFormatLabel.width+8;
            _allowedFormatLabel.height=NaN;
            _allowedFormatLabel.validate();
            _allowedFormatLabel.y=(actualHeight-_allowedFormatLabel.height)/2;

            _allowedFormatLabelSeporator.x=_allowedFormatLabel.x-8;
            _allowedFormatLabelSeporator.y=(actualHeight-_allowedFormatLabelSeporator.height)/2;


            _backgroundList.x=0;

            _backgroundList.height=82/2;
            _backgroundList.y=(actualHeight-_backgroundList.height)/2;

            _backgroundList.width=_allowedFormatLabelSeporator.x;



            _areSureWantToDeleteLabel.x=actualWidth/2;
            _areSureWantToDeleteLabel.width=150;
            _areSureWantToDeleteLabel.height=NaN;
            _areSureWantToDeleteLabel.validate();
            _areSureWantToDeleteLabel.y=(actualHeight-_areSureWantToDeleteLabel.height)/2;


            _cancelLabel.x=_areSureWantToDeleteLabel.x+_areSureWantToDeleteLabel.width+25;
            _cancelLabel.width=47;
            _cancelLabel.height=NaN;
            _cancelLabel.validate();
            _cancelLabel.y=(actualHeight-_cancelLabel.height)/2;


            _fillColorList.x=actualWidth/2;
//				_fillColorList.height=20;
            _fillColorList.y=(actualHeight-20)/2;
            _fillColorList.width=actualWidth/2;



        }

        if (!closeContainer)
        {
            closeContainer = new Sprite();
            addChild(closeContainer);

            var tx : Number = _closeButton.x - 4;
            var wt:Number = (_closeButton.width + 15);
            var ht:Number = _closeButton.height + 15;

            var skn : Quad = new Quad(wt,ht,0xff0000);
            skn.alpha = backAlpha;
            var cl:Button = new Button();
            cl.addEventListener(Event.TRIGGERED, onCloseHandler);
            cl.defaultSkin = skn;
            cl.x = tx;
            cl.y = 0;
            closeContainer.addChild(cl);
        }

        if(isInvalid("carBackgrounds")){

            _backgroundList.dataProvider=new ListCollection(_carBackgrounds);

            if(_car && _carBackgrounds){
                refreshSelected();
            }
        }

        if(isInvalid("car")){

            if(_car && _carBackgrounds){
                refreshSelected();
            }
        }
    }

    private function refreshSelected():void{

        var isSelected:Boolean=false;
        if(_car.selectedBackground){

            var index:int=0;
            for each( var carBackground:CarBackground in _carBackgrounds){

                if(_car.selectedBackground.id==carBackground.id){

                    _backgroundList.selectedIndex=index;

                    if(_backgroundList.selectedItem){

                        _selectedBackground=_backgroundList.selectedItem as CarBackground;
                        isSelected=true;
                        var carBackgroundTmp:CarBackground=_backgroundList.selectedItem as CarBackground;
                        if(carBackgroundTmp.isCustom){

                            _deleteButton.isEnabled=true;
                            _deleteButton.alpha=1;
                        }else{

                            _deleteButton.isEnabled=false;
                            _deleteButton.alpha=0.5;
                        }
                    }
                    break;
                }

                index++;
            }
        }

        if(!isSelected){

            _deleteButton.isEnabled=false;
            _deleteButton.alpha=0.5;
        }
    }

    protected function fillColorListItemSelectedAgain(event:Event):void{

        _fillColorList.selectedIndex=-1;
        _selectedFillColor=null;
        //enterNormalState();

        //_deleteButton.isEnabled=false;
        //_deleteButton.alpha=0.5;

        this.dispatchEventWith(FILL_COLOR_SELECTED)
    }

    protected function fillColorListChangeHandler(event:Event):void{

        if(_fillColorList.selectedItem){


            var fillColor:Object=_fillColorList.selectedItem as Object;

            //_selectedBackground=null;;

            _selectedFillColor=fillColor;

            //enterNormalState();

            dispatchEventWith(FILL_COLOR_SELECTED)


        }
    }

    protected function backgroundListItemSelectedAgain(event:Event):void{

        _backgroundList.selectedIndex=-1;
        _selectedBackground=null;
        _selectedFillColor=null;

        _deleteButton.isEnabled=false;
        _deleteButton.alpha=0.5;

        this.dispatchEventWith(BACKGROUND_UNSELECTED)
    }

    protected function backgroundListChangeHandler(event:Event):void{

        if(_backgroundList.selectedItem){
            SoundManager.playSound(SoundManager.SELECT_PART);

            var carBackground:CarBackground=_backgroundList.selectedItem as CarBackground;

            setBackground(carBackground);


        }
    }


    protected function backgroundListScrollHandler(event:Event):void{


    }



    public function open():void{

        this.visible=true;
    }

    public function close():void{

        this.visible=false;
    }

    private function createSeporator():Image{

        var seporatorTexture:Texture=AlertView.assetManager.getTexture("a_seporator_1");

        var seporatorImage:Image=new Image(seporatorTexture);

        seporatorImage.width=1.5;
        seporatorImage.height=34;

        return seporatorImage;
    }

    public function setBackground(carBackground : CarBackground) : void {
		_selectedBackground = carBackground;
		_selectedFillColor = null;
		if(carBackground.isCustom)
		{
			_deleteButton.isEnabled = true;
			_deleteButton.alpha = 1;
		}
		else {
            _deleteButton.isEnabled = false;
            _deleteButton.alpha = 0.5;
		}
		dispatchEventWith(BACKGROUND_SELECTED)
	}

    private function uploadPhotoButtonHandler(event:Event):void{
        SoundManager.playSound(SoundManager.PART_MENU);
        this.dispatchEventWith(BACKGROUND_UPLOAD);
    }



    private function onFillColorHandler(event:Event):void{
        enterFillColorState();
    }

    private function onDeleteCancel(event:Event):void{
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        enterNormalState();
    }

    private function onDeleteHandler(event:Event):void{
        SoundManager.playSound(SoundManager.SELECT_PART);
        if(_deleteBackgroundImage.visible){
            dispatchEventWith(BACKGROUND_DELETE);
            enterNormalState();
        }else{
            enterDeleteState();
        }
    }

    private function onCloseHandler(event:Event):void{
        if(_fillColorList.visible){
            _fillColorList.visible=false;
            _fillColorBackgroundImage.visible=false;
            return
        }
        SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        close();
        dispatchEventWith(ON_CLOSE);
    }

    public function browseForUpload():void{
        if(Device.isEmulator()){
            /*var fileToOpen:File = new File();
            var txtFilter:FileFilter = new FileFilter("Image", "*.jpeg;*.jpg;*.png");
            try {
                fileToOpen.browseForOpen("Open", [txtFilter]);
                fileToOpen.addEventListener(Event.SELECT, fileSelected);
            }  catch (error:Error){
                trace("Failed:", error.message);
            }*/


        }else {
            addImagePickerListeners();
            _imagePicker.pickImage();
        }
    }

    /*private static function fileSelected(event:flash.events.Event):void {
        var stream:FileStream = new FileStream();
        stream.open(event.target as File, FileMode.READ);
        var fileData:String = stream.readUTFBytes(stream.bytesAvailable);
        trace(fileData);
    }*/

    private function imagePickedHandler(e : Event) : void{
        removeImagePickerListeners();
        dispatchEvent(new Event(BACKGROUND_UPLOAD_COMPLETE, false, e.data));
    }

    private function imagePickerFailed(e : Event) : void{
        removeImagePickerListeners();
        if(e.type == ImagePicker.ERROR_OCCURRED){
            AlertView.show('Something went wrong ...');
        }else  if(e.type == ImagePicker.NOT_SUPPORTED){
            AlertView.show('Image picking is not supported on your device ...', 0, false);
        }else  if(e.type == ImagePicker.DENIED){
            if(e.data.canOpenDeviceSettings){
                CustomAlertView.show('Image picking is denied on your device', {label:"Open Settings", callback: function():void{
                        _imagePicker.openSettings();
                    }}, {label: "Close", callback: function():void{}});
            }else{
                AlertView.show('Image picking is denied on your device', 0, false);
            }
        }
    }

    private function addImagePickerListeners() : void{
        _imagePicker.addEventListener(ImagePicker.SUCCEED, imagePickedHandler);
        _imagePicker.addEventListener(ImagePicker.ERROR_OCCURRED, imagePickerFailed);
        _imagePicker.addEventListener(ImagePicker.NOT_SUPPORTED, imagePickerFailed);
        _imagePicker.addEventListener(ImagePicker.DENIED, imagePickerFailed);
    }

    private function removeImagePickerListeners() : void{
        _imagePicker.removeEventListener(ImagePicker.SUCCEED, imagePickedHandler);
        _imagePicker.removeEventListener(ImagePicker.ERROR_OCCURRED, imagePickerFailed);
        _imagePicker.removeEventListener(ImagePicker.NOT_SUPPORTED, imagePickerFailed);
        _imagePicker.removeEventListener(ImagePicker.DENIED, imagePickerFailed);
    }
}
}