package com.despark.tuning3d.view.components.tunings.colors
{
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
import com.despark.tuning3d.model.parts.PartColor;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ColorUtil;
import com.despark.tuning3d.view.components.core.MultilingualLabel;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.ImageLoader;
import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.TextInput;
import feathers.controls.text.StageTextTextEditor;
import feathers.controls.text.TextFieldTextEditor;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.FeathersControl;
import feathers.events.FeathersEventType;
import feathers.layout.HorizontalAlign;
import feathers.layout.VerticalLayout;

import flash.display.Bitmap;
import flash.display.BitmapData;
import flash.display.CapsStyle;
import flash.display.JointStyle;
import flash.display.LineScaleMode;
import flash.display.Shape;
import flash.geom.Point;
import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.animation.Transitions;
import starling.animation.Tween;
import starling.core.Starling;
import starling.display.Image;
import starling.display.Quad;
import starling.display.Sprite;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.textures.Texture;
import starling.utils.AssetManager;
import starling.utils.Color;
import starling.utils.MathUtil;
import starling.utils.MathUtil;

public class TuningColorPicker extends FeathersControl implements Multilingual
	{
		public static const COLOR_CONFIGURED : String = "ColorConfigured";
		public static const DISMISSED : String = "Dismissed";
		
		private static var HELPER_POINT : Point = new Point;
		private static var HANDLE_RECT : Rectangle = new Rectangle();
		private static var TOUCH_RECT : Rectangle = new Rectangle();
		
		public var assetManager : AssetManager;
		public var overlayQuad : Quad;
		public var overlayButton : Button;
		
		[Embed(source="/assets/textures/1x/colors.png")]
		private var colorsClass_x1 : Class;

        [Embed(source="/assets/textures/1.5x/colors.png")]
        private var colorsClass_x15 : Class;
		
		[Embed(source="/assets/textures/2x/colors.png")]
		private var colorsClass_x2 : Class;

        [Embed(source="/assets/textures/3.375x/colors.png")]
        private var colorsClass_x3375 : Class;
		
		[Embed(source="/assets/textures/4x/colors.png")]
		private var colorsClass_x4 : Class;
		
		private var colorsImage : ImageLoader;
		private var colorsImageCont:Sprite;
		private var colorsBitmapData : BitmapData;
		private var colorBrightnessImage : ImageLoader;
		private var colorBrightnessImageCont : Sprite;
		private var inputPoint : Point;
		private var outputPoint : Point;
		private var brightnessHandle : Image;
		private var colorHandle : Image;
		private var arrowsBackgroundQuad : Quad;
		private var startTouchY : int;
		private var startArrowsY : int;
		private var _selectedColor : uint;
		private var _brightness : Number;
		private var partColor : PartColor;
		private var colorsInitialized : Boolean;
		private var backgroundImage : Image;
		private var titleBackgroundImage : Image;
		private var titleLabel : MultilingualLabel;
		private var startY : int;
		private var startHeight : int;
		private var beganOutside : Boolean;
		private var handleHitArea : Rectangle;
		private var touchPoint : Point;
		private var tweening : Boolean;
		private var touchMoved : Boolean;
		
		private var _partSlot:PartSlot;

		private var hexGroup:LayoutGroup;
		private var hexColorPreview:Image;
		private var colorInput:TextInput;
		private var hsvColor:Object;
		public var touchPhase:String;

		public function TuningColorPicker() {
			super();
            width = 265;
            height = 140;
			inputPoint = new Point();
			outputPoint = new Point();
			handleHitArea = new Rectangle();
			touchPoint = new Point();
			
			partColor = PartColor.customColor();
			brightness = 1.0;
			selectedColor = partColor.value;
			overlayQuad = new Quad(1, 1, 0x222728);

		}
		
		public function set partSlot(value:PartSlot):void{
			_partSlot=value;
			var color:uint=0xfeffff;
			if(_partSlot.selectedPart && _partSlot.selectedPart.color){
				color=_partSlot.selectedPart.color.value;
				partColor=_partSlot.selectedPart.color;
			}else if(_partSlot.selectedColor){
				color=_partSlot.selectedColor.value;
				partColor=_partSlot.selectedColor;
			}else if(_partSlot.parentCar && _partSlot.parentCar.bodyPartColor){
				color= _partSlot.parentCar.bodyPartColor.value;
				partColor=_partSlot.parentCar.bodyPartColor;
			}
			_selectedColor=color;
			invalidate("partSlot");
            setColorInputText(partColor.value);
		}
		
		public function get brightness():Number
		{
			return _brightness;
		}

		public function set brightness(value:Number):void
		{
			_brightness = value;
			invalidate("brightness");
		}

		public function get selectedColor():uint
		{
			return _selectedColor;
		}

		public function set selectedColor(value:uint):void {
			_selectedColor = value;
			invalidate("selectedColor");
		}
		
		public function updateWithLanguage(language : Language) : void {
			titleLabel.updateWithLanguage(language);
			titleLabel.text = language.translate("mob_colors", "customColor").toUpperCase();
		}

		protected override function initialize() : void
		{
			super.initialize();  
			
			var texture:Texture = assetManager.getTexture("popUpBackground");
			//var backgroundTextures : Scale9Textures = new Scale9Textures(assetManager.getTexture("popUpBackground"), new Rectangle( 24, 22, 11, 7));
			backgroundImage = new Image(texture); 
			
			backgroundImage.scale9Grid =  new Rectangle( 24, 22, 11, 7);
			addChild(backgroundImage);
			
			texture = assetManager.getTexture("popUpTitleBackground");
			
			titleBackgroundImage = new Image(texture);
			
			titleBackgroundImage.scale9Grid =  new Rectangle( 21, 0, 3, texture.frameHeight);
			addChild(titleBackgroundImage);
			
			titleLabel = new MultilingualLabel();
			titleLabel.height = 13;
			titleLabel.textFormat = new TextFormat(FontSelector.getFont(), 8, 0x6A7275);
			titleLabel.textFormat.align = TextFormatAlign.CENTER;
			addChild(titleLabel);
			
			var colorsBitmap : Bitmap;

			switch (assetManager.scaleFactor)
			{
				case 1:
                    colorsBitmap = new colorsClass_x1();
					break;
				case 1.5:
                    colorsBitmap = new colorsClass_x15();
					break;
				case 2:
                    colorsBitmap = new colorsClass_x2();
					break;
				case 3.375:
                    colorsBitmap = new colorsClass_x3375();
					break;
				case 4:
                    colorsBitmap = new colorsClass_x4();
					break;
				default:
                    colorsBitmap = new colorsClass_x2();
			}

			colorsBitmapData = colorsBitmap.bitmapData;
			colorsImage = new ImageLoader();
			var colorsTexture : Texture = Texture.fromBitmap(colorsBitmap, false, false, assetManager.scaleFactor);
			
			colorsTexture.root.onRestore = function():void{
				colorsTexture.root.uploadBitmap(colorsBitmap);
			};
			colorsImage.source = colorsTexture;
			colorsImage.setSize(colorsTexture.width, colorsTexture.height);
			//colorsImage.maintainAspectRatio = false;
			colorsImage.addEventListener(TouchEvent.TOUCH, colorsImageTouchHandler);
			
			
			
			colorsImageCont=new Sprite();
			colorsImageCont.addChild(colorsImage);
			
			addChild(colorsImageCont);
						
			
			var brightnessTexture : Texture = assetManager.getTexture("colorBrightness");
			colorBrightnessImage = new ImageLoader();
			colorBrightnessImage.addEventListener(TouchEvent.TOUCH, test);
			colorBrightnessImage.setSize(brightnessTexture.width, brightnessTexture.height);
			colorBrightnessImage.source = brightnessTexture;
			colorBrightnessImage.color = 0xFEFFFF;
			colorBrightnessImage.maintainAspectRatio = false;			
			colorBrightnessImageCont=new Sprite();
			colorBrightnessImageCont.addChild(colorBrightnessImage);
			addChild(colorBrightnessImageCont);

			arrowsBackgroundQuad = new Quad(40, 20, 0xFF0000);
			arrowsBackgroundQuad.x = -10;
			arrowsBackgroundQuad.pivotY = int(arrowsBackgroundQuad.height / 2);
			arrowsBackgroundQuad.alpha = 0.0;
			
			var brightnessHandleTexture:Texture=assetManager.getTexture("colorHandle");

		
			brightnessHandle = new Image(brightnessHandleTexture);
			brightnessHandle.touchable = false;
			
			
			
			brightnessHandle.pivotY = int(brightnessHandle.height / 2);
			brightnessHandle.x=0;
			colorBrightnessImageCont.addChild(brightnessHandle);
			
			colorHandle = new Image(assetManager.getTexture("colorHandle"));
			colorHandle.pivotX = int(colorHandle.width / 2);
			colorHandle.pivotY = int(colorHandle.height / 2);
			colorHandle.touchable = false;
			colorsImageCont.addChild(colorHandle);
			addEventListener(TouchEvent.TOUCH, touchHandler);
			// updateColorHandle(new Point(20, 20));
			//
			createHexSection();
		}
		
		protected override function draw() : void
		{
			super.draw();
			
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
				
				colorsImageCont.x = 10;
				colorsImageCont.y = 10;
				
				
				colorsImage.x = 0;
				colorsImage.y = 0;
				
				
				
				colorBrightnessImageCont.x = colorsImageCont.x  + colorsImage.width + 10;
				colorBrightnessImageCont.y = colorsImageCont.y;
				//colorBrightnessImageCont.height = colorsImageCont.height;
				
				colorBrightnessImage.x = 0;
				colorBrightnessImage.y = 0;
				colorBrightnessImage.height = colorsImageCont.height;

				brightnessHandle.x = int((colorBrightnessImage.width - brightnessHandle.width) / 2);
				
				backgroundImage.width = actualWidth;
				backgroundImage.height = actualHeight;
				
				titleBackgroundImage.width = 140;
				titleBackgroundImage.y = -titleBackgroundImage.height + 4;
				titleBackgroundImage.x = int((actualWidth - titleBackgroundImage.width) / 2);
				
				titleLabel.width = titleBackgroundImage.width - 20;
				titleLabel.x = int((actualWidth - titleLabel.width) / 2);
				titleLabel.y = titleBackgroundImage.y + 4;

				//
                hexGroup.x = colorBrightnessImageCont.x + colorBrightnessImageCont.width + 15;
                if(width != hexGroup.x + hexGroup.width + 15){
                    width = hexGroup.x + hexGroup.width + 15;
					x = (stage.stageWidth - width)/2;
                }
			}
			
			if(isInvalid("selectedColor")) {
				colorBrightnessImage.color = selectedColor;
				if(hexColorPreview){
                    hexColorPreview.color = partColor.value;
				}
			}

			if(isInvalid("partSlot")) {
				// var result:Object=ColorUtil.RGBtoHSV(Color.getRed(selectedColor), Color.getGreen(selectedColor), Color.getBlue(selectedColor));
				hsvColor = ColorUtil.HexToHSV(partColor.value);

				var H:Number = hsvColor.h;
				var S:Number = hsvColor.s / 100;
				var V:Number = hsvColor.v / 100;

				var paletteRect:Rectangle = new Rectangle(-colorHandle.pivotX, -colorHandle.pivotY, colorsImage.width + colorHandle.pivotX, colorsImage.height + colorHandle.pivotY);
				var X:Number = (H/360)*(paletteRect.x + paletteRect.width);
				var Y:Number = (1 - S)*(paletteRect.y + paletteRect.height);

				if(X > paletteRect.right){
					X = paletteRect.right;
				}else if(X < paletteRect.left){
					X = paletteRect.left;
				}

				if(Y > paletteRect.bottom){
					Y = paletteRect.bottom;
				}else if(Y < paletteRect.top){
					Y = paletteRect.top;
				}

				//var c:uint = colorsBitmapData.getPixel(X * assetManager.scaleFactor, Y * assetManager.scaleFactor);
                var c:uint = getColor(X, Y);  // colorsBitmapData.getPixel(X * assetManager.scaleFactor, Y * assetManager.scaleFactor);

				if(c==0xffffff) c=0xfeffff;
				
				colorBrightnessImage.color=c;

				_brightness = V;
			}
			
			if(isInvalid("brightness") || isInvalid("selectedColor")) {
				var red : int = Color.getRed(selectedColor);
				var green : int = Color.getGreen(selectedColor);
				var blue : int = Color.getBlue(selectedColor);

				hsvColor = ColorUtil.HexToHSV(_selectedColor);
				hsvColor.v = _brightness * 100;
								
				if(colorsInitialized) {
					partColor.value = ColorUtil.HSVtoHEX(hsvColor.h, hsvColor.s, hsvColor.v);//Color.rgb(red * brightness, green * brightness, blue * brightness);
					doSelectedColor(partColor);
				}
                if(colorInput){
                    var movePointer:Boolean = colorInput.text.indexOf("#") != -1;
                    if(movePointer && colorInput.selectionBeginIndex < colorInput.text.length) {
                        colorInput.selectRange(colorInput.selectionBeginIndex + 1);
                    }
                }
				colorsInitialized = true;
                setColorInputText(partColor.value);
            }
            updatePreview();
        }
		
		private function doSelectedColor(selectedColor:PartColor):void{
			if(!_partSlot || !_partSlot.selectedPart || !_partSlot.selectedPart.selectedMaterial){
				return;
			}
			var isExistWhiteMaterial:Boolean=_partSlot.selectedPart.getPossibleMaterial("white");
			var isExistGlossyMaterial:Boolean=_partSlot.selectedPart.getPossibleMaterial("glossy");
			if(_partSlot.selectedPart.selectedMaterial.name=="mate" || _partSlot.selectedPart.selectedMaterial.name=="redblack"){
				dispatchEventWith(TuningColorsView.COLOR_SELECTED, true, {selectedColor: selectedColor});
			}else if(isExistWhiteMaterial){
				if(selectedColor.value!=0xfeffff && _partSlot.selectedPart.selectedMaterial.name=="white" && isExistGlossyMaterial){
					dispatchEventWith(TuningColorsView.MATERIAL_SELECTED, true, {selectedMaterial: _partSlot.selectedPart.getPossibleMaterial("glossy"), partSlot: _partSlot, onComplete:function():void{
						dispatchEventWith(TuningColorsView.COLOR_SELECTED, true, {selectedColor: selectedColor});
					}});
				}else if(_partSlot.selectedPart.selectedMaterial.name=="metal" && isExistGlossyMaterial){
					dispatchEventWith(TuningColorsView.MATERIAL_SELECTED, true, {selectedMaterial: _partSlot.selectedPart.getPossibleMaterial("glossy"), partSlot: _partSlot, onComplete:function():void{
						dispatchEventWith(TuningColorsView.COLOR_SELECTED, true, {selectedColor: selectedColor});
					}});
				}else{
					dispatchEventWith(TuningColorsView.COLOR_SELECTED, true, {selectedColor: selectedColor});
				}
			}else{
				if(_partSlot.selectedPart.selectedMaterial.name!="glossy" && isExistGlossyMaterial){
					dispatchEventWith(TuningColorsView.MATERIAL_SELECTED, true, {selectedMaterial: _partSlot.selectedPart.getPossibleMaterial("glossy"), partSlot: _partSlot, onComplete:function():void{
						dispatchEventWith(TuningColorsView.COLOR_SELECTED, true, {selectedColor: selectedColor});
					}});
				}else{
					dispatchEventWith(TuningColorsView.COLOR_SELECTED, true, {selectedColor: selectedColor});
				}
			}
		}
		
		private function colorsImageTouchHandler(event : TouchEvent) : void {
			var overStageMoved:Touch = event.getTouch(stage);
			if(overStageMoved && (overStageMoved.phase == TouchPhase.MOVED || overStageMoved.phase == TouchPhase.ENDED)){
                overStageMoved.getLocation(colorsImage, HELPER_POINT);
                HELPER_POINT.x = MathUtil.clamp(HELPER_POINT.x, 0, HANDLE_RECT.width + 2 * colorHandle.pivotX);
                HELPER_POINT.y = MathUtil.clamp(HELPER_POINT.y, 0, HANDLE_RECT.height + 2 * colorHandle.pivotY);
                updateColorHandle(HELPER_POINT);
                event.stopImmediatePropagation();
                stage.removeEventListener(TouchEvent.TOUCH, touchMovedHandler);
				if(overStageMoved.phase == TouchPhase.ENDED){
					touchPhase = overStageMoved.phase;
				}else{
					touchPhase = null;
				}
            }else{
				touchPhase = null;
			}
		}

		private function updateColorHandle(point : Point) : void {
            HANDLE_RECT.setTo(0, 0, colorsImage.width, colorsImage.height);
			point.x = MathUtil.clamp(point.x, HANDLE_RECT.left, HANDLE_RECT.right);
			point.y = MathUtil.clamp(point.y, HANDLE_RECT.top, HANDLE_RECT.bottom);
            HELPER_POINT.setTo(point.x, point.y);
			if(HANDLE_RECT.containsPoint(HELPER_POINT)) {
				var color:uint = getColor(HELPER_POINT.x, HELPER_POINT.y);
				selectedColor = color;
				if(selectedColor == 0xFFFFFF) {
					selectedColor = 0xFEFFFF;
				}
			}
		}

		private function getColor(toX:Number, toY:Number):uint{
            var h:Number = 360 * toX * assetManager.scaleFactor / colorsBitmapData.width;
            var s:Number = 100 * (colorsBitmapData.height - toY * assetManager.scaleFactor) / colorsBitmapData.height;
            var color:uint = ColorUtil.HSVtoHEX( h, s, 100);
			return color;
		}

        private function updateColorHandlePosition():void{
            var hsv:Object = ColorUtil.HexToHSV(partColor.value);
			//if(hsv.h == 0 && hsv.s == 0){
				var hsv2:Object = ColorUtil.HexToHSV(_selectedColor);
				hsv2.v = hsv.v;
				hsv = hsv2;
			//}
            HANDLE_RECT.setTo(0, 0, colorsImage.width, colorsImage.height);
            colorHandle.x = HANDLE_RECT.x + HANDLE_RECT.width * hsv.h / 360;
            colorHandle.y = HANDLE_RECT.y + HANDLE_RECT.height * (1 - hsv.s / 100);
            HELPER_POINT.setTo(colorHandle.x, colorHandle.y);
            brightnessHandle.y = HANDLE_RECT.y + HANDLE_RECT.height * (1 - _brightness);//(1 - hsv.v / 100);
        }
		
		private function test(event : TouchEvent) : void {
			var touch : Touch = event.getTouch(colorBrightnessImage);
			if(!touch) {
				return;
			}
			
			if(touch.phase == TouchPhase.MOVED || touch.phase == TouchPhase.ENDED) {
				var location : Point = touch.getLocation(colorBrightnessImage);
                location.y = MathUtil.clamp(location.y, 0, colorBrightnessImage.height);
				brightness = 1 - location.y/colorBrightnessImage.height;
				if(touch.phase == TouchPhase.ENDED){
					touchPhase = TouchPhase.ENDED;
				}else{
					touchPhase = null;
				}
			}else{
				touchPhase = null;
			}
		}
		
		public function showIn(view : Sprite) : void {
			overlayQuad.height = 100;
			overlayQuad.width = view.width;
			overlayQuad.y = view.height - overlayQuad.height;
			overlayQuad.alpha = 0;
			view.addChild(overlayQuad);
			
			overlayButton = new Button();
			overlayButton.addEventListener(Event.TRIGGERED, overlayButtonTriggeredHandler);
			overlayButton.x = overlayQuad.x;
			overlayButton.y = overlayQuad.y;
			overlayButton.width = overlayQuad.width;
			overlayButton.height = overlayQuad.height;
			view.addChild(overlayButton);
			
			var tween2 : Tween = new Tween(overlayQuad, 0.3, Transitions.EASE_OUT);
			tween2.animate("alpha", 0.8);
			
			Starling.juggler.add(tween2);
			
			view.addChild(this);
			
			x = int((view.width - width) / 2);
			y = view.height;
			
			var tween : Tween = new Tween(this, 0.3);
			tween.animate("y", view.height - height + 20);
			
			Starling.juggler.add(tween);
		}
		
		private function touchHandler(event : TouchEvent) : void {
			const touch : Touch = event.getTouch(this);
			if(!touch) {
				return;
			}
			
			if(touch.phase == TouchPhase.BEGAN) {
				touch.getLocation(this, touchPoint);
				handleHitArea.setTo(titleBackgroundImage.x - 30, titleBackgroundImage.y - 10, titleBackgroundImage.width + 60, titleBackgroundImage.height + 20);
				if(!handleHitArea.containsPoint(touchPoint)) {
					beganOutside = true;
					return;
				}
				startY = touch.globalY;
				startHeight = y;
				stage.addEventListener(TouchEvent.TOUCH, touchMovedHandler);
			}
		}

		override protected function feathersControl_removedFromStageHandler(event:Event):void{
			super .feathersControl_removedFromStageHandler(event);
			stage.removeEventListener(TouchEvent.TOUCH, touchMovedHandler);
		}

		private function touchMovedHandler(event : TouchEvent) : void {
			if(!stage){
                event.currentTarget.removeEventListener(TouchEvent.TOUCH, touchMovedHandler);
			}
			const touch : Touch = event.getTouch(stage);
			if(!touch) {
				return;
			}
			if(touch.phase == TouchPhase.MOVED) {
				if(beganOutside) {
					return;
				}
				touchMoved = true;
				y = int(Math.min(parent.height + 10, Math.max(parent.height - height + 20, startHeight + touch.globalY - startY)));
				updateOverlayQuadAlpha();
				event.stopImmediatePropagation();
			} else if(touch.phase == TouchPhase.ENDED) {
				if(beganOutside) {
					beganOutside = false;
					return;
				}

				if(!touchMoved) {
					touchMoved = false;
					animate(1);
				} else if(!tweening) {
					var dY : int = height / 2 - (parent.height - y);
					animate(dY);
				}
                stage.removeEventListener(TouchEvent.TOUCH, touchMovedHandler);
				event.stopImmediatePropagation();
			}
		}
		
		private function animate(dY : int) : void {
			tweening = true;
			var _this : TuningColorPicker = this;	
			
			var tween : Tween = new Tween(this, 0.3, Transitions.EASE_OUT);
			tween.onComplete = function() : void {
				tweening = false;
				
				if(!parent)
					return;
				if(y == parent.height + 10) {
					dispatchEventWith(DISMISSED);
					
					parent.removeChild(overlayQuad);
					parent.removeChild(overlayButton);
					parent.removeChild(_this);
					
				}
			};
			
			tween.onUpdate = function() : void {
				updateOverlayQuadAlpha();
			};
				if(!parent)
					return;
			
			if(dY < 0) 
			{
				tween.animate("y", parent.height - height + 20);
			}
			else 
			{
				
				tween.animate("y", parent.height  + 10);
			}
			
			Starling.juggler.add(tween);
		}
		
		private function updateOverlayQuadAlpha() : void {
			try {
				overlayQuad.alpha = 0.8 * (1 - ((y - parent.height + height - 20) / (height - 20)));
			} catch (e:Error){

			}
		}
		
		private function overlayButtonTriggeredHandler(event : Event) : void {
			animate(1);
		}

		private function createHexSection():void{
			hexGroup = new LayoutGroup();
			var layout:VerticalLayout = new VerticalLayout();
			layout.horizontalAlign = HorizontalAlign.CENTER;
            layout.gap = 7;
			hexGroup.layout = layout;
			var label:Label = new Label();
			label.textRendererFactory = function() : Object {
                var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
                textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 12, 0xFFFFFF);
                textRenderer.textFormat.align = TextFormatAlign.CENTER;
                textRenderer.embedFonts = Language.supportsEmbeddedFont;
                return textRenderer;
            };
			label.text = "HEX Color";
            //
			var bd:BitmapData = new BitmapData( 100, 100, true, 0);
			var shape:Shape = new Shape();
			shape.graphics.beginFill(0xFF0000);
			shape.graphics.drawCircle(bd.width/2,bd.height/2,bd.width/2-1);
			shape.graphics.endFill();
			bd.draw(shape);
			hexColorPreview = new Image(Texture.fromBitmapData(bd));
            hexColorPreview.width = hexColorPreview.height = 40;

            colorInput = new TextInput();
            colorInput.textEditorFactory = function() : Object {
                var textRenderer : StageTextTextEditor = new StageTextTextEditor();
                textRenderer.fontFamily = FontSelector.getFont();
                textRenderer.fontSize = 12;
                textRenderer.color = 0xFFFFFF;
                textRenderer.textAlign = TextFormatAlign.CENTER;
                return textRenderer;
            };
            colorInput.width = 75;
            colorInput.height = 22;
            setColorInputText(0);
			var img:Image = new Image(ButtonFactory.assetManager.getTexture("color_picker_border"));
            img.scale9Grid = new Rectangle(img.width / 3 + 1, img.height / 3 + 1, img.width / 3 - 1, img.height / 3 - 1);
			colorInput.backgroundSkin = img;
            colorInput.addEventListener(Event.CHANGE, textInputHandler);

            hexGroup.addChild(label);
            hexGroup.addChild(hexColorPreview);
            hexGroup.addChild(colorInput);
            hexGroup.validate();
			//
			addChild(hexGroup);
            hexGroup.x = colorBrightnessImageCont.x + colorBrightnessImageCont.width + 15;
            hexGroup.y = 5;
			if(width != hexGroup.x + hexGroup.width + 15){
                width = hexGroup.x + hexGroup.width + 15;
			}
		}

        private function textInputHandler(event:Event):void {
			var text:String = colorInput.text;
			text = text.replace("#", '');
            var WHITE_COLOR:int = parseInt('FFFFFE', 16);
            var INPUT_COLOR:int = parseInt(text, 16);
			if(text.length > 0){
                partColor.value = MathUtil.clamp(INPUT_COLOR, 0, WHITE_COLOR);
				var hsv:Object = ColorUtil.HexToHSV(partColor.value);
				_selectedColor = ColorUtil.HSVtoHEX(hsv.h, hsv.s, 100);
				_brightness = hsv.v/100;
                invalidate("partSlot");
                doSelectedColor(partColor);
			}
        }

		private function setColorInputText(val:uint):void{
			if(!colorInput){
				return;
			}
            var str:String = val.toString(16).toUpperCase();
			var count:int = 6 - str.length;
			var zero:String = '#';
			for(var i:int = 0; i < count; i++){
				zero += '0';
			}
            colorInput.removeEventListener(Event.CHANGE, textInputHandler);
            colorInput.text = zero + str;
            colorInput.addEventListener(Event.CHANGE, textInputHandler);
		}

		private function updatePreview():void{
            if(hexColorPreview){
                hexColorPreview.color = partColor.value;
				updateColorHandlePosition();
            }
		}
	}
}