package com.despark.tuning3d.view.components.tunings.editing {
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusInChallenge;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.utils.FontSelector;

    import feathers.controls.Label;

    import flash.events.TimerEvent;
    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import flash.utils.Timer;
    import flash.utils.getTimer;

    import starling.display.Image;
    import starling.textures.Texture;
    import starling.utils.Align;

    public class ChallengeTimer extends Label {
        private var _timer:Timer;
        private var _startTime:Number;
        private var _endTime:Number;
        private var _iconImage:Image;
        private var _layoutUpdateHandler:Function;
        private static const SECOND:Number = 1000;
        private static const MINUTE:Number = SECOND * 60;
        private static const HOUR:Number = MINUTE * 60;

        public function ChallengeTimer() {
            super();
            text = "0:00";
            alignPivot(Align.RIGHT, Align.CENTER);
            textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansBold(), 14, 0xFFFFFF, TextFormatAlign.RIGHT, false);
        }

        override protected function initialize():void{
            super .initialize();

            var buttonTexture:Texture = ButtonFactory.assetManager.getTexture("button");
            var buttonImage:Image = new Image(buttonTexture);
            buttonImage.scale9Grid = new Rectangle(14,0,15, buttonTexture.frameHeight);
            buttonImage.color = ChallengeStatusInChallenge.TIME_TRIAL_COLOR;
            backgroundSkin = buttonImage;

            _iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_time_trails"));
            _iconImage.height = 15;
            _iconImage.scale = _iconImage.scaleY;
            _iconImage.color = 0xFEFFFF;
            addChild(_iconImage);

            paddingLeft = 8 + _iconImage.width;
            paddingRight = 7;
            paddingTop = 1;

            _timer = new Timer(1000);
            _timer.addEventListener(TimerEvent.TIMER, timerEventHandler);
        }

        private function timerEventHandler(event:TimerEvent):void {
            var num:Number = getTimer() - _startTime;
            text = strToTime(num);
        }

        public static function strToTime(num:Number):String{
            var secs:Number = Math.floor((num % MINUTE) / SECOND);
            var mins:Number = Math.floor((num % HOUR) / MINUTE);
            var hrs:Number = Math.floor(num / HOUR);
            return (hrs?hrs.toString() + "h:":"") + ((mins < 10 && hrs)?"0":"") + mins.toString() + "m:" + (secs < 10?"0":"") + secs.toString() + "s";
        }

        override protected function draw():void{
            super.draw();

            if(_iconImage){
                _iconImage.x = 5;
                _iconImage.y = (actualHeight - _iconImage.height) / 2 - 1;
            }
            if(_layoutUpdateHandler != null){
                _layoutUpdateHandler();
            }

            paddingTop = (actualHeight - 18) / 2;
        }

        override public function dispose():void{
            super.dispose();
            if(_timer){
                _timer.stop();
                _timer.removeEventListener(TimerEvent.TIMER, timerEventHandler);
                _timer = null;
            }
        }

        public function start():void{
            _startTime = getTimer();
            _timer.start();
        }

        public function stop():void {
            _timer.stop();
            _endTime = getTimer();
        }

        public function get totalTime():Number{
            return _endTime - _startTime;
        }

        public function set layoutUpdateHandler(value:Function):void {
            _layoutUpdateHandler = value;
        }
    }
}
