package com.despark.tuning3d.view.core
{
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.LanguagesProxy;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.settings.SettingsProxy;
	import com.despark.tuning3d.view.components.core.AlertView;
	
	import feathers.events.FeathersEventType;
	
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;
	
	import starling.display.DisplayObject;
	import starling.events.Event;
	
	public class ExtendedMediator extends Mediator
	{
		public var alertView : AlertView;
		protected var currentLanguage : Language;
		
		public function ExtendedMediator(mediatorName : String = null, displayObject : DisplayObject = null) {
			super(mediatorName, displayObject);
		}
		
		public function get displayObject() : DisplayObject {
			return viewComponent as DisplayObject;
		}
		
		override public function onRegister() : void {
			super.onRegister();
			
			displayObject.addEventListener(FeathersEventType.INITIALIZE, displayObjectAddedToStageHandler);
		}
		
		override public function onRemove() : void
		{
			super.onRemove();
			
			displayObject.removeEventListener(FeathersEventType.INITIALIZE, displayObjectAddedToStageHandler);
			
			if(alertView) {
				alertView.hide();
			}
			
			//sendNotification(Notifications.CANCEL_ALL_HTTP_REQUESTS);
		}
		
		override public function listNotificationInterests() : Array {
			return [Notifications.LANGUAGE_CHANGED];
		}
		
		override public function handleNotification(note : INotification) : void
		{
			switch(note.getName())
			{
				case Notifications.LANGUAGE_CHANGED:
					handleLanguageChanged();
					break;
			}
		}
		
		private function updateLanguageIfNecessary() : void
		{
			if(viewComponent is Multilingual)
			{
				var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
				var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
								
				if(settingsProxy.currentSettings.selectedLanguageCode)
				{
					var selectedLanguage : Language = languagesProxy.getLanguage(settingsProxy.currentSettings.selectedLanguageCode);
				
					if(selectedLanguage) {
						currentLanguage = selectedLanguage;
						Multilingual(viewComponent).updateWithLanguage(selectedLanguage);
					}

				}

			} 
		}
		
		private function displayObjectAddedToStageHandler(event : Event) : void {
			updateLanguageIfNecessary();
		}
		
		private function handleLanguageChanged() : void {
			updateLanguageIfNecessary();
		}
	}
}
