package com.despark.tuning3d.view.core {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.levels.LevelData;
    import com.despark.tuning3d.model.mails.MailsProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.challenges.controls.UserLevelAvatar;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.AlertViewHL;
    import com.despark.tuning3d.view.components.core.BlockerView;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.CustomAlertView;
    import com.despark.tuning3d.view.components.core.HomeScreenButton;
    import com.despark.tuning3d.view.components.core.HomeSubMenuItem;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.MainMenuItem;
    import com.despark.tuning3d.view.components.core.MainMenuView;
    import com.despark.tuning3d.view.components.core.StackedBannerScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.friends.FriendsListPopUp;
    import com.despark.tuning3d.view.components.news.NewsFeedData;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.Console;
    import com.utils.DateParser;
    import com.utils.DisplayObjectUtil;
    import com.utils.FileUtil;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.core.FeathersControl;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.filesystem.File;
    import flash.text.TextFormatAlign;
    import flash.utils.setTimeout;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;

    public class HomeScreenPopUp extends StackedBannerScreen{

    public static const MODE_SELECTED : String = "ModeSelected";
    public static const RESTORE_SELECTED : String = "HomeScreenPopUpRestoreSelected";

    public static var UPDATE_OBJECT : Object;


    private var _challengesButton : HomeScreenButton;

    private var _blockerView : BlockerView;

    private var _waitingForBrandsLoad : Boolean = false;
    private var _headerContainer:LayoutGroup;
    private var _bottomMenu:LayoutGroup;
    private var _itemGap:Number = 10;

    private var _language:Language = null;
    private var _userSignedIn:Boolean;
    private var _selectedTuning:Tuning;
    //

    private var galleryButton:HomeSubMenuItem;
    private var garageButton:HomeSubMenuItem;
    private var tuningButton : HomeSubMenuItem;



    private var leaderboardButton:HomeSubMenuItem;
    private var votingButton:HomeSubMenuItem;
    private var designsButton:HomeSubMenuItem;

    private var settingsButton:MainMenuItem;
    private var mailButton:MainMenuItem;
    private var friendsButton:MainMenuItem;
    private var notificationsButton:MainMenuItem;
    private var timelineButton:MainMenuItem;
    private var scheduleButton:MainMenuItem;
    private var newsButton:MainMenuItem;

    private var avatarContainer:Sprite;
    private var loginButton : Button;
    private var avatarButton:UserCircularAvatar;
    private var levelButton:UserLevelAvatar;


    private var _userLabel:FeathersControl;
    private var _loginLabel:Label;

    public function HomeScreenPopUp() {
        super("HomeScreenPopUp");
        AppMode.self.setModeHome();
    }

    private function initBg() : void {
        var newBg : Image = new Image(assetManager.getTexture("firstScreenBg"));
        var aspect : Number = newBg.width / newBg.height;
        var currentAspect : Number = stage.stageWidth / stage.stageHeight;
        if(currentAspect > aspect) {
            newBg.width = stage.stageWidth;
            newBg.height = stage.stageWidth / aspect;
        } else {
            newBg.height = stage.stageHeight;
            newBg.width = stage.stageHeight * aspect;
        }
        newBg.x = (stage.stageWidth - newBg.width)/2;
        newBg.y = (stage.stageHeight - newBg.height)/2;
        addChildAt(newBg, 0);
    }


    override protected function initialize():void {
        super.initialize();

        setSizeInternal(this.stage.stageWidth, this.stage.stageHeight, false);
        initBg();
        initHeader();

        _challengesButton = ButtonFactory.homeScreenButton("challenge", "START CHALLENGES", 0xec0000, Constants.isNarrowScreen);
        _challengesButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
        addChild(_challengesButton);

        locateMainButtons();

        settingsButton = new MainMenuItem("profile", null, false, 0x707070, 21);
        settingsButton.label = "SETTINGS";
        settingsButton.addEventListener(Event.TRIGGERED, profileButtonTriggeredHandler);

        newsButton = new MainMenuItem("news_btn", null, true, 0x707070);
        newsButton.addEventListener(Event.TRIGGERED, newsButtonTriggeredHandler);
        newsButton.label = "NEWS";
        newsButton.noteCount = 0;

        mailButton = new MainMenuItem("messages_btn",null, true, 0x707070, 21);
        mailButton.name = "mailsButton";
        mailButton.label = "MESSAGES";
        mailButton.addEventListener(Event.TRIGGERED, mailButtonTriggeredHandler);

        friendsButton = new MainMenuItem("users-solid",null, true, 0x707070, 23);
        friendsButton.name = "friendButton";
        friendsButton.label = "FRIENDS";
        friendsButton.addEventListener(Event.TRIGGERED, friendsButtonTriggeredHandler);

        scheduleButton = new MainMenuItem("shed_btn",null, false, 0x707070, 19);
        scheduleButton.label = "SCHEDULE";
        scheduleButton.addEventListener(Event.TRIGGERED, scheduleButtonTriggeredHandler);
        if(selectedLanguage){
            scheduleButton.label = selectedLanguage.translate("mob_home","Schedule").toUpperCase();
        }

        notificationsButton = new MainMenuItem("globe",null, true, 0x707070);
        notificationsButton.name = "challengeNotesButton";
        notificationsButton.label = "NOTIF";
        notificationsButton.addEventListener(Event.TRIGGERED, notificationButtonTriggeredHandler);

        timelineButton = new MainMenuItem("clock-menu",null, true, 0x707070);
        timelineButton.name = "timelineButton";
        timelineButton.label = "FEED";
        timelineButton.addEventListener(Event.TRIGGERED, timelineButtonTriggeredHandler);

        _loginLabel = LabelFactory.createSimpleLabel(FontSelector.getGoldBold(), 14, 0xFFFFFF, TextFormatAlign.LEFT, false, false);

        _userLabel = LabelFactory.createHomeScreenUserLabel();
        if(UsersProxy.loggedInUser()){
            LabelFactory.setLabelText(_userLabel, UsersProxy.loggedInUser().nameToShow);
        }

        //
        var menuItemWidth:Number = 67;
        var bottomPadding:Number = 5;
        if(Constants.isUltraWideScreen){
            menuItemWidth = 77;
            bottomPadding = 26;
        }else if(Constants.isWideScreen){
            menuItemWidth = 72;
            bottomPadding = 5;
        }

        galleryButton = new HomeSubMenuItem("gallery_icon",  0xFFFFFE, 0.8, false, menuItemWidth);
        galleryButton.label = "GALLERY";
        galleryButton.iconOffsetY = 3;
        galleryButton.labelOffsetY = 1;
        galleryButton.addEventListener(Event.TRIGGERED, galleryButtonTriggeredHandler);

        garageButton = new HomeSubMenuItem("garage_icon", 0xFFFFFE, 1, false, menuItemWidth);
        garageButton.label = "GARAGE";
        garageButton.iconOffsetY = 1;
        garageButton.labelOffsetY = -2;
        garageButton.addEventListener(Event.TRIGGERED, garageButtonTriggeredHandler);

        tuningButton = new HomeSubMenuItem("buy_car_icon", 0xFFFFFE, 1.25, false, menuItemWidth);
        tuningButton.label = "CARS";
        tuningButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
        var labelBackground:Quad = new Quad(1,1, 0x0058e3);
        labelBackground.width = tuningButton.width - 16;
        labelBackground.height = 14;
        tuningButton.addChildAt(labelBackground, 0);
        labelBackground.x = (tuningButton.width - labelBackground.width) / 2;
        labelBackground.y = tuningButton.height - labelBackground.height - 6;

        tuningButton.defaultIcon.height = tuningButton.downIcon.height = 24;
        tuningButton.defaultIcon.scale = tuningButton.downIcon.scale = tuningButton.defaultIcon.scaleY;

        tuningButton.iconOffsetY = 2;
        tuningButton.labelOffsetY = 2;
        //
        leaderboardButton = new HomeSubMenuItem("challenges_leaderboard_icon",  0xFFFFFE, 1.4, true, menuItemWidth);
        leaderboardButton.label = "LEADERBOARD";
        leaderboardButton.iconOffsetY = 1;
        leaderboardButton.labelOffsetY = 3;
        leaderboardButton.addEventListener(Event.TRIGGERED, leaderboardButtonTriggeredHandler);

        votingButton = new HomeSubMenuItem("challenges_likes_icon", 0xFFFFFE, 1.15, true, menuItemWidth);
        votingButton.label = "VOTING";
        votingButton.labelOffsetY = 3;
        votingButton.addEventListener(Event.TRIGGERED, votingButtonTriggeredHandler);

        designsButton = new HomeSubMenuItem("challenges_users_icon", 0xFFFFFE, 1.2, true, menuItemWidth);
        designsButton.label = "MY DESIGNS";
        designsButton.labelOffsetY = 4;
        designsButton.addEventListener(Event.TRIGGERED, designsButtonTriggeredHandler);
        //

        loginButton = ButtonFactory.circleButton("login");
        loginButton.name = "loginButton";
        loginButton.setSize(28, 28);
        loginButton.visible = false;
        loginButton.addEventListener(Event.TRIGGERED, loginButtonTriggeredHandler);

        avatarContainer = new Sprite();
        avatarContainer.addChild(loginButton);

        _bottomMenu = new LayoutGroup();
        var bottomMenuLayout:HorizontalLayout = new HorizontalLayout();
        bottomMenuLayout.horizontalAlign = HorizontalAlign.CENTER;
        bottomMenuLayout.paddingLeft = bottomMenuLayout.paddingRight = bottomPadding;
        bottomMenuLayout.paddingTop = 5;
        _bottomMenu.layout = bottomMenuLayout;
        var bottomMenuBackground:Quad = new Quad(1,1,0x111111);
        _bottomMenu.backgroundSkin = bottomMenuBackground;
        _bottomMenu.width = actualWidth;
        _bottomMenu.height = marginBottom;

        _bottomMenu.addChild(newsButton);
        _bottomMenu.addChild(createBottomMenuBorder());
        _bottomMenu.addChild(notificationsButton);
        _bottomMenu.addChild(createBottomMenuBorder());
        _bottomMenu.addChild(timelineButton);
        _bottomMenu.addChild(createBottomMenuBorder());
        _bottomMenu.addChild(mailButton);
        _bottomMenu.addChild(createBottomMenuBorder());
        _bottomMenu.addChild(friendsButton);
        _bottomMenu.addChild(createBottomMenuBorder());
        _bottomMenu.addChild(scheduleButton);
        _bottomMenu.addChild(createBottomMenuBorder());
        _bottomMenu.addChild(settingsButton);
        addChild(_bottomMenu);

        newsButton.validate();
        notificationsButton.validate();
        mailButton.validate();
        friendsButton.validate();
        scheduleButton.validate();
        settingsButton.validate();
        _bottomMenu.validate();

        addChild(leaderboardButton);
        addChild(votingButton);
        addChild(designsButton);

        addChild(galleryButton);
        addChild(garageButton);
        addChild(tuningButton);

        if(Constants.isUltraWideScreen){
            _itemGap = (actualWidth - (leaderboardButton.width + _challengesButton.width + galleryButton.width)) / 4;
        }else{
            _itemGap = (actualWidth - (leaderboardButton.width + _challengesButton.width + galleryButton.width)) / 6;
        }


        addEventListener(FriendsListPopUp.PENDING_FRIENDS_READ, pendingFriendsLoadHandler);
        var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
        updateUserData(usersProxy.currentUserJSON);
        //checkState(usersProxy.currentUser);

        if (!_mainMenuView){
            mainMenuView = new MainMenuView();
            mainMenuView.name = "mainMenuView";
            addChild(_mainMenuView);
            _mainMenuView.visible = false;


            if(avatarContainer){
                if(avatarButton){
                    avatarContainer.addChild(avatarButton);
                    avatarButton.y = 1;
                }
            }

            if(avatarContainer){
                addChild(avatarContainer);
                addChild(_userLabel);
                addChild(_loginLabel);
            }


            _mainMenuView.removeFromParent();
            addEventListener(RESTORE_SELECTED, restoreEventHandler);
        }

        updateNotificationCount();
        updateMailCount();
        Starling.current.juggler.delayCall(function():void{
            invalidate(INVALIDATION_FLAG_SIZE);
        }, 0.1);
    }

        private function restoreEventHandler(event:Event):void {
            mainMenuView.restoreButtonTriggeredHandler(null);
        }

    private function locateMainButtons():void{
        if(stage) {
            _challengesButton.x = (actualWidth - _challengesButton.width) / 2;
        }
    }

        public function profileButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.PART_MENU);
            dispatchExternalEvent(StackedScreen.USER_PROFILE_SELECTED);
        }

        public function newsButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.PART_MENU);
            dispatchExternalEvent(StackedScreen.NEWS_SELECTED);
        }

        private function challengeButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedScreen.CHALLENGES_SELECTED);
            setTimeout(function():void{
                dispatchExternalEvent(BrandsScreen.CLEANUP_MEMORY);
            }, 2000);
        }


        private function timelineButtonTriggeredHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedScreen.TIMELINE_SELECTED);
        }

        private function garageButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            var currentUser:User = UsersProxy.loggedInUser();
            if(currentUser && Constants.selectedUser && Constants.selectedUser.id != currentUser.id){
                Constants.selectedUser = null;
            }

            dispatchExternalEvent(StackedScreen.GARAGE_SELECTED);
        }

        private function friendsButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedScreen.FRIENDS_SELECTED);
        }

        private function scheduleButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedScreen.VOTE_SELECTED);
            setTimeout(function():void{
                dispatchExternalEvent(BrandsScreen.CLEANUP_MEMORY);
            }, 1500);
        }

        private function galleryButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedScreen.GALLERY_SELECTED);
            setTimeout(function():void{
                dispatchExternalEvent(BrandsScreen.CLEANUP_MEMORY);
            }, 1500);
        }

        private function leaderboardButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedBannerScreen.CHALLENGES_LEADERBOARD_SELECTED);
            setTimeout(function():void{
                dispatchExternalEvent(BrandsScreen.CLEANUP_MEMORY);
            }, 1500);
        }

        private function votingButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedBannerScreen.CHALLENGES_RATE_SELECTED);
            setTimeout(function():void{
                dispatchExternalEvent(BrandsScreen.CLEANUP_MEMORY);
            }, 1500);
        }

        private function designsButtonTriggeredHandler(event:Event) : void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedBannerScreen.CHALLENGES_GARAGE_SELECTED);
            setTimeout(function():void{
                dispatchExternalEvent(BrandsScreen.CLEANUP_MEMORY);
            }, 1500);
        }

        private function mailButtonTriggeredHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedScreen.MAILS_SELECTED);
        }

        private function notificationButtonTriggeredHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchExternalEvent(StackedScreen.IN_APP_NOTIFICATIONS_SELECTED);
        }

        public function loginButtonTriggeredHandler(event:Event=null) : void {
            if (event) {
                SoundManager.playSound(SoundManager.SELECT_PART);
                dispatchExternalEvent(BrandsScreen.LOGIN_SELECTED);
            }
        }

        private function checkState(loggedIn:Boolean) : void {
            if (loggedIn) {
                if(loginButton){
                    loginButton.visible = false;
                }
                if(avatarButton){
                    avatarButton.removeFromParent(true);
                }
                var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
                avatarButton = new UserCircularAvatar(usersProxy.currentUser, 35, true, 1);
                avatarButton.name = "avatarButton";
                if(avatarContainer){
                    avatarContainer.addChild(avatarButton);
                }
                avatarButton.y = 1;
                avatarButton.addEventListener(TouchEvent.TOUCH, avatarTouchHandler);
                updateWithLanguage(AlertView.language);
                if(_userLabel){
                    _userLabel.visible = true;
                }
                if(_loginLabel){
                    _loginLabel.visible = false;
                }

                if(_loginLabel){
                    _loginLabel.visible = false;
                }
                if(_userLabel){
                    LabelFactory.setLabelText(_userLabel, usersProxy.currentUser.nameToShow);
                    _userLabel.visible = true;
                }
            }else{
                if(avatarButton){
                    avatarButton.removeFromParent(true);
                    avatarButton = null;
                }
                if(loginButton){
                    loginButton.visible = true;
                    if(_loginLabel){
                        _loginLabel.text = "LOGIN";
                        _loginLabel.visible = true;
                    }

                    if(_userLabel){
                        _userLabel.visible = false;
                    }
                }

                if(_loginLabel){
                    _loginLabel.text = "LOGIN";
                    _loginLabel.visible = true;
                }
                if(_userLabel){
                    _userLabel.visible = false;
                }
            }
        }

        private function avatarTouchHandler(event:TouchEvent):void {
            var touch:Touch = event.getTouch(avatarButton, TouchPhase.ENDED);
            if(touch){
                AlertView.show("Are you sure you want to log out?", 0, false, function (value:Boolean):void {
                    if(value){
                        logoutButtonTriggeredHandler(null);
                    }
                }, false);

            }
        }

        private function levelTouchHandler(event:TouchEvent):void {
            var touch:Touch = event.getTouch(levelButton, TouchPhase.ENDED);
            if(touch){
                dispatchEventWith(USER_LEVEL_INFO_SELECTED);
            }
        }

        private function logoutButtonTriggeredHandler(event:Event) : void {
            dispatchExternalEvent(BrandsScreen.LOGOUT_SELECTED);
        }

        private function pendingFriendsLoadHandler(event:Event):void {
            if(event.data && event.data.hasOwnProperty("unread") && event.data.unread){
                friendsButton.noteCount = Constants.UNREAD_FRIEND_REQUESTS = event.data.friends.length;
            }else{
                friendsButton.noteCount = Constants.UNREAD_FRIEND_REQUESTS = 0;
            }
        }

        public function updateNotificationCount() :void{
            if(UPDATE_OBJECT){
                var newsCacheFile:File = File.applicationStorageDirectory.resolvePath("news_feed.json");
                var newsCacheContent:Object;
                if(newsCacheFile.exists){
                    var contentStr:String = FileUtil.getFileContent(newsCacheFile);
                    if(contentStr && contentStr.length){
                        newsCacheContent = JSON.parse(contentStr);
                    }else{
                        newsCacheContent = {};
                    }
                }else{
                    newsCacheContent = {};
                }
                var newsCount:int = 0;
                for(var feedType:String in UPDATE_OBJECT){
                    if(NewsFeedData.isAllowedType(feedType)){
                        for(var i:int = 0; i < UPDATE_OBJECT[feedType].length; i++){
                            var updateDate:Date = UPDATE_OBJECT[feedType][i].updateDate?DateParser.parseDate(UPDATE_OBJECT[feedType][i].updateDate):new Date();
                            var feedId:String = UPDATE_OBJECT[feedType][i]._id + updateDate.time.toString();
                            if(!newsCacheContent.hasOwnProperty(feedId)){
                                newsCacheContent[feedId] = true;
                                newsCount ++;
                            }
                        }
                    }
                }
                if(newsButton){
                    newsButton.noteCount = newsCount;
                }
            }
            if(notificationsButton){
                notificationsButton.noteCount = Constants.ALL_UNREAD_NOTIFICATION_NUM;
            }
        }

        private static function createBottomMenuBorder():Quad{
            var result:Quad = new Quad(1,1, 0x707070);
            result.alpha = 0.35;
            result.width = 0.75;
            result.height = 22;
            return result;
        }

        public function updateMailCount():void{
            var facade : ApplicationFacade = ApplicationFacade.getInstance();
            var mailsProxy:MailsProxy = facade.retrieveProxy(MailsProxy.NAME) as MailsProxy;
            mailsProxy.getUnreadCount(function (count:int):void {
                if(mailButton){
                    mailButton.noteCount = count;
                }
            });
        }

    override protected function draw():void {
        super.draw();

        var leftVerticalGap:Number = (_challengesButton.height - (leaderboardButton.height + votingButton.height + designsButton.height)) / 2;
        leaderboardButton.x = votingButton.x = designsButton.x = _challengesButton.x - (leaderboardButton.width + _itemGap);
        leaderboardButton.y = _challengesButton.y - 15;
        votingButton.y = leaderboardButton.y + leaderboardButton.height + leftVerticalGap;
        designsButton.y = votingButton.y + votingButton.height + leftVerticalGap;

        var rightVerticalGap:Number = (_challengesButton.height - (galleryButton.height + galleryButton.height + tuningButton.height)) / 2;
        galleryButton.x = garageButton.x = tuningButton.x = _challengesButton.x + _challengesButton.width + _itemGap;
        tuningButton.y = _challengesButton.y - 15;
        galleryButton.y = tuningButton.y + tuningButton.height + rightVerticalGap;
        garageButton.y = galleryButton.y + galleryButton.height + rightVerticalGap;


        initBottomMenu();

        if(_headerContainer && tuningButton && _challengesButton) {
            var yCoord : Number ;
            _headerContainer.validate();
            _headerContainer.validate();
            tuningButton.validate();
            //
            locateMainButtons();
            if(_challengesButton.height == 0){
                Starling.juggler.delayCall(invalidate, .25);
                return;
            }
            //
            _challengesButton.y = marginTop + (contentHeight - _challengesButton.height) / 2 + 15;

            _userLabel.validate();
            avatarContainer.x = _sideMargin + (Device.isIphoneXVariant()? 6: 0);
            avatarContainer.y = (_headerContainer.height - avatarContainer.height) / 2;

            _userLabel.x = avatarContainer.x + avatarContainer.width + 7.5;
            _userLabel.y = avatarContainer.y + (avatarContainer.height - _userLabel.height) / 2;

            _loginLabel.x = _userLabel.x;
            _loginLabel.y = avatarContainer.y + (avatarContainer.height - _loginLabel.height) / 2;
        }

        if(levelButton){
            levelButton.x = actualWidth - (levelButton.width / 2 + _sideMargin);
            levelButton.y = marginTop / 2 - 1;
            _ratingLabel.paddingRight = 30 + _sideMargin;
        }else if(!levelButton){
            _ratingLabel.paddingRight = _sideMargin;
        }
    }

    private function initBottomMenu():void{
        if(_bottomMenu){
            var menuItem:MainMenuItem;
            _bottomMenu.y = actualHeight - _bottomMenu.height;
            var cWidth:Number = 0;
            var i:int;
            for(i = 0; i < _bottomMenu.numChildren; i++){
                menuItem = _bottomMenu.getChildAt(i) as MainMenuItem;
                if(menuItem){
                    menuItem.validate();
                    cWidth += menuItem.width - (menuItem.paddingLeft + menuItem.paddingRight);
                }else{
                    cWidth += _bottomMenu.getChildAt(i).width;
                }
            }
            Console.log("initBottomMenu", cWidth);
            HorizontalLayout(_bottomMenu.layout).gap = 0;
            var menuPadding:Number = HorizontalLayout(_bottomMenu.layout).paddingLeft + HorizontalLayout(_bottomMenu.layout).paddingRight;
            var paddingNum:Number = (_bottomMenu.width - cWidth - menuPadding - 5) / (_bottomMenu.numChildren - 1);
            var lp:Number = paddingNum - 7;
            var rp:Number = paddingNum + 5;
            for(i = 0; i < _bottomMenu.numChildren; i++){
                menuItem = _bottomMenu.getChildAt(i) as MainMenuItem;
                if(menuItem){
                    if(i == 0){
                        menuItem.paddingLeft = 0;
                        menuItem.paddingRight = rp;
                    }else if(i == _bottomMenu.numChildren - 1){
                        menuItem.paddingLeft = lp;
                        menuItem.paddingRight = 6;
                    }else{
                        menuItem.paddingLeft = lp;
                        menuItem.paddingRight = rp;
                    }
                }
            }
        }
    }

    override public function updateUserData(result:Object):void {
        super .updateUserData(result);
        checkState(result);

        if(avatarButton && LevelData.levelDataList){
            updateLevelData(avatarButton.user);
        }else if(!result){
            updateLevelData(null);
        }
    }

    private function initHeader():void {
        var verticalLayout : VerticalLayout = new VerticalLayout();
        verticalLayout.horizontalAlign = HorizontalAlign.RIGHT;
        verticalLayout.verticalAlign = VerticalAlign.MIDDLE;
        verticalLayout.gap = -4;

        _headerContainer  = new LayoutGroup();
        _headerContainer.width = stage.stageWidth;
        _headerContainer.height = 47;
        _headerContainer.layout = verticalLayout;
    }

    public function brandsLoaded() : Boolean {
        if(_waitingForBrandsLoad) {
            Tracer.timeStamp('HomeButton', "Opened", true);
            Starling.juggler.delayCall(function () : void {
                if(_blockerView){
                    _blockerView.hide();
                }
                _blockerView = null;
            }, 1);
            return true;
        }
        return false;
    }


    private function buttonTriggeredHandler(e:Event):void {
        if(_waitingForBrandsLoad) {
            return;
        }

        switch (e.currentTarget){
            case tuningButton:
                    SoundManager.playSound(SoundManager.PART_MENU);
                    Tracer.timeStamp('HomeButton', "Tuning");
                    AppMode.self.setModeTuning();
                    _waitingForBrandsLoad = true;
                    Starling.juggler.delayCall(function () : void  {
                        dispatchEventWith(MODE_SELECTED);
                    }, 0.25);
                break;
            case _challengesButton:
                    Tracer.timeStamp('HomeButton', "Challenges");
                    challengeButtonTriggeredHandler(null);
                break;
        }
    }

    public function get userSignedIn():Boolean{
        return _userSignedIn;
    }

    public function set userSignedIn(value:Boolean):void {
        _userSignedIn = value;
        if (!stage)	{
            return;
        }
        invalidate("userSignedIn");
    }

    override public function dispose():void {
        super.dispose();


        if(tuningButton){
            tuningButton.removeFromParent(true);
            tuningButton = null;
        }
        if(_challengesButton){
            _challengesButton.removeFromParent(true);
            _challengesButton = null;
        }
        if(_mainMenuView){
            _mainMenuView.removeFromParent(true);
            _mainMenuView = null;
        }
        if(_blockerView){
            _blockerView.removeFromParent(true);
            _blockerView = null;
        }
        if(_headerContainer){
            _headerContainer.removeFromParent(true);
            _headerContainer = null;
        }
        if(galleryButton){
            galleryButton.removeFromParent(true);
            galleryButton = null;
        }
        if(garageButton){
            garageButton.removeFromParent(true);
            garageButton = null;
        }
        if(timelineButton){
            timelineButton.removeFromParent(true);
            timelineButton = null;
        }

        if(scheduleButton){
            scheduleButton.removeFromParent(true);
            scheduleButton = null;
        }
        if(mailButton){
            mailButton.removeFromParent(true);
            mailButton = null;
        }
        if(friendsButton){
            friendsButton.removeFromParent(true);
            friendsButton = null;
        }
        if(notificationsButton){
            notificationsButton.removeFromParent(true);
            notificationsButton = null;
        }
        if(settingsButton){
            settingsButton.removeFromParent(true);
            settingsButton = null;
        }
        if(avatarContainer){
            avatarContainer.removeFromParent(true);
            avatarContainer = null;
        }
        if(newsButton){
            newsButton.removeFromParent(true);
            newsButton = null;
        }
        if(_bottomMenu){
            _bottomMenu.removeFromParent(true);
            _bottomMenu = null;
        }
        if(leaderboardButton){
            leaderboardButton.removeFromParent(true);
            leaderboardButton = null;
        }
        if(votingButton){
            votingButton.removeFromParent(true);
            votingButton = null;
        }
        if(designsButton){
            designsButton.removeFromParent(true);
            designsButton = null;
        }
        if(loginButton){
            loginButton.removeFromParent(true);
            loginButton = null;
        }
        if(avatarButton){
            avatarButton.removeFromParent(true);
            avatarButton = null;
        }
        if(_userLabel){
            _userLabel.removeFromParent(true);
            _userLabel = null;
        }
        if(_loginLabel){
            _loginLabel.removeFromParent(true);
            _loginLabel = null;
        }

        _selectedTuning = null;
        if(_mainMenuView){
            DisplayObjectUtil.dispose(_mainMenuView);
            _mainMenuView = null;
        }
        _selectedTuning = null;
        DisplayObjectUtil.dispose(this);
    }

    override public function updateWithLanguage(language:Language) : void {
        _language = language;

        AlertView.language = language;
        CustomAlertView.language = language;
        AlertViewHL.language = language;

        if (mainMenuView){
            mainMenuView.updateWithLanguage(language);
        }
    }

    override public function dispatchExternalEvent(msg:String):void {
        if (mainMenuView) mainMenuView.hide();
        dispatchEventWith(msg);
    }

    override public function set mainMenuView(value : MainMenuView): void {
        _mainMenuView = value;
        _mainMenuView.owner = this;
        _mainMenuView.visible = false;
    }

    override public function brandsScreenButtonTriggeredHandler(event : Event=null) : void{
        mainMenuView.hide(false);
        //_blockerView = BlockerView.show();
        _waitingForBrandsLoad = true;
        Starling.juggler.delayCall(function () : void  {
            dispatchEventWith(MODE_SELECTED);
        }, 1)
    }

        public function updateLevelData(currentUser:User):void {
            if(currentUser){
                levelButton ||= new UserLevelAvatar(35);
                if(!levelButton.parent){
                    addChild(levelButton);
                }
                if(!levelButton.hasEventListener(TouchEvent.TOUCH, levelTouchHandler)){
                    levelButton.addEventListener(TouchEvent.TOUCH, levelTouchHandler);
                }
                levelButton.validate();
                levelButton.update(currentUser);
                levelButton.invalidate();
            }else if(levelButton){
                levelButton.removeFromParent(true);
                levelButton = null;
            }

            if(levelButton){
                levelButton.x = actualWidth - (levelButton.width / 2 + 10);
                levelButton.y = marginTop / 2 - 1;
                if(_ratingLabel){
                    _ratingLabel.paddingRight = 40;
                }
            }else if(!levelButton && _ratingLabel){
                _ratingLabel.paddingRight = 10;
            }
        }
    }
}
