package com.distriqt {
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.distriqt.extension.gameservices.GameServices;
    import com.distriqt.extension.gameservices.Player;
    import com.distriqt.extension.gameservices.Service;
    import com.distriqt.extension.gameservices.events.GameServicesEvent;
    import com.distriqt.extension.gameservices.events.PlayerEvent;
    import com.distriqt.extension.gameservices.events.PlayerIconEvent;
    import com.utils.Console;

    import starling.textures.Texture;

    public class GameServiceManager {
        private static var _instance:GameServiceManager;
        private var _initialized:Boolean = false;
        private var _isSignedOut:Boolean = false;
        private var _loginCallback:Function;
        private var _loginUnableCallback:Function;
        private var _iconCompleteCallback:Function;
        private var _profileObj:Object;
        private var _player:Player;
        public function GameServiceManager() {

        }

        private function initialize():void{
            var service:Service;
            if (GameServices.service.isServiceSupported( Service.HUAWEI_GAME_SERVICE )) {
                service = new Service( Service.HUAWEI_GAME_SERVICE );
            }else if (GameServices.service.isServiceSupported( Service.GOOGLE_PLAY_GAME_SERVICES )) {
                service = new Service( Service.GOOGLE_PLAY_GAME_SERVICES );
            }else if (GameServices.service.isServiceSupported( Service.IOS_GAME_CENTER )){
                service = new Service( Service.IOS_GAME_CENTER );
            }
            GameServices.service.addEventListener(GameServicesEvent.INITIALISED, initialisedHandler);
            GameServices.service.addEventListener(GameServicesEvent.INITIALISE_ERROR, gameServices_initialiseErrorHandler);
            GameServices.service.initialiseService(service);
        }

        public static function init(loginCallback:Function = null):void{
            if(GameServices.isSupported && !_instance){
                _instance = new GameServiceManager();
                _instance.loginCallback = loginCallback;
                _instance.initialize();
            }
        }

        public static function get instance():GameServiceManager {
            return _instance;
        }

        public function signIn():void{
            if (_initialized && !GameServices.service.isSignedIn()) {
                GameServices.service.signIn();
            }else if(_initialized && _isSignedOut && _loginUnableCallback != null){
                _loginUnableCallback();
            }
        }

        public function signOut():void{
            if (GameServices.isSupported && GameServices.service.isSignedIn()) {
                if (!GameServices.service.disconnect()) {
                    GameServices.service.signOut();
                    _isSignedOut = true;
                }
            }
        }

        private function initialisedHandler(event:GameServicesEvent ):void {
            _initialized = true;
            GameServices.service.addEventListener(GameServicesEvent.SIGN_IN_SUCCESS, signInSuccessHandler);
            GameServices.service.addEventListener(GameServicesEvent.SIGN_IN_FAILED, signInFailedHandler);
            GameServices.service.addEventListener(GameServicesEvent.SIGN_OUT_SUCCESS, signOutSuccessHandler);

            GameServices.service.addEventListener(PlayerEvent.LOADED, loadPlayer_loadedHandler);
            GameServices.service.addEventListener(PlayerEvent.ERROR, loadPlayer_errorHandler);
            if (GameServices.service.isSignedIn()) {
                GameServices.service.loadPlayer();
            }
        }

        private function playerIconLoadedHandler(event:PlayerIconEvent):void {
            var texture:Texture = Texture.fromBitmapData(event.icon);
            if(_iconCompleteCallback != null){
                _iconCompleteCallback(texture);
            }
        }

        private function playerIconErrorHandler( event:PlayerIconEvent ):void {
            trace( "Error: "+ event.error );
        }

        private function gameServices_initialiseErrorHandler( event:GameServicesEvent ):void {
            // An error occurred, try initialising the service again later
            // Check the error code for developer errors and potentially network issues
        }

        private function signInSuccessHandler(event:GameServicesEvent):void {
            var player:Player = GameServices.service.getPlayer();
            if (player != null) {
                sendUserData(player);
            }
            GameServices.service.register();
            _isSignedOut = false;
        }

        private function signOutSuccessHandler(event:GameServicesEvent):void {
            Console.log("User successfully signed out");
            _isSignedOut = true;
        }

        private function signInFailedHandler(event:GameServicesEvent):void {
            Console.log("Game Service sign in failed");
        }

        private function loadPlayer_loadedHandler(event:PlayerEvent):void {
            var player:Player = event.player;
            sendUserData(player);
        }

        private function getPlayerIcon(player:Player):void{
            GameServices.service.addEventListener( PlayerIconEvent.LOADED, playerIconLoadedHandler );
            GameServices.service.addEventListener( PlayerIconEvent.ERROR,  playerIconErrorHandler );
            GameServices.service.loadPlayerIcon( player );
        }

        private function loadPlayer_errorHandler(event:PlayerEvent):void {
            trace( "loadPlayer_errorHandler(): " + event.error);
        }

        private function sendUserData(player:Player):void{
            var profileObj:Object = {};
            profileObj.typeOfSN = UsersProxy.typeOfSN_GS;
            profileObj.user_id = player.id;
            profileObj.user_name = player.displayName;

            _profileObj = profileObj;
            if(_loginCallback != null){
                _loginCallback(profileObj);
            }
            _loginCallback = null;
            _player = player;
        }

        public function set loginCallback(value:Function):void {
            _loginCallback = value;
        }

        public function set iconCompleteCallback(value:Function):void {
            _iconCompleteCallback = value;
        }

        public function loadUserIcon():void {
            if(_player){
                getPlayerIcon(_player);
                _player = null;
            }
        }

        public function get loginUnableCallback():Function {
            return _loginUnableCallback;
        }

        public function set loginUnableCallback(value:Function):void {
            _loginUnableCallback = value;
        }
    }
}
